/*****************************************************************************
 *  Rohde&Schwarz RTO Digital Oscilloscope instrument driver
 *  Callbacks
 *
 *  Original Release: December 2010
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsscope.h"
#include <float.h>
#include <stdlib.h>
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter


 /*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/


 /*****************************************************************************
  * Function: rsscope_ConvertFromBinary_ReadCallback
  * Purpose:  This function overrides standard calback funtion. Converts
  *           binary number
  *****************************************************************************/
ViStatus rsscope_ConvertFromBinary_ReadCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  int32Value;
    ViInt64  int64Value;
    ViChar*  cmd = NULL;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

    switch (attr->dataType)
    {
    case RS_VAL_INT32:
        int32Value = (ViInt32)RsCore_Convert_BinaryString_To_ViInt64(response);
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, &int32Value));
        break;
    case RS_VAL_INT64:
        int64Value = RsCore_Convert_BinaryString_To_ViInt64(response);
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, &int64Value));
        break;
    default:
        viCheckErrElab(RS_ERROR_INVALID_TYPE,
            "rsscope_ConvertFromBinary_ReadCallback only accepts RS_VAL_INT32 and RS_VAL_INT64 data types");
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatSearchName_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchName_ReadCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   name[RS_MAX_MESSAGE_LEN];
    ViChar*  responseUnknownLen = NULL;
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? '", name, "'"));
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameFrameWordByte_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of selectedFrame
Replaces <Word> with value of selectedWord
Replaces <Byte> with value of selectedByte
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameFrameWordByte_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr,
                                                            ViAttr             selectedFrame,
                                                            ViAttr             selectedWord,
                                                            ViAttr             selectedByte)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   name[RS_MAX_MESSAGE_LEN];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL;
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  frame, _byte;
    ViInt64  word;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedWord > 0)
    {
        // Replaces <Word> with number
        checkErr(RsCore_GetAttributeViInt64(instrSession, NULL, selectedWord, 0, &word));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%lld", word);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Word>", aux, VI_TRUE);
    }

    if (selectedByte > 0)
    {
        // Replaces <Byte> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedWord, 0, &_byte));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", _byte);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Byte>", aux, VI_TRUE);
    }

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? '", name, "'"));
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameFrameWordByte_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of selectedFrame
Replaces <Word> with value of selectedWord
Replaces <Byte> with value of selectedByte
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameFrameWordByte_WriteCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr,
                                                             void*              value,
                                                             ViAttr             selectedFrame,
                                                             ViAttr             selectedWord,
                                                             ViAttr             selectedByte)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_LEN];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  frame, _byte;
    ViInt64  word;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedWord > 0)
    {
        // Replaces <Word> with number
        checkErr(RsCore_GetAttributeViInt64(instrSession, NULL, selectedWord, 0, &word));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%lld", word);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Word>", aux, VI_TRUE);
    }

    if (selectedByte > 0)
    {
        // Replaces <Byte> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedWord, 0, &_byte));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", _byte);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Byte>", aux, VI_TRUE);
    }

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, ",", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, ",", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, ",", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, ",", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",\'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, ",", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameBoolean_ReadCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchName_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameInt32_ReadCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchName_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameInt64_ReadCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchName_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameReal64_ReadCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchName_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameString_ReadCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatSearchName_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchName_WriteCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr,
                                                void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_LEN];

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));

    if ((attr->rangeTableDiscrete))
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, ",", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, ",", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, ",", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, ",", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",'", (ViString)value, "'");
            break;
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",", (ViString)value, NULL);
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, ",", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameBoolean_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameBoolean_WriteCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr,
                                                       void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameEvent_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameEvent_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameInt64_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameInt64_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameInt32_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameInt32_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameReal64_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameReal64_WriteCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr,
                                                      void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameString_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameString_WriteCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr,
                                                      void*              value)
{
    return rsscope_FormatSearchName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameSource_ReadCallback
Adds RSSCOPE_ATTR_SELECTED_SEARCH_NAME and RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameSource_ReadCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   name[RS_MAX_MESSAGE_LEN];
    ViInt32  source = 0;
    ViChar*  p2source = NULL;
    ViChar*  responseUnknownLen = NULL;
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE,
        RS_VAL_DIRECT_USER_CALL, &source));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngSearchSource,
        source, &p2source));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "',"));
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, p2source, NULL, NULL));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? '", name, "',"));
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, p2source, NULL, NULL));
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);

            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameSourceBoolean_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameSource_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name after the command
ViStatus rsscope_FormatSearchNameSourceInt32_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameSource_ReadCallback(instrSession, repCapName, attr);
}

//Adds search name and source after the command
ViStatus rsscope_FormatSearchNameSourceReal64_ReadCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameSource_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameSource_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_SEARCH_NAME and RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameSource_WriteCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr,
                                                      void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_LEN];
    ViInt32  source = 0;
    ViChar*  p2source = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_LEN, name));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE,
        RS_VAL_DIRECT_USER_CALL, &source));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngSearchSource,
        source, &p2source));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "',"));
    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, p2source, NULL, NULL));

    if ((attr->rangeTableDiscrete))
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, ",", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, ",", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, ",", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, ",", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",'", (ViString)value, "'");
            break;
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",", (ViString)value, NULL);
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, ",", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalLabelSource_ReadCallback
Adds RSSCOPE_ATTR_SELECTED_LABEL_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatUniversalLabelSource_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  source = 0;
    ViChar*  p2source = NULL;
    ViChar*  responseUnknownLen = NULL;
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_LABEL_SOURCE,
        RS_VAL_DIRECT_USER_CALL, &source));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter,
        source, &p2source));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2source, NULL));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? ", p2source, NULL));
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);

            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalLabelSource_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_LABEL_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatUniversalLabelSource_WriteCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr,
                                                          void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViInt32  source = 0;
    ViChar*  p2source = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_LABEL_SOURCE,
        RS_VAL_DIRECT_USER_CALL, &source));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter,
        source, &p2source));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2source, NULL));

    if ((attr->rangeTableDiscrete))
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, ",", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, ",", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, ",", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, ",", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",'", (ViString)value, "'");
            break;
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",", (ViString)value, NULL);
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, ",", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatLabelSourceBoolean_ReadCallback
Adds RSSCOPE_ATTR_SELECTED_LABEL_SOURCE value after the command
******************************************************************************************************/
ViStatus rsscope_FormatLabelSourceBoolean_ReadCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalLabelSource_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatLabelSourceBoolean_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_LABEL_SOURCE value after the command
******************************************************************************************************/
ViStatus rsscope_FormatLabelSourceBoolean_WriteCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr,
                                                        void*              value)
{
    return rsscope_FormatUniversalLabelSource_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameSourceBoolean_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_SEARCH_NAME and RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameSourceBoolean_WriteCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr,
                                                             void*              value)
{
    return rsscope_FormatSearchNameSource_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameSourceInt32_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_SEARCH_NAME and RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameSourceInt32_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    return rsscope_FormatSearchNameSource_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameSourceReal64_WriteCallback
Adds RSSCOPE_ATTR_SELECTED_SEARCH_NAME and RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE after the command
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameSourceReal64_WriteCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr,
                                                            void*              value)
{
    return rsscope_FormatSearchNameSource_WriteCallback(instrSession, repCapName, attr, value);
}


/****************************************************************************************************
 *  rsscope_FormatUniversalFrameFieldRepCap_ReadCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
******************************************************************************************************/
ViStatus rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            ViAttr             selectedFrame,
                            ViAttr             selectedField)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  field, frame;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalFrameFieldRepCap_WriteCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
    ******************************************************************************************************/
ViStatus rsscope_FormatUniversalFrameFieldRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value,
                            ViAttr             selectedFrame,
                            ViAttr             selectedField)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  field, frame;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalPacketRepCap_ReadCallback
Replaces <Packet> with value of ViAttr selectedPacket
******************************************************************************************************/
ViStatus rsscope_FormatUniversalPacketRepCap_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr,
                                                          ViAttr             selectedPacket)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  packet;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedPacket > 0)
    {
        // Replaces <Packet> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedPacket, 0, &packet));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", packet);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Packet>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalPacketRepCap_WriteCallback
Replaces <Packet> with value of ViAttr selectedPacket
    ******************************************************************************************************/
ViStatus rsscope_FormatUniversalPacketRepCap_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value,
                                                           ViAttr             selectedPacket)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  packet;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedPacket > 0)
    {
        // Replaces <Packet> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedPacket, 0, &packet));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", packet);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Packet>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Word> with value of ViAttr selectedWord
Replaces <Byte> with value of ViAttr selectedByte
******************************************************************************************************/
ViStatus rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            ViAttr             selectedFrame,
                            ViAttr             selectedWord,
                            ViAttr             selectedByte)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  frame, _byte;
    ViInt64  word;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedWord > 0)
    {
        // Replaces <Word> with number
        checkErr(RsCore_GetAttributeViInt64(instrSession, NULL, selectedWord, 0, &word));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%lld", word);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Word>", aux, VI_TRUE);
    }

    if (selectedByte > 0)
    {
        // Replaces <Byte> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedWord, 0, &_byte));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", _byte);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Byte>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_formatMaskName_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskName_ReadCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;

    // ViInt32 violation;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Appends 'MaskName'
    checkErr(
        RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_MASK_NAME, 0, RS_MAX_MESSAGE_BUF_SIZE, aux
        ));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", aux, "'"));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? '", aux, "'"));
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_formatMaskNameInt32_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameInt32_ReadCallback(ViSession          instrSession,
                                                  ViConstString      repCapName,
                                                  RsCoreAttributePtr attr)
{
    return rsscope_formatMaskName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameInt64_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameInt64_ReadCallback(ViSession          instrSession,
                                                  ViConstString      repCapName,
                                                  RsCoreAttributePtr attr)
{
    return rsscope_formatMaskName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameString_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameString_ReadCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr)
{
    return rsscope_formatMaskName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameReal64_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameReal64_ReadCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr)
{
    return rsscope_formatMaskName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameBoolean_ReadCallback
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameBoolean_ReadCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr)
{
    return rsscope_formatMaskName_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskName_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskName_WriteCallback(ViSession          instrSession,
                                              ViConstString      repCapName,
                                              RsCoreAttributePtr attr,
                                              void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_MASK_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_BUF_SIZE, name));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, ",", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, ",", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, ",", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, ",", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, ",'", (ViString)value, "'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, ",", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_formatMaskNameInt32_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameInt32_WriteCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr,
                                                   void*              value)
{
    return rsscope_formatMaskName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameInt64_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameInt64_WriteCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr,
                                                   void*              value)
{
    return rsscope_formatMaskName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameString_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameString_WriteCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr,
                                                    void*              value)
{
    return rsscope_formatMaskName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameReal64_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameReal64_WriteCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr,
                                                    void*              value)
{
    return rsscope_formatMaskName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameBoolean_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_MASK_NAME after the command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameBoolean_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return rsscope_formatMaskName_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameViolation_ReadCallback
Replaces <Violation> with value of RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_INDEX
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameViolation_ReadCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  violation;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Violation> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_INDEX, 0, &
            violation));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", violation);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Violation>", aux, VI_TRUE);

    // Appends 'MaskName'
    checkErr(
        RsCore_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_MASK_NAME, 0, RS_MAX_MESSAGE_BUF_SIZE, aux
        ));
    strcat(cmd, " '");
    strcat(cmd, aux);
    strcat(cmd, "'");

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_RAWSTRING:
        case RS_VAL_STRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_formatMaskNameViolationInt32_ReadCallback
Replaces <Violation> with value of RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_INDEX
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameViolationInt32_ReadCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr)
{
    return rsscope_formatMaskNameViolation_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_formatMaskNameViolationReal64_ReadCallback
Replaces <Violation> with value of RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_INDEX
Appends 'RSSCOPE_ATTR_SELECTED_MASK_NAME' after command
******************************************************************************************************/
ViStatus rsscope_formatMaskNameViolationReal64_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr)
{
    return rsscope_formatMaskNameViolation_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatFlexRayFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatFlexRayFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME, 0);
}

/****************************************************************************************************
 *  rsscope_FormatFlexRayFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatFlexRayFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME, 0);
}

/****************************************************************************************************
 *  rsscope_FormatFlexRayFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatFlexRayFrameRepCapString_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME, 0);
}

/****************************************************************************************************
 *  rsscope_FormatDDRFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatDDRFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME, 0);
}

/****************************************************************************************************
 *  rsscope_FormatDDRFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatDDRFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatLINFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINFrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatLINFrameRepCapInt64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatLINFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatLINFrameRepCapString_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINSearchNameFrameRepCapString_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of ViAttr selectedFrame
******************************************************************************************************/
ViStatus rsscope_FormatLINSearchNameFrameRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr, RSSCOPE_ATTR_SEARCH_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINSearchNameFrameRepCapString_WriteCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of ViAttr selectedFrame
******************************************************************************************************/
ViStatus rsscope_FormatLINSearchNameFrameRepCapString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatSearchNameFrameWordByte_WriteCallback(instrSession, repCapName, attr, value, RSSCOPE_ATTR_SEARCH_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatLINByteRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Byte> with value of RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsscope_FormatLINByteRepCapInt32_ReadCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0,
        RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_BYTE);
}

/****************************************************************************************************
 *  rsscope_FormatCANFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatCANFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatCANFrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatCANFrameRepCapInt64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatCANFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatCANFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatCANFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatCANFrameRepCapString_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatCANByteRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Byte> with value of RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsscope_FormatCANByteRepCapInt32_ReadCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0,
        RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_BYTE);
}

/****************************************************************************************************
 *  rsscope_FormatI2CFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatI2CFrameRepCapString_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatI2CFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatI2CFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatI2CFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatI2CFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatI2CFrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatI2CFrameRepCapInt64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatI2CFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatI2CFrameRepCapBoolean_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatI2CByteRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsscope_FormatI2CByteRepCapInt32_ReadCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE);
}

/****************************************************************************************************
 *  rsscope_FormatI2CByteRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsscope_FormatI2CByteRepCapReal64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE);
}

/****************************************************************************************************
 *  rsscope_FormatI2CByteRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsscope_FormatI2CByteRepCapBoolean_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0,
        RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_BYTE);
}

/****************************************************************************************************
 *  rsscope_FormatSPIFrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSPIFrameRepCapInt64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSPIFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSPIFrameRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSPIFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSPIFrameRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSPIWordRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_FormatSPIWordRepCapInt32_ReadCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSPIWordRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_FormatSPIWordRepCapReal64_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSPIWordRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_FormatSPIWordRepCapString_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_WORD, 0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1FrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1FrameRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, 0,
        0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1FrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1FrameRepCapInt64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, 0,
        0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1FrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1FrameRepCapReal64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, 0,
        0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1FrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1FrameRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, 0,
        0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1WordRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1WordRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_WORD, 0);
}

/****************************************************************************************************
 *  rsscope_Format100BASE_T1WordRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_Format100BASE_T1WordRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_WORD, 0);
}

/****************************************************************************************************
 *  rsscope_FormatDDRFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatDDRFieldRepCapInt32_ReadCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatDDRFieldRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatDDRFieldRepCapReal64_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatDDRFieldRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatDDRFieldRepCapString_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1FrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1FrameRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1FrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1FrameRepCapInt64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1FrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1FrameRepCapReal64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1FrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1FrameRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1WordRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1WordRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD,
        0);
}

/****************************************************************************************************
 *  rsscope_Format1000BASE_T1WordRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_Format1000BASE_T1WordRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameWordByteRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD,
        0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOFrameInt32_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOFrameInt32_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOFrameInt64_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOFrameInt64_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOFrameReal64_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOFrameReal64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOFrameString_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOFrameString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        0, 0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOWordInt32_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOWordInt32_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD,
        0);
}

/****************************************************************************************************
 *  rsscope_FormatSearchNameTBTOWordString_ReadCallback
Adds search name RSSCOPE_ATTR_SELECTED_SEARCH_NAME after the command
Replaces <Frame> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME
Replaces <Word> with value of RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD
******************************************************************************************************/
ViStatus rsscope_FormatSearchNameTBTOWordString_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    return rsscope_FormatSearchNameFrameWordByte_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
        RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD,
        0);
}


/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCap_ReadCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  component;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Component> with number
    checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT, 0, &component));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", component);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Component>", aux, VI_TRUE);

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapBoolean_ReadCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapBoolean_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatDeembeddingComponentRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapInt32_ReadCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatDeembeddingComponentRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapReal64_ReadCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapReal64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatDeembeddingComponentRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapString_ReadCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatDeembeddingComponentRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCap_WriteCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViInt32  component;
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Component> with number
    checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT, 0, &component));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", component);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Component>", aux, VI_TRUE);

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapBoolean_WriteCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatDeembeddingComponentRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapInt32_WriteCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatDeembeddingComponentRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapReal64_WriteCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapReal64_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatDeembeddingComponentRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatDeembeddingComponentRepCapString_WriteCallback
Replaces <Component> with value of RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT
******************************************************************************************************/
ViStatus rsscope_FormatDeembeddingComponentRepCapString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatDeembeddingComponentRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapBoolean_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapRawString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapRawString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalFrameFieldRepCap_ReadCallback(instrSession, repCapName, attr,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatUniversalFrameFieldRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatUniversalFrameFieldRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameFieldRepCapRawString_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Field> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameFieldRepCapRawString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatUniversalFrameFieldRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME,
        RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCap_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCap_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  cell, frame;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Cell> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL, 0, &cell));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", cell);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Cell>", aux, VI_TRUE);

    // Replaces <Frame> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, 0, &frame));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatManchesterNRZFrameRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
//Replaces <Frame> and <Cell> with numbers in the command string
ViStatus rsscope_FormatManchesterNRZFrameRepCapInt32_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatManchesterNRZFrameRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapInt64_ReadCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
//Replaces <Frame> and <Cell> with numbers in the command string
ViStatus rsscope_FormatManchesterNRZFrameRepCapInt64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatManchesterNRZFrameRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCap_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViInt32  cell, frame;
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Cell> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL, 0, &cell));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", cell);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Cell>", aux, VI_TRUE);

    // Replaces <Frame> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, 0, &frame));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapInt64_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCapInt64_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatManchesterNRZFrameRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapString_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCapString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatManchesterNRZFrameRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapInt32_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatManchesterNRZFrameRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatManchesterNRZFrameRepCapEvent_WriteCallback
Replaces <Frame> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME
Replaces <Cell> with value of RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL
******************************************************************************************************/
ViStatus rsscope_FormatManchesterNRZFrameRepCapEvent_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsscope_FormatManchesterNRZFrameRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatPCIePacketRepCapInt64_WriteCallback
Replaces <Packet> with value of RSSCOPE_ATTR_PROTOCOL_PCIE_SELECTED_PACKET
******************************************************************************************************/
ViStatus rsscope_FormatPCIePacketRepCapInt64_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    return rsscope_FormatUniversalPacketRepCap_WriteCallback(instrSession, repCapName, attr, value, RSSCOPE_ATTR_PROTOCOL_PCIE_SELECTED_PACKET);
}

/****************************************************************************************************
 *  rsscope_FormatPCIePacketRepCapInt64_ReadCallback
Replaces <Packet> with value of RSSCOPE_ATTR_PROTOCOL_PCIE_SELECTED_PACKET
******************************************************************************************************/
ViStatus rsscope_FormatPCIePacketRepCapInt64_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    return rsscope_FormatUniversalPacketRepCap_ReadCallback(instrSession, repCapName, attr, RSSCOPE_ATTR_PROTOCOL_PCIE_SELECTED_PACKET);
}


/*****************************************************************************
 * Function: rsscope_logging_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsscope_logging_ReadCallback(ViSession          instrSession,
                                      ViConstString      repCapName,
                                      RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsspecan_logging_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsscope_logging_WriteCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr,
                                       void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsscope_empty_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsscope_empty_ReadCallback(ViSession          instrSession,
                                    ViConstString      repCapName,
                                    RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsscope_empty_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Does not write
 *           anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rsscope_empty_WriteCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr,
                                     void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
rsscope_FormatSearchNamePattern_ReadCallback
Adds additional parameter after the command query: string from RSSCOPE_ATTR_SELECTED_SEARCH_NAME
Receives response starting with #H..., returns only the portion after #H
Attribute types: RS_VAL_STRING
*****************************************************************************/
ViStatus rsscope_FormatSearchNamePattern_ReadCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL;
    ViInt32  responseSize = 0;
    ViChar*  p2response = NULL;
    ViChar*  searchPtr = NULL;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_STRING)
    {
        viCheckErrElab(RS_ERROR_INVALID_TYPE,
            "rsscope_FormatSearchNamePattern_ReadCallback only accepts STRING data types");
    }

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_BUF_SIZE, name));

    if (attr->access == RS_VAL_READ_ONLY)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));
    }
    else
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, "? '", name, "'"));
    }

    // Query the instrument, get the unlimited size response
    checkErr(RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
    // For string-type attributes remove trailing Whitespaces and pair quotations
    RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
    p2response = responseUnknownLen;

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    // Cut the #H if it exists at the beginning
    searchPtr = strstr(p2response, "#H");
    if (searchPtr == p2response)
    {
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response + 2));
    }
    else
    {
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/*****************************************************************************
rsscope_FormatSearchNamePattern_WriteCallback
Adds additional parameter after the command query: string from RSSCOPE_ATTR_SELECTED_SEARCH_NAME
Sends string value parameter preceeded with #H
Attribute types: RS_VAL_STRING
*****************************************************************************/
ViStatus rsscope_FormatSearchNamePattern_WriteCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr,
                                                       void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   name[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_STRING)
    {
        viCheckErrElab(RS_ERROR_INVALID_TYPE,
            "rsscope_FormatSearchNamePattern_WriteCallback only accepts STRING data types");
    }

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, RS_VAL_DIRECT_USER_CALL,
        RS_MAX_MESSAGE_BUF_SIZE, name));

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " '", name, "'"));

    RsCore_StrCatDynamicString(instrSession, &cmd, ",#H", (ViString)value, NULL);
    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *    rsscope_FormatUSBPDPacketType_ReadCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPacketType_ReadCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   searchName[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCapTrig[10];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL, * typeString = NULL;
    ViInt32  type;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (attr->access != RS_VAL_READ_ONLY)
        RsCore_StrCatDynamicString(instrSession, &cmd, "?", NULL, NULL);

    if (strncmp(cmd, "SEAR", 4) == 0)
    {
        checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_USB_PD_SEARCH_PACKET_TYPE,
            RS_VAL_DIRECT_USER_CALL, &type));
        checkErr(RsCore_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME,
            RS_VAL_DIRECT_USER_CALL,
            RS_MAX_MESSAGE_BUF_SIZE, searchName));

        RsCore_StrCatDynamicString(instrSession, &cmd, " '", searchName, "',");
    }
    else
    {
        if (strlen(repCapName) > 4) strncpy(repCapTrig, repCapName, 5);
        checkErr(rsscope_GetAttributeViInt32(instrSession, repCapTrig, RSSCOPE_ATTR_USB_PD_TRIGGER_PACKET_TYPE, &type));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", NULL, NULL);
    }

    RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSBPDPacketType, type, &typeString);
    RsCore_StrCatDynamicString(instrSession, &cmd, NULL, typeString, NULL);

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *    rsscope_FormatUSBPDPacketTypeInt32_ReadCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPacketTypeInt32_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return rsscope_FormatUSBPDPacketType_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *    rsscope_FormatUSBPDPacketTypeString_ReadCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPacketTypeString_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    return rsscope_FormatUSBPDPacketType_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatUSBPDPPacketType_WriteCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPPacketType_WriteCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr,
                                                      void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   repCapTrig[10];
    ViChar   searchName[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  type;
    ViChar*  typeString = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Check if it is search attribute
    if (strncmp(cmd, "SEAR", 4) == 0)
    {
        checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_USB_PD_SEARCH_PACKET_TYPE,
            RS_VAL_DIRECT_USER_CALL, &type));
        checkErr(RsCore_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME,
            RS_VAL_DIRECT_USER_CALL,
            RS_MAX_MESSAGE_BUF_SIZE, searchName));

        RsCore_StrCatDynamicString(instrSession, &cmd, " '", searchName, "',");
    }
    else
    {
        if (strlen(repCapName) > 4) strncpy(repCapTrig, repCapName, 5);
        checkErr(rsscope_GetAttributeViInt32(instrSession, repCapTrig, RSSCOPE_ATTR_USB_PD_TRIGGER_PACKET_TYPE, &type));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", NULL, NULL);
    }

    RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSBPDPacketType, type, &typeString);
    RsCore_StrCatDynamicString(instrSession, &cmd, NULL, typeString, ",");

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, NULL, *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, NULL, *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, NULL, *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, "\'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, NULL, *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatUSBPDPacketTypeInt32_WriteCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPacketTypeInt32_WriteCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr,
                                                          void*              value)
{
    return rsscope_FormatUSBPDPPacketType_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_FormatUSBPDPacketTypeString_WriteCallback
Formats the Packet type as parameter to command
Only for USBPDP attributes
******************************************************************************************************/
ViStatus rsscope_FormatUSBPDPacketTypeString_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    return rsscope_FormatUSBPDPPacketType_WriteCallback(instrSession, repCapName, attr, value);
}

/****************************************************************************************************
 *  rsscope_quotedString_WriteCallback
For an attribute without a range table, it works as the standard RsCore_WriteCallback
For attributes with discrete range tables, it writes the command value in single apotrophes
******************************************************************************************************/
ViStatus rsscope_quotedString_WriteCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr,
                                            void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " '", p2cmdString, "'");
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsscope_quotedInt_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes string
 *           with apostrophes
 *****************************************************************************/
ViStatus rsscope_quotedInt_WriteCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr,
                                         void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'");
}

/****************************************************************************************************
 *  rsscope_FormatExternalFrontendMultiBandRepCap_ReadCallback
Replaces <MultiBand> with value of RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND
******************************************************************************************************/
ViStatus rsscope_FormatExternalFrontendMultiBandRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  multiBand;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // Replaces <Cell> with number
    checkErr(
        RsCore_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND, 0, &multiBand));
    snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", multiBand);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<MultiBand>", aux, VI_TRUE);

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsscope_FormatExternalFrontendMultiBandRepCapString_ReadCallback
Replaces <MultiBand> with value of RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND
******************************************************************************************************/
ViStatus rsscope_FormatExternalFrontendMultiBandRepCapString_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatExternalFrontendMultiBandRepCap_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsscope_FormatExternalFrontendMultiBandRepCapReal64_ReadCallback
Replaces <MultiBand> with value of RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND
******************************************************************************************************/
ViStatus rsscope_FormatExternalFrontendMultiBandRepCapReal64_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr)
{
    return rsscope_FormatExternalFrontendMultiBandRepCap_ReadCallback(instrSession, repCapName, attr);
}
