/*****************************************************************************
 *  Rohde&Schwarz RTO Instrument Driver
 *  LabWindows/CVI Instrument Driver
 *  Original Release: 2010-12-17
 *  By: Jiri Kominek, Martin Krcmar, Petr Berger
 *
 *  Modification History:
 *
 *       See ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rsscope.h"
#include "rsscope_internal.h"
#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#if defined (_CVI_)
#include <utility.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString protocolI2CAckBit_Arr[] = {"EITH", "ACK", "NACK", "INC", NULL};
static ViString protocolUARTLine_Arr[] = {"RX", "TX", NULL};
static ViString protocolUARTStatus_Arr[] = {"OK", "FRST", "FREN", "FRME", "STER", "SPER", "PRER", "INS", "BRE", NULL};
static ViString protocolCANFrameStatus_Arr[] = {
    "OK", "BTST", "CRCD", "CRC", "EOFD", "ACKD", "NOAC", "INS", "CAER", "FCER", "FORM",
    "SERR", "SFER", "SCER", "SAER", "SCAE", "SCFE", NULL
};
static ViString protocolCANFieldStatus_Arr[] = {"OK", "UNDF", "ERR", NULL};
static ViString protocolCANFrameType_Arr[] = {"DATA", "REM", "ERR", "OVLD", "ANY", NULL};
static ViString protocolCANIDType_Arr[] = {"B11", "B29", "ANY", NULL};
static ViString protocolCANFrameFormErrorCause_Arr[] = {"NONE", "CRCD", "ACKD", "FSBE", "RES", NULL};
static ViString protocolCANFDSearchFrameStandard_Arr[] = {"CAN", "CANF", NULL};
static ViString polarityArr[] = {"Negative", "Positive"};
static ViString qualifyMultiChannelArr[] = {"", "AB", "CD", "ABCD", NULL};
static ViString qualifyMultiChannelLogicArr[] = {"AND", "OR", "NAND", "NOR", NULL};
static ViString modeArr[] = {"ABS", "REL"};
static ViString protocolAudioFrameStatus_Arr[] = {"OK", "ERR", "INS", NULL};
static ViString orientationArr[] = {"VERT", "HOR", NULL};
static ViString eyeShapeArr[] = {"SQU", "DIAM", "HEX", "OCT", NULL};
static ViString sourceRepCapArr[] = {
    // When deprecated rsscope_ConfigureReference* functions are deleted, this array can be deleted.
    "WfmNone", "WfmC1W1", "WfmC1W2", "WfmC1W3", "WfmC2W1", "WfmC2W2", "WfmC2W3", "WfmC3W1",
    "WfmC3W2", "WfmC3W3", "WfmC4W1", "WfmC4W2", "WfmC4W3", "WfmMath1", "WfmMath2", "WfmMath3",
    "WfmMath4", "", "WfmRef1", "WfmRef2", "WfmRef3", "WfmRef4", "WfmXY1", "WfmXY2", "WfmXY3",
    "WfmXY4", "WfmMeas1", "WfmMeas2", "WfmMeas3", "WfmMeas4", "WfmMeas5", "WfmMeas6", "WfmMeas7",
    "WfmMeas8", "WfmSbus1", "WfmSbus2", "WfmSbus3", "WfmSbus4", "WfmD0", "WfmD1", "WfmD2", "WfmD3",
    "WfmD4", "WfmD5", "WfmD6", "WfmD7", "WfmD8", "WfmD9", "WfmD10", "WfmD11", "WfmD12", "WfmD13",
    "WfmD14", "WfmD15", "WfmMSO1", "WfmMSO2", "WfmMSO3", "WfmMSO4", "WfmTRK1", "WfmTRK2", "WfmTRK3",
    "WfmTRK4", "WfmTRK5", "WfmTRK6", "WfmTRK7", "WfmTRK8", NULL
};
static ViString protocolMILStatus_Arr[] = {"OK", "SYNC", "MANC", "PAR", "GAP", "RT", NULL};
static ViString protocolMILType_Arr[] = {"UNKN", "DATA", "CMD", "STAT", "CMST", "IM", NULL};
static ViString protocolARINCStatus_Arr[] = {"OK", "CODE", "GAP", "PAR", NULL};
static ViString eyeJitterMeasFunctionArr[] = {
    "", "", "ERP", "ERDB", "EHE", "EWID", "ETOP",
    "EBAS", "", "", "QFAC", "", "", "", "RMSN", "SNR", "DCD",
    "ERT", "EFT", "EBR", "EAMP", "", "", "", "", "", "", "",
    "PPJ", "STDJ", "RMSJ", NULL
};
static ViString histogramMeasFunctionArr[] = {
    "", "WCO", "WSAM", "HSAM", "HPE", "PEAK", "UPE",
    "LPE", "HMAX", "HMIN", "MED", "MAXM", "HME",
    "HSTD", "M1ST", "M2ST", "M3ST", "MKP", "MKN", NULL
};
static ViString jitterMeasFunctionArr[] = {"CCJ", "NCJ", "CCW", "CCD", "TIE", "UINT", "DRAT", "SKWD", "SKWP", NULL};
static ViString protocolFlexRayStatus_Arr[] = {
    "OK", "FSS", "BSS", "FES", "IND", "HCRC",
    "CRC", "LENER", "WAK", "MTS", "INS", "HCFC", NULL
};
static ViString protocolFlexRayType_Arr[] = {"UNKN", "STAT", "DYN", "WAKE", "SYMB", NULL};
static ViString condition_Arr[] = {"NEQ", "EQU", "LTH", "LETH", "GTH", "GETH", "INR", "OOR", "OFF", NULL};
static ViString transfer_Arr[] = {"BEND", "LEND", NULL};
static ViString indicatorBits_Arr[] = {"ZERO", "ONE", "DC", NULL};
static ViString symbol_Arr[] = {"CASM", "WAK", NULL};
static ViString protocolCANStandard_Arr[] = {"CAN", "CANF", NULL};
static ViString protocolRFFEStatusCode_Arr[] = {"OK", "PAR", NULL};
static ViString protocolMDIOStatus_Arr[] = {"OK", "UNSYN", "OPCO", "SHORT", "INC", "TA_ERROR", NULL};
static ViString protocolEthernetWordFrameType_Arr[] = {"MAC", "IDLE", "SLE", "EOS", "UNKN", NULL};
static ViString protocolUSBStatus_Arr[] = {"OK", "PID", "CRC", "BTST", "GLITCH", "BYTE", NULL};
static ViString protocolUSBPID_Arr[] = {
    "RES", "OUT", "IN", "SOF", "SETUP", "DATA0", "DATA1", "DATA2", "MDATA",
    "ACK", "NAK", "STALL", "NYET", "PRE", "SPLIT", "PING", "UNK", NULL
};
static ViString protocolManchesterNRZStatus_Arr[] = {"OK", "LENG", "UNKN", "INC", "PAR", "CRC", "NULL"};
static ViString symbolStatus_Arr[] = {"OK", "UNK", "DISPARITY", "GLITCH", "NULL"};
static ViString protocolManchesterNRZStatus[] = {"OK", "LENG", "UNKN", "INC", "PAR", "CRC", "NULL"};
static ViString protocolSENTSearchFrameID_Arr[] = {"B4", "B8", "NULL"};
static ViString protocolEBTBSearchSymbolStatus_Arr[] = {"OK", "UNK", "DISPARITY", "GLITCH", "NULL"};
static ViString protocolSENTFrameStatus_Arr[] = {"OK", "SYNC", "PULS", "CRC", "IRFL", "FORM", "INS", "NULL"};
static ViString protocolSENTFrameStatusType_Arr[] = {"TRSQ", "SMSG", "EMSG", "NULL"};
static ViString protocolSENTFrameIDType_Arr[] = {"B4", "B8", "NULL"};
static ViString protocolSENTFieldStatus_Arr[] = {"OK", "UNDF", "NULL"};
static ViString protocolSENTSearchFrameStatus_Arr[] = {"OK", "SYNC", "PULS", "CRC", "IRFL", "FORM", "INS", "NULL"};
static ViString protocolSENTSearchFrameStatusType_Arr[] = {"TRSQ", "SMSG", "EMSG", "NULL"};
static ViString protocolSENTFrameNibbleStatus_Arr[] = {"OK", "UNDF", "NULL"};
static ViString protocolI2CSearchFrameAck_Arr[] = {"INC", "ACK", "NACK", "EITH", "NULL"};

static ViString protocolDPHYStatus_Arr[] = {"OK", "SYNC", "GAP", "PAR", "LENG", "UNKNOWN_FRAME", "INC", NULL};
static ViString protocolDPHYType_Arr[] = {"HS_SP", "HS_LP", "HS_INCOMPLETE", "LP_HSREQ", "LP_TA", "LP_ESC", NULL};

static ViString protocolSpaceWireStatus_Arr[] = {"OK", "PAR", "ESC", "AMB", "INC", NULL};
static ViString protocolSpaceWireType_Arr[] = {"DATA", "FCT", "EOP", "EEP", "ESC", "NULL", "TCOD", NULL};
static ViString protocolMPHYType_Arr[] = {
    "BURST", "ADAPT", "LCC", "DLPDUSOF", "DLPDUCOF", "DLPDUEOF", "DLPDUNAC",
    "DLPDUAFC", "PACP", "TRIGUPR0", "TRIGUPR1", "TRIGUPR2", "SKIP", "FILLER",
    "SPACER", "UNKN", NULL
};
static ViString protocolCXPITypeArr[] = {
    "OK", "INC", "ERR_IBS", "ERR_IFS", "ERR_LENGTH", "ERR_CRC", "ERR_PARITY", "ERR_DLC", "ERR_UART", NULL
};
static ViString protocolUSB31PacketTypeArr[] = {
    "TSEQ", "TS1T", "TS2T", "SLF", "U2IT", "VDT", "PCAP", "PCNF",
    "PCNR", "LDM", "ACK", "NRDY", "ERDY", "STAT", "STAL", "FWAK",
    "LTM", "BIAM", "HRR", "SSPD", "PING", "PNGR", "DPH", "DPP",
    "DPPA", "ITP", "LC", "SKIP", "BRST", "BDAT", "BERC", "BCNT",
    "IDLE", "UHP", "ULMP", "UTP", "UDVN", "UDEF", "ERR",NULL
};
static ViString protocolUSB31FrameStatusArr[] = {"OK", "INC", "UNC", "UNKN", "CRC", "LENG", "VOOR",NULL};
static ViString protocolUSBPDFrameStatusArr[] = {"OK", "UNKN", "INC", "PRE", "SOP", "CRC", "LENG", "FBFB",NULL};
static ViString protocolUSBPDFrameTypeArr[] = {
    "STAR", "RES", "CTRL", "DATA", "BIST", "RQST", "SINK", "SRC",
    "VEND", "BATT", "ALRT", "XMSG", "XSRC", "XSTA", "XGBC", "XGBS",
    "XBAC", "XGMI", "XMFI", "XSRQ", "XSRS", "XFRQ", "XFRS", "TEST",
    "LOWP", "UNKN", NULL
};

static ViString protocolPCIEPacketStatusArr[] = {
    "OK", "MALF", "CRC16", "ECRC", "LCRC", "LEN", "POE", "MEE", "UNC",NULL
};
static ViString protocolPCIEPacketTypeArr[] = {
    "MRD32", "MRD64", "MRDLK32", "MRDLK64", "MWR32", "MWR64",
    "IORD", "IOWR", "CFGRD0", "CFGWR0", "CFGRD1", "CFGWR1",
    "MSG", "MSGD", "CPL", "CPLD", "CPLLK", "CPLDLK", "FA32", "FA64",
    "SWP32", "SWP64", "CAS32", "CAS64", "LPRFX", "EPRFX", "ACK",
    "NAK", "PMEL1", "PMEL23", "PMASRL1", "PMRA", "VENDS", "IFC1P",
    "IFC1NP", "IFC1CPL", "IFC2P", "IFC2NP", "IFC2CPL", "UPDFCP",
    "UPDFCNP", "UPDFCCPL", "MRUPDFC", "MRIFC1", "MRIFC2",
    "MRINIT", "MRRESET", "UNK", "SKPOS", "EIOS", "FTSOS", "TS1OS",
    "TS2OS", "EIEOS", "COMPL", "MCOMPL", NULL
};
static ViString protocolPCIEPacketCompletionStatusArr[] = {"RESERVED", "SC", "UR", "CRS", "CA", NULL};
static ViString deembeddingComponentModeArr[] = {"PROB", "TERM", NULL};
static ViString deembeddingComponentImpedanceTypeArr[] = {"DEF", "CONS", "S1P", "S2P", NULL};
static ViString deembeddingPortArr[] = {"", "ONE", "TWO", "THR", "FOUR", NULL};
static ViString deembeddingPortLoadArr[] = {"", "SPON", "SPTW", NULL};
static ViString protocolDDRFrameStatusArr[] = {"OK", "UNKN", "INC", "LENG",NULL};
static ViString protocol100BASE_T1FrameStatusArr[] = {
    "OK", "ERR_PREAMBLE", "ERR_LENGTH", "ERR_SFD", "ERR_CRC", "UNC", "INC", NULL
};

static ViString protocol100BASE_T1FrameTypeArr[] = {"IDLE", "MAC", "FILL", "UNKN", NULL};
static ViString protocolMPHYFrameCellStateArr[] = {
    "OK", "RSVDERROR", "CMDERROR", "LENGTHERROR", "SYMBOLERROR", "UNKNOWNERROR", "INVALID", "INCOMPLETE", "UNC",
    "CRCERROR", "VOOR",NULL
};
static ViString measurementStatisticsResultArr[] = {"ACT", "AVG", "EVT", "NPE", "PPE", "RMS", "STDD", "WFMC", "REL"};
static ViString questionableRegisterArr[] = {
    "COV", "TEMP", "LIM", "MARG", "MASK", "ADCS", "FREQ", "LAMP", "PPL", "ZVC", "DEEM"
};
static ViString registerPartArr[] = {"COND", "ENAB", "EVEN", "NTR", "PTR"};

//This array maps values from the TriggerSource repcap
//(RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH1..COMMON2) to the corresponding values in
//rsscope_rngTriggerSource (RSSCOPE_VAL_CHANNEL1..COMMON2)
static ViInt32 TriggerSource_rng_to_repCap_map[] = {
    VI_FALSE,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH1, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH2,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH3, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH4,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_EXT, VI_FALSE,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_LINE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE,
    VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE,
    VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE,
    VI_FALSE, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1V1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1V2, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1V3,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1V4, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1I1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1I2, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1I3,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z1I4, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2V1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2V2, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2V3,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2V4, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2I1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2I2, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2I3,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_Z2I4, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_DIFF1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_DIFF2,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_COMMON1,
    RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_COMMON2
};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function:
 rsscope_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsscope_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsscope_init(ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == 0)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsscope_InitWithOptions(resourceName, IDQuery, resetDevice,
        "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsscope_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsscope_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr*)rsscope_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsscope", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        rsscope_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsscope_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "RTO2000", VI_FALSE: "RTPO" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSSCOPE_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.7.1
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 7, 1));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsscope_RsCoreRepCapTable, rsscope_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "NRP33SN", false : "NRP"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSSCOPE_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSSCOPE_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsscope_reset(instrSession));
    }
    else
    {
        checkErr(rsscope_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSSCOPE_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);
    return error;
}

/// HIFN  Selects the histogram on which the measurement is based.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the histogram on which the measurement is based.
ViStatus _VI_FUNC rsscope_SelectHistogram(ViSession instrSession,
                                          ViInt32   measurement,
                                          ViString  measurementType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_SELECT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_SELECT, measurementType),
            3, "Measurement Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the histogram measurement probability domain
/// HIFN  marker settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR reference/Defines the marker reference in the probability domain.
/// HIPAR delta/Defines a range around the probability marker.
ViStatus _VI_FUNC rsscope_ConfigureHistogramProbabilityDomainMarkerSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   reference,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_PROBABILITY_TYPE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_PROBABILITY_TYPE, reference),
            3, "Reference");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_PROBABILITY_LIMIT, delta),
            4, "Delta");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables limit checking for histogram measurements in the
/// HIFN  specified measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Histogram measurement type.
/// HIPAR limitCheck/Enables or disables limit checking for histogram measurements in the
/// HIPAR limitCheck/specified measurement channel.
ViStatus _VI_FUNC rsscope_ConfigureHistogramLimitCheck(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViInt32   measurementType,
                                                       ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  maxHistMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "HistogramMeasurementType", &maxHistMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_WAVEFORM_COUNT, maxHistMeasType),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LIMIT_CHECK,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LIMIT_CHECK,
            "HistogramMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_LIMIT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper limit for limit checking, respectively.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Histogram measurement type.
/// HIPAR lowerLimit/Define the lower limit for limit checking.
/// HIPAR upperLimit/Define the upper limit for limit checking.
ViStatus _VI_FUNC rsscope_ConfigureHistogramLimitCheckLimits(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementType,
                                                             ViReal64  lowerLimit,
                                                             ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  maxHistMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "HistogramMeasurementType", &maxHistMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_WAVEFORM_COUNT, maxHistMeasType),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_VALUE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_VALUE,
            "HistogramMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_VALUE, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_UPPER_LIMIT_VALUE, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper margins for the limit check, respectively.
/// HIFN  Margins are not as strict as limits and must be within the valid value
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Histogram measurement type.
/// HIPAR lowerMargin/Define the lower margin for the limit check. Margins are not as strict
/// HIPAR lowerMargin/as limits and must be within the valid value range.
/// HIPAR upperMargin/Define the upper margins for the limit check. Margins are not as
/// HIPAR upperMargin/strict as limits and must be within the valid value range.
ViStatus _VI_FUNC rsscope_ConfigureHistogramLimitCheckMargins(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViReal64  lowerMargin,
                            ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  maxHistMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "HistogramMeasurementType", &maxHistMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_WAVEFORM_COUNT, maxHistMeasType),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_MARGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_MARGIN,
            "HistogramMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_LOWER_LIMIT_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_UPPER_LIMIT_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays a histogram for the source of the selected
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR displaySourceHistogram/Displays a histogram for the source of the selected measurement.
ViStatus _VI_FUNC rsscope_ConfigureHistogramDisplay(ViSession instrSession,
                                                    ViInt32   measurement,
                                                    ViBoolean displaySourceHistogram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_DISPLAY,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_DISPLAY, displaySourceHistogram),
            3, "Display Source Histogram");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines and displays a new histogram for the specified source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/Defines the histogram name which is displayed in the Histogram
/// HIPAR histogramSource/Data source of the histogram
/// HIPAR relativeValues/Defines whether relative or absolute values are used for the value
/// HIPAR relativeValues/range definition.
/// HIPAR xStart/Defines the start value of the x-value range.
/// HIPAR xStop/Defines the stop value of the x-value range.
/// HIPAR yStart/Defines the start value of the y-value range.
/// HIPAR yStop/Defines the stop value of the y-value range.
/// HIPAR orientation/Sets the orientation of the new histogram.
ViStatus _VI_FUNC rsscope_CreateNewHistogram(ViSession instrSession,
                                             ViString  histogramName,
                                             ViInt32   histogramSource,
                                             ViBoolean relativeValues,
                                             ViReal64  xStart,
                                             ViReal64  xStop,
                                             ViReal64  yStart,
                                             ViReal64  yStop,
                                             ViInt32   orientation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, orientation, RSSCOPE_VAL_HISTOGRAM_VERTICAL, RSSCOPE_VAL_HISTOGRAM_HORIZONTAL),
            9, "Orientation");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngWaveformParameterHistogram, histogramSource, &p2string),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:ADD '%s',%s,%.12lG,%.12lG,%.12lG,%.12lG,%s,%s", histogramName,
             p2string,
             xStart,
             xStop,
             yStart,
             yStop,
             relativeValues ? "ON" : "OFF",
             orientationArr[orientation]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the source of the histogram. Any input signal, math or
/// HIFN  reference waveform can be selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
/// HIPAR histogramSource/Defines the source of the histogram. Any input signal, math or
/// HIPAR histogramSource/reference waveform can be selected.
ViStatus _VI_FUNC rsscope_ConfigureHistogramSource(ViSession instrSession,
                                                   ViString  histogramName,
                                                   ViInt32   histogramSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngWaveformParameterHistogram, histogramSource, &p2string),
                3, "Histogram Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:SOUR '%s',%s", histogramName, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the values to begin a new histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
ViStatus _VI_FUNC rsscope_ResetHistogram(ViSession instrSession,
                                         ViString  histogramName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_RESET_HISTOGRAM, histogramName),
            2, "Histogram Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the specified histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
ViStatus _VI_FUNC rsscope_RemoveHistogram(ViSession instrSession,
                                          ViString  histogramName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REMOVE_HISTOGRAM, histogramName),
            2, "Histogram Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of histogram
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
/// HIPAR type/Selects the type of histogram.
ViStatus _VI_FUNC rsscope_ConfigureHistogramType(ViSession instrSession,
                                                 ViString  histogramName,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:MODE '%s',%s", histogramName, type ? "HOR" : "VERT");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical settings of histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
/// HIPAR mode/Defines whether the vertical histogram settings are configured using
/// HIPAR mode/absolute or relative values.
/// HIPAR start/Defines the vertical start value of the histogram.
/// HIPAR stop/Defines the vertical stop value of the histogram.
ViStatus _VI_FUNC rsscope_ConfigureHistogramVertical(ViSession instrSession,
                                                     ViString  histogramName,
                                                     ViInt32   mode,
                                                     ViReal64  start,
                                                     ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:VERT:MODE '%s',%s", histogramName, mode ? "REL" : "ABS");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:VERT:%s:STAR '%s',%.12lG", mode ? "REL" : "ABS", histogramName,
             start);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:VERT:%s:STOP '%s',%.12lG", mode ? "REL" : "ABS", histogramName,
             stop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal settings of histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/The name of the histogram.
/// HIPAR mode/Defines whether the hozirontal histogram settings are configured using
/// HIPAR mode/absolute or relative values.
/// HIPAR start/Defines the hozirontal start value of the histogram.
/// HIPAR stop/Defines the hozirontal stop value of the histogram.
ViStatus _VI_FUNC rsscope_ConfigureHistogramHorizontal(ViSession instrSession,
                                                       ViString  histogramName,
                                                       ViInt32   mode,
                                                       ViReal64  start,
                                                       ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:HORZ:MODE '%s',%s", histogramName, mode ? "REL" : "ABS");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:HORZ:%s:STAR '%s',%.12lG", mode ? "REL" : "ABS", histogramName,
             start);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:HORZ:%s:STOP '%s',%.12lG", mode ? "REL" : "ABS", histogramName,
             stop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Enables or disables the histogram evaluation and display. The
///HINF  histogram settings are kept until the histogram is deleted.
///HIRET Returns the status code of this operation.
///HIPAR histogramName/The name of the histogram.
///HIPAR histogramState/Enables or disables the histogram evaluation
///HIPAR histogramState/and display.
ViStatus _VI_FUNC rsscope_ConfigureHistogramLayoutState(ViSession instrSession,
                                                        ViString  histogramName,
                                                        ViBoolean histogramState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:HIST:ENAB '%s',%s", histogramName, histogramState ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the gating.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR useGate/Enables gating.
ViStatus _VI_FUNC rsscope_ConfigureGateState(ViSession instrSession,
                                             ViInt32   measurement,
                                             ViBoolean useGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GATING_ENABLED, useGate),
            3, "Use Gate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is displayed in the
/// HIFN  source diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR showGate/Indicates the gate area in the source diagram.
ViStatus _VI_FUNC rsscope_ConfigureGateShow(ViSession instrSession,
                                            ViInt32   measurement,
                                            ViBoolean showGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_SHOW_GATE_AREA,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GATING_SHOW_GATE_AREA, showGate),
            3, "Show Gate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is defined identically
/// HIFN  to the zoom area for the selected active zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR zoomCoupling/If enabled, the gate area is defined identically to the zoom area for
/// HIPAR zoomCoupling/the zoom diagram.
/// HIPAR zoomDiagram/This control is reserved for future use.
ViStatus _VI_FUNC rsscope_ConfigureGateDefinitionZoom(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViBoolean zoomCoupling,
                                                      ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_ZOOM_COUPLING_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GATING_ZOOM_COUPLING_ENABLED, zoomCoupling),
            3, "Zoom Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If RSSCOPE_ATTR_GATING_ZOOM_COUPLING_ENABLED is enabled (with function
/// HIFN  rsscope_ConfigureGateDefinitionZoom), the gate area is defined
/// HIFN  identically to the zoom area for the selected zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR diagramName/Name of the diagram on which the zoom is based
/// HIPAR zoomName/Name of the zoom diagram
ViStatus _VI_FUNC rsscope_ConfigureGateZoomDiagram(ViSession instrSession,
                                                   ViInt32   measurement,
                                                   ViString  diagramName,
                                                   ViString  zoomName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:GATE:ZDI '%s','%s'", measurement, diagramName, zoomName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures cursor coupling. If enabled, the gate area is
/// HIFN  defined by the cursor lines of an active cursor measurement. If several
/// HIFN  cursor measurements are enabled, select the cursor set to be used for
/// HIFN  gating.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR coupling/Enables the cursor coupling for automatic measurements.
/// HIPAR cursorSet/Selects the cursor set to be used for measurement gating. The gate
/// HIPAR cursorSet/area is defined by the cursor lines.
ViStatus _VI_FUNC rsscope_ConfigureGateDefinitionCursor(ViSession instrSession,
                                                        ViInt32   measurement,
                                                        ViBoolean coupling,
                                                        ViInt32   cursorSet)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_CURSOR_COUPLING_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GATING_CURSOR_COUPLING_ENABLED, coupling),
            3, " Coupling");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_GATING_CURSOR_SET, cursorSet),
            4, " Cursor Set");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If you enable the gate coupling, the gate settings of the selected
/// HIFN  measurement are copied to all other measurements. If zoom or cursor
/// HIFN  coupling is active in a measurement, the zoom size and cursor positions
/// HIFN  are adjusted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR coupling/If you enable the gate coupling, the gate settings of the selected
/// HIPAR coupling/measurement are copied to all other measurements. If zoom or cursor
/// HIPAR coupling/coupling is active in a measurement, the zoom size and cursor positions
/// HIPAR coupling/are adjusted.
ViStatus _VI_FUNC rsscope_ConfigureGateCoupling(ViSession instrSession,
                                                ViInt32   measurement,
                                                ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_COUPLING_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GATING_COUPLING_ENABLED, coupling),
            3, " Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and end values for the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR start/Defines the starting value for the gate.
/// HIPAR stop/Defines the ending value for the gate.
ViStatus _VI_FUNC rsscope_ConfigureGateDefinitionStartStop(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   mode,
                                                           ViReal64  start,
                                                           ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_ABSOLUTE_STOP,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_GATING_MODE, mode),
            3, "Mode");

    switch (mode)
    {
    case RSSCOPE_VAL_ABS:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_GATING_ABSOLUTE_START, start),
                4, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_GATING_ABSOLUTE_STOP, stop),
                5, "Stop");
        break;

    case RSSCOPE_VAL_REL:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_GATING_RELATIVE_START, start),
                4, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_GATING_RELATIVE_STOP, stop),
                5, "Stop");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the noise evaluation area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR evaluationArea/Selects the noise evaluation area.
ViStatus _VI_FUNC rsscope_ConfigureGateNoise(ViSession instrSession,
                                             ViInt32   measurement,
                                             ViInt32   evaluationArea)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GATING_NOISE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_GATING_NOISE, evaluationArea),
            3, "EvaluationArea");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables long term measurement for a defined number of measurement
/// HIFN  points or a specified time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR longTermState/Enables long term measurement for a defined number of measurement
/// HIPAR longTermState/points or a specified time.
ViStatus _VI_FUNC rsscope_ConfigureLongTermState(ViSession instrSession,
                                                 ViInt32   measurement,
                                                 ViBoolean longTermState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_MEASUREMENT_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_MEASUREMENT_ENABLED, longTermState),
            3, "Long Term State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the total duration of the long term
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR totalMeasurementTime/Defines the total duration of the long term measurement.
ViStatus _VI_FUNC rsscope_ConfigureLongTermTotalMeasurementTime(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   totalMeasurementTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_DURATION,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_DURATION, totalMeasurementTime),
            3, "Total Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the total number of points to be measured during the long term
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementPoints/Defines the total number of points to be measured during the long term
/// HIPAR measurementPoints/measurement.
ViStatus _VI_FUNC rsscope_ConfigureLongTermMeasurementPoints(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_COUNT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_COUNT, measurementPoints),
            3, "Measurement Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical scale and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR verticalScale/Defines the vertical scaling per division, so that the scaling can be
/// HIPAR verticalScale/adapted automatically during the long term measurement period.
/// HIPAR verticalOffset/Defines a vertical offset for the long term measurement.
ViStatus _VI_FUNC rsscope_ConfigureLongTermVerticalScaling(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViReal64  verticalScale,
                                                           ViReal64  verticalOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_VERTICAL_SCALING,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_VERTICAL_SCALING, verticalScale),
            3, "Vertical Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_VERTICAL_OFFSET, verticalOffset),
            4, "Vertical Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures continuous auto scaling. If enabled,
/// HIFN  automatic vertical scaling is performed whenever the waveform does not
/// HIFN  fit in the diagram during the long term measurement period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR continuousAutoScale/If enabled, automatic vertical scaling is performed whenever the
/// HIPAR continuousAutoScale/waveform does not fit in the diagram during the long term measurement
/// HIPAR continuousAutoScale/period.
ViStatus _VI_FUNC rsscope_ConfigureLongTermVerticalContinuousAutoScaling(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean continuousAutoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_VERTICAL_CONTINUOUS_AUTO_SCALING,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_VERTICAL_CONTINUOUS_AUTO_SCALING, continuousAutoScale),
            3, "Continuous Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically adapts vertical scaling to the current measurement
/// HIFN  results during the long term measurement period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
ViStatus _VI_FUNC rsscope_LongTermVerticalAutoScale(ViSession instrSession,
                                                    ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_VERTICAL_AUTOSCALE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_VERTICAL_AUTOSCALE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables statistics calculation for the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR statistics/Enables statistics calculation for the measurement.
ViStatus _VI_FUNC rsscope_ConfigureStatisticsState(ViSession instrSession,
                                                   ViInt32   measurement,
                                                   ViBoolean statistics)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_ENABLED, statistics),
            3, "Statistics");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function displays histogram of the statistical results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR showHistogram/Enables statistics histogram for the measurement.
ViStatus _VI_FUNC rsscope_ConfigureHistogramState(ViSession instrSession,
                                                  ViInt32   measurement,
                                                  ViBoolean showHistogram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_HISTOGRAM_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_HISTOGRAM_ENABLED, showHistogram),
            3, "showHistogram");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the reset statistics settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR resetStatisticsMode/Defines when the statistics for long term measurements are reset.
/// HIPAR resetTime/Defines the time or period after which the statistics are reset.
/// HIPAR resetCount/Defines the number of measurements after which the statistics are
/// HIPAR resetCount/reset.
ViStatus _VI_FUNC rsscope_ConfigureResetStatisticsSettings(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   resetStatisticsMode,
                                                           ViReal64  resetTime,
                                                           ViInt32   resetCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_RESET_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_RESET_MODE, resetStatisticsMode),
            3, "Reset Statistics Mode");

    switch (resetStatisticsMode)
    {
    case RSSCOPE_VAL_STAT_MODE_TIME:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_RESET_TIME, resetTime),
                4, "Reset Time");

        break;

    case RSSCOPE_VAL_STAT_MODE_WAVEFORMS:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_RESET_COUNT, resetCount),
                5, "Reset Count");
        break;

    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of measurement results from which
/// HIFN  one point of the long term measurement is created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementsCount/Defines the number of measurement results from which one point of the
/// HIPAR measurementsCount/long term measurement is created.
ViStatus _VI_FUNC rsscope_ConfigureResetStatisticsMeasurementsCount(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementsCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_HISTOGRAM_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_RESET_MEASUREMENTS_COUNT, measurementsCount),
            3, "Measurements Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the statistics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
ViStatus _VI_FUNC rsscope_ResetStatistics(ViSession instrSession,
                                          ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_RESET,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the number of bins - the number of vertical bars
/// HIFN  that build the histogram. If 'Continuous auto scale' is enabled, the
/// HIFN  instrument determines the number of bins based on the time base, the
/// HIFN  current measurements, and other settings. To set the number of bins
/// HIFN  manually, disable 'Continuous auto scale'.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_STATISTICS_NUMBER_OF_BINS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:STATistics:HBINs
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR numberOfBins/Sets the number of bins - the number of vertical bars that build the
/// HIPAR numberOfBins/histogram. If 'Continuous auto scale' is enabled, the instrument
/// HIPAR numberOfBins/determines the number of bins based on the time base, the current
/// HIPAR numberOfBins/measurements, and other settings. To set the number of bins manually,
/// HIPAR numberOfBins/disable 'Continuous auto scale'.
ViStatus _VI_FUNC rsscope_ConfigureStatisticsNumberOfBins(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   numberOfBins)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_STATISTICS_NUMBER_OF_BINS,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_STATISTICS_NUMBER_OF_BINS, numberOfBins),
            3, "Number Of Bins");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the evaluation of a connected environment sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR useSensor/Enables the evaluation of a connected environment sensor.
ViStatus _VI_FUNC rsscope_ConfigureEnvironmentSensorState(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViBoolean useSensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_LONG_TERM_ENVIRONMENT_SENSOR_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_LONG_TERM_ENVIRONMENT_SENSOR_ENABLED, useSensor),
            3, "Use Sensor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines or queries the main measurement. This measurement is the one
/// HIFN  referred to if the measurement waveform is used as a source for math
/// HIFN  calculations. The main measurement type must belong to the same
/// HIFN  category as the other types assigned to the same measurement waveform,
/// HIFN  if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repcap Measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement. This measurement is the one
/// HIPAR measurementFunction/referred to if the measurement waveform is used as a source for math
/// HIPAR measurementFunction/calculations. The main measurement type must belong to the same
/// HIPAR measurementFunction/category as the other types assigned to the same measurement waveform,
/// HIPAR measurementFunction/if there are any.
ViStatus _VI_FUNC rsscope_ConfigureProtocolMainMeasurement(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSSCOPE_REPCAP_MEASUREMENT_MEAS1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MAIN_MEASUREMENT, measurementFunction),
            3, "Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds additional measurements to the indicated measurement group. Only
/// HIFN  one measurement can be enabled or disabled per command.
/// HIFN     Note that each measurement group must contain measurements from the
/// HIFN  same category. For example, if the main measurement of measurement
/// HIFN  group 1 is amplitude, then you cannot enable an eye width measurementin
/// HIFN  the same measurement group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repcap Measurement.
/// HIPAR measurementFunction/Defines additional measurement.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureProtocolAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2measurement;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSSCOPE_REPCAP_MEASUREMENT_MEAS1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngProtocolMeasurementType,
                    measurementFunction, &p2measurement),
                3, "Measurement Function");
    snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "%s,%s", p2measurement, enabled ? "ON" : "OFF");

    //MEAS{Measurement}:ADD
    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ADDITIONAL_MEASUREMENT, params),
            3, "Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame to frame protocol measurement. Valid values vary
/// HIFN  by protocol.
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_FRAME
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_FIELD
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_VALUE
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_TO_FRAME
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_TO_FIELD
/// HIFN     RSSCOPE_ATTR_FRAME_TO_FRAME_TO_VALUE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR fromFrame/Sets the type of start frame for the frame to frame protocol
/// HIPAR fromFrame/measurement. Valid values vary by protocol.
/// HIPAR fromField/Sets the type of field for the start frame of the frame to frame
/// HIPAR fromField/protocol measurement. Valid values vary by frame.
/// HIPAR fromValue/Sets the field value of the start frame for the frame to frame
/// HIPAR fromValue/protocol measurement.
/// HIPAR toFrame/Sets the type of end frame for the frame to frame protocol
/// HIPAR toFrame/measurement. Valid values vary by protocol.
/// HIPAR toField/Sets the type of field for the end frame of the frame to frame
/// HIPAR toField/protocol measurement. Valid values vary by frame.
/// HIPAR toValue/Sets the field value of the end frame for the frame to frame protocol
/// HIPAR toValue/measurement.
ViStatus _VI_FUNC rsscope_ConfigureFrameToFrame(ViSession instrSession,
                                                ViInt32   measurement,
                                                ViString  fromFrame,
                                                ViString  fromField,
                                                ViString  fromValue,
                                                ViString  toFrame,
                                                ViString  toField,
                                                ViString  toValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    if (fromFrame)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_FRAME, fromFrame),
                3, "From Frame");
    }

    if (fromField)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_FIELD, fromField),
                4, "From Field");
    }

    if (fromValue)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_FROM_VALUE, fromValue),
                5, "From Value");
    }

    if (toFrame)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_TO_FRAME, toFrame),
                6, "To Frame");
    }

    if (toField)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_TO_FIELD, toField),
                7, "To Field");
    }

    if (toValue)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FRAME_TO_FRAME_TO_VALUE, toValue),
                8, "To Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures trigger to frame protocol measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR direction/Sets the direction of the field for trigger to frame protocol
/// HIPAR direction/measurement.
/// HIPAR frame/Sets the value of the frame for trigger to frame protocol measurement.
/// HIPAR frame/Valid values vary by protocol.
/// HIPAR field/Sets the type of the field for trigger to frame protocol measurement.
/// HIPAR field/Valid values vary by frame.
/// HIPAR value/Sets the value of the field for trigger to frame protocol measurement.
ViStatus _VI_FUNC rsscope_ConfigureTriggerToFrame(ViSession instrSession,
                                                  ViInt32   measurement,
                                                  ViInt32   direction,
                                                  ViString  frame,
                                                  ViString  field,
                                                  ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_DIRECTION, direction),
            3, "Direction");

    if (frame)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_FRAME, frame),
                4, "Frame");
    }

    if (field)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_FIELD, field),
                5, "Field");
    }

    if (value)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_VALUE, value),
                6, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the field value protocol measurement. Valid values vary by
/// HIFN  protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR frame/Sets the type frame for the field value protocol measurement. Valid
/// HIPAR frame/values vary by protocol.
/// HIPAR field/Sets the type of field for the frame of the field value protocol
/// HIPAR field/measurement. Valid values vary by frame.
/// HIPAR value/Sets the field value of the frame for the field value protocol
/// HIPAR value/measurement.
/// HIPAR fieldToBeTracked/Sets the field to be tracked value of the frame for the field value
/// HIPAR fieldToBeTracked/protocol measurement.
ViStatus _VI_FUNC rsscope_ConfigureFieldValueMeasurement(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViString  frame,
                                                         ViString  field,
                                                         ViString  value,
                                                         ViString  fieldToBeTracked)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    if (frame)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_FRAME, frame),
                3, "Frame");
    }

    if (field)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_FIELD, field),
                4, "Field");
    }

    if (value)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TO_FRAME_VALUE, value),
                5, "Value");
    }

    if (fieldToBeTracked)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FIELD_VALUE_FIELD_TO_BE_TRACKED, fieldToBeTracked),
                6, "Field To Be Tracked");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the field value protocol measurement. Valid values vary by
/// HIFN  protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR name/Sets the type frame for the main bit rate protocol measurement.
/// HIPAR field/Sets the type of field for the frame of the main bit rate protocol
/// HIPAR field/measurement.
/// HIPAR value/Sets the field value of the frame for the main bit rate protocol
/// HIPAR value/measurement.
ViStatus _VI_FUNC rsscope_ConfigureMainBitRate(ViSession instrSession,
                                               ViInt32   measurement,
                                               ViString  name,
                                               ViString  field,
                                               ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MAIN_BIT_RATE_FRAME_NAME,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    if (name)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MAIN_BIT_RATE_FRAME_NAME, name),
                3, "Name");
    }

    if (field)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MAIN_BIT_RATE_FIELD_NAME, field),
                4, "Field");
    }

    if (value)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MAIN_BIT_RATE_ID_VALUE, value),
                5, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the field value protocol measurement. Valid values vary by
/// HIFN  protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR name/Sets the type frame for the secondary bit rate protocol measurement.
/// HIPAR field/Sets the type of field for the frame of the secondary bit rate
/// HIPAR field/protocol measurement.
/// HIPAR value/Sets the field value of the frame for the secondary bit rate protocol
/// HIPAR value/measurement.
ViStatus _VI_FUNC rsscope_ConfigureSecondaryBitRate(ViSession instrSession,
                                                    ViInt32   measurement,
                                                    ViString  name,
                                                    ViString  field,
                                                    ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SECONDARY_BIT_RATE_FRAME_NAME,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    if (name)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SECONDARY_BIT_RATE_FRAME_NAME, name),
                3, "Name");
    }

    if (field)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SECONDARY_BIT_RATE_FIELD_NAME, field),
                4, "Field");
    }

    if (value)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SECONDARY_BIT_RATE_ID_VALUE, value),
                5, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the reference level settings.
/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevel instead. Note that the
/// HIFN  values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevel.
/// HIFN     This function configures the reference level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR referenceLevelMode/Defines whether the reference level is configured manually or
/// HIPAR referenceLevelMode/automatically.
/// HIPAR levelDefinition/Defines whether the reference is configured using absolute or relative
/// HIPAR levelDefinition/values.
/// HIPAR userLevelSelection/Defines whether the user-defined signal levels or user-defined
/// HIPAR userLevelSelection/reference levels are used for the measurements.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevel(ViSession instrSession,
                                                  ViInt32   source,
                                                  ViInt32   referenceLevelMode,
                                                  ViInt32   levelDefinition,
                                                  ViInt32   userLevelSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE, referenceLevelMode),
            3, "Reference Level Mode");

    if (referenceLevelMode == RSSCOPE_VAL_MANUAL)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_DEFINITION, levelDefinition),
                4, "Level Definition");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_USER_LEVEL, userLevelSelection),
                5, "User Level Selection");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelAutoSettings instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelAutoSettings.
/// HIFN     This function configures the reference level automatic configuration
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR signalLevelMode/Defines which signal level is used as a reference.
/// HIPAR histogramAveraging/Enables averaging over several histograms to determine the reference
/// HIPAR histogramAveraging/levels.
/// HIPAR averageCount/Defines the number of histograms to calculate the average from.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelAutoSettings(
                            ViSession instrSession,
                            ViInt32   source,
                            ViInt32   signalLevelMode,
                            ViBoolean histogramAveraging,
                            ViInt32   averageCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AUTO_MODE, signalLevelMode),
            3, "Signal Level Mode");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AVERAGING_ENABLED, histogramAveraging),
            4, "Histogram Averaging");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AVERAGING_COUNT, averageCount),
            5, "Average Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceUserSignalLevel instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceUserSignalLevel.
/// HIFN     This function configures the user signal level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR topDistance/The distance between the high signal level and the upper reference
/// HIPAR topDistance/level.
/// HIPAR bottomDistance/The distance between the lower reference level and the low signal
/// HIPAR bottomDistance/value.
/// HIPAR highSignalLevel/The signal value that represents a high level.
/// HIPAR lowSignalLevel/The signal value that represents a high level.
ViStatus _VI_FUNC rsscope_ConfigureReferenceUserSignalLevel(ViSession instrSession,
                                                            ViInt32   source,
                                                            ViReal64  topDistance,
                                                            ViReal64  bottomDistance,
                                                            ViReal64  highSignalLevel,
                                                            ViReal64  lowSignalLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE, topDistance),
            3, "Top Distance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_BOTTOM_DISTANCE, bottomDistance),
            4, "Bottom Distance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_SIGNAL_HIGH, highSignalLevel),
            5, "High Signal Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_SIGNAL_LOW, lowSignalLevel),
            6, "Low Signal Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformUserReferenceLevel instead. Note that the
/// HIFN  values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformUserReferenceLevel.
/// HIFN     This function configures the user reference level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR topDistance/The distance between the high signal level and the upper reference
/// HIPAR topDistance/level.
/// HIPAR bottomDistance/The distance between the lower reference level and the low signal
/// HIPAR bottomDistance/value.
/// HIPAR upperLevel/The upper reference level, required e.g. to determine a rise.
/// HIPAR lowerLevel/The lower reference level, required e.g. to determine a fall.
ViStatus _VI_FUNC rsscope_ConfigureUserReferenceLevel(ViSession instrSession,
                                                      ViInt32   source,
                                                      ViReal64  topDistance,
                                                      ViReal64  bottomDistance,
                                                      ViReal64  upperLevel,
                                                      ViReal64  lowerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_UPPER_LEVEL, upperLevel),
            5, "Upper Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_LOWER_LEVEL, lowerLevel),
            6, "Lower Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE, topDistance),
            3, "Top Distance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_BOTTOM_DISTANCE, bottomDistance),
            4, "Bottom Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformUserMiddleReferenceLevel instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformUserMiddleReferenceLevel.
/// HIFN     This function configures the user middle reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR middleLevel/The middle reference level.
ViStatus _VI_FUNC rsscope_ConfigureUserMiddleReferenceLevel(ViSession instrSession,
                                                            ViInt32   source,
                                                            ViReal64  middleLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_MIDDLE_LEVEL, middleLevel),
            3, "Middle Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelRelativeLevels instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelRelativeLevels.
/// HIFN     This function configures the lower, middle and upper reference levels,
/// HIFN  defined as percentages of the high signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR relativeLevels/The lower, middle and upper reference levels, defined as percentages
/// HIPAR relativeLevels/of the high signal level.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelRelativeLevels(
                            ViSession instrSession,
                            ViInt32   source,
                            ViInt32   relativeLevels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_RELATIVE_LEVELS, relativeLevels),
            3, "Relative Levels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelRelativeUserLevels instead.
/// HIFN  Note that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelRelativeUserLevels.
/// HIFN     This function configures the lower, middle and upper reference levels,
/// HIFN  defined as percentages of the high signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR upperLevel/The upper relative reference level.
/// HIPAR middleLevel/The middle relative reference level.
/// HIPAR lowerLevel/The middle relative reference level.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelRelativeUserLevels(
                            ViSession instrSession,
                            ViInt32   source,
                            ViInt32   upperLevel,
                            ViInt32   middleLevel,
                            ViInt32   lowerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_RELATIVE_LEVELS, RSSCOPE_VAL_USER_LEVEL));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_LOWER_LEVEL, lowerLevel),
            5, "Lower Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_MIDDLE_LEVEL, middleLevel),
            4, "Middle Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_UPPER_LEVEL, upperLevel),
            3, "Upper Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelHysteresis instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelHysteresis.
/// HIFN     This function configures the hysteresis for measurements that
/// HIFN  determine zero-crossings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR hysteresis/Defines a hysteresis for the middle reference level. A rise or fall
/// HIPAR hysteresis/from the middle reference value that does not exceed the hysteresis is
/// HIPAR hysteresis/rejected as noise.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelHysteresis(ViSession instrSession,
                                                            ViInt32   source,
                                                            ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_HYSTERESIS, hysteresis),
            3, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelTube instead. Note that the
/// HIFN  values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelTube.
/// HIFN     This function configures an area of signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR topOuter/Defines an area above the high signal level which is still considered
/// HIPAR topOuter/to be high level.
/// HIPAR topInner/Defines an area beneath the high signal level which is still
/// HIPAR topInner/considered to be high level.
/// HIPAR bottomInner/Defines an area above the low signal level which is still considered
/// HIPAR bottomInner/to be low level.
/// HIPAR bottomOuter/Defines an area beneath the low signal level which is still considered
/// HIPAR bottomOuter/to be low level.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelTube(ViSession instrSession,
                                                      ViInt32   source,
                                                      ViReal64  topOuter,
                                                      ViReal64  topInner,
                                                      ViReal64  bottomInner,
                                                      ViReal64  bottomOuter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_TOP_OUTER_DISTANCE, topOuter),
            3, "Top Outer");

    //snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,TopInner", repCap);
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_TOP_INNER_DISTANCE, topInner),
            4, "Top Inner");

    //snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,BottomOuter", repCap);
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_BOTTOM_OUTER_DISTANCE, bottomInner),
            5, "Bottom Inner");

    //snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,BottomInner", repCap);
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_BOTTOM_INNER_DISTANCE, bottomOuter),
            6, "Bottom Outer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformReferenceLevelTubeRelative instead. Note
/// HIFN  that the values of the second parameter are not the same in
/// HIFN  rsscope_ConfigureWaveformReferenceLevelTubeRelative.
/// HIFN     This function configures a percentage of the signal level by which the
/// HIFN  absolute signal level may be different than the signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source from which the reference is taken.
/// HIPAR relativeOuter/Defines a percentage of the signal level by which the absolute signal
/// HIPAR relativeOuter/level may be larger than the high signal level or lower than the low
/// HIPAR relativeOuter/signal level to be considered high or low, respectively.
/// HIPAR relativeInner/Defines a percentage of the signal level by which the absolute signal
/// HIPAR relativeInner/level may be higher than the low signal level or lower than the high
/// HIPAR relativeInner/signal level to be considered low or high, respectively.
ViStatus _VI_FUNC rsscope_ConfigureReferenceLevelTubeRelative(
                            ViSession instrSession,
                            ViInt32   source,
                            ViInt32   relativeOuter,
                            ViInt32   relativeInner)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_C1W1, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    /*
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
                "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
                2, "source");
    */
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", sourceRepCapArr[source]);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_RELATIVE_OUTER_DISTANCE, (ViReal64)relativeOuter),
                3, "Relative Outer");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_RELATIVE_INNER_DISTANCE, (ViReal64)relativeInner),
                4, "Relative Inner");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the reference level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR referenceLevelMode/Defines whether the reference level is configured manually or
/// HIPAR referenceLevelMode/automatically.
/// HIPAR levelDefinition/Defines whether the reference is configured using absolute or relative
/// HIPAR levelDefinition/values.
/// HIPAR userLevelSelection/Defines whether the user-defined signal levels or user-defined
/// HIPAR userLevelSelection/reference levels are used for the measurements.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevel(ViSession instrSession,
                                                          ViInt32   waveform,
                                                          ViInt32   referenceLevelMode,
                                                          ViInt32   levelDefinition,
                                                          ViInt32   userLevelSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_MODE, referenceLevelMode),
            3, "Reference Level Mode");

    if (referenceLevelMode == RSSCOPE_VAL_MANUAL)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_DEFINITION, levelDefinition),
                4, "Level Definition");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_USER_LEVEL, userLevelSelection),
                5, "User Level Selection");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the reference level automatic configuration
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR signalLevelMode/Defines which signal level is used as a reference.
/// HIPAR histogramAveraging/Enables averaging over several histograms to determine the reference
/// HIPAR histogramAveraging/levels.
/// HIPAR averageCount/Defines the number of histograms to calculate the average from.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelAutoSettings(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViInt32   signalLevelMode,
                            ViBoolean histogramAveraging,
                            ViInt32   averageCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_AUTO_MODE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AUTO_MODE, signalLevelMode),
            3, "Signal Level Mode");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AVERAGING_ENABLED, histogramAveraging),
            4, "Histogram Averaging");

    if (histogramAveraging)
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_AVERAGING_COUNT, averageCount),
                5, "Average Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the user signal level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR topDistance/The distance between the high signal level and the upper reference
/// HIPAR topDistance/level.
/// HIPAR bottomDistance/The distance between the lower reference level and the low signal
/// HIPAR bottomDistance/value.
/// HIPAR highSignalLevel/The signal value that represents a high level.
/// HIPAR lowSignalLevel/The signal value that represents a high level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceUserSignalLevel(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  topDistance,
                            ViReal64  bottomDistance,
                            ViReal64  highSignalLevel,
                            ViReal64  lowSignalLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_SIGNAL_HIGH, highSignalLevel),
            5, "High Signal Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_SIGNAL_LOW, lowSignalLevel),
            6, "Low Signal Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE, topDistance),
            3, "Top Distance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_BOTTOM_DISTANCE, bottomDistance),
            4, "Bottom Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the user reference level settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR topDistance/The distance between the high signal level and the upper reference
/// HIPAR topDistance/level.
/// HIPAR bottomDistance/The distance between the lower reference level and the low signal
/// HIPAR bottomDistance/value.
/// HIPAR upperLevel/The upper reference level, required e.g. to determine a rise.
/// HIPAR lowerLevel/The lower reference level, required e.g. to determine a fall.
ViStatus _VI_FUNC rsscope_ConfigureWaveformUserReferenceLevel(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  topDistance,
                            ViReal64  bottomDistance,
                            ViReal64  upperLevel,
                            ViReal64  lowerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_UPPER_LEVEL, upperLevel),
            5, "Upper Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_LOWER_LEVEL, lowerLevel),
            6, "Lower Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_TOP_DISTANCE, topDistance),
            3, "Top Distance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_USER_BOTTOM_DISTANCE, bottomDistance),
            4, "Bottom Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the user middle reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR middleLevel/The middle reference level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformUserMiddleReferenceLevel(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  middleLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USER_REFERENCE_MIDDLE_LEVEL,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_MIDDLE_LEVEL, middleLevel),
            3, "Middle Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the lower, middle and upper reference levels,
/// HIFN  defined as percentages of the high signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR relativeLevels/The lower, middle and upper reference levels, defined as percentages
/// HIPAR relativeLevels/of the high signal level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelRelativeLevels(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViInt32   relativeLevels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_LEVEL_RELATIVE_LEVELS,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_LEVEL_RELATIVE_LEVELS, relativeLevels),
            3, "Relative Levels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the lower, middle and upper reference levels,
/// HIFN  defined as percentages of the high signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR upperLevel/The upper relative reference level.
/// HIPAR middleLevel/The middle relative reference level.
/// HIPAR lowerLevel/The middle relative reference level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelRelativeUserLevels(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViInt32   upperLevel,
                            ViInt32   middleLevel,
                            ViInt32   lowerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_UPPER_LEVEL,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_LOWER_LEVEL, lowerLevel),
            5, "Lower Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_MIDDLE_LEVEL, middleLevel),
            4, "Middle Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_RELATIVE_UPPER_LEVEL, upperLevel),
            3, "Upper Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the hysteresis for measurements that
/// HIFN  determine zero-crossings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR hysteresis/Defines a hysteresis for the middle reference level. A rise or fall
/// HIPAR hysteresis/from the middle reference value that does not exceed the hysteresis is
/// HIPAR hysteresis/rejected as noise.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelHysteresis(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USER_REFERENCE_HYSTERESIS,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_USER_REFERENCE_HYSTERESIS, hysteresis),
            3, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures an area of signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR topOuter/Defines an area above the high signal level which is still considered
/// HIPAR topOuter/to be high level.
/// HIPAR topInner/Defines an area beneath the high signal level which is still
/// HIPAR topInner/considered to be high level.
/// HIPAR bottomInner/Defines an area above the low signal level which is still considered
/// HIPAR bottomInner/to be low level.
/// HIPAR bottomOuter/Defines an area beneath the low signal level which is still considered
/// HIPAR bottomOuter/to be low level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelTube(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  topOuter,
                            ViReal64  topInner,
                            ViReal64  bottomInner,
                            ViReal64  bottomOuter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TUBE_ABSOLUTE_TOP_OUTER_DISTANCE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_TOP_OUTER_DISTANCE, topOuter),
            3, "Top Outer");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_TOP_INNER_DISTANCE, topInner),
            4, "Top Inner");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_BOTTOM_OUTER_DISTANCE, bottomOuter),
            5, "Bottom Inner");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_ABSOLUTE_BOTTOM_INNER_DISTANCE, bottomInner),
            6, "Bottom Outer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures a percentage of the signal level by which the
/// HIFN  absolute signal level may be different than the signal level.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_TUBE_RELATIVE_OUTER_DISTANCE
/// HIFN     RSSCOPE_ATTR_TUBE_RELATIVE_INNER_DISTANCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     REFLevel<m>:RELative:OTUBe
/// HIFN     REFLevel<m>:RELative:ITUBe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Defines the source from which the reference is taken.
/// HIPAR relativeOuter/Defines a percentage of the signal level by which the absolute signal
/// HIPAR relativeOuter/level may be larger than the high signal level or lower than the low
/// HIPAR relativeOuter/signal level to be considered high or low, respectively.
/// HIPAR relativeInner/Defines a percentage of the signal level by which the absolute signal
/// HIPAR relativeInner/level may be higher than the low signal level or lower than the high
/// HIPAR relativeInner/signal level to be considered low or high, respectively.
ViStatus _VI_FUNC rsscope_ConfigureWaveformReferenceLevelTubeRelative(
                            ViSession instrSession,
                            ViInt32   waveform,
                            ViReal64  relativeOuter,
                            ViReal64  relativeInner)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1, RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK2),
            2, "Waveform");
    if (((RSSCOPE_REPCAP_WFMSUFFIX_WFMREF4 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMTRK8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMDIFF1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMMATH8 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK1)) ||
        ((RSSCOPE_REPCAP_WFMSUFFIX_WFMRJTRACK1 < waveform) && (waveform < RSSCOPE_REPCAP_WFMSUFFIX_WFMSRTRACK2))
    )
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveform),
                    2, "Waveform");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TUBE_RELATIVE_OUTER_DISTANCE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_RELATIVE_OUTER_DISTANCE, relativeOuter),
            3, "Relative Outer");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TUBE_RELATIVE_INNER_DISTANCE, relativeInner),
            4, "Relative Inner");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the record length, the number of recorded
/// HIFN  waveform points that build the waveform across the acquisition time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR actualRecordLength/Returns the actual number of points the oscilloscope acquires for each
/// HIPAR actualRecordLength/channel.  The value is equal to or greater than the minimum number of
/// HIPAR actualRecordLength/points you specify in the rsscope_ConfigureAcquisitionRecord function.
ViStatus _VI_FUNC rsscope_ActualRecordLength(ViSession instrSession,
                                             ViInt32*  actualRecordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_RECORD_LENGTH, actualRecordLength),
            2, "Actual Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the sample mode the oscilloscope is currently
/// HIFN  using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleMode/Returns the method of adding waveform points to the samples of the ADC
/// HIPAR sampleMode/in order to fill the record length.
ViStatus _VI_FUNC rsscope_SampleMode(ViSession instrSession,
                                     ViInt32*  sampleMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAMPLE_MODE, sampleMode),
            2, "SampleMode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the effective sample rate of the acquired
/// HIFN  waveform using the current configuration in samples per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Return the sample rate, that is the number of recorded waveform
/// HIPAR sampleRate/samples per second.
ViStatus _VI_FUNC rsscope_SampleRate(ViSession instrSession,
                                     ViReal64* sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the attenuation of the probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR autoProbeSenseValue/Queries the attenuation of the probe.
ViStatus _VI_FUNC rsscope_AutoProbeSenseValue(ViSession instrSession,
                                              ViInt32   channel,
                                              ViReal64* autoProbeSenseValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_ATTENUATION,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_ATTENUATION, autoProbeSenseValue),
            3, "Auto Probe Sense Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs an autoset process: analyzes the enabled channel signals, and obtains appropriate horizontal, vertical, and trigger settings to display stable waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsscope_Autoset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_AUTOSCALE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures how the oscilloscope acquires data and fills the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR acquisitionType/Specify the manner in which you want the oscilloscope to acquire data and fill the waveform record.
ViStatus _VI_FUNC rsscope_ConfigureAcquisitionType(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   waveform,
                                                   ViInt32   acquisitionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ACQUISITION_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ACQUISITION_TYPE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_w),
            3, "Waveform");
    strcat(repCap, ",");
    strcat(repCap, repCap_w);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ACQUISITION_TYPE, acquisitionType),
            4, "Acquisition Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common attributes of the acquisition subsystem.  These attributes are the time per record and acquisition start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timePerRecord/Pass the time in seconds that corresponds to the record length.
/// HIPAR acquisitionStartTime_Offset/Specifies the length of time from the trigger event to the first point in the waveform record.  If the value is positive, the first point in the waveform record occurs after the trigger event.  If the
///           value is negative, the first point in the waveform record occurs before the trigger event.
ViStatus _VI_FUNC rsscope_ConfigureAcquisitionRecord(ViSession instrSession,
                                                     ViReal64  timePerRecord,
                                                     ViReal64  acquisitionStartTime_Offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_TIME_PER_RECORD, timePerRecord),
            2, "Time Per Record");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_ACQUSITION_START_TIME, acquisitionStartTime_Offset),
            3, "Acqusition Start Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the current number of acquisitions that have been
/// HIFN  acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfAcquisitions/Shows the current number of acquisitions that have been acquired.
ViStatus _VI_FUNC rsscope_QueryNumberOfAcquisitions(ViSession instrSession,
                                                    ViInt32*  numberOfAcquisitions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NUMBER_OF_ACQUISITIONS, numberOfAcquisitions),
            2, "Number Of Acquisitions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the basic settings for the time axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timeScale/Sets the horizontal scale - the time per division on the x-axis - for all channel and math waveforms.
/// HIPAR referencePoint/Sets the reference point of the time scale in % of the display.
/// HIPAR restrictOffset/Ensures that the trigger occurs within one acquisition cycle.
ViStatus _VI_FUNC rsscope_ConfigureTimeBase(ViSession instrSession,
                                            ViReal64  timeScale,
                                            ViInt32   referencePoint,
                                            ViBoolean restrictOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_SCALE, timeScale),
            2, "Time Scale");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_REFERENCE, referencePoint),
            3, "Reference Point");

    if (RsCore_IsInstrumentModel(instrSession, "RTO"))
    {
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_ACQUISITION_RESTRICT_OFFSET, restrictOffset),
                4, "Restrict Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the roll mode settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR mode/Activates the automatic roll mode.
/// HIPAR minRollModeGain/The roll mode is enabled automatically if the acquisition time exceeds the given value, and if the Mode is set to AUTO.
ViStatus _VI_FUNC rsscope_ConfigureRollMode(ViSession instrSession,
                                            ViInt32   mode,
                                            ViInt32   minRollModeGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_ROLL_MODE_AUTO, mode),
            2, "Roll Mode Auto");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_ROLL_MODE_THRESHOLD, minRollModeGain),
            3, "Roll Mode Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of horizontal divisions on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR divisions/Queries the number of horizontal divisions on the screen
ViStatus _VI_FUNC rsscope_QueryDivisions(ViSession instrSession,
                                         ViInt32*  divisions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_DIVISIONS, divisions),
            2, "Divisions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the selection to change either the resolution or the record length. The other value remains constant and cannot be changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timeSelectDependency/Selection to change either the resolution or the record length.
ViStatus _VI_FUNC rsscope_ConfigureTimeSelectDependency(ViSession instrSession,
                                                        ViInt32   timeSelectDependency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_RECORD_SELECTION, timeSelectDependency),
            2, "Time Select Dependency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Prevents undersampling and ensures a sufficient resolution to acquire
/// HIFN  the correct waveform if the time scale is changed. The setting takes
/// HIFN  effect if the changed parameter - resolution or record length - reaches
/// HIFN  a limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAdjustment/Prevents undersampling and ensures a sufficient resolution to acquire
/// HIPAR autoAdjustment/the correct waveform if the time scale is changed. The setting takes
/// HIPAR autoAdjustment/effect if the changed parameter - resolution or record length - reaches
/// HIPAR autoAdjustment/a limit.
ViStatus _VI_FUNC rsscope_ConfigureAutoAdjustment(ViSession instrSession,
                                                  ViBoolean autoAdjustment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_ACQUISITION_AUTO_ADJUSTMENT, autoAdjustment),
            2, "Auto Adjustment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the sample rate, that is the number of recorded waveform samples per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR sampleRate/Sets the sample rate, that is the number of recorded waveform samples per second.
ViStatus _VI_FUNC rsscope_ConfigureSampleRate(ViSession instrSession,
                                              ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the time between two waveform points in the record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR resolution/Sets the time between two waveform points in the record.
ViStatus _VI_FUNC rsscope_ConfigureResolution(ViSession instrSession,
                                              ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_RESOLUTION, resolution),
            2, "Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the sample rate of the ADC, that is the number of points that are sampled by the ADC in one second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR adcSampleRate/Returns the sample rate of the ADC, that is the number of points that are sampled by the ADC in one second.
ViStatus _VI_FUNC rsscope_QueryADCSampleRate(ViSession instrSession,
                                             ViReal64* adcSampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_ADC_SAMPLE_RATE, adcSampleRate),
            2, "ADC Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the record length, the number of recorded waveform points that build the waveform across the acquisition time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR recordLength/Sets the record length, the number of recorded waveform
ViStatus _VI_FUNC rsscope_ConfigureRecordLength(ViSession instrSession,
                                                ViInt32   recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_RECORD_LENGTH, recordLength),
            2, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the limit for the record length to prevent
/// HIFN  very large records. If you increase the time scale, the resolution
/// HIFN  remains constant and the record length increases until the limit is
/// HIFN  reached. Further increase of the time scale changes the resolution and
/// HIFN  keeps the record length limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lengthLimit/Sets a limit for the record length.
ViStatus _VI_FUNC rsscope_ConfigureRecordLengthLimit(ViSession instrSession,
                                                     ViInt32   lengthLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HORZ_RECORD_LENGTH_LIMIT, lengthLimit),
            2, "Length Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of captured waveform points per second. It considers
/// HIFN  the samples of the ADC, and the reduction of waveform points by
/// HIFN  decimation.
/// HIFN     If interpolation is not active, the sample rate is the reciprocal
/// HIFN  value of the resolution and thus also depends on the acquisition time
/// HIFN  and the record length.
/// HIFN     If interpolation is active, the sample rate is limited to the ADC
/// HIFN  sample rate.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ACQUISITION_SAMPLE_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ACQuire:SRReal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionSampleRate/Sets the number of captured waveform points per second. It considers
/// HIPAR acquisitionSampleRate/the samples of the ADC, and the reduction of waveform points by
/// HIPAR acquisitionSampleRate/decimation.
ViStatus _VI_FUNC rsscope_ConfigureAcquisitionSampleRate(ViSession instrSession,
                                                         ViReal64  acquisitionSampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_ACQUISITION_SAMPLE_RATE, acquisitionSampleRate),
            2, "Acquisition Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the time of one acquisition, that is the time across the 10 divisions of the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR acquisitionTime/Sets the time of one acquisition.
ViStatus _VI_FUNC rsscope_ConfigureAcquisitionTime(ViSession instrSession,
                                                   ViReal64  acquisitionTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HORZ_TIME_PER_RECORD, acquisitionTime),
            2, "Acquisition Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the method of adding waveform points to the samples of the ADC in order to fill the record length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR resolutionEnhancement/Selects the method of adding waveform points to the samples of the ADC in order to fill the record length.
ViStatus _VI_FUNC rsscope_ConfigureResolutionEnhancement(ViSession instrSession,
                                                         ViInt32   resolutionEnhancement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAMPLE_MODE, resolutionEnhancement),
            2, "Resolution Enhancement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the interpolation method the oscilloscope uses when it cannot sample a voltage for a point in the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR interpolation/Pass the interpolation method you want the oscilloscope to
ViStatus _VI_FUNC rsscope_ConfigureInterpolation(ViSession instrSession,
                                                 ViInt32   interpolation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_INTERPOLATION, interpolation),
            2, "Interpolation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the individual waveforms of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR enableWaveform/Activates or deactivates a waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformState(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   waveform,
                                                 ViBoolean enableWaveform)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wfm[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_STATE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wfm),
            3, "Waveform");

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_wfm);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_STATE, enableWaveform),
            3, "Enable Waveform");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the method to reduce the data stream of the ADC to a stream of waveform points with lower sample rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR arithmetic/Selects the method to reduce the data stream of the ADC to a stream of waveform points with lower sample rate.
ViStatus _VI_FUNC rsscope_ConfigureWaveformArithmetic(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   waveform,
                                                      ViInt32   arithmetic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_ACQUISITION_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_ACQUISITION_TYPE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_w),
            3, "Waveform");
    strcat(repCap, ",");
    strcat(repCap, repCap_w);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_ACQUISITION_TYPE, arithmetic),
            4, "Arithmetic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of waveforms that the oscilloscope  acquires and averages.  After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR numberofAverages/Pass the number of waveforms you want the oscilloscope to acquire and to average. After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.
ViStatus _VI_FUNC rsscope_ConfigureNumberOfAverages(ViSession instrSession,
                                                    ViInt32   numberofAverages)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NUM_AVERAGES, numberofAverages),
            2, "Number of Averages");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ultra segmentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR ultraSegmentationEnabled/Switches the Ultra Segmentation mode on and off.
/// HIPAR ultraSegmentationDependecy/The number of acquisitions in a Ultra Segmentation acquisition series depends on the record length.
ViStatus _VI_FUNC rsscope_ConfigureUltraSegmentation(ViSession instrSession,
                                                     ViBoolean ultraSegmentationEnabled,
                                                     ViBoolean ultraSegmentationDependecy)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_ULTRA_SEGMENTATION_ENABLED, ultraSegmentationEnabled),
            2, "Ultra Segmentation Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_ULTRA_SEGMENTATION_DEPENDENCY, ultraSegmentationDependecy),
            3, "Ultra Segmentation Dependency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the instrument starts processing and displaying the data
/// HIFN  as soon as the acquisition series is captured completely. Depending on
/// HIFN  the number of acquisitions, it may take some time until the acquisition
/// HIFN  series is displayed. If the setting is disabled, the instrument only
/// HIFN  captures the data and stores it in the sample memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoReplay/Switches auto replay On or Off.
ViStatus _VI_FUNC rsscope_ConfigureUltraSegmentationAutoReplay(
                            ViSession instrSession,
                            ViBoolean autoReplay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_ULTRA_SEGMENTATION_AUTO_REPLAY, autoReplay),
            2, "Auto Replay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the acquisition mode and the waveform arithmetic of all channels
/// HIFN  to the last set value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_WAVEFORM_ACQUISITION_COUPLE_CHANNELS_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ACQuire:CMODode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoReplay/Sets the acquisition mode and the waveform arithmetic of all channels
/// HIPAR autoReplay/to the last set value.
ViStatus _VI_FUNC rsscope_ConfigureWaveformAcquisitionCoupleChannelsEnabled(
                            ViSession instrSession,
                            ViBoolean autoReplay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_ACQUISITION_COUPLE_CHANNELS_ENABLED, autoReplay),
            2, "Auto Replay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For each channel, up to three waveforms can be shown and analyzed. The
/// HIFN  decimation mode and the waveform arithmetic are specific for each
/// HIFN  waveform. So you can analyze several aspects of the signal: For
/// HIFN  example, waveform1 shows the peaks, and waveform2 shows the average of
/// HIFN  the signal.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MULTIWAVEFORM_ACQUISITION_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ACQuire:MUWaveform
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiwaveform/For each channel, up to three waveforms can be shown and analyzed. The
/// HIPAR multiwaveform/decimation mode and the waveform arithmetic are specific for each
/// HIPAR multiwaveform/waveform. So you can analyze several aspects of the signal: For
/// HIPAR multiwaveform/example, waveform1 shows the peaks, and waveform2 shows the average of
/// HIPAR multiwaveform/the signal.
ViStatus _VI_FUNC rsscope_ConfigureMultiwaveformAcquisitionEnabled(
                            ViSession instrSession,
                            ViBoolean multiwaveform)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_MULTIWAVEFORM_ACQUISITION_ENABLED, multiwaveform),
            2, "Multiwaveform");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common attributes of the oscilloscope's channel subsystem for a particular channel.  These attributes are the vertical range, vertical offset, coupling, probe attenuation,
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalRange/Sets the voltage range across the 10 vertical divisions of the diagram.
/// HIPAR verticalOffset/Pass the location of the center of the range that you specify with for the Vertical Range parameter. Express the value with respect to ground.
/// HIPAR verticalCoupling/Specify how you want the oscilloscope to couple the input signal for the channel.
/// HIPAR channelEnabled/Switches the channel signal on or off.
ViStatus _VI_FUNC rsscope_ConfigureChannel(ViSession instrSession,
                                           ViInt32   channel,
                                           ViBoolean channelEnabled,
                                           ViReal64  verticalRange,
                                           ViReal64  verticalOffset,
                                           ViInt32   verticalCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_ENABLED, channelEnabled),
            3, "Channel Enabled");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_VERTICAL_RANGE, verticalRange),
            4, "Vertical Range");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_VERTICAL_OFFSET, verticalOffset),
            5, "Vertical Offset");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_VERTICAL_COUPLING, verticalCoupling),
            6, "Vertical Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the inversion of the signal amplitude on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR invert/Turns the inversion of the signal amplitude on or off.
ViStatus _VI_FUNC rsscope_ConfigureChannelInvert(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean invert)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_INVERT, invert),
            3, "Invert");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the connection of the signal to the ground.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR groundConnection/Connects the signal to the ground.
ViStatus _VI_FUNC rsscope_ConfigureChannelGroundConnection(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean groundConnection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GROUND_CONNECTION_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_GROUND_CONNECTION_ENABLED, groundConnection),
            3, "Ground Connection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the offset voltage. The offset is subtracted to correct an offset-affected signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR offset/Sets the offset voltage.
ViStatus _VI_FUNC rsscope_ConfigureChannelOffset(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_OFFSET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical scale for the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalScale/Sets the vertical scale for the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigureChannelVerticalScale(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViReal64  verticalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_VERTICAL_SCALE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_VERTICAL_SCALE, verticalScale),
            3, "Vertical Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the bandwidth limit for the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bandwidth/Selects the bandwidth limit for the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigureChannelBandwidth(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_BANDWIDTH_LIMIT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_BANDWIDTH_LIMIT, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the impedance of the channel for power calculations and measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR inputImpedance/Sets the impedance of the channel for power calculations and measurements.
ViStatus _VI_FUNC rsscope_ConfigureChannelMeasurementImpedance(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  inputImpedance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_INPUT_IMPEDANCE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_INPUT_IMPEDANCE, inputImpedance),
            3, "Input Impedance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the overload status of the specified channel from the status bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR overload/Retrieves the overload status of the specified channel from the status bit. When the overload problem is solved, the command resets the status bit. Use VI_FALSE to clear the overload condition.
ViStatus _VI_FUNC rsscope_ConfigureChannelOverload(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean overload)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_OVERLOAD,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_OVERLOAD, overload),
            3, "Overload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the anti-aliasing filter. The filter removes spurious and
/// HIFN  undersampled components of the signal and thus provides additional
/// HIFN  bandwidth limitation. The command is available for R&S RTO2064 and
/// HIFN  models with bandwidth extension option B2xx. Filter setting is only
/// HIFN  possible in remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR antialiasing/Activates the anti-aliasing filter.
ViStatus _VI_FUNC rsscope_ConfigureChannelAntialiasing(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   antialiasing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ANTIALIASING,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_ANTIALIASING, antialiasing),
            3, "Antialising");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures Digital Signal Processing (DSP) filtering of
/// HIFN  the acquisition channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR enable/Enables the DSP filter.
/// HIPAR cutOff/Sets the limit frequency of the Lowpass filter for input channels.
ViStatus _VI_FUNC rsscope_ConfigureChannelDigitalFilter(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean enable,
                                                        ViReal64  cutOff)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIGITAL_FILTER_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIGITAL_FILTER_STATE, enable),
            3, "Enable");

    if (enable)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIGITAL_FILTER_CUT_OFF, cutOff),
                4, "Cut Off");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the filter for the trigger channel(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger_obsolete/Selects trigger event. This parameter is included for backwards
/// HIPAR trigger_obsolete/compatibility, it is now ignored and can be NULL.
/// HIPAR channel_obsolete/This control selects the channel.This parameter is included for
/// HIPAR channel_obsolete/backwards compatibility, it is now ignored and can be NULL.
/// HIPAR coupling/Selects the filter for the trigger channel(s). Other channels must use
/// HIPAR coupling/the same filter, or proceed unfiltered.
/// HIPAR rfrejectBW/Sets the limit frequency.
ViStatus _VI_FUNC rsscope_ConfigureChannelDigitalFilterTrigger(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   reserved,
                            ViInt32   coupling,
                            ViReal64  rfrejectBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    reserved;

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_DIGITAL_FILTER_TRIGGER_COUPLING, coupling),
            4, "Coupling");

    if (coupling == RSSCOPE_VAL_DSP_TRIGGER_RF_REJECT)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_DIGITAL_FILTER_TRIGGER_RF_REJECT, rfrejectBW),
                5, "RF Reject BW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the voltage divider that is part of the DUT
/// HIFN  before the measuring point. The external attenuation is included in the
/// HIFN  measurement, and the instrument shows the results that would be
/// HIFN  measured before the divider.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR attenuation/Sets the attenuation of an external voltage divider.
/// HIPAR scale/This control selects the attenuation scale for an external divider.
ViStatus _VI_FUNC rsscope_ConfigureChannelExternalAttenuation(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   scale,
                            ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_ATTENUATION_SCALE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EXTERNAL_ATTENUATION_SCALE, scale),
            3, "Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EXTERNAL_ATTENUATION_VALUE, attenuation),
            4, "Attenuation");

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates and configures the differential signal. Two differential
/// HIFN  signals are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR enable/Activates the differential signal.
/// HIPAR positiveInput/Select the analog input channel of the positive signal. For
/// HIPAR positiveInput/differential signal 1, analog Ch1 and Ch2 are used. Differential signal
/// HIPAR positiveInput/2 uses Ch3 and Ch4.
/// HIPAR negativeInput/Select the analog input channel of the negative signal. For
/// HIPAR negativeInput/differential signal 1, analog Ch1 and Ch2 are used. Differential signal
/// HIPAR negativeInput/2 uses Ch3 and Ch4.
/// HIPAR outputA/Select the waveform that is the result of differential processing. 2
/// HIPAR outputA/output waveforms per digital signal can be defined.
/// HIPAR outputB/Select the waveform that is the result of differential processing. 2
/// HIPAR outputB/output waveforms per digital signal can be defined.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialSignal(ViSession instrSession,
                                                      ViInt32   differentialSignal,
                                                      ViBoolean enable,
                                                      ViInt32   positiveInput,
                                                      ViInt32   negativeInput,
                                                      ViInt32   outputA,
                                                      ViInt32   outputB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, differentialSignal, 1, 2),
            2, "Differential Signal");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_STATE,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_POSITIVE_INPUT, positiveInput),
            4, "Positive Input");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_NEGATIVE_INPUT, negativeInput),
            5, "Negative Input");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_OUTPUT_A, outputA),
            6, "Output A");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_OUTPUT_B, outputB),
            7, "Output B");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SIGNAL_STATE, enable),
            3, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the vertical settings of the coupled channel to the values of the
/// HIFN  active channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR coupledChannel/This control selects the channel that will be coupled to the active
/// HIPAR coupledChannel/channel.
/// HIPAR couplingEnabled/Sets the vertical settings of the coupled channel to the values of the
/// HIPAR couplingEnabled/active channel.
ViStatus _VI_FUNC rsscope_ConfigureCoupleVerticalChannelSettingsEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   coupledChannel,
                            ViBoolean couplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_coupled[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (channel == coupledChannel)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, coupledChannel), 3, "Channel and Coupled Channel");
    }

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_COUPLE_VERTICAL_CHANNEL_SETTINGS_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    strcat(repCap, ",");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, coupledChannel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            3, "Coupled Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_COUPLE_VERTICAL_CHANNEL_SETTINGS_ENABLED,
            "CoupledChannel", coupledChannel- 1, RS_REPCAP_BUF_SIZE, repCap_coupled),
            3, "Coupled Channel");
    strcat(repCap, repCap_coupled);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_COUPLE_VERTICAL_CHANNEL_SETTINGS_ENABLED, couplingEnabled),
            4, "Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the differential offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR commonOffset/Set the offset of differential and common mode waveforms.
/// HIPAR differentialOffset/Set the offset of differential and common mode waveforms.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialOffset(ViSession instrSession,
                                                      ViInt32   differentialSignal,
                                                      ViReal64  commonOffset,
                                                      ViReal64  differentialOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_OFFSET,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_OFFSET, commonOffset),
            3, "Common Offset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_OFFSET, differentialOffset),
            4, "Differential Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the differential position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR commonPosition/Set the vertical position of differential and common mode waveforms.
/// HIPAR differentialPosition/Set the vertical position of differential and common mode waveforms.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialPosition(ViSession instrSession,
                                                        ViInt32   differentialSignal,
                                                        ViReal64  commonPosition,
                                                        ViReal64  differentialPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_POSITION,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_POSITION, commonPosition),
            3, "Common Position");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_POSITION, differentialPosition),
            4, "Differential Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the differential scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR commonScale/Set the vertical scale of differential and common mode waveforms,
/// HIPAR commonScale/respectively.
/// HIPAR differentialScale/Set the vertical scale of differential and common mode waveforms,
/// HIPAR differentialScale/respectively.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialScale(ViSession instrSession,
                                                     ViInt32   differentialSignal,
                                                     ViReal64  commonScale,
                                                     ViReal64  differentialScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_SCALE,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_SCALE, commonScale),
            3, "Common Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_SCALE, differentialScale),
            4, "Differential Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the differential enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR commonEnabled/Enable the output waveforms if deembedding by software is active. In
/// HIPAR commonEnabled/this case, all output waveforms can be displayed at the same time, and
/// HIPAR commonEnabled/each output is enabled separately.
/// HIPAR differentialEnabled/Enable the output waveforms if deembedding by software is active. In
/// HIPAR differentialEnabled/this case, all output waveforms can be displayed at the same time, and
/// HIPAR differentialEnabled/each output is enabled separately.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialEnabled(ViSession instrSession,
                                                       ViInt32   differentialSignal,
                                                       ViBoolean commonEnabled,
                                                       ViBoolean differentialEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_ENABLED,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_COMMON_ENABLED, commonEnabled),
            3, "Common Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_ENABLED, differentialEnabled),
            4, "Differential Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the differential signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR negativeSignal/Enable the output waveforms if deembedding by software is active. In
/// HIPAR negativeSignal/this case, all output waveforms can be displayed at the same time, and
/// HIPAR negativeSignal/each output is enabled separately.
/// HIPAR positiveSignal/Enable the output waveforms if deembedding by software is active. In
/// HIPAR positiveSignal/this case, all output waveforms can be displayed at the same time, and
/// HIPAR positiveSignal/each output is enabled separately.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialSignalPositiveNegative(
                            ViSession instrSession,
                            ViInt32   differentialSignal,
                            ViBoolean negativeSignal,
                            ViBoolean positiveSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_NEGATIVE_SIGNAL_ENABLED,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_NEGATIVE_SIGNAL_ENABLED, negativeSignal),
            3, "Negative Signal");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_POSITIVE_SIGNAL_ENABLED, positiveSignal),
            4, "Positive Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled for a differential signal, the vertical scales of P, N,
/// HIFN  differential and common mode waveforms are coupled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR differentialSignal/This control selects the differential signal.
/// HIPAR coupling/If enabled for a differential signal, the vertical scales of P, N,
/// HIPAR coupling/differential and common mode waveforms are coupled.
ViStatus _VI_FUNC rsscope_ConfigureDifferentialCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   differentialSignal,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DIFFERENTIAL_COUPLING_ENABLED,
            "DifferentialSignal", differentialSignal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Differential Signal");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DIFFERENTIAL_COUPLING_ENABLED, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether the vertical settings of the coupled channel are
/// HIFN  coupled to the values of the active channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR coupledChannel/This control selects the channel that will be coupled to the active
/// HIPAR coupledChannel/channel.
/// HIPAR couplingEnabled/Returns whether the vertical settings of the coupled channel are
/// HIPAR couplingEnabled/coupled to the values of the active channel.
ViStatus _VI_FUNC rsscope_QueryVerticalChannelSettingsCoupled(
                            ViSession  instrSession,
                            ViInt32    channel,
                            ViInt32    coupledChannel,
                            ViBoolean* couplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_coupled[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_QUERY_VERTICAL_CHANNEL_SETTINGS_COUPLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    strcat(repCap, ",");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, coupledChannel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            3, "Coupled Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_QUERY_VERTICAL_CHANNEL_SETTINGS_COUPLED,
            "CoupledChannel", coupledChannel- 1, RS_REPCAP_BUF_SIZE, repCap_coupled),
            3, "Coupled Channel");
    strcat(repCap, repCap_coupled);

    viCheckParm(rsscope_GetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_QUERY_VERTICAL_CHANNEL_SETTINGS_COUPLED, couplingEnabled),
            4, "Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR channelVisible/Switches the channel signal on or off.
/// HIPAR offset/Sets the offset voltage.
ViStatus _VI_FUNC rsscope_ConfigureProbe(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean channelVisible,
                                         ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_ENABLED, channelVisible),
            3, "Channel Visible");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries if the probe at the specified input channel is active
/// HIFN  (detected) or not active (not detected).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR state/Queries if the probe at the specified input channel is active
/// HIPAR state/(detected) or not active (not detected).
ViStatus _VI_FUNC rsscope_QueryProbeState(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the probe characteristics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR type/Queries the type of the probe.
/// HIPAR name/Queries the name of the probe.
/// HIPAR inputImpedance/Queries the termination of the probe.
/// HIPAR inputCapacitance/Queries the input capacitance of the probe.
/// HIPAR attenuation/Queries the attenuation of the probe.
/// HIPAR bandwidth/Queries the bandwidth of the probe.
ViStatus _VI_FUNC rsscope_QueryProbeCharacteristics(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   bufferSize,
                                                    ViChar    type[],
                                                    ViChar    name[],
                                                    ViReal64* inputImpedance,
                                                    ViReal64* inputCapacitance,
                                                    ViReal64* attenuation,
                                                    ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_TYPE,
            bufferSize, type),
            4, "Type");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_NAME,
            bufferSize, name),
            5, "Name");

    if (strstr (type, "active") != NULL)
    {
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_INPUT_IMPEDANCE, inputImpedance),
                6, "Input Impedance");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_INPUT_CAPACITANCE, inputCapacitance),
                7, "Input Capacitance");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_ATTENUATION, attenuation),
                8, "Attenuation");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_BANDWIDTH, bandwidth),
                9, "Bandwidth");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the action that is started with the micro
/// HIFN  button on the probe head.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR microButtonAction/Select the action that is started with the micro button on the probe
/// HIPAR microButtonAction/head.
ViStatus _VI_FUNC rsscope_ConfigureProbeMicroButtonAction(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   microButtonAction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_MICRO_BUTTON_ACTION,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_MICRO_BUTTON_ACTION, microButtonAction),
            3, "Micro Button Action");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs an automatic correction of the zero error. If
/// HIFN  the DUT is ground-referenced, the Auto Zero function can improve the
/// HIFN  measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeAutoZero(ViSession instrSession,
                                        ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_AUTO_ZERO,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_AUTO_ZERO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function includes the AutoZero offset in measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR useAutoZero/Includes the AutoZero offset in measurement results.
ViStatus _VI_FUNC rsscope_ProbeUseAutoZero(ViSession instrSession,
                                           ViInt32   channel,
                                           ViBoolean useAutoZero)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_USE_AUTO_ZERO,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_USE_AUTO_ZERO, useAutoZero),
            3, "Use Auto Zero");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the waveform to zero position. It corrects the
/// HIFN  effect of a voltage offset or temperature drift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR zeroAdjustValue/Set the waveform to zero position. It corrects the effect of a voltage
/// HIPAR zeroAdjustValue/offset or temperature drift.
ViStatus _VI_FUNC rsscope_ConfigureProbeZeroAdjustValue(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViReal64  zeroAdjustValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_ZERO_ADJUST_VALUE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_ZERO_ADJUST_VALUE, zeroAdjustValue),
            3, "Zero Adjust Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function saves the zero adjust vue in the probe box. If you
/// HIFN  connect the probe to other channel or to other R&S RTx
/// HIFN  oscilloscope, the vue is read out again.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeSaveToProbe(ViSession instrSession,
                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_SAVE_TO_PROBE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_SAVE_TO_PROBE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function performs an automatic compensation for a DC component of
/// HIFN  the specified input signal using the result of a background mean
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeSetOffsetToMean(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_SET_OFFSET_TO_MEAN,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_SET_OFFSET_TO_MEAN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function demagnetizes the core if it has been magnetized by
/// HIFN  switching the power on and off, or by an excessive input. Always carry
/// HIFN  out demagnetizing before measurement. The demagnetizing process takes
/// HIFN  about one second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeDegauss(ViSession instrSession,
                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEGAUSS,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEGAUSS, NULL),
            2, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the probe attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Software Version, Production Date, Part Number and Serial Number
/// HIPAR bufferSize/values.
/// HIPAR softwareVersion/Queries the version of the probe firmware.
/// HIPAR productionDate/Queries the production date of the probe.
/// HIPAR partNumber/Queries the R&S part number of the probe.
/// HIPAR serialNumber/Queries the serial number of the probe.
ViStatus _VI_FUNC rsscope_QueryProbeAttributes(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   bufferSize,
                                               ViChar    softwareVersion[],
                                               ViChar    productionDate[],
                                               ViChar    partNumber[],
                                               ViChar    serialNumber[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_SOFTWARE_VERSION,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_SOFTWARE_VERSION,
            bufferSize, softwareVersion),
            4, "Software Version");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_PRODUCTION_DATE,
            bufferSize, productionDate),
            5, "Production Date");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_PART_NUMBER,
            bufferSize, partNumber),
            6, "Part Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_SERIAL_NUMBER,
            bufferSize, serialNumber),
            7, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the path and file name of the update package.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR firmwarePath/Configures the path and file name of the update package.
ViStatus _VI_FUNC rsscope_ConfigureProbeFirmwarePath(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViString  firmwarePath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_FIRMWARE_PATH,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_FIRMWARE_PATH, firmwarePath),
            3, "Firmware Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the update of the probe firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_UpdateProbeFirmware(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_FIRMWARE_UPDATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_FIRMWARE_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the update status of the probe firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR firmwareUpdateStatus/Queries the update status of the probe firmware.
ViStatus _VI_FUNC rsscope_QueryProbeFirmwareUpdateStatus(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32*  firmwareUpdateStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_FIRMWARE_UPDATE_STATUS,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_FIRMWARE_UPDATE_STATUS, firmwareUpdateStatus),
            3, "Firmware Update Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the selftest of the probe connected to the
/// HIFN  specified channel input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeSelftest(ViSession instrSession,
                                        ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_SELFTEST,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_SELFTEST, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the result of a probe selftest for the specified
/// HIFN  channel input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Selftest Results value.
/// HIPAR selftestResult/Queries the pass/fail results of all steps of the probe
ViStatus _VI_FUNC rsscope_QueryProbeSelftestResult(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   bufferSize,
                                                   ViChar    selftestResult[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2data = NULL;

    //ViUInt32     count = 0;


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "PROB%ld:SERV:STES:RES?", channel);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &p2data));

    memcpy(selftestResult, p2data, bufferSize);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (p2data) free(p2data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status of a probe selftest for the specified
/// HIFN  channel input. When the selftest has finished, the status is passed or
/// HIFN  failed, otherwise it is undefined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR selftestStatus/Queries the status of a probe selftest for the specified channel
/// HIPAR selftestStatus/input. When the selftest has finished, the status is passed or failed,
/// HIPAR selftestStatus/otherwise it is undefined.
ViStatus _VI_FUNC rsscope_QueryProbeSelftestStatus(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32*  selftestStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_SELFTEST_STATUS,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_SELFTEST_STATUS, selftestStatus),
            3, "Selftest Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the probe detection mode. Active and
/// HIFN passive voltage probes from Rohde & Schwarz, and also many
/// HIFN common passive voltage probes are detected by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR mode/This control sets mode of probe detection.
ViStatus _VI_FUNC rsscope_ConfigureProbeDetection(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DETECTION_MODE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DETECTION_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe attenuation. Active and passive
/// HIFN  voltage probes from Rohde & Schwarz, and also many common passive
/// HIFN  voltage probes are detected by the instrument. For other probes, you
/// HIFN  can set the unit and attenuation manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalUnit/This control sets the unit for the connected probe type.
/// HIPAR attenuation/Sets the attenuation for the connected probe.
ViStatus _VI_FUNC rsscope_ConfigureProbeAttenuation(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   verticalUnit,
                                                    ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_VERTICAL_UNIT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_VERTICAL_UNIT, verticalUnit),
            3, "Vertical Unit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_ATTENUATION_MANUAL, attenuation),
            4, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the probe gain manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR gain/Sets the gain of a current probe.
ViStatus _VI_FUNC rsscope_ConfigureProbeGain(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_GAIN_MANUAL,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_GAIN_MANUAL, gain),
            4, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the gain of an automatically detected current
/// HIFN  probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR gain/Returns the gain of an automatically detected current probe.
ViStatus _VI_FUNC rsscope_QueryDetectedProbeGain(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64* gain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DETECTED_PROBE_GAIN,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DETECTED_PROBE_GAIN, gain),
            4, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the current probes. Current probes are
/// HIFN not recognized automatically but the parameters of R&S current
/// HIFN probes (R&S RT-ZCxx) are known to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR predefinedProbe/Document parameter here.
ViStatus _VI_FUNC rsscope_ConfigureCurrentProbePredefined(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   predefinedProbe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_PREDEFINED_CURRENT_PROBE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_PREDEFINED_CURRENT_PROBE, predefinedProbe),
            3, "Predefined Probe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the Tektronics probe that is connected to the R&S RT-Z2T
/// HIFN  adapter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR predefinedTektronicsProbe/Selects the Tektronics probe that is connected to the R&S RT-Z2T
/// HIPAR predefinedTektronicsProbe/adapter.
ViStatus _VI_FUNC rsscope_ConfigureProbePredefinedTektronicsProbe(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   predefinedTektronicsProbe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_PREDEFINED_TEKTRONICS_PROBE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_PREDEFINED_TEKTRONICS_PROBE, predefinedTektronicsProbe),
            3, "Predefined Tektronics Probe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the differential offset to compensate a DC voltage applied to the
/// HIFN  positive and the negative input terminal.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_DM_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:DMOFfset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR dmOffset/Sets the differential offset to compensate a DC voltage applied to the
/// HIPAR dmOffset/positive and the negative input terminal.
ViStatus _VI_FUNC rsscope_ConfigureProbeDMOffset(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  dmOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DM_OFFSET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_DM_OFFSET, dmOffset),
            3, "DM Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common-mode offset. The setting is only
/// HIFN  available for differential probes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR cmOffset/Sets the common-mode offset. The setting is only available for
/// HIPAR cmOffset/differential probes.
ViStatus _VI_FUNC rsscope_ConfigureProbeCMOffset(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  cmOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_CM_OFFSET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_CM_OFFSET, cmOffset),
            3, "CM Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the negative offset to compensate a DC voltage applied to the
/// HIFN  negative input terminal (Vp) referenced to ground.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_NEGATIVE_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:NOFFset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR negativeOffset/Sets the negative offset to compensate a DC voltage applied to the
/// HIPAR negativeOffset/negative input terminal (Vp) referenced to ground.
ViStatus _VI_FUNC rsscope_ConfigureProbeNegativeOffset(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64  negativeOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_NEGATIVE_OFFSET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_NEGATIVE_OFFSET, negativeOffset),
            3, "Negative Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the positive offset to compensate a DC voltage applied to the
/// HIFN  positive input terminal (Vp) referenced to ground.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_POSITIVE_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:POFFset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR positiveOffset/Sets the positive offset to compensate a DC voltage applied to the
/// HIPAR positiveOffset/positive input terminal (Vp) referenced to ground.
ViStatus _VI_FUNC rsscope_ConfigureProbePositiveOffset(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64  positiveOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_POSITIVE_OFFSET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_POSITIVE_OFFSET, positiveOffset),
            3, "Positive Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the measurement mode of the modular probe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:PRMode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR probeMode/Sets the measurement mode of the modular probe.
ViStatus _VI_FUNC rsscope_ConfigureProbeMode(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   probeMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_MODE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_MODE, probeMode),
            3, "Probe Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If you use the external attenuator R&S RT-ZA15 together with one of
/// HIFN  the differential active probes R&S RT-ZD10/20/30, enable RT-ZA15 to
/// HIFN  include the external attenuation in the measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR attenuatorRTZA15/If you use the external attenuator R&S RT-ZA15 together with one of
/// HIPAR attenuatorRTZA15/the differential active probes R&S RT-ZD10/20/30, enable RT-ZA15 to
/// HIPAR attenuatorRTZA15/include the external attenuation in the measurements.
ViStatus _VI_FUNC rsscope_ConfigureProbeAttenuatorRTZA15(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean attenuatorRTZA15)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_ATTENUATOR_ZA15,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_ATTENUATOR_ZA15, attenuatorRTZA15),
            3, "Attenuator RT-ZA15");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures R&S RT-ZHD High-Voltage Differential Probes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR voltageRange/Sets the voltage range of a R&S RT-ZHD probe.
/// HIPAR bandwidthLimit/Enables the lowpass filter in the probe control box.
/// HIPAR overrangeBuzzer/Activates the acustic overrange warning in the probe control box.
ViStatus _VI_FUNC rsscope_ConfigureProbeRTZHD(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   voltageRange,
                                              ViBoolean bandwidthLimit,
                                              ViBoolean overrangeBuzzer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_RT_ZHD_VOLTAGE_RANGE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_RT_ZHD_VOLTAGE_RANGE, voltageRange),
            3, "Voltage Range");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_RT_ZHD_BANDWIDTH_LIMIT, bandwidthLimit),
            4, "Bandwidth Limit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_RT_ZHD_OVERRANGE_BUZZER, overrangeBuzzer),
            5, "Overrange Buzzer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the adapter status, whether the instrument identified the
/// HIFN  adapter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR adapterType/Queries the adapter status, whether the instrument identified the
/// HIPAR adapterType/adapter.
ViStatus _VI_FUNC rsscope_QueryProbeAdapterType(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32*  adapterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_ADAPTER_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_ADAPTER_TYPE, adapterType),
            3, "Adapter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the adapter status, whether the instrument identified the
/// HIFN  adapter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR converterType/Queries the adapter status, whether the instrument identified the
/// HIPAR converterType/adapter.
ViStatus _VI_FUNC rsscope_QueryProbeConverterType(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32*  converterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_CONVERTER_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_CONVERTER_TYPE, converterType),
            3, "Converter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the bandwidth limit of the R&S RT-Z1M probe adapter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bandwidthLimit/Sets the bandwidth limit of the R&S RT-Z1M probe adapter.
ViStatus _VI_FUNC rsscope_ConfigureProbeRTZ1MBandwidthLimit(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   bandwidthLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_RTZ1M_BANDWIDTH_LIMIT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_RTZ1M_BANDWIDTH_LIMIT, bandwidthLimit),
            3, "Bandwidth Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the self-alignment of the R&S RT-Z1M probe adapter. When the
/// HIFN  adapter is detached from the oscilloscope, the values of the
/// HIFN  self-alignment are discarded, and the original factory-alignment is
/// HIFN  reloaded. Thus, repeat the self-alignment when you reattach the probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ProbeRTZ1MSelfAlign(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_RTZ1M_SELF_ALIGN,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_RTZ1M_SELF_ALIGN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the tip module that is used for measurement. If R&S RT-ZMA40
/// HIFN  is selected, choose also the used submodule: semi-rigid cables or none.
/// HIFN  For the browser module R&S RT-ZMA30, measure the space between the
/// HIFN  pins, and select the appropriate value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR tipModule/Selects the tip module that is used for measurement.
/// HIPAR zma30Submodule/Selects the space between the pins of the RT-ZMA30 module tip.
/// HIPAR zma40Submodule/Choose the used submodule: semi-rigid cables or none.
ViStatus _VI_FUNC rsscope_ConfigureProbeDeembeddingSelectTipModule(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   tipModule,
                            ViInt32   zma30Submodule,
                            ViInt32   zma40Submodule)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEEMBEDDING_SELECT_TIP_MODULE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_SELECT_TIP_MODULE, tipModule),
            3, "Tip Module");

    if (tipModule == RSSCOPE_VAL_PROBE_TIP_ZMA30)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_ZMA30_SELECT_TIP_SUBMODULE, zma30Submodule),
                4, "ZMA 30 Submodule");
    }

    if (tipModule == RSSCOPE_VAL_PROBE_TIP_ZMA40)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_ZMA40_SELECT_TIP_SUBMODULE, zma40Submodule),
                5, "ZMA 40 Submodule");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the file name, file format and path of the S-parameter file, and
/// HIFN  loads the S-parameters of user-defined tip modules. Parameter is string
/// HIFN  with path, file name, and extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR deembeddingCustomTipFile/Sets the file name, file format and path of the S-parameter file, and
/// HIPAR deembeddingCustomTipFile/loads the S-parameters of user-defined tip modules. Parameter is string
/// HIPAR deembeddingCustomTipFile/with path, file name, and extension.
ViStatus _VI_FUNC rsscope_ConfigureProbeDeembeddingCustomTipFile(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViString  deembeddingCustomTipFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEEMBEDDING_CUSTOM_TIP_FILE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_CUSTOM_TIP_FILE, deembeddingCustomTipFile),
            3, "Deembedding Custom Tip File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the probing mode, bandwidth and measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR mode/Sets the probing mode.
/// HIPAR bandwidth/Sets the maximum bandwidth until which the signal is corrected. This
/// HIPAR bandwidth/maximum value is the minimum bandwidth value of probe, probe tip and
/// HIPAR bandwidth/oscilloscope bandwidth, and can not be higher than the highest
/// HIPAR bandwidth/frequency in a used S-parameter file. Consider that most tip modules
/// HIPAR bandwidth/support the full bandwidth of the probe amplifier, but some tip modules
/// HIPAR bandwidth/have limited bandwidth.
/// HIPAR measurementMode/Sets the measurement mode of the modular probe.
ViStatus _VI_FUNC rsscope_ConfigureProbeDeembeddingMode(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   mode,
                                                        ViReal64  bandwidth,
                                                        ViInt32   measurementMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEEMBEDDING_PROBING_MODE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_PROBING_MODE, mode),
            3, "Mode");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_BANDWIDTH, bandwidth),
            4, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_MODE, measurementMode),
            5, "Measurement Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a probe mode for deembedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR deembeddingInputMode/Selects a probe mode for deembedding.
ViStatus _VI_FUNC rsscope_ConfigureProbeDeembeddingInputMode(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   deembeddingInputMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEEMBEDDING_MODE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_MODE, deembeddingInputMode),
            3, "Deembedding Input Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the source impedance parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR removeProbeLoading/Removes the loading of the probe if
/// HIPAR removeProbeLoading/RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_MODE is set to PROBing. If the probe
/// HIPAR removeProbeLoading/loading is removed, you see the signal that would be at the measurement
/// HIPAR removeProbeLoading/point if the probe's input impedance is ideal (infinite impedance).
/// HIPAR removeProbeLoading/Without removing the loading, you see the real signal at the
/// HIPAR removeProbeLoading/measurement point, including the probe loading.
/// HIPAR impedanceType/Sets the impedance type of the DUT.
/// HIPAR impedance/Impedance of the DUT. Shows the default source impedance or sets the
/// HIPAR impedance/user-defined constant impedance value. If an R&S RT-ZM probe is
/// HIPAR impedance/connected, the value depends on the selected probe mode: common mode,
/// HIPAR impedance/differential, or single-ended measurement.
/// HIPAR sParameterFile/Sets the file name, file format and path of the S-parameter file, and
/// HIPAR sParameterFile/loads the S-parameters.
ViStatus _VI_FUNC rsscope_ConfigureProbeDeembeddingSourceImpedance(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean removeProbeLoading,
                            ViInt32   impedanceType,
                            ViReal64  impedance,
                            ViString  sParameterFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_DEEMBEDDING_REMOVE_PROBE_LOADING,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_REMOVE_PROBE_LOADING, removeProbeLoading),
            3, "Remove Probe Loading");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_IMPEDANCE_TYPE, impedanceType),
            4, "Impedance Type");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_IMPEDANCE, impedance),
            5, "Impedance");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROBE_DEEMBEDDING_LOAD_S_PARAMETER_FILE, sParameterFile),
            6, "S-Parameter File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe meter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR probeMeter/Switches the channel signal on or off.
/// HIPAR visibility/Activates the integrated R&S ProbeMeter of active R&S probes.
/// HIPAR measureVoltage/Selects the input voltages to be measured by the ProbeMeter of an R&S
/// HIPAR measureVoltage/differential active probe.
ViStatus _VI_FUNC rsscope_ConfigureProbeMeter(ViSession instrSession,
                                              ViInt32   channel,
                                              ViBoolean probeMeter,
                                              ViBoolean visibility,
                                              ViInt32   measureVoltage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PMETER_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PMETER_ENABLED, probeMeter),
            3, "Probe Meter");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PMETER_VISIBILITY, visibility),
            4, "Visibility");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PMETER_DISPLAY_DIFF, measureVoltage),
            5, "Measure Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the probe meter results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR resultType/Selects the input voltages to be measured by the ProbeMeter of an R&S
/// HIPAR resultType/differential active probe.
/// HIPAR result/Returns the value for the selected type.
ViStatus _VI_FUNC rsscope_QueryProbeMeterResults(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   resultType,
                                                 ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_PMETER_MEASUREMENT_COMMON_RESULT,
             RSSCOPE_ATTR_PMATER_MEASUREMENT_DIFFERENTIAL_RESULT,
             RSSCOPE_ATTR_PMETER_MEASUREMENT_POSITIVE_RESULT,
             RSSCOPE_ATTR_PMETER_MEASUREMENT_NEGATIVE_RESULT,
             RSSCOPE_ATTR_PMETER_MEASUREMENT_SINGLE_RESULT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSSCOPE_VAL_PROBE_RES_COMMON, RSSCOPE_VAL_PROBE_RES_SINGLE_ENDED),
            3, "Result Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PMETER_MEASUREMENT_COMMON_RESULT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, attr[resultType], result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR channelVisible/Switches the channel signal on or off.
/// HIPAR offset/Sets the offset voltage.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbe(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean channelVisible,
                                                     ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_ENABLED, channelVisible),
            3, "Channel Visible");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CHANNEL_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries if the probe at the specified input channel is active
/// HIFN  (detected) or not active (not detected).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Queries if the probe at the specified input channel is active
/// HIPAR state/(detected) or not active (not detected).
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeState(ViSession instrSession,
                                                      ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the probe characteristics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Type
/// HIPAR bufferSize/Parameter.
/// HIPAR type/Queries the type of the probe.
/// HIPAR name/Queries the name of the probe.
/// HIPAR inputImpedance/Queries the termination of the probe.
/// HIPAR inputCapacitance/Queries the input capacitance of the probe.
/// HIPAR attenuation/Queries the attenuation of the probe.
/// HIPAR bandwidth/Queries the bandwidth of the probe.
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeCharacteristics(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    type[],
                            ViChar    name[],
                            ViReal64* inputImpedance,
                            ViReal64* inputCapacitance,
                            ViReal64* attenuation,
                            ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_TYPE,
            bufferSize, type),
            3, "Type");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_NAME,
            bufferSize, name),
            4, "Name");

    if (strstr (type, "active") != NULL)
    {
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_INPUT_IMPEDANCE, inputImpedance),
                5, "Input Impedance");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_INPUT_CAPACITANCE, inputCapacitance),
                6, "Input Capacitance");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_ATTENUATION, attenuation),
                7, "Attenuation");

        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_BANDWIDTH, bandwidth),
                8, "Bandwidth");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the action that is started with the micro
/// HIFN  button on the probe head.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR microButtonAction/Select the action that is started with the micro button on the probe
/// HIPAR microButtonAction/head.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeMicroButtonAction(
                            ViSession instrSession,
                            ViInt32   microButtonAction)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_MICRO_BUTTON_ACTION, microButtonAction),
            2, "Micro Button Action");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs an automatic correction of the zero error. If
/// HIFN  the DUT is ground-referenced, the Auto Zero function can improve the
/// HIFN  measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_TriggerInputProbeAutoZero(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_AUTO_ZERO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common-mode offset. The setting is only
/// HIFN  available for differential probes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cmOffset/Sets the common-mode offset. The setting is only available for
/// HIPAR cmOffset/differential probes.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeCMOffset(ViSession instrSession,
                                                             ViReal64  cmOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_CM_OFFSET, cmOffset),
            2, "CM Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If you use the external attenuator R&S RT-ZA15 together with one of
/// HIFN  the differential active probes R&S RT-ZD10/20/30, enable RT-ZA15 to
/// HIFN  include the external attenuation in the measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuatorRTZA15/If you use the external attenuator R&S RT-ZA15 together with one of
/// HIPAR attenuatorRTZA15/the differential active probes R&S RT-ZD10/20/30, enable RT-ZA15 to
/// HIPAR attenuatorRTZA15/include the external attenuation in the measurements.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeAttenuatorRTZA15(
                            ViSession instrSession,
                            ViBoolean attenuatorRTZA15)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_ATTENUATOR_ZA15, attenuatorRTZA15),
            2, "Attenuator RT-ZA15");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs an automatic compensation for a DC component of
/// HIFN  the specified input signal using the result of a background mean
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_TriggerInputProbeSetOffsetToMean(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_SET_OFFSET_TO_MEAN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trigger input probe attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Software Version, Production Date, Part Number and Serial Number
/// HIPAR bufferSize/values.
/// HIPAR softwareVersion/Queries the version of the probe firmware.
/// HIPAR productionDate/Queries the production date of the probe.
/// HIPAR partNumber/Queries the R&S part number of the probe.
/// HIPAR serialNumber/Queries the serial number of the probe.
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeAttributes(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    softwareVersion[],
                                                           ViChar    productionDate[],
                                                           ViChar    partNumber[],
                                                           ViChar    serialNumber[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_SOFTWARE_VERSION,
            bufferSize, softwareVersion),
            3, "Software Version");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_PRODUCTION_DATE,
            bufferSize, productionDate),
            4, "Production Date");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_PART_NUMBER,
            bufferSize, partNumber),
            5, "Part Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_SERIAL_NUMBER,
            bufferSize, serialNumber),
            6, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the path and file name of the update package.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firmwarePath/Configures the path and file name of the update package.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeFirmwarePath(
                            ViSession instrSession,
                            ViString  firmwarePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_FIRMWARE_PATH, firmwarePath),
            2, "Firmware Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the update of the probe firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_UpdateTriggerInputProbeFirmware(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_FIRMWARE_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the update status of the probe firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firmwareUpdateStatus/Queries the update status of the probe firmware.
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeFirmwareUpdateStatus(
                            ViSession instrSession,
                            ViInt32*  firmwareUpdateStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_FIRMWARE_UPDATE_STATUS, firmwareUpdateStatus),
            2, "Firmware Update Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the selftest of the probe connected to the
/// HIFN  specified channel input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_TriggerInputProbeSelftest(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_SELFTEST, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the result of a probe selftest for the specified
/// HIFN  channel input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Selftest Results value.
/// HIPAR selftestResult/Queries the pass/fail results of all steps of the probe
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeSelftestResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    selftestResult[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2data = NULL;

    //ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    //ViUInt32     count = 0;


    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "TRPR:SERV:STES:RES?", &p2data));

    memcpy(selftestResult, p2data, bufferSize);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (p2data) free(p2data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status of a probe selftest for the specified
/// HIFN  channel input. When the selftest has finished, the status is passed or
/// HIFN  failed, otherwise it is undefined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selftestStatus/Queries the status of a probe selftest for the specified channel
/// HIPAR selftestStatus/input. When the selftest has finished, the status is passed or failed,
/// HIPAR selftestStatus/otherwise it is undefined.
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeSelftestStatus(
                            ViSession instrSession,
                            ViInt32*  selftestStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_SELFTEST_STATUS, selftestStatus),
            2, "Selftest Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe detection mode. Active and passive
/// HIFN  voltage probes from Rohde & Schwarz, and also many common passive
/// HIFN  voltage probes are detected by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control sets mode of probe detection.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeDetection(
                            ViSession instrSession,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_DETECTION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe attenuation manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verticalUnit/This control sets the unit for the connected probe type.
/// HIPAR attenuation/Sets the attenuation for the connected probe.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeAttenuation(
                            ViSession instrSession,
                            ViInt32   verticalUnit,
                            ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_VERTICAL_UNIT, verticalUnit),
            2, "Vertical Unit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_ATTENUATION_MANUAL, attenuation),
            3, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the probe gain manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gain/Sets the gain of a current probe.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeGain(ViSession instrSession,
                                                         ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_GAIN_MANUAL, gain),
            2, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the gain of an automatically detected current
/// HIFN  probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gain/Returns the gain of an automatically detected current probe.
ViStatus _VI_FUNC rsscope_QueryDetectedTriggerInputProbeGain(ViSession instrSession,
                                                             ViReal64* gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_DETECTED_PROBE_GAIN, gain),
            2, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the current probes. Current probes are not
/// HIFN  recognized automatically but the parameters of R&S current probes (R&S
/// HIFN  RT-ZCxx) are known to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR predefinedProbe/Selects a predefined current probe.
ViStatus _VI_FUNC rsscope_ConfigureCurrentTriggerInputProbePredefined(
                            ViSession instrSession,
                            ViInt32   predefinedProbe)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PROBE_PREDEFINED_CURRENT_PROBE, predefinedProbe),
            2, "Predefined Probe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables AC coupling in the R&S RT-ZPR20 probe, which removes DC and
/// HIFN  very low-frequency components. The R&S RT-ZPR20 probe requires 50 Ohm
/// HIFN  input termination, for which the channel AC coupling is not available.
/// HIFN  The probe setting allows AC coupling also at 50 Ohm inputs.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_COUPLING_AC
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRPRobe:SETup:ACCoupling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acCoupling/Enables AC coupling in the R&S RT-ZPR20 probe, which removes DC and
/// HIPAR acCoupling/very low-frequency components. The R&S RT-ZPR20 probe requires 50 Ohm
/// HIPAR acCoupling/input termination, for which the channel AC coupling is not available.
/// HIPAR acCoupling/The probe setting allows AC coupling also at 50 Ohm inputs.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeACCoupling(
                            ViSession instrSession,
                            ViBoolean acCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_COUPLING_AC, acCoupling),
            2, "AC Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Demagnetizes the core if it has been magnetized by switching the power
/// HIFN  on and off, or by an excessive input. Always carry out demagnetizing
/// HIFN  before measurement. The demagnetizing process takes about one second.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_DEGAUSS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRPRobe:SETup:DEGauss
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_TriggerInputProbeDegauss(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_DEGAUSS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the measurement mode of the modular probe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRPRobe:SETup:PRMode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeMode/Sets the measurement mode of the modular probe.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeMode(ViSession instrSession,
                                                         ViInt32   probeMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_MODE, probeMode),
            2, "Probe Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures R&S RT-ZHD High-Voltage Differential Probes.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_RT_ZHD_BANDWIDTH_LIMIT
/// HIFN     RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_RT_ZHD_OVERRANGE_BUZZER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRPRobe:SETup:ADVanced:FILTer
/// HIFN     TRPRobe:SETup:ADVanced:AUDioverload
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidthLimit/Enables the lowpass filter in the probe control box.
/// HIPAR overrangeBuzzer/Activates the acustic overrange warning in the probe control box.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeRTZHD(ViSession instrSession,
                                                          ViBoolean bandwidthLimit,
                                                          ViBoolean overrangeBuzzer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_RT_ZHD_BANDWIDTH_LIMIT, bandwidthLimit),
            2, "Bandwidth Limit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_INPUT_PROBE_RT_ZHD_OVERRANGE_BUZZER, overrangeBuzzer),
            3, "Overrange Buzzer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger input probe meter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeMeter/Switches the channel signal on or off.
/// HIPAR visibility/Activates the integrated R&S ProbeMeter of active R&S probes.
/// HIPAR measureVoltage/Selects the input voltages to be measured by the ProbeMeter of an R&S
/// HIPAR measureVoltage/differential active probe.
ViStatus _VI_FUNC rsscope_ConfigureTriggerInputProbeMeter(ViSession instrSession,
                                                          ViBoolean probeMeter,
                                                          ViBoolean visibility,
                                                          ViInt32   measureVoltage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_PMETER_ENABLED, probeMeter),
            2, "Probe Meter");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_PMETER_VISIBILITY, visibility),
            3, "Visibility");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_PMETER_DISPLAY_DIFF, measureVoltage),
            4, "Measure Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the probe meter results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Selects the input voltages to be measured by the ProbeMeter of an R&S
/// HIPAR resultType/differential active probe.
/// HIPAR result/Returns the value for the selected type.
ViStatus _VI_FUNC rsscope_QueryTriggerInputProbeMeterResults(ViSession instrSession,
                                                             ViInt32   resultType,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSSCOPE_ATTR_TRIGGER_PMETER_MEASUREMENT_COMMON_RESULT,
             RSSCOPE_ATTR_TRIGGER_PMATER_MEASUREMENT_DIFFERENTIAL_RESULT,
             RSSCOPE_ATTR_TRIGGER_PMETER_MEASUREMENT_POSITIVE_RESULT,
             RSSCOPE_ATTR_TRIGGER_PMETER_MEASUREMENT_NEGATIVE_RESULT,
             RSSCOPE_ATTR_TRIGGER_PMETER_MEASUREMENT_SINGLE_RESULT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSSCOPE_VAL_PROBE_RES_COMMON, RSSCOPE_VAL_PROBE_RES_SINGLE_ENDED),
            3, "Result Type");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", attr[resultType], result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Setting the termination voltage is relevant if you use the R&S
/// HIFN  RT-ZMA40 SMA module.
/// HIFN     You can set a termination voltage to correct the internal 50
/// HIFN  termination of the SMA module by the common mode voltage. To control
/// HIFN  the termination voltage by the instrument, connect the VT terminal of
/// HIFN  the R&S RT-ZM probe amplifier to the VT terminal of the SMA module
/// HIFN  using the red DC lead (see R&S RT-ZM User Manual).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:TERM:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR mode/Setting the termination voltage is relevant if you use the R&S
/// HIPAR mode/RT-ZMA40 SMA module.
ViStatus _VI_FUNC rsscope_ConfigureProbeTerminationVoltageMode(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MODE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures Probe Termination voltage parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_STATE
/// HIFN     RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_ADJUSTMENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:TERM:STATe
/// HIFN     PROBe<m>:SETup:TERM:ADJust
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR state/Activates the instrument control of the termination voltage.
/// HIPAR adjustment/Sets the voltage to be used for termination correction.
ViStatus _VI_FUNC rsscope_ConfigureProbeTerminationVoltage(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean state,
                                                           ViReal64  adjustment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_STATE, state),
            3, "State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_ADJUSTMENT, adjustment),
            4, "Adjustment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the measured common mode voltage.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MEASURE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     PROBe<m>:SETup:TERM:MEASure?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR measurement/Returns the measured common mode voltage.
ViStatus _VI_FUNC rsscope_QueryProbeTerminationVoltageMeasure(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64* measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MEASURE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROBE_TERMINATION_VOLTAGE_MEASURE, measurement),
            3, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ZVC Probe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_BANDWIDTH
/// HIFN     RSSCOPE_ATTR_ZVC_DECIMATION_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:BANDwidth
/// HIFN     ZVC:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Sets the bandwidth limit of the probe. The bandwidth specifies the
/// HIPAR bandwidth/maximum frequency at which a purely sinusoidal signal is still
/// HIPAR bandwidth/transferred at 89 % (0.1 dB) of its amplitude.
/// HIPAR decimationMode/Sets the decimation mode for the R&S RT-ZVC probe. Decimation reduces
/// HIPAR decimationMode/the data stream of the ADC to a stream of waveform points with lower
/// HIPAR decimationMode/sample rate and a less precise time resolution.
ViStatus _VI_FUNC rsscope_ConfigureZVCProbe(ViSession instrSession,
                                            ViReal64  bandwidth,
                                            ViInt32   decimationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_ZVC_BANDWIDTH, bandwidth),
            2, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_ZVC_DECIMATION_MODE, decimationMode),
            3, "Decimation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ZVC Probe Informations.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_NAME
/// HIFN     RSSCOPE_ATTR_ZVC_PART_NUMBER
/// HIFN     RSSCOPE_ATTR_ZVC_SERIAL_NUMBER
/// HIFN     RSSCOPE_ATTR_ZVC_FIRMWARE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:ID:NAME?
/// HIFN     ZVC:Z<m>:ID:PARTnumber?
/// HIFN     ZVC:Z<m>:ID:SRNumber?
/// HIFN     ZVC:Z<m>:ID:SWVersion?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probeName/Queries the name of the probe.
/// HIPAR partNumber/Queries the R&S part number of the probe.
/// HIPAR serialNumber/Queries the serial number of the probe.
/// HIPAR firmware/Queries the version of the probe firmware.
ViStatus _VI_FUNC rsscope_QueryZVCProbeInformations(ViSession instrSession,
                                                    ViInt32   multiChannel,
                                                    ViInt32   arraySize,
                                                    ViChar    probeName[],
                                                    ViChar    partNumber[],
                                                    ViChar    serialNumber[],
                                                    ViChar    firmware[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ZVC_NAME,
            "ZVCZ", multiChannel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Multi Channel");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_NAME,
            arraySize, probeName),
            4, "Probe Name");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_PART_NUMBER,
            arraySize, partNumber),
            5, "Part Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_SERIAL_NUMBER,
            arraySize, serialNumber),
            6, "Serial Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_FIRMWARE,
            arraySize, firmware),
            7, "Firmware");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the corresponding parameter of the probe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_ENABLED
/// HIFN     RSSCOPE_ATTR_ZVC_V_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>[:STATe]
/// HIFN     ZVC:Z<m>:V<n>[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR enabled/Enables the corresponding parameter of the probe.
ViStatus _VI_FUNC rsscope_ConfigureZVCEnabled(ViSession instrSession,
                                              ViInt32   multiChannel,
                                              ViInt32   probe,
                                              ViInt32   probeParameter,
                                              ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_ENABLED, enabled),
                5, "Enabled");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_ENABLED, enabled),
                5, "Enabled");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the bandwidth of the selected parameter.
/// HIFN     The bandwidth of some current channels is restricted to 300KHz due to
/// HIFN  their vertical settings.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_BANDWIDTH
/// HIFN     RSSCOPE_ATTR_ZVC_V_BANDWIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:BANDwidth?
/// HIFN     ZVC:Z<m>:V<n>:BANDwidth?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR bandwidth/Queries the bandwidth of the selected parameter.
ViStatus _VI_FUNC rsscope_QueryZVCBandwidth(ViSession instrSession,
                                            ViInt32   multiChannel,
                                            ViInt32   probe,
                                            ViInt32   probeParameter,
                                            ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_BANDWIDTH, bandwidth),
                5, "Bandwidth");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_BANDWIDTH, bandwidth),
                5, "Bandwidth");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the resolution of all R&S RT-ZVC channels. If TRUE, the
/// HIFN  resolution of the analog channels is applied to R&S RT-ZVC channels.
/// HIFN  The signal is automatically interpolated or decimated to get the analog
/// HIFN  resolution. If FALSE, the resolution of R&S RT-ZVC channels is set in a
/// HIFN  way so that the record length of the waveforms is minimum 1000 samples.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupled/Sets the resolution of all R&S RT-ZVC channels. If TRUE, the
/// HIPAR coupled/resolution of the analog channels is applied to R&S RT-ZVC channels.
/// HIPAR coupled/The signal is automatically interpolated or decimated to get the analog
/// HIPAR coupled/resolution. If FALSE, the resolution of R&S RT-ZVC channels is set in a
/// HIPAR coupled/way so that the record length of the waveforms is minimum 1000 samples.
ViStatus _VI_FUNC rsscope_ConfigureZVCResolutionCoupled(ViSession instrSession,
                                                        ViBoolean coupled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_RESOLUTION_COUPLED, coupled),
            2, "Coupled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the current impedance of the probe parameter for power
/// HIFN  calculations and measurements.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_IMPEDANCE
/// HIFN     RSSCOPE_ATTR_ZVC_V_IMPEDANCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:IMPedance
/// HIFN     ZVC:Z<m>:V<n>:IMPedance
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR impedance/Sets the current impedance of the probe parameter for power
/// HIPAR impedance/calculations and measurements.
ViStatus _VI_FUNC rsscope_ConfigureZVCImpedance(ViSession instrSession,
                                                ViInt32   multiChannel,
                                                ViInt32   probe,
                                                ViInt32   probeParameter,
                                                ViInt32   impedance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_IMPEDANCE, impedance),
                5, "Impedance");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_IMPEDANCE, impedance),
                5, "Impedance");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the ZVC current channel offsets.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_OFFSET
/// HIFN     RSSCOPE_ATTR_ZVC_I_DESKEW_OFFSET
/// HIFN     RSSCOPE_ATTR_ZVC_V_OFFSET
/// HIFN     RSSCOPE_ATTR_ZVC_V_DESKEW_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:OFFSet
/// HIFN     ZVC:Z<m>:I<n>:SKEW
/// HIFN     ZVC:Z<m>:V<n>:OFFSet
/// HIFN     ZVC:Z<m>:V<n>:SKEW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR offset/Sets the offset current for the probe parameter.
/// HIPAR deskew/Sets the skew offset value for the selected ZVC parameter. This is a
/// HIPAR deskew/delay value, that is known from the circuit specifics but cannot be
/// HIPAR deskew/compensated by the instrument automatically.
ViStatus _VI_FUNC rsscope_ConfigureZVCOffsets(ViSession instrSession,
                                              ViInt32   multiChannel,
                                              ViInt32   probe,
                                              ViInt32   probeParameter,
                                              ViReal64  offset,
                                              ViReal64  deskew)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_OFFSET, offset),
                5, "Offset");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_DESKEW_OFFSET, deskew),
                6, "Deskew");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_OFFSET, offset),
                5, "Offset");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_DESKEW_OFFSET, deskew),
                6, "Deskew");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the ZVC selected parameter vertical values.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_VERTICAL_POSITION
/// HIFN     RSSCOPE_ATTR_ZVC_I_VERTICAL_SCALE
/// HIFN     RSSCOPE_ATTR_ZVC_V_VERTICAL_POSITION
/// HIFN     RSSCOPE_ATTR_ZVC_V_VERTICAL_SCALE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:POSition
/// HIFN     ZVC:Z<m>:I<n>:SCALe
/// HIFN     ZVC:Z<m>:V<n>:POSition
/// HIFN     ZVC:Z<m>:V<n>:SCALe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR position/Sets the vertical position of the indicated probe parameter as a
/// HIPAR position/graphical value.
/// HIPAR scale/Sets the vertical scale for the selected parameter in Volts per
/// HIPAR scale/division. Increasing the scale compresses the display of the signal.
ViStatus _VI_FUNC rsscope_ConfigureZVCVertical(ViSession instrSession,
                                               ViInt32   multiChannel,
                                               ViInt32   probe,
                                               ViInt32   probeParameter,
                                               ViReal64  position,
                                               ViReal64  scale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_VERTICAL_POSITION, position),
                5, "Position");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_VERTICAL_SCALE, scale),
                6, "Scale");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_VERTICAL_POSITION, position),
                5, "Position");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_VERTICAL_SCALE, scale),
                6, "Scale");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the overload indication at the probe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_RESET_OVERLOAD
/// HIFN     RSSCOPE_ATTR_ZVC_V_RESET_OVERLOAD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:OVERload:RSTO
/// HIFN     ZVC:Z<m>:V<n>:OVERload:RSTO
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR resetOverload/Resets the overload indication at the probe.
ViStatus _VI_FUNC rsscope_ConfigureZVCResetOverload(ViSession instrSession,
                                                    ViInt32   multiChannel,
                                                    ViInt32   probe,
                                                    ViInt32   probeParameter,
                                                    ViInt32   resetOverload)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_RESET_OVERLOAD, resetOverload),
                5, "Reset Overload");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_RESET_OVERLOAD, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries if an overload of the selected parameter was detected.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_OVERLOAD_DETECTED
/// HIFN     RSSCOPE_ATTR_ZVC_V_OVERLOAD_DETECTED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:OVERload:VALue?
/// HIFN     ZVC:Z<m>:V<n>:OVERload:VALue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probe/Sets the probe parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR overloadDetected/Queries if an overload of the current channel was detected.
ViStatus _VI_FUNC rsscope_QueryZVCOverload(ViSession  instrSession,
                                           ViInt32    multiChannel,
                                           ViInt32    probe,
                                           ViInt32    probeParameter,
                                           ViBoolean* overloadDetected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            4, "Probe Parameter");

    switch (probe)
    {
    case RSSCOPE_VAL_ZVC_PROBE_CURRENT:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_GetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_OVERLOAD_DETECTED, overloadDetected),
                5, "Overload Detected");
        break;

    case RSSCOPE_VAL_ZVC_PROBE_VOLTAGE:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);
        viCheckParm(rsscope_GetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_V_OVERLOAD_DETECTED, overloadDetected),
                5, "Overload Detected");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, probe), 3, "Probe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the internal or external shunt mode.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_SHUNT_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:SHUNt:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR currentChannel/Sets the current channel parameter.
/// HIPAR mode/Sets the internal or external shunt mode.
ViStatus _VI_FUNC rsscope_ConfigureZVCShuntMode(ViSession instrSession,
                                                ViInt32   multiChannel,
                                                ViInt32   currentChannel,
                                                ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, currentChannel, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, currentChannel);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_SHUNT_MODE, mode),
            4, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the external shunt.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_SHUNT_EXTERNAL_MAXIMUM_VOLTAGE
/// HIFN     RSSCOPE_ATTR_ZVC_I_SHUNT_EXTERNAL_VALUE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:SHUNt:MAXVoltage
/// HIFN     ZVC:Z<m>:I<n>:SHUNt:EVALue
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR currentChannel/Sets the current channel parameter.
/// HIPAR maximumVoltage/Sets the maximum voltage for the external shunt.
/// HIPAR externalValue/Defines the value of the external shunt resistor to calculate the
/// HIPAR externalValue/correct current values.
ViStatus _VI_FUNC rsscope_ConfigureZVCShuntExternal(ViSession instrSession,
                                                    ViInt32   multiChannel,
                                                    ViInt32   currentChannel,
                                                    ViInt32   maximumVoltage,
                                                    ViReal64  externalValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, currentChannel, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, currentChannel);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_SHUNT_EXTERNAL_MAXIMUM_VOLTAGE, maximumVoltage),
            4, "Maximum Voltage");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_SHUNT_EXTERNAL_VALUE, externalValue),
            5, "External Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the maximum current and the internal shunt value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_SHUNT_INTERNAL_MAXIMUM
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:SHUNt:MAXCurrent
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR currentChannel/Sets the current channel parameter.
/// HIPAR maximumCurrent/Sets the maximum current and the internal shunt value.
ViStatus _VI_FUNC rsscope_ConfigureZVCShuntInternalMaximum(ViSession instrSession,
                                                           ViInt32   multiChannel,
                                                           ViInt32   currentChannel,
                                                           ViInt32   maximumCurrent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, currentChannel, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, currentChannel);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_SHUNT_INTERNAL_MAXIMUM, maximumCurrent),
            4, "Maximum Current");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the maximum current and the internal shunt value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ZVC_I_SHUNT_MAXIMUM_CURRENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ZVC:Z<m>:I<n>:SHUNt:MXCValue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR currentChannel/Sets the current channel parameter.
/// HIPAR maximumCurrent/Queries the maximum current.
ViStatus _VI_FUNC rsscope_ConfigureZVCShuntMaximumCurrent(ViSession instrSession,
                                                          ViInt32   multiChannel,
                                                          ViInt32   currentChannel,
                                                          ViReal64* maximumCurrent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, currentChannel, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCI%ld", multiChannel, currentChannel);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_SHUNT_MAXIMUM_CURRENT, maximumCurrent),
            4, "Maximum Current");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the header of amperemeter and voltmeter waveform data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSource/Selects the source of the data to be retrieved.
/// HIPAR xStart/Acquisition time before trigger, in s
/// HIPAR xStop/Acquisition time after trigger, in s
/// HIPAR recordLength/Record length of the waveform in Samples
/// HIPAR valuesperSample/Number of values per sample interval.
ViStatus _VI_FUNC rsscope_FetchZVCWaveformHeader(ViSession instrSession,
                                                 ViInt32   waveformSource,
                                                 ViReal64* xStart,
                                                 ViReal64* xStop,
                                                 ViInt32*  recordLength,
                                                 ViInt32*  valuesperSample)
{
    ViStatus error = VI_SUCCESS;
    ViString probeString[] = {
    "1:V1",  "1:V2", "1:V3", "1:V4",
    "1:I1",  "1:I2", "1:I3", "1:I4",
    "2:V1",  "2:V2", "2:V3", "2:V4",
    "2:I1",  "2:I2", "2:I3", "2:I4"
             };
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 response[4];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveformSource, RSSCOPE_VAL_WAV_Z1V1, RSSCOPE_VAL_WAV_Z2I4),
            2, "Waveform Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ZVC:Z%s:DATA:HEAD?", probeString[waveformSource - RSSCOPE_VAL_WAV_Z1V1]);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, 0, 4, response, NULL));
    checkErr(rsscope_CheckStatus(instrSession));

    *xStart = response[0];
    *xStop = response[1];
    *recordLength = (ViInt32)response[2];
    *valuesperSample = (ViInt32)response[3];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the waveform data of the amperemeter and voltmeter for
/// HIFN  transmission from the instrument to the controlling computer. The data
/// HIFN  can be used in MATLAB, for example.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSource/Selects the source of the data to be retrieved.
/// HIPAR bufferSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the data of track points for transmission from the instrument
/// HIPAR waveformArray/to the controlling computer.
/// HIPAR actualPoints/Indicates the number of waveform points returned by the instrument. If
/// HIPAR actualPoints/this number is larger than Buffer Size, part of the waveform data was
/// HIPAR actualPoints/lost. Consider resizing the buffer and trying again.
ViStatus _VI_FUNC rsscope_FetchZVCWaveform(ViSession instrSession,
                                           ViInt32   waveformSource,
                                           ViInt32   bufferSize,
                                           ViReal64  waveformArray[],
                                           ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViString probeString[] = {
    "1:V1",  "1:V2", "1:V3", "1:V4",
    "1:I1",  "1:I2", "1:I3", "1:I4",
    "2:V1",  "2:V2", "2:V3", "2:V4",
    "2:I1",  "2:I2", "2:I3", "2:I4"
             };
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveformSource, RSSCOPE_VAL_WAV_Z1V1, RSSCOPE_VAL_WAV_Z2I4),
            2, "Waveform Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "ZVC:Z%s:DATA?", probeString[waveformSource - RSSCOPE_VAL_WAV_Z1V1]);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Measures the zero offset, the mean value on a currentless DUT. If
/// HIFN  temperature changes, or if you change the shunt or other probe
/// HIFN  settings, repeat the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probeParameter/Sets the probe parameter.
ViStatus _VI_FUNC rsscope_DetectZVCZeroOffset(ViSession instrSession,
                                              ViInt32   multiChannel,
                                              ViInt32   probeParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_ZERO_OFFSET_DETECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the automatic compensation of the measured zero offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR use/Enables the automatic compensation of the measured zero offset.
ViStatus _VI_FUNC rsscope_ConfigureZVCZeroOffsetUse(ViSession instrSession,
                                                    ViInt32   multiChannel,
                                                    ViInt32   probeParameter,
                                                    ViBoolean use)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_ZERO_OFFSET_USE, use),
            4, "Use");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the status of the zero compensation of the indicated current
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiChannel/Sets the multi channel parameter.
/// HIPAR probeParameter/Sets the probe parameter.
/// HIPAR status/Returns the status of the zero compensation of the indicated current
/// HIPAR status/channel.
ViStatus _VI_FUNC rsscope_QueryZVCZeroOffsetStatus(ViSession  instrSession,
                                                   ViInt32    multiChannel,
                                                   ViInt32    probeParameter,
                                                   ViBoolean* status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiChannel, 1, 2),
            2, "Multi Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeParameter, 1, 4),
            3, "Current Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "ZVCZ%ld,ZVCV%ld", multiChannel, probeParameter);

    viCheckParm(rsscope_GetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ZVC_I_ZERO_OFFSET_STATUS, status),
            4, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the state of the internal calibration source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSourceState/Defines the state of the internal calibration source.
ViStatus _VI_FUNC rsscope_ConfigureCalibrationSourceState(ViSession instrSession,
                                                          ViBoolean calibrationSourceState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CALIBRATION_SOURCE_STATE, calibrationSourceState),
            2, "Calibration Source State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the calibration settings to align channels if
/// HIFN  cables of different length are connected to the input connectors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceFrequency/Frequency value of the internal calibration source. Possible values
/// HIPAR sourceFrequency/are a 1 GHz sine wave or a 10 MHz square wave.
/// HIPAR operatingMode/Sets the calibration operating mode. To determine the skew offset, use
/// HIPAR operatingMode/the "Skew" operating mode. It uses the internal channel alignment for
/// HIPAR operatingMode/best results.
/// HIPAR destination/Sets the output for the calbration signal.
ViStatus _VI_FUNC rsscope_ConfigureCalibration(ViSession instrSession,
                                               ViInt32   sourceFrequency,
                                               ViInt32   operatingMode,
                                               ViInt32   destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CALIBRATION_SOURCE_FREQUENCY, sourceFrequency),
            2, "Source Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CALIBRATION_OPERATING_MODE, operatingMode),
            3, "Operating Mode");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CALIBRATION_DESTINATION, destination),
            4, "Destination");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the result of the last selfalignment and the current alignment
/// HIFN  status. In remote mode, *CAL? provides more detailed information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationResult/Returns the result of the last selfalignment and the current alignment
/// HIPAR calibrationResult/status. In remote mode, *CAL? provides more detailed information.
ViStatus _VI_FUNC rsscope_QueryCalibrationResult(ViSession instrSession,
                                                 ViInt32*  calibrationResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_CALIBRATION_RESULT, calibrationResult),
            2, "Calibration Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the date of the last selfalignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR date/Returns the date of the last selfalignment.
/// HIPAR returnedValues/Displays the returned values.
ViStatus _VI_FUNC rsscope_QueryCalibrationDate(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    date[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CAL:DATE?", bufferSize, date, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the time of the last selfalignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR time/Returns the time of the last selfalignment.
/// HIPAR returnedValues/Displays the returned values.
ViStatus _VI_FUNC rsscope_QueryCalibrationTime(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    time[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CAL:TIME?", bufferSize, time, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the extrenal reference oscilator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR externalReference/Sets the frequency of an external reference input signal that is
/// HIPAR externalReference/connected to the external reference input on the rear panel.
/// HIPAR useExternalReference/Enables the use of the external reference signal instead of the
/// HIPAR useExternalReference/internal OCXO reference.
ViStatus _VI_FUNC rsscope_ConfigureReferenceOscillator(ViSession instrSession,
                                                       ViBoolean useExternalReference,
                                                       ViReal64  externalReference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "",
                    RSSCOPE_ATTR_REF_OSCILLATOR_SOURCE,
                    useExternalReference ? RSSCOPE_VAL_EXT : RSSCOPE_VAL_INT
                ), 2, "Use External Reference");

    if (useExternalReference)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_REF_OSCILLATOR_FREQUENCY, externalReference),
                3, "External Reference");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures skew.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR useSkewOffset/Enables/disables skewOffset.
/// HIPAR skewOffset/Sets an delay value, that is known from the circuit specifics
/// HIPAR skewOffset/but cannot be compensated by the instrument automatically.
ViStatus _VI_FUNC rsscope_ConfigureSkew(ViSession instrSession,
                                        ViInt32   channel,
                                        ViBoolean useSkewOffset,
                                        ViReal64  skewOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SKEW_OFFSET_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SKEW_OFFSET_STATE, useSkewOffset),
            3, "Use Skew Offset");

    if (useSkewOffset == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SKEW_OFFSET, skewOffset),
                3, "Skew Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the skew of all connected active probes is measured,
/// HIFN  displayed, and used for deskewing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeSkewOffset/If enabled, the skew of all connected active probes is measured,
/// HIPAR probeSkewOffset/displayed, and used for deskewing.
ViStatus _VI_FUNC rsscope_ConfigureProbeSkewOffset(ViSession instrSession,
                                                   ViBoolean probeSkewOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PROBE_SKEW_STATE, probeSkewOffset),
            3, "Probe Skew Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables a higher skew value range that can be set with
/// HIFN  RSSCOPE_ATTR_SKEW_OFFSET. The skew extension reduces the maximum number
/// HIFN  of acquisitions in the memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR extendedRange/Enables a higher skew value range that can be set with
/// HIPAR extendedRange/RSSCOPE_ATTR_SKEW_OFFSET. The skew extension reduces the maximum number
/// HIPAR extendedRange/of acquisitions in the memory.
ViStatus _VI_FUNC rsscope_ConfigureSkewExtendedRange(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean extendedRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SKEW_EXTENSION_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SKEW_EXTENSION_STATE, extendedRange),
            3, "Extended Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the high definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates the high definition mode of the instrument.
/// HIPAR bandwidth/Sets the filter bandwidth for the high definition mode.
ViStatus _VI_FUNC rsscope_ConfigureHighDefinition(ViSession instrSession,
                                                  ViBoolean state,
                                                  ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_HIGH_DEFINITION_ENABLED, state),
            2, "State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HIGH_DEFINITION_BANDWIDTH, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the resulting digital resolution in high
/// HIFN  definition mode.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_HIGH_DEFINITION_RESOLUTION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     HDEFinition:RESolution?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resolution/Returns the resulting digital resolution in high definition mode.
ViStatus _VI_FUNC rsscope_QueryHighDefinitionResolution(ViSession instrSession,
                                                        ViReal64* resolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_HIGH_DEFINITION_BANDWIDTH, resolution),
            2, "Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common attributes of the trigger subsystem.  These attributes are the trigger type and holdoff.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Sets the trigger type.
ViStatus _VI_FUNC rsscope_ConfigureTrigger(ViSession instrSession,
                                           ViInt32   trigger,
                                           ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the serial bus to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR serialBusSource/Selects the serial bus to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureSerialBusSource(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   serialBusSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SERIAL_BUS_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SERIAL_BUS_SOURCE, serialBusSource),
            3, "Serial Bus Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets coupling of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR eventCoupling/Sets the trigger levels of the channels to the values of the indicated event.
/// HIPAR sourceCoupling/Sets the trigger levels of all channels to the value of channel 1 for the indicated trigger event.
ViStatus _VI_FUNC rsscope_ConfigureTriggerCoupling(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViBoolean eventCoupling,
                                                   ViBoolean sourceCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_EVENT_COUPLING,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_EVENT_COUPLING, eventCoupling),
            3, "Event Coupling");
    if (trigger == RSSCOPE_VAL_TRIGGER_A)
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_SOURCE_COUPLING, sourceCoupling),
                4, "Source Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerModifier/Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
ViStatus _VI_FUNC rsscope_ConfigureTriggerModifier(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   triggerModifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_MODIFIER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_MODIFIER, triggerModifier),
            3, "Trigger Modifier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger level automatically. The attribute is not available for an external trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
ViStatus _VI_FUNC rsscope_TriggerLevelAutoFind(ViSession instrSession,
                                               ViInt32   trigger)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_LEVEL_AUTO_FIND,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_LEVEL_AUTO_FIND, NULL),
            3, "Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures selected trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR channel/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and channel.
ViStatus _VI_FUNC rsscope_ConfigureTriggerChannelLevel(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   triggerSource,
                                                       ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_triggerSource[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);

    if (TriggerSource_rng_to_repCap_map[triggerSource] == VI_FALSE)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_LEVEL,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_LEVEL,
            "TriggerSource", TriggerSource_rng_to_repCap_map[triggerSource]- 1, RS_REPCAP_BUF_SIZE, repCap_triggerSource),
            3, "Trigger Source");
    strcat(repCap, ",");
    strcat(repCap, repCap_triggerSource);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_LEVEL, triggerLevel),
            4, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the edge trigger.  An edge trigger occurs when the trigger signal passes through the voltage threshold that you specify with the Trigger Level parameter and has the slope that
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR triggerSlope/Sets the slope for the edge trigger.
ViStatus _VI_FUNC rsscope_ConfigureEdgeTriggerSource(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   triggerSource,
                                                     ViReal64  triggerLevel,
                                                     ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_LOGIC)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    switch (TriggerSource_rng_to_repCap_map[triggerSource])
    {
    case RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_EXT: // External source
        checkErr(rsscope_ConfigureTrigger(instrSession, trigger, RSSCOPE_VAL_EDGE_TRIGGER));
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_TRIGGER_LEVEL, triggerLevel),
                4, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_TRIGGER_SLOPE, triggerSlope),
                5, "Trigger Slope");
        break;

    case VI_FALSE: // Digital sources
        checkErr(rsscope_ConfigureMSOTrigger(instrSession, RSSCOPE_VAL_MSO_EDGE_TRIGGER));
        checkErr(rsscope_ConfigureMSOEdgeTriggerSource(instrSession, triggerSource - 8, triggerSlope, NULL));
        break;

    default: // Analogue sources
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
                "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Trigger");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
                3, "Trigger Source");
        checkErr(rsscope_ConfigureTrigger(instrSession, trigger, RSSCOPE_VAL_EDGE_TRIGGER));
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SLOPE, triggerSlope),
                5, "Trigger Slope");
        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, trigger,
                        triggerSource, triggerLevel),
                    4, "Trigger Level");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameters of edge trigger for external source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsscope_ConfigureEdgeExternalTrigger(ViSession instrSession,
                                                       ViInt32   coupling,
                                                       ViInt32   filter,
                                                       ViInt32   cutOffFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arrFrequencyAttr[] = {
             VI_FALSE,
             RSSCOPE_ATTR_EXTERNAL_TRIGGER_FILTER_CUT_OFF_FREQUENCY_LOWPASS,
             RSSCOPE_ATTR_EXTERNAL_TRIGGER_FILTER_CUT_OFF_FREQUENCY_HIGHPASS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_TRIGGER_COUPLING, coupling),
            2, "Coupling");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_TRIGGER_FILTER, filter),
            3, "Filter");

    if (arrFrequencyAttr[filter])
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, arrFrequencyAttr[filter], cutOffFrequency),
                4, "Cut-Off Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Connects the analog signal to the ground.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.
/// HIPAR instrSession/instrument session.
/// HIPAR connectToGround/Connects the analog signal to the ground.
ViStatus _VI_FUNC rsscope_ConfigureEdgeExternalTriggerGround(ViSession instrSession,
                                                             ViBoolean connectToGround)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_EXTERNAL_TRIGGER_GROUND_CONNECTION_ENABLED, connectToGround),
            2, "External Trigger Ground Connection Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disbales the noise reject for the external trigger input.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSSCOPE_VAL_EDGE_TRIGGER and source is set to
/// HIFN  RSSCOPE_VAL_EXTERNAL. Call the rsscope_ConfigureTrigger and function to
/// HIFN  set the Trigger Type before you call this function. Call the
/// HIFN  rsscope_ConfigureEdgeTriggerSource to set the Trigger Source before you
/// HIFN  call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseReject/Enables/disbales the noise reject for the external trigger input.
ViStatus _VI_FUNC rsscope_ConfigureEdgeExternalTriggerNoiseReject(
                            ViSession instrSession,
                            ViBoolean noiseReject)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_EXTERNAL_TRIGGER_NOISE_REJECT, noiseReject),
            2, "Noise Reject");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the runt trigger.  A runt trigger occurs when
/// HIFN  the trigger signal crosses one of the runt thresholds twice without
/// HIFN  crossing the other runt threshold.  You specify the runt thresholds
/// HIFN  with the  Runt Low Threshold and Runt High Threshold parameters.  You
/// HIFN  specify the polarity of the runt with the Runt Polarity parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR runtLowThreshold/Sets the lower voltage threshold.
/// HIPAR runtHighThreshold/Sets the upper voltage threshold.
/// HIPAR runtPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR runtPolarity/pulse slope.
ViStatus _VI_FUNC rsscope_ConfigureRuntTriggerSource(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   triggerSource,
                                                     ViReal64  runtLowThreshold,
                                                     ViReal64  runtHighThreshold,
                                                     ViInt32   runtPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_LINE < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_RUNT_LOW_THRESHOLD,
            "TriggerSource", TriggerSource_rng_to_repCap_map[triggerSource]- 1, RS_REPCAP_BUF_SIZE, repCap_c),
            3, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_RUNT_TRIGGER));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_RUNT_POLARITY, runtPolarity),
            6, "Runt Polarity");

    strcat(repCap, ",");
    strcat(repCap, repCap_c);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_RUNT_LOW_THRESHOLD, runtLowThreshold),
            4, "Runt Low Threshold");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_RUNT_HIGH_THRESHOLD, runtHighThreshold),
            5, "Runt High Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Runt trigger range and condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR trigger/runtWidth/Defines the upper or lower voltage threshold.
/// HIPAR trigger/runtDelta/Defines a range around the runt width specified using Runt Width. This
/// HIPAR trigger/control is only available if Runt Condition is set to Within or Outside
/// HIPAR trigger/runtCondition/Defines the time limit of the runt pulse in relation to the Runt Width
/// HIPAR trigger/and Runt Delta settings.
ViStatus _VI_FUNC rsscope_ConfigureRuntTriggerRange(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViReal64  runtWidth,
                                                    ViReal64  runtDelta,
                                                    ViInt32   runtCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_RUNT_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_RUNT_CONDITION, runtCondition),
            5, "Runt Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_RUNT_WIDTH, runtWidth),
            3, "Runt Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_RUNT_DELTA, runtDelta),
            4, "Runt Delta");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the glitch trigger.  A glitch trigger occurs
/// HIFN  when the trigger signal has a pulse with a width that is less than the
/// HIFN  glitch width.  You specify the glitch width in the Glitch Width
/// HIFN  parameter.  You specify the polarity of the pulse with the Glitch
/// HIFN  Polarity parameter.  The trigger does not actually occur until the edge
/// HIFN  of a pulse that corresponds to the Glitch Width and Glitch Polarity
/// HIFN  crosses the threshold you specify in the Trigger Level parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR glitchWidth/Sets the length of a glitch. The instrument triggers on pulses shorter
/// HIPAR glitchWidth/ or longer than this value.
/// HIPAR glitchPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR glitchPolarity/ pulse slope.
/// HIPAR glitchCondition/Selects which glitches are identified: shorter or longer than the
/// HIPAR glitchPolarity/ width.
ViStatus _VI_FUNC rsscope_ConfigureGlitchTriggerSource(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   triggerSource,
                                                       ViReal64  triggerLevel,
                                                       ViReal64  glitchWidth,
                                                       ViInt32   glitchPolarity,
                                                       ViInt32   glitchCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_LINE < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_GLITCH_TRIGGER));

    viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, trigger, triggerSource, triggerLevel), 4,
                "Trigger Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_GLITCH_WIDTH, glitchWidth),
            5, "Glitch Width");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_GLITCH_POLARITY, glitchPolarity),
            6, "Glitch Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_GLITCH_CONDITION, glitchCondition),
            7, "Glitch Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the width trigger.  A width trigger occurs
/// HIFN  when the oscilloscope detects a positive or negative pulse with a width
/// HIFN  between, or optionally outside, the width thresholds.  You specify the
/// HIFN  width thresholds with the Width Low Threshold and Width High Threshold
/// HIFN  parameters.  You specify whether the oscilloscope triggers on pulse
/// HIFN  widths that are within or outside the width thresholds with the Width
/// HIFN  Condition parameter.  You specify the polarity of the pulse with the
/// HIFN  Width Polarity parameter.  The trigger does not actually occur until
/// HIFN  the edge of a pulse that corresponds to the Width Low Threshold, Width
/// HIFN  High Threshold, Width Condition, and Width Polarity crosses the
/// HIFN  threshold you specify in the Trigger Level parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR width/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR width/ a range which is defined by the limits "+/-Delta". For the ranges
/// HIPAR width/ "Shorter" and "Longer", the width defines the maximum and minimum pulse
/// HIPAR width/ width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR trigger/Selects trigger event.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR widthCondition/Sets how the measured pulse width is compared with the given limit(s).
ViStatus _VI_FUNC rsscope_ConfigureWidthTriggerSource(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   triggerSource,
                                                      ViReal64  triggerLevel,
                                                      ViReal64  width,
                                                      ViReal64  widthDelta,
                                                      ViInt32   widthPolarity,
                                                      ViInt32   widthCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_LOGIC || triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_MSOB4 < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    switch (TriggerSource_rng_to_repCap_map[triggerSource])
    {
    case VI_FALSE: // Digital sources
        checkErr(rsscope_ConfigureMSOTrigger(instrSession, RSSCOPE_VAL_MSO_WIDTH_TRIGGER));
        checkErr(rsscope_ConfigureMSOWidthTriggerSource(instrSession, triggerSource - 8,
            widthPolarity, widthCondition, width, widthDelta, NULL));
        break;

    default: // Analogue sources
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
                "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Trigger");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
                3, "Trigger Source");

        checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_WIDTH_TRIGGER));

        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, trigger,
                        triggerSource, triggerLevel),
                    4, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WIDTH, width),
                5, "Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WIDTH_DELTA, widthDelta),
                6, "Width Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WIDTH_POLARITY, widthPolarity),
                7, "Width Polarity");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WIDTH_CONDITION, widthCondition),
                8, "Width Condition");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the window trigger. The window trigger checks
/// HIFN  the signal run in relation to a "window". The window is formed by the
/// HIFN  upper and lower voltage levels. The event condition is fulfilled, if
/// HIFN  the waveform enters or leaves the window, or if the waveform stays
/// HIFN  inside or outside for a time longer or shorter than specified. With the
/// HIFN  window trigger, you can display longer transient effects.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR windowLowerLimit/Sets the lower voltage limit for the window.
/// HIPAR windowUpperLimit/Sets the upper voltage limit for the window.
/// HIPAR windowWidth/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR windowWidth/ a time range which is defined by the limits "Delta".
/// HIPAR windowDelta/Defines a range around the "Width" value.
/// HIPAR windowCondition/Defines the signal run in relation to the window.
ViStatus _VI_FUNC rsscope_ConfigureWindowTriggerSource(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   triggerSource,
                                                       ViReal64  windowLowerLimit,
                                                       ViReal64  windowUpperLimit,
                                                       ViReal64  windowWidth,
                                                       ViReal64  windowDelta,
                                                       ViInt32   windowCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_LINE < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_RUNT_LOW_THRESHOLD,
            "TriggerSource", TriggerSource_rng_to_repCap_map[triggerSource]- 1, RS_REPCAP_BUF_SIZE, repCap_c),
            3, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_WINDOW_TRIGGER));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WINDOW_CONDITION, windowCondition),
            8, "Window Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WINDOW_WIDTH, windowWidth),
            6, "Window Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WINDOW_DELTA, windowDelta),
            7, "Window Delta");

    strcat(repCap, ",");
    strcat(repCap, repCap_c);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WINDOW_UPPER_LIMIT, windowUpperLimit),
            4, "Window Upper Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WINDOW_LOWER_LIMIT, windowLowerLimit),
            3, "Window Lower Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects how the time limit of the window is
/// HIFN defined. Time conditioning is available for the vertical
/// HIFN conditions "Stay within" and "Stay outside".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR timeCondition/Selects how the time limit of the window is defined. Time
/// HIPAR timeCondition/conditioning is available for the vertical conditions "Stay
/// HIPAR timeCondition/within" and "Stay outside".
ViStatus _VI_FUNC rsscope_ConfigureWindowTriggerTimeCondition(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   timeCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger != RSSCOPE_VAL_TRIGGER_A && trigger != RSSCOPE_VAL_TRIGGER_R)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, trigger), 2, "Trigger");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WINDOW_TIME_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WINDOW_TIME_CONDITION, timeCondition),
            8, "Time Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the timeout trigger. The timeout trigger
/// HIFN  event checks if the signal stays above or below the threshold voltage
/// HIFN  for a specified time lapse. In other words, the event occurs if the
/// HIFN  trigger source does not have the expected transition within the
/// HIFN  specified time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR timeoutValue/Defines the time limit for the timeout at which the instrument
/// HIPAR timeoutValue/ triggers.
/// HIPAR timeoutCondition/Defines the relation of the signal level to the trigger level.
ViStatus _VI_FUNC rsscope_ConfigureTimeoutTriggerSource(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   triggerSource,
                                                        ViReal64  triggerLevel,
                                                        ViReal64  timeoutValue,
                                                        ViInt32   timeoutCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_MSOB4 < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    switch (TriggerSource_rng_to_repCap_map[triggerSource])
    {
    case VI_FALSE: // Digital sources
        checkErr(rsscope_ConfigureMSOTrigger(instrSession, RSSCOPE_VAL_MSO_TIMEOUT_TRIGGER));
        checkErr(rsscope_ConfigureMSOTimeoutTriggerSource(instrSession, triggerSource - 8,
            timeoutValue, timeoutCondition, NULL));
        break;

    default: // Analogue sources

        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
                "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Trigger");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
                3, "Trigger Source");

        checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_TIMEOUT_TRIGGER));

        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, trigger, triggerSource, triggerLevel),
                    4, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TIMEOUT_VALUE, timeoutValue),
                5, "Timeout Value");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TIMEOUT_CONDITION, timeoutCondition),
                6, "Timeout Condition");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The interval trigger analyzes the time between two pulses. While the
/// HIFN  interval trigger can only analyze either positive or negative polarity,
/// HIFN  searching for an interval is also possible for both polarities at the
/// HIFN  same time ("Either").
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR intervalWidth/Defines the time between two pulses.
/// HIPAR intervalDelta/Defines a range around the "Interval width" value.
/// HIPAR intervalPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR intervalPolarity/ pulse slope.
/// HIPAR intervalCondition/Defines the range of an interval in relation to the interval width.
ViStatus _VI_FUNC rsscope_ConfigureIntervalTriggerSource(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   triggerSource,
                                                         ViReal64  triggerLevel,
                                                         ViReal64  intervalWidth,
                                                         ViReal64  intervalDelta,
                                                         ViInt32   intervalPolarity,
                                                         ViInt32   intervalCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_LINE < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_INTERVAL_TRIGGER));

    viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, trigger, triggerSource, triggerLevel),
                4, "Trigger Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_INTERVAL_WIDTH, intervalWidth),
            5, "Interval Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_INTERVAL_DELTA, intervalDelta),
            6, "Interval Delta");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_INTERVAL_POLARITY, intervalPolarity),
            7, "Interval Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_INTERVAL_CONDITION, intervalCondition),
            8, "Interval Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The slew rate trigger, also known as transition trigger, can detect
/// HIFN  fast or slow edges selectively. It triggers on edges, if the transition
/// HIFN  time from the lower to higher voltage level (or vice versa) is shorter
/// HIFN  or longer as defined, or outside a specified time range. The trigger
/// HIFN  event finds slew rates faster than expected or permissible to avoid
/// HIFN  overshooting and other interfering effects. It also detects very slow
/// HIFN  edges violating the timing in pulse series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR slewRateCondition/Defines the time limit for the slew rate in relation to the upper or lower trigger level.
/// HIPAR slewRateUpperLimit/Defines the upper voltage threshold.
/// HIPAR slewRatePolarity/Selects the edge type for the trigger event.
/// HIPAR slewRateLowerLimit/Defines the lower voltage threshold.
ViStatus _VI_FUNC rsscope_ConfigureSlewRateTriggerSource(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   triggerSource,
                                                         ViReal64  slewRateLowerLimit,
                                                         ViReal64  slewRateUpperLimit,
                                                         ViInt32   slewRatePolarity,
                                                         ViInt32   slewRateCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    if (triggerSource > RSSCOPE_VAL_CHANNEL4 && trigger != RSSCOPE_VAL_TRIGGER_A)
        checkErr(RS_ERROR_INVALID_CONFIGURATION);
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    if (RSSCOPE_VAL_LINE < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 3, "Trigger Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SOURCE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SLEW_UPPER_LIMIT,
            "TriggerSource", TriggerSource_rng_to_repCap_map[triggerSource]- 1, RS_REPCAP_BUF_SIZE, repCap_c),
            3, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_SLEW_RATE_TRIGGER));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SLEW_POLARITY, slewRatePolarity),
            6, "Slew Rate Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SLEW_CONDITION, slewRateCondition),
            7, "Slew Rate Condition");

    strcat(repCap, ",");
    strcat(repCap, repCap_c);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SLEW_UPPER_LIMIT, slewRateUpperLimit),
            5, "Slew Rate Upper Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SLEW_LOWER_LIMIT, slewRateLowerLimit),
            4, "Slew Rate Lower Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the slew rate range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR slewRate/For the ranges "Within" and "Outside", the slew rate defines the
/// HIPAR slewRate/ center of a range which is defined by the limits "Delta".
/// HIPAR slewRateDelta/Defines a time range around the slew rate.
ViStatus _VI_FUNC rsscope_ConfigureSlewRateTriggerRange(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViReal64  slewRate,
                                                        ViReal64  slewRateDelta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SLEW_RATE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SLEW_RATE, slewRate),
            3, "Slew Rate");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SLEW_DELTA, slewRateDelta),
            4, "Slew Rate Delta");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  With the Data2Clock event - also known as setup/hold - you can analyze
/// HIFN  the relative timing between two signals: a data signal and the
/// HIFN  synchronous clock signal. Many systems require, that the data signal
/// HIFN  must be steady for some time before and after the clock edge, for
/// HIFN  example, the data transmission on parallel interfaces. With this
/// HIFN  trigger type, you can also test the time correlation of sideband and
/// HIFN  inband signals. The event occurs if the data signal crosses the data
/// HIFN  level during the setup and hold time. The reference point for the time
/// HIFN  measurement is defined by clock level and clock edge.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR holdTime/Sets the minimum time after the clock edge while the data signal must
/// HIPAR holdTime/ stay steady above or below the data level. The hold time can be
/// HIPAR holdTime/ negative. In this case, the setup time is always positive. The
/// HIPAR holdTime/hold interval starts before the clock edge (setup time) and ends
/// HIPAR holdTime/ before the clock edge (hold time). If you change the negative hold
/// HIPAR holdTime/ time, the setup time is adjusted by the intrument.
/// HIPAR setupTime/Sets the minimum time before the clock edge while the data signal must
/// HIPAR setupTime/ stay steady above or below the data level. The setup time can be
/// HIPAR setupTime/ negative. In this case, the hold time is always positive. The
/// HIPAR setupTime/ hold interval starts after the clock edge (setup time) and ends
/// HIPAR setupTime/ after the clock edge (hold time). If you change the negative setup
/// HIPAR setupTime/ time, the hold time is adjusted by the intrument.
/// HIPAR clockLevel/Sets the voltage level for the clock signal.
/// HIPAR data2ClockPolarity/Sets the edge of the clock signal to define the time reference point
/// HIPAR data2ClockPolarity/ for the setup and hold time.
/// HIPAR clockSource/Sets the source of clock signal.
/// HIPAR dataLevel/Sets the voltage level of data signal.
ViStatus _VI_FUNC rsscope_ConfigureData2ClockTriggerSource(ViSession instrSession,
                                                           ViInt32   triggerSource,
                                                           ViReal64  dataLevel,
                                                           ViInt32   clockSource,
                                                           ViReal64  clockLevel,
                                                           ViReal64  holdTime,
                                                           ViReal64  setupTime,
                                                           ViInt32   data2ClockPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            2, "Trigger Source");
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 2, "Trigger Source");
    }

    if (RSSCOPE_VAL_MSOB4 < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 2, "Trigger Source");
    }

    if (TriggerSource_rng_to_repCap_map[triggerSource])
    {
        // For analogue sources, clock source must be analogue
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, clockSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
                3, "Clock Source");
    }
    else // For digital sources, clock source must be digital
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, clockSource, RSSCOPE_VAL_D0, RSSCOPE_VAL_D15),
                3, "Clock Source");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TRESHOLD,
                "DigitalChannel", triggerSource- RSSCOPE_VAL_D0, RS_REPCAP_BUF_SIZE, repCap),
                2, "Trigger Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    switch (TriggerSource_rng_to_repCap_map[triggerSource])
    {
    case VI_FALSE: // Digital sources
        checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_TRIGGER_TYPE, RSSCOPE_VAL_MSO_DATA_TO_CLOCK_TRIGGER));
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_CLOCK_SOURCE, clockSource),
                4, "Clock Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_TRESHOLD, clockLevel),
                5, "Clock Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_HOLD_TIME, holdTime),
                6, "Hold Time");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_SETUP_TIME, setupTime),
                7, "Setup Time");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_CLOCK_EDGE, data2ClockPolarity),
                8, "Data2Clock Polarity");
        break;

    default: // Analogue sources
        checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_DATA_TO_CLOCK_TRIGGER));

        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, RSSCOPE_VAL_TRIGGER_A, triggerSource, dataLevel),
                    3, "Data Level");
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_SOURCE, clockSource),
                4, "Clock Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_LEVEL, clockLevel),
                5, "Clock Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_HOLD_TIME, holdTime),
                6, "Hold Time");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_SETUP_TIME, setupTime),
                7, "Setup Time");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_POLARITY, data2ClockPolarity),
                8, "Data2Clock Polarity");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the edge of the clock signal to define the time
///HINF  reference point for the setup and hold time.
///HIRET Returns the status code of this operation.
///HIPAR signalEdge/Sets the edge of the clock signal to define the
///HIPAR signalEdge/time reference point for the setup and hold time.
ViStatus _VI_FUNC rsscope_ConfigureData2ClockSignalEdge(ViSession instrSession,
                                                        ViInt32   signalEdge)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_DATA2CLOCK_SIGNAL_EDGE, signalEdge),
            2, "Signal Edge");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the range of a pulse width is defined for keeping up the
/// HIFN  true result of the pattern condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternMode/Adds additional time limitation to the pattern definition.
/// HIPAR timeoutCondition/Defines the condition for the timeout.
/// HIPAR patternWidth/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR patternWidth/ a range which is defined by the limits "Delta".
/// HIPAR patternDelta/Defines a range around the width value.
/// HIPAR patternCondition/Defines how the range of a pulse width is defined for keeping up the
/// HIPAR patternCondition/  true result of the pattern condition.
/// HIPAR timeout/Defines how long the result of the pattern condition must be true or
/// HIPAR timeout/ false.
ViStatus _VI_FUNC rsscope_ConfigurePatternTriggerSource(ViSession instrSession,
                                                        ViInt32   patternMode,
                                                        ViInt32   timeoutCondition,
                                                        ViReal64  timeout,
                                                        ViReal64  patternWidth,
                                                        ViReal64  patternDelta,
                                                        ViInt32   patternCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_PATTERN_TRIGGER));
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PATTERN_MODE, patternMode),
            2, "Pattern Mode");

    if (patternMode == RSSCOPE_VAL_PATTERN_TIMEOUT)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PATTERN_TIMEOUT_MODE, timeoutCondition),
                3, "Timeout Condition");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PATTERN_TIMEOUT, timeout),
                4, "Timeout");
    }
    if (patternMode == RSSCOPE_VAL_PATTERN_WIDTH)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PATTERN_WIDTH, patternWidth),
                5, "Pattern Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PATTERN_DELTA, patternDelta),
                6, "Pattern Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PATTERN_CONDITION, patternCondition),
                7, "Pattern Condition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The serial pattern event is used to trigger on signals with serial
/// HIFN  data patterns in relation to a clock signal - for example, on bus
/// HIFN  signals like the I2C bus. For convenient and comprehensive triggering
/// HIFN  on specific serial data, options for serial protocol analysis are
/// HIFN  provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR clockSource/Defines the input channel of the clock signal.
/// HIPAR clockLevel/Defines the voltage level for the clock signal.
/// HIPAR pattern/The pattern contains the bits of the serial data to be found in the
/// HIPAR pattern/ data stream. The maximum length of the pattern is 128 bit. You can
/// HIPAR pattern/ enter the pattern in binary or hexadecimal format.
/// HIPAR patternPolarity/Together with the clock level, the clock edge defines the point in
/// HIPAR patternPolarity/ time when the state of the data signal is checked.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR dataLevel/Sets the trigger level for the specified event and source.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource,
                            ViReal64  dataLevel,
                            ViInt32   clockSource,
                            ViReal64  clockLevel,
                            ViString  pattern,
                            ViInt32   patternPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            2, "Trigger Source");
    if (triggerSource == RSSCOPE_VAL_EXTERNAL)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 2, "Trigger Source");
    }
    if (RSSCOPE_VAL_MSOB4 < triggerSource && triggerSource < RSSCOPE_VAL_DIFF1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 2, "Trigger Source");
    }

    if (TriggerSource_rng_to_repCap_map[triggerSource])
    {
        // For analogue sources, clock source must be analogue
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, clockSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
                3, "Clock Source");
    }
    else // For digital sources, clock source must be digital
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, clockSource, RSSCOPE_VAL_D0, RSSCOPE_VAL_D15),
                3, "Clock Source");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TRESHOLD,
                "DigitalChannel", triggerSource- RSSCOPE_VAL_D0, RS_REPCAP_BUF_SIZE, repCap),
                2, "Trigger Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    switch (TriggerSource_rng_to_repCap_map[triggerSource])
    {
    case VI_FALSE: // Digital sources
        checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_TRIGGER_TYPE, RSSCOPE_VAL_MSO_SERIAL_PATTERN_TRIGGER));
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_CLOCK_SOURCE, clockSource),
                4, "Clock Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_TRESHOLD, clockLevel),
                5, "Clock Level");

        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_PATTERN, pattern),
                6, "Pattern");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_CLOCK_EDGE, patternPolarity),
                7, "Pattern Polarity");
        break;

    default: // Analogue sources
        checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_TRIGGER));
        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession,
                        RSSCOPE_VAL_TRIGGER_A, triggerSource, dataLevel),
                    3, "Data Level");
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_SOURCE, clockSource),
                4, "Clock Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_LEVEL, clockLevel),
                5, "Clock Level");

        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN, pattern),
                6, "Pattern");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_POLARITY, patternPolarity),
                7, "Pattern Polarity");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects if an explicit clock or a CDR signal is used as a
///HINF  reference signal.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR referenceSource/Selects if an explicit clock or a CDR signal
///HIPAR referenceSource/is used as a reference signal.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternReferenceSource(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   referenceSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SERIAL_PATTERN_REFERENCE_SOURCE,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repcap, RSSCOPE_ATTR_SERIAL_PATTERN_REFERENCE_SOURCE, referenceSource),
            3, "Reference Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the pattern type for the serial pattern trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternType/Selects the pattern type for the serial pattern trigger.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternTriggerType(ViSession instrSession,
                                                            ViInt32   patternType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, patternType),
            2, "Pattern Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and configures the single bit pattern trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Defines the bit pattern.
/// HIPAR patternLength/Sets the length of the bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternSingleBitPattern(
                            ViSession instrSession,
                            ViString  pattern,
                            ViInt32   patternLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_SBP));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_A, pattern),
            2, "Pattern");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_LENGTH_A, patternLength),
            3, "Pattern Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and configures the dual bit pattern trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternA/Defines bit pattern A.
/// HIPAR patternLengthA/Sets the length of bit pattern A.
/// HIPAR patternB/Defines bit pattern B.
/// HIPAR patternLengthB/Sets the length of bit pattern B.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternDualBitPattern(
                            ViSession instrSession,
                            ViString  patternA,
                            ViInt32   patternLengthA,
                            ViString  patternB,
                            ViInt32   patternLengthB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_DBP));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_A, patternA),
            2, "Pattern A");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_LENGTH_A, patternLengthA),
            3, "Pattern Length A");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_A, patternB),
            4, "Pattern B");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_BIT_PATTERN_LENGTH_A, patternLengthB),
            5, "Pattern Length B");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the complex word pattern trigger type and configures settings
/// HIFN  for pattern matching and for aligning to a frame ID in the form of a
/// HIFN  bit pattern or timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignmentType/Selects if the alignment for the complex word pattern triggering is
/// HIPAR alignmentType/done on a pattern value or on a defined timeout.
/// HIPAR alignmentLength/Sets the length of the pattern for the trigger alignment condition. If
/// HIPAR alignmentLength/alignment type is not pattern, this parameter is ignored and can be
/// HIPAR alignmentLength/NULL.
/// HIPAR alignmentPattern/Sets the bit pattern value for the trigger alignment condition, if
/// HIPAR alignmentPattern/RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_TYPE is set to
/// HIPAR alignmentPattern/PATTern. If alignment type is not pattern, this parameter is ignored
/// HIPAR alignmentPattern/and can be NULL.
/// HIPAR alignmentTimeout/Sests the timeout, a duration that defines an idle mark, if
/// HIPAR alignmentTimeout/RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_TYPE is set to
/// HIPAR alignmentTimeout/TIMEout. If alignment type is not timeout, this parameter is ignored
/// HIPAR alignmentTimeout/and can be NULL.
/// HIPAR idleRange/Selects if the timeout should be within the range or out of range. If
/// HIPAR idleRange/alignment type is not timeout, this parameter is ignored and can be
/// HIPAR idleRange/NULL.
/// HIPAR patternLogic/Selects the logic for the condition, if more than one pattern
/// HIPAR patternLogic/conditions are defined.
/// HIPAR bitOrder/Sets the most significant bit (MSB) or least significant bit (LSB) to
/// HIPAR bitOrder/define the data bit order.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternComplexWordAlignment(
                            ViSession instrSession,
                            ViInt32   alignmentType,
                            ViInt32   alignmentLength,
                            ViString  alignmentPattern,
                            ViReal64  alignmentTimeout,
                            ViInt32   idleRange,
                            ViInt32   patternLogic,
                            ViInt32   bitOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_CBP));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_TYPE, alignmentType),
            2, "Alignment Type");

    switch (alignmentType)
    {
    case RSSCOPE_VAL_COMPLEX_WORD_ALIGNMENT_PATTERN:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_LENGTH, alignmentLength),
                3, "Alignment Length");

        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_PATTERN, alignmentPattern),
                4, "Alignment Pattern");
        break;
    case RSSCOPE_VAL_COMPLEX_WORD_ALIGNMENT_TIMEOUT:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_ALIGNMENT_TIMEOUT, alignmentTimeout),
                5, "Alignment Timeout");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_IDLE_RANGE, idleRange),
                6, "Idle Range");
        break;
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_LOGIC, patternLogic),
            7, "Pattern Logic");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_BIT_ORDER, bitOrder),
            8, "Bit Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a pattern condition for a complex word trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR conditionEnabled/Enables a new pattern condition. If this is set to Off, all other
/// HIPAR conditionEnabled/parameters are ignored and can be NULL.
/// HIPAR complexCondition/Selects a complex word pattern comdition to configure.
/// HIPAR bitOffset/Sets the number of arbitrary bits between the first bit after the
/// HIPAR bitOffset/frame id and the first bit of the word pattern.
/// HIPAR patternLength/Sets the word pattern length in bits. The maximal bit length of all
/// HIPAR patternLength/selected pattern conditions is 160 bits.
/// HIPAR searchRange/Sets the search range of the word pattern.
/// HIPAR condition/Sets the operator to define the exact position or range where the
/// HIPAR condition/instrument looks for the specified pattern.
/// HIPAR pattern/Sets the pattern or a start value of a pattern range.
/// HIPAR patternRangeEnd/Sets the end value of a pattern range if
/// HIPAR patternRangeEnd/RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_PATTERN_CONDITION is set to
/// HIPAR patternRangeEnd/INRange or OORange. If any other pattern condition is set, this
/// HIPAR patternRangeEnd/paramter is ignored and can be NULL.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternComplexWordCondition(
                            ViSession instrSession,
                            ViInt32   complexCondition,
                            ViBoolean conditionEnabled,
                            ViInt32   bitOffset,
                            ViInt32   patternLength,
                            ViInt32   searchRange,
                            ViInt32   condition,
                            ViString  pattern,
                            ViString  patternRangeEnd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, complexCondition, 1, 4),
            2, "Complex Condition");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_CONDITION_ENABLED,
            "ComplexCondition", complexCondition- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Complex Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_BIT_OFFSET, bitOffset),
            4, "Bit Offset");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_PATTERN_LENGTH, patternLength),
            5, "Pattern Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_SEARCH_RANGE, searchRange),
            6, "Search Range");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_PATTERN_CONDITION, condition),
            7, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_PATTERN, pattern),
            8, "Pattern");
    if (condition <= RSSCOPE_VAL_COMPLEX_PATTERN_CONDITION_OUT_OF_RANGE)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_PATTERN_RANGE_END, patternRangeEnd),
                9, "Pattern Range End");
    }

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_COMPLEX_WORD_CONDITION_ENABLED, conditionEnabled),
            3, "Condition Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and configures the PRBS error pattern trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitErrorRate/Sets the bit error rate for the PRBS sequence.
/// HIPAR nominalBitRate/Sets the nominal bit error rate for the PRBS sequence. It corresponds
/// HIPAR nominalBitRate/to the data rate of the data stream from which the clock is to be
/// HIPAR nominalBitRate/recovered.
/// HIPAR sequence/Selects a predefined pseudorandom binary sequence.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternPRBSError(ViSession instrSession,
                                                          ViReal64  bitErrorRate,
                                                          ViReal64  nominalBitRate,
                                                          ViInt32   sequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_PRBS));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_PRBS_ERROR_RATE, bitErrorRate),
            2, "Bit Error Rate");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_PRBS_NOMINAL_BIT_RATE, nominalBitRate),
            3, "Nominal Bit Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_PRBS_SEQUENCE, sequence),
            4, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects one of the comma control symbols used for synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignmentSymbol/Selects one of the comma control symbols used for synchronization. The
/// HIPAR alignmentSymbol/alignment symbol cannot be reused in a pattern.
/// HIPAR scrambling/Selects if the data is scrambled or unscrambled.
/// HIPAR disparityErrorEnabled/Enables triggering on disparity errors.
/// HIPAR symbolErrorEnabled/Enables triggering on symbol errors.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternEbTb(ViSession instrSession,
                                                     ViInt32   alignmentSymbol,
                                                     ViInt32   scrambling,
                                                     ViBoolean disparityErrorEnabled,
                                                     ViBoolean symbolErrorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_EBTB));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ALIGNMENT_SYMBOL, alignmentSymbol),
            2, "Alignment Symbol");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_SCRAMBLING, scrambling),
            3, "Scrambling");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_DISPARITY_ERROR_ENABLED, disparityErrorEnabled),
            4, "Disparity Error Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_SYMBOL_ERROR_ENABLED, symbolErrorEnabled),
            5, "Symbol Error Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of 8b/10b symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rowCount/Returns the number of 8b/10b symbols.
ViStatus _VI_FUNC rsscope_QuerySerialPatternEbTbRowCount(ViSession instrSession,
                                                         ViInt32*  rowCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT, rowCount),
            2, "Row Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Appends a new row to the bottom of 8b/10b table, returning a count of
/// HIFN  the rows in the table, which will equal the row number of the added
/// HIFN  row.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR korDSymbol/Sets the data character (Dx.y) or the control character (Kx.y). This
/// HIPAR korDSymbol/cannot be the same as the alignment symbol
/// HIPAR korDSymbol/RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ALIGNMENT_SYMBOL.
/// HIPAR ebBit/Sets the 8b bit value.
/// HIPAR rowCount/Returns the new count of 8b/10b symbols, after appending this one.
/// HIPAR rowCount/This will equal the row number of the added row.
ViStatus _VI_FUNC rsscope_SerialPatternEbTbAppendSymbol(ViSession instrSession,
                                                        ViString  korDSymbol,
                                                        ViString  ebBit,
                                                        ViInt32*  rowCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_APPEND_ROW, NULL));
    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT, rowCount),
            4, "Row Count");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, *rowCount, 1, 16),
            4, "Row Count");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_KD_SYMBOL,
            "EbTbSymbol", *rowCount- 1, RS_REPCAP_BUF_SIZE, repCap),
            4, "Row Count");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_KD_SYMBOL, korDSymbol),
            2, "K or D Symbol");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_EIGHT_BIT_VALUE, ebBit),
            3, "Eb Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Modifies a specified row of the 8b/10b table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableRow/Selects a table row for modification. To find the number of rows
/// HIPAR tableRow/currently in the table, see RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT
/// HIPAR tableRow/or rsscope_QuerySerialPatternEbTbRowCount.
/// HIPAR korDSymbol/Sets the data character (Dx.y) or the control character (Kx.y). This
/// HIPAR korDSymbol/cannot be the same as the alignment symbol
/// HIPAR korDSymbol/RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ALIGNMENT_SYMBOL.
/// HIPAR ebBit/Sets the 8b bit value.
ViStatus _VI_FUNC rsscope_SerialPatternEbTbModifySymbol(ViSession instrSession,
                                                        ViInt32   tableRow,
                                                        ViString  korDSymbol,
                                                        ViString  ebBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  rowCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT, &rowCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tableRow, 1, rowCount),
            2, "Table Row");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_TEN_BIT_D_PLUS,
            "Channel", tableRow- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Table Row");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_KD_SYMBOL, korDSymbol),
            3, "K or D Symbol");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_EIGHT_BIT_VALUE, ebBit),
            4, "Eb Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value for the positive running disparity RD+. Buffers
/// HIFN  passed to fields 'Tb RD Plus' and 'Tb RD Minus' must be at least 64
/// HIFN  bytes in length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableRow/Selects a table row for the running disparity query. To find the
/// HIPAR tableRow/number of rows currently in the table, see
/// HIPAR tableRow/RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT or
/// HIPAR tableRow/rsscope_QuerySerialPatternEbTbRowCount.
/// HIPAR tbRDPlus/Returns the value for the positive running disparity RD+. Buffer
/// HIPAR tbRDPlus/passed for this field must be at least 64 bytes in length.
/// HIPAR tbRDMinus/Returns the value for the negative running disparity RD-. Buffer
/// HIPAR tbRDMinus/passed for this field must be at least 64 bytes in length.
ViStatus _VI_FUNC rsscope_QuerySerialPatternEbTbRunningDisparity(
                            ViSession instrSession,
                            ViInt32   tableRow,
                            ViChar    tbRDPlus[],
                            ViChar    tbRDMinus[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  rowCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_ROW_COUNT, &rowCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tableRow, 1, rowCount),
            2, "Table Row");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_TEN_BIT_D_PLUS,
            "Channel", tableRow- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Table Row");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_TEN_BIT_D_PLUS,
            RS_MAX_SHORT_MESSAGE_BUF_SIZE, tbRDPlus),
            3, "Tb RD Plus");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_TEN_BIT_D_MINUS,
            RS_MAX_SHORT_MESSAGE_BUF_SIZE, tbRDMinus),
            4, "Tb RD Minus");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all entries from the 8b/10b table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_SerialPatternEbTbClearTable(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_CLEAR_TABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the frame ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ebtbFrameID/Selects the frame ID.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternEbTbFrameID(ViSession instrSession,
                                                            ViInt32   ebtbFrameID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_EBTB_FRAME_ID, ebtbFrameID),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and configures the 128b/130b trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects repeated capability Trigger.
/// HIPAR alignmentPattern/Sets the alignment pattern.
/// HIPAR skipStart/Sets the start skip value.
/// HIPAR skipEnd/Sets the skip end symbol.
/// HIPAR skipBytes/Sets the skip bytes symbols send after a skip end symbol.
/// HIPAR type/Selects the type of pattern, control block or the data block.
/// HIPAR pattern/Defines the control block or data block pattern in binary or hex
/// HIPAR pattern/format.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternI30B(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  alignmentPattern,
                                                     ViString  skipStart,
                                                     ViString  skipEnd,
                                                     ViInt32   skipBytes,
                                                     ViInt32   type,
                                                     ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrigger = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &maxTrigger);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, maxTrigger),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_LENGTH,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_I30B));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_ALIGNMENT_PATTERN, alignmentPattern),
            3, "Alignment Pattern");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_SKIP_START, skipStart),
            4, "Skip Start");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_SKIP_END, skipEnd),
            5, "Skip End");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_SKIP_BYTES, skipBytes),
            6, "Skip Bytes");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_TYPE, type),
            7, "Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SERIAL_PATTERN_I30B_PATTERN, pattern),
            8, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and configures the 128b/132b trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignmentPattern/Sets the alignment pattern.
/// HIPAR skipStart/Sets the start skip value.
/// HIPAR skipEnd/Sets the skip end symbol.
/// HIPAR skipBytes/Sets the skip bytes symbols send after a skip end symbol.
/// HIPAR type/Selects the type of pattern, control block or the data block.
/// HIPAR pattern/Defines the control block or data block pattern in binary or hex
/// HIPAR pattern/format.
ViStatus _VI_FUNC rsscope_ConfigureSerialPatternI32B(ViSession instrSession,
                                                     ViString  alignmentPattern,
                                                     ViString  skipStart,
                                                     ViString  skipEnd,
                                                     ViInt32   skipBytes,
                                                     ViInt32   type,
                                                     ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_TYPE, RSSCOPE_VAL_SERIAL_PATTERN_MODE_SBP));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_ALIGNMENT_PATTERN, alignmentPattern),
            2, "Alignment Pattern");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_SKIP_START, skipStart),
            3, "Skip Start");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_SKIP_END, skipEnd),
            4, "Skip End");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_SKIP_BYTES, skipBytes),
            5, "Skip Bytes");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_TYPE, type),
            6, "Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SERIAL_PATTERN_I32B_PATTERN, pattern),
            7, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the decoding of the selected bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocol/Enables the decoding of the selected bus.
ViStatus _VI_FUNC rsscope_ConfigureSerialBusTriggerSource(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   protocol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_SOURCE, RSSCOPE_VAL_SBUS));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_TYPE, protocol),
            3, "Protocol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the rising or falling edges of the AC power input for the
/// HIFN  trigger condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSlope/Selects the rising or falling edges of the AC power input for the
/// HIPAR triggerSlope/trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureLineTriggerSlope(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_AC_POWER_INPUT_SLOPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_AC_POWER_INPUT_SLOPE, triggerSlope),
            3, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the NFC trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR technology/Selects the NFC technology, the communication protocol used by the
/// HIPAR technology/input signal.
/// HIPAR event/Sets the event to be triggered on.
/// HIPAR bitRate/Sets the bit rate of the signal.
ViStatus _VI_FUNC rsscope_ConfigureNFCTrigger(ViSession instrSession,
                                              ViInt32   triggerSource,
                                              ViInt32   technology,
                                              ViInt32   bitRate,
                                              ViInt32   event,
                                              ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
            2, "Trigger Source");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "TrigA,CH%ld", triggerSource);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_NFC_TRIGGER));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NFC_TRIGGER_TECHNOLOGY, technology),
            3, "Technology");

    if (technology == RSSCOPE_VAL_NFC_F)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NFC_TRIGGER_BIT_RATE, bitRate),
                4, "Bit Rate");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NFC_TRIGGER_EVENT, event),
            5, "Event");

    if (triggerSource <= RSSCOPE_VAL_EXTERNAL)
        viCheckParm(rsscope_ConfigureTriggerChannelLevel(instrSession, RSSCOPE_VAL_TRIGGER_A, triggerSource, triggerLevel),
                6, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the serial standard, data level, software algorithm,
///HINF  sampling time and the channel signal that are used for clock
///HINF  data recovery.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR serialStandard/Sets the serial standard. Set CUSTOM to adjust
///HIPAR serialStandard/the bit rate and the bandwidth manually.
///HIPAR dataLevel/Sets the edge detection threshold for the hardware
///HIPAR dataLevel/CDR.
///HIPAR algorithm/Sets the software algorithm that is used for clock
///HIPAR algorithm/data recovery.
///HIPAR samplingTime/Sets a sampling time for the clock signal, an
///HIPAR samplingTime/offset for the clock edge in relation to the bit
///HIPAR samplingTime/start. The clock edge sets the beginning of the unit
///HIPAR samplingTime/interval. The sampling time is a number between 0
///HIPAR samplingTime/and 1. Value 0 sets the clock edge to the
///HIPAR samplingTime/beginning of the bit period; value 0.5 sets the clock edge
///HIPAR samplingTime/to the middle of the bit period.
ViStatus _VI_FUNC rsscope_ConfigureHWCDRTrigger(ViSession instrSession,
                                                ViInt32   trigger,
                                                ViInt32   serialStandard,
                                                ViReal64  dataLevel,
                                                ViInt32   algorithm,
                                                ViReal64  samplingTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_TRIGGER_EYE_SERIAL_STANDARD,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_SERIAL_STANDARD, serialStandard),
            3, "Serial Standard");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_LEVEL, dataLevel),
            4, "Data Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_ALGORITHM, algorithm),
            5, "Algorithm");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_SAMPLING_TIME, samplingTime),
            6, "Sampling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the channel signal that is used for clock recovery.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR source/Selects the channel signal that is used for clock
///HIPAR source/recovery.
ViStatus _VI_FUNC rsscope_QueryCDRTriggerEyeSource(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32*  source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_TRIGGER_EYE_SOURCE,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Analyzes the source signal based on the given "Nominal bit
///HINF  rate" and corrects the value.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR nominalBitRate/Sets the frequency of the feed forward CDR. It
///HIPAR nominalBitRate/corresponds to the data rate of the data stream
///HIPAR nominalBitRate/from which the clock is to be recovered.
///HIPAR continuousEstimation/Enables ongoing correction of the bit
///HIPAR continuousEstimation/rate for the hardware CDR.
ViStatus _VI_FUNC rsscope_ConfigureHWCDRTriggerBitRate(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViReal64  nominalBitRate,
                                                       ViBoolean continuousEstimation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BIT_RATE,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BIT_RATE, nominalBitRate),
            3, "Nominal Bit Rate");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_CONTINUOUS_BIT_RATE_ESTIMATE, continuousEstimation),
            4, "Continuous Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Estimates the bite rate once, for the current acquisition.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
ViStatus _VI_FUNC rsscope_EstimateCDRTriggerEyeBitRate(ViSession instrSession,
                                                       ViInt32   trigger)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BIT_RATE_ESTIMATE,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    checkErr(rsscope_SetAttributeViString(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BIT_RATE_ESTIMATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Define the bandwidth settings for software algorithm that is
///HINF  set to Feed Forward.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR bandwidth/Sets the PLL bandwidth. It defines the part of the
///HIPAR bandwidth/spectrum that the PLL can follow during
///HIPAR bandwidth/synchronization. The PLL bandwidth is usually defined by the
///HIPAR bandwidth/transmission standard.
///HIPAR relativeBandwidth/Sets the relative bandwidth, that is the
///HIPAR relativeBandwidth/ratio of the nominal bit rate to the PLL
///HIPAR relativeBandwidth/bandwidth.
ViStatus _VI_FUNC rsscope_ConfigureHWCDRTriggerBandwidth(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViReal64  bandwidth,
                                                         ViInt32   relativeBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BANDWIDTH,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, triggerRepCap, RSSCOPE_ATTR_CDR_TRIGGER_EYE_RELATIVE_BANDWIDTH, relativeBandwidth),
            4, "Relative Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the TV trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR standard/Sets the TV standard.
/// HIPAR mode/Selects the lines or fields on which the instrument can trigger.
/// HIPAR mode/Available modes depend on the scanning system.
/// HIPAR lineNumber/Specifies the line number to trigger on. This control is relevant if
/// HIPAR lineNumber/Mode is set to Line.
/// HIPAR lineField/Sets the field in which the line number is counted. This control is
/// HIPAR lineField/only relevant for NTSC signals
/// HIPAR triggerLevel/Defines the voltage level for the data signal.
/// HIPAR signalPolarity/Sets the polarity of the signal. Note that the sync pulse has the
/// HIPAR signalPolarity/opposite polarity, for example, a positive signal has a negative sync
/// HIPAR signalPolarity/pulse.
ViStatus _VI_FUNC rsscope_ConfigureTVTrigger(ViSession instrSession,
                                             ViInt32   triggerSource,
                                             ViInt32   standard,
                                             ViInt32   mode,
                                             ViInt32   lineNumber,
                                             ViInt32   lineField,
                                             ViReal64  triggerLevel,
                                             ViInt32   signalPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
            2, "Trigger Source");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "TrigA,CH%ld", triggerSource);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_TYPE, RSSCOPE_VAL_TV_TRIGGER));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_STANDARD, standard),
            3, "Standard");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_MODE, mode),
            4, "Mode");

    if (mode == RSSCOPE_VAL_TV_MODE_LINE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_LINE_NUMBER, lineNumber),
                5, "Line Number");
    }

    if (standard == RSSCOPE_VAL_TV_STANDARD_NTSC)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_FIELD, lineField),
                6, "Line Field");
    }

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_LEVEL, triggerLevel),
            7, "Trigger Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_POLARITY, signalPolarity),
            8, "Signal Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the TV trigger custom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseType/Sets the type of the sync pulse.
/// HIPAR scanMode/Sets the scanning system.
/// HIPAR linePeriod/Sets the duration of a line, the time between two successive sync
/// HIPAR linePeriod/pulses.
/// HIPAR pulseWidth/Sets the width of the sync pulse.
ViStatus _VI_FUNC rsscope_ConfigureTVTriggerCustomSettings(ViSession instrSession,
                                                           ViInt32   pulseType,
                                                           ViReal64  linePeriod,
                                                           ViReal64  pulseWidth,
                                                           ViInt32   scanMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_PULSE_TYPE, pulseType),
            2, "Pulse Type");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_LINE_PERIOD, linePeriod),
            3, "Line Period");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_PULSE_WIDTH, pulseWidth),
            4, "Pulse Width");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TV_TRIGGER_SCAN_MODE, scanMode),
            5, "Scan Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The rejection of noise by settting a hysteresis avoids unwanted
/// HIFN  trigger events caused by noise oscillation around the trigger level.
/// HIFN  You can select the hysteresis mode and value for each channel, or
/// HIFN  couple the trigger levels and set the same hysteresis for all channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR hysteresisMode/Selects how the hysteresis is set.
/// HIPAR absoluteHysteresis/Defines a range in absolute values around the trigger level. If the
/// HIPAR absoluteHysteresis/ signal jitters inside this range and crosses the trigger level thereby,
/// HIPAR absoluteHysteresis/ no trigger event occurs.
ViStatus _VI_FUNC rsscope_ConfigureNoiseReject(ViSession instrSession,
                                               ViInt32   trigger,
                                               ViInt32   triggerSource,
                                               ViInt32   hysteresisMode,
                                               ViReal64  absoluteHysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA,";
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    trigger;
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH1, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_COMMON2),
            3, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_MODE,
            "TriggerSource", triggerSource- 1, RS_REPCAP_BUF_SIZE, repCap_c),
            3, "Trigger Source");
    strcat(repCap, repCap_c);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_MODE, hysteresisMode),
            4, "Hysteresis Mode");

    if (hysteresisMode == RSSCOPE_VAL_MANUAL)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS, absoluteHysteresis),
                5, "Absolute Hysteresis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects how the hysteresis is set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR mode/Selects how the hysteresis is set.
/// HIPAR absoluteHysteresis/Defines a range in absolute values around the trigger level. If the
/// HIPAR absoluteHysteresis/signal jitters inside this range and crosses the trigger level thereby,
/// HIPAR absoluteHysteresis/no trigger event occurs. Only relevant if Mode is set to Absolute.
/// HIPAR relativeHysteresis/Defines a range in percent around the trigger level. If the signal
/// HIPAR relativeHysteresis/jitters inside this range and crosses the trigger level thereby, no
/// HIPAR relativeHysteresis/trigger event occurs.
ViStatus _VI_FUNC rsscope_ConfigureNoiseRejectHysteresisScale(
                            ViSession instrSession,
                            ViInt32   triggerSource,
                            ViInt32   mode,
                            ViReal64  absoluteHysteresis,
                            ViReal64  relativeHysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA,";
    ViChar   repCap_source[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_CH1, RSSCOPE_VAL_TRIGGER_SOURCE_REPCAP_COMMON2),
            2, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_SCALE_MODE,
            "TriggerSource", triggerSource- 1, RS_REPCAP_BUF_SIZE, repCap_source),
            2, "Trigger Source");
    strcat(repCap, repCap_source);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_SCALE_MODE, mode),
            3, "Mode");
    switch (mode)
    {
    case RSSCOPE_VAL_ABS:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS, absoluteHysteresis),
                4, "Absolute Hysteresis");
        break;
    case RSSCOPE_VAL_REL:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_RELATIVE, relativeHysteresis),
                5, "Relative Hysteresis");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a range in divisions around the trigger level. If the signal
/// HIFN  oscillates inside this range and crosses the trigger level thereby, no
/// HIFN  trigger event occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR rangeInDivisions/Defines a range in divisions around the trigger level. If the signal
/// HIPAR rangeInDivisions/oscillates inside this range and crosses the trigger level thereby, no
/// HIPAR rangeInDivisions/trigger event occurs.
ViStatus _VI_FUNC rsscope_ConfigureNoiseRejectHysteresisRangeInDivisions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   triggerSource,
                            ViReal64  rangeInDivisions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA,";
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    trigger;
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_COMMON2),
            3, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_RANGE_IN_DIVISIONS,
            "TriggerSource", triggerSource- 1, RS_REPCAP_BUF_SIZE, repCap_c),
            3, "Trigger Source");
    strcat(repCap, repCap_c);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_NOISE_REJECT_HYSTERESIS_RANGE_IN_DIVISIONS, rangeInDivisions),
            4, "Range In Divisions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the trigger levels in the diagrams until you disable
/// HIFN  this option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showTriggerLinesPermanently/Displays the trigger levels in the diagrams until you disable
/// HIPAR showTriggerLinesPermanently/this option.
ViStatus _VI_FUNC rsscope_ConfigureTriggerLinesDisplay(ViSession instrSession,
                                                       ViBoolean showTriggerLinesPermanently)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_LINES_DISPLAY, showTriggerLinesPermanently),
            2, "ShowTrigger Lines Permanently");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the pulse that is provided to the EXT TRIGGER
/// HIFN  OUT connector on the rear panel when a trigger occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enable/Enables/disbales the trigger out signal that is provided to the EXT
/// HIPAR enable/TRIGGER OUT connector on the rear panel when a trigger occurs.
/// HIPAR polarity/Sets the polarity of the trigger out pulse.
/// HIPAR pulseLength/Sets the length of the trigger out pulse.
ViStatus _VI_FUNC rsscope_ConfigureTriggerOutSignal(ViSession instrSession,
                                                    ViBoolean enable,
                                                    ViInt32   polarity,
                                                    ViReal64  pulseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_OUT_STATE, enable),
            2, "Enable");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_OUT_POLARITY, polarity),
            3, "Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_OUT_PULSE_LENGTH, pulseLength),
            4, "Pulse Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the delay of the first pulse edge to the trigger
/// HIFN  point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delay/Returns the delay of the first pulse edge to the trigger point.
ViStatus _VI_FUNC rsscope_QueryTriggerOutDelay(ViSession instrSession,
                                               ViReal64* delay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_OUT_DELAY, delay),
            2, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the event trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR beep/Generates a beep sound if the commd is set to trigger.
/// HIPAR print/Saves a screenshot at each trigger if the commd is set to trigger.
/// HIPAR saveWaveform/Saves the waveform data to file at each trigger if the commd is set
/// HIPAR saveWaveform/to trigger.
ViStatus _VI_FUNC rsscope_ConfigureEventTrigger(ViSession instrSession,
                                                ViInt32   beep,
                                                ViInt32   print,
                                                ViInt32   saveWaveform)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_EVENT_BEEP, beep),
            2, "Beep");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_EVENT_PRINT, print),
            3, "Print");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_EVENT_SAVE_WAVEFORM, saveWaveform),
            4, "Save Waveform");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts an external application if the attribute is set to trigger.
/// HIFN     Use the following attributes to set up the application:
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_NAME
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_PARAMETER
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_WORKING_DIRECTORY
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR runExternalApplication/Starts an external application if the control is set to trigger.
ViStatus _VI_FUNC rsscope_ConfigureEventTriggerRunExternalApplication(
                            ViSession instrSession,
                            ViInt32   runExternalApplication)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_EVENT_RUN_EXTERNAL_APPLICATION, runExternalApplication),
            2, "Run External Application");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sends the trigger out signal to the Trigger Out connector  if
///HINF  the command is set to TRIGger and a trigger occurs.
///HIRET Returns the status code of this operation.
///HIPAR triggerEventTriggerOut/Sends the trigger out signal to the
///HIPAR triggerEventTriggerOut/Trigger Out connector  if the command is
///HIPAR triggerEventTriggerOut/set to TRIGger and a trigger occurs.
ViStatus _VI_FUNC rsscope_ConfigureEventTriggerOut(ViSession instrSession,
                                                   ViInt32   triggerOut)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_EVENT_TRIGGER_OUT, triggerOut),
            2, "Trigger Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the zone trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR expression/Defines the zone trigger.
ViStatus _VI_FUNC rsscope_ConfigureZoneLogicalExpression(ViSession instrSession,
                                                         ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_ZONE_LOGICAL_EXPRESSION, expression),
            3, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the zone trigger condition to the acquisitions in the history
/// HIFN  memory during history replay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Applies the zone trigger condition to the acquisitions in the history
/// HIPAR state/memory during history replay.
ViStatus _VI_FUNC rsscope_ConfigureZoneTriggerInHistoryReplayState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_APPLY_ZONE_TRIGGER_HISTORY, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  By qualifying a trigger event, you can logically combine the trigger
/// HIFN  signal with the state of other digital channel signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerType/Sets the trigger type.
/// HIPAR trigger/Selects trigger event.
/// HIPAR qualificationEnabled/Enables the use of the qualification definition for the selected
/// HIPAR qualificationEnabled/ trigger event.
/// HIPAR channel/Sets the trigger source for the selected trigger type.
/// HIPAR channelEnabled/Selects one of the channels and the specified trigger event to be
/// HIPAR channelEnabled/ considered for qualification. You can select all channel signals except
/// HIPAR channelEnabled/ for the trigger source. In pattern trigger setup, the trigger source
/// HIPAR channelEnabled/ channel is selected by default, and you can select all other channel
/// HIPAR channelEnabled/  signals.
/// HIPAR channelLogic/Defines the logic for the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigureTriggerQualification(ViSession instrSession,
                                                        ViInt32   triggerType,
                                                        ViInt32   trigger,
                                                        ViBoolean qualificationEnabled,
                                                        ViInt32   channel,
                                                        ViBoolean channelEnabled,
                                                        ViInt32   channelLogic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA,";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrChannelEnabledAttr[] = {
             VI_FALSE,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_1,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_2,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_3,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_4
             };
    ViAttr   arrChannelLogicAttr[] = {
             VI_FALSE,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_1,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_2,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_3,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_4
             };

    checkErr(RsCore_LockSession(instrSession));

    if (trigger) // trigger can be NULL
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                3, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
            5, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrChannelEnabledAttr[channel],
            "TriggerType", triggerType, RS_REPCAP_BUF_SIZE, repCap_type),
            2, "Trigger Type");
    strcat(repCap, repCap_type);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, arrChannelEnabledAttr[channel], channelEnabled),
            6, "Channel Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrChannelLogicAttr[channel], channelLogic),
            7, "Channel Logic");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED, qualificationEnabled),
            4, "Qualification Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerType/Sets the trigger type.
/// HIPAR trigger/Selects trigger event.
/// HIPAR qualificationEnabled/Enables the use of the qualification definition for the selected
/// HIPAR qualificationEnabled/  trigger event.
/// HIPAR channelCombination/Sets the trigger source for the selected trigger type.
/// HIPAR multichannelLogic/Defines the logic for the indicated channels.
ViStatus _VI_FUNC rsscope_ConfigureTriggerQualificationMultichannelLogic(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   triggerType,
                            ViBoolean qualificationEnabled,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA,";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   multiChannelLogicAttr[] = {
             VI_FALSE,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_1_2,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_3_4,
             RSSCOPE_ATTR_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_ALL
             };

    checkErr(RsCore_LockSession(instrSession));

    if (trigger) // trigger can be NULL
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                3, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelCombination, RSSCOPE_VAL_CHANNEL1_AND_2, RSSCOPE_VAL_CHANNEL_ALL),
            5, "Channel Combination");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, multiChannelLogicAttr[channelCombination],
            "TriggerType", triggerType, RS_REPCAP_BUF_SIZE, repCap_type),
            2, "Trigger Type");

    strcat(repCap, repCap_type);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, multiChannelLogicAttr[channelCombination], multichannelLogic),
            6, "Multichannel Logic");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_TRIGGER_QUALIFICATION_ENABLED, qualificationEnabled),
            4, "Qualification Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerType/Sets the trigger type.
/// HIPAR trigger/Selects trigger event.
/// HIPAR qualificationEnabled/Enables the use of the qualification definition for the selected
/// HIPAR qualificationEnabled/  trigger event.
/// HIPAR channelCombination/Sets the trigger source for the selected trigger type.
/// HIPAR multichannelLogic/Defines the logic for the indicated channels.
ViStatus _VI_FUNC rsscope_ConfigureTriggerQualificationMulitchannelLogic(
                            ViSession instrSession,
                            ViInt32   triggerType,
                            ViInt32   trigger,
                            ViBoolean qualificationEnabled,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_ConfigureTriggerQualificationMultichannelLogic(instrSession,
        triggerType, trigger, qualificationEnabled, channelCombination, multichannelLogic));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of the sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequenceMode/Selects the type of the sequence.
ViStatus _VI_FUNC rsscope_ConfigureTriggerSequenceMode(ViSession instrSession,
                                                       ViInt32   sequenceMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TRIGGER_SEQUENCE_MODE, sequenceMode),
            2, "Sequence Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The "A only" sequence contains an A-event and the holdoff setting as
/// HIFN  optional condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR holdoffMode/Selects the method to define the holdoff condition. The trigger
/// HIPAR holdoffMode/ holdoff defines when the next trigger after the current will be
/// HIPAR holdoffMode/ recognized. Thus, it affects the next trigger to occur after the
/// HIPAR holdoffMode/ current one. Holdoff helps to obtain stable triggering when the
/// HIPAR holdoffMode/ oscilloscope is triggering on undesired events.
/// HIPAR holdoffTime/Defines the holdoff time period. The next trigger occurs only after
/// HIPAR holdoffTime/ this time has passed.
ViStatus _VI_FUNC rsscope_ConfigureAOnlyTriggerSequence(ViSession instrSession,
                                                        ViInt32   holdoffMode,
                                                        ViReal64  holdoffTime,
                                                        ViInt32   holdoffEvents,
                                                        ViReal64  randomMinTime,
                                                        ViReal64  randomMaxTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_MODE, RSSCOPE_VAL_SEQ_AONLY));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_MODE, holdoffMode),
            2, "Holdoff Mode");

    switch (holdoffMode)
    {
    case RSSCOPE_VAL_HOLDOFF_TIME:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_TIME, holdoffTime),
                3, "Holdoff Time");
        break;

    case RSSCOPE_VAL_HOLDOFF_EVENTS:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_EVENTS, holdoffEvents),
                4, "Holdoff Events");
        break;

    case RSSCOPE_VAL_HOLDOFF_RANDOM:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_RANDOM_MINIMUM_TIME, randomMinTime),
                5, "Random Min Time");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_RANDOM_MAXIMUM_TIME, randomMaxTime),
                6, "Random Max Time");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the resulting holdoff time if the holdoff mode is set to AUTO: Auto time = Auto time scaling * Horizontal scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_QueryAOnlyTriggerSequenceAutoTimeResult(
                            ViSession instrSession,
                            ViReal64* autoTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_AUTO_TIME_RESULT, autoTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the auto time scaling factor the horizontal scale is multipied with: Auto time =
/// HIFN  Auto time scaling * Horizontal scale. The setting is relevant if the holdoff mode is set to AUTO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_TriggerSequenceHoldoffAutoScale(ViSession instrSession,
                                                          ViReal64  autoScale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_HOLDOFF_AUTO_SCALE, autoScale),
            2, "Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The complex trigger sequence "A -> B -> R" consists of two trigger
/// HIFN  events - A and B - and an optional reset condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ConfigureABRTriggerSequence(ViSession instrSession,
                                                      ViBoolean eventCoupling,
                                                      ViReal64  sequenceDelay,
                                                      ViInt32   bEventsCount,
                                                      ViBoolean resetByTimeoutEnabled,
                                                      ViReal64  resetTimeout,
                                                      ViBoolean resetByEventEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_MODE, RSSCOPE_VAL_SEQ_ABR));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "TrigA", RSSCOPE_ATTR_TRIGGER_EVENT_COUPLING, eventCoupling),
            2, "Event Coupling");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_DELAY, sequenceDelay),
            3, "Sequence Delay");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_B_EVENTS_COUNT, bEventsCount),
            4, "B-Events Count");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_RESET_BY_TIMEOUT_ENABLED, resetByTimeoutEnabled),
            5, "Reset By Timeout Enabled");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_RESET_TIMEOUT, resetTimeout),
            6, "Reset Timeout");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_TRIGGER_SEQUENCE_RESET_EVENT_ENABLED, resetByEventEnabled),
            7, "Reset By Event Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects, if you want to trigger on a single event, or on a
///HINF  series of events.
///HIRET Returns the status code of this operation.
///HIPAR trigger/Selects trigger event.
///HIPAR sequenceEvent/Selects, if you want to trigger on a single
///HIPAR sequenceEvent/event, or on a series of events.
ViStatus _VI_FUNC rsscope_ConfigureTriggerSequenceEvent(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   sequenceEvent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   triggerRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRIGGER_SEQUENCE_EVENT,
        "Trigger", trigger - 1, RS_REPCAP_BUF_SIZE, triggerRepCap), 2 ,"Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, triggerRepCap, RSSCOPE_ATTR_TRIGGER_SEQUENCE_EVENT, sequenceEvent),
            3, "Sequence Event");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches on or off the update of all display elements during remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayInRemote/This control enables or disables instrument display when in remote control mode.
ViStatus _VI_FUNC rsscope_ConfigureDisplayUpdate(ViSession instrSession,
                                                 ViBoolean displayInRemote)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DISPLAY_UPDATE, displayInRemote),
            2, "Display In Remote");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables d disables the display of  additon text in
/// HIFN  remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Hdle returned by the Initiize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Enables d disables the display of  additon text in remote
/// HIPAR state/control.
/// HIPAR text/Defines  addition text that is displayed during remote control
/// HIPAR text/operation.
ViStatus _VI_FUNC rsscope_ConfigureDisplayMessage(ViSession instrSession,
                                                  ViBoolean state,
                                                  ViString  text)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DISPLAY_MESSAGE_ENABLED, state),
            2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DISPLAY_MESSAGE_TEXT, text),
                3, "Text");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the transparency of the area that is defined as measurement or
/// HIFN  search gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transparency/Sets the transparency of the area that is defined as measurement or
/// HIPAR transparency/search gate.
ViStatus _VI_FUNC rsscope_ConfigureDisplayGateTransparency(ViSession instrSession,
                                                           ViInt32   transparency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DISPLAY_GATE_TRANSPARENCY, transparency),
            2, "Transparency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all measurement results including long term measurement and
/// HIFN  statistics, all waveforms, and the history.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ConfigureDisplayClearResults(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_DISPLAY_CLEAR_RESULTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If an external monitor is connected to the instrument, enables the
/// HIFN  display of result boxes on the external monitor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If an external monitor is connected to the instrument, enables the
/// HIPAR enabled/display of result boxes on the external monitor.
ViStatus _VI_FUNC rsscope_ConfigureDisplayResultsOnExternalMonitorEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_DISPLAY_RESULTS_ON_EXTERNAL_MONITOR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If an external monitor is connected to the instrument, enables the
/// HIFN  display of dialog boxes on the external monitor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If an external monitor is connected to the instrument, enables the
/// HIPAR enabled/display of dialog boxes on the external monitor.
ViStatus _VI_FUNC rsscope_ConfigureDisplayDialogsOnExternalMonitorEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_DISPLAY_DIALOGS_ON_EXTERNAL_MONITOR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures persistence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enable/If enabled, each new data point in the diagram area remains on the
/// HIPAR enable/screen for the defined duration. If disabled, the signal value is only
/// HIPAR enable/displayed as long as it actually occurs.
/// HIPAR infinite/If persistence is enabled, each new data point in the diagram area
/// HIPAR infinite/remains on the screen infinitely until is disabled.
/// HIPAR time/If persistence is enabled, each new data point in the diagram area
/// HIPAR time/remains on the screen for the duration defined here.
ViStatus _VI_FUNC rsscope_ConfigurePersistence(ViSession instrSession,
                                               ViBoolean enable,
                                               ViBoolean infinite,
                                               ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PERSISTENCE_STATE, enable),
            2, "Enable");

    if (enable == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_INFINITE_PERSISTENCE, infinite),
                3, "Infinite");
        if (infinite == VI_FALSE)
        {
            viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_PERSISTENCE_TIME, time),
                    4, "Time");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the display, removing persistent values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ResetPersistence(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PERSISTENCE_RESET, NULL),
            1, "Session");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures signal intensity and style.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR intensity/This value determines the strength of the waveform line in the
/// HIPAR intensity/diagram. Enter a percentage between 0 (not visible) and 100% (very
/// HIPAR intensity/strong).
/// HIPAR style/Select the style in which the waveform is displayed.
ViStatus _VI_FUNC rsscope_ConfigureSignalStyle(ViSession instrSession,
                                               ViInt32   intensity,
                                               ViInt32   style)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SIGNAL_STYLE_INTENSITY, intensity),
            2, "Intensity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SIGNAL_STYLE, style),
            3, "Style");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is deprecated, and will be deleted in a future update.
/// HIFN  Use rsscope_ConfigureWaveformColor instead. Note that the values of the
/// HIFN  second parameter are not the same in rsscope_ConfigureWaveformColor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Waveform to be assigned.
/// HIPAR useColorTable/If enabled, the selected waveform is displayed according to its
/// HIPAR useColorTable/assigned color table. If this option is disabled, the default color
/// HIPAR useColorTable/table is used, i.e. the intensity of the specific signal color varies
/// HIPAR useColorTable/according to the cumulative occurance of the values.
/// HIPAR tableName/Color table name to be assigned to the signal.
ViStatus _VI_FUNC rsscope_ConfigureColorTableAssignment(ViSession instrSession,
                                                        ViInt32   source,
                                                        ViBoolean useColorTable,
                                                        ViString  tableName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_NONE, RSSCOPE_VAL_WAV_TRK8),
            2, "Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USE_COLOR_TABLE,
            "WfmSuffix", source, RS_REPCAP_BUF_SIZE, repCap),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_USE_COLOR_TABLE, useColorTable),
            3, "Use Color Table");

    if (useColorTable == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ASSIGN_COLOR_TABLE, tableName),
                4, "Table Name");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the waveform colors to suit your preferences.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Selects the waveform to be configured.
/// HIPAR useColorTable/If enabled, the selected waveform is displayed according to its
/// HIPAR useColorTable/assigned color table. If this option is disabled, the default color
/// HIPAR useColorTable/table is used, i.e. the intensity of the specific signal color varies
/// HIPAR useColorTable/according to the cumulative occurance of the values.
/// HIPAR tableName/Color table name to be assigned to the signal.
/// HIPAR signalColor/Decimal value of the ARGB color. Use the color dialog box on the
/// HIPAR signalColor/instrument to get the hex value of the color, and convert the hex value
/// HIPAR signalColor/to a decimal value.
ViStatus _VI_FUNC rsscope_ConfigureWaveformColor(ViSession instrSession,
                                                 ViInt32   waveform,
                                                 ViBoolean useColorTable,
                                                 ViString  tableName,
                                                 ViInt64   signalColor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveform, RSSCOPE_REPCAP_WFMSUFFIX_WFMC1W1),
                2, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USE_COLOR_TABLE,
            "WfmSuffix", waveform- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Waveform");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_USE_COLOR_TABLE, useColorTable),
            3, "Use Color Table");

    if (useColorTable)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ASSIGN_COLOR_TABLE, tableName),
                4, "Table Name");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                        RSSCOPE_ATTR_SIGNAL_COLOR, signalColor),
                    5, "Signal Color");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds a new color table with the specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the new table.
ViStatus _VI_FUNC rsscope_AddColorTable(ViSession instrSession,
                                        ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_ADD_COLOR_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes a existing color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
ViStatus _VI_FUNC rsscope_RemoveColorTable(ViSession instrSession,
                                           ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REMOVE_COLOR_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function appends a new row at the end of the color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
ViStatus _VI_FUNC rsscope_ColorTableAppendRow(ViSession instrSession,
                                              ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_COLOR_TABLE_APPEND_NEW_ROW, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inserts the entry at the specified index in the color
/// HIFN  table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
/// HIPAR index/Row number in the color table.
ViStatus _VI_FUNC rsscope_ColorTableInsertRow(ViSession instrSession,
                                              ViString  tableName,
                                              ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:COL:PAL:POIN:INS '%s',%ld", tableName, index);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inserts a new entry in the specified color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
/// HIPAR index/Row number of the new entry in the color table.
/// HIPAR cumulativePercentage/Cumulative occurance value
/// HIPAR color/ARGB value of the color to be used for the table entry.
ViStatus _VI_FUNC rsscope_ColorTableDefineRow(ViSession instrSession,
                                              ViString  tableName,
                                              ViInt32   index,
                                              ViReal64  cumulativePercentage,
                                              ViInt64   color)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:COL:PAL:POIN:VAL '%s',%ld,%.12lG,%lld", tableName,
             index,
             cumulativePercentage,
             color);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes the entry with the specified index from the
/// HIFN  color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
/// HIPAR index/Row number in the color table.
ViStatus _VI_FUNC rsscope_ColorTableRemoveRow(ViSession instrSession,
                                              ViString  tableName,
                                              ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:COL:PAL:POIN:REM '%s',%ld", tableName, index);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of configured color maps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/The number of configured color maps.
ViStatus _VI_FUNC rsscope_QueryColorTableCount(ViSession instrSession,
                                               ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_COLOR_TABLE_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
/// HIPAR count/Rows count in the color table.
ViStatus _VI_FUNC rsscope_QueryColorTableRowCount(ViSession instrSession,
                                                  ViString  tableName,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:COL:PAL:POIN:COUN? '%s'", tableName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function query a entry from the specified color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Name of the existing color table.
/// HIPAR index/Row number of the entry in the color table.
/// HIPAR cumulativePercentage/Cumulative occurance value
/// HIPAR color/ARGB value of the color to be used for the table entry.
ViStatus _VI_FUNC rsscope_QueryColorTableRow(ViSession instrSession,
                                             ViString  tableName,
                                             ViInt32   index,
                                             ViReal64* cumulativePercentage,
                                             ViInt32*  color)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64 aux;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:COL:PAL:POIN:VAL? '%s',%ld", tableName, index);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, cumulativePercentage, &aux, NULL, NULL));
    *color = (ViInt32)aux;

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the basic diagram layout and the appearence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showGrid/If enabled, a grid is displayed in the diagram area.
/// HIPAR showCrosshair/If selected, a crosshair is displayed in the diagram area.
/// HIPAR showLabels/If enabled, labels mark values on the x and y-axes in specified
/// HIPAR showLabels/intervals in the diagram.
/// HIPAR showTabsAlways/If enabled, the tab titles of all diagrams are displayed.
/// HIPAR yGridFixed/If enabled, the horizontal grid lines remain in their position when
/// HIPAR yGridFixed/the position of the curve is changed. Only the values at the grid lines
/// HIPAR yGridFixed/are adapted. This reflects the behavior of traditional oscilloscopes.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDiagram(ViSession instrSession,
                                                      ViBoolean showGrid,
                                                      ViBoolean showCrosshair,
                                                      ViBoolean showLabels,
                                                      ViBoolean showTabsAlways,
                                                      ViBoolean yGridFixed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_SHOW_GRID, showGrid),
            2, "Show Grid");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_SHOW_CROSSHAIR, showCrosshair),
            3, "Show Crosshair");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_SHOW_LABELS, showLabels),
            4, "Show Labels");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_SHOW_TAB_TITLES, showTabsAlways),
            5, "Show Tabs Always");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_KEEP_Y_GRID_FIXED, yGridFixed),
            6, "Y Grid Fixed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the vertical grid lines remain in their position when the
/// HIFN  horizontal position is changed. Only the values at the grid lines are
/// HIFN  adapted.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_DIAGRAM_KEEP_X_GRID_FIXED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:DIAGram:XFIXed
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xGridFixed/If enabled, the vertical grid lines remain in their position when the
/// HIPAR xGridFixed/horizontal position is changed. Only the values at the grid lines are
/// HIPAR xGridFixed/adapted.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementXGridFixed(ViSession instrSession,
                                                         ViBoolean xGridFixed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_KEEP_X_GRID_FIXED, xGridFixed),
            2, "X-Grid Fixed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures fine grid scaling.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_DIAGRAM_SHOW_FINE_GRID_SCALE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:DIAGram:FINegrid
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showFineGridScale/If On, the crosshair is displayed as a ruler with scale markers. If
/// HIPAR showFineGridScale/Off, the crosshair is shown as dashed lines.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDiagramFineGridScale(
                            ViSession instrSession,
                            ViBoolean showFineGridScale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_SHOW_FINE_GRID_SCALE, showFineGridScale),
            2, "showFineGridScale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures appearence of signal bar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enable/If enabled, the signal bar is displayed in the diagram area.
/// HIPAR position/Selects vertical position of the signal bar.
ViStatus _VI_FUNC rsscope_ShowSignalBar(ViSession instrSession,
                                        ViBoolean enable,
                                        ViInt32   position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_STATE, enable),
            2, "Enable");

    if (enable)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_POSITION, position),
                3, "Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures behavior of signal bar auto-hiding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoHide/If enabled, the signal bar disappears automatically after specified
/// HIPAR autoHide/time.
/// HIPAR hideHead/If enabled, the "Auto hide" function hides also the horizontal and
/// HIPAR hideHead/trigger label at the top of the signal bar.
/// HIPAR hideAfter/Sets the time when the signal bar is faded out if auto hide is
/// HIPAR hideAfter/enabled.
/// HIPAR hidingOpacity/Sets the opacity, or transparency, of the signal bar. The minimum
/// HIPAR hidingOpacity/value is 30% for the least visibility of the signal bar, the maximum
/// HIPAR hidingOpacity/value is 80%.
ViStatus _VI_FUNC rsscope_ConfigureSignalBarAutoHide(ViSession instrSession,
                                                     ViBoolean autoHide,
                                                     ViBoolean hideHead,
                                                     ViReal64  hideAfter,
                                                     ViReal64  hidingTransparency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_AUTO_HIDE, autoHide),
            2, "Auto Hide");
    if (autoHide)
    {
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_HIDE_HEAD, hideHead),
                3, "Hide Head");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_HIDE_TIME, hideAfter),
                4, "Hide After");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_HIDING_TRANSPARENCY, hidingTransparency),
                5, "Hiding Transparency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures color settings of signal bar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR borderColor/Sets the ARGB color value of the signal bar border.
/// HIPAR fillColor/Sets the ARGB fill color value of the signal bar.
ViStatus _VI_FUNC rsscope_ConfigureSignalBarColor(ViSession instrSession,
                                                  ViInt32   borderColor,
                                                  ViInt32   fillColor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_BORDER_COLOR, borderColor),
            2, "Border Color");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SIGNAL_BAR_FILL_COLOR, fillColor),
            3, "Fill Color");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines where a new result box opens.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Defines where a new result box opens.
ViStatus _VI_FUNC rsscope_ConfigureResultBoxPosition(ViSession instrSession,
                                                     ViInt32   position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_RESULT_BOX_POSITION, position),
            2, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a new diagram with a waveform on the screen, in
/// HIFN  relation to an existing diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR existingDiagram/Name of the existing diagram
/// HIPAR newDiagramPosition/Position of the new diagram in relation to the existing one.
/// HIPAR insertBefore/If on, the new diagram is inserted to the left (for HORizontal), above
/// HIPAR insertBefore/(for VERTical) or in a tab before the existing one.
/// HIPAR source/Waveform to be diplayed in the new diagram.
/// HIPAR newDiagramName/Name of the new diagram.
ViStatus _VI_FUNC rsscope_AddDiagram(ViSession instrSession,
                                     ViString  existingDiagram,
                                     ViInt32   newDiagramPosition,
                                     ViBoolean insertBefore,
                                     ViInt32   source,
                                     ViString  newDiagramName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2pos;
    ViChar*  p2source;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngDiagramType, newDiagramPosition, &p2pos),
                3, "New Diagram Position");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                5, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD '%s',%s,%s,%s,'%s'", existingDiagram,
             p2pos,
             insertBefore ? "ON" : "OFF",
             p2source,
             newDiagramName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the existing diagram
ViStatus _VI_FUNC rsscope_DisplayDiagram(ViSession instrSession,
                                         ViString  diagramName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DISPLAY_DIAGRAM, diagramName),
            2, "Diagram Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the existing diagram
ViStatus _VI_FUNC rsscope_RemoveDiagram(ViSession instrSession,
                                        ViString  diagramName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REMOVE_DIAGRAM, diagramName),
            2, "Diagram Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Changes the diagram name, which is displayed on the diagram tab.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/The current name of the diagram to be changed.
/// HIPAR newName/The new name to be given to the diagram to be changed.
ViStatus _VI_FUNC rsscope_RenameDiagram(ViSession instrSession,
                                        ViString  diagramName,
                                        ViString  newName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:DIAG:REN '%s','%s'", diagramName, newName);

    checkErr(RsCore_WriteWithOpc(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the specified waveform in the selected diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the new diagram.
/// HIPAR source/Waveform to be displayed in the diagram.
ViStatus _VI_FUNC rsscope_ShowWaveformInDiagram(ViSession instrSession,
                                                ViString  diagramName,
                                                ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2source;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:SIGN:ASS '%s',%s", diagramName, p2source);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates an XY-diagram by adding a second waveform to a
/// HIFN  diagram with a channel, math or reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram where the waveform is added.
/// HIPAR source/Waveform to be added.
/// HIPAR assignToXAxis/If On, the added waveform is assigned to the x-axis.
ViStatus _VI_FUNC rsscope_AddWaveformToDiagram(ViSession instrSession,
                                               ViString  diagramName,
                                               ViInt32   source,
                                               ViBoolean assignToXAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameterXY, source, &p2string),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:SIGN:AXIS '%s',%s,%s", diagramName, p2string,
             assignToXAxis ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes the specified waveform from the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the waveform.
ViStatus _VI_FUNC rsscope_RemoveWaveformFromDiagram(ViSession instrSession,
                                                    ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    // No range checking required; LAYout:SIGNal:UNASsign accepts all possible waveform
    // parameters, including NONE

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_REMOVE_WAVEFORM_FROM_DIAGRAM, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new waveform label for the specified source
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR labelText/String with the label text that is shown on the display.
/// HIPAR positionMode/Selects the position mode.
/// HIPAR xPosition/Horizontal position of the label text. Values, range and unit depend
/// HIPAR xPosition/on the position mode, the waveform position and scaling.
/// HIPAR yPosition/Vertical position of the label text. Values, range and unit depend on
/// HIPAR yPosition/the position mode, the waveform position and scaling. For relative
/// HIPAR yPosition/position mode, the range is 0 to 100 %.
ViStatus _VI_FUNC rsscope_AddLabel(ViSession instrSession,
                                   ViString  labelID,
                                   ViInt32   source,
                                   ViString  labelText,
                                   ViInt32   positionMode,
                                   ViReal64  xPosition,
                                   ViReal64  yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, positionMode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            5, "Position Mode");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2string),
                5, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:ADD '%s',%s,'%s',%s,%lf,%lf",
             labelID, p2string, labelText, modeArr[positionMode], xPosition, yPosition);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the specifies waveform label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
ViStatus _VI_FUNC rsscope_RemoveLabel(ViSession instrSession,
                                      ViString  labelID,
                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2string),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:REM '%s',%s", labelID, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Returns a list of label IDs for the specified source. Labels
///HINF  that are defined in the user interface get an automatic label
///HINF  'DiagramText<No>'.
///HIRET Returns the status code of this operation.
///HIPAR source/Waveform to which the label belongs.
///HIPAR bufferSize/Defines the size of buffer passed to 'Label IDs'.
///HIPAR labelIDs/Returns the symbolic label (or translation) of the destination
///HIPAR labelIDs/address of the specified word, if the label list is enabled.
ViStatus _VI_FUNC rsscope_ListLabelIDs(ViSession instrSession,
                                       ViInt32   source,
                                       ViInt32   bufferSize,
                                       ViChar    labelIDs[],
                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2string),  2, "Source");
    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSSCOPE_ATTR_HIDDEN_LABEL_ID_LIST, p2string, bufferSize, labelIDs, returnedValues), 4, "Label IDs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function modifies the text of the specified label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR labelText/String with the label text that is shown on the display.
ViStatus _VI_FUNC rsscope_EditLabelText(ViSession instrSession,
                                        ViString  labelID,
                                        ViInt32   source,
                                        ViString  labelText)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2string),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:TEXT '%s',%s,'%s'", labelID, p2string, labelText);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the text of the specified label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR labelText/Returns the text of the specified label.
ViStatus _VI_FUNC rsscope_QueryLabelText(ViSession instrSession,
                                         ViString  labelID,
                                         ViInt32   source,
                                         ViInt32   bufferSize,
                                         ViChar    labelText[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2data = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2source;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:TEXT? '%s',%s", labelID, p2source);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &p2data));
    RsCore_CopyToUserBufferAsciiData(labelText, bufferSize, p2data);

Error:
    if (p2data) free(p2data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source for the label display and enables it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelShow/Selects the source for the label display and enables it.
ViStatus _VI_FUNC rsscope_ConfigureLabelShow(ViSession instrSession,
                                             ViInt32   source,
                                             ViBoolean labelShow)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_LABEL_SOURCE, source),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_LABEL_SHOW, labelShow),
            3, "Label Show");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures label position mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR positionMode/Selects the position mode.
ViStatus _VI_FUNC rsscope_ConfigureLabelPositionMode(ViSession instrSession,
                                                     ViInt32   source,
                                                     ViInt32   positionMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2source;
    ViChar*  p2AbsRel;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                2, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                2, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngAbsRel, positionMode, &p2AbsRel),
                3, "Position Mode");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:POSM %s,%s", p2source, p2AbsRel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures label absolute position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR xPosition/Horizontal position of the label text. Values, range and unit depend
/// HIPAR xPosition/on the position mode, the waveform position and scaling.
/// HIPAR yPosition/Vertical position of the label text. Values, range and unit depend on
/// HIPAR yPosition/the position mode, the waveform position and scaling.
ViStatus _VI_FUNC rsscope_ConfigureLabelAbsolutePosition(ViSession instrSession,
                                                         ViString  labelID,
                                                         ViInt32   source,
                                                         ViReal64  xPosition,
                                                         ViReal64  yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2source;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:HOR:ABS:POS '%s',%s,%lf", labelID, p2source,
             xPosition);
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:VERT:ABS:POS '%s',%s,%lf", labelID, p2source,
             yPosition);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures label relative position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelID/String with the label identifier. The <LabelID> and <Source>
/// HIPAR source/Waveform to that the label belongs.
/// HIPAR xPosition/Horizontal position of the label text. Values, range and unit depend
/// HIPAR xPosition/on the position mode, the waveform position and scaling.
/// HIPAR yPosition/Vertical position of the label text. Values, range and unit depend on
/// HIPAR yPosition/the position mode, the waveform position and scaling.
ViStatus _VI_FUNC rsscope_ConfigureLabelRelativePosition(ViSession instrSession,
                                                         ViString  labelID,
                                                         ViInt32   source,
                                                         ViReal64  xPosition,
                                                         ViReal64  yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN];
    ViChar*  p2source;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_WAV_C1W1),
                3, "Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2source),
                3, "Source");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xPosition, 0.0, 100.0), 4, "X Position");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, yPosition, 0.0, 100.0), 5, "Y Position");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:HOR:REL:POS '%s',%s,%lf", labelID, p2source,
             xPosition);
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "DISP:SIGN:LAB:VERT:REL:POS '%s',%s,%lf", labelID, p2source,
             yPosition);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the font size of waveform labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fontSize/Sets the font size of waveform labels.
ViStatus _VI_FUNC rsscope_ConfigureLabelFontSize(ViSession instrSession,
                                                 ViInt32   fontSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_LABEL_FONT_SIZE, fontSize),
            2, "Font Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Displays or hides a single label that is identified by the
///HINF  signal source and label ID.
///HIRET Returns the status code of this operation.
///HIPAR labelID/String with the label identifier. The <LabelID> and
///HIPAR labelID/<Source>
///HIPAR source/Waveform to which the label belongs.
///HIPAR state/Shows or hides a single label.
ViStatus _VI_FUNC rsscope_WaveformLabelState(ViSession instrSession,
                                             ViString  labelID,
                                             ViInt32   source,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   parameters[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, source, &p2string),  3, "Source");

    snprintf (parameters, RS_MAX_MESSAGE_BUF_SIZE, "'%s',%s,%s", labelID, p2string, (state == VI_TRUE) ? "ON" : "OFF");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_HIDDEN_WAVEFORM_LABEL_STATE, parameters),
            2, "Label ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a new zoom diagram based on the specified waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of diagram to be zoomed
/// HIPAR newDiagramPosition/Position of the new diagram in relation to the existing one.
/// HIPAR insertBefore/If on, the new diagram is inserted to the left (for HORizontal), above
/// HIPAR insertBefore/(for VERTical) or in a tab before the existing one.
/// HIPAR newZoomDiagramName/Defines the name of the new zoom diagram.
/// HIPAR xStart/Defines the x-value at the beginning of the zoom area.
/// HIPAR xStop/Defines the x-value at the end of the zoom area.
/// HIPAR yStart/Defines the y-value at the beginning of the zoom area.
/// HIPAR yStop/Defines the y-value at the end of the zoom area.
ViStatus _VI_FUNC rsscope_AddDiagramZoom(ViSession instrSession,
                                         ViString  diagramName,
                                         ViInt32   newDiagramPosition,
                                         ViBoolean insertBefore,
                                         ViString  newZoomDiagramName,
                                         ViReal64  xStart,
                                         ViReal64  xStop,
                                         ViReal64  yStart,
                                         ViReal64  yStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2pos;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngDiagramType, newDiagramPosition, &p2pos),
                3, "New Diagram Position");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:ADD '%s',%s,%s,%.12lG,%.12lG,%.12lG,%.12lG,'%s'", diagramName,
             p2pos,
             insertBefore ? "ON" : "OFF",
             xStart,
             xStop,
             yStart,
             yStop,
             newZoomDiagramName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new zoom diagram based on the settings of an
/// HIFN  existing zoom area for the same source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Defines the name of the zoom diagram to be copied.
/// HIPAR newZoomDiagramName/Defines the name of the new zoom diagram.
/// HIPAR xOffset/Defines an offset to the existing zoom area in x direction.
/// HIPAR yOffset/Defines an offset to the existing zoom area in y direction.
ViStatus _VI_FUNC rsscope_AddDiagramZoomCoupled(ViSession instrSession,
                                                ViString  diagramName,
                                                ViString  newZoomDiagramName,
                                                ViReal64  xOffset,
                                                ViReal64  yOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:ADDC '%s',%.12lG,%.12lG,'%s'", diagramName,
             xOffset,
             yOffset,
             newZoomDiagramName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes the specified zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
ViStatus _VI_FUNC rsscope_RemoveDiagramZoom(ViSession instrSession,
                                            ViString  diagramName,
                                            ViString  zoomDiagramName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:REM '%s','%s'", diagramName, zoomDiagramName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows all zooms of a diagram in one zoom window. The
/// HIFN  zoomed areas are overlaid for better comparison of the zoomed
/// HIFN  waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zoomOverlay/Shows all zooms of a diagram in one zoom window.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomOverlay(ViSession instrSession,
                                                      ViBoolean zoomOverlay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_DIAGRAM_ZOOM_OVERLAY, zoomOverlay),
            2, "Zoom Overlay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical settings of zoom area using
/// HIFN  start and stop values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
/// HIPAR mode/Defines whether absolute or relative values are used to specify the
/// HIPAR mode/y-axis values. Since the zoom area refers to the active signal,
/// HIPAR mode/relative values ensure that the zoom area remains the same.
/// HIPAR start/Defines the lower limit of the zoom area on the y-axis.
/// HIPAR stop/Defines the upper limit of the zoom area on the y-axis.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomVerticalStartStop(
                            ViSession instrSession,
                            ViString  diagramName,
                            ViString  zoomDiagramName,
                            ViInt32   mode,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:MODE '%s','%s',%s", diagramName,
             zoomDiagramName,
             modeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:%s:STAR '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             start);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:%s:STOP '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             stop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal settings of zoom area using
/// HIFN  start and stop values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
/// HIPAR mode/Defines whether absolute or relative values are used to specify the
/// HIPAR mode/x-axis values. Since the zoom area refers to the active signal,
/// HIPAR mode/relative values ensure that the zoom area remains the same.
/// HIPAR start/Defines the lower limit of the zoom area on the x-axis.
/// HIPAR stop/Defines the upper limit of the zoom area on the x-axis.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomHorizontalStartStop(
                            ViSession instrSession,
                            ViString  diagramName,
                            ViString  zoomDiagramName,
                            ViInt32   mode,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:MODE '%s','%s',%s", diagramName,
             zoomDiagramName,
             modeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:%s:STAR '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             start);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:%s:STOP '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             stop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical settings of zoom area using
/// HIFN  position and range values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
/// HIPAR mode/Defines whether absolute or relative values are used to specify the
/// HIPAR mode/y-axis values. Since the zoom area refers to the active signal,
/// HIPAR mode/relative values ensure that the zoom area remains the same.
/// HIPAR position/Defines the y-value of the centerpoint of the zoom area.
/// HIPAR range/Defines the height of the zoom area.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomVerticalPositionRange(
                            ViSession instrSession,
                            ViString  diagramName,
                            ViString  zoomDiagramName,
                            ViInt32   mode,
                            ViReal64  position,
                            ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:MODE '%s','%s',%s", diagramName,
             zoomDiagramName,
             modeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:%s:POS '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             position);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:VERT:%s:SPAN '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             range);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal settings of zoom area using
/// HIFN  position and range values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
/// HIPAR mode/Defines whether absolute or relative values are used to specify the
/// HIPAR mode/y-axis values. Since the zoom area refers to the active signal,
/// HIPAR mode/relative values ensure that the zoom area remains the same.
/// HIPAR position/Defines the x-value of the centerpoint of the zoom area.
/// HIPAR range/Defines the width of the zoom area.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomHorizontalPositionRange(
                            ViSession instrSession,
                            ViString  diagramName,
                            ViString  zoomDiagramName,
                            ViInt32   mode,
                            ViReal64  position,
                            ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:MODE '%s','%s',%s", diagramName,
             zoomDiagramName,
             modeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:%s:POS '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             position);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:HORZ:%s:SPAN '%s','%s',%.12lG", modeArr[mode],
             diagramName,
             zoomDiagramName,
             range);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the position coupling of coupled
/// HIFN  zooms. If position coupling is enabled and one zoom area is moved, the
/// HIFN  other coupled zoom areas are moved, too, and keep their distance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the diagram on which the zoom area is based.
/// HIPAR zoomDiagramName/Name of the zoom diagram
/// HIPAR positionCoupling/Enables or disables the position coupling of coupled zooms.
ViStatus _VI_FUNC rsscope_ConfigureDiagramZoomPositionCoupling(
                            ViSession instrSession,
                            ViString  diagramName,
                            ViString  zoomDiagramName,
                            ViBoolean positionCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ZOOM:POSC '%s','%s',%s", diagramName, zoomDiagramName,
             positionCoupling ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the XY diagram settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xyDiagramNumber/Sets the XY Diagram number.
/// HIPAR xyDiagramState/Activates an XY-waveform.
/// HIPAR constantXYRatio/If enabled, the x- and y-axes maintain a constant ratio in the
/// HIPAR constantXYRatio/diagram.
/// HIPAR xSource/Defines the signal source that supplies the x-values of the
/// HIPAR xSource/XY-diagram.
/// HIPAR ySource/Defines the signal source that supplies the y-values of the
/// HIPAR ySource/XY-diagram.
ViStatus _VI_FUNC rsscope_ConfigureXYDiagram(ViSession instrSession,
                                             ViInt32   xyDiagramNumber,
                                             ViBoolean xyDiagramState,
                                             ViBoolean constantXYRatio,
                                             ViInt32   xSource,
                                             ViInt32   ySource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xyDiagramNumber, RSSCOPE_REPCAP_XYDIAGRAM_XY1, RSSCOPE_REPCAP_XYDIAGRAM_XY4),
            2, "XY Diagram Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_XY_DIAGRAM_STATE,
            "XYDiagram", xyDiagramNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "XY Diagram Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_XY_DIAGRAM_STATE, xyDiagramState),
            3, "XY Diagram State");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_XY_DIAGRAM_RATIO, constantXYRatio),
            4, "XY Ratio");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_XY_DIAGRAM_X_SOURCE, xSource),
            5, "X Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_XY_DIAGRAM_Y_SOURCE, ySource),
            6, "Y Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces the source of the x-axis with the source of the
/// HIFN  y-axis and vice versa.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xyDiagramNumber/Sets the XY Diagram number.
ViStatus _VI_FUNC rsscope_SwapXYDiagram(ViSession instrSession,
                                        ViInt32   xyDiagramNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SWAP_XY_DIAGRAM,
            "XYDiagram", xyDiagramNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "XYDiagram");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SWAP_XY_DIAGRAM, NULL),
            2, "XY Diagram Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the history display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR enableHistory/Enables or disables the history display.
ViStatus _VI_FUNC rsscope_ConfigureHistoryState(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   waveform,
                                                ViBoolean enableHistory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_STATE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HISTORY_STATE, enableHistory),
            4, "Enable History");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures start and stop point of history viewer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR start/Sets the start point for the history replay.
/// HIPAR stop/Sets the end point for the history viewer.
ViStatus _VI_FUNC rsscope_ConfigureHistoryStartStop(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   waveform,
                                                    ViInt32   start,
                                                    ViInt32   stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_START_INDEX,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_START_INDEX,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTORY_START_INDEX, start),
            4, "Start");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTORY_STOP_INDEX, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a particular acquisition in the memory to
/// HIFN  display it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR acquisition/Select the acquisition number in the memory.
ViStatus _VI_FUNC rsscope_ConfigureHistoryAcquisitionView(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   waveform,
                                                          ViInt32   acquisition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_ACQUISITION_VIEW,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_ACQUISITION_VIEW,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTORY_ACQUISITION_VIEW, acquisition),
            4, "Acquisition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the display time for one acquisition. The
/// HIFN  shorter the time, the faster is the replay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR timeperAcquisition/Sets the display time for one acquisition.
ViStatus _VI_FUNC rsscope_ConfigureHistoryAcquisitionTime(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   waveform,
                                                          ViReal64  timeperAcquisition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_ACQUISITION_TIME,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_ACQUISITION_TIME,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTORY_ACQUISITION_TIME, timeperAcquisition),
            4, "Timeper Acquisition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures if the replay of the history waveform
/// HIFN  sequence repeats automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR autoRepeat/If ON, the replay of the history waveform sequence repeats
/// HIPAR autoRepeat/automatically. Otherwise, the replay stops at the stop index set with
/// HIPAR autoRepeat/rsscope_ConfigureHistoryStartStop
ViStatus _VI_FUNC rsscope_ConfigureHistoryAutoRepeat(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   waveform,
                                                     ViBoolean autoRepeat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_AUTO_REPEAT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_AUTO_REPEAT,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HISTORY_AUTO_REPEAT, autoRepeat),
            4, "Auto Repeat");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts and stops the replay of the history waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
ViStatus _VI_FUNC rsscope_HistoryPlaybackControl(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   waveform)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_PLAYBACK,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_PLAYBACK,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_HISTORY_PLAYBACK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the time stamp of the currently displayed
/// HIFN  history acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR mode/This control selects absolute or relative timestamp:
/// HIPAR timeStamp/Returns the time of the currently displayed history acquisition.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the time
/// HIPAR bufferSize/stamp value.
ViStatus _VI_FUNC rsscope_QueryHistoryTimeStamp(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   waveform,
                                                ViInt32   mode,
                                                ViInt32   bufferSize,
                                                ViChar    timeStamp[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrAttr[] = {
             RSSCOPE_ATTR_HISTORY_ABSOLUTE_TIME_STAMP,
             RSSCOPE_ATTR_HISTORY_RELATIVE_TIME_STAMP
             };
             waveform;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            4, "Mode");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrAttr[mode],
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    strcat(repCap, ",W0");
    checkErr(rsscope_GetAttributeViString(instrSession, repCap,
        arrAttr[mode],
        bufferSize,
        timeStamp));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the relative time of the currently selected acquisition and
/// HIFN  the internal reference time (horizontal alignment) in history view with
/// HIFN  respect to the acquisition with index 0.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_HISTORY_TIME_REFERENCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CHANnel<m>[:WAVeform<n>]:HISTory:TSRReference?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR timeReference/Returns the relative time of the currently selected acquisition and
/// HIPAR timeReference/the internal reference time (horizontal alignment) in history view with
/// HIPAR timeReference/respect to the acquisition with index 0.
ViStatus _VI_FUNC rsscope_QueryHistoryTimeReference(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   waveform,
                                                    ViReal64* timeReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_TIME_REFERENCE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_TIME_REFERENCE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_HISTORY_TIME_REFERENCE, timeReference),
            4, "Time Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the date of the current acquisition that is shown in the history viewer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/This control selects the waveform.
/// HIPAR date/Returns the date of the current acquisition that is shown in the history viewer
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the time
/// HIPAR bufferSize/stamp value.
ViStatus _VI_FUNC rsscope_QueryHistoryDate(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   waveform,
                                           ViInt32   bufferSize,
                                           ViChar    date[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_wav[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveform, 1, 3),
            3, "Waveform");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_DATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTORY_DATE,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_wav),
            3, "Waveform");

    strcat(repCap, ",");
    strcat(repCap, repCap_wav);

    checkErr(rsscope_GetAttributeViString(instrSession, repCap,
        RSSCOPE_ATTR_HISTORY_DATE,
        bufferSize,
        date));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of acquisitions currently saved in
/// HIFN  the memory. This number of acquisitions is available for history
/// HIFN  viewing. It is also the number of acquisitions in an Ultra Segmentation
/// HIFN  acquisition series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control is reserved.
/// HIPAR count/Returns the number of acquisitions currently saved in the memory.
ViStatus _VI_FUNC rsscope_QueryHistoryAcquisitionsCount(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HISTORY_GET_ACQUISITIONS_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR measurementState/Switches the indicated measurement on or off.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementState(ViSession instrSession,
                                                    ViInt32   measurement,
                                                    ViBoolean measurementState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MEASUREMENT_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap_measurement, RSSCOPE_ATTR_MEASUREMENT_ENABLED, measurementState),
            3, "Measurement State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR signalSource/Selects the source of the measurement.
/// HIPAR signalSource2/Selects the source of the measurement.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementSource(ViSession instrSession,
                                                     ViInt32   measurement,
                                                     ViInt32   signalSource,
                                                     ViInt32   signalSource2)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  maxMeas = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSSCOPE_REPCAP_MEASUREMENT_MEAS1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEFINE_FIRST_MEASUREMENT_SOURCE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, signalSource, RSSCOPE_VAL_WAV_C1W1),
                3, "Signal Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEFINE_FIRST_MEASUREMENT_SOURCE, signalSource),
            3, "Signal Source");

    if (signalSource2 != RSSCOPE_VAL_WAV_NONE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEFINE_SECOND_MEASUREMENT_SOURCE, signalSource2),
                4, "Signal Source 2");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR signalSource/Selects the source of the measurement.
/// HIPAR signalSource2/Selects the source of the measurement.
ViStatus _VI_FUNC rsscope_ConfigureMeasSource(ViSession instrSession,
                                              ViInt32   measurement,
                                              ViInt32   signalSource,
                                              ViInt32   signalSource2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2source1;
    ViChar*  p2source2;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSSCOPE_REPCAP_MEASUREMENT_MEAS1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, signalSource, RSSCOPE_VAL_WAV_C1W1),
                3, "Signal Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, signalSource, &p2source1),
                3, "Signal Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngWaveformParameter, signalSource2, &p2source2),
                4, "Signal Source 2");

    if ((((signalSource >= RSSCOPE_VAL_WAV_D0) && (signalSource <= RSSCOPE_VAL_WAV_D15)) ||
            ((signalSource2 >= RSSCOPE_VAL_WAV_D0) && (signalSource2 <= RSSCOPE_VAL_WAV_D15))) &&
        !(RsCore_HasInstrumentOptions(instrSession, "B1")))
    {
        checkErr(RS_ERROR_INSTRUMENT_OPTION);
    }

    //checkErr(RsCoreInt_ClearBeforeRead(instrSession));
    checkErr(RsCore_ClearStatus(instrSession));

    if (signalSource2 == RSSCOPE_VAL_WAV_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:SOUR %s", measurement, p2source1);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:SOUR %s,%s", measurement, p2source1, p2source2);
    }

    checkErr(RsCore_WriteWithOpc(instrSession, cmd, 0));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the type of the limit check that can run together with the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR limitCheck/Selects the type of the limit check that can run together with the measurement.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementLimitCheck(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MEASUREMENT_LIMIT_TYPE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_MEASUREMENT_LIMIT_TYPE, limitCheck),
            3, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the multiple measurement. The measurement is performed repeatedly if the measured parameter occurs several times inside the defined gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR multipleMeasurement/Switches the multiple measurement on or off.
ViStatus _VI_FUNC rsscope_ConfigureMultipleMeasurement(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViBoolean multipleMeasurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MULTIPLE_MEASUREMENT_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap_measurement, RSSCOPE_ATTR_MULTIPLE_MEASUREMENT_ENABLED, multipleMeasurement),
            3, "Multiple Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the maximum number of measurements per acquisition if multiple measurement is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions
/// HIPAR instrSession/function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR measurementCount/Sets the maximum number of measurements per acquisition if multiple measurement is enabled.
ViStatus _VI_FUNC rsscope_ConfigureMultipleMeasurementNumber(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MULTIPLE_MEASUREMENT_COUNT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_MULTIPLE_MEASUREMENT_COUNT, measurementCount),
            3, "Measurement Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the measurement category.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR category/Selects the measurement category.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementCategory(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViInt32   category)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MEASUREMENT_CATEGORY,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_MEASUREMENT_CATEGORY, category),
            3, "Category");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables all additional measurements in all categories of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsscope_EnableAdditionalMeasurements(ViSession instrSession,
                                                       ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADDITIONAL_MEASUREMENTS_ENABLE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap_measurement, RSSCOPE_ATTR_ADDITIONAL_MEASUREMENTS_ENABLE, NULL),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function disables all additional measurements in all categories of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsscope_DisableAdditionalMeasurements(ViSession instrSession,
                                                        ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADDITIONAL_MEASUREMENTS_DISABLE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap_measurement, RSSCOPE_ATTR_ADDITIONAL_MEASUREMENTS_DISABLE, NULL),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the statistic results of the indicated mesurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsscope_ClearMeasurementStatisticData(ViSession instrSession,
                                                        ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CLEAR_MEASUREMENT_STATISTIC_DATA,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap_measurement, RSSCOPE_ATTR_CLEAR_MEASUREMENT_STATISTIC_DATA, NULL),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures what happens when the mask test has failed or when it has passed successfully. Each action can be initiated either on failure or on success.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR event/This Control selects event to be performed.
/// HIPAR action/This Control selects action for specified event.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementEventActions(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   event,
                                                           ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_measurement[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_GENERATE_BEEP_ON_MEASUREMENT_EVENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_measurement),
            2, "Measurement");
    switch (event)
    {
    case RSSCOPE_VAL_MEASUREMENT_EVENT_BEEP:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_GENERATE_BEEP_ON_MEASUREMENT_EVENT, action),
                3, "Event: Beep");
        break;

    case RSSCOPE_VAL_MEASUREMENT_EVENT_STOP:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_STOP_ACQUISITION_ON_MEASUREMENT_EVENT, action),
                3, "Event: Stop");
        break;

    case RSSCOPE_VAL_MEASUREMENT_EVENT_PRINT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_PRINT_ON_MEASUREMENT_EVENT, action),
                3, "Event: Print");
        break;

    case RSSCOPE_VAL_MEASUREMENT_EVENT_SAVE_WFM:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_SAVE_WAVEFORM_DATA_ON_MEASUREMENT_EVENT, action),
                3, "Event: Save Waveform");
        break;
    case RSSCOPE_VAL_MEASUREMENT_EVENT_TRIGGEROUT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_CREATE_TRIGGER_OUT_PULSE, action),
                3, "Event: Trigger Out Pulse");
        break;
    case RSSCOPE_VAL_MEASUREMENT_EVENT_SAVE_REPORT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_SAVE_REPORT_ON_MEASUREMENT_EVENT, action),
                3, "Event: Save Report");
        break;
    case RSSCOPE_VAL_MEASUREMENT_EVENT_RUN_EXE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_measurement, RSSCOPE_ATTR_START_EXTERNAL_APPLICATION_ON_MEASUREMENT_EVENT, action),
                3, "Event: Start External Application");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, event), 3, "Event");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the measurement display intermediate results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR displayResultLines/If enabled, the intermediate result lines required to obtain the
/// HIPAR displayResultLines/measurement result (e.g. signal thresholds) are displayed in the
/// HIPAR displayResultLines/measurement diagram.
/// HIPAR displayReferenceLevels/If enabled, the reference levels used for the measurement are
/// HIPAR displayReferenceLevels/displayed in the diagram.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDisplayIntermediateResults(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean displayResultLines,
                            ViBoolean displayReferenceLevels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DISPLAY_RESULT_LINES,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DISPLAY_RESULT_LINES, displayResultLines),
            3, "Display Result Lines");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DISPLAY_REFERENCE_LEVELS, displayReferenceLevels),
            4, "Display Reference Levels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the style in which the measurement waveform
/// HIFN  is displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR displayStyle/Selects the style in which the measurement waveform is displayed
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDisplayStyle(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   displayStyle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MEASUREMENT_DISPLAY_STYLE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MEASUREMENT_DISPLAY_STYLE, displayStyle),
            3, "Display Style");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the position of the measurement results on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Defines the position of the measurement results on the display.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDisplayResultTablePosition(
                            ViSession instrSession,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_DISPLAY_RESULT_TABLE_DEFAULT_POSITION, position),
            3, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If ON, all measurement results are shown in one result box. If
/// HIFN  disabled, the results of each measurement group are shown in a separate
/// HIFN  result box. The default position is ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR groupResultDialogs/If ON, all measurement results are shown in one result box. If
/// HIPAR groupResultDialogs/disabled, the results of each measurement group are shown in a separate
/// HIPAR groupResultDialogs/result box. The default position is ignored.
ViStatus _VI_FUNC rsscope_ConfigureMeasurementDisplayGroupResultDialogs(
                            ViSession instrSession,
                            ViBoolean groupResultDialogs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_DISPLAY_GROUP_RESULT_DIALOGS, groupResultDialogs),
            3, "Group Result Dialogs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the results of all selected measurements in all categories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR arraySize/Sets the number of results requested.
/// HIPAR results/Returns the results of all selected measurements in all categories.
/// HIPAR actualPoints/Number of points actually returned in the Results array.
ViStatus _VI_FUNC rsscope_ReadMeasurementResults(ViSession instrSession,
                                                 ViInt32   measurement,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ARES?", measurement);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, actualPoints));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the cursor settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR cursorState/Switches the indicated cursor on or off.
/// HIPAR source/Defines the source of the cursor measurement.
/// HIPAR type/Defines the cursor measurement type.
ViStatus _VI_FUNC rsscope_ConfigureCursor(ViSession instrSession,
                                          ViInt32   cursor,
                                          ViBoolean cursorState,
                                          ViInt32   source,
                                          ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if ((source >= RSSCOPE_VAL_CURSOR_D0) && (source <= RSSCOPE_VAL_CURSOR_MSOB4) && !(RsCore_HasInstrumentOptions(
        instrSession, "B1")))
    {
        checkErr(RS_ERROR_INSTRUMENT_OPTION);
    }
    if ((source >= RSSCOPE_VAL_CURSOR_TRK1) && (source <= RSSCOPE_VAL_CURSOR_TRK8) && !(RsCore_HasInstrumentOptions(
        instrSession, "K5")))
    {
        checkErr(RS_ERROR_INSTRUMENT_OPTION);
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_ENABLED,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CURSOR_ENABLED, cursorState),
            3, "Cursor State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_SOURCE, source),
            4, "Cursor Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_MEASUREMENT_TYPE, type),
            5, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the second cursor source.
/// HIFN     If enabled, the second corsor lines Cx.2 measure on the second source.
/// HIFN  Using a second source, you can measure differences between two channels
/// HIFN  with cursors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR enabled/Enables the second cursor source.
/// HIPAR secondSource/Selects the second cursor source.
ViStatus _VI_FUNC rsscope_ConfigureCursorSecondSource(ViSession instrSession,
                                                      ViInt32   cursor,
                                                      ViBoolean enabled,
                                                      ViInt32   secondSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cursor, RSSCOPE_VAL_CURSOR_1, RSSCOPE_VAL_CURSOR_4),
            2, "Cursor");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_SOURCE_2_ENABLED,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_SOURCE_2, secondSource),
            4, "Second Source");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CURSOR_SOURCE_2_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches all cursors off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsscope_AllCursorsOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_ALL_CURSORS_OFF, NULL),
            2, "All Cursors Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows the enabled cursor measurements in all active diagrams of the
/// HIFN  same domain (time/ spectrum). All cursor measurements are affected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showInAllDiagrams/Shows the enabled cursor measurements in all active diagrams of the
/// HIPAR showInAllDiagrams/same domain (time/ spectrum). All cursor measurements are affected.
ViStatus _VI_FUNC rsscope_ConfigureCursorShowInAllDiagrams(ViSession instrSession,
                                                           ViBoolean showInAllDiagrams)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_CURSOR_SHOW_IN_ALL_DIAGRAMS, showInAllDiagrams),
            2, "Show In All Diagrams");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the position of the cursor measurement results on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tablePosition/Defines the position of the cursor measurement results on the screen.
ViStatus _VI_FUNC rsscope_ConfigureCursorResultTablePosition(ViSession instrSession,
                                                             ViInt32   tablePosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_CURSOR_RESULT_TABLE_POSITION, tablePosition),
            2, "Table Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal and vertical position of selected cursor or cursor line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR xPosition1/Defines the position of the left vertical cursor line.
/// HIPAR xPosition2/Defines the position of the right vertical cursor line.
/// HIPAR yPosition1/Defines the position of the lower horizontal cursor line.
/// HIPAR yPosition2/Defines the position of the upper horizontal cursor line.
ViStatus _VI_FUNC rsscope_ConfigureCursorPosition(ViSession instrSession,
                                                  ViInt32   cursor,
                                                  ViReal64  xPosition1,
                                                  ViReal64  xPosition2,
                                                  ViReal64  yPosition1,
                                                  ViReal64  yPosition2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cursor[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_X_POSITION,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap_cursor),
            2, "Cursor");
    strcpy(repCap, repCap_cursor);
    strcat(repCap, ",P1");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_X_POSITION, xPosition1),
            3, "X Position 1");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_Y_POSITION, yPosition1),
            5, "Y Position 1");

    strcpy(repCap, repCap_cursor);
    strcat(repCap, ",P2");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_X_POSITION, xPosition2),
            4, "X Position 2");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_Y_POSITION, yPosition2),
            6, "Y Position 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the positioning mode of the horizontal and vertical cursors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR xCoupling/Defines the positioning mode of the vertical cursor.
/// HIPAR yCoupling/Defines the positioning mode of the horizontal cursor.
ViStatus _VI_FUNC rsscope_ConfigureCursorCoupling(ViSession instrSession,
                                                  ViInt32   cursor,
                                                  ViBoolean xCoupling,
                                                  ViBoolean yCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_X_POSITION_COUPLING,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CURSOR_X_POSITION_COUPLING, xCoupling),
            3, "X Coupling");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CURSOR_Y_POSITION_COUPLING, yCoupling),
            4, "Y Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the cursor waveform tracking
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR tracking/Switches the cursor waveform tracking.
ViStatus _VI_FUNC rsscope_ConfigureCursorWaveformTracking(ViSession instrSession,
                                                          ViInt32   cursor,
                                                          ViBoolean tracking)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_TRACKING,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CURSOR_TRACKING, tracking),
            3, "Cursor Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the first and second horizontal cursor position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR firstCursorPosition/Defines whether the first horizontal cursor is positioned to the
/// HIPAR secondCursorPosition/Defines whether the second horizontal cursor is positioned to the
ViStatus _VI_FUNC rsscope_ConfigureCursorEnvelopeWaveformSelection(
                            ViSession instrSession,
                            ViInt32   cursor,
                            ViInt32   firstCursorPosition,
                            ViInt32   secondCursorPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_ENVELOPE_CURVE_1,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_ENVELOPE_CURVE_1, firstCursorPosition),
            3, "First Cursor Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_ENVELOPE_CURVE_2, secondCursorPosition),
            4, "Second Cursor Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the delta value (distance) of two horizontal or vertical cursors (RTM) / cursor lines (RTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR cursorDistance/Returns the delta value (distance) of two horizontal or vertical cursors (RTM) / cursor lines (RTO).
/// HIPAR direction/Selects the distance direction
ViStatus _VI_FUNC rsscope_QueryCursorDistance(ViSession instrSession,
                                              ViInt32   cursor,
                                              ViInt32   direction,
                                              ViReal64* cursorDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_CURSOR_HORIZONTAL_DISTANCE,
             RSSCOPE_ATTR_CURSOR_VERTICAL_DISTANCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cursor, RSSCOPE_VAL_CURSOR_1, RSSCOPE_VAL_CURSOR_4),
            2, "Cursor");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSSCOPE_VAL_CURSOR_HORIZONTAL, RSSCOPE_VAL_CURSOR_VERTICAL),
            3, "Direction");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_HORIZONTAL_DISTANCE,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, attr[direction], cursorDistance),
            4, "Cursor Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the inverse value of the delta value (distance) of the two vertical or horizontal cursors (RTM) / cursor lines (RTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR cursorDistance/Returns the inverse value of the delta value (distance) of the two vertical or horizontal cursors (RTM) / cursor lines (RTO).
ViStatus _VI_FUNC rsscope_QueryCursorInverseDistance(ViSession instrSession,
                                                     ViInt32   cursor,
                                                     ViReal64* cursorDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_INVERSE_DISTANCE,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_INVERSE_DISTANCE, cursorDistance),
            3, "Cursor Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the inverse value of the voltage difference - the reciprocal
/// HIFN  of the vertical distance of two horizontal cursor lines: 1/delta_V.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR horizontalDeltaSlope/Returns the inverse value of the voltage difference - the reciprocal
/// HIPAR horizontalDeltaSlope/of the vertical distance of two horizontal cursor lines: 1/delta_V.
ViStatus _VI_FUNC rsscope_QueryCursorHorizontalDeltaSlope(ViSession instrSession,
                                                          ViInt32   cursor,
                                                          ViReal64* horizontalDeltaSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_HORIZONTAL_DELTA_SLOPE,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_HORIZONTAL_DELTA_SLOPE, horizontalDeltaSlope),
            3, "Horizontal Delta Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets cursor to selected peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR search/Sets cursor to selected peak.
ViStatus _VI_FUNC rsscope_CursorSearch(ViSession instrSession,
                                       ViInt32   cursor,
                                       ViInt32   search)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrCursorSearchAttr[] = {
             RSSCOPE_ATTR_CURSOR_SEARCH_PEAK,
             RSSCOPE_ATTR_CURSOR_SEARCH_NEXT_LEFT,
             RSSCOPE_ATTR_CURSOR_SEARCH_NEXT_RIGHT,
             RSSCOPE_ATTR_CURSOR_SEARCH_NEXT_ABSOLUTE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, search, 0, 3),
            3, "Search");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrCursorSearchAttr[search],
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, arrCursorSearchAttr[search], NULL),
            1, "Session");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the minimum level by which the waveform must rise or fall so that it will be identified as a maximum or a minimum by the search functions.
///
///           Attribute(s):
///           RSSCOPE_ATTR_CURSOR_PEAK_EXCURSION
///
///           Remote-control command(s):
///           CURSor<Cursor>:PEXCursion
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR peakExcursion/Defines the minimum level by which the waveform must rise or fall so that it will be identified as a maximum or a minimum by the search functions.
ViStatus _VI_FUNC rsscope_ConfigureCursorPeakExcursion(ViSession instrSession,
                                                       ViInt32   cursor,
                                                       ViInt32   peakExcursion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cursor, RSSCOPE_VAL_CURSOR_1, RSSCOPE_VAL_CURSOR_4),
            2, "Cursor");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_PEAK_EXCURSION,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CURSOR_PEAK_EXCURSION, peakExcursion),
            3, "Peak Excursion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines  absolute threshold as  addition condition
/// HIFN  for the peak search. Only peaks that exceed the threshold are detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR peakExcursion/Defines  absolute threshold as  addition condition for the peak
/// HIPAR peakExcursion/search. Only peaks that exceed the threshold are detected.
ViStatus _VI_FUNC rsscope_ConfigureCursorThreshold(ViSession instrSession,
                                                   ViInt32   cursor,
                                                   ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cursor, RSSCOPE_VAL_CURSOR_1, RSSCOPE_VAL_CURSOR_4),
            2, "Cursor");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_THRESHOLD,
            "Cursor", cursor- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cursor");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CURSOR_THRESHOLD, threshold),
            3, "threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines a label to be displayed with the vertical (X) or
///HINF  horizontal (Y) cursors.
///HIRET Returns the status code of this operation.
///HIPAR x1Label/Defines a label to be displayed with the vertical
///HIPAR x1Label/cursors. X1 is the left cursor line. By default, the
///HIPAR x1Label/cursors are labeled as Cu1.X1, Cu2.X1, Cu3.X1, Cu4.X1.
///HIPAR x2Label/Defines a label to be displayed with the vertical
///HIPAR x2Label/cursors. X2 is the right cursor line. By default, the
///HIPAR x2Label/cursors are labeled as Cu1.X2, Cu2.X2, Cu3.X2, Cu4.X2.
///HIPAR y1Label/Defines a label to be displayed with the horizontal
///HIPAR y1Label/cursors. Y1 is the lower cursor line. By default, the
///HIPAR y1Label/cursors are labeled as Cu1.Y1, Cu2.Y1, Cu3.Y1, Cu4.Y1.
///HIPAR y2Label/Defines a label to be displayed with the horizontal
///HIPAR y2Label/cursors. Y2 is the upper cursor line. By default, the
///HIPAR y2Label/cursors are labeled as Cu1.Y2, Cu2.Y2, Cu3.Y2, Cu4.Y2.
///HIPAR cursor/This control selects the cursor.
ViStatus _VI_FUNC rsscope_ConfigureCursorLabel(ViSession instrSession,
                                               ViInt32   cursor,
                                               ViString  x1Label,
                                               ViString  x2Label,
                                               ViString  y1Label,
                                               ViString  y2Label)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cursorRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  cursorCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Cursor", &cursorCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cursor, 1, cursorCount),
            2, "Cursor");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CURSOR_VERTICAL_LABEL_LEFT,
        "Cursor", cursor - 1, RS_REPCAP_BUF_SIZE, cursorRepCap), 2 ,"Cursor");

    viCheckParm(rsscope_SetAttributeViString(instrSession, cursorRepCap, RSSCOPE_ATTR_CURSOR_VERTICAL_LABEL_LEFT, x1Label),
            3, "X1 Label");

    viCheckParm(rsscope_SetAttributeViString(instrSession, cursorRepCap, RSSCOPE_ATTR_CURSOR_VERTICAL_LABEL_RIGHT, x2Label),
            4, "X2 Label");

    viCheckParm(rsscope_SetAttributeViString(instrSession, cursorRepCap, RSSCOPE_ATTR_CURSOR_HORIZONTAL_LABEL_LOWER, y1Label),
            5, "Y1 Label");

    viCheckParm(rsscope_SetAttributeViString(instrSession, cursorRepCap, RSSCOPE_ATTR_CURSOR_HORIZONTAL_LABEL_UPPER, y2Label),
            6, "Y2 Label");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to
/// HIFN  if the measurement waveform is used as a source for math calculations.
/// HIFN  The main measurement type must belong to the same category as the other
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement. This measurement is the one
/// HIPAR measurementFunction/referred to if the measurement waveform is used as a source for math
/// HIPAR measurementFunction/calculations. The main measurement type must belong to the same
/// HIPAR measurementFunction/category as the other types assigned to the same measurement waveform,
/// HIPAR measurementFunction/if there are any.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeMainMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_MAIN_MEASUREMENT, measurementFunction),
            3, "Amplitude Time Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines additional measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects additional measurement type.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngAmplitudeTimeMeasurementType, measurementFunction, &
                    p2string),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ADD %s,%s", measurement, p2string, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the amplitude time measurement signal
/// HIFN  threashold and area level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR signalThreshold/Defines the value above which measurement results are displayed.
/// HIPAR signalThreshold/Values beneath the threshold are considered to be noise and are
/// HIPAR signalThreshold/ignored.
/// HIPAR areaLevel/Defines the reference level used to integrate the waveform.
/// HIPAR pulseTrainCount/Sets the number of positive pulses for the pulse train
/// HIPAR pulseTrainCount/measurement. It measures the duration of N positive pulses from
/// HIPAR pulseTrainCount/the rising edge of the first pulse to the falling edge of the
/// HIPAR pulseTrainCount/N-th pulse.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeSettings(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   signalThreshold,
                                                         ViReal64  areaLevel,
                                                         ViInt32   pulseTrainCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_SIGNAL_DETECTION_THRESHOLD,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_SIGNAL_DETECTION_THRESHOLD, signalThreshold),
            3, "signalThreshold");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPT_AREA_LEVEL, areaLevel),
            4, "areaLevel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_PULSE_TRAIN_COUNT, pulseTrainCount),
            5, "Pulse Train Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the first slope of the pulses or the edges to be counted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR countType/Switches between Pulse, Edge or Clock count.
/// HIPAR slope/This control sets the first slope of the pulses ot the edges to be counted.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeSlope(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViInt32   countType,
                                                      ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViAttr   arrCountType[] = {
             RSSCOPE_ATTR_AMPLITUDE_PULSES_SLOPE,
             RSSCOPE_ATTR_AMPLITUDE_EDGES_SLOPE,
             RSSCOPE_ATTR_AMPLITUDE_CLOCK_SLOPE,
             RSSCOPE_ATTR_AMPLITUDE_MEASURED_SLOPE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, countType, 0, 3),
            3, "Count Type");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrCountType[countType],
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrCountType[countType], slope),
            4, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the specific settings for delay measurement to allow you to measure
/// HIFN the time between any two slopes at any reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR sourceNumber/Selects the source number.
/// HIPAR levelSelection/Selects the reference level on which the time is measured for each source.
/// HIPAR slope/This control sets the edge of each source, between which the delay is measured.
/// HIPAR direction/Selects the direction for counting slopes for each source.
/// HIPAR edgeNumber/Sets the number of the edge that is relevant for delay measurement for each source.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeAdvancedDelaySetup(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   sourceNumber,
                            ViInt32   levelSelection,
                            ViInt32   slope,
                            ViInt32   direction,
                            ViInt32   edgeNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_source[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_DELAY_LEVEL_SELECTION,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_DELAY_LEVEL_SELECTION,
            "SourceNumber", sourceNumber- 1, RS_REPCAP_BUF_SIZE, repCap_source),
            3, "Source Number");
    strcat(repCap, ",");
    strcat(repCap, repCap_source);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_DELAY_LEVEL_SELECTION, levelSelection),
            4, "Level Selection");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_DELAY_SLOPE, slope),
            5, "Slope");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_DELAY_DIRECTION, direction),
            6, "Direction");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_DELAY_EDGE_NUMBER, edgeNumber),
            7, "Edge Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the reference level on which the time is measured for each source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR type/Switches between Clock and Data refernce level.
/// HIPAR referenceLevel/Selects the reference level on which the time is measured for each source.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeReferenceLevel(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   type,
                            ViInt32   referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrRefLevelType[] = {
             RSSCOPE_ATTR_AMPLITUDE_TIME_CLOCK_REFERENCE_LEVEL,
             RSSCOPE_ATTR_AMPLITUDE_TIME_DATA_REFERENCE_LEVEL
             };
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_AMPT_REF_TYPE_CLOCK, RSSCOPE_VAL_AMPT_REF_TYPE_DATA),
            3, "Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrRefLevelType[type],
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrRefLevelType[type], referenceLevel),
            4, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables limit checking for amplitude vs. time measurements
/// HIFN  in the specified measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Amplitude/time measurement type.
/// HIPAR limitCheck/Enables or disables limit checking for amplitude vs. time measurements
/// HIPAR limitCheck/in the specified measurement channel.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeLimitCheck(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementType,
                                                           ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_HIGH, RSSCOPE_VAL_MEAS_SLEF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_LIMIT_CHECK,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPT_LIMIT_CHECK,
            "AmptMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPT_LIMIT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper limit for limit checking, respectively.
/// HIFN  MEASurement<Measurement>:AMPTime:LCHeck<AmptMeasurementType>:<LowUpp>:LIMit
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Amplitude/time measurement type.
/// HIPAR lowerLimit/Define the lower limit for limit checking.
/// HIPAR upperLimit/Define the upper limit for limit checking.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeLimitCheckLimits(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_HIGH, RSSCOPE_VAL_MEAS_SLEF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_VALUE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_VALUE,
            "AmptMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_VALUE, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_UPPER_LIMIT_VALUE, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper margins for the limit check, respectively.
/// HIFN  Margins are not as strict as limits and must be within the valid value
/// HIFN  range.
/// HIFN  MEASurement<Measurement>:AMPTime:LCHeck<AmptMeasurementType>:<LowUpp>:MARGin
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Amplitude/time measurement type.
/// HIPAR lowerMargin/Define the lower margin for the limit check. Margins are not as strict
/// HIPAR lowerMargin/as limits and must be within the valid value range.
/// HIPAR upperMargin/Define the upper margins for the limit check. Margins are not as
/// HIPAR upperMargin/strict as limits and must be within the valid value range.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeLimitCheckMargins(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViReal64  lowerMargin,
                            ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_HIGH, RSSCOPE_VAL_MEAS_SLEF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_MARGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_MARGIN,
            "AmptMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_LOWER_LIMIT_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_UPPER_LIMIT_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the envelope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR envelope/Envelope selection.
ViStatus _VI_FUNC rsscope_ConfigureEnvelopeMeasurement(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViInt32   envelope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ENVELOPE_SELECTION,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ENVELOPE_SELECTION, envelope),
            3, "Envelope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the first slope of the pulses or the edges to be
/// HIFN counted. The setting is only relevant for pulse count or edge
/// HIFN count measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR slope/Sets the edge direction to be used for delay measurement:
/// HIPAR slope/positive, negative, or either edge.
/// HIPAR referenceLevel/Selects the reference level of the measurement source on which
/// HIPAR referenceLevel/the delay is measured: upper, middle, or lower level.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeDelayToTrigger(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   slope,
                            ViInt32   referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_DELAY_TO_TRIGGER_SLOPE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_DELAY_TO_TRIGGER_SLOPE, slope),
            3, "Slope");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_DELAY_TO_TRIGGER_REFERENCE_LEVEL, referenceLevel),
            4, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the type of amplitude arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR measurementType/Sets the type of amplitude arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticParameterMeasurementType(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   measurementType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_TYPE, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_TYPE, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_TYPE, measurementType),
            4, "Measurement Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the amplitude/time arithmetics measurements for the measurment
/// HIFN  group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR operand/Selects the amplitude/time arithmetics measurements for the measurment
/// HIPAR operand/group.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticParameterOperand(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   operand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_AMPTIME_TYPE, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_AMPTIME_TYPE, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_AMPTIME_TYPE, operand),
            4, "Operand");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the measurement group for the second operand of the
/// HIFN  amplitude/time arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR measurementGroup/Sets the measurement group for the second operand of the
/// HIPAR measurementGroup/amplitude/time arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticParameterGroup(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   measurementGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_GROUP, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_GROUP, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_GROUP, measurementGroup),
            4, "Measurement Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the spectrum arithmetics measurements for the measurment
/// HIFN  group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR spectrum/Selects the spectrum arithmetics measurements for the measurment
/// HIPAR spectrum/group.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticParameterMeasurementSpectrum(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   spectrum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, spectrum),
            4, "Spectrum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the operation type to be performed on the selected operands
/// HIFN  for the amplitude/time arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR operator/Selects the operation type to be performed on the selected operands
/// HIPAR operator/for the amplitude/time arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticOperator(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   arithmeticOperator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_OPERATOR,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_OPERATOR, arithmeticOperator),
            3, "Operator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the numeric value and constant unit as the second operand
/// HIFN  of the amplitude/time arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR constant/Sets a constant numeric value as the second operand of the
/// HIPAR constant/amplitude/time arithmetic measurement.
/// HIPAR constantUnit/Selects the operation type to be performed on the selected operands
/// HIPAR constantUnit/for the amplitude/time arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticConstantSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  constant,
                            ViInt32   constantUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CONSTANT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CONSTANT, constant),
            3, "Constant");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CONSTANT_UNIT, constantUnit),
            4, "Constant Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures settings for the user-defined custom name for the
/// HIFN  amplitude/time arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR state/Enables the use of user-defined custom name for the amplitude/time
/// HIPAR state/arithmetic measurement.
/// HIPAR name/reserved for future use
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticCustomNameSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean state,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CUSTOM_NAME_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CUSTOM_NAME_ENABLED, state),
            3, "State");

    name;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures settings for the user-defined value of the unit for the
/// HIFN  amplitude/time arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR state/Enables the use of user-defined custom unit for the amplitude/time
/// HIPAR state/arithmetic measurement.
/// HIPAR unit/Sets a user-defined value of the unit for the amplitude/time
/// HIPAR unit/arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureAmplitudeTimeArithmeticCustomUnitSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean state,
                            ViString  unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CUSTOM_UNIT_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CUSTOM_UNIT_ENABLED, state),
            3, "State");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_AMPLITUDE_TIME_ARITHMETIC_CUSTOM_UNIT, unit),
            4, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the jitter main measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureJitterMainMeasurement(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_MAIN_MEASUREMENT, measurementFunction),
            3, "Jitter Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines additional measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureJitterAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngJitterMainMeasurement, measurementFunction, &p2string),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ADD %s,%s", measurement, p2string, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the slope at which the periods and thus the jitter is
/// HIFN  measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR cycleBegin/Selects the slope at which the periods and thus the jitter is
/// HIPAR cycleBegin/measured.
ViStatus _VI_FUNC rsscope_ConfigureJitterClockMeasurementCycleBegin(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   cycleBegin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_CYCLE_BEGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_CYCLE_BEGIN, cycleBegin),
            3, "Cycle Begin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the distance between the cycles whose periods are taken for the
/// HIFN  jitter measeurement.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available for the N-cycle jitter measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_CYCLE_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<Measurement>:JITTer:COFFset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR cycleOffset/Sets the distance between the cycles whose periods are taken for the
/// HIPAR cycleOffset/jitter measeurement.
ViStatus _VI_FUNC rsscope_ConfigureJitterClockMeasurementCycleOffset(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   cycleOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_CYCLE_OFFSET,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_CYCLE_OFFSET, cycleOffset),
            3, "Cycle Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the polarity of pulses for which the pulse width is measured to
/// HIFN  obtain the cycle-cycle width and the cycle-cycle duty cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR pulsePolarity/Sets the polarity of pulses for which the pulse width is measured to
/// HIPAR pulsePolarity/obtain the cycle-cycle width and the cycle-cycle duty cycle.
ViStatus _VI_FUNC rsscope_ConfigureJitterClockMeasurementPulsePolarity(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   pulsePolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_PULSE_POLARITY,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_CLOCK_MEASUREMENT_PULSE_POLARITY, pulsePolarity),
            3, "Pulse Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the origin of the clock signal - wether a real clock signal or
/// HIFN  a clock generated using one of the CDR methods.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The function is available for the following measurements:
/// HIFN  time-interval error, unit interval and data rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR clockMode/Defines the origin of the clock signal - wether a real clock signal or
/// HIPAR clockMode/a clock generated using one of the CDR methods.
ViStatus _VI_FUNC rsscope_ConfigureJitterDataMeasurementClockMode(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   clockMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_CLOCK_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_CLOCK_MODE, clockMode),
            3, "Clock Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the clock and data edges that are used for measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR source/Selects clock or data slope.
/// HIPAR slope/Sets the clock and data edges that are used for measurements.
ViStatus _VI_FUNC rsscope_ConfigureJitterDataMeasurementSlope(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   source,
                            ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_DATA_SLOPE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_JITTER_SOURCE_DATA, RSSCOPE_VAL_JITTER_SOURCE_CLOCK),
            3, "Source");

    if (source == RSSCOPE_VAL_JITTER_SOURCE_DATA)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_DATA_SLOPE, slope),
                4, "Slope");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_CLOCK_SLOPE, slope),
                4, "Slope");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the clock and data edges that are used for measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement1/Selects first waveform measurement.
/// HIPAR edgePolarity/Sets the edge of the first waveform from which the skew delay or phase
/// HIPAR edgePolarity/is measured.
/// HIPAR measurement2/Selects second waveform measurement.
/// HIPAR relativePolarity/Sets the edge of the second waveform relative to the first waveform.
ViStatus _VI_FUNC rsscope_ConfigureJitterDelayMeasurement(ViSession instrSession,
                                                          ViInt32   measurement1,
                                                          ViInt32   edgePolarity,
                                                          ViInt32   measurement2,
                                                          ViInt32   relativePolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_DELAY_MEASUREMENT_EDGE_POLARITY,
            "Measurement", measurement1- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement 1");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DELAY_MEASUREMENT_EDGE_POLARITY, edgePolarity),
            3, "Edge Polarity");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_DELAY_MEASUREMENT_RELATIVE_POLARITY,
            "Measurement", measurement2- 1, RS_REPCAP_BUF_SIZE, repCap),
            4, "Measurement 2");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DELAY_MEASUREMENT_RELATIVE_POLARITY, relativePolarity),
            5, "Relative Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the unit for data rate measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR unit/Sets the unit for data rate measurements.
ViStatus _VI_FUNC rsscope_ConfigureJitterDataMeasurementUnit(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_UNIT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_DATA_MEASUREMENT_UNIT, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables limit checking for jitter measurements in the
/// HIFN  specified measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR limitCheck/Enables or disables limit checking for jitter measurements in the
/// HIPAR limitCheck/specified measurement channel.
ViStatus _VI_FUNC rsscope_ConfigureJitterLimitCheck(ViSession instrSession,
                                                    ViInt32   measurement,
                                                    ViInt32   measurementType,
                                                    ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  countJitterMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "JitterMeasurementType", &countJitterMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType,
                    RSSCOPE_VAL_MEAS_JITTER_CCJ, countJitterMeasType - 1),
                3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_CHECK,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_CHECK,
            "JitterMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_JITTER_LIMIT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Define the lower and upper limit for limit checking, respectively.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR lowerLimit/Define the lower limit for limit checking.
/// HIPAR upperLimit/Define the upper limit for limit checking.
ViStatus _VI_FUNC rsscope_ConfigureJitterLimitCheckLimits(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   measurementType,
                                                          ViReal64  lowerLimit,
                                                          ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  countJitterMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "JitterMeasurementType", &countJitterMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType,
                    RSSCOPE_VAL_MEAS_JITTER_CCJ, countJitterMeasType - 1),
                3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_VALUE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_VALUE,
            "JitterMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_VALUE, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_JITTER_LIMIT_UPPER_VALUE, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Define the lower and upper margins for the limit check, respectively.
/// HIFN  Margins are not as strict as limits and must be within the valid value
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR lowerMargin/Define the lower margin for the limit check. Margins are not as strict
/// HIPAR lowerMargin/as limits and must be within the valid value range.
/// HIPAR upperMargin/Define the upper margins for the limit check. Margins are not as
/// HIPAR upperMargin/strict as limits and must be within the valid value range.
ViStatus _VI_FUNC rsscope_ConfigureJitterLimitCheckMargins(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementType,
                                                           ViReal64  lowerMargin,
                                                           ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  countJitterMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "JitterMeasurementType", &countJitterMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_JITTER_CCJ, countJitterMeasType - 1),
                3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_MARGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_MARGIN,
            "JitterMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_JITTER_LIMIT_LOWER_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_JITTER_LIMIT_UPPER_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the source and signal type for the advanced jitter
/// HIFN  analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR analyzedSource/Selects the source for the advanced jitter analysis.
/// HIPAR signalType/Select, if the analyzed signal is a data or clock signal.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterAnalyzedSignal(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   analyzedSource,
                            ViInt32   signalType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_ANALYZED_SOURCE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_ANALYZED_SOURCE, analyzedSource),
            3, "Analyzed Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_SIGNAL_TYPE, signalType),
            4, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the serial standard. Set CUSTOM to adjust the bit rate and the
/// HIFN  bandwidth manually. The serial standards are available only if the
/// HIFN  instrument is able to support its maximum nominal bit rate.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ADVANCED_JITTER_SERIAL_STANDARD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ADVJitter<m>:SIGNal:SERStandard
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR serialStandard/Sets the serial standard. Set Custom to adjust the bit rate and the
/// HIPAR serialStandard/bandwidth manually. The serial standards are available only if the
/// HIPAR serialStandard/instrument is able to support its maximum nominal bit rate.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterSerialStandard(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   serialStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_SERIAL_STANDARD,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_SERIAL_STANDARD, serialStandard),
            3, "Serial Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts a quick advanced jitter analysis. Before starting the quick
/// HIFN  start analysis, set the analyzed source
/// HIFN  (RSSCOPE_ATTR_ADVANCED_JITTER_SIGNAL_SOURCE) and the signal type
/// HIFN  (RSSCOPE_ATTR_ADVANCED_JITTER_SIGNAL_TYPE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterQuickStartAnalysis(ViSession instrSession,
                                                           ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_QUICK_START_ANALYSIS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_QUICK_START_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets that a CDR or clock signal is used as a reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR referenceSource/Sets that a CDR or clock signal is used as a reference signal.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceSource(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   referenceSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE, referenceSource),
            3, "Reference Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures a clock reference signal, if
/// HIFN  RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE is set to CLOCk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR clockSource/Selects the reference source for the clock signal, if
/// HIPAR clockSource/RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE is set to CLOCk.
/// HIPAR nominalBitrate/Sets the frequency of the feed forward CDR. It corresponds to the data
/// HIPAR nominalBitrate/rate of the data stream from which the clock is to be recovered.
/// HIPAR clockEdge/The clock edge sets the point in time when the state of the data
/// HIPAR clockEdge/signal is checked.
/// HIPAR uiOffset/Defines the position of the last bit of the bit pattern to be
/// HIPAR uiOffset/triggered on, relative to the trigger time. The unit interval offset is
/// HIPAR uiOffset/a number between 0 and 1:
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceSignalClock(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   clockSource,
                            ViInt64   nominalBitrate,
                            ViInt32   clockEdge,
                            ViReal64  uiOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_SOURCE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_SOURCE, clockSource),
            3, "Clock Source");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_NOMINAL_BITRATE, nominalBitrate),
                4, "Nominal Bitrate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_EDGE, clockEdge),
            5, "Clock Edge");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_UI_OFFSET, uiOffset),
            6, "UI Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the clock frequency for the explicit clock signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR clockFrequency/Sets the clock frequency for the explicit clock signal.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceClockFrequency(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  clockFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_FREQUENCY,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_FREQUENCY, clockFrequency),
            3, "Clock Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Executes an estimation of the signal for the clock frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
ViStatus _VI_FUNC rsscope_RunAdvancedJitterReferenceClockFreqEstimation(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_FREQ_ESTIMATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_FREQ_ESTIMATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a clock multiplier factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR clockMultiplier/Sets a clock multiplier factor.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceClockMultiplierFactor(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   clockMultiplier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_MULTIPLIER_FACTOR,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_MULTIPLIER_FACTOR, clockMultiplier),
            3, "Clock Multiplier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a clock offset for the explicit clock signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR clockOffset/Sets a clock offset for the explicit clock signal.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceClockOffset(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  clockOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_OFFSET,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_OFFSET, clockOffset),
            3, "Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Executes an estimation of the signal for the clock offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR advjittermeas/Selects repeated capability AdvJitterMeas.
ViStatus _VI_FUNC rsscope_RunAdvancedJitterReferenceClockOffsetEstimation(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_OFFSET_ESTIMATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_OFFSET_ESTIMATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a sampling time for the explicit clock signal, an offset for the
/// HIFN  clock edge in relation to the bit start. The clock edge sets the
/// HIFN  beginning of the unit interval. The sampling time is a number between 0
/// HIFN  and 1. Value 0 sets the clock edge to the beginning of the bit period;
/// HIFN  value 0.5 sets the clock edge to the middle of the bit period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR samplingTime/Sets a sampling time for the explicit clock signal, an offset for the
/// HIPAR samplingTime/clock edge in relation to the bit start. The clock edge sets the
/// HIPAR samplingTime/beginning of the unit interval. The sampling time is a number between 0
/// HIPAR samplingTime/and 1. Value 0 sets the clock edge to the beginning of the bit period;
/// HIPAR samplingTime/value 0.5 sets the clock edge to the middle of the bit period.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceClockSamplingTime(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  samplingTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_SAMPLING_TIME,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_SAMPLING_TIME, samplingTime),
            3, "Sampling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays the calculated data rate of the explicit clock signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR dataRate/Displays the calculated data rate of the explicit clock signal.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterReferenceClockDataRate(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64* dataRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_DATA_RATE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CLOCK_DATA_RATE, dataRate),
            3, "Data Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the reference source for the CDR signal, if
/// HIFN  RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE is set to CDR.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR cdrSource/Selects the reference source for the CDR signal, if
/// HIPAR cdrSource/RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_SOURCE is set to CDR.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterReferenceSignalCDR(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   cdrSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CDR_SOURCE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_REFERENCE_CDR_SOURCE, cdrSource),
            3, "CDR Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables and configures coupling for the gating setup for the advanced
/// HIFN  jitter analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR useGate/Enables the gating setup for the advanced jitter analysis.
/// HIPAR couplingMode/Sets if the gate is coupled to the zoom or cursor.
/// HIPAR coupledCursor/Sets the cursor the gate is coupled to, if Coupling Mode is set to
/// HIPAR coupledCursor/Cursor.
/// HIPAR diagramName/Sets the name of the diagram on which the zoom is based, if Coupling
/// HIPAR diagramName/Mode is set to Zoom.
/// HIPAR zoomName/Sets the name of the zoom diagram, if Coupling Mode is set to Zoom.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterGating(ViSession instrSession,
                                                        ViInt32   measurement,
                                                        ViBoolean useGate,
                                                        ViInt32   couplingMode,
                                                        ViInt32   coupledCursor,
                                                        ViString  diagramName,
                                                        ViString  zoomName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   rawstring[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_ENABLED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_ENABLED, useGate),
            3, "Use Gate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_COUPLING_MODE, couplingMode),
            4, "Coupling Mode");

    switch (couplingMode)
    {
    case RSSCOPE_VAL_GATE_COUPLING_MODE_CURSOR:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_COUPLED_CURSOR, coupledCursor),
                5, "Coupled Cursor");
        break;
    case RSSCOPE_VAL_GATE_COUPLING_MODE_ZOOM:
        snprintf(rawstring, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "'%s','%s'", diagramName, zoomName);
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_COUPLED_ZOOM_DIAGRAM, rawstring));
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and configures the gate area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR mode/Selects if the gate area is defined in relative values (percent) or
/// HIPAR mode/absolute.
/// HIPAR start/Sets the start time of the gate area. If
/// HIPAR start/RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE is set to ABS, units are s and
/// HIPAR start/the valid range is -1.0E+026 s to 1.0E+026 s. If
/// HIPAR start/RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE is set to REL, units are % and
/// HIPAR start/the valid range is 0.0% to 100.0%.
/// HIPAR stop/Sets the stop time of the gate area. If
/// HIPAR stop/RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE is set to ABS, units are s and
/// HIPAR stop/the valid range is -1.0E+026 s to 1.0E+026 s. If
/// HIPAR stop/RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE is set to REL, units are % and
/// HIPAR stop/the valid range is 0.0% to 100.0%.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterGateArea(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   mode,
                                                          ViReal64  start,
                                                          ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   startAttr[2] = {
             RSSCOPE_ATTR_ADVANCED_JITTER_GATING_ABSOLUTE_START,
             RSSCOPE_ATTR_ADVANCED_JITTER_GATING_RELATIVE_START
             };
    ViAttr   stopAttr[2] = {
             RSSCOPE_ATTR_ADVANCED_JITTER_GATING_ABSOLUTE_STOP,
             RSSCOPE_ATTR_ADVANCED_JITTER_GATING_RELATIVE_STOP
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_GATING_MODE, mode),
            3, "Mode");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, startAttr[mode], start),
            4, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, stopAttr[mode], stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the  step response length in unit intervals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR length/Set the  step response length in unit intervals.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionStepResponse(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_RESPONSE_LENGTH,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_RESPONSE_LENGTH, length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Runs an estimate of the step response for the advanced jitter analysis
/// HIFN  measurements. To query the result of the estimate, use
/// HIFN  rsscope_QueryAdvancedJitterDecompositionStepResponseEstimatedMax.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionEstimateStepResponse(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_ESTIMATE_STEP_RESPONSE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_ESTIMATE_STEP_RESPONSE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the estimated maximum step length response. To run an estimate
/// HIFN  first, use rsscope_AdvancedJitterDecompositionEstimateStepResponse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR estimatedMax/Queries the estimated maximum step length response.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterDecompositionStepResponseEstimatedMax(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64* estimatedMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_RESPONSE_ESTIMATED_MAX,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_RESPONSE_ESTIMATED_MAX, estimatedMax),
            3, "Estimated Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the steps position. It defines what percentage of the unit
/// HIFN  lengths are displayed before the step response.
/// HIFN     For example, if the Length is set to 20 UI and the Step position is
/// HIFN  set to 75%, the step response diagram displays 15 UI before the step
/// HIFN  response and 5 UI after that.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR stepPosition/Sets the steps position. It defines what percentage of the unit
/// HIPAR stepPosition/lengths are displayed before the step response.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionStepPosition(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   stepPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_POSITION,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_STEP_POSITION, stepPosition),
            3, "Step Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the target bit error rate for total jitter at BER components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR targetExponent/Sets the target bit error rate for total jitter at BER components.
/// HIPAR targetExponent/This value is the exponent n where BER = 10^-n.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionTargetBER(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   targetExponent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_TARGET_BER_EXPONENT,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_TARGET_BER_EXPONENT, targetExponent),
            3, "Target Exponent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures values displayed values for jitter and noise.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability AdvJitterMeas.
/// HIPAR jitterInterleaved/Displays interleaved histogram values.
/// HIPAR periodicJitterLabels/Enables the display of the periodic jitter components labels.
/// HIPAR periodicNoiseLabels/Enables the display of the periodic noise components labels.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionDisplaySettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean jitterInterleaved,
                            ViBoolean periodicJitterLabels,
                            ViBoolean periodicNoiseLabels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "AdvJitterMeas", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_DISPLAY_INTERLEAVED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_DISPLAY_INTERLEAVED, jitterInterleaved),
            3, "Jitter Interleaved");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_DISPLAY_PERIODIC_JITTER_LABELS, periodicJitterLabels),
            4, "Periodic Jitter Labels");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_DISPLAY_PERIODIC_NOISE_LABELS, periodicNoiseLabels),
            5, "Periodic Noise Labels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects all basic jitter components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionSelectAllBasicJitterComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_BASIC_JITTER_COMPONENTS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_BASIC_JITTER_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects all jitter components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionSelectAllJitterComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_JITTER_COMPONENTS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_JITTER_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears all selected basic jitter components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionClearSelectedBasicJitterComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_BASIC_JITTER_COMPONENTS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_BASIC_JITTER_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears all selected jitter components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionClearSelectedJitterComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_JITTER_COMPONENTS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_JITTER_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the measurement for the specified jitter component.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_COMPONENT_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ADVJitter<m>:DCOMposition:COMPonents<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR enabled/Enables the measurement for the specified jitter component.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionComponentEnabled(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_COMPONENT_ENABLED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_COMPONENT_ENABLED,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_COMPONENT_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects all noise components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionSelectAllNoiseComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_NOISE_COMPONENTS,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_SELECT_ALL_NOISE_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all selected noise components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
ViStatus _VI_FUNC rsscope_AdvancedJitterDecompositionClearSelectedNoiseComponents(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_NOISE_COMPONENTS,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_CLEAR_SELECTED_NOISE_COMPONENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the display of the periodic jitter components table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR enabled/Enables the display of the periodic jitter components table.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionPeriodicComponentsEnabled(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_PERIODIC_COMPONENTS_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_PERIODIC_COMPONENTS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the periodic jitter component. Requires the periodic jitter
/// HIFN  component to be enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR verticalComponents/Sets the maximum number of periodic components in the vertical
/// HIPAR verticalComponents/direction that are considered for the measurement. If the actual number
/// HIPAR verticalComponents/of periodic components is higher than the set value, they are
/// HIPAR verticalComponents/considered for the random jitter component.
/// HIPAR horizontalComponents/Sets the maximum number of periodic components in the horizontal
/// HIPAR horizontalComponents/direction that are considered for the measurement. If the actual number
/// HIPAR horizontalComponents/of periodic components is higher than the set value, they are
/// HIPAR horizontalComponents/considered for the random jitter component.
/// HIPAR pjTableEnabled/Enables the display of the periodic jitter components table.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionMaxPeriodicJitterComponents(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   verticalComponents,
                            ViInt32   horizontalComponents,
                            ViBoolean pjTableEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_MAX_PERIODIC_VERTICAL_COMPONENTS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_MAX_PERIODIC_VERTICAL_COMPONENTS, verticalComponents),
            3, "Vertical Components");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_MAX_PERIODIC_HORIZONTAL_COMPONENTS, horizontalComponents),
            4, "Horizontal Components");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_PERIODIC_JITTER_TABLE_ENABLED, pjTableEnabled),
            5, "PJ Table Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, allows the selection of the (other) bounded uncorrelated
/// HIFN  jitter/noise: (O)BUJ, (O)BUJ(d -d) , RJ, (O)BUN, (O)BUN(d -d), RN.
/// HIFN     The other bounded uncorrelated jitter/noise includes the periodic
/// HIFN  jitter/noise components. It is displayed, if Periodic is enabled.
/// HIFN  Otherwise the bounded uncorrelated jitter/noise is displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR enabled/If enabled, allows the selection of the (other) bounded uncorrelated
/// HIPAR enabled/jitter/noise: (O)BUJ, (O)BUJ(d -d) , RJ, (O)BUN, (O)BUN(d -d), RN.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterDecompositionOtherBoundedUncorrelatedEnabled(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_OTHER_BOUNDED_UNCORRELATED_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_OTHER_BOUNDED_UNCORRELATED_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the advanced jitter analysis. It displays all enabled diagrams
/// HIFN  and displays a table with the results for the component measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR resultsEnabled/Enables the advanced jitter analysis. It displays all enabled diagrams
/// HIPAR resultsEnabled/and displays a table with the results for the component measurements.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsEnabled(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean resultsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_ENABLED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_ENABLED, resultsEnabled),
            3, "Results Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the display of measurement results for the advanced jitter
/// HIFN  analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR statisticsEnabled/Enables statistics measurements for the advanced jitter analysis.
/// HIPAR stepResponseEnabled/Enables a step response diagram for the advanced jitter measurements.
/// HIPAR bathtubEnabled/Enables a bathtub plot for the advanced jitter measurements.
/// HIPAR bathtubPlot/Selects if you want to display the bathtub results versus total jitter
/// HIPAR bathtubPlot/(TJ) or data-dependent jitter (DDJ) and random jitter (RJ) components.
/// HIPAR syntheticEyeEnabled/Selects if you want to display synthetic eye results only for the
/// HIPAR syntheticEyeEnabled/deterministic jitter (DJ) components or the data-dependent jitter (DDJ)
/// HIPAR syntheticEyeEnabled/components.
/// HIPAR syntheticEyePlot/Selects if you want to display synthetic eye results only for the
/// HIPAR syntheticEyePlot/deterministic jitter (DDP) components or the data-dependent jitter (DD)
/// HIPAR syntheticEyePlot/components.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterGeneralResults(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean statisticsEnabled,
                            ViBoolean stepResponseEnabled,
                            ViBoolean bathtubEnabled,
                            ViInt32   bathtubPlot,
                            ViBoolean syntheticEyeEnabled,
                            ViInt32   syntheticEyePlot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_STATISTICS_ENABLED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_STATISTICS_ENABLED, statisticsEnabled),
            3, "Statistics Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_STEP_RESPONSE_DIAGRAM_ENABLED, stepResponseEnabled),
            4, "Step Response Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BATHTUB_PLOT_ENABLED, bathtubEnabled),
            5, "Bathtub Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BATHTUB_PLOT_TYPE, bathtubPlot),
            6, "Bathtub Plot");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_PLOT_ENABLED, syntheticEyeEnabled),
            7, "Synthetic Eye Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_PLOT_TYPE, syntheticEyePlot),
            8, "Synthetic Eye Plot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables periodic or other bounded uncorrelated jitter components to be
/// HIFN  considered for the jitter bathtub.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR periodicComponentsEnabled/If enabled, the periodic jitter components are considered for the
/// HIPAR periodicComponentsEnabled/jitter bathtub. Requires periodic components to be enabled
/// HIPAR periodicComponentsEnabled/(RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_PERIODIC_COMPONENTS_ENABLED).
/// HIPAR obujComponentsEnabled/If enabled, the other bounded uncorrelated jitter components are
/// HIPAR obujComponentsEnabled/considered for the jitter bathtub. Requires other bounded uncorrelated
/// HIPAR obujComponentsEnabled/components to be enabled
/// HIPAR obujComponentsEnabled/(RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_OTHER_BOUNDED_UNCORRELATED_ENABLED).
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterBathtubComponents(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean periodicComponentsEnabled,
                            ViBoolean obujComponentsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BATHTUB_PERIODIC_COMPONENTS_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BATHTUB_PERIODIC_COMPONENTS_ENABLED, periodicComponentsEnabled),
            3, "Periodic Components Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BATHTUB_OBUJ_COMPONENTS_ENABLED, obujComponentsEnabled),
            4, "OBUJ Components Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures components for consideration in the noise
/// HIFN  bathtub plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR plotEnabled/Enables a noise bathtub plot, a graph of BER in a log scale versus
/// HIPAR plotEnabled/data-dependent jitter (DDN).
/// HIPAR periodicComponentsEnabled/If enabled, the periodic noise components are considered for the noise
/// HIPAR periodicComponentsEnabled/bathtub. Requires periodic components to be enabled
/// HIPAR periodicComponentsEnabled/(RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_PERIODIC_COMPONENTS_ENABLED).
/// HIPAR obunComponentsEnabled/If enabled, the other bounded uncorrelated noise components are
/// HIPAR obunComponentsEnabled/considered for the noise bathtub. Requires other bounded uncorrelated
/// HIPAR obunComponentsEnabled/components to be enabled
/// HIPAR obunComponentsEnabled/(RSSCOPE_ATTR_ADVANCED_JITTER_DECOMPOSITION_OTHER_BOUNDED_UNCORRELATED_ENABLED).
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsNoiseBathtubPlot(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean plotEnabled,
                            ViBoolean periodicComponentsEnabled,
                            ViBoolean obunComponentsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_BATHTUB_PLOT_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_BATHTUB_PLOT_ENABLED, plotEnabled),
            3, "Plot Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_BATHTUB_PERIODIC_COMPONENTS_ENABLED, periodicComponentsEnabled),
            4, "Periodic Components Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_BATHTUB_OBUN_COMPONENTS_ENABLED, obunComponentsEnabled),
            5, "OBUN Components Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables horizontal and vertical periodic components to the
/// HIFN  data-dependent synthetic eye.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN
/// HIFN  RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_HORIZONTAL_COMPONENTS_ENABLED
/// HIFN
/// HIFN  RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_VERTICAL_COMPONENTS_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ADVJitter<m>:RESult:PHORizontal
/// HIFN     ADVJitter<m>:RESult:PVERtical
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR horizontalComponentsEnabled/If enabled, adds horizontal periodic components to the data-dependent
/// HIPAR horizontalComponentsEnabled/synthetic eye.
/// HIPAR verticalComponentsEnabled/If enabled, adds vertical periodic components to the data-dependent
/// HIPAR verticalComponentsEnabled/synthetic eye.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsSyntheticEyePeriodicComponentsEnabled(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean horizontalComponentsEnabled,
                            ViBoolean verticalComponentsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_HORIZONTAL_COMPONENTS_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_HORIZONTAL_COMPONENTS_ENABLED, horizontalComponentsEnabled),
            3, "Horizontal Components Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SYNTHETIC_EYE_VERTICAL_COMPONENTS_ENABLED, verticalComponentsEnabled),
            4, "Vertical Components Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the display of results for the selected measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an analysis component.
/// HIPAR histogram/Enables a histogram for the selected measurement component. Valid for
/// HIPAR histogram/f_symb, T_symb, RJ, TJ, DJ, DDJ, and PJ components.
/// HIPAR track/Enables a track for the advanced jitter measurements. Valid for
/// HIPAR track/f_symb, T_symb, RJ, TJ, DDJ, and PJ components.
/// HIPAR spectrum/Enables a spectrum diagram for the selected measurement component.
/// HIPAR spectrum/Valid for RJ, TJ, DDJ, and PJ components.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterComponentSpecificResults(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViBoolean histogram,
                            ViBoolean track,
                            ViBoolean spectrum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_HISTOGRAM_ENABLED,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_HISTOGRAM_ENABLED,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    switch (component)
    {
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_F_SYMB:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_T_SYMB:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_RJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_TJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DDJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_PJ:
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_HISTOGRAM_ENABLED, histogram),
                4, "Histogram");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, component), 3, "Component");
        break;
    }

    switch (component)
    {
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_F_SYMB:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_T_SYMB:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_RJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_TJ:
        //case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DDJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_PJ:
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_TRACK_ENABLED, track),
                5, "Track");
        break;
    default:
        break;
    }

    switch (component)
    {
        //case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_F_SYMB:
        //case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_T_SYMB:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_RJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_TJ:
        //case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_DDJ:
    case RSSCOPE_REPCAP_ADVJITTERCOMPONENT_PJ:
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SPECTRUM_ENABLED, spectrum),
                6, "Spectrum");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects which transitions are considered for the display of the
/// HIFN  diagrams, rising, falling or both. This setting is mostly relevant for
/// HIFN  histograms, tracks and the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR transitions/Selects which transitions are considered for the display of the
/// HIPAR transitions/diagrams, rising, falling or both. This setting is mostly relevant for
/// HIPAR transitions/histograms.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsDiagramTransitions(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   transitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_DIAGRAM_TRANSITIONS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_DIAGRAM_TRANSITIONS, transitions),
            3, "Transitions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects which levels are considered for the display of the diagrams,
/// HIFN  rising, falling or both. This setting is mostly relevant for
/// HIFN  histograms, tracks and the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR diagramLevels/Selects which levels are considered for the display of the diagrams,
/// HIPAR diagramLevels/rising, falling or both. This setting is mostly relevant for
/// HIPAR diagramLevels/histograms, tracks and the measurement results.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsDiagramLevels(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   diagramLevels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_DIAGRAM_LEVELS,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_DIAGRAM_LEVELS, diagramLevels),
            3, "Diagram Levels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures options for scaling of results displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR maxNumOfBins/Sets the number of bins, the number of data intervals, of the
/// HIPAR maxNumOfBins/histogram.
/// HIPAR continuousAutoScale/If enabled, the measurement scale is set automatically. If disabled
/// HIPAR continuousAutoScale/you can manually set a
/// HIPAR continuousAutoScale/RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE and a
/// HIPAR continuousAutoScale/RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET.
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsScaling(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   maxNumOfBins,
                            ViBoolean continuousAutoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SCALING_MAX_NUM_OF_BINS,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SCALING_MAX_NUM_OF_BINS, maxNumOfBins),
            3, "Max Num Of Bins");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_CONTINUOUS_AUTO_SCALE_ENABLED, continuousAutoScale),
            4, "Continuous Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the basic measurement type to apply the vertical scale to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR measurementType/Selects the basic measurement type to apply the vertical scale to.
/// HIPAR scale/Sets the scale for the selected component. Requires
/// HIPAR scale/RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_CONTINUOUS_AUTO_SCALE_ENABLED to
/// HIPAR scale/be off. Units depend on which value is being set. You can set separate
/// HIPAR scale/scale values for:
/// HIPAR offset/Sets the offest for the selected component. Requires
/// HIPAR offset/RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_CONTINUOUS_AUTO_SCALE_ENABLED to
/// HIPAR offset/be off. Units depend on which value is being set. You can set separate
/// HIPAR offset/offset values for:
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsScalingManual(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViReal64  scale,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SCALING_MEASUREMENT_TYPE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    switch (measurementType)
    {
    case RSSCOPE_VAL_ADVANCED_JITTER_SCALING_SYMBOL_RATE:
    case RSSCOPE_VAL_ADVANCED_JITTER_SCALING_SYMBOL_DURATION:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_SCALING_MEASUREMENT_TYPE, measurementType),
                3, "Measurement Type");
        break;
    default:
        break;
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE,
            "AdvJitterComponent", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE, scale),
            4, "Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables continuous auto scale for jitter results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR basicJitterAutoScale/If enabled, the measurement scale of the basic jitter measurement
/// HIPAR basicJitterAutoScale/histogram and track is set automatically. If disabled you can manually
/// HIPAR basicJitterAutoScale/set a scale (RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE) and
/// HIPAR basicJitterAutoScale/an offset (RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET).
/// HIPAR spectrumAutoScale/If enabled, the measurement scale of the jitter spectrum is set
/// HIPAR spectrumAutoScale/automatically. If disabled you can manually set a scale
/// HIPAR spectrumAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE) and an offset
/// HIPAR spectrumAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET).
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsScalingAutoJitter(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean basicJitterAutoScale,
                            ViBoolean spectrumAutoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BASIC_JITTER_CONTINUOUS_AUTO_SCALE_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_BASIC_JITTER_CONTINUOUS_AUTO_SCALE_ENABLED, basicJitterAutoScale),
            3, "Basic Jitter Auto Scale");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_JITTER_SPECTRUM_CONTINUOUS_AUTO_SCALE_ENABLED, spectrumAutoScale),
            3, "spectrum Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables continuous auto scale for noise results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR noiseAutoScale/If enabled, the measurement scale of the noise histogram and track is
/// HIPAR noiseAutoScale/set automatically. If disabled you can manually set a scale
/// HIPAR noiseAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE) and an offset
/// HIPAR noiseAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET).
/// HIPAR spectrumAutoScale/If enabled, the measurement scale for the noise spectrum is set
/// HIPAR spectrumAutoScale/automatically. If disabled you can manually set a scale
/// HIPAR spectrumAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_SCALE) and an offset
/// HIPAR spectrumAutoScale/(RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_OFFSET).
ViStatus _VI_FUNC rsscope_ConfigureAdvancedJitterResultsScalingAutoNoise(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean noiseAutoScale,
                            ViBoolean spectrumAutoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_CONTINUOUS_AUTO_SCALE_ENABLED,
            "AdvJitterMeas", measurement, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_CONTINUOUS_AUTO_SCALE_ENABLED, noiseAutoScale),
            3, "Noise Auto Scale");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_NOISE_SPECTRUM_CONTINUOUS_AUTO_SCALE_ENABLED, spectrumAutoScale),
            4, "Spectrum Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute/peak to peak value of the specified jitter
/// HIFN  component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR peakToPeak/Queries the absolute/peak to peak value of the specified jitter
/// HIPAR peakToPeak/component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentPeakToPeak(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* peakToPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK, peakToPeak),
            4, "Peak To Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the standard deviation value of the selected component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR standardDeviation/Queries the standard deviation value of the selected component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStandardDeviation(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* standardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION, standardDeviation),
            4, "Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of waveforms (acquisitions) the measurement is
/// HIFN  based on for the specified measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR waveformCount/Returns the number of waveforms (acquisitions) the measurement is
/// HIPAR waveformCount/based on for the specified measurement component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsWaveformCount(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViInt64*  waveformCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_WAVEFORM_COUNT,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_WAVEFORM_COUNT,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_WAVEFORM_COUNT, waveformCount),
                4, "Waveform Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the peak to peak value of the long-term measurement results
/// HIFN  for the specified measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR peakToPeak/Returns the peak to peak value of the long-term measurement results
/// HIPAR peakToPeak/for the specified measurement component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsPeakToPeak(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* peakToPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_PEAK_TO_PEAK, peakToPeak),
            4, "Peak To Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the standard deviation of the long-term measurement results
/// HIFN  for the specified measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR standardDeviation/Returns the standard deviation of the long-term measurement results
/// HIPAR standardDeviation/for the specified measurement component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsStandardDeviation(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* standardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STANDARD_DEVIATION, standardDeviation),
            4, "Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the absolute maximum value of the specified measurement
/// HIFN  component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR maxPeakToPeak/Returns the absolute maximum value of the specified measurement
/// HIPAR maxPeakToPeak/component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsMaxPeakToPeak(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* maxPeakToPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_PEAK_TO_PEAK,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_PEAK_TO_PEAK,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_PEAK_TO_PEAK, maxPeakToPeak),
            4, "Max Peak To Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return the standard deviation of the maximum value of the specified
/// HIFN  measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR maxStandardDeviation/Return the standard deviation of the maximum value of the specified
/// HIPAR maxStandardDeviation/measurement component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsMaxStandardDeviation(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* maxStandardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_STANDARD_DEVIATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_STANDARD_DEVIATION,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MAX_STANDARD_DEVIATION, maxStandardDeviation),
            4, "Max Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the absolute minimum value of the specified measurement
/// HIFN  component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR minPeakToPeak/Returns the absolute minimum value of the specified measurement
/// HIPAR minPeakToPeak/component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsMinPeakToPeak(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* minPeakToPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_PEAK_TO_PEAK,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_PEAK_TO_PEAK,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_PEAK_TO_PEAK, minPeakToPeak),
            4, "Min Peak To Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return the standard deviation of the minimum value of the specified
/// HIFN  measurement component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR component/Selects an advanced jitter analysis component.
/// HIPAR minStandardDeviation/Return the standard deviation of the minimum value of the specified
/// HIPAR minStandardDeviation/measurement component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsComponentStatisticsMinStandardDeviation(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   component,
                            ViReal64* minStandardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_STANDARD_DEVIATION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_STANDARD_DEVIATION,
            "AdvJitterComponent", component- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_COMPONENT_STATISTICS_MIN_STANDARD_DEVIATION, minStandardDeviation),
            4, "Min Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of periodic components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR componentCount/Returns the number of periodic components.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsPeriodicComponentCount(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32*  componentCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_COUNT,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_COUNT, componentCount),
            3, "Component Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the frequency of the selected periodic component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR periodicComponent/Selects a periodic component from the period jitter results table. The
/// HIPAR periodicComponent/total number of rows in the table can be queried with
/// HIPAR periodicComponent/rsscope_QueryAdvancedJitterResultsPeriodicComponentCount.
/// HIPAR frequency/Returns the frequency of the selected periodic component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsPeriodicComponentFrequency(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   periodicComponent,
                            ViInt64*  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_FREQUENCY,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_FREQUENCY,
            "AdvJitterPeriodicComponent", periodicComponent- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Periodic Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_FREQUENCY, frequency),
                4, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the value of the selected periodic component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR periodicComponent/Selects a periodic component from the period jitter results table. The
/// HIPAR periodicComponent/total number of rows in the table can be queried with
/// HIPAR periodicComponent/rsscope_QueryAdvancedJitterResultsPeriodicComponentCount.
/// HIPAR value/Returns the value of the selected periodic component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsPeriodicComponentValue(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   periodicComponent,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_VALUE,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_VALUE,
            "AdvJitterPeriodicComponent", periodicComponent- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Periodic Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_VALUE, value),
            4, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the direction of the selected periodic component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the advanced jitter measurement repeated capability.
/// HIPAR periodicComponent/Selects a periodic component from the period jitter results table. The
/// HIPAR periodicComponent/total number of rows in the table can be queried with
/// HIPAR periodicComponent/rsscope_QueryAdvancedJitterResultsPeriodicComponentCount.
/// HIPAR direction/Returns the direction of the selected periodic component.
ViStatus _VI_FUNC rsscope_QueryAdvancedJitterResultsPeriodicComponentDirection(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   periodicComponent,
                            ViInt32*  direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_component[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_DIRECTION,
            "AdvJitterMeas", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_DIRECTION,
            "AdvJitterPeriodicComponent", periodicComponent- 1, RS_REPCAP_BUF_SIZE, repCap_component),
            3, "Periodic Component");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap, repCap_component);

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ADVANCED_JITTER_RESULTS_PERIODIC_COMPONENT_DIRECTION, direction),
            4, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the main measurement. This measurement is the one referred to
/// HIFN  if the measurement waveform is used as a source for math calculations.
/// HIFN  The main measurement type must belong to the same category as the other
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterMainMeasurement(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_MAIN_MEASUREMENT, measurementFunction),
            3, "Eye Jitter Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines additional measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngEyeJitterMeasurementType, measurementFunction, &
                    p2string),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ADD %s,%s", measurement, p2string, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables limit checking for eye/jitter measurements in the
/// HIFN  specified measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Eye Jitter measurement type.
/// HIPAR limitCheck/Enables or disables limit checking for eye/jitter measurements in the
/// HIPAR limitCheck/specified measurement channel.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterLimitCheck(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViInt32   measurementType,
                                                       ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  maxEyeMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "EyeMeasurementType", &maxEyeMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_EXTINCTION_RATIO_PCT, maxEyeMeasType),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_LIMIT_CHECK,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_LIMIT_CHECK,
            "EyeMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_LIMIT_CHECK, limitCheck),
            3, "limitCheck");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper limit for limit checking, respectively.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Eye Jitter measurement type.
/// HIPAR lowerLimit/Define the lower limit for limit checking.
/// HIPAR upperLimit/Define the upper limit for limit checking.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterLimitCheckLimits(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementType,
                                                             ViReal64  lowerLimit,
                                                             ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
    ViInt32  maxEyeMeasType = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "EyeMeasurementType", &maxEyeMeasType));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_EXTINCTION_RATIO_PCT, maxEyeMeasType),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_VALUE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_VALUE,
            "EyeMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_VALUE, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_JITTER_UPPER_LIMIT_VALUE, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper margins for the limit check, respectively.
/// HIFN  Margins are not as strict as limits and must be within the valid value
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Eye Jitter measurement type.
/// HIPAR lowerMargin/Define the lower margin for the limit check. Margins are not as strict
/// HIPAR lowerMargin/as limits and must be within the valid value range.
/// HIPAR upperMargin/Define the upper margins for the limit check. Margins are not as
/// HIPAR upperMargin/strict as limits and must be within the valid value range.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterLimitCheckMargins(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViReal64  lowerMargin,
                            ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_EXTINCTION_RATIO_PCT, RSSCOPE_VAL_MEAS_RMS_JITTER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_MARGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_MARGIN,
            "EyeMeasurementType", measurementType- 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_JITTER_LOWER_LIMIT_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_JITTER_UPPER_LIMIT_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the start and stop of the measurement window in
///HINF  relative values, algorithm and threshold of the measurement
///HINF  histogram.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR start/Defines the start of the measurement window in relative
///HIPAR start/values.
///HIPAR stop/Defines the stop of the measurement window in relative
///HIPAR stop/values.
///HIPAR algorithm/Defines how to compute the top/base level and the
///HIPAR algorithm/corresponding sigma values.
///HIPAR threshold/Sets the threshold used to separate peaks: two
///HIPAR threshold/peaks are separated if they are split by a valley of
///HIPAR threshold/height Threshold * Maximal_histogram_value.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterGeneralSettings(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViReal64  start,
                                                            ViReal64  stop,
                                                            ViInt32   algorithm,
                                                            ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_START,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_START, start),
            3, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_STOP, stop),
            4, "Stop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_ALGORITHM, algorithm),
            5, "Algorithm");

    if (algorithm != RSSCOPE_VAL_EYE_JITTER_ALGORITHM_MEAN);
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_THRESHOLD, threshold),
                6, "Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the settings for the eye height measurements.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR algorithm/Selects the algorithm for the measurement of the
///HIPAR algorithm/height.
///HIPAR start/Defines the start of the measurement window in relative
///HIPAR start/values.
///HIPAR stop/Defines the start of the measurement window in relative
///HIPAR stop/values.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterHeight(ViSession instrSession,
                                                   ViInt32   measurement,
                                                   ViInt32   algorithm,
                                                   ViReal64  start,
                                                   ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_HEIGHT_START,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_HEIGHT_ALGORITHM, algorithm),
            3, "Algorithm");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_HEIGHT_START, start),
            4, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_HEIGHT_STOP, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the settings for the eye width measurements.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR algorithm/Selects the algorithm for the measurement of the
///HIPAR algorithm/width.
///HIPAR useMeasurementWindow/If enabled, you can define a user
///HIPAR useMeasurementWindow/defined start and stop values for the eye width
///HIPAR useMeasurementWindow/measurement.
///HIPAR start/Sets the start of the measurement window in relative
///HIPAR start/values.
///HIPAR stop/Sets the stop of the measurement window in relative
///HIPAR stop/values.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterWidth(ViSession instrSession,
                                                  ViInt32   measurement,
                                                  ViInt32   algorithm,
                                                  ViBoolean useMeasurementWindow,
                                                  ViReal64  start,
                                                  ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_WIDTH_ALGORITHM,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_WIDTH_ALGORITHM, algorithm),
            3, "Algorithm");

    if (algorithm == RSSCOPE_VAL_EYE_JITTER_WIDTH_ALGORITHM_ACTWIDTH);
    {
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_WIDTH_WINDOW_ENABLED, useMeasurementWindow),
                4, "Use Measurement Window");

        if (useMeasurementWindow == VI_TRUE)
        {
            viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_WIDTH_START, start),
                    5, "Start");

            viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_WIDTH_STOP, stop),
                    6, "Stop");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the level of the measurement window in relative
///HINF  values.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR level/Defines the level for the eye measurement jitter.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterDutyCycleDistortion(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_DUTY_CYCLE_DISTORTION_LEVEL,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_DUTY_CYCLE_DISTORTION_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the settings for the eye measurement jitter.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR useMeasurementLevel/If enabled, you can define a user defined
///HIPAR useMeasurementLevel/level for the eye measurement jitter.
///HIPAR level/Defines the level of the measurement window in relative
///HIPAR level/values.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterLevel(ViSession instrSession,
                                                  ViInt32   measurement,
                                                  ViBoolean useMeasurementLevel,
                                                  ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_USE_MEAS_LEVEL,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_USE_MEAS_LEVEL, useMeasurementLevel),
            3, "Use Measurement Level");

    if (useMeasurementLevel == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_LEVEL, level),
                4, "Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the local start level and local stop level of the range
///HINF  for eye rise/fall time measurements.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR localStartLevel/Sets the local start level of range for eye
///HIPAR localStartLevel/measurements rise/fall time.
///HIPAR localStopLevel/Sets the local stop level of range for eye
///HIPAR localStopLevel/measurements rise/fall time.
ViStatus _VI_FUNC rsscope_ConfigureEyeJitterRiseTimeFallTimeLevel(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  localStartLevel,
                            ViReal64  localStopLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_RTFT_START_LEVEL,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_RTFT_START_LEVEL, localStartLevel),
            3, "Local Start Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_EYE_JITTER_MEASUREMENT_RTFT_STOP_LEVEL, localStopLevel),
            4, "Local Stop Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to
/// HIFN  if the measurement waveform is used as a source for math calculations.
/// HIFN  The main measurement type must belong to the same category as the other
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumMainMeasurement(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_MAIN_MEASUREMENT, measurementFunction),
            3, "Spectrum Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines additional measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumMeasurementType, measurementFunction, &p2string
                ),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ADD %s,%s", measurement, p2string, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the spectrum settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR ndBDown/Defines the threshold until which the samples to the left and right of
/// HIPAR ndBDown/the peak value are analyzed in order to determine the "N dB down
/// HIPAR ndBDown/bandwidth".
/// HIPAR channelBandwidth/Defines the bandwidth over which the channel power is calculated.
/// HIPAR channelCenterFrequency/Defines the center frequency from which the channel power is
/// HIPAR channelCenterFrequency/calculated over the specified bandwidth.
/// HIPAR occupiedBandwidth/Defines the percentage of the total power used to determine the
/// HIPAR occupiedBandwidth/occupied bandwidth.
/// HIPAR noiseReject/Defines the threshold beneath which values are rejected as noise.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumSettings(ViSession instrSession,
                                                    ViInt32   measurement,
                                                    ViInt32   ndBDown,
                                                    ViReal64  channelBandwidth,
                                                    ViReal64  channelCenterFrequency,
                                                    ViReal64  occupiedBandwidth,
                                                    ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_N_DB_DOWN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_N_DB_DOWN, ndBDown),
            3, "N dB Down");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_CHANNEL_POWER_BANDWIDTH, channelBandwidth),
            4, "Channel Bandwidth");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_CHANNEL_POWER_CENTER_FREQUENCY, channelCenterFrequency),
            5, "Channel Center Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_OCCUPIED_BANDWIDTH, occupiedBandwidth),
            6, "Occupied Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a relative threshold, the minimum level vue by
/// HIFN  which the waveform must rise or fl to be considered as a peak. To
/// HIFN  avoid identifying noise peaks, enter a peak excursion vue that is
/// HIFN  higher th the noise levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR peakExcursion/Defines a relative threshold, the minimum level vue by which the
/// HIPAR peakExcursion/waveform must rise or fl to be considered as a peak. To avoid
/// HIPAR peakExcursion/identifying noise peaks, enter a peak excursion vue that is higher
/// HIPAR peakExcursion/th the noise levels.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumPeakExcursion(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   peakExcursion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_PEAK_EXCURSION,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_PEAK_EXCURSION, peakExcursion),
            3, "Peak Excursion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables limit checking for spectrum measurements in the
/// HIFN  specified measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR limitCheck/Enables or disables limit checking for spectrum measurements in the
/// HIPAR limitCheck/specified measurement channel.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumLimitCheck(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViInt32   measurementType,
                                                      ViInt32   limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_TYPE_CPOW, RSSCOPE_VAL_MEAS_TYPE_THDF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LIMIT_CHECK,
            "Measurement", measurement, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LIMIT_CHECK,
            "SpectrumMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_LIMIT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper limit for limit checking, respectively.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR lowerLimit/Define the lower limit for limit checking.
/// HIPAR upperLimit/Define the upper limit for limit checking.
/// HIPAR measurementType/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumLimitCheckLimits(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViInt32   measurementType,
                                                            ViReal64  lowerLimit,
                                                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_TYPE_CPOW, RSSCOPE_VAL_MEAS_TYPE_THDF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_VALUE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_VALUE,
            "SpectrumMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_VALUE, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_UPPER_LIMIT_VALUE, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the lower and upper margins for the limit check, respectively.
/// HIFN  Margins are not as strict as limits and must be within the valid value
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR lowerMargin/Define the lower margin for the limit check. Margins are not as strict
/// HIPAR lowerMargin/as limits and must be within the valid value range.
/// HIPAR upperMargin/Define the upper margins for the limit check. Margins are not as
/// HIPAR upperMargin/strict as limits and must be within the valid value range.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumLimitCheckMargins(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementType,
                                                             ViReal64  lowerMargin,
                                                             ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_TYPE_CPOW, RSSCOPE_VAL_MEAS_TYPE_THDF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_MARGIN,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_MARGIN,
            "SpectrumMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_LOWER_LIMIT_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_UPPER_LIMIT_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum number of measurement results that are listed in the
/// HIFN  result table. Available for peak list and harmonic search mesurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be read.
/// HIPAR maxNumberOfResults/Sets the maximum number of measurement results that are listed in the
/// HIPAR maxNumberOfResults/result table. Available for peak list and harmonic search mesurements.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumMaxNumberOfResults(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViInt32   maxNumberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_TYPE_CPOW, RSSCOPE_VAL_MEAS_TYPE_THDF),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_MAX_NUMBER_OF_RESULTS,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_MAX_NUMBER_OF_RESULTS,
            "SpectrumMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Measurement Type");

    strcpy(repCap, repCap_m);
    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_MAX_NUMBER_OF_RESULTS, maxNumberOfResults),
            4, "Max Number Of Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the way the measurement results are displayed. Available only
/// HIFN  for peak list and harmonic search measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects measurement to be configured. Only peak list or harmonic
/// HIPAR measurementType/search measurements may be configured with this command.
/// HIPAR mode/Sets the way the measurement results are displayed. Available only
/// HIPAR mode/if RSSCOPE_ATTR_SPECTRUM_MAIN_MEASUREMENT is set to PLISt or HAR.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultMode(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViInt32   measurementType,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    strcat(repCap, ",");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSCOPE_VAL_MEAS_TYPE_PLIS, RSSCOPE_VAL_MEAS_TYPE_HAR),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE,
            "SpectrumMeasurementType", measurementType, RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Measurement Type");
    strcat(repCap, repCap_type);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_RESULT_MODE, mode),
            4, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an absolute threshold as an additional condition for the peak
/// HIFN  search. Only peaks that exceed the threshold are detected.
/// HIFN     This setting is only available for spectrum waveforms. It is valid for
/// HIFN  cursor measurements, spectrum measurements and peak search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SPECTRUM_ABSOLUTE_THRESHOLD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:SPECtrum:ATHReshold
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR absoluteThreshold/Defines an absolute threshold as an additional condition for the peak
/// HIPAR absoluteThreshold/search. Only peaks that exceed the threshold are detected.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAbsoluteThreshold(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViReal64  absoluteThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ABSOLUTE_THRESHOLD,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ABSOLUTE_THRESHOLD, absoluteThreshold),
            3, "Absolute Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of spectrum arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR measuerementType/Sets the type of spectrum arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticParameterMeasuerementType(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   measuerementType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUEREMENT_TYPE, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUEREMENT_TYPE, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUEREMENT_TYPE, measuerementType),
            4, "Measurement Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the amplitude/time arithmetics measurements for the measurment
/// HIFN  group.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_AMPLITUDE_TIME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:SPECtrum:ARIThmetic:PARameter<n>:AMPTime
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR operand/Selects the amplitude/time arithmetics measurements for the measurment
/// HIPAR operand/group.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticParameterOperand(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   operand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_AMPLITUDE_TIME, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_AMPLITUDE_TIME, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_AMPLITUDE_TIME, operand),
            4, "Operand");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the measurement group for the second operand of the spectrum
/// HIFN  arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR measurementGroup/Sets the measurement group for the second operand of the spectrum
/// HIPAR measurementGroup/arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticParameterGroup(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   measurementGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_GROUP, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_GROUP, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_GROUP, measurementGroup),
            4, "Measueremnt Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the first operand for the sepctrum arithmetics measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR parameter/Selects repeated capability ArithmeticParameter.
/// HIPAR spectrum/Selects the first operand for the sepctrum arithmetics measurements.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticParameterMeasurementSpectrum(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   parameter,
                            ViInt32   spectrum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMeas[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;
    ViChar   repCapParm[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  parmCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ArithmeticParameter", &parmCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, 1, parmCount),
            3, "Parameter");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCapMeas),
                2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, "ArithmeticParameter", parameter - 1, RS_REPCAP_BUF_SIZE, repCapParm),
                3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMeas, repCapParm);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_PARAMETER_MEASUREMENT_SPECTRUM, spectrum),
            4, "Spectrum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the operation type to be performed on the selected operands
/// HIFN  for the spectrum arithmetic measurement.
/// HIFN     MEASurement<m>:SPECtrum:ARIThmetic:OPERator
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR operator/Selects the operation type to be performed on the selected operands
/// HIPAR operator/for the spectrum arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticOperator(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   arithmeticOperator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_OPERATOR,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_OPERATOR, arithmeticOperator),
            3, "ArithmeticOperator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the numeric value and constant unit as the second operand
/// HIFN  of the spectrum arithmetic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR constant/Sets a constant numeric value as the second operand of the spectrum
/// HIPAR constant/arithmetic measurement.
/// HIPAR constantUnit/Selects the operation type to be performed on the selected operands
/// HIPAR constantUnit/for the amplitude/time arithmetic measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticConstantSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  constant,
                            ViInt32   constantUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CONSTANT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CONSTANT, constant),
            3, "Constant");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CONSTANT_UNIT, constantUnit),
            4, "Constant Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the user-defined custom name for the spectrum arithmetic
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR state/Enables the use of user-defined custom name for the spectrum
/// HIPAR state/arithmetic measurement.
/// HIPAR name/reserved for future use
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticCustomNameSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean state,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_NAME_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_NAME_ENABLED, state),
            3, "State");

    name;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the user-defined unit settings for the spectrum arithmetic
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects repeated capability Measurement.
/// HIPAR state/Enables the use of user-defined custom unit for the amplitude/time
/// HIPAR state/arithmetic measurement. You can set the unit with
/// HIPAR state/RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_UNIT.
/// HIPAR unit/Sets a user-defined value of the unit for the spectrum arithmetic
/// HIPAR unit/measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumArithmeticCustomUnitSettings(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViBoolean state,
                            ViString  unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_UNIT_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_UNIT_ENABLED, state),
            3, "State");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ARITHMETIC_CUSTOM_UNIT, unit),
            4, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the unit for the optical measurements.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR unit/Selects the unit for the optical measurements.
ViStatus _VI_FUNC rsscope_ConfigureOpticalMeasurementUnit(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_UNIT,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, measurementRepCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_UNIT, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the start and stop value of the range for the optical
///HINF  modulation amplitude.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR start/Sets the start value of the range for the optical
///HIPAR start/modulation amplitude.
///HIPAR stop/Sets the end value of the range for the optical
///HIPAR stop/modulation amplitude.
ViStatus _VI_FUNC rsscope_ConfigureOpticalMeasurementModulationAmplitude(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_MODULATION_AMPLITUDE_START,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_MODULATION_AMPLITUDE_START, start),
            3, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, measurementRepCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_MODULATION_AMPLITUDE_STOP, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the condition for the limit check of the optical
///HINF  measurement.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR opticalMeasurement/Selects optical measurement.
///HIPAR validRange/Selects the condition for the limit check of the
///HIPAR validRange/optical measurement.
ViStatus _VI_FUNC rsscope_ConfigureOpticalMeasurementValidRange(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   opticalMeasurement,
                            ViInt32   validRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   opticalMeasurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;
    ViInt32  opticalMeasurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_VALID_RANGE,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "OpticalMeasurement", &opticalMeasurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, opticalMeasurement, 1, opticalMeasurementCount),
            3, "Optical Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_VALID_RANGE,
        "OpticalMeasurement", opticalMeasurement - 1, RS_REPCAP_BUF_SIZE, opticalMeasurementRepCap), 3 ,"Optical Measurement");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", measurementRepCap, opticalMeasurementRepCap);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_VALID_RANGE, validRange),
            4, "Valid Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Define the lower/ upper limit for limit checking of the
///HINF  optical measurements.
///HIRET Returns the status code of this operation.
///HIPAR measurement/Selects waveform measurement.
///HIPAR opticalMeasurement/Selects optical measurement.
///HIPAR lowerLimit/Define the lower limit for limit checking of the
///HIPAR lowerLimit/optical measurements.
///HIPAR upperLimit/Define the upper limit for limit checking of the
///HIPAR upperLimit/optical measurements.
ViStatus _VI_FUNC rsscope_ConfigureOpticalMeasurementLimitCheckLimits(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   opticalMeasurement,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   opticalMeasurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;
    ViInt32  opticalMeasurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_LIMIT,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "OpticalMeasurement", &opticalMeasurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, opticalMeasurement, 1, opticalMeasurementCount),
            3, "Optical Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_LIMIT,
        "OpticalMeasurement", opticalMeasurement - 1, RS_REPCAP_BUF_SIZE, opticalMeasurementRepCap), 3 ,"Optical Measurement");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", measurementRepCap, opticalMeasurementRepCap);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_LIMIT, lowerLimit),
            4, "Lower Limit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_UPPER_LIMIT, upperLimit),
            5, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Define the lower/ upper margin for limit checking of the
///HINF  optical measurements.
///HIRET Returns the status code of this operation.
///HIPAR lowerMargin/Define the lower margin for limit checking of the
///HIPAR lowerMargin/optical measurements.
///HIPAR measurement/Selects waveform measurement.
///HIPAR opticalMeasurement/Selects optical measurement.
///HIPAR upperMargin/Define the upper margin for limit checking of the
///HIPAR upperMargin/optical measurements.
ViStatus _VI_FUNC rsscope_ConfigureOpticalMeasurementLimitCheckMargins(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   opticalMeasurement,
                            ViReal64  lowerMargin,
                            ViReal64  upperMargin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   measurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   opticalmeasurementRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  measurementCount = 0;
    ViInt32  opticalmeasurementCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &measurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, measurementCount),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_MARGIN,
        "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, measurementRepCap), 2 ,"Measurement");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "OpticalMeasurement", &opticalmeasurementCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, opticalMeasurement, 1, opticalmeasurementCount),
            3, "OpticalMeasurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_MARGIN,
        "OpticalMeasurement", opticalMeasurement - 1, RS_REPCAP_BUF_SIZE, opticalmeasurementRepCap), 3 ,"OpticalMeasurement");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", measurementRepCap, opticalmeasurementRepCap);

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_LOWER_MARGIN, lowerMargin),
            4, "Lower Margin");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_OPTICAL_MEASUREMENT_UPPER_MARGIN, upperMargin),
            5, "Upper Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a description for each detected peak in the spectrum diagram.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:RESult:SHLabels
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the measurement type.
/// HIPAR state/Displays a description for each detected peak in the spectrum diagram.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultLabelsState(ViSession instrSession,
                                                             ViInt32   measurement,
                                                             ViInt32   measurementType,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumResultMeasType, measurementType, &p2string),
                3, "Measurement Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:SHL %s,%s", measurement, p2string, state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the layout of the labels.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:RESult:LABorder
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the measurement type.
/// HIPAR frameType/Defines the layout of the labels.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultLabelsFrameType(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2strFT;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumResultMeasType, measurementType, &p2string),
                3, "Measurement Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngSpectrumFrameType, frameType, &p2strFT),
                4, "Frame Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:LAB %s,%s", measurement, p2string, p2strFT);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays labels with black font on white background using the Full
/// HIFN  frame label type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:RESult:INVerse
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the measurement type.
/// HIPAR inverse/Displays labels with black font on white background using the Full
/// HIPAR inverse/frame label type.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultLabelsInverse(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViBoolean inverse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumResultMeasType, measurementType, &p2string),
                3, "Measurement Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:INV %s,%s", measurement, p2string, inverse ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum number of results that are returned for the
/// HIFN  specified measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:RESult:MAXCount
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the measurement type.
/// HIPAR maximumPeaks/Defines the maximum number of results that are returned for the
/// HIPAR maximumPeaks/specified measurement.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultLabelsMaximumPeaks(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViInt32   maximumPeaks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumResultMeasType, measurementType, &p2string),
                3, "Measurement Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:MAXC %s,%ld", measurement, p2string, maximumPeaks);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Includes the frequency of the detected peak in the diagram labels.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASurement<m>:RESult:SHFRequency
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementType/Selects the measurement type.
/// HIPAR showFrequency/Includes the frequency of the detected peak in the diagram labels.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumResultLabelsShowFrequency(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementType,
                            ViBoolean showFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumResultMeasType, measurementType, &p2string),
                3, "Measurement Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:SHFR %s,%s", measurement, p2string,
             showFrequency ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to
/// HIFN  if the measurement waveform is used as a source for math calculations.
/// HIFN  The main measurement type must belong to the same category as the other
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureHistogramMainMeasurement(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViInt32   measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HISTOGRAM_MAIN_MEASUREMENT,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HISTOGRAM_MAIN_MEASUREMENT, measurementFunction),
            3, "Histogram Measurement Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines additional measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR enabled/Enables or disables the additional measurement.
ViStatus _VI_FUNC rsscope_ConfigureHistogramAdditionalMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngHistogramMeasurementType, measurementFunction, &
                    p2string),
                3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:ADD %s,%s", measurement, p2string, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the test results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects mask to be de/activates.
/// HIPAR acquisitionsCompleted/Returns the number of tested acquisitions.
/// HIPAR acquisitionsRemaining/Returns the remaining acquisitions until "Average count / Nx Single count" is reached.
/// HIPAR state/Returns whether the test is running or has finished. The state is set to "Finished" if no acquisitions remain.
/// HIPAR sampleHits/Returns the number of sample hits that violated the mask.
/// HIPAR acquisitionHits/Returns the number of acquisitions that contained at least one sample hit.
/// HIPAR failRate/Returns the ratio of acquisition hits to the number of tested acquisitions.
/// HIPAR testResult/Returns the test result.
ViStatus _VI_FUNC rsscope_QueryMaskTestResults(ViSession instrSession,
                                               ViString  maskName,
                                               ViUInt32* acquisitionsCompleted,
                                               ViUInt32* acquisitionsRemaining,
                                               ViInt32*  state,
                                               ViUInt32* sampleHits,
                                               ViUInt32* acquisitionHits,
                                               ViUInt32* failRate,
                                               ViInt32*  testResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   xxstr[16] = "";
    ViInt32  intValue;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:COUN:WAV? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *acquisitionsCompleted = (ViUInt32)intValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:COUN:REM? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *acquisitionsRemaining = (ViUInt32)intValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:STAT? '%s'", maskName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 16, xxstr));
    *state = 0;
    if (strncmp(xxstr, "RUNN", 4) == 0) *state = 1;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:COUN:FAIL? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *sampleHits = intValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:COUN:FWAV? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *acquisitionHits = intValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:FRAT? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *failRate = intValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:RES:RES? '%s'", maskName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, &intValue));
    *acquisitionsRemaining = intValue;
    checkErr(rsscope_QueryViString(instrSession, cmd, 16, xxstr));
    *testResult = 0;
    if (strncmp(xxstr, "PASS", 4) == 0) *testResult = 1;
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures what happens when the mask test has failed or when it has passed successfully. Each action can be initiated either on failure or on success.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR event/This Control selects event to be performed.
/// HIPAR action/This Control selects action for specified event.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestActionsOnEvent(ViSession instrSession,
                                                          ViString  maskName,
                                                          ViInt32   event,
                                                          ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString event_Arr[] = {"BEEP", "STOP", "PRIN", "SAV"};
    ViString action_Arr[] = {"NOAC", "SUCC", "VIOL", "REP", "RUN", "TRIG"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, event, RSSCOPE_VAL_MASK_TEST_EVENT_BEEP, RSSCOPE_VAL_MASK_TEST_EVENT_TRIG_OUT),
            3, "Event");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, RSSCOPE_VAL_MASK_TEST_ACTION_NOACTION, RSSCOPE_VAL_MASK_TEST_ACTION_VIOLATION),
            4, "Action");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:ONV:%s '%s',%s", event_Arr[event], maskName, action_Arr[action]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new mask or deletes the mask test definition with the specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR action/This Control selects the requested operation.
/// HIPAR maskName/This Control selects mask to be de/activates.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestDefinition(ViSession instrSession,
                                                      ViInt32   action,
                                                      ViString  maskName)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arrEventAttr[] = {RSSCOPE_ATTR_MASK_CREATE,RSSCOPE_ATTR_MASK_DELETE};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, RSSCOPE_VAL_MASK_TEST_ACTION_CREATE, RSSCOPE_VAL_MASK_TEST_ACTION_DELETE),
            2, "Action");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, arrEventAttr[action], maskName),
            3, "Mask Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Converts the test and mask definitions of the current mask test to a
/// HIFN  new mask test of type User.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskName/Converts the test and mask definitions of the current mask test to a
/// HIPAR maskName/new mask test of type User.
ViStatus _VI_FUNC rsscope_ConvertMaskTestToUser(ViSession instrSession,
                                                ViString  maskName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_MASK_CONVERT_TO_USER, maskName),
            2, "Mask Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates and deactivates the mask test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects mask to be de/activates.
/// HIPAR state/This Control activates or deactivates mask test.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestEnable(ViSession instrSession,
                                                  ViString  maskName,
                                                  ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:STAT '%s',%s", maskName, state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the method of mask definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions
/// HIPAR instrSession/function. The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects mask.
/// HIPAR definitionType/This control selects the method of mask definition.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestDefinitionType(ViSession instrSession,
                                                          ViString  maskName,
                                                          ViInt32   definitionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, definitionType, RSSCOPE_VAL_MASK_TEST_TYPE_USER, RSSCOPE_VAL_MASK_TEST_TYPE_PROT),
            3, "Definition Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngMaskTestDefinitionType, definitionType, &p2string),
    3, "Definition Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:CTYP '%s',%s", maskName, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the waveform to be tested against the mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects mask.
/// HIPAR maskSource/This Control selects Mask Source.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSource(ViSession instrSession,
                                                  ViString  maskName,
                                                  ViInt32   maskSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_MASK_NAME, maskName),
            2, "Mask Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MASK_TEST_SOURCE, maskSource),
            3, "Mask Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets two criteria for a failed test,
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects mask.
/// HIPAR failCondition/This Control sets the first criteria for a failed test.
/// HIPAR violationTolerance/This Control sets the second criteria for a failed test, the number of tolerable sample hits or acquisition hits.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSettings(ViSession instrSession,
                                                    ViString  maskName,
                                                    ViInt32   failCondition,
                                                    ViUInt32  violationTolerance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, failCondition, RSSCOPE_VAL_MASK_SAMPLES, RSSCOPE_VAL_MASK_ACQUISITIONS),
            3, "failCondition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:COND '%s',%s", maskName, failCondition == 0 ? "SAMP" : "ACQ");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:TOL '%s',%ld", maskName, violationTolerance);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies a file to save the mask test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR maskTestName/This control specifies the string with the name of the mask test.
/// HIPAR savePath/This control specifies the string containing path and file name,
/// HIPAR savePath/format .xml.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSavePath(ViSession instrSession,
                                                    ViString  maskTestName,
                                                    ViString  savePath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:FILE:NAME '%s','%s'", maskTestName, savePath);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves, opens and deletes specified mask test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR maskTestName/This control specifies the string with the name of the mask test.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestManagerOperations(ViSession instrSession,
                                                             ViInt32   operation,
                                                             ViString  maskTestName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_MASK_TEST_SAVE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MASK_TEST_SAVE, maskTestName),
                3, "Mask Test Name");
        break;

    case RSSCOPE_VAL_MASK_TEST_OPEN:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "",
                        RSSCOPE_ATTR_MASK_TEST_OPEN, maskTestName), 3, "Mask Test Name")
        break;

    case RSSCOPE_VAL_MASK_TEST_DELETE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "",
                        RSSCOPE_ATTR_MASK_TEST_DELETE, maskTestName), 3, "Mask Test Name")
        break;

    case RSSCOPE_VAL_MASK_TEST_CLEAR_ALL:
        checkErr(rsscope_SetAttributeViString(instrSession, "",
            RSSCOPE_ATTR_MASK_TEST_CLEAR_ALL, NULL))
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all mask segments of the indicated mask test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR maskTestName/Deletes all mask segments of the indicated mask test.
ViStatus _VI_FUNC rsscope_ClearMaskTestSegments(ViSession instrSession,
                                                ViString  maskTestName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MASK_TEST_SEGMENT_CLEAR, maskTestName),
            2, "Mask Test Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the region of the segment that builds the mask and enables and disables the mask segment. Disabled segments are not considered by running tests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR maskSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR state/This Control enables and disables the mask segment.
/// HIPAR region/This Control defines the region of the segment that builds the mask and enables and disables the mask segment.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSegments(ViSession instrSession,
                                                    ViString  maskName,
                                                    ViInt32   maskSegment,
                                                    ViBoolean state,
                                                    ViInt32   region)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString region_Arr[] = {"UPPER", "LOWER", "INNER"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, RSSCOPE_VAL_MASK_TEST_UPPER, RSSCOPE_VAL_MASK_TEST_INNER),
            5, "region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:STAT '%s',%ld,%s", maskName, maskSegment, state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:REG '%s',%ld,%s", maskName, maskSegment, region_Arr[region]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of segments in the mask definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR maskSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR segmentsNumber/This Output shows number of segments in the mask definition.
ViStatus _VI_FUNC rsscope_QueryMaskTestSegmentsCount(ViSession instrSession,
                                                     ViString  maskName,
                                                     ViInt32   maskSegment,
                                                     ViInt32*  segmentsNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:COUN? '%s',%ld", maskName, maskSegment);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, segmentsNumber));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function manipulates (creates, removes or inserts) with the specified segment in the mask definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR maskSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR action/This Control selects action to be performed.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSegmentsOperation(ViSession instrSession,
                                                             ViString  maskName,
                                                             ViInt32   maskSegment,
                                                             ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (action)
    {
    case RSSCOPE_VAL_ADD:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:ADD '%s'", maskName);
        break;
    case RSSCOPE_VAL_REMOVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:REM '%s',%ld", maskName, maskSegment);
        break;
    case RSSCOPE_VAL_INSERT:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:INS '%s',%ld", maskName, maskSegment);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 4, "action");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the x-value and y-value of the mask segment point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR maskSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR pointIndex/This Control sets index of the mask segment point in the mask definition.
/// HIPAR xValue/This Control defines the x-value of the mask segment point.
/// HIPAR yValue/This Control defines the y-value of the mask segment point.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSegmentDefinition(ViSession instrSession,
                                                             ViString  maskName,
                                                             ViInt32   maskSegment,
                                                             ViInt32   pointIndex,
                                                             ViReal64  xValue,
                                                             ViReal64  yValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xValue, -100.0E+24, 100.0E+24), 5, "xValue");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, yValue, -100.0E+24, 100.0E+24), 6, "yValue");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:X '%s',%ld,%ld,%.12lG", maskName, maskSegment, pointIndex,
             xValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:Y '%s',%ld,%ld,%.12lG", maskName, maskSegment, pointIndex,
             yValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of defined points for the specified mask segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR pointsNumber/This Output shows number of segments in the mask definition.
/// HIPAR maskSegment/This Control sets index of the mask segment in the mask definition.
ViStatus _VI_FUNC rsscope_QueryMaskTestSegmentDefinitionCount(
                            ViSession instrSession,
                            ViString  maskName,
                            ViInt32   maskSegment,
                            ViInt32*  pointsNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:COUN? '%s',%ld", maskName, maskSegment);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, pointsNumber));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function manipulates (creates, removes or inserts) with the specified point of the mask segment definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR definitionOfSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR pointIndex/This Control sets index of the point in the mask segment in the mask definition.
/// HIPAR action/This Control selects action to be performed.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestSegmentDefinitionOperation(
                            ViSession instrSession,
                            ViString  maskName,
                            ViInt32   definitionOfSegment,
                            ViInt32   pointIndex,
                            ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (action)
    {
    case RSSCOPE_VAL_ADD:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:ADD '%s',%ld", maskName, definitionOfSegment);
        break;
    case RSSCOPE_VAL_REMOVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:REM '%s',%ld,%ld", maskName, definitionOfSegment,
                 pointIndex);
        break;
    case RSSCOPE_VAL_INSERT:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:POIN:INS '%s',%ld,%ld", maskName, definitionOfSegment,
                 pointIndex);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 5, "action");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function moves or stretches the mask segment horizontally/vertically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR definitionOfSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR offsetX/This Control is reserved for future use.
/// HIPAR factorX/This Control defines rate of stretches or compresses the selected mask segment in horizontal direction.
/// HIPAR offsetY/This Control moves the mask segment vertically.
/// HIPAR factorY/This Control defines rate of stretches or compresses the selected mask segment in vertical direction.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestRescale(ViSession instrSession,
                                                   ViInt32   definitionOfSegment,
                                                   ViString  maskName,
                                                   ViReal64  offsetX,
                                                   ViInt32   factorX,
                                                   ViReal64  offsetY,
                                                   ViInt32   factorY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetX, -50, 50), 5, "Offset X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, factorX, -100, 100), 5, "factorX");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetY, -1000.0, 1000.0), 6, "offsetY");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, factorY, -100, 100), 7, "factorY");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:RESC:XOFF '%s',%ld,%.12lG", maskName, definitionOfSegment,
             offsetX);
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:RESC:XFAC '%s',%ld,%ld", maskName, definitionOfSegment, factorX);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:RESC:YOFF '%s',%ld,%.12lG", maskName, definitionOfSegment,
             offsetY);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:RESC:YFAC '%s',%ld,%ld", maskName, definitionOfSegment, factorY);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for mask test violation highlighting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR highlightHits/If ON, the mask hits are highlighted on the screen. You can define the
/// HIPAR highlightHits/color and the time of the hit display.
/// HIPAR infitineHighlight/If TRUE, the mask hits are highlighted for an unlimited period of
/// HIPAR infitineHighlight/time.
/// HIPAR highlightTime/Sets the time how long the mask hits are highlighted. Ignored if
/// HIPAR highlightTime/Infitine Highlight in On.
/// HIPAR matchColor/Sets the color of samples that violated the mask.
ViStatus _VI_FUNC rsscope_ConfigureMaskViolationHighlighting(ViSession instrSession,
                                                             ViBoolean highlightHits,
                                                             ViBoolean infiniteHighlight,
                                                             ViReal64  highlightTime,
                                                             ViInt64   matchColor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_MASK_TEST_HIGHLIGHT_ENDLESS, infiniteHighlight),
            3, "Infinite Highlight");

    if (infiniteHighlight == VI_FALSE)
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_MASK_TEST_HIGHLIGHT_TIME, highlightTime),
                4, "Highlight Time");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_MASK_TEST_COLOR_MATCHED, matchColor),
                5, "Color Match");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_MASK_TEST_HIGHLIGHT_STATE, highlightHits),
            2, "Highlight Hits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the color of masks segments that were not hit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskWithoutViolation/Sets the color of masks segments that were not hit.
/// HIPAR maskWithViolation/Sets the color of mask segments the signal has entered into.
/// HIPAR maskWithContact/Sets the color of masks segments that were touched at the border. In
/// HIPAR maskWithContact/this case, the resolution is not sufficient to detect if the mask was
/// HIPAR maskWithContact/really hit or not. Zoom into the concerned area to see the actual
/// HIPAR maskWithContact/result.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestMaskColors(ViSession instrSession,
                                                      ViInt64   maskWithoutViolation,
                                                      ViInt64   maskWithViolation,
                                                      ViInt64   maskWithContact)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_MASK_TEST_COLOR_UNMATCHED, maskWithoutViolation),
                2, "Mask Without Violation");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_MASK_TEST_COLOR_INTERIOR, maskWithViolation),
                3, "Mask With Violation");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_MASK_TEST_COLOR_BORDER, maskWithContact),
                4, "Mask With Contact");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function creates the upper and lower mask limit from the
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions
/// HIPAR instrSession/function. The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR usedReference/Sets the reference waveform from which the mask is created.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestWaveformCopy(ViSession instrSession,
                                                        ViString  maskName,
                                                        ViInt32   usedReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString reference_arr[] = {"REFERENCE1", "REFERENCE2", "REFERENCE3", "REFERENCE4",NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, usedReference, RSSCOPE_VAL_MASK_TEST_REFERENCE1, RSSCOPE_VAL_MASK_TEST_REFERENCE4),
            3, "Used Reference");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:REFW \"%s\",%s", maskName, reference_arr[usedReference - 1]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MASK_COPY_WAVEFORM, maskName),
            2, "Mask Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function moves or stretches the waveform mask horizontally/vertically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions
/// HIPAR instrSession/function. The handle identifies a particular instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR horizontalWidth/Sets the width of the mask in horizontal direction.
/// HIPAR verticalWidth/Sets the width of the waveform mask in vertical direction.
/// HIPAR verticalStretch/This control sets the vertical scaling to stretch the mask in y-direction.
/// HIPAR verticalPosition/This control moves the mask vertically within the display.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestWaveformRescale(ViSession instrSession,
                                                           ViString  maskName,
                                                           ViReal64  horizontalWidth,
                                                           ViReal64  verticalWidth,
                                                           ViInt32   verticalStretch,
                                                           ViReal64  verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, horizontalWidth, 0, 1000),
            3, "horizontalWidth");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, verticalWidth, 0, 1000),
            4, "verticalWidth");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalStretch, 10, 100),
            5, "verticalStretch");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, verticalPosition, -1000, 1000), 6, "verticalPosition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:WFMR:XWID '%s',%.12lG", maskName, horizontalWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:WFMR:YWID '%s',%.12lG", maskName, verticalWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:WFMR:YSTR '%s',%ld", maskName, verticalStretch);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:WFMR:YPOS '%s',%.12lG", maskName, verticalPosition);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function multiplies and adds the given x- and y-factors and offsets to the coordinates of all points of the selected mask segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or rsscope_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR definitionOfSegment/This Control sets index of the mask segment in the mask definition.
/// HIPAR maskName/This Control selects the mask.
ViStatus _VI_FUNC rsscope_MaskTestRecalculate(ViSession instrSession,
                                              ViInt32   definitionOfSegment,
                                              ViString  maskName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SEGM:RESC:REC '%s',%ld", maskName, definitionOfSegment);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches the display of all mask segments on or off.
///HIRET Returns the status code of this operation.
///HIPAR showLabels/Switches the display of all mask segments on or
///HIPAR showLabels/off.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestLabelState(ViSession instrSession,
                                                      ViBoolean showLabels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_MASK_TEST_SHOW_LABEL_STATE, showLabels),
            2, "Show Labels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches the display of the mask test name on or off.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MASK_TEST_LABEL_STATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:LABel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showName/Switches the display of the mask test name on or off.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestNameState(ViSession instrSession,
                                                     ViBoolean showName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_MASK_TEST_LABEL_STATE, showName),
            2, "Show Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the name of the mask test.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:REName
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oldMaskName/String with the existing mask test name.
/// HIPAR newMaskName/String with the new mask test name.
ViStatus _VI_FUNC rsscope_RenameMaskTest(ViSession instrSession,
                                         ViString  oldMaskName,
                                         ViString  newMaskName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:REN '%s','%s'", oldMaskName, newMaskName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the outline of the eye mask.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Function rsscope_ConfigureMaskTestDefinitionType must be set to
/// HIFN  "Eye".
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:EYEMask:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR shape/This control defines the outline of the eye mask.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestEyeShape(ViSession instrSession,
                                                    ViString  maskName,
                                                    ViInt32   shape)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K12|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, shape, RSSCOPE_VAL_MASK_TEST_EYE_SHAPE_SQUARE, RSSCOPE_VAL_MASK_TEST_EYE_SHAPE_OCTAGON),
            3, "Used Reference");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:TYPE '%s',%s", maskName, eyeShapeArr[shape]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures polygon setup.
/// HIFN
/// HIFN     Note(s):
/// HIFN     Function rsscope_ConfigureMaskTestDefinitionType must be set to "Eye".
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:EYEMask:WIDTh<m>[:VALue]
/// HIFN     MTESt:EYEMask:HEIGht<m>[:VALue]
/// HIFN     MTESt:EYEMask:MSKLeft
/// HIFN     MTESt:EYEMask:MSKRight
/// HIFN     MTESt:EYEMask:HPERiod
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR mainWidth/Main width defines the width af all eye mask shapes.
/// HIPAR mainHeight/Main height defines the heigth af all eye mask shapes.
/// HIPAR minorWidth/Minor width defines the secondary width for hexagon and octagon mask
/// HIPAR minorWidth/shapes.
/// HIPAR minorHeight/Minor height defines the secondary height for octagon mask shapes.
/// HIPAR interpatternLength/Defines the time distance between the shape centers if Copy Left
/// HIPAR interpatternLength/and/or Copy Right are On.
/// HIPAR copyLeft/Copies the eye shape to the left.
/// HIPAR copyRight/Copies the eye shape to the right.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestEyePolygon(ViSession instrSession,
                                                      ViString  maskName,
                                                      ViReal64  mainWidth,
                                                      ViReal64  mainHeight,
                                                      ViReal64  minorWidth,
                                                      ViReal64  minorHeight,
                                                      ViBoolean copyLeft,
                                                      ViReal64  interpatternLength,
                                                      ViBoolean copyRight)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K12|K0"));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, interpatternLength, 0.0, 100.0), 8, "Interpattern Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:WIDT1 '%s',%.12lG", maskName, mainWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:HEIG1 '%s',%.12lG", maskName, mainHeight);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:WIDT2 '%s',%.12lG", maskName, minorWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:HEIG2 '%s',%.12lG", maskName, minorHeight);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:MSKL '%s',%s", maskName, copyLeft ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:MSKR '%s',%s", maskName, copyRight ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    if (copyLeft || copyRight)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:HPER '%s',%.12lG", maskName, interpatternLength);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables configures the upper and/or lower regions of the
/// HIFN  mask.
/// HIFN
/// HIFN     Note(s):
/// HIFN     Function rsscope_ConfigureMaskTestDefinitionType must be set to "Eye".
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:EYEMask:MSKTop
/// HIFN     MTESt:EYEMask:TOFFset
/// HIFN     MTESt:EYEMask:TBWidth
/// HIFN     MTESt:EYEMask:MSKBottom
/// HIFN     MTESt:EYEMask:BOFFset
/// HIFN     MTESt:EYEMask:TBSYmmetric
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR topState/Enable the upper (top) mask region.
/// HIPAR offsetTop/Voltage distance from the eye shape center that limit the upper
/// HIPAR offsetTop/regions.
/// HIPAR width/Time width of the outer regions, symmetric to the eye shape center.
/// HIPAR bottomState/Enable the lower (bottom) mask region.
/// HIPAR offsetBottom/Voltage distance from the eye shape center that limit the lower
/// HIPAR offsetBottom/regions.
/// HIPAR symmetric/Sets bottom and top offsets to the same value so that the outer
/// HIPAR symmetric/regions are symmetric to the eye shape.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestEyeTopBottom(ViSession instrSession,
                                                        ViString  maskName,
                                                        ViBoolean topState,
                                                        ViReal64  offsetTop,
                                                        ViReal64  width,
                                                        ViBoolean bottomState,
                                                        ViReal64  offsetBottom,
                                                        ViBoolean symmetric)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K12|K0"));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetTop, 0.0, 100.0), 4, "Offset Top");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, width, 0.0, 100.0), 5, "Width");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetBottom, 0.0, 100.0), 7, "Offset Bottom");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:MSKT '%s',%s", maskName, topState ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:TOFF '%s',%.12lG", maskName, offsetTop);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:TBW '%s',%.12lG", maskName, width);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:MSKB '%s',%s", maskName, bottomState ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:BOFF '%s',%.12lG", maskName, offsetBottom);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:TBSY '%s',%s", maskName, symmetric ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the horizontal (time) and vertical (voltage) values
/// HIFN  of the eye shape enter and thus define the position of the eye shape on
/// HIFN  the display.
/// HIFN
/// HIFN     Note(s):
/// HIFN     Function rsscope_ConfigureMaskTestDefinitionType must be set to "Eye".
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MTESt:EYEMask:HPOSition
/// HIFN     MTESt:EYEMask:VPOSition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskName/This Control selects the mask.
/// HIPAR horizontal/Sets the horizontal (time) value of the eye shape enter and thus
/// HIPAR horizontal/defines the horizontal position of the eye shape on the display.
/// HIPAR vertical/Sets the vertical (voltage) value of the eye shape enter and thus
/// HIPAR vertical/defines the vertical position of the eye shape on the display.
ViStatus _VI_FUNC rsscope_ConfigureMaskTestEyeCenter(ViSession instrSession,
                                                     ViString  maskName,
                                                     ViReal64  horizontal,
                                                     ViReal64  vertical)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K12|K0"));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, horizontal, -100.0, 100.0), 32, "Horizontal");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, vertical, -100.0, 100.0), 4, "Vertical");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:HPOS '%s',%.12lG", maskName, horizontal);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:EYEM:VPOS '%s',%.12lG", maskName, vertical);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new search definition with the specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies the name of the new search definition.
ViStatus _VI_FUNC rsscope_NewSearch(ViSession instrSession,
                                    ViString  searchName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_NEW_SEARCH, searchName),
            2, "Search Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the specified search definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Deletes the specified search definition.
ViStatus _VI_FUNC rsscope_DeleteSearch(ViSession instrSession,
                                       ViString  searchName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DELETE_SEARCH, searchName),
            2, "Search Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source of the search. The source can be
/// HIFN  any input signal, math or reference waveform. Depending on the selected
/// HIFN  source, not all search conditions are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchConditionSource/Selects the source of the search.
ViStatus _VI_FUNC rsscope_ConfigureSearchSource(ViSession instrSession,
                                                ViString  searchName,
                                                ViInt32   searchConditionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchConditionSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            3, "Search Condition Source");
    if (((RSSCOPE_VAL_MEASUREMENT_SOURCE_D15 < searchConditionSource) &&
            (searchConditionSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_SBUS1)) ||
        ((RSSCOPE_VAL_MEASUREMENT_SOURCE_COMMON2 < searchConditionSource) &&
            (searchConditionSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5)))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchConditionSource), 3, "Search Condition Source");
    }

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_SOURCE, searchConditionSource),
            3, "Search Condition Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns the search definition to a search category.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchCategory/Selects the search category.
ViStatus _VI_FUNC rsscope_ConfigureSearchCategory(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   searchCategory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString searchCategory_Arr[] = {"TRIG", "FREQ"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchCategory, RSSCOPE_VAL_SEARCH_CATEGORY_TRIG, RSSCOPE_VAL_SEARCH_CATEGORY_FREQ),
            3, "Search Category");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:CAT '%s',%s", searchName, searchCategory_Arr[searchCategory]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables a repeated search for each new data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchOnline/If enabled, a search is performed repeatedly for each new data
/// HIPAR searchOnline/acquisition.
ViStatus _VI_FUNC rsscope_ConfigureSearchControl(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViBoolean searchOnline)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:ONL '%s',%s", searchName, (searchOnline == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs a single search on the existing data from the
/// HIFN  selected source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchResults/Selects the search results.
ViStatus _VI_FUNC rsscope_PerformSearch(ViSession instrSession,
                                        ViString  searchName,
                                        ViInt32   searchResults)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchResults, RSSCOPE_VAL_SEARCH_RESULT_NEXT, RSSCOPE_VAL_SEARCH_RESULT_ALL),
            3, "Search Results");

    switch (searchResults)
    {
    case RSSCOPE_VAL_SEARCH_RESULT_NEXT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_NEXT_RESULT, searchName),
                2, "Search Name");
        break;

    case RSSCOPE_VAL_SEARCH_RESULT_ALL:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ALL_RESULTS, searchName),
                2, "Search Name");
        break;

    default: break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears the search results once to start a new search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
ViStatus _VI_FUNC rsscope_ClearSearchResults(ViSession instrSession,
                                             ViString  searchName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CLEAR_SEARCH_RESULTS, searchName),
            2, "Search Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all search results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Pass the number of elements in the Search Results parameter.
/// HIPAR acquisitionIndex/Acquistion index.
/// HIPAR xPosition/ X-position of the search result
/// HIPAR yPosition/Y-position of the search result, currently not relevant
/// HIPAR typeOfSearchResult/Type of the search result.
/// HIPAR slope/Slope or polarity of the search result.
/// HIPAR timeResult/For runt, glitch, width, and window searches, the value contains
/// HIPAR timeResult/the width. For timeout and interval searches, it contains the
/// HIPAR timeResult/timeout. For transition searches, it contains the slew rate. For
/// HIPAR timeResult/all other searches, the value is not relevant.
/// HIPAR timeResult/If a value is not relevant, 9.91E+37 is returned.
/// HIPAR actualValues/Indicates the number of points the function places in the Search
/// HIPAR actualValues/Results parameter.
ViStatus _VI_FUNC rsscope_QuerySearchResults(ViSession instrSession,
                                             ViString  searchName,
                                             ViInt32   bufferSize,
                                             ViInt32   acquisitionIndex[],
                                             ViReal64  xPosition[],
                                             ViReal64  yPosition[],
                                             ViInt32   typeOfSearchResult[],
                                             ViInt32   slope[],
                                             ViReal64  timeResult[],
                                             ViInt32*  actualValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViChar*  data = NULL;
    ViString typeArr[] = {"Edge", "Glitch", "Width", "Runt", "Window", "Timeout", "Interval", "Slew Rate", "Data2Clock"};

    //ViUInt32 count = 0;
    ViInt32 i = 0;


    checkErr(RsCore_LockSession(instrSession));

    sprintf(header, "SEAR:RES:ALL? '%s'", searchName);

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, header, &data));
    checkErr(rsscope_CheckStatus(instrSession));

    p2buf = strtok(data, ",");

    while (p2buf && (i < bufferSize))
    {
        acquisitionIndex[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        if (p2buf != NULL)
            xPosition[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        if (p2buf != NULL)
            yPosition[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        if (p2buf != NULL)
            typeOfSearchResult[i] = RsCore_FindStringIndex(typeArr, p2buf);
        p2buf = strtok(NULL, ",");

        if (p2buf != NULL)
            slope[i] = RsCore_FindStringIndex(polarityArr, p2buf);
        p2buf = strtok(NULL, ",");

        if (p2buf != NULL)
            timeResult[i++] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");
    }
    *actualValues = i;

Error:
    if (data)
        free(data);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies the trigger event configuration from Trigger A or
/// HIFN  B for the selected channel source to the search condition settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerType/Sets the trigger type.
/// HIPAR copyFrom/Selects the trigger event configuration to copy from.
ViStatus _VI_FUNC rsscope_SearchTriggerCopyConfiguration(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   triggerType,
                                                         ViInt32   copyFrom)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr arrCopyFromA[] = {
             RSSCOPE_ATTR_SEARCH_TRIGGER_EDGE_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_GLITCH_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_TIMEOUT_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_COPY_FROM_A,
             RSSCOPE_ATTR_SEARCH_TRIGGER_STATE_COPY_FROM_A
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerType, RSSCOPE_VAL_EDGE_TRIGGER, RSSCOPE_VAL_STATE_TRIGGER),
            3, "Trigger Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    switch (copyFrom)
    {
    case RSSCOPE_VAL_COPY_FROM_A:
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, arrCopyFromA[triggerType], NULL),
                4, "Copy From");
        break;

    case RSSCOPE_VAL_COPY_FROM_B:
        if (triggerType == RSSCOPE_VAL_EDGE_TRIGGER)
        {
            viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_EDGE_COPY_FROM_B, NULL),
                    4, "Copy From");
        }
        else
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, copyFrom), 4, "Copy From");
        }
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, copyFrom), 4, "Copy From");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the edge trigger search.
/// HIFN  An edge trigger occurs when the trigger signal passes through the
/// HIFN  voltage threshold that you specify with the Trigger Level parameter and
/// HIFN  has the slope that you specify with the Trigger Slope parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the edge trigger event type in the
/// HIPAR triggerState/next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR triggerLevel/Sets the voltage level for the trigger level that is used to determine
/// HIPAR triggerLevel/other parameters.
/// HIPAR triggerSlope/Selects the edge type.
ViStatus _VI_FUNC rsscope_ConfigureEdgeTriggerSearch(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean triggerState,
                                                     ViInt32   searchSource,
                                                     ViReal64  triggerLevel,
                                                     ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_EDGE_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, triggerLevel),
            5, "Trigger Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_EDGE_SLOPE, triggerSlope),
            6, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the glitch trigger search.
/// HIFN   A glitch trigger occurs when the trigger signal has a pulse with a
/// HIFN  width that is less than the glitch width.  You specify the glitch width
/// HIFN  in the Glitch Width parameter.  You specify the polarity of the pulse
/// HIFN  with the Glitch Polarity parameter.  The trigger does not actually
/// HIFN  occur until the edge of a pulse that corresponds to the Glitch Width
/// HIFN  and Glitch Polarity crosses the threshold you specify in the Trigger
/// HIFN  Level parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the glitch trigger event type in
/// HIPAR triggerState/the next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR triggerLevel/Sets the voltage level for the trigger level that is used to determine
/// HIPAR triggerLevel/other parameters.
/// HIPAR glitchWidth/Sets the length of a glitch. The instrument triggers on pulses shorter
/// HIPAR glitchWidth/or longer than this value.
/// HIPAR glitchPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR glitchPolarity/pulse slope.
/// HIPAR glitchCondition/Selects which glitches are identified: shorter or longer than the
/// HIPAR glitchCondition/width.
ViStatus _VI_FUNC rsscope_ConfigureGlitchTriggerSearch(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViBoolean triggerState,
                                                       ViInt32   searchSource,
                                                       ViReal64  triggerLevel,
                                                       ViReal64  glitchWidth,
                                                       ViInt32   glitchPolarity,
                                                       ViInt32   glitchCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_GLITCH_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, triggerLevel),
                5, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_GLITCH_WIDTH, glitchWidth),
                6, "Glitch Width");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_GLITCH_POLARITY, glitchPolarity),
                7, "Glitch Polarity");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_GLITCH_CONDITION, glitchCondition),
                8, "Glitch Condition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the width trigger search.
/// HIFN  A width trigger occurs when the oscilloscope detects a positive or
/// HIFN  negative pulse with a width between, or optionally outside, the width
/// HIFN  thresholds.  You specify the width thresholds with the Width Low
/// HIFN  Threshold and Width High Threshold parameters.  You specify whether the
/// HIFN  oscilloscope triggers on pulse widths that are within or outside the
/// HIFN  width thresholds with the Width Condition parameter.  You specify the
/// HIFN  polarity of the pulse with the Width Polarity parameter.  The trigger
/// HIFN  does not actually occur until the edge of a pulse that corresponds to
/// HIFN  the Width Low Threshold, Width High Threshold, Width Condition, and
/// HIFN  Width Polarity crosses the threshold you specify in the Trigger Level
/// HIFN  parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the width trigger event type in the
/// HIPAR triggerState/next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR triggerLevel/Sets the voltage level for the trigger level that is used to determine
/// HIPAR triggerLevel/other parameters.
/// HIPAR width/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR width/a range which is defined by the limits "+/-Delta". For the ranges
/// HIPAR width/"Shorter" and "Longer", the width defines the maximum and minimum pulse
/// HIPAR width/width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR widthRange/Sets how the measured pulse width is compared with the given limit(s).
ViStatus _VI_FUNC rsscope_ConfigureWidthTriggerSearch(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViBoolean triggerState,
                                                      ViInt32   searchSource,
                                                      ViReal64  triggerLevel,
                                                      ViReal64  width,
                                                      ViReal64  widthDelta,
                                                      ViInt32   widthPolarity,
                                                      ViInt32   widthRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, triggerLevel),
                5, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH, width),
                6, "Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH_DELTA, widthDelta),
                7, "Width Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH_POLARITY, widthPolarity),
                8, "Width Polarity");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WIDTH_RANGE, widthRange),
                9, "Width Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the runt trigger search.
/// HIFN  A runt trigger occurs when the trigger signal crosses one of the runt
/// HIFN  thresholds twice without crossing the other runt threshold.  You
/// HIFN  specify the runt thresholds with the  Runt Low Threshold and Runt High
/// HIFN  Threshold parameters.  You specify the polarity of the runt with the
/// HIFN  Runt Polarity parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the runt trigger event type in the
/// HIPAR triggerState/next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR runtLowThreshold/Sets the lower voltage threshold.
/// HIPAR runtHighThreshold/Sets the upper voltage threshold.
/// HIPAR runtPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR runtPolarity/pulse slope.
ViStatus _VI_FUNC rsscope_ConfigureRuntTriggerSearch(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean triggerState,
                                                     ViInt32   searchSource,
                                                     ViReal64  runtLowThreshold,
                                                     ViReal64  runtHighThreshold,
                                                     ViInt32   runtPolarity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_LOWER_LEVEL, runtLowThreshold),
                5, "Runt Low Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_UPPER_LEVEL, runtHighThreshold),
                6, "Runt High Threshold");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_POLARITY, runtPolarity),
                7, "Runt Polarity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Runt trigger search range and condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR runtWidth/For the ranges "Shorter" and "Longer", the runt width defines the
/// HIPAR runtWidth/maximum and minimum pulse width, respectively.
/// HIPAR runtDelta/Defines a range around the runt width specified using Runt Width. This
/// HIPAR runtDelta/control is only available if Runt Range is set to Within or Outside
/// HIPAR runtRange/Selects how the time limit of the runt pulse is defined based on the
/// HIPAR runtRange/runt width and delta
ViStatus _VI_FUNC rsscope_ConfigureRuntTriggerSearchRange(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViReal64  runtWidth,
                                                          ViReal64  runtDelta,
                                                          ViInt32   runtRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_WIDTH, runtWidth),
            3, "Runt Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_DELTA, runtDelta),
            4, "Runt Delta");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_RUNT_RANGE, runtRange),
            5, "Runt Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the window trigger search.
/// HIFN  The window trigger checks the signal run in relation to a "window". The
/// HIFN  window is formed by the upper and lower voltage levels. The event
/// HIFN  condition is fulfilled, if the waveform enters or leaves the window, or
/// HIFN  if the waveform stays inside or outside for a time longer or shorter
/// HIFN  than specified. With the window trigger, you can display longer
/// HIFN  transient effects.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the window trigger event type in
/// HIPAR triggerState/the next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR windowLowerLimit/Sets the lower voltage limit for the window.
/// HIPAR windowUpperLimit/Sets the upper voltage limit for the window.
ViStatus _VI_FUNC rsscope_ConfigureWindowTriggerSearch(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViBoolean triggerState,
                                                       ViInt32   searchSource,
                                                       ViReal64  windowLowerLimit,
                                                       ViReal64  windowUpperLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_LOWER_LEVEL, windowLowerLimit),
                5, "Window Lower Limit");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_UPPER_LEVEL, windowUpperLimit),
                6, "Window Upper Limit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Window trigger search range and condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR verticalCondition/Defines the signal run in relation to the window.
/// HIPAR timeCondition/Selects how the time limit of the window is defined. Time conditioning
/// HIPAR timeCondition/is available for the vertical conditions "WITHin" and "OUTSide".
/// HIPAR windowWidth/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR windowWidth/a time range which is defined by the limits "Delta".
/// HIPAR windowDelta/Defines a range around the "Width" value.
ViStatus _VI_FUNC rsscope_ConfigureWindowTriggerSearchRange(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   verticalCondition,
                                                            ViInt32   timeCondition,
                                                            ViReal64  windowWidth,
                                                            ViReal64  windowDelta)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_VERTICAL_CONDITION, verticalCondition),
            3, "Vertical Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_TIME_CONDITION, timeCondition),
            4, "Time Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_WIDTH, windowWidth),
            5, "Window Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_WINDOW_DELTA, windowDelta),
            6, "Window Delta");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the timeout trigger
/// HIFN  search. The timeout trigger event checks if the signal stays above or
/// HIFN  below the threshold voltage for a specified time lapse. In other words,
/// HIFN  the event occurs if the trigger source does not have the expected
/// HIFN  transition within the specified time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the timeout trigger event type in
/// HIPAR triggerState/the next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR timeoutValue/Defines the time limit for the timeout at which the instrument
/// HIPAR timeoutValue/triggers.
/// HIPAR timeoutCondition/Defines the relation of the signal level to the trigger level.
ViStatus _VI_FUNC rsscope_ConfigureTimeoutTriggerSearch(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViBoolean triggerState,
                                                        ViInt32   searchSource,
                                                        ViReal64  triggerLevel,
                                                        ViReal64  timeoutValue,
                                                        ViInt32   timeoutCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_TIMEOUT_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, triggerLevel),
                5, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_TIMEOUT_VALUE, timeoutValue),
                6, "Timeout Value");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_TIMEOUT_CONDITION, timeoutCondition),
                7, "Timeout Condition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the interval trigger
/// HIFN  search. The interval trigger analyzes the time between two pulses.
/// HIFN  While the interval trigger can only analyze either positive or negative
/// HIFN  polarity, searching for an interval is also possible for both
/// HIFN  polarities at the same time ("Either").
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the interval trigger event type in
/// HIPAR triggerState/the next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR intervalWidth/Defines the time between two pulses.
/// HIPAR intervalDelta/Defines a range around the "Interval width" value.
/// HIPAR intervalPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR intervalPolarity/pulse slope.
/// HIPAR intervalCondition/Defines the range of an interval in relation to the interval width.
ViStatus _VI_FUNC rsscope_ConfigureIntervalTriggerSearch(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean triggerState,
                                                         ViInt32   searchSource,
                                                         ViReal64  triggerLevel,
                                                         ViReal64  intervalWidth,
                                                         ViReal64  intervalDelta,
                                                         ViInt32   intervalPolarity,
                                                         ViInt32   intervalCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, triggerLevel),
            5, "Trigger Level");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_WIDTH, intervalWidth),
                6, "Interval Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_DELTA, intervalDelta),
                7, "Interval Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_POLARITY, intervalPolarity),
                8, "Interval Polarity");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_INTERVAL_CONDITION, intervalCondition),
                9, "Interval Condition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the slew rate trigger
/// HIFN  search. The slew rate trigger, also known as transition trigger, can
/// HIFN  detect fast or slow edges selectively. It triggers on edges, if the
/// HIFN  transition time from the lower to higher voltage level (or vice versa)
/// HIFN  is shorter or longer as defined, or outside a specified time range. The
/// HIFN  trigger event finds slew rates faster than expected or permissible to
/// HIFN  avoid overshooting and other interfering effects. It also detects very
/// HIFN  slow edges violating the timing in pulse series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the slew rate trigger event type in
/// HIPAR triggerState/the next search.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR slewRateLowerLimit/Defines the lower voltage threshold. When the signal crosses this
/// HIPAR slewRateLowerLimit/level, the slew rate measurement starts or stops depending on the
/// HIPAR slewRateLowerLimit/selected slope.
/// HIPAR slewRateUpperLimit/Defines the upper voltage threshold. When the signal crosses this
/// HIPAR slewRateUpperLimit/level, the slew rate measurement starts or stops depending on the
/// HIPAR slewRateUpperLimit/selected slope.
ViStatus _VI_FUNC rsscope_ConfigureSlewRateTriggerSearch(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean triggerState,
                                                         ViInt32   searchSource,
                                                         ViReal64  slewRateLowerLimit,
                                                         ViReal64  slewRateUpperLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");
    if (triggerState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_LOWER_LEVEL, slewRateLowerLimit),
                5, "Slew Rate Lower Limit");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_UPPER_LEVEL, slewRateUpperLimit),
                6, "Slew Rate Upper Limit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures slew rate trigger search range and condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR slewRate/For the ranges "Within" and "Outside", the slew rate defines the
/// HIPAR slewRate/center of a range which is defined by the limits "Delta".
/// HIPAR slewRateDelta/Defines a time range around the slew rate.
/// HIPAR slewRateCondition/Selects how the time limit for the slew rate is defined. The time
/// HIPAR slewRateCondition/measurement starts when the signal crosses the first trigger level -
/// HIPAR slewRateCondition/the upper or lower level depending on the selected slope - and stops
/// HIPAR slewRateCondition/when the signal crosses the second level.
/// HIPAR slewRatePolarity/Selects the edge type for the trigger event.
ViStatus _VI_FUNC rsscope_ConfigureSlewRateTriggerSearchRange(
                            ViSession instrSession,
                            ViString  searchName,
                            ViReal64  slewRate,
                            ViReal64  slewRateDelta,
                            ViInt32   slewRateCondition,
                            ViInt32   slewRatePolarity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_VALUE, slewRate),
            3, "Slew Rate");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_DELTA, slewRateDelta),
            4, "Slew Rate Delta");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_CONDITION, slewRateCondition),
            5, "Slew Rate Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_SLEW_RATE_POLARITY, slewRatePolarity),
            6, "Slew Rate Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the data 2 clock trigger
/// HIFN  search. With the Data2Clock event - also known as setup/hold - you can
/// HIFN  analyze the relative timing between two signals: a data signal and the
/// HIFN  synchronous clock signal. Many systems require, that the data signal
/// HIFN  must be steady for some time before and after the clock edge, for
/// HIFN  example, the data transmission on parallel interfaces. With this
/// HIFN  trigger type, you can also test the time correlation of sideband and
/// HIFN  inband signals. The event occurs if the data signal crosses the data
/// HIFN  level during the setup and hold time. The reference point for the time
/// HIFN  measurement is defined by clock level and clock edge.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the data2clock trigger event type
/// HIPAR triggerState/in the next search.
/// HIPAR clockEdge/Sets the edge of the clock signal to define the time reference point
/// HIPAR clockEdge/for the setup and hold time.
/// HIPAR dataLevel/Sets the trigger level for the specified event and source.
/// HIPAR searchSource/Selects the source of the search.
/// HIPAR holdTime/Sets the minimum time after the clock edge while the data signal must
/// HIPAR holdTime/stay steady above or below the data level. The hold time can be
/// HIPAR holdTime/negative. In this case, the setup time is always positive. The
/// HIPAR holdTime/setup/hold interval starts before the clock edge (setup time) and ends
/// HIPAR holdTime/before the clock edge (hold time). If you change the negative hold
/// HIPAR holdTime/time, the setup time is adjusted by the intrument.
/// HIPAR setupTime/Sets the minimum time before the clock edge while the data signal must
/// HIPAR setupTime/stay steady above or below the data level. The setup time can be
/// HIPAR setupTime/negative. In this case, the hold time is always positive. The
/// HIPAR setupTime/setup/hold interval starts after the clock edge (setup time) and ends
/// HIPAR setupTime/after the clock edge (hold time). If you change the negative setup
/// HIPAR setupTime/time, the hold time is adjusted by the intrument.
/// HIPAR clockLevel/Sets the voltage level for the clock signal.
/// HIPAR clockSource/Selects the waveform used for the clock signal.
ViStatus _VI_FUNC rsscope_ConfigureData2ClockTriggerSearch(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViBoolean triggerState,
                                                           ViInt32   searchSource,
                                                           ViReal64  dataLevel,
                                                           ViInt32   clockSource,
                                                           ViReal64  clockLevel,
                                                           ViReal64  holdTime,
                                                           ViReal64  setupTime,
                                                           ViInt32   clockEdge)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            4, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource), 4, "Search Source");
    }
    //viCheckParm(RsCore_InvalidViReal64Range(instrSession, dataLevel, -10.0, 10.0 ), 5, "Data Level");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            6, "Clock Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_R4 < clockSource) && (clockSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, clockSource), 6, "Clock Source");
    }

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            4, "Search Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_LEVEL, dataLevel),
            5, "Data Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_SOURCE, clockSource),
            6, "Clock Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_LEVEL, clockLevel),
            7, "Clock Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_HOLD_TIME, holdTime),
            8, "Hold Time");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_SETUP_TIME, setupTime),
            9, "Setup Time");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_DATA2CLOCK_EDGE, clockEdge),
            10, "Clock Edge");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the conditions for the state trigger search.
/// HIFN  The state search is a qualified edge search. The state search is only
/// HIFN  available for analog channel sources (Ch1 to Ch4).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerState/Includes the search conditions for the state trigger event type in the
/// HIPAR triggerState/next search.
/// HIPAR clockSource/Sets the source of the clock signal.
/// HIPAR clockLevel/Sets the trigger level of the clock signal.
/// HIPAR clockEdge/Sets the trigger edge of the clock signal.
ViStatus _VI_FUNC rsscope_ConfigureStateTriggerSearch(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViBoolean triggerState,
                                                      ViInt32   clockSource,
                                                      ViInt32   clockEdge,
                                                      ViReal64  clockLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_STATE_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_STATE_CLOCK_SOURCE, clockSource),
            4, "Clock Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_STATE_CLOCK_EDGE, clockEdge),
            5, "Clock Edge");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_STATE_LEVEL, clockLevel),
            6, "Clock Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  By qualifying a trigger event, you can logically combine the trigger
/// HIFN  signal with the state of other digital channel signals.
/// HIFN     The instrument triggers if both of the following apply:
/// HIFN     - The basic conditions of the trigger event definition are fulfilled.
/// HIFN     - The logical conditions of the trigger qualification are true.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channel/Sets the trigger source for state trigger search.
/// HIPAR channelEnabled/Enables the channel to be considered in the state search. You can
/// HIPAR channelEnabled/enable all channel signals except for the trigger source.
/// HIPAR channelLogic/Defines the logic for the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigureStateTriggerSearchQualification(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channel,
                            ViBoolean channelEnabled,
                            ViInt32   channelLogic)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr arrEnabledAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_1,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_3,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_4
             };
    const    ViAttr arrLogicAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_1,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_3,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_4
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
            3, "Channel");

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, arrEnabledAttr[channel - 1], channelEnabled),
                4, "Channel Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, arrLogicAttr[channel - 1], channelLogic),
                5, "Channel Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channelCombination/Sets the trigger source for the state trigger search.
/// HIPAR multichannelLogic/Defines the logic for the indicated channel combination.
ViStatus _VI_FUNC rsscope_ConfigureStateTriggerSearchQualificationMultichannelLogic(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr arrAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_1_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_3_4,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_ALL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelCombination, RSSCOPE_VAL_CHANNEL1_AND_2, RSSCOPE_VAL_CHANNEL_ALL),
            3, "Channel Combination");

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, arrAttr[channelCombination - 1], multichannelLogic),
                4, "Multichannel Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channelCombination/Sets the trigger source for the state trigger search.
/// HIPAR multichannelLogic/Defines the logic for the indicated channel combination.
ViStatus _VI_FUNC rsscope_ConfigureStateTriggerSearchQualificationMulitchannelLogic(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelCombination, RSSCOPE_VAL_CHANNEL1_AND_2, RSSCOPE_VAL_CHANNEL_ALL),
            3, "Channel Combination");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multichannelLogic, RSSCOPE_VAL_AND, RSSCOPE_VAL_NOR),
            4, "Multichannel Logic");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:STAT:%s:LOGic '%s',%s",
             qualifyMultiChannelArr[channelCombination],
             searchName, qualifyMultiChannelLogicArr[multichannelLogic]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the range of a pulse width is defined for keeping up the
/// HIFN  true result of the pattern condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR patternMode/Adds additional time limitation to the pattern definition.
/// HIPAR timeoutCondition/Defines the condition for the timeout.
/// HIPAR timeout/Defines how long the result of the pattern condition must keep the
/// HIPAR timeout/given state.
/// HIPAR patternWidth/For the ranges "Within" and "Outside", the width defines the center of
/// HIPAR patternWidth/a range which is defined by the limits "Delta".
/// HIPAR patternDelta/Defines a range around the width value.
/// HIPAR patternCondition/Defines how the range of a pulse width is defined for keeping up the
/// HIPAR patternCondition/true result of the pattern condition.
/// HIPAR triggerState/Includes the search conditions for the pattern trigger event type in
/// HIPAR triggerState/the next search.
ViStatus _VI_FUNC rsscope_ConfigurePatternTriggerSearch(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViBoolean triggerState,
                                                        ViInt32   patternMode,
                                                        ViInt32   timeoutCondition,
                                                        ViReal64  timeout,
                                                        ViReal64  patternWidth,
                                                        ViReal64  patternDelta,
                                                        ViInt32   patternCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_ENABLED, triggerState),
            3, "Trigger State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_MODE, patternMode),
            4, "Pattern Mode");

    switch (patternMode)
    {
    case RSSCOPE_VAL_PATTERN_TIMEOUT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_TIMEOUT_CONDITION, timeoutCondition),
                5, "Timeout Condition");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_TIMEOUT, timeout),
                6, "Timeout");
        break;
    case RSSCOPE_VAL_PATTERN_WIDTH:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH, patternWidth),
                7, "Pattern Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH_DELTA, patternDelta),
                8, "Pattern Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH_CONDITION, patternCondition),
                9, "Pattern Condition");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  By qualifying a trigger event, you can logically combine the trigger
/// HIFN  signal with the state of other digital channel signals.
/// HIFN     The instrument triggers if both of the following apply:
/// HIFN     - The basic conditions of the trigger event definition are fulfilled.
/// HIFN     - The logical conditions of the trigger qualification are true.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channel/Sets the trigger source for the pattern trigger search.
/// HIPAR channelEnabled/Enables the channel to be considered in the pattern search. You can
/// HIPAR channelEnabled/enable all channel signals except for the trigger source.
/// HIPAR channelLogic/Defines the logic for the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigurePatternTriggerSearchQualification(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channel,
                            ViBoolean channelEnabled,
                            ViInt32   channelLogic)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr arrEnabledAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_1,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_3,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_ENABLED_CHANNEL_4
             };
    const    ViAttr arrLogicAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_1,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_3,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNEL_4
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
            3, "Channel");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, arrEnabledAttr[channel - 1], channelEnabled),
                4, "Channel Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, arrLogicAttr[channel - 1], channelLogic),
                5, "Channel Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channelCombination/Sets the trigger source for the state trigger search.
/// HIPAR multichannelLogic/Defines the logic for the indicated channel combination.
ViStatus _VI_FUNC rsscope_ConfigurePatternTriggerSearchQualificationMultichannelLogic(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr arrLogicAttr[] = {
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_1_2,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_3_4,
             RSSCOPE_ATTR_SEARCH_PATTERN_TRIGGER_QUALIFICATION_LOGIC_CHANNELS_ALL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelCombination, RSSCOPE_VAL_CHANNEL1_AND_2, RSSCOPE_VAL_CHANNEL_ALL),
            3, "Channel Combination");

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, arrLogicAttr[channelCombination - 1], multichannelLogic),
                4, "Multichannel Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the logic for the indicated channel combination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR channelCombination/Sets the trigger source for the state trigger search.
/// HIPAR multichannelLogic/Defines the logic for the indicated channel combination.
ViStatus _VI_FUNC rsscope_ConfigurePatternTriggerSearchQualificationMulitchannelLogic(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   channelCombination,
                            ViInt32   multichannelLogic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelCombination, RSSCOPE_VAL_CHANNEL1_AND_2, RSSCOPE_VAL_CHANNEL_ALL),
            3, "Channel Combination");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multichannelLogic, RSSCOPE_VAL_AND, RSSCOPE_VAL_NOR),
            4, "Multichannel Logic");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:PATT:%s:LOG '%s',%s", qualifyMultiChannelArr[channelCombination],
             searchName, qualifyMultiChannelLogicArr[multichannelLogic]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trigger search condition. It provides
/// HIFN  logical combinations of the input channels, math or reference
/// HIFN  waveforms, and supports you in verifying the operation of digital
/// HIFN  logic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchLogic/Returns regular expression defining the logic search.
ViStatus _VI_FUNC rsscope_QueryTriggerSearchCondition(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   bufferSize,
                                                      ViChar    searchLogic[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_GetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SEARCH_TRIGGER_CONDITIONS,
            bufferSize, searchLogic),
            4, "Search Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs the search only on the defined gate area of the
/// HIFN  source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR useGate/Enables gating.
ViStatus _VI_FUNC rsscope_ConfigureSearchGateState(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViBoolean useGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:STAT '%s',%s", searchName, useGate ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is displayed in the
/// HIFN  source diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR showGate/Indicates the gate area in the source diagram.
ViStatus _VI_FUNC rsscope_ConfigureSearchGateShow(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViBoolean showGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:SHOW '%s',%s", searchName, showGate ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is defined identically
/// HIFN  to the zoom area for the selected active zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR zoomCoupling/If enabled, the gate area is defined identically to the zoom area for
/// HIPAR zoomCoupling/the zoom diagram.
/// HIPAR zoomDiagram/This control is reserved for future use.
ViStatus _VI_FUNC rsscope_ConfigureSearchGateDefinitionZoom(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViBoolean zoomCoupling,
                                                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:ZCO '%s',%s", searchName, zoomCoupling ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the zoom to which the gate area is set if Zoom Coupling in
/// HIFN  rsscope_ConfigureSearchGateDefinitionZoom is set to On.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:GATE:ZDIagram
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR diagram/Selects the diagram
/// HIPAR zoom/Select zoom as gate area.
ViStatus _VI_FUNC rsscope_ConfigureSearchGateDefinitionZoomDiagram(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  diagram,
                            ViString  zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:ZDI '%s','%s','%s'", searchName, diagram, zoom);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and end values for the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR mode/Defines whether the gate settings are configured using absolute or
/// HIPAR mode/relative values.
/// HIPAR start/Defines the starting value for the gate.
/// HIPAR stop/Defines the ending value for the gate.
ViStatus _VI_FUNC rsscope_ConfigureSearchGateDefinitionStartStop(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   mode,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:MODE '%s',%s", searchName, modeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:%s:STAR '%s',%.12lG", modeArr[mode], searchName, start);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:GATE:%s:STOP '%s',%.12lG", modeArr[mode], searchName, stop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the search result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR showResultTable/Displays or hides the search result table.
/// HIPAR sortMode/Sorts the search result table by x-value position or value of the
/// HIPAR sortMode/result.
/// HIPAR sortAscending/If enabled, the results are listed in asscending order, i.e. the
/// HIPAR sortAscending/smallest value at the top.
/// HIPAR maxResultCount/Defines the maximum number of entries in the search result table.
ViStatus _VI_FUNC rsscope_ConfigureSearchResultTable(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean showResultTable,
                                                     ViInt32   sortMode,
                                                     ViBoolean sortAscending,
                                                     ViInt32   maxResultCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString sortMode_Arr[] = {"POS", "VAL"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sortMode, RSSCOPE_VAL_SORT_MODE_POSITION, RSSCOPE_VAL_SORT_MODE_VALUE),
            4, "Sort Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SHOW '%s',%s", searchName, showResultTable ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SORT:MODE '%s',%s", searchName, sortMode_Arr[sortMode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SORT:ASC '%s',%s", searchName, sortAscending ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIM '%s',%ld", searchName, maxResultCount);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the zoom window display state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR showSearchZoomWindows/If enabled, a zoom window is displayed for the currently selected
/// HIPAR showSearchZoomWindows/search result. The zoom area is indicated in the diagram that displays
/// HIPAR showSearchZoomWindows/the source waveform of the search.
ViStatus _VI_FUNC rsscope_ConfigureSearchZoomWindowState(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean showSearchZoomWindows)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:SHOW '%s',%s", searchName, showSearchZoomWindows ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the horizontal settings of the search zoom
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR horizonatalMode/Defines whether absolute or relative values are used to specify the
/// HIPAR horizonatalMode/x-axis values.
/// HIPAR position/Defines the x-value of the centerpoint of the zoom area.
/// HIPAR range/Defines the width of the zoom area.
ViStatus _VI_FUNC rsscope_ConfigureSearchZoomWindowXAxis(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   horizonatalMode,
                                                         ViReal64  position,
                                                         ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizonatalMode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Horizonatal Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:HORZ:MODE '%s',%s", searchName, modeArr[horizonatalMode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:HORZ:%s:POS '%s',%.12lG", modeArr[horizonatalMode], searchName,
             position);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:HORZ:%s:SPAN '%s',%.12lG", modeArr[horizonatalMode], searchName,
             range);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the vertical settings of the search zoom
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR verticalMode/Defines whether absolute or relative values are used to specify the
/// HIPAR verticalMode/y-axis values.
/// HIPAR position/Defines the y-value of the centerpoint of the zoom area.
/// HIPAR range/Defines the width of the zoom area.
ViStatus _VI_FUNC rsscope_ConfigureSearchZoomWindowYAxis(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   verticalMode,
                                                         ViReal64  position,
                                                         ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalMode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            3, "Vertical Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:VERT:MODE '%s',%s", searchName, modeArr[verticalMode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:VERT:%s:POS '%s',%.12lG", modeArr[verticalMode], searchName,
             position);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RESD:VERT:%s:SPAN '%s',%.12lG", modeArr[verticalMode], searchName,
             range);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the noise rejection by setting a hysteresis
/// HIFN  in order to avoid finding trigger events caused
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchConditionSource/Selects the source of the search.
/// HIPAR useNoiseReject/If enabled, the noise reject settings for the waveform are considered
/// HIPAR useNoiseReject/for the search.
/// HIPAR hysteresis/Defines a range in absolute or relative values around the trigger
/// HIPAR hysteresis/level. If the signal jitters inside this range and crosses the trigger
/// HIPAR hysteresis/level, no trigger event is detected.
/// HIPAR hysteresisMode/Defines whether absolute values or relative values to the vertical
/// HIPAR hysteresisMode/scaling are used as a hysteresis for noise rejection.
ViStatus _VI_FUNC rsscope_ConfigureSearchNoiseReject(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   searchSource,
                                                     ViBoolean useNoiseReject,
                                                     ViInt32   hysteresisMode,
                                                     ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    const    ViAttr attrHysteresis[] = {
             RSSCOPE_ATTR_SEARCH_NOISE_REJECT_ABSOLUTE_HYSTERESIS,
             RSSCOPE_ATTR_SEARCH_NOISE_REJECT_RELATIVE_HYSTERESIS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchSource, RSSCOPE_VAL_MEASUREMENT_SOURCE_C1W1, RSSCOPE_VAL_MEASUREMENT_SOURCE_M8),
            3, "Search Source");
    if ((RSSCOPE_VAL_MEASUREMENT_SOURCE_TRK8 < searchSource) && (searchSource < RSSCOPE_VAL_MEASUREMENT_SOURCE_Z1V1))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, searchSource),
                    3, "Search Source");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hysteresisMode, RSSCOPE_VAL_ABS, RSSCOPE_VAL_REL),
            5, "Hysteresis Mode");

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_SOURCE, searchSource),
            3, "Search Source");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SEARCH_NOISE_REJECT_ENABLED, useNoiseReject),
            4, "Use Noise Reject");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_NOISE_REJECT_HYSTERESIS_MODE, hysteresisMode),
            5, "Hystersis Mode");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, attrHysteresis[hysteresisMode], hysteresis),
            6, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the decoding of the selected bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables the decoding of the selected bus.
ViStatus _VI_FUNC rsscope_ConfigureDecodeState(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_STATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a label to be displayed with the bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR busLabel/Defines a label to be displayed with the bus.
ViStatus _VI_FUNC rsscope_ConfigureBusLabel(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViString  busLabel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LABEL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LABEL, busLabel),
            3, "Bus Label");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function opens a table with decoded data of the serial signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR showTable/Opens a table with decoded data of the serial signal.
ViStatus _VI_FUNC rsscope_ShowDecodeTable(ViSession instrSession,
                                          ViInt32   serialBus,
                                          ViBoolean showTable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DISPLAY_RESULT_TABLE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DISPLAY_RESULT_TABLE, showTable),
            3, "Show Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the protocol decode zoom and result table are
/// HIFN  synchronized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR zoomCouplingEnabled/If enabled, the protocol decode zoom and result table are
/// HIPAR zoomCouplingEnabled/synchronized.
ViStatus _VI_FUNC rsscope_ConfigureProtocolResultTableZoomCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean zoomCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RESULT_TABLE_ZOOM_COUPLING_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RESULT_TABLE_ZOOM_COUPLING_ENABLED, zoomCouplingEnabled),
            3, "Zoom Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the position of the protocol decode result table on the
/// HIFN  screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Defines the position of the protocol decode result table on the
/// HIPAR position/screen.
ViStatus _VI_FUNC rsscope_ConfigureProtocolResultTablePosition(
                            ViSession instrSession,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_RESULT_TABLE_POSITION, position),
            2, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the data format for decoded data values in the
/// HIFN  "Decode results" box and on the display of the decoded signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataFormat/Sets the data format for decoded data values in the "Decode results"
/// HIPAR dataFormat/box and on the display of the decoded signal.
ViStatus _VI_FUNC rsscope_ConfigureDataFormat(ViSession instrSession,
                                              ViInt32   dataFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_DATA_FORMAT, dataFormat),
            3, "Data Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number format for decoded data values of the indicated bus.
/// HIFN  It defines the format in the "Decode table" and in the combs of the
/// HIFN  decoded signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataFormat/Sets the data format for decoded data values in the "Decode results"
/// HIPAR dataFormat/box and on the display of the decoded signal.
ViStatus _VI_FUNC rsscope_ConfigureBusDataFormat(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   dataFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_BUS_DATA_FORMAT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_BUS_DATA_FORMAT, dataFormat),
            3, "dataFormat");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If ON, the threshold levels are displayed in the diagram.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_SHOW_THRESHOLD_LINES
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:THReshold
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/If ON, the threshold levels are displayed in the diagram.
ViStatus _VI_FUNC rsscope_ConfigureShowThresholdLines(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SHOW_THRESHOLD_LINES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SHOW_THRESHOLD_LINES, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows detailed information for the frames. The attribute is relevant
/// HIFN  for FlexRay, D-PHY, M-PHY, Ethernet, CXPI, and Custom protocols.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_SHOW_DETAILS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:RESDetail
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Shows detailed information for the frames. The attribute is relevant
/// HIPAR state/for FlexRay, D-PHY, M-PHY, Ethernet, CXPI, and Custom protocols.
ViStatus _VI_FUNC rsscope_ConfigureShowDetails(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SHOW_DETAILS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SHOW_DETAILS, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the label list to be used for decoding. The
/// HIFN  "Label" appear in the "Decode results" table and in the frame captions
/// HIFN  of the decoded signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Activates the label list to be used for decoding. The "Label" appear
/// HIPAR state/in the "Decode results" table and in the frame captions of the decoded
/// HIPAR state/signal.
ViStatus _VI_FUNC rsscope_ConfigureLabelList(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LABEL_LIST,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LABEL_LIST, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function loads a label list file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR fileName/Loads a label list file.
ViStatus _VI_FUNC rsscope_ConfigureBusLabelListFile(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LABEL_LIST_FILE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LABEL_LIST_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function executes the measurement of reference levels and sets
/// HIFN  the thresholds to the middle reference level of the measured amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_SetReferenceLevels(ViSession instrSession,
                                             ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_REFERENCE_LEVELS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_REFERENCE_LEVELS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts software algorithms for determining the signal threshold levels
/// HIFN  and bitrate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_AutosetProtocolThresholdAndBitrate(ViSession instrSession,
                                                             ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUTOSET_THRESHOLD_AND_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUTOSET_THRESHOLD_AND_BITRATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures items to be included in the exported protocol results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR includeFrameTiming/Includes the frame timing for the exported protocol results.
/// HIPAR includeDetails/Includes the detailed results for all frames for the exported protocol
/// HIPAR includeDetails/results.
ViStatus _VI_FUNC rsscope_ConfigureProtocolExportSettings(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViBoolean includeFrameTiming,
                                                          ViBoolean includeDetails)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_EXPORT_INCLUDE_FRAME_TIMING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_EXPORT_INCLUDE_FRAME_TIMING, includeFrameTiming),
            3, "Include Frame Timing");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_EXPORT_INCLUDE_DETAILS, includeDetails),
            4, "Include Details");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the selected results to the indicated file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR fileName/Saves the selected results to the indicated file.
ViStatus _VI_FUNC rsscope_ExportProtocolToFile(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_EXPORT_INCLUDE_FRAME_TIMING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_EXPORT_SAVE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures source waveform of the data line (SDA) and
/// HIFN  clock line (SCL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select I2C line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureI2CSource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   line,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrSourceAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_I2C_SCL_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_I2C_SDA_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_I2C_SCL, RSSCOPE_VAL_I2C_SDA),
            3, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrSourceAttr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_I2C_SCL, RSSCOPE_VAL_I2C_SDA),
            3, "Line");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                4, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrSourceAttr[line], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs the threshold voltage for various signal
/// HIPAR preset/technologies from a list.
/// HIPAR SCLThreshold/Sets user-defined value of the threshold voltage.
/// HIPAR SDAThreshold/Sets user-defined value of the threshold voltage.
ViStatus _VI_FUNC rsscope_ConfigureI2CThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   preset,
                                                ViReal64  SCLThreshold,
                                                ViReal64  SDAThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TECHNOLOGY, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_SCL_THRESHOLD, SCLThreshold),
                4, "SCL Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_SDA_THRESHOLD, SDAThreshold),
                5, "SDA Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines if the R/W bit of a 7-bit address is considered
/// HIFN separately or as part of the address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR RWBit/This control defines if the R/W bit of a 7-bit address is
/// HIPAR RWBit/considered separately or as part of the address.
ViStatus _VI_FUNC rsscope_ConfigureI2CRWBit(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   RWBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_RW_BIT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_RW_BIT, RWBit),
            3, "RW Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger type for I2C analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/This control select trigger type.
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerType(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the indicated ADOR address in
/// HIFN  the "address OR" trigger condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR orSlotState/If set to "On" the indicated ADOR address is included in the "address
/// HIPAR orSlotState/OR" trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerAddressORState(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   orSlot,
                                                            ViBoolean orSlotState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, orSlot, 1, 4),
            3, "OR Slot");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_STATE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_STATE,
            "ORSlot", orSlot- 1, RS_REPCAP_BUF_SIZE, repCap_slot),
            3, "OR Slot");

    strcat(repCap, ",");
    strcat(repCap, repCap_slot);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_STATE, orSlotState),
            4, "OR Slot State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures triggering on one to four address conditions.
/// HIFN  Each condition requires an exact address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR addressType/This control select address length.
/// HIPAR rwBitAddress/Sets the R/W bit of the indicated ADOR address in the "address OR"
/// HIPAR rwBitAddress/trigger condition.
/// HIPAR address/Defines the address pattern of the address.
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerAddressOR(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   orSlot,
                                                       ViInt32   addressType,
                                                       ViInt32   rwBitAddress,
                                                       ViString  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, orSlot, 1, 4),
            3, "OR Slot");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_TYPE,
            "ORSlot", orSlot- 1, RS_REPCAP_BUF_SIZE, repCap_slot),
            3, "OR Slot");

    strcat(repCap, ",");
    strcat(repCap, repCap_slot);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_TYPE, addressType),
            4, "Address Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR_ACCESS_BIT, rwBitAddress),
            5, "RW Bit Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_OR, address),
            6, "Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the data trigger conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR addressMin/Sets a data pattern, or sets the start value of a data
/// HIPAR addressMin/pattern range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR addressMax/Sets the end value of an data range if "Condition" "In  range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerData(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  addressMin,
                                                  ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_PATTERN_MIN, addressMin),
            4, "Address Min");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_PATTERN_MAX, addressMax),
                5, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the position of the first byte of data bit pattern within the payload segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific position or position range.
/// HIPAR indexMin/Sets the number of data bytes to be skipped after start of the payload segment.
/// HIPAR indexMax/Defines the last byte of interest,if "Condition" is "Range".
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerDataPosition(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   condition,
                                                          ViInt32   indexMin,
                                                          ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_POSITION_CONDITION, condition),
            3, "Condition");
    if (condition != RSSCOPE_VAL_I2C_COND_ANY)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_POSITION_MIN, indexMin),
                4, "Index Min");
        if (condition == RSSCOPE_VAL_I2C_COND_RAN)
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_POSITION_MAX, indexMax),
                    5, "Index Max");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which missing acknowledge bits is detected if
/// HIFN  the trigger type is set to "Missing Ack".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR addressNack/Triggers if the address acknowledge bit is missing.
/// HIPAR dataWriteNack/Triggers if a data acknowledge bit is missing
/// HIPAR dataReadNack/Triggers on the end of the read process when the master reads data
/// HIPAR dataReadNack/from the slave.
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerNoAck(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViBoolean addressNack,
                                                   ViBoolean dataWriteNack,
                                                   ViBoolean dataReadNack)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_NACK, addressNack),
            3, "Address Nack");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_DATA_WRITE_NACK, dataWriteNack),
            4, "Data Write Nack");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_DATA_READ_NACK, dataReadNack),
            5, "Data Read Nack");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the address trigger conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR addressType/This control select address length.
/// HIPAR RWBitAddress/This control select trigger type.
/// HIPAR addressMin/Sets slave address, or the start value of an address range. Enter
/// HIPAR addressMin/values in binary format, use the following characters: 1; 0; or X (any
/// HIPAR addressMin/bit).
/// HIPAR condition/This control is reserved for future use.
/// HIPAR addressMax/This control is reserved for future use.
ViStatus _VI_FUNC rsscope_ConfigureI2CTriggerAddress(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   addressType,
                                                     ViInt32   RWBitAddress,
                                                     ViInt32   condition,
                                                     ViString  addressMin,
                                                     ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_TYPE, addressType),
            3, "Address Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ACCESS_BIT, RWBitAddress),
            4, "RW Bit Address");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_CONDITION, condition),
            5, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_MIN, addressMin),
            6, "Address Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_MAX, addressMax),
                7, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryI2CFrameCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_I2C_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR status/The overall state of the frame.
/// HIPAR frameStart/Time of frame start
/// HIPAR frameStop/Time of frame end.
/// HIPAR addressType/This control show address length.
/// HIPAR address/Value of the frame address.
/// HIPAR RWBit/Value of the R/W bit.
/// HIPAR ackBit/Value of the address acknowledge bit.
ViStatus _VI_FUNC rsscope_QueryI2CFrameStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32*  status,
                                              ViReal64* frameStart,
                                              ViReal64* frameStop,
                                              ViInt32*  addressType,
                                              ViInt32*  address,
                                              ViInt32*  RWBit,
                                              ViInt32*  ackBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:STAT?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameStatus, response, status), 4, "Status");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:AMOD?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameAddressType, response, addressType), 7, "Address Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:ADDR?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:ACC?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameRWBit, response, RWBit), 9, "RW Bit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:AACC?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameAckBit, response, ackBit), 10, "Ack Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start time of the access ack bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR accessBitStart/Time of access bit start.
ViStatus _VI_FUNC rsscope_QueryI2CFrameAccessBitStart(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViReal64* accessBitStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:RWBS?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, accessBitStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the slave address of the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR address/Value of the frame slave address.
ViStatus _VI_FUNC rsscope_QueryI2CFrameSlaveAddress(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32*  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:ADEV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start time of the address for the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR addressStart/Time of address start.
ViStatus _VI_FUNC rsscope_QueryI2CFrameAddressAckBitStartTime(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViReal64* addressStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:AST?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, addressStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start time of the address acknowledge bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR ackBitStart/Time of ack bit start.
ViStatus _VI_FUNC rsscope_QueryI2CFrameAddressAckBitStart(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   frame,
                                                          ViReal64* ackBitStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:ADBS?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, ackBitStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function checks if address is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR complete/Status of address.
ViStatus _VI_FUNC rsscope_QueryI2CFrameAddressComplete(ViSession  instrSession,
                                                       ViInt32    serialBus,
                                                       ViInt32    frame,
                                                       ViBoolean* complete)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:ACOM?", serialBus, frame);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, complete));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QueryI2CFrameBitRate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_I2C_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_I2C_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_I2C_FRAME_BIT_RATE, bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of bytes in the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR length/Returns the number of bytes in the specified frame
ViStatus _VI_FUNC rsscope_QueryI2CFrameLength(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BCO?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, length));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR ackBit/Value of the data acknowledge bit.
ViStatus _VI_FUNC rsscope_QueryI2CFrameData(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   frame,
                                            ViInt32   byteNumber,
                                            ViInt32*  data,
                                            ViInt32*  ackBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BYTE%ld:VAL?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BYTE%ld:ACC?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolI2CAckBit_Arr, ackBit));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start time of the specified data byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR startTime/Start time of the byte.
ViStatus _VI_FUNC rsscope_QueryI2CFrameDataByteStart(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   byteNumber,
                                                     ViReal64* startTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BYTE%ld:STAR?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start time of the acknowledge bit of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR startTime/Start time.
ViStatus _VI_FUNC rsscope_QueryI2CFrameDataByteAcknowledgeBitStart(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   byteNumber,
                            ViReal64* startTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BYTE%ld:ACKS?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function checks if data byte is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR complete/Start time of the byte.
ViStatus _VI_FUNC rsscope_QueryI2CFrameDataComplete(ViSession  instrSession,
                                                    ViInt32    serialBus,
                                                    ViInt32    frame,
                                                    ViInt32    byteNumber,
                                                    ViBoolean* complete)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:BYTE%ld:COMP?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, complete));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/Returns the symbolic label of the specified frame if the label list is
/// HIPAR label/enabled.
ViStatus _VI_FUNC rsscope_QueryI2CFrameLabel(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   bufferSize,
                                             ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:SYMB?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the search type for I2C analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the search type for I2C analysis.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchType(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_I2C_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the search type for LIN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the search type for LIN analysis.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchType(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures search criteria.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startOfFrame/Enables the search for the stop bit of the sync field, which marks the
/// HIPAR startOfFrame/frame start.
/// HIPAR identifier/Enables the search for one specific identifier or an identifier range.
/// HIPAR identifierOR/Enables the search for one to four address conditions.
/// HIPAR identifierData/Enables the search for a combination of identifier and data
/// HIPAR identifierData/conditions.
/// HIPAR wakeupFrame/Enables the search for wakeup frames.
/// HIPAR errorCondition/Enables the search for various errors in the frame.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchCriteria(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean startOfFrame,
                                                     ViBoolean identifier,
                                                     ViBoolean identifierOR,
                                                     ViBoolean identifierData,
                                                     ViBoolean wakeupFrame,
                                                     ViBoolean errorCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_START_OF_FRAME, startOfFrame),
            3, "Start Of Frame");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_IDENTIFIER, identifier),
            4, "Identifier");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_IDENTIFIER_OR, identifierOR),
            5, "Identifier OR");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_IDENTIFIER_DATA, identifierData),
            6, "Identifier Data");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_WAKEUP_FRAME, wakeupFrame),
            7, "Wakeup Frame");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRITERIA_ERROR_CONDITION, errorCondition),
            8, "Error Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for ID search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR idMin/Sets the message identifier pattern, or sets the start value of an
/// HIPAR idMin/identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" is "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureLINSearchID(ViSession instrSession,
                                               ViString  searchName,
                                               ViInt32   condition,
                                               ViString  idMin,
                                               ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_IDENTIFIER_CONDITION, condition),
            3, "Start Of Frame");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_PATTERN_IDENTIFIER_MIN, idMin),
            4, "Id Min");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_PATTERN_IDENTIFIER_MAX, idMax),
                5, "Id Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the state of the indicated ADOR address in
/// HIFN  the "address OR" search condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR orSlotState/If set to "On" the indicated ADOR address is included in the "address
/// HIPAR orSlotState/OR" search condition.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchAddressORState(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   orSlot,
                                                           ViBoolean orSlotState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR_STATE,
            "ORSlot", orSlot- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "OR Slot");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR_STATE, orSlotState),
            4, "Or Slot State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures search on one to four address conditions.
/// HIFN  Each condition requires an exact address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR addressType/This control select address length.
/// HIPAR rwBitAddress/Sets the R/W bit of the indicated ADOR address in the 'address OR'
/// HIPAR rwBitAddress/trigger condition.
/// HIPAR address/Defines the address pattern of the address.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchAddressOR(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   orSlot,
                                                      ViInt32   addressType,
                                                      ViInt32   rwBitAddress,
                                                      ViString  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR_TYPE,
            "ORSlot", orSlot- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "OR Slot");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR_TYPE, addressType),
            4, "Address Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR_ACCESS_BIT, rwBitAddress),
            5, "Rw Bit Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_OR, address),
            6, "Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the address search conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR addressType/This control select address length.
/// HIPAR rwBitAddress/Sets the transfer direction of the data.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR addressMin/Sets slave address, or the start value of an address range. Enter
/// HIPAR addressMin/values in binary format, use the following characters: 1; 0; or X (any
/// HIPAR addressMin/bit).
/// HIPAR addressMax/Sets the end value of an address range if the condition is set to
/// HIPAR addressMax/an address range.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchAddress(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   addressType,
                                                    ViInt32   rwBitAddress,
                                                    ViInt32   condition,
                                                    ViString  addressMin,
                                                    ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_TYPE, addressType),
            3, "Address Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ACCESS_BIT, rwBitAddress),
            4, "RW Bit Address");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_CONDITION, condition),
            5, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_MIN, addressMin),
            6, "Address Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADDRESS_MAX, addressMax),
                7, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the position of the first byte of data bit pattern
/// HIFN  within the payload segment. You can define an exact position, or a
/// HIFN  position range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the data position. You can defined an exact
/// HIPAR condition/position, or a position range.
/// HIPAR indexMin/Sets the number of data bytes to be skipped after start of the payload
/// HIPAR indexMin/segment.
/// HIPAR indexMax/Defines the last byte of interest,if "Condition" is "Range".
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchDataPosition(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   condition,
                                                         ViInt32   indexMin,
                                                         ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_POSITION_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_POSITION_MIN, indexMin),
            4, "Index Min");
    if (condition >= RSSCOPE_VAL_I2C_COND_RAN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_POSITION_MAX, indexMax),
                5, "Index Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the data search conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR addressMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR addressMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR addressMax/Sets the end value of an data range if "Condition" "In range" and
/// HIPAR addressMax/"Out of range".
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchData(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   condition,
                                                 ViString  addressMin,
                                                 ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_I2C_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_I2C_PATTERN_MIN, addressMin),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_I2C_PATTERN_MAX, addressMax),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures which missing acknowledge bits is detected if
/// HIFN  the search type is set to "Missing Ack".

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR addressNack/Search if the address acknowledge bit is missing.
/// HIPAR dataWriteNack/Search if a data acknowledge bit is missing
/// HIPAR dataReadNack/Search on the end of the read process when the master reads data from
/// HIPAR dataReadNack/the slave.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchNoAck(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViBoolean addressNack,
                                                  ViBoolean dataWriteNack,
                                                  ViBoolean dataReadNack)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_ADRESS_NACK, addressNack),
            3, "Address Nack");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_DATA_WRITE_NACK, dataWriteNack),
            4, "Sata Write Nack");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_DATA_READ_NACK, dataReadNack),
            5, "Data Read Nack");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR status/The overl state of the frame.
/// HIPAR frameStart/Time of frame start
/// HIPAR frameStop/Time of frame end.
/// HIPAR addressType/This control show address length.
/// HIPAR address/Value of the frame address.
/// HIPAR rwBit/Value of the R/W bit.
/// HIPAR ackBit/Value of the address acknowledge bit.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameStatus(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViInt32*  status,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  addressType,
                                                    ViInt32*  address,
                                                    ViInt32*  rwBit,
                                                    ViInt32*  ackBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameStatus, response, status), 4, "Status");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:AMOD? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameAddressType, response, addressType), 7, "Address Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:ADDR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:ACC? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameRWBit, response, rwBit), 9, "RW Bit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:AACC? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngI2CFrameAckBit, response, ackBit), 10, "Ack Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the start time of the access ack bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR accessBitStart/Time of access bit start.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameAccessBitStart(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   frame,
                                                            ViReal64* accessBitStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:RWBS? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, accessBitStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the slave address of the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR address/Value of the frame slave address.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameSlaveAddress(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32*  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:ADEV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start time of the address field for the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR addressStart/Time of address start.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameAddressAckBitStartTime(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViReal64* addressStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:AST? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, addressStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the start time of the address acknowledge bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR ackBitStart/Time of ack bit start.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameAddressAckBitStart(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViReal64* ackBitStart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:ADBS? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, ackBitStart));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks if address is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR complete/Status of address.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameAddressComplete(ViSession  instrSession,
                                                             ViString   searchName,
                                                             ViInt32    frame,
                                                             ViBoolean* complete)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:ACOM? '%s'", frame, searchName);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, complete));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the number of bytes in the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR length/Returns the number of bytes in the specified frame
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameLength(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BCO? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, length));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data vue of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR ackBit/Value of the data acknowledge bit.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   frame,
                                                  ViInt32   byteNumber,
                                                  ViInt32*  data,
                                                  ViInt32*  ackBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BYTE%ld:VAL? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BYTE%ld:ACC? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolI2CSearchFrameAck_Arr, ackBit));

    checkErr(rsscope_CheckStatus(instrSession));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the start time of the specified data byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR startTime/Start time of the byte.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameDataByteStart(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32   byteNumber,
                                                           ViReal64* startTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BYTE%ld:STAR? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start time of the acknowledge bit of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR startTime/Start time.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameDataByteAcknowledgeBitStart(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   byteNumber,
                            ViReal64* startTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BYTE%ld:ACKS? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks if data byte is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession hdle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The hdle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR complete/Indicates if specified data byte is complete.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameDataComplete(ViSession  instrSession,
                                                          ViString   searchName,
                                                          ViInt32    frame,
                                                          ViInt32    byteNumber,
                                                          ViBoolean* complete)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:BYTE%ld:COMPl? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, complete));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all data words of the specified frame in comma-separated
/// HIFN  list of decimal values of the data bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameAllData(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32   arraySize,
                                                     ViInt32   data[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:DATA? %s", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/Returns the symbolic label of the specified frame if the label list is
/// HIPAR label/enabled.
ViStatus _VI_FUNC rsscope_QueryI2CSearchFrameLabel(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   frame,
                                                   ViInt32   bufferSize,
                                                   ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:I2C:FRAM%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the all data words of the specified frame in comma-separated
/// HIFN  list of decimal values of the data bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryI2CFrameAllData(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32   arraySize,
                                               ViInt32   data[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2C:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures input channel of the selected line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select configured SPI line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureSPISource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   line,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrSPISourceAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_SPI_CLOCK_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_SPI_SLAVE_SELECT_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_SPI_MISO_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_SPI_MOSI_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SPI_SCLK, RSSCOPE_VAL_SPI_MOSI),
            3, "Line");
    if ((waveformSource == RSSCOPE_VAL_PROTOCOL_WAV_NONE) && (line == RSSCOPE_VAL_SPI_SCLK))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveformSource), 4, "Waveform Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrSPISourceAttr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrSPISourceAttr[line], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures he threshold value for digitization of
/// HIFN  signals for each line. If the signal value on the line is higher than
/// HIFN  the threshold, the signal state is high (1 or true for the boolean
/// HIFN  logic). Otherwise, the signal state is considered low (0 or false) if
/// HIFN  the signal value is below the threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs the threshold voltage for various signal
/// HIPAR preset/technologies from a list.
/// HIPAR clockThreshold/Sets user-defined value of the threshold voltage.
/// HIPAR SSThreshold/Sets user-defined value of the threshold voltage.
/// HIPAR misoThreshold/Sets user-defined value of the threshold voltage.
/// HIPAR mosiThreshold/Sets user-defined value of the threshold voltage.
ViStatus _VI_FUNC rsscope_ConfigureSPIThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   preset,
                                                ViReal64  clockThreshold,
                                                ViReal64  SSThreshold,
                                                ViReal64  misoThreshold,
                                                ViReal64  mosiThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_bus[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap_bus),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_SPI_TECHNOLOGY, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_SPI_CLOCK_THRESHOLD, clockThreshold),
                4, "Clock Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_SPI_SLAVE_SELECT_THRESHOLD, SSThreshold),
                5, "SS Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_SPI_MISO_THRESHOLD, misoThreshold),
                6, "MISOThreshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_SPI_MOSI_THRESHOLD, mosiThreshold),
                7, "MOSIThreshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether transmitted data or the slave select
/// HIFN  signal is high active (high = 1) or low active (low = 1).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select configured SPI line.
/// HIPAR polarity/Selects line polarity.
ViStatus _VI_FUNC rsscope_ConfigureSPIPolarity(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   line,
                                               ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrSPIPolarityAttr[] = {
             VI_FALSE,
             RSSCOPE_ATTR_PROTOCOL_SPI_SLAVE_SELECT_POLARITY,
             RSSCOPE_ATTR_PROTOCOL_SPI_MISO_POLARITY,
             RSSCOPE_ATTR_PROTOCOL_SPI_MOSI_POLARITY
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SPI_SSEL, RSSCOPE_VAL_SPI_MOSI),
            3, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrSPIPolarityAttr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrSPIPolarityAttr[line], polarity),
            4, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures bit count and it's order in message. The
/// HIFN  display of the decoded signal considers this setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitOrder/Select if the data of the messages starts with msb or lsb.
/// HIPAR wordLength/Sets the number of bits in a word.
ViStatus _VI_FUNC rsscope_ConfigureSPIWord(ViSession instrSession,
                                           ViInt32   serialBus,
                                           ViInt32   bitOrder,
                                           ViInt32   wordLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_BIT_ORDER,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_BIT_ORDER, bitOrder),
            3, "Bit Order");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_WORD_LENGTH, wordLength),
            4, "Word Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start of a frame. A frame contains a
/// HIFN  number of successive words, at least one word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCondition/Select the start of a frame.
/// HIPAR clockTimeout/Sets a timeout on the clock line SCLK as limiter between two frames.
ViStatus _VI_FUNC rsscope_ConfigureSPIFrameStart(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frameCondition,
                                                 ViReal64  clockTimeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_START_CONDITION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_START_CONDITION, frameCondition),
            3, "Frame Condition");

    if (frameCondition == RSSCOPE_VAL_SPI_FRAME_START_CLK)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_START_TIMEOUT, clockTimeout),
                4, "Clock Timeout");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for SPI analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/This control select trigger type.
ViStatus _VI_FUNC rsscope_ConfigureSPITriggerType(ViSession instrSession,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure MOSI and MISO trigger conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Selects the operator for the MISO and MOSI pattern.
/// HIPAR mosiPattern/Specifies the data pattern to be found on the MOSI line. Enter bin
/// HIPAR mosiPattern/words in msb first bit order.
/// HIPAR misoPattern/Specifies the data pattern to be found on the MISO line. Enter bin
/// HIPAR misoPattern/words in msb first bit order.
ViStatus _VI_FUNC rsscope_ConfigureSPITriggerData(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  mosiPattern,
                                                  ViString  misoPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_EQ),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_MOSI_PATTERN, mosiPattern),
            4, "MOSI Pattern");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_MISO_PATTERN, misoPattern),
            5, "MISO Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure MOSI and MISO trigger conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Reserved for future use.
/// HIPAR searchMode/Selects how the specified data pattern is searched:
/// HIPAR indexMin/Sets the number of bits or words before the first word of interest.
/// HIPAR indexMax/Reserved for future use.
ViStatus _VI_FUNC rsscope_ConfigureSPITriggerDataPosition(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   condition,
                                                          ViInt32   searchMode,
                                                          ViInt32   indexMin,
                                                          ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             indexMax;
             condition;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_DATA_ALIGNMENT,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_POSITION_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_DATA_ALIGNMENT, searchMode),
            4, "Search Mode");

    if (condition != RSSCOPE_VAL_SPI_COND_ANY)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_DATA_INDEX_MIN, indexMin),
                5, "Index Min");

        if (condition == RSSCOPE_VAL_SPI_COND_RAN)
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_TRIGGER_DATA_INDEX_MAX, indexMax),
                    6, "Index Max");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QuerySPIFrameCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of words in frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Number of words.
ViStatus _VI_FUNC rsscope_QuerySPIWordCount(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   frame,
                                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WCO?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStart/Returns the start time of the specified frame.
/// HIPAR frameStop/Returns the end time of the specified frame.
/// HIPAR status/The overall state of the frame.
ViStatus _VI_FUNC rsscope_QuerySPIFrameStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViReal64* frameStart,
                                              ViReal64* frameStop,
                                              ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:STAT?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngSPIFrameStatus, response, status), 6, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QuerySPIFrameBitRate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_SPI_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_SPI_FRAME_BIT_RATE, bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/This control selects the word.
/// HIPAR valueStart/Start time of the data word.
/// HIPAR valueStop/Stop time of the data word.
/// HIPAR mosiValue/Returns the data value of the specified word on the MOSI line.
/// HIPAR misoValue/Returns the data value of the specified word on the MISO line.
ViStatus _VI_FUNC rsscope_QuerySPIFrameData(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   frame,
                                            ViInt32   word,
                                            ViReal64* valueStart,
                                            ViReal64* valueStop,
                                            ViInt32*  mosiValue,
                                            ViInt32*  misoValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:STAR?", serialBus, frame, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, valueStart));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:STOP?", serialBus, frame, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, valueStop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:MOSI?", serialBus, frame, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, mosiValue));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:MISO?", serialBus, frame, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, misoValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all data words of the specified frame in comma-separated
/// HIFN  list of decimal values of the data bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySPIFrameAllData(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32   arraySize,
                                               ViInt32   data[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the formatted value of the specified word on the MISO/MOSI
/// HIFN  line.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K1 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:SPI:FRAMe<n>:WORD<o>:FMOSi?
/// HIFN     BUS<m>:SPI:FRAMe<n>:WORD<o>:FMISo?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/This control selects the word.
/// HIPAR line/This control selects the line.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Formatted Value'.
/// HIPAR formattedValue/Formated value.
ViStatus _VI_FUNC rsscope_QuerySPIFrameFormattedValue(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViInt32   word,
                                                      ViInt32   line,
                                                      ViInt32   bufferSize,
                                                      ViChar    formattedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    switch (line)
    {
    case RSSCOPE_VAL_SPI_MISO:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:FMIS?", serialBus, frame, word);
        checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, formattedValue));
        break;

    case RSSCOPE_VAL_SPI_MOSI:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SPI:FRAM%ld:WORD%ld:FMOS?", serialBus, frame, word);
        checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, formattedValue));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, line), 5, "Line");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the input channels for the transmitter and
/// HIFN  receiver signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select UART line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureUARTSource(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   line,
                                              ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrUARTSourceAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_UART_RX_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_UART_TX_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_UART_RX, RSSCOPE_VAL_UART_TX),
            3, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrUARTSourceAttr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrUARTSourceAttr[line], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs the threshold voltage for various signal
/// HIPAR preset/technologies from a list.
/// HIPAR rxThreshold/Sets user-defined value of the threshold voltage.
/// HIPAR txThreshold/Sets user-defined value of the threshold voltage.
ViStatus _VI_FUNC rsscope_ConfigureUARTThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   preset,
                                                 ViReal64  rxThreshold,
                                                 ViReal64  txThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TECHNOLOGY, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_RX_THRESHOLD, rxThreshold),
                4, "Rx Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TX_THRESHOLD, txThreshold),
                5, "Tx Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures UART.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
/// HIPAR stopbits/Selects the number of stop bits.
/// HIPAR databits/Sets the number of data bits in a message.
/// HIPAR parity/Selects the optional parity bit that is used for error detection.
/// HIPAR polarity/Sets if the transmitted data on the bus is high (high = 1) or low (low
/// HIPAR polarity/= 1) active.
ViStatus _VI_FUNC rsscope_ConfigureUART(ViSession instrSession,
                                        ViInt32   serialBus,
                                        ViInt32   bitRate,
                                        ViInt32   stopbits,
                                        ViInt32   databits,
                                        ViInt32   parity,
                                        ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_BIT_RATE, bitRate),
            3, "Bit Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_STOP_BITS, stopbits),
            4, "Stop Bits");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_DATA_BITS_COUNT, databits),
            5, "Data bits");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_PARITY, parity),
            6, "Parity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_POLARITY, polarity),
            7, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows to define packets of several words in the data stream.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR packets/Defines the method of packet separation. A packet is a number of
/// HIPAR packets/subsequent words in a date stream.
/// HIPAR endWord/Sets the end pattern of the packets. A new packet starts with the
/// HIPAR endWord/first start bit after the defined end pattern. This property using
/// HIPAR endWord/hexadecimal format (prefix #H).
/// HIPAR packetsTimeout/Sets the timeout between packets in a UART data stream. A new packet
/// HIPAR packetsTimeout/starts with the first start bit after the timeout.
ViStatus _VI_FUNC rsscope_ConfigureUARTPackets(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   packets,
                                               ViInt32   endWord,
                                               ViReal64  packetsTimeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_PACKETS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_PACKETS, packets),
            2, "Packets");

    if (packets == RSSCOPE_VAL_UART_PACKET_EWOR)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_END_OF_FRAME, endWord),
                3, "End Word");
    }
    else if (packets == RSSCOPE_VAL_UART_PACKET_TOUT)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TOUT, packetsTimeout),
                4, "Packets Timeout");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type and source for UART
/// HIFN  analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger even
/// HIPAR type/This control select trigger type.
/// HIPAR source/This control select trigger type.
ViStatus _VI_FUNC rsscope_ConfigureUARTTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type,
                                                   ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[10];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_MODE, type),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_SOURCE, source),
            4, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the data conditions if the trigger type is set
/// HIFN  to Data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger even
/// HIPAR condition/Selects the operator for the data pattern.
/// HIPAR pattern/Specifies the data pattern to be found on the specified trigger
/// HIPAR pattern/source, in hex format. Enter the words in msb first bit order.
/// HIPAR position/Operator for the data position. You can defined an exact position, or
/// HIPAR position/a position range.
/// HIPAR indexMin/Sets the number of words before the first word of interest. These
/// HIPAR indexMin/offset words are ignored. If the Position defines a range, the first
/// HIPAR indexMin/word of interest is defined.
/// HIPAR indexMax/Sets the last word of interest, if the Position defines a range.
ViStatus _VI_FUNC rsscope_ConfigureUARTTriggerData(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   condition,
                                                   ViString  pattern,
                                                   ViInt32   position,
                                                   ViInt32   indexMin,
                                                   ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[10];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN, pattern),
            4, "Pattern");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_POSITION, position),
            5, "Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_POSITION_INDEX_MIN, indexMin),
            6, "Index Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_TRIGGER_DATA_POSITION_INDEX_MAX, indexMax),
            7, "Index Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of decoded words.
ViStatus _VI_FUNC rsscope_QueryUARTFrameCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_WORD_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_UART_WORD_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the detailed decoded data of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR source/Line on which the specified word was tranferred.
/// HIPAR status/The status of the specified word.
/// HIPAR start/Time of the word start (start bit)
/// HIPAR txValue/Decimal value of the Tx word
/// HIPAR rxValue/Decimal value of the Rx word
ViStatus _VI_FUNC rsscope_QueryUARTFrameData(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   word,
                                             ViInt32*  source,
                                             ViInt32*  status,
                                             ViReal64* start,
                                             ViInt32*  txValue,
                                             ViInt32*  rxValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  esr;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K2|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:UART:WORD%ld:SOUR?", serialBus, word);
    error = RsCore_QueryStringIndex(instrSession, cmd, protocolUARTLine_Arr, source);
    if (error != VI_SUCCESS)
    {
        (void)RsCore_QueryViInt32(instrSession, "*ESR?", &esr);

        if ((esr & 32) != 0)
            error = VI_SUCCESS;
        else
            error = RS_ERROR_INSTRUMENT_STATUS;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:UART:WORD%ld:STAT?", serialBus, word);
    error = RsCore_QueryStringIndex(instrSession, cmd, protocolUARTStatus_Arr, status);
    if (error != VI_SUCCESS)
    {
        (void)RsCore_QueryViInt32(instrSession, "*ESR?", &esr);

        if ((esr & 32) != 0)
            error = VI_SUCCESS;
        else
            error = RS_ERROR_INSTRUMENT_STATUS;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:UART:WORD%ld:STAR?", serialBus, word);
    error = RsCore_QueryViReal64(instrSession, cmd, start);
    if (error != VI_SUCCESS)
    {
        (void)RsCore_QueryViInt32(instrSession, "*ESR?", &esr);

        if ((esr & 32) != 0)
            error = VI_SUCCESS;
        else
            error = RS_ERROR_INSTRUMENT_STATUS;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:UART:WORD%ld:RXV?", serialBus, word);
    error = RsCore_QueryViInt32(instrSession, cmd, rxValue);
    if (error != VI_SUCCESS)
    {
        (void)RsCore_QueryViInt32(instrSession, "*ESR?", &esr);

        if ((esr & 32) != 0)
            error = VI_SUCCESS;
        else
            error = RS_ERROR_INSTRUMENT_STATUS;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:UART:WORD%ld:TXV?", serialBus, word);
    error = RsCore_QueryViInt32(instrSession, cmd, txValue);
    if (error != VI_SUCCESS)
    {
        (void)RsCore_QueryViInt32(instrSession, "*ESR?", &esr);

        if ((esr & 32) != 0)
            error = VI_SUCCESS;
        else
            error = RS_ERROR_INSTRUMENT_STATUS;
    }

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QueryUARTPrimaryBitRate(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_UART_PRIMARY_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_UART_PRIMARY_BIT_RATE, bitRate),
                3, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures source waveform of the data line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureLINSource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Waveform Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_SOURCE, waveformSource),
            3, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIFN     If the signal value is higher than the threshold, the signal state is
/// HIFN  high (logic 1). Otherwise, the signal state is considered low (logic
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs the threshold voltage for various signal
/// HIPAR preset/technologies from a list.
/// HIPAR manualThreshold/Sets user-defined value of the threshold voltage.
ViStatus _VI_FUNC rsscope_ConfigureLINThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   preset,
                                                ViReal64  manualThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TECHNOLOGY, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_LIN_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_THRESHOLD, manualThreshold),
                4, "Manual Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the idle state of the bus. The idle state is
/// HIFN  the rezessive state and corresponds to a logic 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Selects line polarity.
ViStatus _VI_FUNC rsscope_ConfigureLINPolarity(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_POLARITY, polarity),
            3, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of transmitted bits per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR standard/Selects the version of the LIN standard that is used in the DUT.
/// HIPAR standard/The setting mainly defines the checksum version used during decoding.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsscope_ConfigureLINStandard(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   standard,
                                               ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_STANDARD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_STANDARD, standard),
            3, "Standard");

    if (standard != RSSCOPE_VAL_LIN_STANDARD_J2602)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_BIT_RATE, bitRate),
                4, "Bit Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for LIN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for LIN analysis.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerType(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR idMin/Sets the message identifier pattern, or sets the start value of an
/// HIPAR idMin/identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" is "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerID(ViSession instrSession,
                                                ViInt32   trigger,
                                                ViInt32   condition,
                                                ViString  idMin,
                                                ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_IDENTIFIER_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_IDENTIFIER_MIN, idMin),
            4, "ID Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_IDENTIFIER_MAX, idMax),
                5, "ID Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures triggering on one to four address conditions.
/// HIFN  Enter the patterns in the "Frame ID" fields
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR orSlotState/If set to "On" the indicated ADOR address is included in the
/// HIPAR orSlotState/"identifier OR" trigger condition.
/// HIPAR frameID/Sets the pattern of the indicated IDOR identifier in the "identifier
/// HIPAR frameID/OR" trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerIDOR(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   orSlot,
                                                  ViBoolean orSlotState,
                                                  ViString  frameID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, orSlot, 1, 4),
            3, "OR Slot");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_ADDRESS_OR_STATE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_ADDRESS_OR_STATE,
            "ORSlot", orSlot- 1, RS_REPCAP_BUF_SIZE, repCap_slot),
            3, "OR Slot");

    strcat(repCap, ",");
    strcat(repCap, repCap_slot);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_ADDRESS_OR_STATE, orSlotState),
            4, "OR Slot State");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_ADDRESS_OR, frameID),
            5, "Frame ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR dataMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR dataMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerData(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  dataMin,
                                                  ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION_DATA, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_DATA_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_DATA_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures data trigger setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set the data length code for triggering on LIN data.
/// HIPAR dataLength/Sets the length of the bit pattern to be found, in bytes.
/// HIPAR transfer/Sets the byte order (endianess) of the data transfer.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerDataSetup(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   condition,
                                                       ViInt32   dataLength,
                                                       ViInt32   transfer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_DATA_LENGTH, dataLength),
            4, "Data Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_DATA_TRANSFER, transfer),
            5, "Transfer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR checksum/Sets state of triggering on the CRC error according to the LIN
/// HIPAR checksum/standard.
/// HIPAR idParityError/Sets state of triggering on an error in the identifier parity bits.
/// HIPAR idParityError/These are the bits 6 and 7 of the identifier.
/// HIPAR syncError/Sets state of triggering on a synchronization error.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerErrorConditions(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViBoolean checksum,
                                                             ViBoolean idParityError,
                                                             ViBoolean syncError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_MODE, RSSCOPE_VAL_LIN_ERR_COND));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CHECKSUM_ERROR, checksum),
            3, "Checksum");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_IDENTIFIER_PARITY_ERROR, idParityError),
            4, "ID Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_SYNC_ERROR, syncError),
            5, "Sync Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the bit pattern of the slave identifier
/// HIFN  ("ID") and the number of data bytes ("Data length") when trigger on CRC
/// HIFN  error is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frameID/Sets bit pattern of the slave identifier ("ID").
/// HIPAR dataLength/Sets the number of data bytes to trigger on CRC errors.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerCRCErrorPattern(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViString  frameID,
                                                             ViInt32   dataLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CRC_ERROR_PATTERN, frameID),
            3, "Frame ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CRC_ERROR_DATA_LENGTH, dataLength),
            4, "DataL ength");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the LIN standard to trigger on CRC errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR standard/Sets the LIN standard to trigger on CRC errors.
ViStatus _VI_FUNC rsscope_ConfigureLINTriggerCRCErrorStandard(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CRC_STANDARD,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_TRIGGER_CRC_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryLINFrameCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_LIN_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/The overall state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR version/Returns the version of the LIN standard for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINFrameStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32*  frameStatus,
                                              ViReal64* frameStart,
                                              ViReal64* frameStop,
                                              ViInt32*  version)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:STAT?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFrameStatus, response, frameStatus), 4, "Frame Status");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:VERS?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFrameVersion, response, version), 7, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR value/Returns the version of the LIN standard for the specified frame.
/// HIPAR parity/Returns the value of the identifier parity bits of the selected frame.
/// HIPAR status/Returns the identifier state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryLINFrameID(ViSession instrSession,
                                          ViInt32   serialBus,
                                          ViInt32   frame,
                                          ViInt32*  value,
                                          ViInt32*  parity,
                                          ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:IDV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:IDPV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, parity));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:IDST?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, status), 6, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the selected field state for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINFrameField(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   field,
                                             ViInt32*  fieldValue,
                                             ViInt32*  fieldStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString fieldArr[] = {"SY", "CS"};
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_LIN_FIELD_SYNC, RSSCOPE_VAL_LIN_FIELD_CHKSUM),
            3, "Field");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    if (field == RSSCOPE_VAL_LIN_FIELD_SYNC)
    {
        *fieldValue = -1;
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:%sV?", serialBus, frame, fieldArr[field]);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:%sST?", serialBus, frame, fieldArr[field]);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, fieldStatus), 6, "Field Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QueryLINFrameBitRate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_LIN_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_LIN_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_LIN_FRAME_BIT_RATE, bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the data state for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINFrameData(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   frame,
                                            ViInt32   byteNumber,
                                            ViInt32*  data,
                                            ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:BYTE%ld:VAL?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:BYTE%ld:STAT?", serialBus, frame, byteNumber);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, statusCode), 6, "Status Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryLINFrameAllData(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32   arraySize,
                                               ViInt32   data[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the complete symbolic data of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR symbolicData/Returns the symbolic data of the selected frame.
/// HIPAR returnedValues/Returns length of the symbolic data returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryLINFrameAllSymbolicData(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt32   arraySize,
                                                       ViChar    symbolicData[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:SDAT?",
             serialBus, frame);

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize,
        symbolicData, returnedValues));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the complete symbolic data of the selected frame in export
/// HIFN  format, and saves the data to a file on the local PC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR fileName/
/// HIPAR timeout/Defines timeout in milliseconds. Set to 0 to use current OPC timeout
/// HIPAR timeout/value.
ViStatus _VI_FUNC rsscope_ExportLINFrameAllSymbolicDataToFile(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViString  fileName,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViUInt32 oldTimeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (timeout > 0)
    {
        checkErr(rsscope_GetOPCTimeout(instrSession, &oldTimeout));
        viCheckParm(rsscope_SetOPCTimeout(instrSession, timeout), 5, "Timeout");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "FORM REAL,32;:BUS%ld:LIN:FRAM%ld:SDEX?", serialBus, frame);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, fileName, RS_VAL_TRUNCATE));

    strncpy(cmd, "FORM ASC", RS_MAX_MESSAGE_LEN);
    checkErr(rsscope_WriteInstrData(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (oldTimeout > 0)
        (void)rsscope_SetOPCTimeout(instrSession, oldTimeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryLINFrameLabel(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   bufferSize,
                                             ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:LIN:FRAM%ld:SYMB?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures search criteria.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR start/Enables the search for the start of the message.
/// HIPAR repeatedStart/Enables the search for a start condition without previous stop
/// HIPAR repeatedStart/condition.
/// HIPAR stop/Enables the search for the start of the message.
/// HIPAR nack/Searches for missing address acknowledge bits.
/// HIPAR address/Enables the search for one specific address condition or for a
/// HIPAR address/combination of address conditions.
/// HIPAR addressOR/Enables the search for one to four address conditions.
/// HIPAR addressData/Enables the search for a combination of address and data conditions.
ViStatus _VI_FUNC rsscope_ConfigureI2CSearchCriteria(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean start,
                                                     ViBoolean repeatedStart,
                                                     ViBoolean stop,
                                                     ViBoolean nack,
                                                     ViBoolean address,
                                                     ViBoolean addressOR,
                                                     ViBoolean addressData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_START, start),
            3, "Start");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_REPEATED_START, repeatedStart),
            4, "Repeated Start");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_STOP, stop),
            5, "Stop");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_NACK, nack),
            6, "Nack");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_ADDRESS, address),
            7, "Address");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_ADDRESS_OR, addressOR),
            8, "Address OR");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_I2C_CRITERIA_ADDRESS_DATA, addressData),
            9, "Address Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures search on one to four address conditions.
/// HIFN  Enter the patterns in the "Frame ID" fields
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR orSlot/This control select index of the address in an "address OR" condition
/// HIPAR orSlot/(OR slot).
/// HIPAR orSlotState/If set to "On" the indicated ADOR address is included in the
/// HIPAR orSlotState/"identifier OR" search condition.
/// HIPAR frameID/Sets the pattern of the indicated IDOR identifier in the "identifier
ViStatus _VI_FUNC rsscope_ConfigureLINSearchIDOR(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   ORSlot,
                                                 ViBoolean ORSlotState,
                                                 ViString  frameID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SEARCH_LIN_ADDRESS_OR_STATE,
            "ORSlot", ORSlot- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "OR Slot");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_ADDRESS_OR_STATE, ORSlotState),
            4, "OR Slot State");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_ADDRESS_OR, frameID),
            5, "Frame ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures data search setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set the data length code for triggering on LIN
/// HIPAR condition/data. For big endian transfer direction, you can trigger on a number of
/// HIPAR condition/bytes less than the data length of the frame, that means, on the
/// HIPAR condition/beginning of the data pattern. For little endian transfer direction,
/// HIPAR condition/the exact number of data bytes in the frame must be set.
/// HIPAR dataLength/Sets the length of the bit pattern to be found, in bytes. For "Big
/// HIPAR dataLength/Endian" transfer direction, you can trigger on a number of bytes less
/// HIPAR dataLength/than the data length of the frame, that means, on the beginning of the
/// HIPAR dataLength/data pattern. For "Little Endian" transfer direction, the exact number
/// HIPAR dataLength/of data bytes in the frame must be set.
/// HIPAR transfer/Sets the byte order (endianess) of the data transfer.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchDataSetup(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   condition,
                                                      ViInt32   dataLength,
                                                      ViInt32   transfer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_DATA_LENGTH_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_DATA_LENGTH, dataLength),
            5, "Data Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_DATA_TRANSFER, transfer),
            6, "Transfer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR dataMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR dataMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureLINSearchData(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   condition,
                                                 ViString  dataMin,
                                                 ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CONDITION_DATA, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_PATTERN_DATA_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_PATTERN_DATA_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error Condition search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR checksum/Sets state of search on the CRC error according to the LIN standard.
/// HIPAR idParityError/Searches for errors in the identifier parity bits. These are the bits
/// HIPAR idParityError/6 and 7 of the identifier.
/// HIPAR syncError/Searches for synchronization errors.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchErrorConditions(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViBoolean checksum,
                                                            ViBoolean idParityError,
                                                            ViBoolean syncError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CHECKSUM_ERROR, checksum),
            3, "Checksum");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_IDENTIFIER_PARITY_ERROR, idParityError),
            4, "Id Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_SYNC_ERROR, syncError),
            5, "Sync Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the bit pattern of the slave identifier
/// HIFN  ("ID") and the number of data bytes ("Data length") when searching on
/// HIFN  CRC error is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameID/Sets bit pattern of the slave identifier ("ID").
/// HIPAR dataLength/Sets the number of data bytes to search on CRC errors.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchCRCErrorPattern(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViString  frameID,
                                                            ViInt32   dataLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRC_ERROR_PATTERN, frameID),
            3, "Frame ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRC_ERROR_DATA_LENGTH, dataLength),
            4, "DataL ength");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the LIN standard to search for CRC errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR standard/Sets the LIN standard to search for CRC errors.
ViStatus _VI_FUNC rsscope_ConfigureLINSearchCRCErrorStandard(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_CRC_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_LIN_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/The overall state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR version/Returns the version of the LIN standard for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameStatus(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViInt32*  frameStatus,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  version)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFrameStatus, response, frameStatus), 4, "Frame Status");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:VERS? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFrameVersion, response, version), 7, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR value/Returns the version of the LIN standard for the specified frame.
/// HIPAR parity/Returns the value of the identifier parity bits of the selected frame.
/// HIPAR status/Returns the identifier state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameID(ViSession instrSession,
                                                ViString  searchName,
                                                ViInt32   frame,
                                                ViInt32*  value,
                                                ViInt32*  parity,
                                                ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:IDV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:IDPV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, parity));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:IDST? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, status), 6, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the selected field state for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameField(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   frame,
                                                   ViInt32   field,
                                                   ViInt32*  fieldValue,
                                                   ViInt32*  fieldStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString fieldArr[] = {"SY", "CS"};
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_LIN_FIELD_SYNC, RSSCOPE_VAL_LIN_FIELD_CHKSUM),
            3, "Field");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    if (field == RSSCOPE_VAL_LIN_FIELD_SYNC)
    {
        *fieldValue = -1;
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:%sV? '%s'", frame, fieldArr[field], searchName);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:%sST? '%s'", frame, fieldArr[field], searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, fieldStatus), 6, "Field Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the data state for the specified frame.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   frame,
                                                  ViInt32   byteNumber,
                                                  ViInt32*  data,
                                                  ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:BYTE%ld:VAL? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:BYTE%ld:STAT? '%s'", frame, byteNumber, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngLINFieldStatus, response, statusCode), 6, "Status Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameAllData(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32   arraySize,
                                                     ViInt32   data[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryLINSearchFrameLabel(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   frame,
                                                   ViInt32   bufferSize,
                                                   ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:LIN:FRAM%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the symbolic data of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR data/Returns the symbolic data of the frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySearchLINFrameSymbolicData(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32   bufferSize,
                                                          ViChar    data[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_LIN_SELECTED_FRAME, frame),
            3, "Frame");

    checkErr(rsscope_GetAttributeViStringWithRealLen(instrSession, NULL, RSSCOPE_ATTR_SEARCH_LIN_FRAME_SYMBOLIC_DATA, bufferSize, data, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the symbolic data of the frame for the export format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR data/Returns the symbolic data of the frame for the export format.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySearchLINFrameSymbolicDataExport(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    data[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_LIN_SELECTED_FRAME, frame),
            3, "Frame");

    checkErr(rsscope_GetAttributeViStringWithRealLen(instrSession, NULL, RSSCOPE_ATTR_SEARCH_LIN_FRAME_SYMBOLIC_DATA_EXPORT, bufferSize, data, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures input channel of the CAN line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/Selects the CAN-High or CAN-Low line. Both lines are required for
/// HIPAR line/differential signal transmission used by CAN.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureCANSource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   line,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_LINE_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                4, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_LINE_TYPE, line),
            3, "Line");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SOURCE, waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIFN  If the signal value is higher than the threshold, the signal state is
/// HIFN  high (logic 1). Otherwise, the signal state is considered low (logic
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs the threshold voltage for various signal
/// HIPAR preset/technologies from a list.
/// HIPAR manualThreshold/Sets user-defined value of the threshold voltage.
ViStatus _VI_FUNC rsscope_ConfigureCANThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   preset,
                                                ViReal64  manualThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TECHNOLOGY, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_CAN_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_THRESHOLD, manualThreshold),
                4, "Manual Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of transmitted bits per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsscope_ConfigureCANBitRate(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_BIT_RATE, bitRate),
            3, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures CAN FD protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enable/Enables the CAN FD protocol configuration.
/// HIPAR bitRate/Sets the bit rate of the data phase. The data rate can be higher than
/// HIPAR bitRate/the arbitration rate, but it is uniform and fixed for a given CAN FD
/// HIPAR bitRate/bus.
ViStatus _VI_FUNC rsscope_ConfigureCANFD(ViSession instrSession,
                                         ViInt32   serialBus,
                                         ViBoolean enable,
                                         ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FD_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FD_ENABLED, enable),
            3, "Enable");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FD_BIT_RATE, bitRate),
            4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Only available for CAN FD buses. Selects whether the tested signal is
/// HIFN  an ISO CAN FD signal or not.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_FD_STANDARD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FDATa:PSTandard
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR standard/Only available for CAN FD buses. Selects whether the tested signal is
/// HIPAR standard/an ISO CAN FD signal or not.
ViStatus _VI_FUNC rsscope_ConfigureCANFDStandard(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FD_STANDARD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FD_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute configures maximum number of time quanta for phase
/// HIFN  correction. This mean Time segment1 may be lengthened or Time segment2
/// HIFN  may be shortened due to resynchronization.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_FD_JUMP_WIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FDATa:JWIDth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR jumpWidth/This attribute configures maximum number of time quanta for phase
/// HIPAR jumpWidth/correction. This mean Time segment1 may be lengthened or Time segment2
/// HIPAR jumpWidth/may be shortened due to resynchronization.
ViStatus _VI_FUNC rsscope_ConfigureCANFDJumpWidth(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViReal64  jumpWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FD_JUMP_WIDTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FD_JUMP_WIDTH, jumpWidth),
            3, "Jump Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the number of time quanta before the sample point (T1Segment) and
/// HIFN  after the sample point (T2Segment). T1Segment comprises the segments
/// HIFN  Synch_seg, Prop_seg, and Phase_seg1 which are specified in the CAN
/// HIFN  standard. T2Segment matches Phase_seg2 specified in the CAN standard.
/// HIFN
/// HIFN     Note(s)
/// HIFN
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     (2) The maximum sum of Time segment 1 and Time segment 2 is 24.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_ABS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FDATa:T1Segment
/// HIFN     BUS<m>:CAN:FDATa:T2Segment
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR timeSegment1/Set the number of time quanta before the sample point.
/// HIPAR timeSegment2/Set the number of time quanta after the sample point.
ViStatus _VI_FUNC rsscope_ConfigureCANFDSamplePointAbsolute(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViReal64  timeSegment1,
                                                            ViReal64  timeSegment2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_TS[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    if ((timeSegment1 + timeSegment2) - 24 > 0.01)
    {
        viCheckParm(RsCore_InvalidViReal64Value(instrSession, timeSegment1), 3, "Time Segment");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_ABS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    strcpy(repCap_TS, repCap);
    strcat(repCap_TS, ",TS1");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_TS, RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_ABS, timeSegment1),
            3, "Time Segment 1");

    strcpy(repCap_TS, repCap);
    strcat(repCap_TS, ",TS2");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_TS, RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_ABS, timeSegment2),
            4, "Time Segment 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the position of the sample point within the bit in percent ot the
/// HIFN  nominal bit time.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     (2)The time quanta values of time segmnet 1 and time segmnet 2 are
/// HIFN  adjusted accordingly. It can be read by attribute
/// HIFN  RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_ABS
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_REL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FDATa:SAMPlepoint
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR samplePoint/Sets the position of the sample point within the bit in percent ot the
/// HIPAR samplePoint/nominal bit time.
ViStatus _VI_FUNC rsscope_ConfigureCANFDSamplePointRelative(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViReal64  samplePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_REL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FD_SAMPLE_POINT_REL, samplePoint),
            3, "Sample Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures maximum number of time quanta for phase
/// HIFN  correction. This mean Time segment1 may be lengthened or Time segment2
/// HIFN  may be shortened due to resynchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR jumpWidth/Sets the jump width.
ViStatus _VI_FUNC rsscope_ConfigureCANJumpWidth(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViReal64  jumpWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_JUMP_WIDTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_JUMP_WIDTH, jumpWidth),
            3, "Jump Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of time quanta before the sample
/// HIFN  point (Time segment1) and after the sample point (Time segmnet 2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR timeSegment1/Set the number of time quanta before the sample point.
/// HIPAR timeSegment2/Set the number of time quanta after the sample point.
ViStatus _VI_FUNC rsscope_ConfigureCANSamplePointAbsolute(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViReal64  timeSegment1,
                                                          ViReal64  timeSegment2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_TS[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    if ((timeSegment1 + timeSegment2) - 24 > 0.01)
    {
        viCheckParm(RsCore_InvalidViReal64Value(instrSession, timeSegment1), 3, "Time Segment");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_SAMPLE_POINT_ABS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    strcpy(repCap_TS, repCap);
    strcat(repCap_TS, ",TS1");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_TS, RSSCOPE_ATTR_PROTOCOL_CAN_SAMPLE_POINT_ABS, timeSegment1),
            3, "Time Segment 1");

    strcpy(repCap_TS, repCap);
    strcat(repCap_TS, ",TS2");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap_TS, RSSCOPE_ATTR_PROTOCOL_CAN_SAMPLE_POINT_ABS, timeSegment2),
            4, "Time Segment 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the position of the sample point within the
/// HIFN  bit in percent of the nominal bit time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR samplePoint/Sets the position of the sample point within the bit in percent of the
/// HIPAR samplePoint/nominal bit time.
ViStatus _VI_FUNC rsscope_ConfigureCANSamplePointRelative(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViReal64  samplePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_SAMPLE_POINT_REL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SAMPLE_POINT_REL, samplePoint),
            3, "Sample Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for CAN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for CAN analysis.
/// HIPAR frameType/Selects the frame type which will be used as trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerType(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   type,
                                                  ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_MODE, type),
            3, "Type");

    if (type == RSSCOPE_VAL_CAN_FRAME_TYPE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FRAME_TYPE, frameType),
                3, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the CAN standard. Use Any if the standard of the
/// HIFN  signal is unknown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR standard/Selects the CAN standard.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerStandard(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STANDARD,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR idType/Selects the format of data and remote frames.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR idMin/Sets the message identifier pattern, or sets the start value of an
/// HIPAR idMin/identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerID(ViSession instrSession,
                                                ViInt32   trigger,
                                                ViInt32   idType,
                                                ViInt32   condition,
                                                ViString  idMin,
                                                ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_ID_TYPE, idType),
            3, "ID Type");

    if (idType != RSSCOPE_VAL_CAN_FRAME_ID_TYPE_ANY)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_IDENTIFIER_CONDITION, condition),
                4, "Condition");

        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_IDENTIFIER_MIN, idMin),
                5, "ID Min");
        if (condition >= RSSCOPE_VAL_COND_IN)
        {
            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_IDENTIFIER_MAX, idMax),
                    6, "ID Max");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CAN FD-specific bits trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fdf/Specifies the CAN FD frame format. It corresponds to the EDL bit
/// HIPAR fdf/(extended data length), which only exists in CAN FD format.
/// HIPAR brs/Sets the bit rate switch bit.
/// HIPAR esi/Sets the error state indicator bit.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerFDBits(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   fdf,
                                                    ViInt32   brs,
                                                    ViInt32   esi)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STANDARD,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_FDF, fdf),
            3, "fdf");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_BRS, brs),
            4, "brs");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_ESI, esi),
            5, "esi");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR transfer/Selects the byte order (endianness) of the data transfer.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR dataMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR dataMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerData(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   transfer,
                                                  ViInt32   condition,
                                                  ViString  dataMin,
                                                  ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FRAME_ENDIANNESS, transfer),
            3, "Transfer");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_DATA_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_DATA_MIN, dataMin),
            5, "Data Min");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_DATA_MAX, dataMax),
                6, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for DLC trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Operator to set the data length code for searching on CAN data. For
/// HIPAR condition/big endian transfer direction, you can search for a number of bytes
/// HIPAR condition/less than the DLC of the frame, that means, on the beginning of the
/// HIPAR condition/data pattern. For little endian transfer direction, the exact number of
/// HIPAR condition/data bytes in the frame must be set.
/// HIPAR dcl/Sets the Data Length Code, the number of data bytes to be found
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerDlc(ViSession instrSession,
                                                 ViInt32   trigger,
                                                 ViInt32   condition,
                                                 ViInt32   dlc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_DLC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_DLC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_DLC, dlc),
            4, "DCL");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of data bytes defined by DLC. DLC and NDB are
/// HIFN  different in CAN FD for DLCs > 8.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_NUMBER_OF_DATA_BYTES
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CAN:NDBYtes?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR numberOfDataBytes/Returns the number of data bytes defined by DLC. DLC and NDB are
/// HIPAR numberOfDataBytes/different in CAN FD for DLCs > 8.
ViStatus _VI_FUNC rsscope_QueryCANTriggerNumberOfDataBytes(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViInt32*  numberOfDataBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_NUMBER_OF_DATA_BYTES,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_NUMBER_OF_DATA_BYTES, numberOfDataBytes),
            3, "Number Of Data Bytes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the location in the data field where the instrument
/// HIFN  looks for the specified data pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to define an exact position or a data range where
/// HIPAR condition/the instrument looks for the specified data pattern.
/// HIPAR positionMin/Defines the number of the first data byte at which the data pattern
/// HIPAR positionMin/may start.
/// HIPAR positionMax/Sets the number of the last byte at which the required data pattern
/// HIPAR positionMax/may start.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerDataPosition(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   condition,
                                                          ViInt32   positionMin,
                                                          ViInt32   positionMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_POSITION_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_POSITION_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_POSITION_MIN, positionMin),
            4, "Position Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_POSITION_MAX, positionMax),
                5, "Position Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR crc/Sets state of triggering on the CRC error. A CRC error occurs
/// HIPAR stuffing/Sets state of triggering on the bit stuffing error. A stuff error
/// HIPAR stuffing/occurs when the 6th consecutive equal bit level in the mentioned fields
/// HIPAR stuffing/is detected
/// HIPAR form/Sets state of triggering on the form error. A form error occurs when a
/// HIPAR form/fixed-form bit field contains one or more illegal bits.
/// HIPAR ack/Sets state of triggering on the form error. An acknowledgement error
/// HIPAR ack/occurs when the transmitter does not receive an acknowledgment - a
/// HIPAR ack/dominant bit during the Ack Slot.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerErrorConditions(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViBoolean crc,
                                                             ViBoolean stuffing,
                                                             ViBoolean form,
                                                             ViBoolean ack)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_MODE, RSSCOPE_VAL_CAN_ERR_COND));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_CHECKSUM_ERROR, crc),
            3, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_BIT_STUFFING_ERROR, stuffing),
            4, "Stuffing");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_FORM_ERROR, form),
            5, "Form");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_ACK_ERROR, ack),
            6, "Ack");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers on stuff count errors. A stuff count error occurs if th
/// HIFN  received stuff count value does not match the value calculated from the
/// HIFN  own stuff bit count.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     (2) This function sets trigger type to Error Condition
/// HIFN
/// HIFN     (3) The R&S RTO detects errors in the message and triggers on these
/// HIFN  errors even if no CAN node sends an error flag.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STUFF_COUNT_ERROR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CAN:FDATa:SCERror
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR stuffCountError/Triggers on stuff count errors. A stuff count error occurs if th
/// HIPAR stuffCountError/received stuff count value does not match the value calculated from the
/// HIPAR stuffCountError/own stuff bit count.
ViStatus _VI_FUNC rsscope_ConfigureCANTriggerStuffCountError(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViBoolean stuffCountError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STUFF_COUNT_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_STUFF_COUNT_ERROR, stuffCountError),
            3, "Stuff Count Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures symbolic trigger message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR message/Sets the message to be triggered or searched for.
/// HIPAR triggerOnSignal/Enables the trigger on a specific signal value that is part of the
/// HIPAR triggerOnSignal/selected message.
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicTriggerMessage(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViString  message,
                                                             ViBoolean triggerOnSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_MESSAGE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_MESSAGE, message),
            3, "Message");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_ON_SIGNAL, triggerOnSignal),
            4, "Trigger On Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures symbolic trigger signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR signal/Sets the signal name to be triggered or searched for.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR dataMin/Defines the minimum data value of the signal.
/// HIPAR dataMax/Defines the maximum data value of the signal. This value is required
/// HIPAR dataMax/to specify a range if condition is "In range" or "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicTriggerSignal(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViString  signal,
                                                            ViInt32   condition,
                                                            ViReal64  dataMin,
                                                            ViReal64  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_VALUE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_VALUE, signal),
            3, "Signal");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_TRIGGER_DATA_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_MIN, dataMin),
            5, "Data Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_MAX, dataMax),
                6, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets a symbolic data value for signals with enumerated
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enumValue/Sets a symbolic data value for signals with enumerated values.
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicTriggerSignalEnumValue(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   enumValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_ENUM_VALUE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_SYMBOLIC_TRIGGER_SIGNAL_ENUM_VALUE, enumValue),
            3, "Enum Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryCANFrameCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CAN_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the CAN standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR standard/Returns the CAN standard.
ViStatus _VI_FUNC rsscope_QueryCANFrameFDStandard(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32*  standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K9|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FDAT:FRAM%ld:STAN?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANStandard_Arr, standard));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/The overall state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the frame type of the selected frame.
ViStatus _VI_FUNC rsscope_QueryCANFrameStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32*  frameStatus,
                                              ViReal64* frameStart,
                                              ViReal64* frameStop,
                                              ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR idType/Returns the identifier type of the selected frame, the identifier
/// HIPAR idType/format of data and remote frames.
/// HIPAR idValue/Returns the identifier value of the selected frame.
/// HIPAR statusCode/Return the identifier state.
ViStatus _VI_FUNC rsscope_QueryCANFrameID(ViSession instrSession,
                                          ViInt32   serialBus,
                                          ViInt32   frame,
                                          ViInt32*  idType,
                                          ViInt32*  idValue,
                                          ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:IDTY?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANIDType_Arr, idType));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:IDV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, idValue));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:IDST?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the parameters of selected field of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR value/Returns the value of selected field for the specified frame.
/// HIPAR status/Returns the field state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryCANFrameField(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   field,
                                             ViInt32*  value,
                                             ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString fieldArr[] = {"DLC", "CS", "ACK"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_CAN_FIELD_DLC, RSSCOPE_VAL_CAN_FIELD_ACK),
            3, "Field");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:%sV?", serialBus, frame, fieldArr[field]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:%sSTate?", serialBus, frame, fieldArr[field]);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QueryCANFrameBitRate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CAN_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_CAN_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_CAN_FRAME_BIT_RATE, bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the location of a bit stuffing error of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR errorPosition/Returns the location of a bit stuffing error.
ViStatus _VI_FUNC rsscope_QueryCANFrameBitStuffingError(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViReal64* errorPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:BSEP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, errorPosition));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the state of the specified byte.
ViStatus _VI_FUNC rsscope_QueryCANFrameData(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   frame,
                                            ViInt32   byteNumber,
                                            ViInt32*  data,
                                            ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:BYTE%ld:VAL?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:BYTE%ld:STAT?", serialBus, frame, byteNumber);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryCANFrameAllData(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32   arraySize,
                                               ViInt32   data[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the complete symbolic data of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Result parameter.
/// HIPAR result/String with comma-separated list of symbolic data.
ViStatus _VI_FUNC rsscope_QueryCANFrameAllSymbolicData(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt32   bufferSize,
                                                       ViChar    result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:SDAT?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(result, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryCANFrameLabel(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   bufferSize,
                                             ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:SYMB?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of data bytes.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FRAMe<n>:NDBYtes?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR numberOfBytes/Returns the number of data bytes.
ViStatus _VI_FUNC rsscope_QueryCANFrameNumberOfBytes(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32*  numberOfBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:NDBYtes?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfBytes));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns information on a form error, if the frame status query
/// HIFN  returned a FORM error.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FRAMe<n>:FERCause?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR formErrorCause/Returns information on a form error, if the frame status query
/// HIPAR formErrorCause/returned a FORM error.
ViStatus _VI_FUNC rsscope_QueryCANFrameFormErrorCause(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViInt32*  formErrorCause)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FRAM%ld:FERC?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameFormErrorCause_Arr, formErrorCause));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the stuff bit count modulo 8 value.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CAN:FDATa:FRAMe<n>:SCValue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR stuffCount/Returns the stuff bit count modulo 8 value.
ViStatus _VI_FUNC rsscope_QueryCANFrameFDStuffCount(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32*  stuffCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CAN:FDAT:FRAM%ld:SCV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, stuffCount));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the complete symbolic data of the selected frame in export
/// HIFN  format, and saves the data to a file on the local PC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR fileName/
ViStatus _VI_FUNC rsscope_ExportCANFrameAllSymbolicDataToFile(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViString  fileName,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViUInt32 oldTimeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (timeout > 0)
    {
        checkErr(rsscope_GetOPCTimeout(instrSession, &oldTimeout));
        viCheckParm(rsscope_SetOPCTimeout(instrSession, timeout), 5, "Timeout");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "FORM REAL,32;:BUS%ld:CAN:FRAM%ld:SDEX?", serialBus, frame);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, fileName, RS_VAL_TRUNCATE));

    strncpy(cmd, "FORM ASC", RS_MAX_MESSAGE_LEN);
    checkErr(rsscope_WriteInstrData(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (oldTimeout > 0)
        (void)rsscope_SetOPCTimeout(instrSession, oldTimeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function includes the search conditions for the CAN trigger event
/// HIFN  type in the next search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Includes the search conditions for the CAN trigger event type in the
/// HIPAR enabled/next search.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchState(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN '%s',%s", searchName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies the trigger event configuration from Trigger A
/// HIFN  for the selcted CAN bus to the search condition settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
ViStatus _VI_FUNC rsscope_CANSearchCopyFromA(ViSession instrSession,
                                             ViString  searchName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_COPY_FROM_A, searchName),
            2, "search Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the criteria or combination of criterias to be
/// HIFN  searched for. When more than one criteria is selected, all criterias
/// HIFN  must be fulfilled by a frame for it to be shown in the
/// HIFN     search results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startOfFrame/Enables the search for a start of frame.
/// HIPAR frameType/Enables the search for a specified frame type.
/// HIPAR frameIdentifier/Enables the search for frame indentifier.
/// HIPAR identifierData/Enables the search for identifier and data.
/// HIPAR errorCondition/Enables the search for a specified error.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchCriteria(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean startOfFrame,
                                                     ViBoolean frameType,
                                                     ViBoolean frameIdentifier,
                                                     ViBoolean identifierData,
                                                     ViBoolean errorCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:SSOF '%s',%s", searchName, startOfFrame ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:SFTY '%s',%s", searchName, frameType ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:SFID '%s',%s", searchName, frameIdentifier ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:SIDD '%s',%s", searchName, identifierData ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:SERR '%s',%s", searchName, errorCondition ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the symbolic search and disables all other
/// HIFN  search criteria.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables the symbolic search and disables all other search criteria.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchSymbolic(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the CAN frame type to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameType/Selects the CAN frame type to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchFrameType(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frameType, RSSCOPE_VAL_CAN_FRAME_TYPE_DATA, RSSCOPE_VAL_CAN_FRAME_TYPE_ANY),
            3, "Frame Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:FTYP '%s',%s", searchName, protocolCANFrameType_Arr[frameType]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the CAN standard. Use Any if the standard of the
/// HIFN  signal is unknown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR standard/Selects the CAN standard.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchStandard(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR idType/Selects the format of data and remote frames.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR idMin/Specifies a message identifier pattern, or sets the start value of
/// HIPAR idMin/an identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANSearchID(ViSession instrSession,
                                               ViString  searchName,
                                               ViInt32   idType,
                                               ViInt32   condition,
                                               ViString  idMin,
                                               ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, idType, RSSCOPE_VAL_CAN_FRAME_ID_TYPE_B11, RSSCOPE_VAL_CAN_FRAME_ID_TYPE_ANY),
            3, "ID Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            4, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:ITYP '%s',%s", searchName, protocolCANIDType_Arr[idType]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:ICON '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:IMIN '%s','%s'", searchName, idMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:IMAX '%s','%s'", searchName, idMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CAN FD-specific bits search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fdf/Specifies the CAN FD frame format. It corresponds to the EDL bit
/// HIPAR fdf/(extended data length), which only exists in CAN FD format.
/// HIPAR brs/Sets the bit rate switch bit.
/// HIPAR esi/Sets the error state indicator bit.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchFDBits(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   fdf,
                                                   ViInt32   brs,
                                                   ViInt32   esi)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_FD_BITS_FDF, fdf),
            3, "fdf");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_FD_BITS_BRS, brs),
            4, "brs");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_FD_BITS_ESI, esi),
            4, "esi");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR transfer/Selects the byte order (endianness) of the data transfer.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR dataMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR dataMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANSearchData(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   transfer,
                                                 ViInt32   condition,
                                                 ViString  dataMin,
                                                 ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, transfer, RSSCOPE_VAL_CAN_FRAME_ID_TYPE_B11, RSSCOPE_VAL_CAN_FRAME_ID_TYPE_ANY),
            3, "Transfer");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            4, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:BORD '%s',%s", searchName, transfer_Arr[transfer]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:DCON '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:DMIN '%s','%s'", searchName, dataMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:DMAX '%s','%s'", searchName, dataMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for DLC trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Operator to set the data length code for searching on CAN data. For
/// HIPAR condition/big endian transfer direction, you can search for a number of bytes
/// HIPAR condition/less than the DLC of the frame, that means, on the beginning of the
/// HIPAR condition/data pattern. For little endian transfer direction, the exact number of
/// HIPAR condition/data bytes in the frame must be set.
/// HIPAR dcl/Sets the Data Length Code, the number of data bytes to be found
ViStatus _VI_FUNC rsscope_ConfigureCANSearchDlc(ViSession instrSession,
                                                ViString  searchName,
                                                ViInt32   condition,
                                                ViInt32   dlc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:DLCC '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:DLC '%s',%ld", searchName, dlc);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the location in the data field where the instrument
/// HIFN  looks for the specified data pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the data position if DLC => 9. You can define an
/// HIPAR condition/exact position, or a position range.
/// HIPAR positionMin/Defines the first possible start position of the data pattern.
/// HIPAR positionMax/Defines the last possible start position of the data pattern if
/// HIPAR positionMax/"Condition" is "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANSearchDataPosition(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   condition,
                                                         ViInt32   positionMin,
                                                         ViInt32   positionMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_POSITION_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_POSITION_MIN, positionMin),
            4, "Position Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_POSITION_MAX, positionMax),
                5, "Position Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR crc/Sets state of triggering on the CRC error. A CRC error occurs
/// HIPAR stuffing/Sets state of triggering on the bit stuffing error. A stuff error
/// HIPAR stuffing/occurs when the 6th consecutive equal bit level in the mentioned fields
/// HIPAR stuffing/is detected
/// HIPAR form/Sets state of triggering on the form error. A form error occurs when a
/// HIPAR form/fixed-form bit field contains one or more illegal bits.
/// HIPAR ack/Sets state of triggering on the form error. An acknowledgement error
/// HIPAR ack/occurs when the transmitter does not receive an acknowledgment - a
/// HIPAR ack/dominant bit during the Ack Slot.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchErrorConditions(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViBoolean crc,
                                                            ViBoolean stuffing,
                                                            ViBoolean form,
                                                            ViBoolean ack)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:CRC '%s',%s", searchName, crc ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:BITS '%s',%s", searchName, stuffing ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:FORM '%s',%s", searchName, form ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:ACK '%s',%s", searchName, ack ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers on stuff count errors. A stuff count error occurs if th
/// HIFN  received stuff count value does not match the value calculated from the
/// HIFN  own stuff bit count.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIFN
/// HIFN     (2) This function sets trigger type to Error Condition
/// HIFN
/// HIFN     (3) The R&S RTO detects errors in the message and triggers on these
/// HIFN  errors even if no CAN node sends an error flag.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_CAN_STUFF_COUNT_ERROR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CAN:FDATa:SCERror
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR stuffCountError/Triggers on stuff count errors. A stuff count error occurs if th
/// HIPAR stuffCountError/received stuff count value does not match the value calculated from the
/// HIPAR stuffCountError/own stuff bit count.
ViStatus _VI_FUNC rsscope_ConfigureCANSearchStuffCountError(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViBoolean stuffCountError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CAN:FDAT:SCER '%s',%s", searchName,
             stuffCountError ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures symbolic search message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR message/Sets the message to be triggered or searched for.
/// HIPAR searchOnSignal/Enables the trigger on a specific signal value that is part of the
/// HIPAR searchOnSignal/selected message.
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicSearchMessage(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViString  message,
                                                            ViBoolean searchOnSignal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_MESSAGE, message),
            3, "Message");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_SEARCH_FOR_SIGNAL, searchOnSignal),
            4, "Search On Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures symbolic signal search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR signal/Sets the signal name to be triggered or searched for.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR dataMin/Defines the minimum data value of the signal.
/// HIPAR dataMax/Defines the maximum data value of the signal. This value is required
/// HIPAR dataMax/to specify a range if condition is "In range" or "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicSearchSignal(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViString  signal,
                                                           ViInt32   condition,
                                                           ViReal64  dataMin,
                                                           ViReal64  dataMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_SIGNAL_VALUE, signal),
            3, "Signal");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_DATA_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_SIGNAL_MIN, dataMin),
            5, "Data Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_SIGNAL_MAX, dataMax),
                6, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets a symbolic data value for signals with enumerated
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enumValue/Sets a symbolic data value for signals with enumerated values.
ViStatus _VI_FUNC rsscope_ConfigureCANSymbolicSearchSignalEnumValue(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   enumValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CAN_SYMBOLIC_SIGNAL_ENUM_VALUE, enumValue),
            3, "Enum Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FCO? '%s'", searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameStatus/The overall state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the frame type of the selected frame.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameStatus(ViSession instrSession,
                                                    ViInt32   frame,
                                                    ViString  searchName,
                                                    ViInt32*  frameStatus,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameStatus_Arr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR idType/Returns the identifier type of the selected frame, the identifier
/// HIPAR idType/format of data and remote frames.
/// HIPAR idValue/Returns the identifier value of the selected frame.
/// HIPAR statusCode/Return the identifier state.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameID(ViSession instrSession,
                                                ViInt32   frame,
                                                ViString  searchName,
                                                ViInt32*  idType,
                                                ViInt32*  idValue,
                                                ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:IDTY? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANIDType_Arr, idType));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:IDV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, idValue));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:IDST? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the parameters of selected field of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the field state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameField(ViSession instrSession,
                                                   ViInt32   frame,
                                                   ViString  searchName,
                                                   ViInt32   field,
                                                   ViInt32*  fieldValue,
                                                   ViInt32*  fieldStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString fieldArr[] = {"DLC", "CS", "ACK"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_CAN_FIELD_DLC, RSSCOPE_VAL_CAN_FIELD_ACK),
            4, "Field");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:%sV? '%s'", frame, fieldArr[field], searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:%sST? '%s'", frame, fieldArr[field], searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, fieldStatus));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the location of a bit stuffing error of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorPosition/Returns the location of a bit stuffing error.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameBitStuffingError(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViString  searchName,
                            ViReal64* errorPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:BSEP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, errorPosition));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR searchName/Specifies search definition.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the state of the specified byte.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameData(ViSession instrSession,
                                                  ViInt32   frame,
                                                  ViInt32   byteNumber,
                                                  ViString  searchName,
                                                  ViInt32*  data,
                                                  ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:BYTE%ld:VAL? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:BYTE%ld:STAT? '%s'", frame, byteNumber, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFieldStatus_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameAllData(ViSession instrSession,
                                                     ViInt32   frame,
                                                     ViString  searchName,
                                                     ViInt32   arraySize,
                                                     ViInt32   data[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the symbolic data of the selected result frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Result parameter.
/// HIPAR result/String with comma-separated list of symbolic data.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameSymbolicData(ViSession instrSession,
                                                          ViInt32   frame,
                                                          ViString  searchName,
                                                          ViInt32   bufferSize,
                                                          ViChar    result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K3|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:SDAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(result, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameLabel(ViSession instrSession,
                                                   ViInt32   frame,
                                                   ViString  searchName,
                                                   ViInt32   bufferSize,
                                                   ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the CAN protocol standard: CAN or CAN FD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameStandard/Returns the CAN protocol standard: CAN or CAN FD.
ViStatus _VI_FUNC rsscope_QueryCANFDSearchFrameStandard(ViSession instrSession,
                                                        ViInt32   frame,
                                                        ViString  searchName,
                                                        ViInt32*  frameStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FDAT:FRAM%ld:STAN? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFDSearchFrameStandard_Arr, frameStandard));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of data bytes defined by DLC. DLC and NDB are
/// HIFN  different in CAN FD for DLCs > 8.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR numberOfBytes/Returns the number of data bytes defined by DLC. DLC and NDB are
/// HIPAR numberOfBytes/different in CAN FD for DLCs > 8.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameNumberOfBytes(ViSession instrSession,
                                                           ViInt32   frame,
                                                           ViString  searchName,
                                                           ViInt32*  numberOfBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:NDBY? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfBytes));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns information on a form error, if the frame status query
/// HIFN  returned a FORM error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR formErrorCause/Returns information on a form error, if the frame status query
/// HIPAR formErrorCause/returned a FORM error.
ViStatus _VI_FUNC rsscope_QueryCANSearchFrameFormErrorCause(ViSession instrSession,
                                                            ViInt32   frame,
                                                            ViString  searchName,
                                                            ViInt32*  formErrorCause)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FRAM%ld:FERC? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCANFrameFormErrorCause_Arr, formErrorCause));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the stuff bit count modulo 8.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR stuffCount/Returns the stuff bit count modulo 8.
ViStatus _VI_FUNC rsscope_QueryCANFDSearchFrameStuffCount(ViSession instrSession,
                                                          ViInt32   frame,
                                                          ViString  searchName,
                                                          ViInt32*  stuffCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CAN:FDAT:FRAM%ld:SCV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, stuffCount));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures MSO source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/Selects the line.
/// HIPAR waveformSource/Selects the source of the waveform.
/// HIPAR channel/This control selects the channel on which the signal is measured.
/// HIPAR separateHeaderBits/If ON, the leading five indicator bits of the header are decoded as five single bits.
/// HIPAR separateHeaderBits/Otherwise, the indicator bits are shown as one word with word length five bit
/// HIPAR bitRate/This control selects the bit rate.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySource(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   line,
                                                 ViInt32   waveformSource,
                                                 ViInt32   channel,
                                                 ViBoolean separateHeaderBits,
                                                 ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_bus[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_line[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap_bus),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SOURCE,
            "Line", line, RS_REPCAP_BUF_SIZE, repCap_line),
            3, "Line");

    if ((waveformSource == RSSCOPE_VAL_PROTOCOL_WAV_NONE) && (line != RSSCOPE_VAL_FLEXRAY_SOURCE_ENABLE))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveformSource), 4, "Waveform Source");
    }

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_bus, repCap_line);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SOURCE, waveformSource),
            4, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_CHANNEL, channel),
            5, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SEPARATE_HEADER_BITS, separateHeaderBits),
            6, "Separate Header Bits");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_bus, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_BIT_RATE, bitRate),
            7, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the type of measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sourceType/Selects the source type.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySourceType(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   sourceType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SOURCE_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SOURCE_TYPE, sourceType),
            3, "Source Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures threshold value for digitization of signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs a default threshold voltage.
/// HIPAR polarity/This control selects the wire on which the bus signal is measured in case of SINGle source type.
/// HIPAR manualThresholdHigh/Sets user-defined upper value of the threshold.
/// HIPAR manualThresholdLow/Sets user-defined lower value of the threshold.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayThresholdSingle(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   polarity,
                                                          ViInt32   preset,
                                                          ViReal64  manualThresholdHigh,
                                                          ViReal64  manualThresholdLow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SINGLE_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SINGLE_POLARITY, polarity),
            3, "Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SINGLE_PRESET_TRESHOLD, preset),
            4, "Preset");

    if (preset == RSSCOPE_VAL_FLEXRAY_SINGLE_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_HIGH, manualThresholdHigh),
                5, "Manual Threshold High");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_LOW, manualThresholdLow),
                6, "Manual Threshold Low");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures threshold value for digitization of signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs a default threshold voltage.
/// HIPAR manualThresholdHigh/Sets user-defined upper value of the threshold.
/// HIPAR manualThresholdLow/Sets user-defined lower value of the threshold.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayThresholdDifferential(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   preset,
                            ViReal64  manualThresholdHigh,
                            ViReal64  manualThresholdLow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_DIFF_PRESET_TRESHOLD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_DIFF_PRESET_TRESHOLD, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_FLEXRAY_DIFF_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_HIGH, manualThresholdHigh),
                4, "Manual Threshold High");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_LOW, manualThresholdLow),
                5, "Manual Threshold Low");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures threshold value for digitization of signals from logic source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/This control selecs a default threshold voltage.
/// HIPAR manualThresholdDataLine/Sets the threshold for the data line.
/// HIPAR manualThresholdEnableLine/Sets the threshold for the enable line.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayThresholdLogic(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   preset,
                                                         ViReal64  manualThresholdDataLine,
                                                         ViReal64  manualThresholdEnableLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_LOGIC_PRESET_TRESHOLD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_LOGIC_PRESET_TRESHOLD, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_FLEXRAY_LOGIC_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_DATA, manualThresholdDataLine),
                4, "Manual Threshold Data Line");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_THRESHOLD_ENABLE, manualThresholdEnableLine),
                5, "Manual Threshold Enable Line");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the trigger type for FlexRay analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for FlexRay analysis.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerType(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets to trigger on one or more indicator bits at the beginning of the header segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR payloadPreamble/Triggers on the payload preamble indicator bit that indicates
/// HIPAR payloadPreamble/a Network Management Vector in the payload segment.
/// HIPAR nullFrame/Triggers on the null frame indicator bit, a frame without usable data.
/// HIPAR syncFrame/Triggers on the sync frame used for synchronization of the FlexRay system.
/// HIPAR startupFrame/Triggers on startup frames used for startup of the network.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerIndicatorBits(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   payloadPreamble,
                            ViInt32   nullFrame,
                            ViInt32   syncFrame,
                            ViInt32   startupFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PAYLOAD_PREAMBLE, payloadPreamble),
            3, "Payload Preamble");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_NULL_FRAME, nullFrame),
            4, "Null Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_SYNC_FRAME, syncFrame),
            5, "Sync Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_STARTUP_FRAME, startupFrame),
            6, "Startup Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures conditions for frame ID trigger
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a frame ID or a frame ID range.
/// HIPAR IDMin/Specifies a frame identifier pattern - the number of the slot - or sets the start value
/// HIPAR IDMin/of an identifier range.
/// HIPAR IDMax/Sets the end value of an identifier range if "Condition" is "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerFrameID(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   condition,
                                                         ViString  IDMin,
                                                         ViString  IDMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_FRAME_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PATTERN_FRAME_MIN, IDMin),
            4, "ID Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PATTERN_FRAME_MAX, IDMax),
                5, "ID Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures conditions for payload length trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator for the payload length trigger setting.
/// HIPAR payloadMin/Specifies a payload length - the number of words in the payload segment -
/// HIPAR payloadMin/or sets the start value of an payload length range.
/// HIPAR payloadMax/Sets the end value of a payload length range if "Condition"
/// HIPAR payloadMax/is "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerPayloadLength(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViInt32   payloadMin,
                            ViInt32   payloadMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PAYLOAD_LENGTH_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PAYLOAD_MIN, payloadMin),
            4, "payloadMin");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PAYLOAD_MAX, payloadMax),
                5, "Payload Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures conditions for cycle count trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to define a cycle count or a cycle count range.
/// HIPAR cycleMin/Specifies a cycle count - the number of the current FlexRay cycle -
/// HIPAR cycleMin/or sets the start value of an cycle count range.
/// HIPAR cycleMax/Sets the end value of a cycle count range if "Condition" is "In range" and "Out of range".
/// HIPAR cycleStep/Specifies a step to trigger on each n-th cycle inside the given range.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerCycleCount(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   condition,
                                                            ViInt32   cycleMin,
                                                            ViInt32   cycleMax,
                                                            ViInt32   cycleStep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_CYCLE_COUNT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_CYCLE_MIN, cycleMin),
            4, "Cycle Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_CYCLE_MAX, cycleMax),
                5, "Cycle Max");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_CYCLE_STEP, cycleStep),
                5, "Cycle Step");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern range.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerData(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   condition,
                                                      ViString  dataMin,
                                                      ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PATTERN_DATA_MIN, dataMin),
            4, "Data Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PATTERN_DATA_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the position of the first byte of data bit pattern within the payload segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Sets the operator for the payload length trigger setting.
/// HIPAR condition/Document parameter here.
/// HIPAR positionMin/Sets the number of data bytes to be skipped after start of the payload segment.
/// HIPAR positionMax/Defines the last byte of interest,if "Condition" is "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerDataPosition(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViInt32   positionMin,
                            ViInt32   positionMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_DATA_POSITION_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_DATA_POSITION_MIN, positionMin),
            4, "Position Min");

    if (condition == RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_DATA_POSITION_MAX, positionMax),
                5, "Position Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function triggers on a symbol or on a wakeup pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR symbol/Selects the trigger on symbol or on a wakeup pattern.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerSymbol(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   symbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_SYMBOL,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_SYMBOL, symbol),
            3, "Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR FSS/Triggers on Error in a Frame Start Sequence.
/// HIPAR BSS/Triggers on error in ByteStart Sequence.
/// HIPAR FES/Triggers on error in Frame End Sequence.
/// HIPAR headerCRC/Triggers on error in the Cyclic Redundancy Check of the header data.
/// HIPAR payloadCRC/Triggers on error in the Cyclic Redundancy Check of the payload data.
ViStatus _VI_FUNC rsscope_ConfigureFlexRayTriggerErrorConditions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean FSS,
                            ViBoolean BSS,
                            ViBoolean FES,
                            ViBoolean headerCRC,
                            ViBoolean payloadCRC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "TrigA";

    checkErr(RsCore_LockSession(instrSession));

    if (trigger)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_A),
                2, "Trigger");

    checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_TYPE, RSSCOPE_VAL_FLEXRAY_ERR_COND));
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_FRAME_START_SEQUENCE_ERROR, FSS),
            3, "FSS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_BYTE_START_SEQUENCE_ERROR, BSS),
            4, "BSS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_FRAME_END_SEQUENCE_ERROR, FES),
            5, "FES");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_HEADER_CRC_ERROR, headerCRC),
            6, "Header CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_TRIGGER_PAYLOAD_CRC_ERROR, payloadCRC),
            7, "Payload CRC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryFlexRayFrameCount(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the number of the current FlexRay cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns number of the current FlexRay cycles.
ViStatus _VI_FUNC rsscope_QueryFlexRayCycleCount(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    /*       ViChar repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

*/


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:FLXR:FRAM%ld:CYC?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the frame.
/// HIPAR frameStart/Returns time of frame start.
/// HIPAR frameStop/Returns time of frame stop.
/// HIPAR frameType/Returns the frame type of the selected frame.
ViStatus _VI_FUNC rsscope_QueryFlexRayFrameStatus(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32*  frameStatus,
                                                  ViReal64* frameStart,
                                                  ViReal64* frameStop,
                                                  ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_STATUS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_STATUS, frameStatus),
            4, "Frame Status");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_START, frameStart),
            5, "Frame Start");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_STOP, frameStop),
            6, "Frame Stop");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_TYPE, frameType),
            7, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the parameters of selected field of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the field state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryFlexRayFrameField(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32   field,
                                                 ViInt32*  fieldValue,
                                                 ViInt32*  fieldStatus)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr    arrFieldValueAttr[] = {
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_FLAGS,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_PAYLOAD_LENGTH,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_IDENTIFIER,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_HEADER_CRC,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_CRC
              };
    ViAttr    arrFieldStatusAttr[] = {
    VI_FALSE, VI_FALSE, VI_FALSE,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_HEADER_CRC_STATUS,
              RSSCOPE_ATTR_PROTOCOL_FLEXRAY_FRAME_CRC_STATUS
              };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_FLEXRAY_FIELD_FLAGS, RSSCOPE_VAL_FLEXRAY_FIELD_FCRC),
            4, "Field");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrFieldValueAttr[field],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_FLEXRAY_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, arrFieldValueAttr[field], fieldValue),
            5, "Field Value");

    if (arrFieldStatusAttr[field])
    {
        viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, arrFieldStatusAttr[field], fieldStatus),
                6, "Field Status");
    }
    else
    {
        if (fieldStatus)
            *fieldStatus = -1;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryFlexRayFrameAllData(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   arraySize,
                                                   ViInt32   data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:FLXR:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryFlexRayFrameLabel(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32   bufferSize,
                                                 ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:FLXR:FRAM%ld:SYMB?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function includes the search conditions for the FlexRay trigger
/// HIFN  event type in the next search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Includes the search conditions for the FlexRay trigger event type in
/// HIPAR enabled/the next search.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchState(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:STAT '%s',%s", searchName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies the trigger event configuration from Trigger A
/// HIFN  for the selected FlexRay bus to the search condition settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
ViStatus _VI_FUNC rsscope_FlexRaySearchCopyFromA(ViSession instrSession,
                                                 ViString  searchName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_FLEXRAY_COPY_FROM_A, searchName),
            2, "search Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the criteria or combination of criterias to be
/// HIFN  searched for. When more than one criteria is selected, all criterias
/// HIFN  must be fulfilled by a frame for it to be shown in the
/// HIFN     search results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startOfFrame/Enables the search for a start of frame.
/// HIPAR symbol/Enables the search for specified symbol.
/// HIPAR identifierData/Enables the search for identifier and data.
/// HIPAR errorCondition/Enables the search for specified error or error combination.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchCriteria(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean startOfFrame,
                                                         ViBoolean symbol,
                                                         ViBoolean identifierData,
                                                         ViBoolean errorCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SSOF '%s',%s", searchName, startOfFrame ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SSYM '%s',%s", searchName, symbol ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SIDD '%s',%s", searchName, identifierData ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SERR '%s',%s", searchName, errorCondition ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets to trigger on one or more indicator bits at the
/// HIFN  beginning of the header segment. Each bit can be set to 0, 1, or X
/// HIFN  (don't care).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR payloadPreamble/Triggers on the payload preamble indicator bit that indicates a
/// HIPAR payloadPreamble/Network Management Vector in the payload segment.
/// HIPAR nullFrame/Triggers on the null frame indicator bit, a frame without usable data.
/// HIPAR syncFrame/Triggers on the sync frame used for synchronization of the FlexRay
/// HIPAR syncFrame/system. Only sync nodes can send this frame type.
/// HIPAR startupFrame/Triggers on startup frames used for startup of the network. Only
/// HIPAR startupFrame/specific start nodes can send this frame type.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchIndicatorBits(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   payloadPreamble,
                            ViInt32   nullFrame,
                            ViInt32   syncFrame,
                            ViInt32   startupFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, payloadPreamble, RSSCOPE_VAL_FLEXRAY_BITS_ZERO, RSSCOPE_VAL_FLEXRAY_BITS_DC),
            3, "Payload Preamble");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, nullFrame, RSSCOPE_VAL_FLEXRAY_BITS_ZERO, RSSCOPE_VAL_FLEXRAY_BITS_DC),
            4, "Null Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, syncFrame, RSSCOPE_VAL_FLEXRAY_BITS_ZERO, RSSCOPE_VAL_FLEXRAY_BITS_DC),
            5, "Sync Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startupFrame, RSSCOPE_VAL_FLEXRAY_BITS_ZERO, RSSCOPE_VAL_FLEXRAY_BITS_DC),
            6, "Startup Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:PLPR '%s',%s", searchName, indicatorBits_Arr[payloadPreamble]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:NUFR '%s',%s", searchName, indicatorBits_Arr[nullFrame]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SYFR '%s',%s", searchName, indicatorBits_Arr[syncFrame]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:STFR '%s',%s", searchName, indicatorBits_Arr[startupFrame]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for frame ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a frame ID or a frame ID range.
/// HIPAR idMin/Specifies a frame identifier pattern - the number of the slot - or
/// HIPAR idMin/sets the start value of an identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" is "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchFrameID(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   condition,
                                                        ViString  idMin,
                                                        ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:FCON '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:FMIN '%s','%s'", searchName, idMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:FMAX '%s','%s'", searchName, idMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for payload length trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the payload length trigger setting. You can
/// HIPAR condition/defined an exact value, or a range.
/// HIPAR payloadMin/Specifies a payload length - the number of words in the payload
/// HIPAR payloadMin/segment - or sets the start value of an payload length range.
/// HIPAR payloadMin/Information is transmitted in 2-byte words, so the number of data bytes
/// HIPAR payloadMin/in the payload segment is twice the payload  length.
/// HIPAR payloadMax/Sets the end value of a payload length range if "Condition" is "In
/// HIPAR payloadMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchPayloadLength(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViInt32   payloadMin,
                            ViInt32   payloadMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:PCON '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:PMIN '%s',%ld", searchName, payloadMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:PMAX '%s',%ld", searchName, payloadMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for cycle count trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to define a cycle count or a cycle count range.
/// HIPAR cycleMin/Specifies a cycle count - the number of the current FlexRay cycle - or
/// HIPAR cycleMin/sets the start value of an cycle count range.
/// HIPAR cycleMax/Sets the end value of a cycle count range if "Condition" is "In
/// HIPAR cycleMax/range" and "Out of range".
/// HIPAR cycleStep/Specifies a step to trigger on each n-th cycle inside the given range.
/// HIPAR cycleStep/This allows for specific triggering if slot multiplexing is used.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchCycleCount(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   condition,
                                                           ViInt32   cycleMin,
                                                           ViInt32   cycleMax,
                                                           ViInt32   cycleStep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:CEN '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:CMIN '%s',%ld", searchName, cycleMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:CMAX '%s',%ld", searchName, cycleMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:CST '%s',%ld", searchName, cycleStep);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR dataMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR dataMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR dataMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR dataMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchData(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   condition,
                                                     ViString  dataMin,
                                                     ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DCON '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DMIN '%s','%s'", searchName, dataMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DMAX '%s','%s'", searchName, dataMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the position of the first byte of data bit pattern
/// HIFN  within the payload segment. You can define an exact position, or a
/// HIFN  position range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the payload length trigger setting. You can
/// HIPAR condition/defined an exact value, or a range.
/// HIPAR positionMin/Sets the number of data bytes to be skipped after start of the payload
/// HIPAR positionMin/segment.
/// HIPAR positionMax/Defines the last byte of interest, if "Condition" is "In range" and
/// HIPAR positionMax/"Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchDataPosition(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   condition,
                                                             ViInt32   positionMin,
                                                             ViInt32   positionMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DPOP '%s',%s", searchName, condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DPOS '%s',%ld", searchName, positionMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DPTO '%s',%ld", searchName, positionMax);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Reserved for future use.
/// HIPAR idMin/Specifies a message identifier pattern, or sets the start value of
/// HIPAR idMin/an identifier range.
/// HIPAR idMax/Sets the end value of an identifier range if "Condition" "In
/// HIPAR idMax/range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchID(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   reserved,
                                                   ViString  idMin,
                                                   ViString  idMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    /*
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
                3, "Condition");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:DPOP '%s',%s", searchName, condition_Arr[condition]);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    */
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:IMIN '%s','%s'", searchName, idMin);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    //  if (condition >= RSSCOPE_VAL_COND_IN)
    //  {
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:IMAX '%s','%s'", searchName, idMax);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    //  }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function triggers on a symbol or on a wakeup pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR symbol/Selects the trigger on symbol or on a wakeup pattern.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchSymbol(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   symbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, symbol, RSSCOPE_VAL_FLEXRAY_SYMBOL_CASMTS, RSSCOPE_VAL_FLEXRAY_SYMBOL_WAKEUP),
            3, "Symbol");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:SYMB '%s',%s", searchName, symbol_Arr[symbol]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fss/Triggers on Error in a Frame Start Sequence. FSS follows the
/// HIPAR fss/Transmission Start Sequence TSS at the beginning of each frame.
/// HIPAR bss/Triggers on error in ByteStart Sequence. The BSS is transmitted before
/// HIPAR bss/each byte.
/// HIPAR fes/Triggers on error in Frame End Sequence. FES indicates the end of each
/// HIPAR fes/frame.
/// HIPAR headerCRC/Triggers on error in the Cyclic Redundancy Check of the header data
/// HIPAR headerCRC/(mainly frame ID and payload length).
/// HIPAR payloadCRC/Triggers on error in the Cyclic Redundancy Check of the payload data.
ViStatus _VI_FUNC rsscope_ConfigureFlexRaySearchErrorConditions(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean fss,
                            ViBoolean bss,
                            ViBoolean fes,
                            ViBoolean headerCRC,
                            ViBoolean payloadCRC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:FSS '%s',%s", searchName, fss ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:BSS '%s',%s", searchName, bss ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:FES '%s',%s", searchName, fes ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:HCRC '%s',%s", searchName, headerCRC ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:FLXR:PCRC '%s',%s", searchName, payloadCRC ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchFrameCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FCO? '%s'", searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of the current FlexRay cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns number of the current FlexRay cycles.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchCycleCount(ViSession instrSession,
                                                       ViInt32   frame,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:CYC? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameStatus/Returns the overall state of the frame.
/// HIPAR frameStart/Returns time of frame start.
/// HIPAR frameStop/Returns time of frame stop.
/// HIPAR frameType/Returns the frame type of the selected frame.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchFrameStatus(ViSession instrSession,
                                                        ViInt32   frame,
                                                        ViString  searchName,
                                                        ViInt32*  frameStatus,
                                                        ViReal64* frameStart,
                                                        ViReal64* frameStop,
                                                        ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolFlexRayStatus_Arr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolFlexRayType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the parameters of selected field of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the field state of the selected frame.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchFrameField(ViSession instrSession,
                                                       ViInt32   frame,
                                                       ViString  searchName,
                                                       ViInt32   field,
                                                       ViInt32*  fieldValue,
                                                       ViInt32*  fieldStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString fieldArr[] = {"FLAG", "PAYL", "ADID", "CSV", "FCV"};
    ViString fieldStArr[] = {"", "", "", "CSST", "FCST"};
    ViString outputArr[] = {"OK", "UVAL", "INS", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, field, RSSCOPE_VAL_FLEXRAY_FIELD_FLAGS, RSSCOPE_VAL_FLEXRAY_FIELD_FCRC),
            4, "Field");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:%s? '%s'", frame, fieldArr[field], searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    checkErr(rsscope_CheckStatus(instrSession));

    if (field == RSSCOPE_VAL_FLEXRAY_FIELD_HCRC || field == RSSCOPE_VAL_FLEXRAY_FIELD_FCRC)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:%s? '%s'", frame, fieldStArr[field], searchName);
        checkErr(RsCore_QueryStringIndex(instrSession, cmd, outputArr, fieldStatus));
        checkErr(rsscope_CheckStatus(instrSession));
    }
    else
        *fieldStatus = -1;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/wReturns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchFrameAllData(ViSession instrSession,
                                                         ViInt32   frame,
                                                         ViString  searchName,
                                                         ViInt32   arraySize,
                                                         ViInt32   data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Label'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryFlexRaySearchFrameLabel(ViSession instrSession,
                                                       ViInt32   frame,
                                                       ViString  searchName,
                                                       ViInt32   bufferSize,
                                                       ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:FLXR:FRAM%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the search type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the search type for ARINC 429 analysis.
ViStatus _VI_FUNC rsscope_ConfigureARINC429SearchType(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the values for the source/destination
/// HIFN  identifier (SDI) and the sign/status matrix (SSM) bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sdi/Sets the source/destination identifier (SDI) bits.
/// HIPAR ssm/Sets the sign/status matrix (SSM) bits.
ViStatus _VI_FUNC rsscope_ConfigureARINC429SearchLabelData(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   sdi,
                                                           ViInt32   ssm)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_LABEL_DATA_SDI, sdi),
            3, "SDI");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_LABEL_DATA_SSM, ssm),
            4, "SSM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures ARINC 429 search label pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern.
/// HIPAR labelMin/Specifies a data pattern or sets the start value of a data pattern
/// HIPAR labelMin/range.
/// HIPAR labelMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR labelMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureARINC429SearchLabelPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  labelMin,
                            ViString  labelMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_LABEL_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_LABEL_PATTERN_MIN, labelMin),
            4, "Label Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_LABEL_PATTERN_MAX, labelMax),
                5, "Label Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures ARINC 429 search data pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the data. You can define an exact data patterm
/// HIPAR condition/or a data range.
/// HIPAR dataMin/Specifies a data pattern or sets the start value of a data pattern
/// HIPAR dataMin/range.
/// HIPAR dataMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR dataMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureARINC429SearchDataPattern(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   condition,
                                                             ViString  dataMin,
                                                             ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_DATA_PATTERN_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_DATA_PATTERN_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the error conditions to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR codingError/Enables the search for coding errors.
/// HIPAR parityError/Enables the search for parity errors.
/// HIPAR timingError/Enables the search for timing errors, when the minimum gap is out of
/// HIPAR timingError/range.
ViStatus _VI_FUNC rsscope_ConfigureARINC429SearchErrorCondition(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean codingError,
                            ViBoolean parityError,
                            ViBoolean timingError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_ERROR_CODING_ERROR, codingError),
            3, "Coding Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_ERROR_PARITY_ERROR, parityError),
            4, "Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_ERROR_TIMING_ERROR, timingError),
            5, "TimingError");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded words.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of decoded words.
ViStatus _VI_FUNC rsscope_QueryARINC429SearchWordCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ARINC429_WORDS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR status/Returns the overall state of the selected word.
/// HIPAR start/Return the start time of the selected word.
/// HIPAR stop/Return the stop time of the selected word.
/// HIPAR pattern/Returns all 32 bits of the specified word.
/// HIPAR sdi/Returns the source/destination identifier (SDI) bits of the specified
/// HIPAR sdi/word.
/// HIPAR ssm/Returns the sign/status matrix (SSM) bits of the specified word.
ViStatus _VI_FUNC rsscope_QueryARINC429SearchStatus(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   word,
                                                    ViInt32*  status,
                                                    ViReal64* start,
                                                    ViReal64* stop,
                                                    ViInt32*  pattern,
                                                    ViInt32*  sdi,
                                                    ViInt32*  ssm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:STAT? '%s'", word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolARINCStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:STAR? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:STOP? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:PATT? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, pattern));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:SDI? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sdi));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:SSM? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, ssm));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryARINC429SearchAllData(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   word,
                                                     ViInt32   arraySize,
                                                     ViInt32   data[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:DATA? '%s'", word, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the label name of the word ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR labelName/Returns the label name of the word ID.
/// HIPAR label/Returns the label of the specified word.
ViStatus _VI_FUNC rsscope_QueryARINC429SearchLabel(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   word,
                                                   ViInt32   bufferSize,
                                                   ViChar    labelName[],
                                                   ViInt32*  label)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:SYMB? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(labelName, bufferSize, extBuf);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ARIN:WORD%ld:LAB? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, label));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the source of the RFFE data line. For triggering on a serial bus,
/// HIFN  analog or digital input channels are required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select RFFE line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureRFFESource(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   line,
                                              ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_PROTOCOL_RFFE_CLOCK_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_RFFE_DATA_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SOURCE_CLOCK, RSSCOPE_VAL_SOURCE_DATA),
            3, "Line");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                4, "Waveform Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, attr[line], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIFN     If the signal value is higher than the threshold, the signal state is
/// HIFN  high (logic 1). Otherwise, the signal state is considered low (logic
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets default threshold voltages for 1.2 V or 1.8 V bus. Set the value
/// HIPAR preset/to Manual if you want to define individual thresholds.
/// HIPAR line/This control select RFFE line.
/// HIPAR thresholdType/Selects the threshold to be set.
/// HIPAR threshold/Sets the threshold or hysteresis.
ViStatus _VI_FUNC rsscope_ConfigureRFFEThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   preset,
                                                 ViInt32   line,
                                                 ViInt32   thresholdType,
                                                 ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   thresholdLowAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_RFFE_CLOCK_THRESHOLD_LOW,
             RSSCOPE_ATTR_PROTOCOL_RFFE_DATA_THRESHOLD_LOW
             };
    ViAttr   thresholdHighAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_RFFE_CLOCK_THRESHOLD_HIGH,
             RSSCOPE_ATTR_PROTOCOL_RFFE_DATA_THRESHOLD_HIGH
             };
    ViAttr   hysteresisAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_RFFE_CLOCK_THRESHOLD_HYSTERESIS,
             RSSCOPE_ATTR_PROTOCOL_RFFE_DATA_THRESHOLD_HYSTERESIS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_PRESET_THRESHOLD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_PRESET_THRESHOLD, preset),
            3, "preset");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SOURCE_CLOCK, RSSCOPE_VAL_SOURCE_DATA),
            4, "Line");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, thresholdType, RSSCOPE_VAL_THRESHOLD_TYPE_HIGH, RSSCOPE_VAL_THRESHOLD_TYPE_HYSTERESIS),
            5, "Threshold Type");

    if (preset == RSSCOPE_VAL_RFFE_MAN)
    {
        if (thresholdType == RSSCOPE_VAL_THRESHOLD_TYPE_LOW)
        {
            viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, thresholdLowAttr[line], threshold),
                    6, "Threshold");
        }
        else if (thresholdType == RSSCOPE_VAL_THRESHOLD_TYPE_HIGH)
        {
            viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, thresholdHighAttr[line], threshold),
                    6, "Threshold");
        }
        else
        {
            viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, hysteresisAttr[line], threshold),
                    6, "Threshold");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the data line thresholds are set to the clock line
/// HIFN  threshold values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Enables/disables tresholds coupling.
ViStatus _VI_FUNC rsscope_ConfigureRFFEThresholdsCoupling(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_THRESHOLDS_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_THRESHOLDS_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function Enables the glitch filter on the SCLK and SDATA lines to
/// HIFN  improve decode accuracy. The "Width" field sets the maximum glitch
/// HIFN  width to be ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enable/Enables the glitch filter on the clock and data lines to improve
/// HIPAR enable/decode accuracy.
/// HIPAR width/Sets the maximum glitch width to be ignored.
ViStatus _VI_FUNC rsscope_ConfigureRFFEGlitchFilter(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViBoolean enable,
                                                    ViReal64  width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_GLITCH_FILTER_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_GLITCH_FILTER_ENABLED, enable),
            3, "Enable");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_GLITCH_FILTER_WIDTH, width),
            4, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the idle time between the Bus Park Cycle (BP)
/// HIFN  and Sequence Start Condition(SSC). You can define a minimum time "Min
/// HIFN  gap", and/or a maximum time "Max gap". The standard defines a minimum
/// HIFN  of 10 ns to separate two subsequent command sequences. Timing settings
/// HIFN  are relevant for protocol configuration and error trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enableMinimumGap/If the minimum gap is enabled, the instrument detects the specified
/// HIPAR enableMinimumGap/gap.
/// HIPAR minimumGapTime/Sets the minimum idle time between the Bus Park Cycle (BP) and
/// HIPAR minimumGapTime/Sequence Start Condition (SSC). The setting is only relevant if Enable
/// HIPAR minimumGapTime/Minimum Gap is set to true.
/// HIPAR enableMaximumGap/If the maximum gap is enabled, the instrument detects the specified
/// HIPAR enableMaximumGap/gap.
/// HIPAR maximumGapTime/Sets the maximum idle time between the Bus Park Cycle (BP) and
/// HIPAR maximumGapTime/Sequence Start Condition (SSC). The setting is only relevant if Enable
/// HIPAR maximumGapTime/Maximum Gap is set to true.
ViStatus _VI_FUNC rsscope_ConfigureRFFETiming(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViBoolean enableMinimumGap,
                                              ViReal64  minimumGapTime,
                                              ViBoolean enableMaximumGap,
                                              ViReal64  maximumGapTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_MIN_GAP_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_MIN_GAP_ENABLED, enableMinimumGap),
            3, "Enable Minimum Gap");

    if (enableMinimumGap)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_MIN_GAP_TIME, minimumGapTime),
                4, "Minimum Gap Time");
    }

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_MAX_GAP_ENABLED, enableMaximumGap),
            3, "Enable Maximum Gap");

    if (enableMaximumGap)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_MAX_GAP_TIME, maximumGapTime),
                4, "Maximum Gap Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects, if the standard or synchronous read (SREAD) mode is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR readMode/Selects, if the standard or synchronous read (SREAD) mode is used.
ViStatus _VI_FUNC rsscope_ConfigureRFFEReadMode(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   readMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_READ_MODE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_READ_MODE, readMode),
            3, "Read Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the RFFE version.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR version/Selects the RFFE version.
ViStatus _VI_FUNC rsscope_ConfigureRFFEVersion(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   version)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_VERSION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_VERSION, version),
            3, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the trigger type for RFFE analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Configures the trigger type for RFFE analysis.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the interrupt pattern for the interupt summary and notification
/// HIFN  trigger type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INTERRUPT_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:RFFE:INTerrupt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR pattern/Sets the interrupt pattern for the interupt summary and notification
/// HIPAR pattern/trigger type.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerInterruptPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INTERRUPT_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INTERRUPT_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the addess of the slave. The slave address setup
/// HIFN  consists of the condition and one or two address patterns.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerSlaveAddress(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViInt32   condition,
                                                           ViString  minimum,
                                                           ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_SLAVE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_SLAVE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_SLAVE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_SLAVE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the register address. The register address setup consists of
/// HIFN  the condition and one or two address patterns.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerAddress(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   condition,
                                                      ViString  minimum,
                                                      ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ADDRESS_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ADDRESS_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of data frames to be read or written in the command
/// HIFN  sequence. The setting is available for all "Extended Register" command
/// HIFN  sequences, which can transfer more than one data frame. /// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerByteCount(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_BYTE_COUNT_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_BYTE_COUNT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_BYTE_COUNT_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_BYTE_COUNT_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the data trigger condition. The data setup
/// HIFN  consists of the condition and one or two data patterns.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to trigger on a specific pattern or a range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerData(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   condition,
                                                   ViString  minimum,
                                                   ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_DATA_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_DATA_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the data trigger condition. The data setup
/// HIFN  consists of the condition and one or two data patterns.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerIndex(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   condition,
                                                    ViInt32   minimum,
                                                    ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INDEX_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_INDEX_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the mask trigger condition. The data setup
/// HIFN  consists of the condition and one or two data patterns.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_CONDITION
/// HIFN     RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_MIN
/// HIFN     RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:RFFE:DATA:CONDition
/// HIFN     TRIGger<m>:RFFE:MASK:MIN
/// HIFN     TRIGger<m>:RFFE:MASK:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to trigger on a mask pattern or a range.
/// HIPAR minimum/Specifies a mask pattern, or sets the start value of a mask pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an range if the mask condition is set to In
/// HIPAR maximum/Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerMask(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   condition,
                                                   ViString  minimum,
                                                   ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_RFFE_TRIGGER_COND_INR)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_MASK_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR ssc/Enables the trigger on an SSC error - when no valid SSC sequence has
/// HIPAR ssc/been found after the idle time.
/// HIPAR length/Enables the trigger on a length error - when an incorrect length of
/// HIPAR length/the command sequence has been found.
/// HIPAR busPark/Enables the trigger on a bus park error - when an incorrect bus parlk
/// HIPAR busPark/cycle has been found.
/// HIPAR parity/Enables the trigger on incorrect parity bits.
/// HIPAR noResponse/Enables the trigger on any No Response Frame.
/// HIPAR unknownSequence/Enables the trigger on an unknown sequence - when the instrument
/// HIPAR unknownSequence/cannot detect any supported command sequence.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerErrorConditions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean ssc,
                            ViBoolean length,
                            ViBoolean busPark,
                            ViBoolean parity,
                            ViBoolean noResponse,
                            ViBoolean unknownSequence)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_SCC,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_SCC, ssc),
            3, "SCC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_LENGTH, length),
            4, "length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_BUS_PARK, busPark),
            5, "Bus Park");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_PARITY, parity),
            6, "parity");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_NO_RESPONSE, noResponse),
            7, "No Response");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_UNKNOWN, unknownSequence),
            8, "Unknown Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the trigger on a gap error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR gap/Enables the trigger on a gap error.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerErrorGap(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViBoolean gap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_GAP,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_GAP, gap),
            3, "Gap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the trigger on RFFE version error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR version/Enables the trigger on RFFE version error.
ViStatus _VI_FUNC rsscope_ConfigureRFFETriggerErrorVersion(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViBoolean version)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_VERSION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_TRIGGER_ERROR_VERSION, version),
            3, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of command sequences in the current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of command sequences.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceCount(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_DECODING_RESULTS_SEQUENCE_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_RFFE_DECODING_RESULTS_SEQUENCE_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR status/Returns the overall state of the selected command sequence.
/// HIPAR start/Time of the selected command sequence start.
/// HIPAR stop/Time of the selected command sequence stop.
/// HIPAR type/Returns the type of the selected command sequence.
/// HIPAR slaveAddress/Returns the slave address of the selected command sequence.
/// HIPAR address/Returns the register address of the selected command sequence.
/// HIPAR byteCount/Returns the byte count of the selected command sequence.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceStatus(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   sequence,
                                                  ViInt32*  status,
                                                  ViReal64* start,
                                                  ViReal64* stop,
                                                  ViInt32*  type,
                                                  ViInt32*  slaveAddress,
                                                  ViInt32*  address,
                                                  ViInt32*  byteCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  rngValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:STAT?", serialBus, sequence);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceStatus, response, &rngValue));
    *status = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:STAR?", serialBus, sequence);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:STOP?", serialBus, sequence);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:TYPE?", serialBus, sequence);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceType, response, &rngValue));
    *type = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:SADD?", serialBus, sequence);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, slaveAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:ADDR?", serialBus, sequence);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:BCO?", serialBus, sequence);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, byteCount));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the symbolic label that belongs to the address
/// HIFN  of the selected command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceLabel(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   sequence,
                                                 ViInt32   bufferSize,
                                                 ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:SYMB?", serialBus, sequence);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data bytes of the specified command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the sequence.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the sequence.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceAllData(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   sequence,
                                                   ViInt32   arraySize,
                                                   ViInt32   data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:DATA?", serialBus, sequence);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte.
/// HIPAR statusCode/Returns the state of the specified byte.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceData(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   sequence,
                                                ViInt32   byteNumber,
                                                ViInt32*  data,
                                                ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:BYTE%ld:VAL?", serialBus, sequence, byteNumber);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:BYTE%ld:STAT?", serialBus, sequence, byteNumber);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolRFFEStatusCode_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the parity values for the selected command sequence.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:RFFE:SEQuence<n>:PADZero?
/// HIFN     BUS<m>:RFFE:SEQuence<n>:PADone?
/// HIFN     BUS<m>:RFFE:SEQuence<n>:PCTRl?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR addressZeroParity/Returns the address-zero parity for the selected command sequence.
/// HIPAR addressOneParity/Returns the address-one parity for the selected command sequence.
/// HIPAR controlParity/Returns the control parity for the selected command sequence.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceParity(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   sequence,
                                                  ViInt32*  addressZeroParity,
                                                  ViInt32*  addressOneParity,
                                                  ViInt32*  controlParity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  rngValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:PADZ?", serialBus, sequence);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceStatus, response, &rngValue));
    *addressZeroParity = (ViInt32)rngValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:PAD?", serialBus, sequence);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceStatus, response, &rngValue));
    *addressOneParity = (ViInt32)rngValue;

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:PCTR?", serialBus, sequence);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceStatus, response, &rngValue));
    *controlParity = (ViInt32)rngValue;

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the name of the specified data byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR byteNumber/Selects the byte.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR name/Returns the name of the specified data byte.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceByteName(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   sequence,
                                                    ViInt32   byteNumber,
                                                    ViInt32   bufferSize,
                                                    ViChar    name[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:BYTE%ld:NAME?", serialBus, sequence, byteNumber);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, name, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the read bit rate of the selected command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR bitRate/Returns the read bit rate of the selected command sequence.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceBitRate(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   sequence,
                                                   ViReal64* bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:RBTR?", serialBus, sequence);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, bitRate));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the write bit rate of the selected command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR writeBitRate/Returns the write bit rate of the selected command sequence.
ViStatus _VI_FUNC rsscope_QueryRFFESequenceWriteBitRate(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   sequence,
                                                        ViReal64* writeBitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:RFFE:SEQ%ld:WBTR?", serialBus, sequence);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, writeBitRate));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the event to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the event to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (searchName)
        viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
                2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_RFFE_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches for a slave address or a slave address range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchSlaveAddress(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   condition,
                                                          ViString  minimum,
                                                          ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_SLAVE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_SLAVE_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_SLAVE_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Searches for a register address or an address range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchAddress(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   condition,
                                                     ViString  minimum,
                                                     ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ADDRESS_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ADDRESS_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Searches for a byte count or a byte count range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchByteCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_BYTE_COUNT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_BYTE_COUNT_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_BYTE_COUNT_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Searches for a data pattern or an data word range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_DATA_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_DATA_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines which data frames are relevant for the search. The setting is
/// HIFN  available for all "Extended Register" command sequences, which can
/// HIFN  transfer more than one data frame. To search for any data byte that
/// HIFN  fulfills the data pattern settings, set the index to XX.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchIndex(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   condition,
                                                   ViInt32   minimum,
                                                   ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_INDEX_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_INDEX_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR ssc/Enables the trigger on an SSC error - when no valid SSC sequence has
/// HIPAR ssc/been found after the idle time.
/// HIPAR length/Enables the trigger on a length error - when an incorrect length of
/// HIPAR length/the command sequence has been found.
/// HIPAR reserved1/Reserved for future use (Bus Park no longer supported).
/// HIPAR parity/Enables the trigger on incorrect parity bits.
/// HIPAR reserved2/Reserved for future use (No Response no longer supported).
/// HIPAR reserved3/Reserved for future use (Unknown Sequence no longer supported).
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchErrorConditions(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViBoolean ssc,
                                                             ViBoolean length,
                                                             ViBoolean reserved1,
                                                             ViBoolean parity,
                                                             ViBoolean reserved2,
                                                             ViBoolean reserved3)
{
    ViStatus error = VI_SUCCESS;
             reserved1;
             reserved2;
             reserved3;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_SCC, ssc),
            3, "SCC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_LENGTH, length),
            4, "length");

    //    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_BUS_PARK, busPark),
    //          5, "Bus Park");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_PARITY, parity),
            6, "parity");

    //    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_NO_RESPONSE, noResponse),
    //            7, "No Response");

    //    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_UNKNOWN, unknownSequence),
    //            8, "Unknown Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the pattern of the interrrupt identification sequence, which
/// HIFN  consists of interrupt slots 15 to 0.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_RFFE_INTERRUPT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:RFFE:INTerrupt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR interrupt/Defines the pattern of the interrrupt identification sequence, which
/// HIPAR interrupt/consists of interrupt slots 15 to 0.
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchInterrupt(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViString  interrupt)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_INTERRUPT_PATTERN, interrupt),
            3, "Interrupt");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the search for RFFE gap time error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR gapTime/Enables the search for RFFE gap time error.
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchErrorGapTime(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean gapTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_GAP_TIME, gapTime),
            3, "Gap Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the search for RFFE vresion error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR version/Enables the search for RFFE vresion error.
ViStatus _VI_FUNC rsscope_ConfigureRFFESearchErrorVersion(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean version)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_ERROR_VERSION, version),
            3, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of command sequences in the current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Number of command sequences.
ViStatus _VI_FUNC rsscope_QueryRFFESearchSequenceCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_RFFE_SEQUENCE_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall search state of the command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR status/Returns the overall state of the selected command sequence.
/// HIPAR start/Time of frame start.
/// HIPAR stop/Time of frame stop.
/// HIPAR type/Returns the type of the selected command sequence.
/// HIPAR slaveAddress/Returns the type of the selected command sequence.
/// HIPAR address/Returns the register address of the selected command sequence.
/// HIPAR byteCount/Returns the byte count of the selected command sequence.
ViStatus _VI_FUNC rsscope_QueryRFFESearchSequenceStatus(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   sequence,
                                                        ViInt32*  status,
                                                        ViReal64* start,
                                                        ViReal64* stop,
                                                        ViInt32*  type,
                                                        ViInt32*  slaveAddress,
                                                        ViInt32*  address,
                                                        ViInt32*  byteCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  rngValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:STAT? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceStatus, response, &rngValue));
    *status = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:STAR? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:STOP? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:TYPE? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolRFFESequenceType, response, &rngValue));
    *type = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:SADD? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, slaveAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:ADDR? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:BCO? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, byteCount));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the search symbolic label that belongs to the
/// HIFN  address of the selected command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryRFFESearchSequenceLabel(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   sequence,
                                                       ViInt32   bufferSize,
                                                       ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:SYMB? '%s'", sequence, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data bytes of the specified command sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR sequence/This control selects the sequence.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the sequence.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryRFFESearchSequenceAllData(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   sequence,
                                                         ViInt32   arraySize,
                                                         ViInt32   data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:DATA? '%s'", sequence, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the search data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequence/This control selects the command sequence.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the state of the specified byte.
ViStatus _VI_FUNC rsscope_QueryRFFESearchSequenceData(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   sequence,
                                                      ViInt32   byteNumber,
                                                      ViInt32*  data,
                                                      ViInt32*  statusCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:BYTE%ld:VAL? '%s'", sequence, byteNumber, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:RFFE:SEQ%ld:BYTE%ld:STAT? '%s'", sequence, byteNumber, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolRFFEStatusCode_Arr, statusCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the source of the RFFE data line. For triggering on a serial bus,
/// HIFN  analog or digital input channels are required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select RFFE line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureMDIOSource(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   line,
                                              ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_PROTOCOL_MDIO_CLOCK_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_MDIO_DATA_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attr[line],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SOURCE_CLOCK, RSSCOPE_VAL_SOURCE_DATA),
            3, "Line");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                4, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, attr[line], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures threshold value for digitization of analog
/// HIFN  signals.
/// HIFN     If the signal value is higher than the threshold, the signal state is
/// HIFN  high (logic 1). Otherwise, the signal state is considered low (logic
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Selects the default threshold settings according to the Ethernet
/// HIPAR preset/standard.
/// HIPAR line/This control select RFFE line.
/// HIPAR thresholdLow/Sets the negative going thresholds.
/// HIPAR thresholdHigh/Sets the positive going thresholds.
ViStatus _VI_FUNC rsscope_ConfigureMDIOThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   preset,
                                                 ViInt32   line,
                                                 ViReal64  thresholdLow,
                                                 ViReal64  thresholdHigh)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   thresholdLowAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_MDIO_CLOCK_THRESHOLD_LOW,
             RSSCOPE_ATTR_PROTOCOL_MDIO_DATA_THRESHOLD_LOW
             };
    ViAttr   thresholdHighAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_MDIO_CLOCK_THRESHOLD_HIGH,
             RSSCOPE_ATTR_PROTOCOL_MDIO_DATA_THRESHOLD_HIGH
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MDIO_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_THRESHOLD_PRESET, preset),
            3, "preset");

    if (preset == RSSCOPE_VAL_MDIO_MAN)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SOURCE_CLOCK, RSSCOPE_VAL_SOURCE_DATA),
                4, "Line");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, thresholdLowAttr[line], thresholdLow),
                5, "Threshold Low");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, thresholdHighAttr[line], thresholdHigh),
                6, "Threshold High");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the data line thresholds are set to the clock line
/// HIFN  threshold values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Enables/disables tresholds coupling.
ViStatus _VI_FUNC rsscope_ConfigureMDIOThresholdsCoupling(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_RFFE_THRESHOLDS_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_THRESHOLDS_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the trigger type for MDIO analysis.
/// HIFN     TRIGger<m>:MDIO:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for MDIO analysis.
ViStatus _VI_FUNC rsscope_ConfigureMDIOTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures frame pattern. Available only in trigger type
/// HIFN  "Data", see rsscope_ConfigureMDIOTriggerType.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR startCode/Selects the start of frame code of the frame pattern.
/// HIPAR operationCode/Selects the Type of Frame code (or OP code, OpCode, operation code).
/// HIPAR physicalAddress/Sets the physical address (in Clause 22) or port address (in Clause
/// HIPAR physicalAddress/45) of the frame pattern (5 bits).
/// HIPAR registerAddress/Sets the register address (in Clause 22) or device address (in Clause
/// HIPAR registerAddress/45) of the frame pattern (5 bits).
/// HIPAR dataPattern/Defines the 16-bit payload data pattern (both in Clause 22 or Clause
/// HIPAR dataPattern/45) or the address pattern (in Clause 45, only) to trigger for.
ViStatus _VI_FUNC rsscope_ConfigureMDIOTriggerFramePattern(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViInt32   startCode,
                                                           ViInt32   operationCode,
                                                           ViString  physicalAddress,
                                                           ViString  registerAddress,
                                                           ViString  dataPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_START_CODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_START_CODE, startCode),
            3, "Start Code");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_OPERATION_CODE, operationCode),
            4, "Operation Code");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_PHYSICAL_PORT_ADDRESS, physicalAddress),
            5, "Physical Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_REGISTER_DEVICE_ADDRESS, registerAddress),
            6, "Register Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_TRIGGER_DATA_ADDRESS_PATTERN, dataPattern),
            7, "Data Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum idle time between two frames.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MDIO_MAX_GAP_TIME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MDIO:MAXGap
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR maxGapTime/Sets the maximum idle time between two frames.
ViStatus _VI_FUNC rsscope_ConfigureMDIOMaxGapTime(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViReal64  maxGapTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MDIO_MAX_GAP_TIME,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_MAX_GAP_TIME, maxGapTime),
            3, "Max Gap Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the word count for the selected serial bus, i.e. the number of
/// HIFN  words in the present acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns word count.
ViStatus _VI_FUNC rsscope_QueryMDIOWordCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MDIO_DECODING_RESULTS_WORD_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MDIO_DECODING_RESULTS_WORD_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to output ViChar[] parameters.
/// HIPAR status/Returns the state of the frame in the present acquisition of the
/// HIPAR status/selected serial bus.
/// HIPAR start/Time of word start.
/// HIPAR stop/Time of word stop.
/// HIPAR startCode/Returns the start time of the frame in the selected word of the
/// HIPAR startCode/selected serial bus.
/// HIPAR operationCode/Returns the Type of Frame code (or OP code, OpCode, operation code).
/// HIPAR registerAddress/Returns the 5-bit register or device address field content
/// HIPAR registerAddress/(REGAD/DEVAD) in the present acquisition of the selected word and the
/// HIPAR registerAddress/selected serial bus.
/// HIPAR dataPattern/Returns the 16-bit payload data field content (in Clause 22 or Clause
/// HIPAR dataPattern/45), or the 16-bit address field content (in Clause 45, only) in the
/// HIPAR dataPattern/present acquisition of the selected word and the selected serial bus.
/// HIPAR dataPattern/The most significant bit (MSB) is transmitted first.
/// HIPAR physicalAddress/Returns the 5-bit address field content (PHYAD/PRTAD) in the present
/// HIPAR physicalAddress/acquisition of the selected word and the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryMDIOWordStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   word,
                                              ViInt32   bufferSize,
                                              ViInt32*  status,
                                              ViReal64* start,
                                              ViReal64* stop,
                                              ViChar    startCode[],
                                              ViChar    operationCode[],
                                              ViInt32*  physicalAddress,
                                              ViInt32*  registerAddress,
                                              ViChar    dataPattern[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:STAT?", serialBus, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMDIOStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:STAR?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:STOP?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:ST?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, startCode, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:TYPE?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, operationCode, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:PHYS?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, physicalAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:REGI?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, registerAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:DATA?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, dataPattern, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a textual translation (called Register Name) of the PHY or
/// HIFN  port address label in the present acquisition of the selected word and
/// HIFN  the selected serial bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryMDIOWordLabel(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   word,
                                             ViInt32   bufferSize,
                                             ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MDIO:WORD%ld:SYMB?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the trigger type for MDIO search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the trigger type for MDIO analysis.
ViStatus _VI_FUNC rsscope_ConfigureMDIOSearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MDIO_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures search frame pattern. Available only in
/// HIFN  trigger type "Data", see rsscope_ConfigureMDIOSearchTriggerType.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startCode/Selects the start of frame code of the frame pattern.
/// HIPAR operationCode/Selects the Type of Frame code (or OP code, OpCode, operation code).
/// HIPAR physicalAddress/Sets the physical address (in Clause 22) or port address (in Clause
/// HIPAR physicalAddress/45) of the frame pattern (5 bits).
/// HIPAR registerAddress/Sets the register address (in Clause 22) or device address (in Clause
/// HIPAR registerAddress/45) of the frame pattern (5 bits).
/// HIPAR dataPattern/Defines the 16-bit payload data pattern (both in Clause 22 or Clause
/// HIPAR dataPattern/45) or the address pattern (in Clause 45, only) to trigger for.
ViStatus _VI_FUNC rsscope_ConfigureMDIOSearchFramePattern(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   startCode,
                                                          ViInt32   operationCode,
                                                          ViString  physicalAddress,
                                                          ViString  registerAddress,
                                                          ViString  dataPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_MDIO_START_CODE, startCode),
            3, "Start Code");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_MDIO_OPERATION_CODE, operationCode),
            4, "Operation Code");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_MDIO_PHYSICAL_PORT_ADDRESS, physicalAddress),
            5, "Physical Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_MDIO_REGISTER_DEVICE_ADDRESS, registerAddress),
            6, "Register Address");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_MDIO_DATA_ADDRESS_PATTERN, dataPattern),
            7, "Data Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the word count within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns word count.
ViStatus _VI_FUNC rsscope_QueryMDIOSearchWordCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MDIO_WORD_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the searched word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Defines the size of buffer passed to output ViChar[] parameters.
/// HIPAR status/Returns the state of the frame in the present acquisition of the
/// HIPAR status/selected serial bus.
/// HIPAR start/Time of word start.
/// HIPAR stop/Time of word stop.
/// HIPAR startCode/Returns the start time of the frame in the selected word of the
/// HIPAR startCode/selected serial bus.
/// HIPAR operationCode/Returns the Type of Frame code (or OP code, OpCode, operation code).
/// HIPAR registerAddress/Returns the 5-bit register or device address field content
/// HIPAR registerAddress/(REGAD/DEVAD) in the present acquisition of the selected word and the
/// HIPAR registerAddress/selected serial bus.
/// HIPAR dataPattern/Returns the 16-bit payload data field content (in Clause 22 or Clause
/// HIPAR dataPattern/45), or the 16-bit address field content (in Clause 45, only) in the
/// HIPAR dataPattern/present acquisition of the selected word and the selected serial bus.
/// HIPAR dataPattern/The most significant bit (MSB) is transmitted first.
/// HIPAR physicalAddress/Returns the 5-bit address field content (PHYAD/PRTAD) in the present
/// HIPAR physicalAddress/acquisition of the selected word and the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryMDIOSearchWordStatus(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   word,
                                                    ViInt32   bufferSize,
                                                    ViInt32*  status,
                                                    ViReal64* start,
                                                    ViReal64* stop,
                                                    ViChar    startCode[],
                                                    ViChar    operationCode[],
                                                    ViInt32*  physicalAddress,
                                                    ViInt32*  registerAddress,
                                                    ViChar    dataPattern[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:STAT? '%s'", word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMDIOStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:STAR? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:STOP? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:ST? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, startCode, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:TYPE? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, operationCode, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:PHYS? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, physicalAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:REGI? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, registerAddress));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:DATA? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, dataPattern, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a textual translation (called Register Name) of the PHY or
/// HIFN  port address label from the selected word within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QueryMDIOSearchWordLabel(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   word,
                                                   ViInt32   bufferSize,
                                                   ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MDIO:WORD%ld:SYMB? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the Ethernet protocol variant and transmission
/// HIFN  speed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR variant/Selects variant.
ViStatus _VI_FUNC rsscope_ConfigureEthernetVariant(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   variant)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_VARIANT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_VARIANT, variant),
            3, "variant");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the source channel for the data signal. For triggering on a
/// HIFN  serial bus, analog channels "C1" - "C4" are required. Otherwise, if no
/// HIFN  serial bus trigger has been selected, permitted source selections
/// HIFN  include the mathematical channels "Math1" -
/// HIFN     "Math4" and the reference channels "Ref1" - "Ref4".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureEthernetSource(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_PROTOCOL_WAV_C1W1, RSSCOPE_VAL_PROTOCOL_WAV_M8),
            3, "Source");
    if ((RSSCOPE_VAL_PROTOCOL_WAV_R4 < source) && (source < RSSCOPE_VAL_PROTOCOL_WAV_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source),
                    3, "Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the polarity of the data signal. This setting is
/// HIFN  only available in 10BASE-T.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Selects the polarity.
ViStatus _VI_FUNC rsscope_ConfigureEthernetPolarity(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_POLARITY, polarity),
            3, "polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the threshold value for the digitization of each
/// HIFN  signal line. If the signal value on the line is higher than the upper
/// HIFN  threshold, the signal state is high. Otherwise, if the signal value is
/// HIFN  below the lower threshold, the signal state is considered low.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets the thresholds to predefined or individually definable voltage
/// HIPAR preset/levels.
/// HIPAR thresholdLow/Sets the lower threshold value for the signal digitization. If the
/// HIPAR thresholdLow/signal value is below this threshold, the signal state is considered
/// HIPAR thresholdLow/low.
/// HIPAR thresholdHigh/Sets the upper threshold value for the signal digitization. If the
/// HIPAR thresholdHigh/signal value is higher than the this threshold, the signal state is
/// HIPAR thresholdHigh/considered high.
ViStatus _VI_FUNC rsscope_ConfigureEthernetThreshold(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   preset,
                                                     ViReal64  thresholdLow,
                                                     ViReal64  thresholdHigh)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_THRESHOLD_PRESET, preset),
            3, "preset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_THRESHOLD_LOW, thresholdLow),
            4, "Threshold Low");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_THRESHOLD_HIGH, thresholdHigh),
            3, "Threshold High");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the bit rate value that defines the transmission
/// HIFN  speed in bits per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitrate/Sets the bit rate value that defines the transmission speed in bits
/// HIPAR bitrate/per second.
ViStatus _VI_FUNC rsscope_ConfigureEthernetBitRate(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   bitrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_BIT_RATE, bitrate),
            3, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger for Ethernet variant 10BASE-T.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_ETHERNET_TRIGGER_TYPE
/// HIFN     RSSCOPE_ATTR_ETHERNET_TRIGGER_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:ETHernet:TYPE
/// HIFN     TRIGger<m>:ETHernet:PATTern
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR pattern/Specifies the bit pattern (or data) that is to be triggered.
/// HIPAR type/Sets the trigger type for Ethernet variant 10BASE-T.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTrigger(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type,
                                                   ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_TYPE, type),
            4, "Type");

    if (type == RSSCOPE_VAL_ETHERNET_TRIGGER_TYPE_PATTERN)
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_PATTERN, pattern),
                3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger errors for Ethernet variant
/// HIFN  10BASE-T.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR crc/Enables the trigger on a mismatch of the cyclic redundancy check (CRC)
/// HIPAR crc/value between the transmitting and receiving device.
/// HIPAR length/Enables the trigger on an incorrect length of the sequence - when
/// HIPAR length/additional or missing bits are detected and the sequence of bits is not
/// HIPAR length/as expected.
/// HIPAR preamble/Enables the trigger on a frame with invalid preamble.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTriggerError(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViBoolean crc,
                                                        ViBoolean length,
                                                        ViBoolean preamble)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_ERROR_CRC,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_ERROR_CRC, crc),
            3, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_ERROR_LENGTH, length),
            4, "Length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_ERROR_PREAMBLE, preamble),
            5, "Preamble");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Destination Address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the destination address operator to trigger on a specific pattern
/// HIPAR condition/or a range.
/// HIPAR min/Specifies a destination address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a destination address range.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTriggerDestinationAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_DESTINATION_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_DESTINATION_ADDRESS_PATTERN_MIN, min),
            4, "Minimum");
    if (condition >= RSSCOPE_VAL_ETHERNET_TRIGGER_COND_INR)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_DESTINATION_ADDRESS_PATTERN_MAX, max),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Source Address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets source address operator to trigger on a specific pattern or a
/// HIPAR condition/range.
/// HIPAR min/Specifies a source address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a source address range.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTriggerSourceAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_SOURCE_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_SOURCE_ADDRESS_PATTERN_MIN, min),
            4, "Minimum");
    if (condition >= RSSCOPE_VAL_ETHERNET_TRIGGER_COND_INR)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_SOURCE_ADDRESS_PATTERN_MAX, max),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger CRC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to trigger on a specific frame check sequence value.
/// HIPAR min/Specifies a CRC pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of the frame check sequence range.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTriggerCRC(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   condition,
                                                      ViString  min,
                                                      ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_CRC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_CRC_PATTERN_MIN, min),
            4, "Minimum");
    if (condition >= RSSCOPE_VAL_ETHERNET_TRIGGER_COND_INR)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_CRC_PATTERN_MAX, max),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Type Operator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to trigger on a specific frame check sequence value.
/// HIPAR min/Specifies a length/type value, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a length/type range.
ViStatus _VI_FUNC rsscope_ConfigureEthernetTriggerTypeOperator(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_TYPE_OPERATOR_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_TYPE_OPERATOR_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_TYPE_OPERATOR_PATTERN_MIN, min),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_ETHERNET_TRIGGER_COND_INR)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_ETHERNET_TRIGGER_FRAME_TYPE_OPERATOR_PATTERN_MAX, max),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the word count for the selected serial bus, i.e. the number of
/// HIFN  words in the current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns word count.
ViStatus _VI_FUNC rsscope_QueryEthernetWordCount(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ETHERNET_DECODING_RESULTS_WORD_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ETHERNET_DECODING_RESULTS_WORD_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the primary bit rate of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_QueryEthernetWordBitRate(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   word,
                                                   ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:BITR?", serialBus, word);
    checkErr(RsCore_QueryViInt64(instrSession, cmd, bitRate));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to output ViChar[] parameters.
/// HIPAR status/Returns the frame state of the selected word in the current
/// HIPAR status/acquisition.
/// HIPAR start/Time of word start.
/// HIPAR stop/Time of word stop.
/// HIPAR destination/Returns the destination address of the specified word.
/// HIPAR source/Returns the source address of the specified word.
/// HIPAR crc/Returns the Cyclic Redundancy Code (CRC, or frame check) checksum of
/// HIPAR crc/the selected word.
/// HIPAR type/The sub-protocol (e.g. HTML, video, etc.) determines what meaning this
/// HIPAR type/field has. Since the content of this data area is not decoded, the
/// HIPAR type/interpretation of the type field is ambivalent. The query either
/// HIPAR type/returns the word type (specific for the sub-protocol), or the length of
/// HIPAR type/the selected word.
ViStatus _VI_FUNC rsscope_QueryEthernetWordStatus(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   word,
                                                  ViInt32   bufferSize,
                                                  ViInt32*  status,
                                                  ViReal64* start,
                                                  ViReal64* stop,
                                                  ViChar    destination[],
                                                  ViChar    source[],
                                                  ViInt32*  type,
                                                  ViInt32*  CRC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  rngValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:STAT?", serialBus, word);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolEthernetWordStatus, response, &rngValue));
    *status = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:STAR?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:STOP?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:DEST?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, destination, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:SRC?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, source, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:TYPE?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, type));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:CRC?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, CRC));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the symbolic label (or translation) of the source and
/// HIFN  destination address of the specified word, if the label list is
/// HIFN  enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR destinationLabel/Returns the symbolic label (or translation) of the destination address
/// HIPAR destinationLabel/of the specified word, if the label list is enabled.
/// HIPAR sourceLabel/Returns the symbolic label (or translation) of the source address of
/// HIPAR sourceLabel/the specified word, if the label list is enabled.
ViStatus _VI_FUNC rsscope_QueryEthernetWordLabel(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   word,
                                                 ViInt32   bufferSize,
                                                 ViChar    destinationLabel[],
                                                 ViChar    sourceLabel[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:DSYM?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, destinationLabel, NULL));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:SSYM?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, sourceLabel, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte.
ViStatus _VI_FUNC rsscope_QueryEthernetWordData(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   word,
                                                ViInt32   byteNumber,
                                                ViInt32*  data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:BYTE%ld:VAL?", serialBus, word, byteNumber);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryEthernetWordAllData(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   word,
                                                   ViInt32   arraySize,
                                                   ViInt32   data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_HEX));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:DATA?", serialBus, word);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the frame type of the selected word in the current
/// HIFN  acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:ETHernet:WORD<n>:FTYPe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR frameType/Returns the frame type of the selected word in the current
/// HIPAR frameType/acquisition.
ViStatus _VI_FUNC rsscope_QueryEthernetWordFrameType(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   word,
                                                     ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:FTYP?", serialBus, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolEthernetWordFrameType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of words in the selected frame. The result
/// HIFN  corresponds to the Number of Words column in the results table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:ETHernet:WORD<n>:NUMWords?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR numberOfWords/Returns the number of words in the selected frame. The result
/// HIPAR numberOfWords/corresponds to the Number of Words column in the results table.
ViStatus _VI_FUNC rsscope_QueryEthernetWordNumberOfWords(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   word,
                                                         ViInt32*  numberOfWords)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ETH:WORD%ld:NUMW?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfWords));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Use the "Search criteria" dialog to define the event types to be
/// HIFN  searched. Available event types are "Frame" and "Error".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Defines, whether a search within a frame shall be activated or not.
/// HIPAR error/Defines, whether a search for an error condition shall be activated or
/// HIPAR error/not.
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchCriteria(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean frameEnabled,
                                                          ViBoolean errorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_FRAME_ENABLED, frameEnabled),
            3, "Frame Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_ERROR_ENABLED, errorEnabled),
            4, "Error Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  To search for a destination address, an address pattern or optionally
/// HIFN  an address range have to be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Defines the operator to search a specific destination address within a
/// HIPAR condition/frame.
/// HIPAR minimum/Defines a destination address, or sets the start value of a
/// HIPAR minimum/destination address range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchDestinationAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_DESTINATION_ADDRESS_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_DESTINATION_ADDRESS_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  To search for a source address, an address pattern or optionally an
/// HIFN  address range have to be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR minimum/Specifies a pattern, or sets the start value of a pattern range.
/// HIPAR maximum/Sets the end value of an range if the corresponding condition is
/// HIPAR maximum/set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchSourceAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_SOURCE_ADDRESS_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_SOURCE_ADDRESS_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  To search for a frame length or frame type, a type/length pattern or
/// HIFN  optionally a range of type/length patterns have to be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Defines the operator to search for a specific frame length or type.
/// HIPAR minimum/Defines a frame length/type, or sets the start value for a range of
/// HIPAR minimum/frame lengths/types.
/// HIPAR maximum/Sets the end value of a range of frame lengths/types if the
/// HIPAR maximum/corresponding condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchTypeLength(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   condition,
                                                            ViString  minimum,
                                                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_TYPE_LENGTH_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_TYPE_LENGTH_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_TYPE_LENGTH_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  To search for a specific pattern, this pattern or optionally a range
/// HIFN  of patterns have to be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Defines the operator to search for a Cyclic Redundancy Code (CRC, or
/// HIPAR condition/frame check) error condition within a frame.
/// HIPAR minimum/Defines a CRC error condition pattern, or sets the start value of such
/// HIPAR minimum/a pattern.
/// HIPAR maximum/Sets the end value of a CRC error condition pattern if the
/// HIPAR maximum/corresponding condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchFrameCheck(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   condition,
                                                            ViString  minimum,
                                                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_FRAME_CHECK_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_FRAME_CHECK_MIN, minimum),
            4, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_FRAME_CHECK_MAX, maximum),
                5, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches for two error conditions: "Preamble Error" or
/// HIFN  "Length Error".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR preamble/Defines, whether a search for any preamble error shall be activated or
/// HIPAR preamble/not.
/// HIPAR length/Defines, whether a search for any word length error (too few or too
/// HIPAR length/many bits per word) shall be activated or not.
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchErrorConditions(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean preamble,
                            ViBoolean length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_ERROR_PREAMBLE, preamble),
            3, "preamble");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_ERROR_LENGTH, length),
            4, "length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects, if it searched on a MAC frame or on an error.
///HIRET Returns the status code of this operation.
/// HIPAR searchName/Specifies search definition.
///HIPAR ethernetType/Selects, if it searched on a MAC frame or on an
///HIPAR ethernetType/error.
ViStatus _VI_FUNC rsscope_ConfigureEthernetSearchType(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   ethernetType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_TYPE, ethernetType),
            3, "Ethernet Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns word count.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_ETHERNET_WORD_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Searches the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to output ViChar[] parameters.
/// HIPAR status/Returns the frame state of the selected word within the search result.
/// HIPAR start/Time of word start.
/// HIPAR stop/Time of word stop.
/// HIPAR destination/Returns the destination address of the specified word.
/// HIPAR source/Returns the source address of the specified word.
/// HIPAR crc/Returns the Cyclic Redundancy Code (CRC, or frame check) checksum of
/// HIPAR crc/the selected word.
/// HIPAR type/The sub-protocol (e.g. HTML, video, etc.) determines what meaning this
/// HIPAR type/field has. Since the content of this data area is not decoded, the
/// HIPAR type/interpretation of the type field is ambivalent. The query either
/// HIPAR type/returns the word type (specific for the sub-protocol), or the length of
/// HIPAR type/the selected word.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordStatus(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   word,
                                                        ViInt32   bufferSize,
                                                        ViInt32*  status,
                                                        ViReal64* start,
                                                        ViReal64* stop,
                                                        ViChar    destination[],
                                                        ViChar    source[],
                                                        ViInt32*  crc,
                                                        ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  rngValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:STAT? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringShort(instrSession, cmd, response));
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsscope_rngProtocolEthernetWordStatus, response, &rngValue));
    *status = (ViInt32)rngValue;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:STAR? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:STOP? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:DEST? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, destination, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:SRC? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, source, NULL));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:TYPE? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, type));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:CRC? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, crc));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the symbolic label (or translation) of the source and
/// HIFN  destination address of the specified word, if the label list is
/// HIFN  enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR destinationLabel/Returns the symbolic label (or translation) of the destination address
/// HIPAR destinationLabel/of the specified word, if the label list is enabled.
/// HIPAR sourceLabel/Returns the symbolic label (or translation) of the source address of
/// HIPAR sourceLabel/the specified word, if the label list is enabled.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordLabel(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   word,
                                                       ViInt32   bufferSize,
                                                       ViChar    destinationLabel[],
                                                       ViChar    sourceLabel[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:DSYM? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, destinationLabel, NULL));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:SSYM? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, sourceLabel, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordData(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   word,
                                                      ViInt32   byteNumber,
                                                      ViInt32*  data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:BYTE%ld:VAL? '%s'", word, byteNumber, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data bytes of the specified word within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordAllData(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   word,
                                                         ViInt32   arraySize,
                                                         ViInt32   data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_HEX));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:DATA? '%s'", word, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the frame type of the specified frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:ETHernet:WORD<m>:FTYPe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR frameType/Returns the frame type of the specified frame.
ViStatus _VI_FUNC rsscope_QueryEthernetSearchWordFrameType(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   word,
                                                           ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:ETH:WORD%ld:FTYP? '%s'", word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolEthernetWordFrameType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the USB source. Permitted selections are the analog channels
/// HIFN  "C1"-"C4".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocolType/This control selects USB protocol type.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureUSBSource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   protocolType,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_PROTOCOL_USB_D_PLUS_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_USB_D_MINUS_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_USB_DIFFERENTIAL_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_USB_DATA_SOURCE,
             RSSCOPE_ATTR_PROTOCOL_USB_STROBE_SOURCE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, protocolType, RSSCOPE_VAL_USB_TYPE_D_PLUS, RSSCOPE_VAL_USB_TYPE_STROBE),
            3, "Protocol Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attr[protocolType],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, attr[protocolType], waveformSource),
            4, "Waveform Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the USB protocol technology and transmission
/// HIFN  speed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Defines the USB protocol technology and transmission speed.
ViStatus _VI_FUNC rsscope_ConfigureUSBPreset(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TECHNOLOGY, preset),
            3, "Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the threshold value for the digitization of each
/// HIFN  signal line. If the signal value on the line is higher than the
/// HIFN  threshold, the signal state is high. Otherwise, if the signal value is
/// HIFN  below the threshold, the signal state is considered low.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocolType/This control selects USB protocol type.
/// HIPAR threshold/Sets the thresholds.
ViStatus _VI_FUNC rsscope_ConfigureUSBThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   protocolType,
                                                ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_PROTOCOL_USB_D_PLUS_THRESHOLD,
             RSSCOPE_ATTR_PROTOCOL_USB_D_MINUS_THRESHOLD,
             RSSCOPE_ATTR_PROTOCOL_USB_DIFFERENTIAL_THRESHOLD,
             RSSCOPE_ATTR_PROTOCOL_USB_DATA_THRESHOLD,
             RSSCOPE_ATTR_PROTOCOL_USB_STROBE_THRESHOLD
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, protocolType, RSSCOPE_VAL_USB_TYPE_D_PLUS, RSSCOPE_VAL_USB_TYPE_STROBE),
            3, "Protocol Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attr[protocolType],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, attr[protocolType], threshold),
            4, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the trigger type for USB analysis. The available
/// HIFN  trigger types depend on the activated USB protocol type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for USB analysis. The available trigger types
/// HIPAR type/depend on the activated USB protocol type.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerType(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the trigger to one out of four different token
/// HIFN  types: OUT, IN, SOF, or SETUP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR token/Sets the trigger to one out of four different token types: Out, In,
/// HIPAR token/Start of Frame, or Setup.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerToken(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   token)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_TOKEN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_TOKEN, token),
            3, "token");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the trigger to one out of four different payload
/// HIFN  data types: Data 0, Data 1, Data 2, or MData.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR dataType/Sets the trigger to one out of four different payload data types:
/// HIPAR dataType/DATA0, DATA1, DATA2, or MDATA.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerDataType(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   dataType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_DATA_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_DATA_TYPE, dataType),
            3, "Data Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the trigger to one out of four different handshake
/// HIFN  types: acknowledgment, non-acknowledgment, stall, or no response.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR handshakeType/Sets the trigger to one out of four different handshake types:
/// HIPAR handshakeType/acknowledgment, non-acknowledgment, stall, or no response.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerHandshakeType(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViInt32   handshakeType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_HANDSHAKE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_HANDSHAKE, handshakeType),
            3, "Handshake Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the trigger to one out of four different Special
/// HIFN  PID types: Preamble, Error, Split, or Ping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR specialPID/Sets the trigger to one out of four different Special PID types:
/// HIPAR specialPID/Preamble, Error, Split, or Ping.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerSpecialPID(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   specialPID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SPECIAL_PID,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SPECIAL_PID, specialPID),
            3, "Special PID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the trigger to one of the following eight error condition types:
/// HIFN  PID error, CRC5 error, CRC16 error, Bitstuffing error, unexpected PID
/// HIFN  error, SE1 error, or Glitching error, as well as Any of these errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorConditionType/Sets the trigger to one of the following eight error condition types:
/// HIPAR errorConditionType/PID error, CRC5 error, CRC16 error, Bitstuffing error, unexpected PID
/// HIPAR errorConditionType/error, SE1 error, or Glitching error, as well as Any of these errors.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerErrorCondition(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   errorConditionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ERROR_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ERROR_CONDITION, errorConditionType),
            3, "Error Condition Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether the packet ID error check shall be
/// HIFN  activated or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR pidErrorCheck/Defines, whether the packet ID error check shall be activated or not.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerPIDErrorCheck(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViBoolean pidErrorCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PID_ERROR_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PID_ERROR_CHECK, pidErrorCheck),
            3, "PID Error Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether an endpoint check that meets specific
/// HIFN  conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the endpoint check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific endpoint or an endpoint range.
/// HIPAR minimum/Specifies the endpoint, or sets the start value of an endpoint
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an endpoint range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerEndPointCheck(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViBoolean enabled,
                                                           ViInt32   condition,
                                                           ViString  minimum,
                                                           ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_END_POINT_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_END_POINT_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_END_POINT_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_END_POINT_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_END_POINT_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether an address check that meets specific
/// HIFN  conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the address check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR minimum/Specifies the address, or sets the start value of an address
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an address range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerAddressCheck(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViBoolean enabled,
                                                          ViInt32   condition,
                                                          ViString  minimum,
                                                          ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ADDRESS_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ADDRESS_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ADDRESS_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ADDRESS_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ADDRESS_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether a payload check that meets specific
/// HIFN  conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the payload data check shall be activated or not.
/// HIPAR condition/Sets the operator (any or equal) for the payload data index position.
/// HIPAR position/Specifies the position within a payload data packet, in which a
/// HIPAR position/special data pattern is to be searched.
/// HIPAR pattern/Specifies the payload data pattern that is to be searched.
/// HIPAR dataCondition/Sets the operator (equal or unequal) to set a specific payload data
/// HIPAR dataCondition/pattern.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerPayloadCheck(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViBoolean enabled,
                                                          ViInt32   condition,
                                                          ViInt32   position,
                                                          ViInt32   dataCondition,
                                                          ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK_POSITION, position),
            5, "position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK_DATA_CONDITION, dataCondition),
            6, "Data Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PAYLOAD_CHECK_PATTERN, pattern),
            7, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether a frame number check that meets
/// HIFN  specific conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the frame number check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific frame number or a frame number
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the frame number, or sets the start value of a frame
/// HIPAR minimum/number range.
/// HIPAR maximum/Sets the end value of a frame number range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerFrameCheck(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViBoolean enabled,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_FRAME_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_FRAME_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_FRAME_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_FRAME_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_FRAME_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether a Start / Complete SPLIT transaction
/// HIFN  check is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the frame number check shall be activated or not.
/// HIPAR startComplete/Sets the trigger to a specific start-split or complete-split
/// HIPAR startComplete/transaction endpoint.
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerSCCheck(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViBoolean enabled,
                                                     ViInt32   startComplete)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SC_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SC_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SC_CHECK_START_COMPLETE, startComplete),
            4, "Start Complete");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether a port check that meets specific
/// HIFN  conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the port check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific port number or a port number
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the port number, or sets the start value of a port
/// HIPAR minimum/number range.
/// HIPAR maximum/Sets the end value of a port number range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerPortCheck(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViBoolean enabled,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PORT_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PORT_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PORT_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PORT_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_PORT_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function Defines, whether an SEU check that meets specific
/// HIFN  conditions is executed or not. S and E represent the Start and End of a
/// HIFN  start-split transaction, U represents the reserved/Unused bit of a
/// HIFN  complete-split transaction. Permissible binary SEU values are 00, 01,
/// HIFN  10, and 11.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the SEU check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific SEU or an SEU range.
/// HIPAR minimum/Specifies the SEU, or sets the start value of an SEU range.
/// HIPAR maximum/Sets the end value of an SEU range if the corresponding condition
/// HIPAR maximum/is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerSEUCheck(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViBoolean enabled,
                                                      ViInt32   condition,
                                                      ViString  minimum,
                                                      ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SEU_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SEU_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SEU_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SEU_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_SEU_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether an Endpoint Type (ET) check that meets
/// HIFN  specific conditions is executed or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR enabled/Defines, whether the Endpoint Type (ET) check shall be activated or
/// HIPAR enabled/not.
/// HIPAR condition/Sets the operator to set a specific endpoint type (ET) or an ET range.
/// HIPAR minimum/Specifies the endpoint type, or sets the start value of an
/// HIPAR minimum/endpoint type range.
/// HIPAR maximum/Sets the end value of an endpoint type range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBTriggerETCheck(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViBoolean enabled,
                                                     ViInt32   condition,
                                                     ViString  minimum,
                                                     ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ET_CHECK,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ET_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ET_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ET_CHECK_MIN, minimum),
            5, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_TRIGGER_ET_CHECK_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the packet count for the selected serial bus, i.e. the number
/// HIFN  of packets in the present acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of packets.
ViStatus _VI_FUNC rsscope_QueryUSBPacketCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_DECODING_RESULTS_PACKET_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_DECODING_RESULTS_PACKET_COUNT, count),
            3, "count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR status/Returns the status of the packet with the selected packet number (for
/// HIPAR status/the selected serial bus).
/// HIPAR start/Time of packet start.
/// HIPAR stop/Time of packet stop.
/// HIPAR pid/Returns the packet PID for the selected serial bus and packet number.
ViStatus _VI_FUNC rsscope_QueryUSBPacketStatus(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   packet,
                                               ViInt32*  status,
                                               ViReal64* start,
                                               ViReal64* stop,
                                               ViInt32*  pid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:STAT?", serialBus, packet);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:STAR?", serialBus, packet);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:STOP?", serialBus, packet);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:PID?", serialBus, packet);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPID_Arr, pid));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the packet values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR address/Returns the packet address for the selected serial bus and packet
/// HIPAR address/number.
/// HIPAR endpoint/Returns the endpoint for the selected serial bus and packet number.
/// HIPAR crc/Returns the packet CRC (Cyclic Redundancy Code) for the selected
/// HIPAR crc/serial bus and packet number.
/// HIPAR et/Returns the endpoint type (ET) for the selected serial bus and packet
/// HIPAR et/number.
/// HIPAR frame/Returns the frame number for the selected serial bus and packet
/// HIPAR frame/number.
/// HIPAR port/Returns the port number for the selected serial bus and packet number.
/// HIPAR sc/Returns the Start- / Complete-split transaction (SSPLIT / CSPLIT) flag
/// HIPAR sc/bits for the selected serial bus and packet number.
/// HIPAR seu/Returns the SEU values for the selected serial bus and packet number.
ViStatus _VI_FUNC rsscope_QueryUSBPacketValues(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   packet,
                                               ViInt32*  address,
                                               ViInt32*  endpoint,
                                               ViInt32*  crc,
                                               ViInt32*  et,
                                               ViInt32*  frame,
                                               ViInt32*  port,
                                               ViInt32*  sc,
                                               ViInt32*  seu)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:ADDR?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:ENDP?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, endpoint));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:CRC?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, crc));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:ET?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, et));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:FRAM?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, frame));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:PORT?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, port));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:SC?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sc));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:SEU?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, seu));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the payload data from the packet with the selected packet
/// HIFN  number (for the selected serial bus).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryUSBPacketAllData(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   packet,
                                                ViInt32   arraySize,
                                                ViInt32   data[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USB:PACK%ld:DATA?", serialBus, packet);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the event types to be searched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startOfPacket/Defines, whether a start of packet (SOP) search shall be activated or
/// HIPAR startOfPacket/not.
/// HIPAR token/Defines, whether a search for any token packet shall be activated or
/// HIPAR token/not.
/// HIPAR data/Defines, whether the search for any data packet shall be activated or
/// HIPAR data/not.
/// HIPAR handshake/Defines, whether the search for any handshake packet shall be
/// HIPAR handshake/activated or not.
/// HIPAR specialPID/Defines, whether the search for any special PID packet shall be
/// HIPAR specialPID/activated or not.
/// HIPAR errorCondition/Defines, whether the search for any error condition shall be activated
/// HIPAR errorCondition/or not.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchCriteria(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean startOfPacket,
                                                     ViBoolean token,
                                                     ViBoolean data,
                                                     ViBoolean handshake,
                                                     ViBoolean specialPID,
                                                     ViBoolean errorCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_START_OF_PACKET, startOfPacket),
            3, "Start Of Packet");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_TOKEN, token),
            4, "Token");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_DATA, data),
            5, "Data");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_HANDSHAKE, handshake),
            6, "Handshake");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_SPECIAL_PID, specialPID),
            7, "Special PID");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_CRITERIA_ERROR, errorCondition),
            8, "Error Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, which token packet type is searched for: OUT,
/// HIFN  IN, SOF, or SETUP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR token/Sets the token type.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchToken(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   token)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_TOKEN, token),
            3, "Token");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines, which data packet type is searched for: "DATA0", "DATA1",
/// HIFN  "DATA2", or "MDATA", as well as "Any" data packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dataType/Sets the trigger data type.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchDataType(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   dataType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_DATA_TYPE, dataType),
            3, "Data Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, which handshake type is searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR handshakeType/Sets the trigger handshake type.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchHandshakeType(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   handshakeType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_HANDSHAKE, handshakeType),
            3, "Handshake Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, which special PID packet type is searched for:
/// HIFN  "PREamble", "ERR", "SPLit", "PING", or "Any" special PID packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR specialPID/Sets the trigger special PID.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchSpecialPID(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   specialPID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SPECIAL_PID, specialPID),
            3, "Special PID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines, whether the packet ID error check shall be
/// HIFN  activated or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR pidErrorCheck/Defines, whether a search for any packet ID error shall be activated
/// HIPAR pidErrorCheck/or not.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchPIDErrorCheck(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean pidErrorCheck)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PID_ERROR_CHECK, pidErrorCheck),
            3, "PID Error Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures error conditions to be searched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR pid/Defines, whether a search for any PID error shall be activated or not.
/// HIPAR crc5/Defines, whether a search for any CRC5 error shall be activated or
/// HIPAR crc5/not.
/// HIPAR crc16/Defines, whether a search for any CRC16 error shall be activated or
/// HIPAR crc16/not.
/// HIPAR bitstuff/Defines, whether a search for any bitstuffing error shall be activated
/// HIPAR bitstuff/or not.
/// HIPAR glitch/Defines, whether a search for any glitch error shall be activated or
/// HIPAR glitch/not.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchErrorCondition(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViBoolean pid,
                                                           ViBoolean crc5,
                                                           ViBoolean crc16,
                                                           ViBoolean bitstuff,
                                                           ViBoolean glitch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ERROR_PID, pid),
            3, "PID");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ERROR_CRC5, crc5),
            4, "CRC 5");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ERROR_CRC16, crc16),
            5, "CRC 16");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ERROR_BITSTUFF, bitstuff),
            6, "Bitstuff");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ERROR_GLITCH, glitch),
            7, "Glitch");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for an endpoint that meets specific
/// HIFN  conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the endpoint check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific endpoint or an endpoint range.
/// HIPAR minimum/Specifies the endpoint, or sets the start value of an endpoint
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an endpoint range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchEndPointCheck(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean enabled,
                                                          ViInt32   condition,
                                                          ViString  minimum,
                                                          ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_END_POINT_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_END_POINT_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_END_POINT_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_END_POINT_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for an address that meets specific
/// HIFN  conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the address check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR minimum/Specifies the address, or sets the start value of an address
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an address range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchAddressCheck(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean enabled,
                                                         ViInt32   condition,
                                                         ViString  minimum,
                                                         ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ADDRESS_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ADDRESS_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ADDRESS_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ADDRESS_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for a payload data pattern that meets
/// HIFN  specific conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the payload data check shall be activated or not.
/// HIPAR condition/Sets the operator (any or equal) for the payload data index position.
/// HIPAR position/Specifies the position within a payload data packet, in which a
/// HIPAR position/special data pattern is to be searched.
/// HIPAR pattern/Specifies the payload data pattern that is to be searched.
/// HIPAR dataCondition/Sets the operator (equal or unequal) to set a specific payload data
/// HIPAR dataCondition/pattern.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchPayloadCheck(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean enabled,
                                                         ViInt32   condition,
                                                         ViInt32   position,
                                                         ViInt32   dataCondition,
                                                         ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PAYLOAD_CHECK, enabled),
            3, "enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PAYLOAD_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PAYLOAD_CHECK_POSITION, position),
            5, "position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PAYLOAD_CHECK_DATA_CONDITION, dataCondition),
            6, "Data Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PAYLOAD_CHECK_PATTERN, pattern),
            7, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for a frame number that meets
/// HIFN  specific conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the frame number check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific frame number or a frame number
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the frame number, or sets the start value of a frame
/// HIPAR minimum/number range.
/// HIPAR maximum/Sets the end value of a frame number range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchFrameCheck(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViBoolean enabled,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_FRAME_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_FRAME_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_FRAME_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_FRAME_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches for the selected Start (0) or Complete (1)
/// HIFN  SPLIT transaction endpoint, or X (don't care).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the Start / Complete (SC) check shall be activated or
/// HIPAR enabled/not.
/// HIPAR startComplete/Sets the trigger to a specific start-split or complete-split
/// HIPAR startComplete/transaction endpoint.
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchSCCheck(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViBoolean enabled,
                                                    ViInt32   startComplete)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SC_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SC_CHECK_START_COMPLETE, startComplete),
            4, "Start Complete");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for a port that meets specific
/// HIFN  conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific port number or a port number
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the port number, or sets the start value of a port
/// HIPAR minimum/number range.
/// HIPAR maximum/Sets the end value of a port number range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchPortCheck(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViBoolean enabled,
                                                      ViInt32   condition,
                                                      ViString  minimum,
                                                      ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PORT_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PORT_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PORT_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PORT_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for an SEU that meets specific
/// HIFN  conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the SEU check shall be activated or not.
/// HIPAR condition/Sets the operator to set a specific SEU or an SEU range.
/// HIPAR minimum/Specifies the SEU, or sets the start value of an SEU range.
/// HIPAR maximum/Sets the end value of an SEU range if the corresponding condition
/// HIPAR maximum/is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchSEUCheck(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViBoolean enabled,
                                                     ViInt32   condition,
                                                     ViString  minimum,
                                                     ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SEU_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SEU_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SEU_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_SEU_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for an Endpoint Type (ET) that meets
/// HIFN  specific conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines, whether the Endpoint Type (ET) check shall be activated or
/// HIPAR enabled/not.
/// HIPAR condition/Sets the operator to set a specific endpoint type (ET) or an ET range.
/// HIPAR minimum/Specifies the endpoint type, or sets the start value of an
/// HIPAR minimum/endpoint type range.
/// HIPAR maximum/Sets the end value of an endpoint type range if the corresponding
/// HIPAR maximum/condition is set to "In range" and "Out of range".
ViStatus _VI_FUNC rsscope_ConfigureUSBSearchETCheck(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViBoolean enabled,
                                                    ViInt32   condition,
                                                    ViString  minimum,
                                                    ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ET_CHECK, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ET_CHECK_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ET_CHECK_MIN, minimum),
            5, "minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_ET_CHECK_MAX, maximum),
                6, "maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the search result's packet count, i.e. the number of packets
/// HIFN  found in the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Number of packets.
ViStatus _VI_FUNC rsscope_QueryUSBSearchPacketCount(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PACKET_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR status/Returns the status of the packet with the selected packet number (for
/// HIPAR status/the selected serial bus).
/// HIPAR start/Time of packet start.
/// HIPAR stop/Time of packet stop.
/// HIPAR pid/Returns the packet PID for the selected serial bus and packet number.
ViStatus _VI_FUNC rsscope_QueryUSBSearchPacketStatus(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   packet,
                                                     ViInt32*  status,
                                                     ViReal64* start,
                                                     ViReal64* stop,
                                                     ViInt32*  pid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:STAT? '%s'", packet, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:STAR? '%s'", packet, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:STOP? '%s'", packet, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:PID? '%s'", packet, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPID_Arr, pid));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the packet values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR address/Returns the packet address for the selected serial bus and packet
/// HIPAR address/number.
/// HIPAR endpoint/Returns the endpoint for the selected serial bus and packet number.
/// HIPAR crc/Returns the packet CRC (Cyclic Redundancy Code) for the selected
/// HIPAR crc/serial bus and packet number.
/// HIPAR et/Returns the endpoint type (ET) for the selected serial bus and packet
/// HIPAR et/number.
/// HIPAR frame/Returns the frame number for the selected serial bus and packet
/// HIPAR frame/number.
/// HIPAR port/Returns the port number for the selected serial bus and packet number.
/// HIPAR sc/Returns the Start- / Complete-split transaction (SSPLIT / CSPLIT) flag
/// HIPAR sc/bits for the selected serial bus and packet number.
/// HIPAR seu/Returns the SEU values for the selected serial bus and packet number.
ViStatus _VI_FUNC rsscope_QueryUSBSearchPacketValues(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   packet,
                                                     ViInt32*  address,
                                                     ViInt32*  endpoint,
                                                     ViInt32*  crc,
                                                     ViInt32*  et,
                                                     ViInt32*  frame,
                                                     ViInt32*  port,
                                                     ViInt32*  sc,
                                                     ViInt32*  seu)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:ADDR? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:ENDP? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, endpoint));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:CRC? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, crc));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:ET? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, et));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:FRAM? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, frame));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:PORT? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, port));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:SC? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sc));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:SEU? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, seu));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the payload data from the packet with the selected packet
/// HIFN  number within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryUSBSearchPacketAllData(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   packet,
                                                      ViInt32   arraySize,
                                                      ViInt32   data[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USB:PACK%ld:DATA? '%s'", packet, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the USB 3.1.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_POLARITY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:SOURce
/// HIFN     BUS<m>:USBThree:POLarity
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Selects the source for the data signal in the USB 3.1 protocol.
/// HIPAR source/Permitted selections are the analog channels C1 C4.
/// HIPAR polarity/Defines the polarity of the data signal for the active (U0) state of
/// HIPAR polarity/the port.
ViStatus _VI_FUNC rsscope_ConfigureUSB31(ViSession instrSession,
                                         ViInt32   serialBus,
                                         ViInt32   source,
                                         ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SOURCE, source),
            3, "Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_POLARITY, polarity),
            4, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the USB 3.1 protocol type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocolType/Selects the USB 3.1 protocol type.
ViStatus _VI_FUNC rsscope_ConfigureUSB31ProtocolType(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   protocolType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE, protocolType),
            3, "Protocol Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the threshold levels for digitization of the data signal in
/// HIFN  the USB 3.1 protocol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_THRESHOLD_LOW
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_THRESHOLD_HIGH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:THRLow
/// HIFN     BUS<m>:USBThree:THRHigh
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR low/Defines the lower threshold level for digitization of the data signal
/// HIPAR low/in the USB 3.1 protocol.
/// HIPAR high/Defines the upper threshold level for digitization of the data signal
/// HIPAR high/in the USB 3.1 protocol.
ViStatus _VI_FUNC rsscope_ConfigureUSB31Thresholds(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViReal64  low,
                                                   ViReal64  high)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_THRESHOLD_LOW,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_THRESHOLD_LOW, low),
            3, "Low");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_THRESHOLD_HIGH, high),
            4, "High");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects if the data pattern is scrambled or unscambled.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SCRAMBLE_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:SCRMode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR scrambleMode/Selects if the data pattern is scrambled or unscambled.
ViStatus _VI_FUNC rsscope_ConfigureUSB31ScrambleMode(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   scrambleMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SCRAMBLE_MODE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_SCRAMBLE_MODE, scrambleMode),
            3, "Scramble Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be triggered on. You can trigger on all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:ERRor:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerErrorEnabled(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViString  errorName,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:ERR:ENAB '%s',%s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRAMe:FIELd<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR packetType/Selects the frame (packet) type.
/// HIPAR enable/Enable field name.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   fieldNumber,
                            ViInt32   packetType,
                            ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, packetType, RSSCOPE_VAL_USB31_GEN1_FRAME_TYPE_TSEQ, RSSCOPE_VAL_USB31_GEN1_FRAME_TYPE_ERR),
            4, "Packet Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSB31Gen1PacketType, packetType, &p2string),
                4, "Packet Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM:FIEL%ld:ENAB %s,%s", trigger, fieldNumber, p2string,
             enable ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be triggered on. You can trigger on all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:EREN '%s',%s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be triggered on. You can trigger on all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:ERRor<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorNumber/Selects the error number.
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   errorNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:ERR%ld:ENAB %s", trigger, errorNumber, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FREN '%s',%s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:ENAB %s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected field of
/// HIFN  the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for the selected field of
/// HIPAR enabled/the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FIEN '%s','%s',%s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the checking condition for the selected field of
/// HIFN  the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:FLD<o>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for the selected field of
/// HIPAR enabled/the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:ENAB %s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNameBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:BIT '%s','%s',%s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:FLD<o>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:BIT %s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:DOPerator
/// HIFN     TRIGger<m>:USBThree:DMIN
/// HIFN     TRIGger<m>:USBThree:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNameData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:DOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:DMIN '%s','%s','%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:DMAX '%s','%s','%s'", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:FLD<o>:DOPerator
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:FLD<o>:DMIN
/// HIFN     TRIGger<m>:USBThree:FRAMe<n>:FLD<o>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:DOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:DMIN '%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:DMAX '%s'", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index.
/// HIPAR minimum/Specifies the index, or sets the start value of a index range.
/// HIPAR maximum/Sets the end value of a index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNameIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:IOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:IMIN '%s','%s',%ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:IMAX '%s','%s',%ld", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index.
/// HIPAR minimum/Specifies the index, or sets the start value of a index range.
/// HIPAR maximum/Sets the end value of a index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31TriggerFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_EQ, RSSCOPE_VAL_COND_IN),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:IOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:IMIN %ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBT:FRAM%ld:FLD%ld:IMAX %ld", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be searched for. You can search for all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:ERRor:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable search error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchErrorEnabled(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViString  errorName,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:ERR:ENAB '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRAMe:FIELd<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR packetType/Selects the frame (packet) type.
/// HIPAR enable/Enable field name.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   fieldNumber,
                            ViInt32   packetType,
                            ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fieldNumber, 1, 20),
            3, "Field Number");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSB31Gen1PacketType, packetType, &p2string),
                4, "Packet Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM:FIEL%ld:ENAB '%s',%s,%s", fieldNumber, searchName,
             p2string, enable ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the search for errors. You can search for all error types in
/// HIFN  parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable search error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:EREN '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the search for errors. You can search for all error types in
/// HIFN  parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:ERRor<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorNumber/Selects the error number
/// HIPAR enabled/Enable or disable search error.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   errorNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:ERR%ld:ENABle '%s',%s", errorNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame type to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FREN '%s','%s',%s", searchName, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame type to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:ENAB '%s',%s", frame, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FIEN '%s','%s','%s',%s", searchName, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:FLD<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:ENAB '%s',%s", frame, field, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNameBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:BIT '%s','%s','%s',%s", searchName, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:FLD<n>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:BIT '%s',%s", frame, field, searchName,
             p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:DOPerator
/// HIFN     SEARch:TRIGger:USBThree:DMIN
/// HIFN     SEARch:TRIGger:USBThree:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNameData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:DOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:DMIN '%s','%s','%s','%s'", searchName, frame, field,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:DMAX '%s','%s','%s','%s'", searchName, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:FLD<n>:DOPerator
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:FLD<n>:DMIN
/// HIFN     SEARch:TRIGger:USBThree:FRAMe<m>:FLD<n>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNumericData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:DOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:DMIN '%s','%s'", frame, field, searchName,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:DMAX '%s','%s'", frame, field, searchName,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for index search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index.
/// HIPAR minimum/Specifies the index, or sets the start value of a index range.
/// HIPAR maximum/Sets the end value of a index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNameIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_EQ, RSSCOPE_VAL_COND_IN),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:IOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:IMIN '%s','%s','%s',%ld", searchName, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:IMAX '%s','%s','%s',%ld", searchName, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index.
/// HIPAR minimum/Specifies the index, or sets the start value of a index range.
/// HIPAR maximum/Sets the end value of a index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSB31SearchFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:IOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:IMIN '%s',%ld", frame, field, searchName,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBT:FRAM%ld:FLD%ld:IMAX '%s',%ld", frame, field, searchName,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_USB31_GEN1_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameCount(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB31_GEN1_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the specified frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:STATe?
/// HIFN     SEARch:RESult:USBThree:FRAMe<n>:STARt?
/// HIFN     SEARch:RESult:USBThree:FRAMe<n>:STOP?
/// HIFN     SEARch:RESult:USBThree:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR frameStatus/Returns the state of the specified frame.
/// HIPAR startTime/Returns the start time of the selected frame.
/// HIPAR stopTime/Returns the stop time of the selected frame.
/// HIPAR type/Returns the type of frame.
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameStatus(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   frame,
                                                      ViInt32*  frameStatus,
                                                      ViReal64* startTime,
                                                      ViReal64* stopTime,
                                                      ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31FrameStatusArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31PacketTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns specific frame information details of the selected frame. This
/// HIFN  information is also shown in the Info column of the decode results
/// HIFN  table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:INFO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR info/Returns specific frame information details of the selected frame. This
/// HIPAR info/information is also shown in the Info column of the decode results
/// HIPAR info/table.
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameInfo(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:INFO? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the type of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR type/Returns the type of the field with the selected field number within
/// HIPAR type/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameFieldType(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   field,
                                                         ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:TYPE? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the status of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR status/Returns the status of the field with the selected field number within
/// HIPAR status/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameFieldStatus(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   field,
                                                           ViInt32   frame,
                                                           ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:STAT? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31FrameStatusArr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and stop times of the field with the selected field
/// HIFN  number within the frame with the selected frame number within the
/// HIFN  search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:STARt?
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR startTime/Returns the start time of the field with the selected field number
/// HIPAR startTime/within the frame with the selected frame number within the search
/// HIPAR startTime/result.
/// HIPAR stopTime/Returns the stop time of the field with the selected field number
/// HIPAR stopTime/within the frame with the selected frame number within the search
/// HIPAR stopTime/result.
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameFieldStartStop(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   field,
                            ViInt32   frame,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd1[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   cmd2[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd1, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:STAR? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd1, startTime));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd2, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:STOP? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd2, stopTime));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:VAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameFieldValue(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32   field,
                                                          ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:VAL? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBThree:FRAMe<m>:FLD<n>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR value/Returns the formatted value of the field with the selected field
/// HIPAR value/number within the frame with the selected frame number (for the
/// HIPAR value/selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31SearchFrameFieldFormatedValue(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bufferSize,
                            ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBT:FRAM%ld:FLD%ld:FVAL? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryUSB31FrameCount(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the specified frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:STATe?
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:STARt?
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:STOP?
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR frameStatus/Returns the state of the specified frame.
/// HIPAR startTime/Returns the start time of the selected frame.
/// HIPAR stopTime/Returns the stop time of the selected frame.
/// HIPAR type/Returns the type of frame.
ViStatus _VI_FUNC rsscope_QueryUSB31FrameStatus(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   frame,
                                                ViInt32*  frameStatus,
                                                ViReal64* startTime,
                                                ViReal64* stopTime,
                                                ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31FrameStatusArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31PacketTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns specific frame information details of the selected frame. This
/// HIFN  information is also shown in the Info column of the decode results
/// HIFN  table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:INFO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR info/Returns specific frame information details of the selected frame. This
/// HIPAR info/information is also shown in the Info column of the decode results
/// HIPAR info/table.
ViStatus _VI_FUNC rsscope_QueryUSB31FrameInfo(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:INFO?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the type of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR type/Returns the type of the field with the selected field number within
/// HIPAR type/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31FrameFieldType(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   field,
                                                   ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:TYPE?", serialBus, frame, field);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the status of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR status/Returns the status of the field with the selected field number within
/// HIPAR status/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31FrameFieldStatus(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   field,
                                                     ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:STAT?", serialBus, frame, field);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSB31FrameStatusArr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and stop times of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:STARt?
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR start/Returns the start time of the field with the selected field number
/// HIPAR start/within the frame with the selected frame number (for the selected
/// HIPAR start/serial bus).
/// HIPAR stop/Returns the stop time of the field with the selected field number
/// HIPAR stop/within the frame with the selected frame number (for the selected
/// HIPAR stop/serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31FrameFieldStartStop(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32   field,
                                                        ViReal64* start,
                                                        ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:STAR?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:STOP?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:VAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31FrameFieldValue(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32   field,
                                                    ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:VAL?", serialBus, frame, field);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBThree:RESult:FRAMe<n>:FLD<o>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR field/Selects the field.
/// HIPAR frame/Selects the frame.
/// HIPAR value/Returns the formatted value of the field with the selected field
/// HIPAR value/number within the frame with the selected frame number (for the
/// HIPAR value/selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSB31FrameFieldFormatedValue(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt32   field,
                                                            ViInt32   bufferSize,
                                                            ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBT:RES:FRAM%ld:FLD%ld:FVAL?", serialBus, frame, field);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR bitrate/Sets the bit rate.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPHighSpeedBitrate(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViReal64  bitrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_HIGH_SPEED_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_HIGH_SPEED_BITRATE, bitrate),
            3, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the continuous time linear equalization (CTLE) equalization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR ctle/Enables the continuous time linear equalization (CTLE) equalization.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPCTLEEnabled(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViBoolean ctle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_ENABLED, ctle),
            3, "CTLE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the USB Super Speed channel length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR channelLength/Selects the USB Super Speed channel length.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPCTLEChannelLength(ViSession instrSession,
                                                             ViInt32   serialBus,
                                                             ViInt32   channelLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_CHANNEL_LENGTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_CHANNEL_LENGTH, channelLength),
            3, "Channel Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the frequencies for the transfer function of the CTLE algorithm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR zeroFrequency/Sets the zero frequency for the transfer function of the CTLE
/// HIPAR zeroFrequency/algorithm.
/// HIPAR poleFrequencyFP1/Sets the pole 1 frequency for the transfer function of the CTLE
/// HIPAR poleFrequencyFP1/algorithm.
/// HIPAR poleFrequencyFP2/Sets the pole 2 frequency for the transfer function of the CTLE
/// HIPAR poleFrequencyFP2/algorithm.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPCTLEFrequencies(ViSession instrSession,
                                                           ViInt32   serialBus,
                                                           ViReal64  zeroFrequency,
                                                           ViReal64  poleFrequencyFP1,
                                                           ViReal64  poleFrequencyFP2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_ZERO_FREQUENCY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_ZERO_FREQUENCY, zeroFrequency),
            3, "Zero Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_POLE_FREQUENCY_FP1, poleFrequencyFP1),
            4, "Pole Frequency FP1");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_POLE_FREQUENCY_FP2, poleFrequencyFP2),
            5, "Pole Frequency FP2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the adjustable DC gain. This value can be chosen according to the
/// HIFN  channel loss characteristics. For low-loss channels, you can set a
/// HIFN  value close to 0 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR dcGain/Sets the adjustable DC gain. This value can be chosen according to the
/// HIPAR dcGain/channel loss characteristics. For low-loss channels, you can set a
/// HIPAR dcGain/value close to 0 dB.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPCTLEDCGain(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViReal64  dcGain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_DC_GAIN,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_CTLE_DC_GAIN, dcGain),
            3, "DC Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the use of the decision feedback equalization for USB 3.1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR dfe/Enables the use of the decision feedback equalization for USB 3.1.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPDFEEnabled(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViBoolean dfe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_ENABLED, dfe),
            3, "DFE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the use of estimate coefficients for the decision feedback
/// HIFN  equalization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR estimateCoefficients/Enables the use of estimate coefficients for the decision feedback
/// HIPAR estimateCoefficients/equalization.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPDFEEstimateCoefficientsEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean estimateCoefficients)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_ESTIMATE_COEFFICIENTS_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_ESTIMATE_COEFFICIENTS_ENABLED, estimateCoefficients),
            3, "Estimate Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the value of the coefficient for the DFE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR coefficient/Sets the value of the coefficient for the DFE.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPDFECoefficient(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViReal64  coefficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_COEFFICIENT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_COEFFICIENT, coefficient),
            3, "Coefficient");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a number of taps for the decision feedback filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR numberOfTaps/Sets a number of taps for the decision feedback filter.
ViStatus _VI_FUNC rsscope_ConfigureUSB31DSPDFENumberOfTaps(ViSession instrSession,
                                                           ViInt32   serialBus,
                                                           ViInt32   numberOfTaps)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_NUMBER_OF_TAPS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB31_GEN1_TYPE_DSP_DFE_NUMBER_OF_TAPS, numberOfTaps),
            3, "Number Of Taps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source for the data signal in the USBPD protocol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_SOURCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:SOURce
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataSource/Selects the source for the data signal in the USBPD protocol.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDDataSource(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, dataSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Data Source");
    if ((RSSCOPE_VAL_PROTOCOL_WAV_R4 < dataSource) && (dataSource < RSSCOPE_VAL_PROTOCOL_WAV_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataSource), 3, "Data Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_SOURCE, dataSource),
            3, "Data Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the thresholds values.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_THRESHOLD
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_HYSTERESIS
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRD15
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRD_USB
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRA
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:THReshold
/// HIFN     BUS<m>:USBPd:HYSTeresis
/// HIFN     BUS<m>:USBPd:THRBottom
/// HIFN     BUS<m>:USBPd:THRMid
/// HIFN     BUS<m>:USBPd:THRTop
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataThreshold/Sets the threshold value of the data.
/// HIPAR hysteresis/Sets a value for the hysteresis of the data.
/// HIPAR vrd15/Sets the bottom threshold (for the low current ) for the current
/// HIPAR vrd15/advertisment mode.
/// HIPAR vrdUSB/Sets the middle threshold (at default USB Type-C currnet) for the
/// HIPAR vrdUSB/current advertisment mode.
/// HIPAR vra/Sets the top threshold (at USB Type-C currnet of 1.5 A ) for the
/// HIPAR vra/current advertisment mode
ViStatus _VI_FUNC rsscope_ConfigureUSBPDThresholds(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViReal64  dataThreshold,
                                                   ViReal64  hysteresis,
                                                   ViReal64  vrd15,
                                                   ViReal64  vrdUSB,
                                                   ViReal64  vra)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_THRESHOLD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_THRESHOLD, dataThreshold),
            3, "Data Threshold");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_DATA_HYSTERESIS, hysteresis),
            4, "Hysteresis");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRD15, vrd15),
            5, "VRD 15");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRD_USB, vrdUSB),
            6, "VRD USB");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_THRESHOLD_VRA, vra),
            7, "VRA");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the data words are broken down into subframes. If not
/// HIFN  enabled the data words are displayed as 32-bit data words.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_SHOW_DECODE_DETAIL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:DETail
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR showDecodeDetail/If enabled, the data words are broken down into subframes. If not
/// HIPAR showDecodeDetail/enabled the data words are displayed as 32-bit data words.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDShowDecodeDetail(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean showDecodeDetail)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_PD_SHOW_DECODE_DETAIL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_SHOW_DECODE_DETAIL, showDecodeDetail),
            3, "Show Decode Detail");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the trigger type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBThree:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Selects the trigger type.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerType(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the frame to be triggered on, if the frame does not have any
/// HIFN  fields, e.g. a test frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_PACKET_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR packetType/Selects the frame to be triggered on, if the frame does not have any
/// HIPAR packetType/fields, e.g. a test frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerPacketType(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   packetType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_PACKET_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_PACKET_TYPE, packetType),
            3, "Packet Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific error in the
/// HIFN  selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USPD:ERRor:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerErrorEnabled(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViString  errorName,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:ERR:ENAB '%s',%s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:ENABle
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR frameName/Selects the frame to be triggered on, if the frame does not have any
/// HIPAR frameName/fields, e.g. a test frame.
/// HIPAR enable/Enable field name.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   fieldNumber,
                            ViInt32   frameName,
                            ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fieldNumber, 1, 20),
            3, "Field Number");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSBPDPacketType, frameName, &p2string),
                4, "Frame Name");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM:FIEL%ld:ENAB %s,%s", trigger, fieldNumber, p2string,
             enable ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the USB PD trigger data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_MIN
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:DOPerator
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:DMIN
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:DMAX
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR condition/Sets the condition for the data pattern. You can define no value, an
/// HIPAR condition/exact value or a value range.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerData(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   fieldNumber,
                                                    ViInt32   condition,
                                                    ViString  min,
                                                    ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_field[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_CONDITION,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap_field),
            3, "Field Number");

    strcat(repCap, ",");
    strcat(repCap, repCap_field);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_MIN, min),
            5, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_DATA_MAX, max),
            6, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the USB PD trigger index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:IOPerator
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:IMIN
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:IMAX
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR condition/Sets the operator for the index in the selected field of the selected
/// HIPAR condition/frame.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerIndex(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   fieldNumber,
                                                     ViInt32   condition,
                                                     ViInt32   min,
                                                     ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_field[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_CONDITION,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap_field),
            3, "Field Number");

    strcat(repCap, ",");
    strcat(repCap, repCap_field);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_MIN, min),
            5, "Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_INDEX_MAX, max),
            6, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_TRIGGER_BIT_STATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe:FIELd<n>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerBitState(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   fieldNumber,
                                                        ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_field[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_BIT_STATE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_TRIGGER_BIT_STATE,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap_field),
            3, "Field Number");

    strcat(repCap, ",");
    strcat(repCap, repCap_field);

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_TRIGGER_BIT_STATE, bitState),
            4, "Bit State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the checking condition for a specific error in the
/// HIFN  selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:EREN '%s',%s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific error in the
/// HIFN  selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:ERRor<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorNumber/Selects the error number.
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:ERR%ld:ENAB %s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FREN '%s',%s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:ENAB %s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR enabled/Enables or disables the checking condition for a specific data pattern
/// HIPAR enabled/in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FIEN '%s','%s',%s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:FLD<o>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR enabled/Enables or disables the checking condition for a specific data pattern
/// HIPAR enabled/in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:ENAB %s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNameBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:BIT '%s','%s',%s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:FLD<o>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:BIT %s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:DOPerator
/// HIFN     TRIGger<m>:USBPd:DMIN
/// HIFN     TRIGger<m>:USBPd:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNameData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:DOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:DMIN '%s','%s','%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:DMAX '%s','%s','%s'", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:FLD<o>:DOPerator
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:FLD<o>:DMIN
/// HIFN     TRIGger<m>:USBPd:FRAMe<n>:FLD<o>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:DOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:DMIN '%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:DMAX '%s'", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNameIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:IOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:IMIN '%s','%s',%ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:IMAX '%s','%s',%ld", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame
/// HIPAR field/Selects the field
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDTriggerFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:IOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:IMIN %ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:USBP:FRAM%ld:FLD%ld:IMAX %ld", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_USB_PD_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameCount(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_USB_PD_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_USB_PD_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state, start and stop time, type of the frame with the
/// HIFN  selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:STATe?
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:STARt?
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:STOP?
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR frameStatus/Returns the state of the specified frame.
/// HIPAR startTime/Returns the start time of the frame with the selected frame number
/// HIPAR startTime/(for the selected serial bus).
/// HIPAR stopTime/Returns the stop time of the frame with the selected frame number (for
/// HIPAR stopTime/the selected serial bus).
/// HIPAR type/Returns the type of frame.
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameStatus(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   frame,
                                                ViInt32*  frameStatus,
                                                ViReal64* startTime,
                                                ViReal64* stopTime,
                                                ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameStatusArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns information on the frame with the selected frame number (for
/// HIFN  the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:INFO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR info/Returns information on the frame with the selected frame number (for
/// HIPAR info/the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameInfo(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:INFO?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameDataBytes(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   arraySize,
                                                   ViInt32   data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the type of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR type/Returns the type of the field with the selected field number within
/// HIPAR type/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameFieldType(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   field,
                                                   ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:TYPE?", serialBus, frame, field);
    checkErr(rsscope_QueryViString(instrSession, cmd, 512, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the status of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR status/Returns the status of the field with the selected field number within
/// HIPAR status/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameFieldStatus(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   field,
                                                     ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:STAT?", serialBus, frame, field);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameStatusArr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and stop times of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:STARt?
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR start/Returns the start time of the field with the selected field number
/// HIPAR start/within the frame with the selected frame number (for the selected
/// HIPAR start/serial bus).
/// HIPAR stop/Returns the stop time of the field with the selected field number
/// HIPAR stop/within the frame with the selected frame number (for the selected
/// HIPAR stop/serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameFieldStartStop(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32   field,
                                                        ViReal64* start,
                                                        ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:STAR?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:STOP?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:VAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameFieldValue(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32   field,
                                                    ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:FVAL?", serialBus, frame, field);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:USBPd:RESult:FRAMe<n>:FLD<o>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR value/Returns the formatted value of the field with the selected field
/// HIPAR value/number within the frame with the selected frame number (for the
/// HIPAR value/selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDFrameFieldFormatedValue(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt32   field,
                                                            ViInt32   bufferSize,
                                                            ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:USBP:RES:FRAM%ld:FLD%ld:FVAL?", serialBus, frame, field);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the trigger type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the trigger type.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchType(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_USB_PD_SEARCH_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the frame to be triggered on, if the frame does not have any
/// HIFN  fields, e.g. a test frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_PACKET_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packetType/Selects the frame to be triggered on, if the frame does not have any
/// HIPAR packetType/fields, e.g. a test frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchPacketType(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   packetType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_USB_PD_SEARCH_PACKET_TYPE, packetType),
            3, "Packet Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific error in the
/// HIFN  selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:ERRor:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable search error.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchErrorEnabled(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViString  errorName,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:ERR:ENAB '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for a specific data pattern
/// HIFN  in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR frameName/Selects the frame to be triggered on, if the frame does not have any
/// HIPAR frameName/fields, e.g. a test frame.
/// HIPAR enable/Enable field name.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   fieldNumber,
                            ViInt32   frameName,
                            ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fieldNumber, 1, 20),
            3, "Field Number");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngUSBPDPacketType, frameName, &p2string),
                4, "Frame Name");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM:FIEL%ld:ENAB '%s',%s,%s", fieldNumber, searchName,
             p2string, enable ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the USB PD Search Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_DATA_MIN
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:DOPerator
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:DMIN
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR condition/Sets the condition for the data pattern. You can define no value, an
/// HIPAR condition/exact value or a value range.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchData(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   fieldNumber,
                                                   ViInt32   condition,
                                                   ViString  min,
                                                   ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_SEARCH_DATA_CONDITION,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Field Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_DATA_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_DATA_MIN, min),
            5, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_DATA_MAX, max),
            6, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the USB PD Search Index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:IOPerator
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:IMIN
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR condition/Sets the operator for the index in the selected field of the selected
/// HIPAR condition/frame.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchIndex(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   fieldNumber,
                                                    ViInt32   condition,
                                                    ViInt32   min,
                                                    ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_CONDITION,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Field Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_MIN, min),
            5, "Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_INDEX_MAX, max),
            6, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_USB_PD_SEARCH_BIT_STATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe:FIELd<m>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR fieldNumber/Sets the field number.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchBitState(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   fieldNumber,
                                                       ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_USB_PD_SEARCH_BIT_STATE,
            "FieldNumber", fieldNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Field Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_USB_PD_SEARCH_BIT_STATE, bitState),
            4, "Bit State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  error in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/error in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:EREN '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  error in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:ERRor<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorNumber/Selects the error number
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/error in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   errorNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:ERR%ld:ENABle '%s',%s", errorNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FREN '%s','%s',%s", searchName, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:ENAB '%s',%s", frame, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FIEN '%s','%s','%s',%s", searchName, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:ENAB '%s',%s", frame, field, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNameBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:BIT '%s','%s','%s',%s", searchName, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:BIT '%s',%s", frame, field, searchName,
             p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:DOPerator
/// HIFN     SEARch:TRIGger:USBPd:DMIN
/// HIFN     SEARch:TRIGger:USBPd:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNameData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:DOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:DMIN '%s','%s','%s','%s'", searchName, frame, field,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:DMAX '%s','%s','%s','%s'", searchName, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:DOPerator
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:DMIN
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNumericData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:DOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:DMIN '%s','%s'", frame, field, searchName,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:DMAX '%s','%s'", frame, field, searchName,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for index search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR minimum/Sets the end value of a index range
/// HIPAR maximum/Specifies the index at which the data is to be searched, or sets the
/// HIPAR maximum/start value of an index range in which the data is to be searched.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNameIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:IOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:IMIN '%s','%s','%s',%ld", searchName, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:IMAX '%s','%s','%s',%ld", searchName, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:IOPerator
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:IMIN
/// HIFN     SEARch:TRIGger:USBPd:FRAMe<m>:FLD<n>:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR minimum/Sets the end value of a index range
/// HIPAR maximum/Specifies the index at which the data is to be searched, or sets the
/// HIPAR maximum/start value of an index range in which the data is to be searched.
ViStatus _VI_FUNC rsscope_ConfigureUSBPDSearchFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:IOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:IMIN '%s',%ld", frame, field, searchName,
             minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:USBP:FRAM%ld:FLD%ld:IMAX '%s',%ld", frame, field, searchName,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_USB_PD_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameCount(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_USB_PD_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state, start and stop time, type of the frame with the
/// HIFN  selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:STATe?
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:STARt?
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:STOP?
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR frameStatus/Returns the state of the specified frame.
/// HIPAR startTime/Returns the start time of the frame with the selected frame number
/// HIPAR startTime/(for the selected serial bus).
/// HIPAR stopTime/Returns the stop time of the frame with the selected frame number (for
/// HIPAR stopTime/the selected serial bus).
/// HIPAR type/Returns the type of frame.
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameStatus(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   frame,
                                                      ViInt32*  frameStatus,
                                                      ViReal64* startTime,
                                                      ViReal64* stopTime,
                                                      ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameStatusArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns information on the frame with the selected frame number (for
/// HIFN  the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:INFO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR info/Returns information on the frame with the selected frame number (for
/// HIPAR info/the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameInfo(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:INFO? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameDataBytes(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   arraySize,
                                                         ViInt32   data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the type of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR type/Returns the type of the field with the selected field number within
/// HIPAR type/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameFieldType(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   field,
                                                         ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:TYPE? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 512, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the status of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR status/Returns the status of the field with the selected field number within
/// HIPAR status/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameFieldStatus(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32   field,
                                                           ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:STAT? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolUSBPDFrameStatusArr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and stop times of the field with the selected field
/// HIFN  number within the frame with the selected frame number within the
/// HIFN  search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:STARt?
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR startTime/Returns the start time of the field with the selected field number
/// HIPAR startTime/within the frame with the selected frame number within the search
/// HIPAR startTime/result.
/// HIPAR stopTime/Returns the stop time of the field with the selected field number
/// HIPAR stopTime/within the frame with the selected frame number within the search
/// HIPAR stopTime/result.
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameFieldStartStop(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:STAR? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:STOP? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the value of the field with the selected field number within
/// HIFN  the frame with the selected frame number (for the selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:VAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number (for the selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameFieldValue(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32   field,
                                                          ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:VAL? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected frame number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:USBPd:FRAMe<m>:FLD<n>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR value/Returns the formatted value of the field with the selected field
/// HIPAR value/number within the frame with the selected frame number (for the
/// HIPAR value/selected serial bus).
ViStatus _VI_FUNC rsscope_QueryUSBPDSearchFrameFieldFormatedValue(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bufferSize,
                            ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:USBP:FRAM%ld:FLD%ld:FVAL? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the custom serial bus coding standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR standard/Selects the custom serial bus coding standard.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZCodingStandard(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CODING_STANDARD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CODING_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the bitrate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitrateState/Enables the bit rate settings (must always be enabled for the coding
/// HIPAR bitrateState/standard 'NRZ Unclocked', and also for triggering on signals in any
/// HIPAR bitrateState/coding standard).
/// HIPAR bitrate/Sets the transmission speed setting for the custom serial bus data
/// HIPAR bitrate/signal.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZBitrate(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViBoolean bitrateState,
                                                        ViInt32   bitrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_BITRATE_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_BITRATE_ENABLED, bitrateState),
            3, "Bitrate State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_BITRATE, bitrate),
            4, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the gap time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR gapTimeState/Enables the gap time settings (must always be enabled for the coding
/// HIPAR gapTimeState/standard 'NRZ Unclocked', and also for triggering on signals in any
/// HIPAR gapTimeState/coding standard).
/// HIPAR gapTime/Sets a minimum gap time for synchronization.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZGapTime(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViBoolean gapTimeState,
                                                        ViReal64  gapTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_GAP_TIME_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_GAP_TIME_ENABLED, gapTimeState),
            3, "Gap Time State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_GAP_TIME, gapTime),
            4, "Gap Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the manchester coding standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Selects the source of the waveform.
/// HIPAR polarity/Selects the polarity of the custom serial bus data signal in
/// HIPAR polarity/Manchester coding standards.
/// HIPAR clockPhase/Selects the phase of the custom serial bus clock signal for the
/// HIPAR clockPhase/'Manchester' coding standards.
ViStatus _VI_FUNC rsscope_ConfigureManchesterSettings(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   source,
                                                      ViInt32   polarity,
                                                      ViInt32   clockPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_DATA_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_PROTOCOL_WAV_C1W1, RSSCOPE_VAL_PROTOCOL_WAV_M8),
            3, "Source");
    if ((RSSCOPE_VAL_PROTOCOL_WAV_R4 < source) && (source < RSSCOPE_VAL_PROTOCOL_WAV_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source),
                    3, "Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_DATA_SOURCE, source),
            3, "Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_CLOCK_PHASE, clockPhase),
            5, "Clock Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures threshold values used for digitization of the
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets the Manchester thresholds to predefined or individually definable
/// HIPAR preset/voltage levels.
/// HIPAR thresholdHigh/Sets the upper threshold for data signal digitization in Manchester
/// HIPAR thresholdHigh/coding standards.
/// HIPAR thresholdLow/Sets the lower threshold for data signal digitization in Manchester
/// HIPAR thresholdLow/coding standards.
ViStatus _VI_FUNC rsscope_ConfigureManchesterThreshold(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   preset,
                                                       ViReal64  thresholdHigh,
                                                       ViReal64  thresholdLow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_PRESET, preset),
            3, "Preset");

    if (preset == RSSCOPE_VAL_MANCHESTER_NRZ_THRESHOLD_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_HIGH, thresholdHigh),
                4, "Threshold High");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_LOW, thresholdLow),
                5, "Threshold Low");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples the upper and lower threshold values for the Manchester and
/// HIFN  Manchester II coding standards. The values are coupled to voltages with
/// HIFN  the same magnitude but opposite sign (positive for the upper threshold
/// HIFN  and negative for the lower threshold).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Couples the upper and lower threshold values for the Manchester and
/// HIPAR coupling/Manchester II coding standards. The values are coupled to voltages with
/// HIPAR coupling/the same magnitude but opposite sign (positive for the upper threshold
/// HIPAR coupling/and negative for the lower threshold).
ViStatus _VI_FUNC rsscope_ConfigureManchesterThresholdCoupling(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_THRESHOLD_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the input source for the custom serial bus enable signal.
/// HIFN     If an input is chosen, signals will be only decoded while this channel
/// HIFN  is in the enabled state. This allows you to mark a time when the signal
/// HIFN  on the selected source is active and when not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Selects the input source for the custom serial bus enable signal.
ViStatus _VI_FUNC rsscope_ConfigureNRZEnable(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_ENABLE_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_ENABLE_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source channel for the data or clock signal. For
/// HIFN  triggering on the serial bus when the NRZ clocked coding standard is
/// HIFN  selected, analog or digital channel sources are required. For
/// HIFN  triggering on the serial bus when the NRZ unclocked coding standard is
/// HIFN  selected, analog channel sources are required. Otherwise, if no serial
/// HIFN  bus trigger has been selected, permitted source selections include the
/// HIFN  mathematical channels and the reference channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select the line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsscope_ConfigureNRZSource(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   line,
                                             ViInt32   waveformSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_CLOCK_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSSCOPE_VAL_SOURCE_CLOCK, RSSCOPE_VAL_SOURCE_DATA),
            3, "Line");

    if (line == RSSCOPE_VAL_SOURCE_CLOCK)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_CLOCK_SOURCE, waveformSource),
                4, "Waveform Source");
    }
    else
    {
        if (waveformSource == RSSCOPE_VAL_PROTOCOL_WAV_NONE)
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, waveformSource), 4, "Waveform Source");
        }
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_DATA_SOURCE, waveformSource),
                4, "Waveform Source");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the custom serial bus clock signal polarity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR line/This control select the line.
/// HIPAR polarity/Selects the custom serial bus clock signal.
ViStatus _VI_FUNC rsscope_ConfigureNRZIdlePolarity(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   line,
                                                   ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_IDLE_POLARITY_UNCLOCKED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    if (line == RSSCOPE_VAL_SOURCE_CLOCK)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_IDLE_POLARITY_CLOCKED, polarity),
                3, "Polarity");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_IDLE_POLARITY_UNCLOCKED, polarity),
                3, "Polarity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the polarity of the custom serial bus data signal in NRZ
/// HIFN  coding standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Selects the polarity of the custom serial bus data signal in NRZ
/// HIPAR polarity/coding standards.
ViStatus _VI_FUNC rsscope_ConfigureNRZPolarity(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_POLARITY, polarity),
            3, "polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets whether the transmitted enable signal is active when the voltage
/// HIFN  is below the threshold (ENALow) or higher than it (ENAHIgh).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_NRZ_ENA_POLARITY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:NRZ:ENAPolarity
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Sets whether the transmitted enable signal is active when the voltage
/// HIPAR polarity/is below the threshold (ENALow) or higher than it (ENAHIgh).
ViStatus _VI_FUNC rsscope_ConfigureNRZENAPolarity(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_ENA_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_ENA_POLARITY, polarity),
            3, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the NRZ coding standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR clockPhase/Selects the phase of the custom serial bus clock signal for the NRZ
/// HIPAR clockPhase/coding standards.
ViStatus _VI_FUNC rsscope_ConfigureNRZClockPhase(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   clockPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_CLOCK_PHASE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_CLOCK_PHASE, clockPhase),
            3, "Clock Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures threshold values used for digitization of the
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets the Manchester thresholds to predefined or individually definable
/// HIPAR preset/voltage levels.
/// HIPAR thresholdClock/Sets the threshold for the clock signal digitization in the NRZ
/// HIPAR thresholdClock/Clocked coding standard.
/// HIPAR thresholdData/Sets the threshold for the data signal digitization in NRZ coding
/// HIPAR thresholdData/standards.
ViStatus _VI_FUNC rsscope_ConfigureNRZThreshold(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   preset,
                                                ViReal64  thresholdClock,
                                                ViReal64  thresholdData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_PRESET, preset),
            3, "Preset");
    if (preset == RSSCOPE_VAL_MANCHESTER_NRZ_THRESHOLD_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_CLOCK, thresholdClock),
                4, "Threshold Clock");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_DATA, thresholdData),
                5, "Threshold Data");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples the clock and data threshold values for the NRZ Clocked coding
/// HIFN  standard. The values are coupled to the same number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Couples the clock and data threshold values for the NRZ Clocked coding
/// HIPAR coupling/standard. The values are coupled to the same number.
ViStatus _VI_FUNC rsscope_ConfigureNRZThresholdCoupling(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_COUPLING, coupling),
            3, "coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the threshold for the enable signal digitization in NRZ coding
/// HIFN  standards.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:NRZ:THReshold:ENBLe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enabled/Sets the threshold for the enable signal digitization in NRZ coding
/// HIPAR enabled/standards.
ViStatus _VI_FUNC rsscope_ConfigureNRZThresholdValue(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViReal64  enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_VALUE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_NRZ_THRESHOLD_VALUE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the frame count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the frame count.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZFrameCount(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the user to set a string to describe the frame
/// HIFN  type, typically according to the applicable protocol standard
/// HIFN  specifications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR type/Enables the user to set a string to describe the frame type, typically
/// HIPAR type/according to the applicable protocol standard specifications.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFrameType(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   frame,
                                                          ViString  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates an empty frame format description and adds it to the end of
/// HIFN  the frame description list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ManchesterNRZAddFrame(ViSession instrSession,
                                                ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_ADD_FRAME,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_ADD_FRAME, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Erases all cells and frames that have been created for a specific
/// HIFN  custom protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ManchesterNRZFrameClear(ViSession instrSession,
                                                  ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_CLEAR,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves or opens frame description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR file/This control specifies the string with the name of frame description
/// HIPAR file/file.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ManchesterNRZFrameDescription(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   operation,
                                                        ViString  file)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_SAVE_DESCRIPTION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    if (operation == RSSCOPE_VAL_SAVE)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_SAVE_DESCRIPTION, file),
                3, "File");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_OPEN_DESCRIPTION, file),
                3, "File");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the decoded frames as raw bits.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_EXPORT_FRAMES_AS_RAW_BITS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:EXRBits
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ManchesterNRZExportFramesAsRawBits(ViSession instrSession,
                                                             ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_EXPORT_FRAMES_AS_RAW_BITS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_EXPORT_FRAMES_AS_RAW_BITS, ""),
            3, "Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the cell count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Returns the cell count.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZCellCount(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            4, "Frame");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COUNT, count),
            4, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates an empty frame format description and adds it to the end of
/// HIFN  the frame description list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
ViStatus _VI_FUNC rsscope_ManchesterNRZAddCell(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_ADD_CELL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_ADD_CELL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR name/Enables the user to set a cell name within a frame. Names do not have
/// HIPAR name/to be unique, they are just for user support.
/// HIPAR bitCount/Sets the bit count of a cell, hence its length. Based upon the lengths
/// HIPAR bitCount/of the previous cells, this also defines the position of the cell start
/// HIPAR bitCount/and end within a frame.
/// HIPAR condition/Sets various operators for a cell, to identify, e.g., mandatory values
/// HIPAR condition/such as a CRC checksum or an ID, that help to identify a frame. The
/// HIPAR condition/implemented conditions and functionalities are the 'equal' and 'array'
/// HIPAR condition/operators. The numeric format of the condition needs to be set
/// HIPAR condition/according to RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_FORMAT.
/// HIPAR cellFormat/Selects the numeric data format for the
/// HIPAR cellFormat/RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_CONDITION, as well as for the
/// HIPAR cellFormat/result and honeycomb display.
/// HIPAR bitOrder/Selects in which order the bits of a cell are evaluated, as well as
/// HIPAR bitOrder/presented in the results table and honeycomb display.
/// HIPAR color/Selects a cell's color representation in the honeycomb display.
/// HIPAR resultColumn/Selects which cell shall be displayed in which result column of the
/// HIPAR resultColumn/decode table. The decode table supports three result columns, which
/// HIPAR resultColumn/have to be unique for each frame type. For different frame types,
/// HIPAR resultColumn/though, different result columns can be defined to display unrelated
/// HIPAR resultColumn/information.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZCell(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   cell,
                                                     ViString  name,
                                                     ViInt32   bitCount,
                                                     ViString  condition,
                                                     ViInt32   cellFormat,
                                                     ViInt32   bitOrder,
                                                     ViInt32   color,
                                                     ViInt32   resultColumn)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_NAME,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL, cell),
            4, "Cell");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_NAME, name),
            5, "Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_BIT_COUNT, bitCount),
            6, "Bit Count");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_CONDITION, condition),
            7, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_FORMAT, cellFormat),
            7, "Cell Format");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_BIT_ORDER, bitOrder),
            8, "Bit Order");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COLOR, color),
            9, "Color");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_RESULT_COLUMN, resultColumn),
            10, "Result Column");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the cell's color representation in the honeycomb display.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COLOR_CRGB
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FRAMe<n>:CELL<o>:CRGB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR colorRGB/Selects a cell's color representation in the honeycomb display. ARGB
/// HIPAR colorRGB/value of the color to be used for the table entry.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZCellColorCRGB(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   cell,
                            ViInt64   colorRGB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COLOR_CRGB,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_CELL, cell),
            4, "Cell");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_CELL_COLOR_CRGB,
                    colorRGB), 5, "Color RGB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger type for custom serial bus analysis.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerType(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR dataPattern/Sets the pattern match conditions for a payload data check. The
/// HIPAR dataPattern/trigger is set to the first occurrence of a matching data bit pattern
/// HIPAR dataPattern/(which can be freely specified), starting after the minimum gap time
/// HIPAR dataPattern/and after the detected start of the data frame.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerDataPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  dataPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_DATA_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_DATA_PATTERN, dataPattern),
            3, "Data Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific bit index (data position).
/// HIPAR indexMin/Sets the bit index (data position). If Index Condition is set to IN
/// HIPAR indexMin/Range, IndexMin sets the start value of the bit index range.
/// HIPAR indexMax/Sets the end value of the bit index range (data position range), if
/// HIPAR indexMax/condition is set to In Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerIndex(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViInt32   condition,
                                                             ViInt32   indexMin,
                                                             ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_INDEX_MIN, indexMin),
            4, "Index Min");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MANCHESTER_NRZ_TRIGGER_INDEX_MAX, indexMax),
                5, "Index Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the number of bits in an NRZ unclocked word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR wordLength/Sets the number of bits in an NRZ unclocked word.
ViStatus _VI_FUNC rsscope_ConfigureNRZTriggerWordLength(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   wordLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_NRZ_TRIGGER_WORD_LENGTH,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_NRZ_TRIGGER_WORD_LENGTH, wordLength),
            3, "Word Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables triggering on errors. You can search for all error types in
/// HIFN  parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/The following error strings are available:
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerErrorNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:EREN '%s',%s", trigger, errorName, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables triggering on errors. You can search for all error types in
/// HIFN  parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:ERRor<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorNumber/Selects the error number.
/// HIPAR enabled/Enable or disable trigger error.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerErrorNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   errorNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:ERR%ld:ENAB %s", trigger, errorNumber,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FREN '%s',%s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:ENAB %s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the specific field within the defined frame to be
/// HIPAR enabled/triggered on.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FIEN '%s','%s',%s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the specific field within the defined frame to be
/// HIPAR enabled/triggered on.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:ENABle %s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be triggered on that only consists of
/// HIFN  one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNameBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:BIT '%s','%s',%s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be triggered on that only consists of
/// HIFN  one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                4, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:BIT %s", trigger, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:DOPerator
/// HIFN     TRIGger<m>:CMSB:ADVanced:DMIN
/// HIFN     TRIGger<m>:CMSB:ADVanced:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNameData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:DOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:DMIN '%s','%s','%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:DMAX '%s','%s','%s'", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for data trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:DOPerator
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:DMIN
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if the corresponding
/// HIPAR maximum/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:DOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:DMIN '%s'", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:DMAX '%s'", trigger, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:IOPerator
/// HIFN     TRIGger<m>:CMSB:ADVanced:IMIN
/// HIFN     TRIGger<m>:CMSB:ADVanced:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index in the selected field of the selected
/// HIPAR condition/frame.
/// HIPAR minimum/Specifies the index, or sets the start value of an index range.
/// HIPAR maximum/Sets the end value of an index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNameIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:IOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:IMIN '%s','%s',%ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:IMAX '%s','%s',%ld", trigger, frame, field, maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for index trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:IOPerator
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:IMIN
/// HIFN     TRIGger<m>:CMSB:ADVanced:FRAMe<n>:FLD<o>:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index in the selected field of the selected
/// HIPAR condition/frame.
/// HIPAR minimum/Specifies the index, or sets the start value of an index range.
/// HIPAR maximum/Sets the end value of an index range if the corresponding condition is
/// HIPAR maximum/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZTriggerFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   minimum,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:IOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:IMIN %ld", trigger, frame, field, minimum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:CMSB:ADV:FRAM%ld:FLD%ld:IMAX %ld", trigger, frame, field,
                 maximum);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the count number of decoded result frames in a custom serial
/// HIFN  bus waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the count number of decoded result frames in a custom serial
/// HIPAR count/bus waveform.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultCount(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_RESULT_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_RESULT_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR result/This control selects the result.
/// HIPAR status/Returns the overall state of the frame: either OK or the relevant
/// HIPAR status/error condition. R&S RTO-K50 marks each frame with a status that
/// HIPAR status/indicates whether the decode succeeded or not.
/// HIPAR start/Returns the start time of the frame.
/// HIPAR stop/Returns the stop time of the frame.
/// HIPAR type/Returns the name of the selected frame (n) from the user defined frame
/// HIPAR type/format description.
/// HIPAR bufferSize/This control indicates the allocated memory in Type.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultStatus(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   result,
                                                         ViInt32   bufferSize,
                                                         ViInt32*  status,
                                                         ViReal64* start,
                                                         ViReal64* stop,
                                                         ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:STAT?", serialBus, result);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolManchesterNRZStatus, status));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:STAR?", serialBus, result);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:STOP?", serialBus, result);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:TYPE?", serialBus, result);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(type, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the cell content as specified in the Result Column of the
/// HIFN  'Frame Format' specification table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:RESult<n>:CONE?
/// HIFN     BUS<m>:CMSB:RESult<n>:CTWO?
/// HIFN     BUS<m>:CMSB:RESult<n>:CTHRee?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR result/This control selects the result.
/// HIPAR contentOne/Returns the 1st  cell content as specified in the Result Column of the
/// HIPAR contentOne/'Frame Format' specification table.
/// HIPAR bufferSize/This control indicates the allocated memory in ViChar arrays.
/// HIPAR contentTwo/Returns the 2st cell content as specified in the Result Column of the
/// HIPAR contentTwo/'Frame Format' specification table.
/// HIPAR contentThree/Returns the 3rd cell content as specified in the Result Column of the
/// HIPAR contentThree/'Frame Format' specification table.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultContent(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   result,
                                                          ViInt32   bufferSize,
                                                          ViChar    contentOne[],
                                                          ViChar    contentTwo[],
                                                          ViChar    contentThree[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    //one
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CONE?", serialBus, result);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(contentOne, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

    ///two
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CTWO?", serialBus, result);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(contentTwo, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

    ///three
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CTHR?", serialBus, result);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(contentThree, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded cells.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR result/This control selects the result.
/// HIPAR count/Returns the number of decoded cells.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultCellCount(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   result,
                                                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CCO?", serialBus, result);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR result/This control selects the result.
/// HIPAR cell/This control selects the cell.
/// HIPAR bufferSize/This control indicates the allocated memory in Name.
/// HIPAR status/Returns the state of the cell.
/// HIPAR name/Returns the state of the cell.
/// HIPAR value/Data content of the specified cell.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultCellStatus(ViSession instrSession,
                                                             ViInt32   serialBus,
                                                             ViInt32   result,
                                                             ViInt32   cell,
                                                             ViInt32   bufferSize,
                                                             ViInt32*  status,
                                                             ViChar    name[],
                                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CELL%ld:STAT?", serialBus, result, cell);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolManchesterNRZStatus_Arr, status));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CELL%ld:NAME?", serialBus, result, cell);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(name, bufferSize, extBuf);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CELL%ld:VAL?", serialBus, result, cell);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and stop times of the selected cell.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:RESult<n>:CELL<o>:STARt?
/// HIFN     BUS<m>:CMSB:RESult<n>:CELL<o>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR result/This control selects the result.
/// HIPAR cell/This control selects the cell.
/// HIPAR start/Returns the start time of the selected cell.
/// HIPAR stop/Returns the end time of the selected cell.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZResultCellStartStop(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   result,
                            ViInt32   cell,
                            ViReal64* start,
                            ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CELL%ld:STAR?", serialBus, result, cell);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:RES%ld:CELL%ld:STOP?", serialBus, result, cell);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the error type to be searched for. You can search for all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/Specifies search error.
/// HIPAR enabled/Defines the error type to be searched for. You can search for all
/// HIPAR enabled/error types in parallel.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:EREN '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be searched for. You can search for all
/// HIFN  error types in parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:ERRor<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorNumber/Sets the error number.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines the error type to be searched for. You can search for all
/// HIPAR enabled/error types in parallel.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   errorNumber,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:ERR%ld:ENABle '%s',%s", errorNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FREN '%s','%s',%s", searchName, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameNumber/Sets the frame number.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frameNumber,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:ENAB '%s',%s", frameNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the specific field within the defined frame to be
/// HIPAR enabled/searched for.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FIEN '%s','%s','%s',%s", searchName, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:ENAB '%s',%s", frame, field, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR bitState/Sets the bit state of a field to be searched that only consists of one
/// HIPAR bitState/bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNameBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:BIT '%s','%s','%s',%s", searchName, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bitState/Sets the bit state of a field to be searched that only consists of one
/// HIPAR bitState/bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:BIT '%s',%s", frame, field, searchName,
             p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:DOPerator
/// HIFN     SEARch:TRIGger:CMSB:DMIN
/// HIFN     SEARch:TRIGger:CMSB:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range if
/// HIPAR max/SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:DOPerator is set to INRange or
/// HIPAR max/OORange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNameData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:DOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:DMIN '%s','%s','%s','%s'", searchName, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:DMAX '%s','%s','%s','%s'", searchName, frame, field,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:DOPerator
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:DMIN
/// HIFN     SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range if
/// HIPAR max/SEARch:TRIGger:CMSB:FRAMe<m>:FLD<n>:DOPerator is set to INRange or
/// HIPAR max/OORange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:DOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:DMIN '%s','%s'", frame, field, searchName,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:DMAX '%s','%s'", frame, field, searchName,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Sets the bit index (data position).
/// HIPAR max/Sets the end value of the bit index range (data position range), if
/// HIPAR max/RSSCOPE_FUNC_SEARCH_MANCHESTER_NRZ_FRAME_FIELD_NAME_INDEX_CONDITION is
/// HIPAR max/set to INRange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNameIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:IOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:IMIN '%s','%s','%s',%ld", searchName, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:IMAX '%s','%s','%s',%ld", searchName, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Sets the bit index (data position).
/// HIPAR max/Sets the end value of the bit index range (data position range), if
/// HIPAR max/RSSCOPE_FUNC_SEARCH_MANCHESTER_NRZ_FRAME_FIELD_NUMERIC_INDEX_CONDITION
/// HIPAR max/is set to INRange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZSearchFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:IOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:IMIN '%s',%ld", frame, field, searchName,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:CMSB:FRAM%ld:FLD%ld:IMAX '%s',%ld", frame, field, searchName,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the count number of decoded result frames in a custom serial
/// HIFN  bus waveform.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR count/Returns the count number of decoded result frames in a custom serial
/// HIPAR count/bus waveform.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchFrameCount(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   index,
                                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FCO? '%s'", index, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:STATe?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:STARt?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:STOP?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the frame within the search result:
/// HIPAR frameStatus/either OK or the relevant error condition.
/// HIPAR frameStart/Returns the start time of the frame within the search result.
/// HIPAR frameStop/Returns the stop time of the frame within the search result.
/// HIPAR frameType/Returns the name of the selected frame (n) from the user defined frame
/// HIPAR frameType/format description, labeled according to
/// HIPAR frameType/RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FRAME_TYPE.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchFrameStatus(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   index,
                            ViInt32   frame,
                            ViInt32*  frameStatus,
                            ViReal64* frameStart,
                            ViReal64* frameStop,
                            ViInt32   bufferSize,
                            ViChar    frameType[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:STAT? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolManchesterNRZStatus_Arr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:STAR? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:STOP? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:TYPE? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &data));
    RsCore_CopyToUserBufferAsciiData(frameType, bufferSize, data);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (data)
        free(data);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame content.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CONE?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CTWO?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CTHRee?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR one/Returns the 1st cell content within the search result, as specified in
/// HIPAR one/the 'Result Column' of the 'Frame Format' specification table.
/// HIPAR two/Returns the 2nd cell content within the search result, as specified in
/// HIPAR two/the 'Result Column' of the 'Frame Format' specification table.
/// HIPAR three/Returns the 3rd cell content within the search result, as specified in
/// HIPAR three/the 'Result Column' of the 'Frame Format' specification table.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchFrameContent(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   index,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    one[],
                            ViChar    two[],
                            ViChar    three[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    //one
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CONE? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(one, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

    ///two
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CTWO? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(two, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

    ///three
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CTHR? '%s'", index, frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(three, bufferSize, extBuf);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the count number of cell in the frames, within the search
/// HIFN  result, in a custom serial bus waveform.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Returns the count number of cell in the frames, within the search
/// HIPAR count/result, in a custom serial bus waveform.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchCellCount(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   index,
                                                            ViInt32   frame,
                                                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CCO? '%s'", index, frame, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the cell within the search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CELL<o>:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/Sets the cell parameter.
/// HIPAR status/Returns the state of the cell within the search result.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchCellStatus(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   index,
                                                             ViInt32   frame,
                                                             ViInt32   cell,
                                                             ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CELL%ld:STAT? '%s'", index, frame, cell,
             searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolManchesterNRZStatus_Arr, status));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the times of the cell within the search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CELL<o>:STARt?
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CELL<o>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/Sets the cell parameter.
/// HIPAR start/Returns the start time of the cell within the search result.
/// HIPAR stop/Returns the stop time of the cell within the search result.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchCellTime(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   index,
                                                           ViInt32   frame,
                                                           ViInt32   cell,
                                                           ViReal64* start,
                                                           ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CELL%ld:STAR? '%s'", index, frame, cell,
             searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CELL%ld:STOP? '%s'", index, frame, cell,
             searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the name of the specified cell within the search result. Cell
/// HIFN  names are not necessarily unique.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CELL<o>:NAME?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/Sets the cell parameter.
/// HIPAR name/Returns the name of the specified cell within the search result. Cell
/// HIPAR name/names are not necessarily unique.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchCellName(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   index,
                                                           ViInt32   frame,
                                                           ViInt32   cell,
                                                           ViInt32   bufferSize,
                                                           ViChar    name[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAM%ld:CELL%ld:NAME? '%s'", index, frame, cell,
             searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, name));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Data content of the specified cell within the search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CMSB<m>:FRAMe<n>:CELL<o>:VALue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR index/Selects the index.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/Sets the cell parameter.
/// HIPAR value/Data content of the specified cell within the search result.
ViStatus _VI_FUNC rsscope_QueryManchesterNRZSearchCellValue(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   index,
                                                            ViInt32   frame,
                                                            ViInt32   cell,
                                                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CMSB%ld:FRAMe%ld:CELL%ld:VAL? '%s'", index, frame, cell,
             searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables filtering for the custom serial bus.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enabled/Enables filtering for the custom serial bus.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_ENABLED, enabled),
            3, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be filtered.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:FRAMe<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Sets the frame number.
/// HIPAR enabled/Enables or disables the specific frame to be filtered.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:FRAM%ld:ENAB %s", serialBus, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be filtered.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:FRENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Sets the frame number.
/// HIPAR enabled/Enables or disables the specific frame to be filtered.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:FREN '%s',%s", serialBus, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables filtering on error. You can filter all error types in
/// HIFN  parallel.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:ERRor<n>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR error/Sets the error.
/// HIPAR enabled/Enables filtering on error. You can filter all error types in
/// HIPAR enabled/parallel.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   _error,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:ERR%ld:ENAB %s", serialBus, _error,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables filtering on error.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:ERENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR error/Sets the error.
/// HIPAR enabled/Enables filtering on error.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  _error,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:EREN '%s',%s", serialBus, _error, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  filtered.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:FIENable
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Sets the frame number.
/// HIPAR field/Sets the field.
/// HIPAR enabled/Enables or disables the specific field within the defined frame to be
/// HIPAR enabled/filtered.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFieldEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:FIEN '%s','%s',%s", serialBus, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be filtered on that only consists of
/// HIFN  one bit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:BIT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Sets the frame number.
/// HIPAR field/Sets the field.
/// HIPAR bitState/Sets the bit state of a field to be filtered on that only consists of
/// HIPAR bitState/one bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterBitState(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitState, &p2string),
                5, "Bit State");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:BIT '%s','%s',%s", serialBus, frame, field, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the filter data pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CMSB:FILTer:DOPerator
/// HIFN     BUS<m>:CMSB:FILTer:DMIN
/// HIFN     BUS<m>:CMSB:FILTer:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern.
/// HIPAR min/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data pattern range if
/// HIPAR max/RSSCOPE_FUNC_PROTOCOL_MANCHESTER_NRZ_FILTER_DATA_PATTERN_CONDITION is
/// HIPAR max/set to INRange or OORange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterDataPattern(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:DOP '%s','%s',%s", serialBus, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:DMIN '%s','%s','%s'", serialBus, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:DMAX '%s','%s','%s'", serialBus, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the filter index pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index pattern.
/// HIPAR min/Specifies the index pattern, or sets the start value of a pattern
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a index pattern range if
/// HIPAR max/RSSCOPE_FUNC_PROTOCOL_MANCHESTER_NRZ_FILTER_INDEX_PATTERN_CONDITION is
/// HIPAR max/set to INRange or OORange.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterIndexPattern(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            5, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:IOP '%s','%s',%s", serialBus, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:IMIN '%s','%s',%ld", serialBus, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CMSB:FILT:IMAX '%s','%s',%ld", serialBus, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  filtered.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR numeric/Enables or disables the specific field within the defined frame to be
/// HIPAR numeric/filtered.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean numeric)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_NUMERIC_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD, field),
            4, "Field");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_NUMERIC_ENABLED, numeric),
            5, "Numeric");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be filtered on that only consists of
/// HIFN  one bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field to be filtered on that only consists of
/// HIPAR bitState/one bit.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameFieldBitState(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_BIT_STATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD, field),
            4, "Field");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_BIT_STATE, bitState),
                5, "Bit State")

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the settings for the data pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR patternCondition/Sets the operator for the data pattern.
/// HIPAR minimum/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data pattern range if
/// HIPAR maximum/RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_CONDITION
/// HIPAR maximum/is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameFieldDataPatternSettings(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   patternCondition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_CONDITION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD, field),
            4, "Field");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_CONDITION, patternCondition),
            5, "Pattern Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_MIN, minimum),
            6, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_MAX, maximum),
            7, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the index pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR patternCondition/Sets the operator for the index in the selected field of the selected
/// HIPAR patternCondition/frame.
/// HIPAR minimum/Specifies the index, or sets the start value of an index range.
/// HIPAR maximum/Sets the end value of a index pattern range if
/// HIPAR maximum/RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_INDEX_PATTERN_CONDITION
/// HIPAR maximum/is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureManchesterNRZFilterFrameFieldIndexPatternSettings(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   patternCondition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_DATA_PATTERN_CONDITION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FIELD, field),
            4, "Field");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_INDEX_PATTERN_CONDITION, patternCondition),
            5, "Pattern Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_INDEX_PATTERN_MIN, minimum),
            6, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_FILTER_FRAME_FIELD_INDEX_PATTERN_MAX, maximum),
            7, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the signal type that is used for the decoding. You can choose
/// HIFN  between a differential and single-ended signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signalType/Selects the signal type that is used for the decoding. You can choose
/// HIPAR signalType/between a differential and single-ended signal.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSignalType(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   signalType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_SIGNAL_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_SIGNAL_TYPE, signalType),
            3, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the source of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocolType/This control selects EBTB protocol type.
/// HIPAR source/Defines the source of the signal.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSource(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   protocolType,
                                              ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_D_PLUS_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_NONE, RSSCOPE_VAL_WAV_M8),
            4, "Source");
    if (source < RSSCOPE_VAL_WAV_M1)
    {
        switch (source)
        {
        case RSSCOPE_VAL_WAV_NONE:
            if (protocolType == RSSCOPE_VAL_EBTB_TYPE_DIFF)
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 4, "Waveform Source");
            break;
        case RSSCOPE_VAL_WAV_C1W1:
        case RSSCOPE_VAL_WAV_C2W1:
        case RSSCOPE_VAL_WAV_C3W1:
        case RSSCOPE_VAL_WAV_C4W1:
            // Valid values below RSSCOPE_VAL_WAV_M1
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 4, "Source");
            break;
        }
    }
    else if ((RSSCOPE_VAL_WAV_R4 < source) && (source < RSSCOPE_VAL_WAV_M6))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 4, "Source");
    }

    switch (protocolType)
    {
    case RSSCOPE_VAL_EBTB_TYPE_D_PLUS:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_D_PLUS_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_EBTB_TYPE_D_MINUS:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_D_MINUS_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_EBTB_TYPE_DIFF:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_DIFFERENTIAL_SOURCE, source),
                4, "Source");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, protocolType), 3, "Protocol Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts software algorithms for determining the signal threshold levels
/// HIFN  and bitrate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ConfigureEBTBFullAutoset(ViSession instrSession,
                                                   ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_VARIANT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_FULL_AUTOSET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signalType/Selects the signal type that is used for the decoding. You can choose
/// HIPAR signalType/between a differential and single-ended signal.
/// HIPAR thresholdLow/Sets the lower threshold value for the signal digitization. If the
/// HIPAR thresholdLow/signal value is below this threshold, the signal state is considered
/// HIPAR thresholdLow/low.
/// HIPAR thresholdHigh/Sets the upper threshold value for the signal digitization. If the
/// HIPAR thresholdHigh/signal value is higher than the this threshold, the signal state is
/// HIPAR thresholdHigh/considered high.
ViStatus _VI_FUNC rsscope_ConfigureEBTBThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   signalType,
                                                 ViReal64  thresholdLow,
                                                 ViReal64  thresholdHigh)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_DIFFERENTIAL_THRESHOLD_HIGH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signalType, RSSCOPE_VAL_SIGNAL_TYPE_DIFF, RSSCOPE_VAL_SIGNAL_TYPE_SING),
            3, "Signal Type");

    if (signalType == RSSCOPE_VAL_SIGNAL_TYPE_DIFF)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_DIFFERENTIAL_THRESHOLD_HIGH, thresholdHigh),
                4, "Threshold High");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_DIFFERENTIAL_THRESHOLD_LOW, thresholdLow),
                5, "Threshold Low");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_THRESHOLD_D_PLUS, thresholdHigh),
                4, "Threshold High");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_THRESHOLD_D_MINUS, thresholdLow),
                5, "Threshold Low");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the bitrate.

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitrate/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsscope_ConfigureEBTBBitrate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViReal64  bitrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_BITRATE, bitrate),
            3, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a software algorithm for the automatic determination of the
/// HIFN  bitrate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_ConfigureEBTBBitrateDetermination(ViSession instrSession,
                                                            ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_BITRATE_DETERMINATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_BITRATE_DETERMINATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the estimated bitrate value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOLS_EBTB_ESTIMATED_BITRATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:EBTB:EBTRate?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR estimatedBitrate/Queries the estimated bitrate value.
ViStatus _VI_FUNC rsscope_ConfigureEBTBEstimatedBitrate(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViReal64* estimatedBitrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_ESTIMATED_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_ESTIMATED_BITRATE, estimatedBitrate),
            3, "Estimated Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function disables/enables the automatic software birtate
/// HIFN  determination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Disables/enables the automatic software birtate determination.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSWCDRBitrate(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_SW_CDR_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_SW_CDR_BITRATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the sync symbol, a control symbol used for low level control
/// HIFN  functions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR syncSymbol/Selects the sync symbol, a control symbol used for low level control
/// HIPAR syncSymbol/functions. You can select one of the comma control symbols (K28.1,
/// HIPAR syncSymbol/K28.5, K28.7) used for synchronisation or you can enter a pattern.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSyncSymbol(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   syncSymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_SYNC_SYMBOL,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_SYNC_SYMBOL, syncSymbol),
            3, "Sync Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a pattern that serves as a sync symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR pattern/Selects a pattern that serves as a sync symbol.
ViStatus _VI_FUNC rsscope_ConfigureEBTBPattern(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_PATTERN,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the value for the second pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Selects whether the second pattern is enabled.
/// HIPAR pattern/Sets the value for the second pattern.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSecondPattern(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViBoolean state,
                                                     ViInt32   pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_SECOND_PATTERN_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_SECOND_PATTERN_ENABLED, state),
            3, "State");

    if (state == VI_TRUE)
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_SECOND_PATTERN, pattern),
                4, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the display format for the results in the decode table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR displayFormat/Selects the display format for the results in the decode table.
ViStatus _VI_FUNC rsscope_ConfigureEBTBDisplayFormat(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   displayFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOLS_EBTB_DISPLAY_FORMAT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOLS_EBTB_DISPLAY_FORMAT, displayFormat),
            3, "Display Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of condition to be triggered on. You can trigger on a
/// HIFN  certain symbol or enable a specific error condition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR type/Selects the type of condition to be triggered on. You can trigger on a
/// HIPAR type/certain symbol or enable a specific error condition.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of condition to be triggered on for 8b10b.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use.
/// HIPAR conditionType/Selects the type of condition to be triggered on for 8b10b.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerConditionType(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   conditionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  triggerCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &triggerCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, triggerCount),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EBTB_TRIGGER_CONDITION_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EBTB_TRIGGER_CONDITION_TYPE, conditionType),
            3, "Condition Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data character (Dx.y) or control character to bea
/// HIFN  triggered on. You can specify he value of the data character to be
/// HIFN  triggered on with TRIGger<m>:EBTB:DX and TRIGger<m>:EBTB:DY.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_DATA_CHARACTER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:SYMType
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR dataCharacter/Selects the data character (Dx.y) or control character to bea
/// HIPAR dataCharacter/triggered on. You can specify he value of the data character to be
/// HIPAR dataCharacter/triggered on with TRIGger<m>:EBTB:DX and TRIGger<m>:EBTB:DY.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerDataCharacter(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   dataCharacter)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_DATA_CHARACTER, dataCharacter),
            3, "Data Character");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets thex value of the data character Dx.y to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_DX_SYMBOL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:DX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR dxSymbol/Sets thex value of the data character Dx.y to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerDXSymbol(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   dxSymbol)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_DX_SYMBOL, dxSymbol),
            3, "DX Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the y value of the data character Dx.y to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_DY_SYMBOL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:DY
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR dySymbol/Sets the y value of the data character Dx.y to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerDYSymbol(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   dySymbol)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_DY_SYMBOL, dySymbol),
            3, "DY Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  COnfigures the EBTB trigger patterns.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_FORMAT
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_EB_PATTERN
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_TB_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:SYMFormat
/// HIFN     TRIGger<m>:EBTB:EBPA
/// HIFN     TRIGger<m>:EBTB:TBPA
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR symbolFormat/Selects the format of the symbol to be triggerd on, if
/// HIPAR symbolFormat/TRIGger<m>:EBTB:SSTYpe is set to SYMBol.
/// HIPAR ebPattern/Sets the 8-bit pattern to be triggered on, if
/// HIPAR ebPattern/TRIGger<m>:EBTB:SYMFormat is set to EB.
/// HIPAR tbPattern/Sets the 10-bit pattern to be triggered on, if
/// HIPAR tbPattern/TRIGger<m>:EBTB:SYMFormat is set to TB.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerPattern(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   symbolFormat,
                                                      ViString  ebPattern,
                                                      ViString  tbPattern)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_FORMAT, symbolFormat),
            3, "Symbol Format");

    switch (symbolFormat)
    {
    case RSSCOPE_VAL_EBTB_TRIGGER_SYMB_FORMAT_EB:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_EB_PATTERN, ebPattern),
                4, "EB Pattern");
        break;
    case RSSCOPE_VAL_EBTB_TRIGGER_SYMB_FORMAT_TB:

        viCheckParm(rsscope_SetAttributeViString(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_TB_PATTERN, tbPattern),
                5, "TB Pattern");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EBTB trigger symbol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_TYPE
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_EXPRESSION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:SSTYpe
/// HIFN     TRIGger<m>:EBTB:SYME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR type/Selects the symbol type to be triggered on. You can select a single
/// HIPAR type/symbol or an expression (defined series of symbols).
/// HIPAR expression/Selects the format of the symbol to be triggered on, if
/// HIPAR expression/TRIGger<m>:EBTB:SSTYpe is set to EXPRession.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerSymbol(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   type,
                                                     ViString  expression)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_TYPE, type),
            3, "Type");

    if (type == RSSCOPE_VAL_EBTB_SYMB_TYPE_EXPRESSION)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_SYMBOL_EXPRESSION, expression),
                4, "Expression");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EBTB trigger error conditions.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_DISPARITY_ERROR
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_GLITCH_ERROR
/// HIFN     RSSCOPE_ATTR_EBTB_TRIGGER_UNKNOWN_ERROR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:EBTB:DISParityerr
/// HIFN     TRIGger<m>:EBTB:GLITcherror
/// HIFN     TRIGger<m>:EBTB:UNK
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Reserved for future use
/// HIPAR disparityError/Defines, if a trigger on a disparity error is activated or not.
/// HIPAR glitchError/Defines, if a trigger on a glitch error is activated or not.
/// HIPAR unknownError/Defines, if a trigger on a unknown symbol error is activated or not.
ViStatus _VI_FUNC rsscope_ConfigureEBTBTriggerErrorConditions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean disparityError,
                            ViBoolean glitchError,
                            ViBoolean unknownError)
{
    ViStatus error = VI_SUCCESS;
             trigger;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_DISPARITY_ERROR, disparityError),
            3, "Disparity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_GLITCH_ERROR, glitchError),
            4, "Glitch Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "TrigA", RSSCOPE_ATTR_EBTB_TRIGGER_UNKNOWN_ERROR, unknownError),
            5, "Unknown Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the symbol count for the selected serial bus, i.e. the number
/// HIFN  of symbols in the present acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the symbol count for the selected serial bus, i.e. the number
/// HIPAR count/of symbols in the present acquisition.
ViStatus _VI_FUNC rsscope_QuerEBTBSymbolCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_EBTB_SYMBOL_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_EBTB_SYMBOL_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR symbol/This control selects the symbol.
/// HIPAR status/Returns the status of the specified symbol. It can show what kind of
/// HIPAR status/error was detected or no error.
/// HIPAR start/Returns the start time of the specified symbol.
/// HIPAR stop/Returns the stop time of the specified symbol.
ViStatus _VI_FUNC rsscope_QueryEBTBSymbolStatus(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   symbol,
                                                ViInt32*  status,
                                                ViReal64* start,
                                                ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:STAT?", serialBus, symbol);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, symbolStatus_Arr, status));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:STAR?", serialBus, symbol);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:STOP?", serialBus, symbol);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR symbol/This control selects the symbol.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryEBTBSymbolDataBytes(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   symbol,
                                                   ViInt32   arraySize,
                                                   ViChar    data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:DATA?", serialBus, symbol);
    checkErr(
        RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, cmd, 0, arraySize, data, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the byte value, 10-bit and K/D code of the specified symbol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:EBTB:SYMBol<n>:BYTE?
/// HIFN     BUS<m>:EBTB:SYMBol<n>:TENBit?
/// HIFN     BUS<m>:EBTB:SYMBol<n>:KDCode?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR symbol/This control selects the symbol.
/// HIPAR arraySize/Pass the number of values allocated for the result arrays.
/// HIPAR byteValue/Returns the symbol byte value.
/// HIPAR ten_bit/Returns the 10-bit symbol.
/// HIPAR k/dCode/Returns the K/D code.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryEBTBSymbolParameters(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   symbol,
                                                    ViInt32   arraySize,
                                                    ViChar    byteValue[],
                                                    ViChar    tenBit[],
                                                    ViChar    kdCode[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:BYTE?", serialBus, symbol);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, byteValue, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:TENB?", serialBus, symbol);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, tenBit, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:EBTB:SYMB%ld:KDC?", serialBus, symbol);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, kdCode, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the type of condition to be searched for. You
/// HIFN  can search for a certain symbol or enable a specific error condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the type of condition to be searched for. You can search for a
/// HIPAR type/certain symbol or enable a specific error condition.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the error search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR disparityError/Searches for glitching errors.
/// HIPAR glitchingError/Searches for glitching errors.
/// HIPAR unknownSymbolError/Defines, whether a search for any unknown symbol error shall be
/// HIPAR unknownSymbolError/activated or not.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchError(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViBoolean disparityError,
                                                   ViBoolean glitchingError,
                                                   ViBoolean unknownSymbolError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_DISPARITY_ERROR, disparityError),
            3, "Disparity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_GLITCHING_ERROR, glitchingError),
            4, "Glitching Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_UNKNOWN_SYMBOL_ERROR, unknownSymbolError),
            5, "Unknown Symbol Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the search type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the type of condition to be searched for. You can search for a
/// HIPAR type/certain symbol or enable a specific error condition.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchSearchType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_SEARCH_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the symbol type to be searched for. You can
/// HIFN  select a single symbol or an expression (defined series of symbols).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the symbol type to be searched for. You can select a single
/// HIPAR type/symbol or an expression (defined series of symbols).
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchSymbolType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_SYMBOL_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the format of the symbol to be searched for, if
/// HIFN  'Symbol Type' > 'Symbol'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR format/Selects the format of the symbol to be searched for, if 'Symbol Type'
/// HIPAR format/> 'Symbol'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchSymbolFormat(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_SYMBOL_FORMAT, format),
            3, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the data character (Dx.y) or control character
/// HIFN  to be searched for. You can specify the value of the data character to
/// HIFN  be searched for with 'Dx Value' and 'Dy Value'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR kdCodeSymbol/Selects the data character (Dx.y) or control character to be searched
/// HIPAR kdCodeSymbol/for. You can specify the value of the data character to be searched for
/// HIPAR kdCodeSymbol/with 'Dx Value' and 'Dy Value'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchKDCodeSymbol(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   kdCodeSymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_KD_CODE_SYMBOL, kdCodeSymbol),
            3, "KD Code Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the x and y value of the data character Dx.y to be
/// HIFN  searched for, if 'K/D Codes Symbol' > 'Dx.y'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dx/Sets the x value of the data character Dx.y to be searched for, if
/// HIPAR dx/'K/D Codes Symbol' > 'Dx.y'.
/// HIPAR dy/Sets the y value of the data character Dx.y to be searched for, if
/// HIPAR dy/'K/D Codes Symbol' > 'Dx.y'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchDXY(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   dx,
                                                 ViInt32   dy)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_DX, dx),
            3, "DX");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_DY, dy),
            4, "DY");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the 8-bit pattern to be searched for, if 'Format' >
/// HIFN  '8-bit'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR pattern/Sets the 8-bit pattern to be searched for, if 'Format' > '8-bit'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchEightBitPattern(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_EIGHT_BIT_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the 10-bit pattern to be searched for, if 'Format'
/// HIFN  > '10-bit'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR pattern/Sets the 10-bit pattern to be searched for, if 'Format' > '10-bit'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchTenBitPattern(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViString  pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_TEN_BIT_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the expression to be searched for, if 'Symbol
/// HIFN  Type' > 'Expression'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR expression/Defines the expression to be searched for, if 'Symbol Type' >
/// HIPAR expression/'Expression'.
ViStatus _VI_FUNC rsscope_ConfigureEBTBSearchExpression(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViString  expression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_EXPRESSION, expression),
            3, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the symbol count for the selected serial bus, i.e. the number
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the symbol count for the selected serial bus, i.e. the number
/// HIPAR count/of symbols in the present acquisition.
ViStatus _VI_FUNC rsscope_QueryEBTBSearchSymbolCount(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_EBTB_SYMBOL_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR symbol/This control selects the symbol.
/// HIPAR status/Returns the status of the specified symbol. It can show what kind of
/// HIPAR status/error was detected or no error.
/// HIPAR start/Returns the start time of the specified symbol.
/// HIPAR stop/Returns the stop time of the specified symbol.
ViStatus _VI_FUNC rsscope_QueryEBTBSearchSymbolStatus(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   symbol,
                                                      ViInt32*  status,
                                                      ViReal64* start,
                                                      ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52|K0"));
    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:EBTB:SYMB%ld:STAT? '%s'", symbol, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolEBTBSearchSymbolStatus_Arr, status));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:EBTB:SYMB%ld:STAR? '%s'", symbol, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:EBTB:SYMB%ld:STOP? '%s'", symbol, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR symbol/This control selects the symbol.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryEBTBSearchSymbolDataBytes(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   symbol,
                                                         ViInt32   arraySize,
                                                         ViChar    data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:EBTB:SYMB%ld:DATA? '%s'", symbol, searchName);
    checkErr(
        RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, cmd, 0, arraySize, data, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source of the data line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Defines the source of the signal.
ViStatus _VI_FUNC rsscope_ConfigureSENTDataSource(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_DATA_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Source");
    if ((RSSCOPE_VAL_PROTOCOL_WAV_R4 < source) && (source < RSSCOPE_VAL_PROTOCOL_WAV_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 3, "Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_DATA_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures threshold values used for digitization of the
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR technology/Selects the threshold voltage.
/// HIPAR threshold/Sets a user-defined threshold value.
ViStatus _VI_FUNC rsscope_ConfigureSENTThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   technology,
                                                 ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_TECHNOLOGY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TECHNOLOGY, technology),
            3, "Technology");
    if (technology == RSSCOPE_VAL_SENT_TECHNOLOGY_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_THRESHOLD, threshold),
                4, "Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the transmission settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR clockPeriod/Sets the nominal clock period (clock tick).
/// HIPAR clockTolerance/Specifies a tolerated deviation of the clock.
/// HIPAR dataNibles/Sets the number of data nibbles for a transmission sequence.
/// HIPAR serialFormat/Selects the serial message format.
/// HIPAR crcVersion/Selects the calculaction method for the cyclic redundancy check (CRC).
/// HIPAR crcCalculation/Selects the method for CRC calculation. SENT CRC calculates the
/// HIPAR crcCalculation/checksum over all nibbles except the communication and
/// HIPAR pausePulse/Determines wether a pause pulse is part of the SENT transmission
/// HIPAR pausePulse/sequence.
ViStatus _VI_FUNC rsscope_ConfigureSENTTransmission(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViReal64  clockPeriod,
                                                    ViReal64  clockTolerance,
                                                    ViInt32   dataNibbles,
                                                    ViInt32   serialFormat,
                                                    ViInt32   crcVersion,
                                                    ViInt32   crcCalculation,
                                                    ViInt32   pausePulse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_CLOCK_PERIOD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_CLOCK_PERIOD, clockPeriod),
            3, "Clock Period");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_CLOCK_TOLERANCE, clockTolerance),
            4, "Clock Tolerance");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_DATA_NIBBLES, dataNibbles),
            5, "Data Nibbles");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_SERIAL_FORMAT, serialFormat),
            6, "Serial Format");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_CRC_VERSION, crcVersion),
            7, "Crc Version");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_CRC_CALCULATION, crcCalculation),
            8, "Crc Calculation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_PAUSE_PULSE, pausePulse),
            9, "Pause Pulse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function determines the frame length in terms of ticks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameLength/Determines the frame length in terms of ticks. The dialog displays
/// HIPAR frameLength/this settings parameter, if the signal has a constant frame length.
ViStatus _VI_FUNC rsscope_ConfigureSENTFrameLength(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frameLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_FRAME_LENGTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_FRAME_LENGTH, frameLength),
            3, "Frame Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the results to be displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR resultDisplay/Selects the results to be displayed.
ViStatus _VI_FUNC rsscope_ConfigureSENTResultDisplay(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   resultDisplay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_RESULT_DISPLAY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_RESULT_DISPLAY, resultDisplay),
            3, "Result Display");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger event for the SENT transmission type.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the trigger event for the SENT transmission
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger event for the SENT transmission type.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerTransmissionSequenceType(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for transmission sequence trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR status/Sets the status nibble data, numeric or string pattern. The string
/// HIPAR status/parameter accepts the bit value X (don't care).
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerTransmissionSequence(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  status,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_STATUS,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_STATUS, status),
            3, "Status");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_DATA_MIN, minimum),
            5, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_TRANSMISSION_SEQUENCE_DATA_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger event for the serial message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the trigger event for the serial message.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerSerialMessageType(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for serial message trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the message ID format (4 bit or 8 bit) of the enhanced serial
/// HIPAR type/message.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a message identifier pattern, or sets the start value of an
/// HIPAR minimum/identifier range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerSerialMessageID(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   type,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_ID_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_ID_TYPE, type),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_ID_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_ID_MIN, minimum),
            5, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_ID_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for serial message data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to define a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerSerialMessageData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_DATA_MIN, minimum),
            4, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SENT_TRIGGER_SERIAL_MESSAGE_DATA_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR formError/Triggers on format errors in serial messages.
/// HIPAR calibrationPulseError/Triggers on calibration pulse erors in transmission sequences.
/// HIPAR pulsePeriodError/Triggers on pulse period errors.
/// HIPAR crcError/Triggers on CRC errors in both, the transmission sequences and serial
/// HIPAR crcError/messages.
/// HIPAR irregularFrameLengthError/Triggers on frame length errors in transmission sequences when pause
/// HIPAR irregularFrameLengthError/pulse for constant frame length is set.
ViStatus _VI_FUNC rsscope_ConfigureSENTTriggerErrorConditions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean formError,
                            ViBoolean calibrationPulseError,
                            ViBoolean pulsePeriodError,
                            ViBoolean crcError,
                            ViBoolean irregularFrameLengthError)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_FORM_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_FORM_ERROR, formError),
            3, "Form Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_CALIBRATION_PULSE_ERROR, calibrationPulseError),
            4, "Calibration Pulse Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_PULSE_PERIOD_ERROR, pulsePeriodError),
            5, "Pulse Period Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_CRC_ERROR, crcError),
            6, "crcError");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_TRIGGER_IRREGULAR_FRAME_LENGTH_ERROR, irregularFrameLengthError),
            7, "Irregular Frame Length Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded frames of the acquisition.
ViStatus _VI_FUNC rsscope_QuerySENTFrameCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SENT_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SENT_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of SENT message.
/// HIPAR syncDuration/Returns the time of the synchronization pulse.
/// HIPAR statusCommunicationValue/Returns the value of the status/communication pulse.
ViStatus _VI_FUNC rsscope_QuerySENTFrameStatus(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32*  frameStatus,
                                               ViReal64* frameStart,
                                               ViReal64* frameStop,
                                               ViInt32*  frameType,
                                               ViReal64* syncDuration,
                                               ViInt32*  statusCommunicationValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTFrameStatus_Arr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTFrameStatusType_Arr, frameType));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:SYNC?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, syncDuration));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:SCOM?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, statusCommunicationValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR idType/Returns the identifier type of the selected frame.
/// HIPAR idValue/Returns the identifier value of the selected frame.
ViStatus _VI_FUNC rsscope_QuerySENTFrameID(ViSession instrSession,
                                           ViInt32   serialBus,
                                           ViInt32   frame,
                                           ViInt32*  idType,
                                           ViInt32*  idValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:IDTY?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTFrameIDType_Arr, idType));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:IDV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, idValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the CRC sequence value of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR csValue/Returns the CRC sequence value of the selected frame.
ViStatus _VI_FUNC rsscope_QuerySENTFrameCSValue(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   frame,
                                                ViInt32*  csValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:CSV?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, csValue));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of the pulse pause clock ticks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR pulsePauseTicks/Returns the number of the pulse pause clock ticks.
ViStatus _VI_FUNC rsscope_QuerySENTFramePulsePauseTicks(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32*  pulsePauseTicks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:PAPT?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, pulsePauseTicks));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIFN  label list is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QuerySENTFrameLabel(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32   bufferSize,
                                              ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:SYMB?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the specified frame
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySENTFrameDataBytes(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32   arraySize,
                                                  ViInt32   data[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the complete symbolic data of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Result parameter.
/// HIPAR result/String with comma-separated list of symbolic data.
ViStatus _VI_FUNC rsscope_QuerySENTFrameAllSymbolicData(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32   bufferSize,
                                                        ViChar    result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAMe%ld:SDAT?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(result, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the complete symbolic data of the selected frame in export
/// HIFN  format, and saves the data to a file on the local PC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR fileName/
ViStatus _VI_FUNC rsscope_ExportSENTFrameAllSymbolicDataToFile(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViString  fileName,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViUInt32 oldTimeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (timeout > 0)
    {
        checkErr(rsscope_GetOPCTimeout(instrSession, &oldTimeout));
        viCheckParm(rsscope_SetOPCTimeout(instrSession, timeout), 5, "Timeout");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "FORM REAL,32;:BUS%ld:SENT:FRAM%ld:SDEX?", serialBus, frame);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, fileName, RS_VAL_TRUNCATE));

    strncpy(cmd, "FORM ASC", RS_MAX_MESSAGE_LEN);
    checkErr(rsscope_WriteInstrData(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (oldTimeout > 0)
        (void)rsscope_SetOPCTimeout(instrSession, oldTimeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data value of the specified nibble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR nibble/This control selects the nibble.
/// HIPAR data/Returns the value of the specified nibble.
/// HIPAR status/Returns the state of the specified nibble.
ViStatus _VI_FUNC rsscope_QuerySENTFrameNibbleData(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   nibble,
                                                   ViInt32*  data,
                                                   ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:NIBB%ld:VAL?", serialBus, frame, nibble);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SENT:FRAM%ld:NIBB%ld:STAT?", serialBus, frame, nibble);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTFieldStatus_Arr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the search for the Calibration/Synchronization
/// HIFN  pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR state/Enables the search for the Calibration/Synchronization pulse.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchCalibrationState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean state)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_CALIBRATION_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for a transmission sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR state/Enables the search for a transmission sequence.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchTransmissionSequenceState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean state)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects  the SENT transmission sequence to be searched
/// HIFN  for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the SENT transmission sequence to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchTransmissionSequenceType(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_TYPE, type),
            3, "type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for transmission sequence trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR status/Sets the status nibble data, numeric or string pattern. The string
/// HIPAR status/parameter accepts the bit value X (don't care).
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchTransmissionSequence(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  status,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_STATUS, status),
            3, "Status");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_DATA_MIN, minimum),
            5, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_TRANSMISSION_SEQUENCE_DATA_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search in a serial message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR state/Enables the search in a serial message.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchSerialMessageState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean state)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the serial message sequence to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the serial message sequence to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchSerialMessageType(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_TYPE, type),
            3, "type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for serial message search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the message ID format (4 bit or 8 bit) of the enhanced serial
/// HIPAR type/message.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a message identifier pattern, or sets the start value of an
/// HIPAR minimum/identifier range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchSerialMessageID(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   type,
                                                             ViInt32   condition,
                                                             ViString  minimum,
                                                             ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_ID_TYPE, type),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_ID_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_ID_MIN, minimum),
            5, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_ID_MAX, maximum),
                6, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for serial message data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to define a specific data pattern or a data pattern
/// HIPAR condition/range.
/// HIPAR minimum/Specifies a data pattern, or sets the start value of a data pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of an identifier range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchSerialMessageData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  minimum,
                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_DATA_MIN, minimum),
            4, "Minimum");
    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_SERIAL_MESSAGE_DATA_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables the search for a specified error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR state/Enables the search for a specified error.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchErrorConditionsState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean state)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_ERROR_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error Condition search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR formError/Enables the search for format errors in serial messages.
/// HIPAR calibrationPulseError/Enables the search for pulse errors.
/// HIPAR pulsePeriodError/Enables the search for pulse period errors.
/// HIPAR crcError/Enables the search for errors in the Cyclic Redundancy Check.
/// HIPAR irregularFrameLengthError/Enables the search for irregular frame length errors.
ViStatus _VI_FUNC rsscope_ConfigureSENTSearchErrorConditions(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViBoolean formError,
                                                             ViBoolean calibrationPulseError,
                                                             ViBoolean pulsePeriodError,
                                                             ViBoolean crcError,
                                                             ViBoolean irregularFrameLengthError)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_FORM_ERROR, formError),
            3, "Form Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_CALIBRATION_PULSE_ERROR, calibrationPulseError),
            4, "Calibration Pulse Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_PULSE_PERIOD_ERROR, pulsePeriodError),
            6, "Pulse Period Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_CRC_ERROR, crcError),
            7, "Crc Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_IRREGULAR_FRAME_LENGTH_ERROR, irregularFrameLengthError),
            8, "Irregular Frame Length Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of decoded frames of the acquisition.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameCount(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32*  count)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_SENT_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of SENT message.
/// HIPAR syncDuration/Returns the time of the synchronization pulse.
/// HIPAR statusCommunicationValue/Returns the value of the status/communication pulse.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameStatus(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32*  frameStatus,
                                                     ViReal64* frameStart,
                                                     ViReal64* frameStop,
                                                     ViInt32*  frameType,
                                                     ViReal64* syncDuration,
                                                     ViInt32*  statusCommunicationValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTSearchFrameStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTSearchFrameStatusType_Arr, frameType));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:SYNC? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, syncDuration));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:SCOM? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, statusCommunicationValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR idType/Returns the identifier type of the selected frame.
/// HIPAR idValue/Returns the identifier value of the selected frame.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameID(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   frame,
                                                 ViInt32*  idType,
                                                 ViInt32*  idValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:IDTY? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTSearchFrameID_Arr, idType));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:IDV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, idValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the CRC sequence value of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR csValue/Returns the CRC sequence value of the selected frame.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameCSValue(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   frame,
                                                      ViInt32*  csValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:CSV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, csValue));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of the pulse pause clock ticks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR pulsePauseTicks/Returns the number of the pulse pause clock ticks.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFramePulsePauseTicks(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32*  pulsePauseTicks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:PAPT? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, pulsePauseTicks));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the symbolic label of the specified frame if the
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/String with symbolic name of the identifier.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameLabel(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViInt32   bufferSize,
                                                    ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS:SENT:FRAM%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the specified frame
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameDataBytes(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32   arraySize,
                                                        ViInt32   data[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_MANCHESTER_NRZ_SELECTED_FRAME, frame),
            3, "Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the complete symbolic data of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Result parameter.
/// HIPAR result/String with comma-separated list of symbolic data.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameAllSymbolicData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAMe%ld:SYMB? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(result, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data value of the specified nibble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR nibble/This control selects the nibble.
/// HIPAR data/Returns the value of the specified nibble.
/// HIPAR status/Returns the state of the specified nibble.
ViStatus _VI_FUNC rsscope_QuerySENTSearchFrameNibbleData(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   nibble,
                                                         ViInt32*  data,
                                                         ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:NIBB%ld:VAL? '%s'", frame, nibble, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, data));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SENT:FRAM%ld:NIBB%ld:STAT? '%s'", frame, nibble, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSENTFrameNibbleStatus_Arr, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the protocol running on the interface. You can select between
/// HIFN  the Display Serial Interface (DSI) and Camera Serial Interface (CSI-2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signalType/Selects the protocol running on the interface. You can select between
/// HIPAR signalType/the Display Serial Interface (DSI) and Camera Serial Interface (CSI-2).
ViStatus _VI_FUNC rsscope_ConfigureDPHYVariant(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   variant)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_VARIANT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_VARIANT, variant),
            3, "Variant");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of data lanes and if the clock lane and low power are
/// HIFN  available.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_DPHY_PHYSICAL_CONFIGURATION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:DPHY:CONFig
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR physicalConfiguration/Sets the number of data lanes and if the clock lane and low power are
/// HIPAR physicalConfiguration/available.
ViStatus _VI_FUNC rsscope_ConfigureDPHYPhysicalConfiguration(ViSession instrSession,
                                                             ViInt32   serialBus,
                                                             ViInt32   physicalConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_PHYSICAL_CONFIGURATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_PHYSICAL_CONFIGURATION, physicalConfiguration),
            3, "Physical Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of data lanes. You can select up to four lanes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataLanes/Sets the number of data lanes. You can select up to four lanes.
ViStatus _VI_FUNC rsscope_ConfigureDPHYDataLanes(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   dataLanes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_DATA_LANES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DATA_LANES, dataLanes),
            3, "Data Lanes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the source of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR lane/This control selects DPHY lane.
/// HIPAR source/Defines the source of the signal.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSource(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   lane,
                                              ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_NONE, RSSCOPE_VAL_WAV_M8),
            4, "Source");
    if (source < RSSCOPE_VAL_WAV_M1)
    {
        switch (source)
        {
        case RSSCOPE_VAL_WAV_NONE:
        case RSSCOPE_VAL_WAV_C1W1:
        case RSSCOPE_VAL_WAV_C2W1:
        case RSSCOPE_VAL_WAV_C3W1:
        case RSSCOPE_VAL_WAV_C4W1:
            // Valid values < RSSCOPE_VAL_WAV_M1
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, source),
                        4, "Source");
            break;
        }
    }
    else if ((RSSCOPE_VAL_WAV_R4 < source) && (source < RSSCOPE_VAL_WAV_M6))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source),
                    4, "Source");
    }

    switch (lane)
    {
    case RSSCOPE_VAL_DPHY_LANE_CP:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DN0:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DN_ZERO_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP0:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_ZERO_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP1:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_ONE_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP2:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_TWO_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP3:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_THREE_SOURCE, source),
                4, "Source");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, lane), 3, "Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of probe used for the corresponding data lane.Selects
/// HIFN  the type of probe used for the corresponding lane.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR lane/This control selects DPHY lane.
/// HIPAR probe/Selects the type of probe used for the corresponding lane.
ViStatus _VI_FUNC rsscope_ConfigureDPHYProbe(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   lane,
                                             ViInt32   probe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_PROBE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (lane)
    {
    case RSSCOPE_VAL_DPHY_LANE_CP:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_PROBE, probe),
                4, "probe");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DN0:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DN_ZERO_PROBE, probe),
                4, "probe");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP0:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_ZERO_PROBE, probe),
                4, "probe");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP1:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_ONE_PROBE, probe),
                4, "probe");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP2:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_TWO_PROBE, probe),
                4, "probe");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP3:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DP_THREE_PROBE, probe),
                4, "probe");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, lane), 3, "Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a data rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataRate/Sets a data rate.
ViStatus _VI_FUNC rsscope_ConfigureDPHYDataRate(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViReal64  dataRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_DATA_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DATA_RATE, dataRate),
            3, "Data Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the display of the data vector, which means that the whole
/// HIFN  frame will be decoded.
/// HIFN     When the display vector is disabled only the packet header will be
/// HIFN  decoded and not the actual data payload of the frame. This will speed
/// HIFN  up the decoding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables the display of the data vector, which means that the whole
/// HIPAR state/frame will be decoded.
ViStatus _VI_FUNC rsscope_ConfigureDPHYDisplayDataVector(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_DISPALY_DATA_VECTOR,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_DISPALY_DATA_VECTOR, state),
            3, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function prests the threshold and hysteresis values of the high
/// HIFN  speed data lanes. A preset sets the low power threshold to 1.20V and
/// HIFN  high speed threshold to 200 mV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Prests the threshold and hysteresis values of the high speed data
/// HIPAR preset/lanes. A preset sets the low power threshold to 1.20V and high speed
/// HIPAR preset/threshold to 200 mV.
ViStatus _VI_FUNC rsscope_ConfigureDPHYThresholdPreset(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_THRESHOLD_PRESET, preset),
            3, "preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables coupling, i.e. the same threshold and hysteresis value is used
/// HIFN  for all lanes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables coupling, i.e. the same threshold and hysteresis value is used
/// HIPAR state/for all lanes.
ViStatus _VI_FUNC rsscope_ConfigureDPHYThresholdCoupling(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_THRESHOLD_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_THRESHOLD_COUPLING, state),
            3, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the DPHY high speed threshold and hysteresis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR lane/This control selects DPHY lane.
/// HIPAR threshold/Sets the threshold value for the digitization of the respective high
/// HIPAR threshold/speed line.
/// HIPAR hysteresis/Sets a value for the hysteresis of the respective lane.
ViStatus _VI_FUNC rsscope_ConfigureDPHYHighSpeedThreshold(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   lane,
                                                          ViReal64  threshold,
                                                          ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_PROBE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (lane)
    {
    case RSSCOPE_VAL_DPHY_LANE_CP:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_CP_THRESHOLD, threshold),
                4, "threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_CP_HYSTERESIS, hysteresis),
                5, "hysteresis");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP0:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_ZERO_THRESHOLD, threshold),
                4, "threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_ZERO_HYSTERESIS, hysteresis),
                5, "hysteresis");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP1:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_ONE_THRESHOLD, threshold),
                4, "threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_ONE_HYSTERESIS, hysteresis),
                5, "hysteresis");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP2:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_TWO_THRESHOLD, threshold),
                4, "threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_TWO_HYSTERESIS, hysteresis),
                5, "hysteresis");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP3:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_THREE_THRESHOLD, threshold),
                4, "threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_HIGH_SPEED_DP_THREE_HYSTERESIS, hysteresis),
                5, "hysteresis");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, lane), 3, "Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the low power lower/upper threshold.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DN_ZERO_THRESHOLD_LOWER
/// HIFN     RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DP_ZERO_THRESHOLD_LOWER
/// HIFN     RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DN_ZERO_THRESHOLD_UPPER
/// HIFN     RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DP_ZERO_THRESHOLD_UPPER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:DPHY:DNZero:LPOWer:THLower
/// HIFN     BUS<m>:DPHY:DPZero:LPOWer:THLower
/// HIFN     BUS<m>:DPHY:DNZero:LPOWer:THUPper
/// HIFN     BUS<m>:DPHY:DPZero:LPOWer:THUPper
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR lane/This control selects DPHY lane.
/// HIPAR thresholdLower/Sets the lower threshold value for the respective lane used for the
/// HIPAR thresholdLower/low power mode.
/// HIPAR thresholdUpper/Sets the upper threshold value for the respective lane used for the
/// HIPAR thresholdUpper/low power mode.
ViStatus _VI_FUNC rsscope_ConfigureDPHYLowPowerThreshold(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   lane,
                                                         ViReal64  thresholdLower,
                                                         ViReal64  thresholdUpper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_CP_PROBE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (lane)
    {
    case RSSCOPE_VAL_DPHY_LANE_DN0:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DN_ZERO_THRESHOLD_LOWER, thresholdLower),
                4, "threshold Lower");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DN_ZERO_THRESHOLD_UPPER, thresholdUpper),
                5, "threshold Upper");
        break;
    case RSSCOPE_VAL_DPHY_LANE_DP0:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DP_ZERO_THRESHOLD_LOWER, thresholdLower),
                4, "threshold Lower");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_LOW_POWER_DP_ZERO_THRESHOLD_UPPER, thresholdUpper),
                5, "threshold Upper");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, lane), 3, "Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the display of the results in multiple lanes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR showMultipleLanes/Enables the display of the results in multiple lanes.
ViStatus _VI_FUNC rsscope_ConfigureDPHYShowMultipleLanes(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean showMultipleLanes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_SHOW_MULTIPLE_LANES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_SHOW_MULTIPLE_LANES, showMultipleLanes),
            3, "Show Multiple Lanes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the type of frame to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DPHY_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the virtual channel to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR virtualChannel/Sets the virtual channel to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerVirtualChannel(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViString  virtualChannel)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DPHY_TRIGGER_VIRTUAL_CHANNEL,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_VIRTUAL_CHANNEL, virtualChannel),
            3, "Virtual Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the data type. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a data type pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerDataType(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the word count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the word count. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a word bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerWordCount(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DPHY_TRIGGER_WORD_COUNT_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_WORD_COUNT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_WORD_COUNT_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_WORD_COUNT_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the data type.
/// HIPAR condition/Set the condition for the data value. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies an escape mode data bit pattern, or sets the start value
/// HIPAR minimum/of a pattern range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerData(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type,
                                                   ViInt32   condition,
                                                   ViString  minimum,
                                                   ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;
    ViAttr   condAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_CONDITION,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_CONDITION
             };
    ViAttr   minAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_MIN,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_MIN
             };
    ViAttr   maxAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_MAX,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_MAX
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, condAttr[type],
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DPHY_HS_DATA, RSSCOPE_VAL_COND_LP_ESCAPE_MODE),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, condAttr[type], condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, minAttr[type], minimum),
            4, "Minimum");

    if ((condition == RSSCOPE_VAL_COND_IN) || (condition == RSSCOPE_VAL_COND_OUT))
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, maxAttr[type], maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the data type.
/// HIPAR condition/Sets the condition for the data index. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies a data index minimum, or sets the start value of a
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data index range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerDataIndex(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   type,
                                                        ViInt32   condition,
                                                        ViInt32   minimum,
                                                        ViInt32   maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;
    ViAttr   condAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_INDEX_CONDITION,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_INDEX_CONDITION
             };
    ViAttr   minAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_INDEX_MIN,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_INDEX_MIN
             };
    ViAttr   maxAttr[] = {
             RSSCOPE_ATTR_DPHY_TRIGGER_HIGH_SPEED_DATA_INDEX_MAX,
             RSSCOPE_ATTR_DPHY_TRIGGER_LOW_POWER_ESCAPE_MODE_DATA_INDEX_MAX
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DPHY_HS_DATA, RSSCOPE_VAL_COND_LP_ESCAPE_MODE),
            3, "Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, condAttr[type],
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, condAttr[type], condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, minAttr[type], minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, maxAttr[type], maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the escape mode.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_CONDITION
/// HIFN     RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_MIN
/// HIFN     RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:DPHY:ESCMode:CONDition
/// HIFN     TRIGger<m>:DPHY:ESCMode:MIN
/// HIFN     TRIGger<m>:DPHY:ESCMode:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the escape mode. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies a escape mode bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of an escape mode range for condition In Range
/// HIPAR maximum/or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYTriggerEscapeMode(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   condition,
                                                         ViString  minimum,
                                                         ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DPHY_TRIGGER_ESCAPE_MODE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded frames for the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of DPHY frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameStatus(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32*  frameStatus,
                                               ViReal64* frameStart,
                                               ViReal64* frameStop,
                                               ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDPHYStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDPHYType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the checksum of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR checksum/Returns the checksum of the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameChecksum(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32*  checksum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:CS?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, checksum));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the data type name for the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR name/Returns the data type name for the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameDataTypeName(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   bufferSize,
                                                     ViChar    name[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:CS?", serialBus, frame);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(name, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data type for the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR dataType/Returns the data type for the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameDataType(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32*  dataType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:DTYP?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, dataType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the error correction code (ECC) of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR errorCorrectionCode/Returns the error correction code (ECC) of the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameErrorCorrectionCode(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt32*  errorCorrectionCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:ECC?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, errorCorrectionCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of virtual channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR numberOfVirtualChannels/Returns the number of virtual channels.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameNumberOfVirtualChannels(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32*  numberOfVirtualChannels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:VCH?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfVirtualChannels));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameDataBytes(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32   arraySize,
                                                  ViInt32   data[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of packets in the indicated frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:DPHY:RESult:FRAMe<n>:NUMPackets?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR numberOfPackets/Returns the number of packets in the indicated frame.
ViStatus _VI_FUNC rsscope_QueryDPHYFrameNumberOfPackets(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32*  numberOfPackets)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:NUMP?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfPackets));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the payload data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR packet/This control selects the packet.
/// HIPAR index/Returns the index of the data payload.
/// HIPAR value/Returns the byte value of the data payload.
ViStatus _VI_FUNC rsscope_QueryDPHYFramePacketPayload(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViInt32   packet,
                                                      ViInt32*  index,
                                                      ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:PACK%ld:IDX?", serialBus, frame, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, index));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:DPHY:RES:FRAM%ld:PACK%ld:VAL?", serialBus, frame, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the type of frame to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the virtual channel to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR virtualChannel/Sets the virtual channel to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchVirtualChannel(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViString  virtualChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_VIRTUAL_CHANNEL, virtualChannel),
            3, "Virtual Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the data type. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a data type pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchDataType(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   condition,
                                                      ViString  minimum,
                                                      ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_TRIGGER_DATA_TYPE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the word count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the word count. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a word bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchWordCount(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_WORD_COUNT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_WORD_COUNT_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_WORD_COUNT_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the data type.
/// HIPAR condition/Set the condition for the data value. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies an escape mode data bit pattern, or sets the start value
/// HIPAR minimum/of a pattern range.
/// HIPAR maximum/Sets the end value of a data type range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   condAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_CONDITION,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_CONDITION
             };
    ViAttr   minAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_MIN,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_MIN
             };
    ViAttr   maxAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_MAX,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_MAX
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DPHY_HS_DATA, RSSCOPE_VAL_COND_LP_ESCAPE_MODE),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", condAttr[type], condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", minAttr[type], minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", maxAttr[type], maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the data index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Selects the data type.
/// HIPAR condition/Sets the condition for the data index. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies a data index minimum, or sets the start value of a
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of a data index range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchDataIndex(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   type,
                                                       ViInt32   condition,
                                                       ViInt32   minimum,
                                                       ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   condAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_INDEX_CONDITION,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_INDEX_CONDITION
             };
    ViAttr   minAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_INDEX_MIN,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_INDEX_MIN
             };
    ViAttr   maxAttr[] = {
             RSSCOPE_ATTR_DPHY_SEARCH_HIGH_SPEED_DATA_INDEX_MAX,
             RSSCOPE_ATTR_DPHY_SEARCH_LOW_POWER_ESCAPE_MODE_DATA_INDEX_MAX
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DPHY_HS_DATA, RSSCOPE_VAL_COND_LP_ESCAPE_MODE),
            3, "Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", condAttr[type], condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", minAttr[type], minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", maxAttr[type], maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the escape mode search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_CONDITION
/// HIFN     RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_MIN
/// HIFN     RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:DPHY:ESCMode:CONDition
/// HIFN     SEARch:TRIGger:DPHY:ESCMode:MIN
/// HIFN     SEARch:TRIGger:DPHY:ESCMode:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the escape mode. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies a escape mode bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of an escape mode range for condition In Range
/// HIPAR maximum/or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureDPHYSearchEscapeMode(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DPHY_SEARCH_ESCAPE_MODE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of decoded frames for the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameCount(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_DPHY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of DPHY frame.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameStatus(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32*  frameStatus,
                                                     ViReal64* frameStart,
                                                     ViReal64* frameStop,
                                                     ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDPHYStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDPHYType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the checksum of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR checksum/Returns the checksum of the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameChecksum(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViInt32*  checksum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:CS? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, checksum));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the data type name for the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR name/Returns the data type name for the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameDataTypeName(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32   bufferSize,
                                                           ViChar    name[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:DTN? '%s'", frame, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(name, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data type for the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dataType/Returns the data type for the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameDataType(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViInt32*  dataType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:DTYP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, dataType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the error correction code (ECC) of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR errorCorrectionCode/Returns the error correction code (ECC) of the specified frame.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameErrorCorrectionCode(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32*  errorCorrectionCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:ECC? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, errorCorrectionCode));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of virtual channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR numberOfVirtualChannels/Returns the number of virtual channels.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameNumberOfVirtualChannels(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32*  numberOfVirtualChannels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:VCH? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfVirtualChannels));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFrameDataBytes(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32   arraySize,
                                                        ViInt32   data[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the payload data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR index/Returns the index of the data payload.
/// HIPAR value/Returns the byte value of the data payload.
ViStatus _VI_FUNC rsscope_QueryDPHYSearchFramePacketPayload(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   frame,
                                                            ViInt32   packet,
                                                            ViInt32*  index,
                                                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:PACK%ld:IDX? '%s'", frame, packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, index));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DPHY:FRAM%ld:PACK%ld:VAL? '%s'", frame, packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the protocol running on the interface.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:VARiant
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR protocol/Selects the protocol running on the interface.
ViStatus _VI_FUNC rsscope_ConfigureMPHYProtocol(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   protocol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY, protocol),
            3, "Protocol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of data lanes. You can select up to four lanes.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_LANES
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:DLANes
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataLanes/Sets the number of data lanes. You can select up to four lanes.
ViStatus _VI_FUNC rsscope_ConfigureMPHYDataLanes(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   dataLanes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_LANES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_LANES, dataLanes),
            3, "Data Lanes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the source of the corresponding data lane.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataLane/Selects the Data Lane.
/// HIPAR dataSource/Selects the source of the corresponding data lane.
ViStatus _VI_FUNC rsscope_ConfigureMPHYDataSource(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   dataLane,
                                                  ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_SOURCE_ZERO,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataSource, RSSCOPE_VAL_MPHY_DATA_NONE, RSSCOPE_VAL_MPHY_DATA_M8),
            4, "Data Source");
    if (dataSource < RSSCOPE_VAL_MPHY_DATA_M1)
    {
        switch (dataSource)
        {
            // Invalid values
        case RSSCOPE_VAL_MPHY_DATA_C1W2:
        case RSSCOPE_VAL_MPHY_DATA_C1W3:
        case RSSCOPE_VAL_MPHY_DATA_C2W2:
        case RSSCOPE_VAL_MPHY_DATA_C2W3:
        case RSSCOPE_VAL_MPHY_DATA_C3W2:
        case RSSCOPE_VAL_MPHY_DATA_C3W3:
        case RSSCOPE_VAL_MPHY_DATA_C4W2:
        case RSSCOPE_VAL_MPHY_DATA_C4W3:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataSource), 4, "Data Source");
            break;
        default:
            break;
        }
    }
    else if ((RSSCOPE_VAL_MPHY_DATA_R4 < dataSource) && (dataSource < RSSCOPE_VAL_MPHY_DATA_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataSource), 4, "Data Source");
    }

    switch (dataLane)
    {
    case RSSCOPE_VAL_DLANE1:
        if (dataSource == RSSCOPE_VAL_MPHY_DATA_NONE)
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataSource), 4, "Data Source");
        }
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_SOURCE_ZERO, dataSource),
                4, "Data Source");
        break;
    case RSSCOPE_VAL_DLANE2:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_SOURCE_ONE, dataSource),
                4, "Data Source");
        break;
    case RSSCOPE_VAL_DLANE3:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_SOURCE_TWO, dataSource),
                4, "Data Source");
        break;
    case RSSCOPE_VAL_DLANE4:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_DATA_SOURCE_THREE, dataSource),
                4, "Data Source");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataLane), 3, "Data Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the threshold value for the respective lane.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_ONE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_TWO
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_THREE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_FOUR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:DZERo:THReshold
/// HIFN     BUS<m>:MPHY:DONE:THReshold
/// HIFN     BUS<m>:MPHY:DTWO:THReshold
/// HIFN     BUS<m>:MPHY:DTHRee:THReshold
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR dataLane/Selects the Data Lane.
/// HIPAR threshold/Sets the threshold value for the respective lane.
ViStatus _VI_FUNC rsscope_ConfigureMPHYThreshold(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   dataLane,
                                                 ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_ONE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (dataLane)
    {
    case RSSCOPE_VAL_DLANE1:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_ZERO, threshold),
                4, "Threshold");
        break;
    case RSSCOPE_VAL_DLANE2:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_ONE, threshold),
                4, "Threshold");
        break;
    case RSSCOPE_VAL_DLANE3:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_TWO, threshold),
                4, "Threshold");
        break;
    case RSSCOPE_VAL_DLANE4:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_THREE, threshold),
                4, "Threshold");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, dataLane), 3, "Data Lane");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the same threshold value to be used for all lanes.
/// HIFN     When the threshold coupling is disabled, changes to the threshold
/// HIFN  value only affect the respective lane.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_COUPLING_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:THCoupling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enabled/Enables the same threshold value to be used for all lanes.
ViStatus _VI_FUNC rsscope_ConfigureMPHYThresholdCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_COUPLING_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_COUPLING_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the predefined value to preset the threshold value of the data
/// HIFN  lanes.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_PRESET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:THPReset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Selects the predefined value to preset the threshold value of the data
/// HIPAR preset/lanes.
ViStatus _VI_FUNC rsscope_ConfigureMPHYThresholdPreset(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_THRESHOLD_PRESET, preset),
            3, "Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets if the SSIC data is scrambled or unscrambled.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_MPHY_SCRAMBLED_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:SCRMode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR scrambledMode/Sets if the SSIC data is scrambled or unscrambled.
ViStatus _VI_FUNC rsscope_ConfigureMPHYScrambledMode(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   scrambledMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_SCRAMBLED_MODE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_SCRAMBLED_MODE, scrambledMode),
            3, "Scrambled Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the display of the results in multiple lanes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR showMultipleLanes/Enables the display of the results in multiple lanes.
ViStatus _VI_FUNC rsscope_ConfigureMPHYShowMultipleLanes(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean showMultipleLanes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_SHOW_MULTIPLE_LANES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_SHOW_MULTIPLE_LANES, showMultipleLanes),
            2, "Show Multiple Lanes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the type of frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Selects the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of LCC frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_LCC_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:LCCType
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR lccType/Selects the type of LCC frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerLCCType(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   lccType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_LCC_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_LCC_TYPE, lccType),
            3, "LCC Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_MAX
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_MAX
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_MAX
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:LWONe:CONDition
/// HIFN     TRIGger<m>:MPHY:LWTWo:CONDition
/// HIFN     TRIGger<m>:MPHY:LWTHree:CONDition
/// HIFN     TRIGger<m>:MPHY:LWFour:CONDition
/// HIFN     TRIGger<m>:MPHY:LWONe:MIN
/// HIFN     TRIGger<m>:MPHY:LWTWo:MIN
/// HIFN     TRIGger<m>:MPHY:LWTHree:MIN
/// HIFN     TRIGger<m>:MPHY:LWFour:MIN
/// HIFN     TRIGger<m>:MPHY:LWONe:MAX
/// HIFN     TRIGger<m>:MPHY:LWTWo:MAX
/// HIFN     TRIGger<m>:MPHY:LWTHree:MAX
/// HIFN     TRIGger<m>:MPHY:LWFour:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR word/Selects a word.
/// HIPAR condition/Sets the condition for WORD. You can define an exact value or a value
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the WORD pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the WORD range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerWord(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   word,
                                                   ViInt32   condition,
                                                   ViString  minimum,
                                                   ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    switch (word)
    {
    case RSSCOPE_VAL_MPHY_WORD_ONE:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_ONE_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_TWO:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_TWO_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_THREE:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_THREE_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_FOUR:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_WORD_FOUR_MAX, maximum),
                    6, "Maximum");
        }
        break;

    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, word), 3, "Word");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the TC bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_TC_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:TC
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR tcPattern/Specifies the TC bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerTCPattern(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViString  tcPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_TC_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_TC_PATTERN, tcPattern),
            3, "TC Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the CREQ bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CREQ_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:CREQ
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR creqPattern/Specifies the CREQ bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerCREQPattern(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   CREQPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_CREQ_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CREQ_PATTERN, CREQPattern),
            3, "CREQ Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the RREQ bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_RREQ_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:RREQ
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR rreqPattern/Specifies the RREQ bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerRREQPattern(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   RREQPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_RREQ_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_RREQ_PATTERN, RREQPattern),
            3, "RREQ Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_DATA_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:DATA:DCON
/// HIFN     TRIGger<m>:MPHY:DATA:DMIN
/// HIFN     TRIGger<m>:MPHY:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the data bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Data range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerData(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   condition,
                                                   ViString  minimum,
                                                   ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_DATA_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_DATA_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:DATA:ICON
/// HIFN     TRIGger<m>:MPHY:DATA:IMIN
/// HIFN     TRIGger<m>:MPHY:DATA:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the Index. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the Index bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Index range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerIndex(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   condition,
                                                    ViInt32   minimum,
                                                    ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_EQ, RSSCOPE_VAL_COND_IN),
            3, "Condition");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_INDEX_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger FSN.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_FSN_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_FSN_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_FSN_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:FSNumber:CONDition
/// HIFN     TRIGger<m>:MPHY:FSNumber:MIN
/// HIFN     TRIGger<m>:MPHY:FSNumber:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the FSN. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the FSN bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the FSN range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerFSN(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_FSN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_FSN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_FSN_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_FSN_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CRC_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CRC_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CRC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:CRC:CONDition
/// HIFN     TRIGger<m>:MPHY:CRC:MIN
/// HIFN     TRIGger<m>:MPHY:CRC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the CRC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the CRC bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the CRC range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerCRC(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_CRC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CRC_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CRC_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Credit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:CREDit:CONDition
/// HIFN     TRIGger<m>:MPHY:CREDit:MIN
/// HIFN     TRIGger<m>:MPHY:CREDit:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the Credit. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the Credit bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Credit range for condition In Range or Out
/// HIPAR maximum/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerCredit(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   condition,
                                                     ViString  minimum,
                                                     ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_CREDIT_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger PACP Begin.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:PBEGin:CONDition
/// HIFN     TRIGger<m>:MPHY:PBEGin:MIN
/// HIFN     TRIGger<m>:MPHY:PBEGin:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the PACP Begin. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies the PACP Begin bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of the PACP Begin range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerPACPBegin(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_BEGIN_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger PACP ID.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_CONDITION
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_MIN
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:PFIC:CONDition
/// HIFN     TRIGger<m>:MPHY:PFIC:MIN
/// HIFN     TRIGger<m>:MPHY:PFIC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the PACP ID. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the PACP ID bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of the PACP ID range for condition In Range or Out
/// HIPAR maximum/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerPACPID(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   condition,
                                                     ViString  minimum,
                                                     ViString  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PACP_ID_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Trigger Errors.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_LCC_ERROR
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_LENGTH
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_RESERVED
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_SYMBOL
/// HIFN     RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_UNIDENTIFIED_UNIPRO_FRAME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:MPHY:ERRor:LCMD
/// HIFN     TRIGger<m>:MPHY:ERRor:LENGth
/// HIFN     TRIGger<m>:MPHY:ERRor:REServed
/// HIFN     TRIGger<m>:MPHY:ERRor:SYMBol
/// HIFN     TRIGger<m>:MPHY:ERRor:UNKNown
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR lcc/Enables or disables trigger on LCC error.
/// HIPAR length/Enables or disables trigger on length error.
/// HIPAR reserved/Enables or disables trigger on reserved bit error.
/// HIPAR symbol/Enables or disables trigger on 8b10b symbol error.
/// HIPAR uniproFrame/Enables or disables trigger on unidentified UniPro frame error.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerErrors(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViBoolean lcc,
                                                     ViBoolean length,
                                                     ViBoolean reserved,
                                                     ViBoolean symbol,
                                                     ViBoolean uniproFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_LCC_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_LCC_ERROR, lcc),
            3, "LCC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_LENGTH, length),
            4, "Length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_RESERVED, reserved),
            5, "Reserved");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_SYMBOL, symbol),
            6, "Symbol");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_ERROR_UNIDENTIFIED_UNIPRO_FRAME, uniproFrame),
            7, "UniPro Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the width for the width trigger based upon the data rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR width/Sets the width for the width trigger based upon the data rate.
ViStatus _VI_FUNC rsscope_ConfigureMPHYTriggerPRPWidth(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViReal64  width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MPHY_TRIGGER_PRP_WIDTH,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MPHY_TRIGGER_PRP_WIDTH, width),
            3, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the trigger on an error condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorName/Selects the error name.
/// HIPAR enabled/Enables or disables the trigger on an error condition.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:EREN '%s',%s", trigger, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the trigger on an error condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR errorNumber/Selects the error number.
/// HIPAR enabled/Enables or disables the trigger on an error condition.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   errorNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:ERR%ld:ENAB %s", trigger, errorNumber,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FREN '%s',%s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the checking condition for the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:ENAB %s", trigger, frame, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected field of
/// HIFN  the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for the selected field of
/// HIPAR enabled/the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FIEN '%s','%s',%s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for the selected field of
/// HIFN  the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the checking condition for the selected field of
/// HIPAR enabled/the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:ENAB %s", trigger, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNameBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:BIT '%s','%s',%s", trigger, frame, field,
             indicatorBits_Arr[bitState]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field that only consists of one bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR bitState/Sets the bit state of a field that only consists of one bit.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:BIT %s", trigger, frame, field,
             indicatorBits_Arr[bitState]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern in the selected field of the
/// HIPAR condition/selected frame.
/// HIPAR min/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNameData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:DOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:DMIN '%s','%s',%s", trigger, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:DMAX '%s','%s',%s", trigger, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the data pattern in the selected field of the
/// HIPAR condition/selected frame.
/// HIPAR min/Specifies the data pattern, or sets the start value of a data pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:DOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:DMIN %s", trigger, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:DMAX %s", trigger, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Specifies the index, or sets the start value of an index range.
/// HIPAR max/Sets the end value of an index range if the corresponding condition is
/// HIPAR max/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNameIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:IOP '%s','%s',%s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:IMIN '%s','%s',%ld", trigger, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:IMAX '%s','%s',%ld", trigger, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Specifies the index, or sets the start value of an index range.
/// HIPAR max/Sets the end value of an index range if the corresponding condition is
/// HIPAR max/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICTriggerFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:IOP %s", trigger, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:IMIN %ld", trigger, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:MPHY:SSIC:FRAM%ld:FLD%ld:IMAX %ld", trigger, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MPHY_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MPHY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:STATe?
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:STARt?
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:STOP?
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the state of the frame.
/// HIPAR frameStart/Returns the start time of the selected frame.
/// HIPAR frameStop/Returns the stop time of the selected frame.
/// HIPAR frameType/Returns the type of DPHY frame.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameStatus(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32*  frameStatus,
                                               ViReal64* frameStart,
                                               ViReal64* frameStop,
                                               ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDPHYStatus_Arr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMPHYType_Arr, frameType));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data for the selected frame, corresponds to the Data
/// HIFN  column in the decode results table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameDataBytes(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32   arraySize,
                                                  ViInt32   data[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the selected frame label. This label is on top of the frame as
/// HIFN  seen in the honeycomb display, also corresponds to the Info column in
/// HIFN  the decode results table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:INFO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR info/Returns the selected frame label. This label is on top of the frame as
/// HIPAR info/seen in the honeycomb display, also corresponds to the Info column in
/// HIPAR info/the decode results table.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameInfo(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   frame,
                                             ViInt32   arraySize,
                                             ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:INFO?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, arraySize, info));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded cells for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:CCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Returns the number of decoded cells for the selected frame.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameCellCount(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CCO?", serialBus, frame);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the overall state for the selected cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR frameStatus/Returns the state for the selected cell.
/// HIPAR frameStart/Returns the start time for the selected cell.
/// HIPAR frameStop/Returns the start time for the selected cell.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameCellStatus(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   cell,
                                                   ViInt32*  frameStatus,
                                                   ViReal64* frameStart,
                                                   ViReal64* frameStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CELL%ld:STAT?", serialBus, frame, cell);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMPHYFrameCellStateArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CELL%ld:STAR?", serialBus, frame, cell);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CELL%ld:STOP?", serialBus, frame, cell);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStop));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data type for the selected cell.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:CELL<o>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR type/Returns the data type for the selected cell.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameCellType(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32   cell,
                                                 ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CELL%ld:TYPE?", serialBus, frame, cell);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data value for the selected cell.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:MPHY:RESult:FRAMe<n>:CELL<o>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR data/Returns the data value for the selected cell.
ViStatus _VI_FUNC rsscope_QueryMPHYFrameCellData(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32   cell,
                                                 ViInt32*  data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:CELL%ld:DATA?", serialBus, frame, cell);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, data));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the type of frame for the selected SSIC frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR type/Returns the type of frame for the selected SSIC frame.
ViStatus _VI_FUNC rsscope_QueryMPHYSSICFrameType(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MPHY:RES:FRAM%ld:SSFT?", serialBus, frame);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the type of frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of LCC frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_LCC_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:LCCType
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lccType/Selects the type of LCC frame to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchLCCType(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   LCCType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_LCC_TYPE, LCCType),
            3, "LCC Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_MAX
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_MAX
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_MAX
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:LWONe:CONDition
/// HIFN     SEARch:TRIGger:MPHY:LWTWo:CONDition
/// HIFN     SEARch:TRIGger:MPHY:LWTHree:CONDition
/// HIFN     SEARch:TRIGger:MPHY:LWFour:CONDition
/// HIFN     SEARch:TRIGger:MPHY:LWONe:MIN
/// HIFN     SEARch:TRIGger:MPHY:LWTWo:MIN
/// HIFN     SEARch:TRIGger:MPHY:LWTHree:MIN
/// HIFN     SEARch:TRIGger:MPHY:LWFour:MIN
/// HIFN     SEARch:TRIGger:MPHY:LWONe:MAX
/// HIFN     SEARch:TRIGger:MPHY:LWTWo:MAX
/// HIFN     SEARch:TRIGger:MPHY:LWTHree:MAX
/// HIFN     SEARch:TRIGger:MPHY:LWFour:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR word/Selects a word.
/// HIPAR condition/Sets the condition for WORD. You can define an exact value or a value
/// HIPAR condition/range.
/// HIPAR minimum/Specifies the WORD pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the WORD range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchWord(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   word,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    switch (word)
    {
    case RSSCOPE_VAL_MPHY_WORD_ONE:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_ONE_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_TWO:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_TWO_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_THREE:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_THREE_MAX, maximum),
                    6, "Maximum");
        }
        break;
    case RSSCOPE_VAL_MPHY_WORD_FOUR:
        {
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_CONDITION, condition),
                    4, "Condition");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_MIN, minimum),
                    5, "Minimum");

            viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_WORD_FOUR_MAX, maximum),
                    6, "Maximum");
        }
        break;

    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, word), 3, "Word");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the TC bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_TC_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:TC
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tcPattern/Specifies the TC bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchTCPattern(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViString  TCPattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_TC_PATTERN, TCPattern),
            2, "TC Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the CREQ bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CREQ_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:CREQ
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR creqPattern/Specifies the CREQ bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchCREQPattern(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   CREQPattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CREQ_PATTERN, CREQPattern),
            3, "CREQPattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the RREQ bit pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_RREQ_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:RREQ
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rreqPattern/Specifies the RREQ bit pattern.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchRREQPattern(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   RREQPattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_RREQ_PATTERN, RREQPattern),
            3, "RREQPattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_DATA_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:DATA:DCON
/// HIFN     SEARch:TRIGger:MPHY:DATA:DMIN
/// HIFN     SEARch:TRIGger:MPHY:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the data bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Data range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   condition,
                                                  ViString  minimum,
                                                  ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_DATA_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_DATA_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:DATA:ICONdition
/// HIFN     SEARch:TRIGger:MPHY:DATA:IMIN
/// HIFN     SEARch:TRIGger:MPHY:DATA:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the Index. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the Index bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Index range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchIndex(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   condition,
                                                   ViInt32   minimum,
                                                   ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_INDEX_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_INDEX_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger FSN.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_FSN_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_FSN_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_FSN_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:FSNumber:CONDition
/// HIFN     SEARch:TRIGger:MPHY:FSNumber:MIN
/// HIFN     SEARch:TRIGger:MPHY:FSNumber:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the FSN. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the FSN bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the FSN range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchFSN(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   condition,
                                                 ViString  minimum,
                                                 ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_FSN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_FSN_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_FSN_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CRC_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CRC_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CRC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:CRC:CONDition
/// HIFN     SEARch:TRIGger:MPHY:CRC:MIN
/// HIFN     SEARch:TRIGger:MPHY:CRC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the CRC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the CRC bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the CRC range for condition In Range or Out of
/// HIPAR maximum/Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchCRC(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   condition,
                                                 ViString  minimum,
                                                 ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CRC_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CRC_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger Credit.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:CREDit:CONDition
/// HIFN     SEARch:TRIGger:MPHY:CREDit:MIN
/// HIFN     SEARch:TRIGger:MPHY:CREDit:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the Credit. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the Credit bit pattern, or sets the start value of a pattern
/// HIPAR minimum/range.
/// HIPAR maximum/Sets the end value of the Credit range for condition In Range or Out
/// HIPAR maximum/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchCredit(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   condition,
                                                    ViString  minimum,
                                                    ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_CREDIT_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger PACP Begin.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:PBEGin:CONDition
/// HIFN     SEARch:TRIGger:MPHY:PBEGin:MIN
/// HIFN     SEARch:TRIGger:MPHY:PBEGin:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the PACP Begin. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR minimum/Specifies the PACP Begin bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of the PACP Begin range for condition In Range or
/// HIPAR maximum/Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchPACPBegin(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_BEGIN_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the Trigger PACP ID.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:PFID:CONDition
/// HIFN     SEARch:TRIGger:MPHY:PFID:MIN
/// HIFN     SEARch:TRIGger:MPHY:PFID:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the condition for the PACP ID. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR minimum/Specifies the PACP ID bit pattern, or sets the start value of a
/// HIPAR minimum/pattern range.
/// HIPAR maximum/Sets the end value of the PACP ID range for condition In Range or Out
/// HIPAR maximum/of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchPACPID(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   condition,
                                                    ViString  minimum,
                                                    ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_PACP_ID_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Trigger Errors.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_LCC_ERROR
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_ERROR_LENGTH
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_ERROR_RESERVED
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_ERROR_SYMBOL
/// HIFN     RSSCOPE_ATTR_SEARCH_MPHY_ERROR_UNIDENTIFIED_UNIPRO_FRAME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:ERRor:LCMD
/// HIFN     SEARch:TRIGger:MPHY:ERRor:LENGth
/// HIFN     SEARch:TRIGger:MPHY:ERRor:REServed
/// HIFN     SEARch:TRIGger:MPHY:ERRor:SYMBol
/// HIFN     SEARch:TRIGger:MPHY:ERRor:UNKNown
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lcc/Enables or disables trigger on LCC error.
/// HIPAR length/Enables or disables trigger on length error.
/// HIPAR reserved/Enables or disables trigger on reserved bit error.
/// HIPAR symbol/Enables or disables trigger on 8b10b symbol error.
/// HIPAR uniproFrame/Enables or disables trigger on unidentified UniPro frame error.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSearchErrors(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViBoolean LCC,
                                                    ViBoolean length,
                                                    ViBoolean reserved,
                                                    ViBoolean symbol,
                                                    ViBoolean uniProFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_LCC_ERROR, LCC),
            3, "LCC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_ERROR_LENGTH, length),
            4, "Length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_ERROR_RESERVED, reserved),
            5, "Reserved");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_ERROR_SYMBOL, symbol),
            6, "Symbol");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MPHY_ERROR_UNIDENTIFIED_UNIPRO_FRAME, uniProFrame),
            7, "UniPro Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the MPHY Frame Count
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:MPHY:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of decoded frames for the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameCount(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SEARCH_DPHY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:STATe?
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:STARt?
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:STOP?
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of DPHY frame.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameStatus(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32*  frameStatus,
                                                     ViReal64* frameStart,
                                                     ViReal64* frameStop,
                                                     ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMPHYFrameCellStateArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:FTYP? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMPHYType_Arr, frameType));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameDataBytes(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32   arraySize,
                                                        ViInt32   data[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded cells for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:MPHY:FRAMe<m>:CCOUt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Returns the number of decoded cells for the selected frame.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameCellCount(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CCOU? '%s'", frame, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state within the search result for the selected
/// HIFN  cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR frameStatus/Returns the state type within the search result for the selected cell.
/// HIPAR frameStart/Returns the start time of the cell with the selected field number
/// HIPAR frameStart/within the frame with the selected frame number within the search
/// HIPAR frameStart/result.
/// HIPAR frameStop/Returns the stop time of the cell with the selected field number
/// HIPAR frameStop/within the frame with the selected frame number within the search
/// HIPAR frameStop/result.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameCellStatus(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   cell,
                                                         ViInt32*  frameStatus,
                                                         ViReal64* frameStart,
                                                         ViReal64* frameStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CELL%ld:STAT? '%s'", frame, cell, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMPHYFrameCellStateArr, frameStatus));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CELL%ld:STAR? '%s'", frame, cell, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CELL%ld:STOP? '%s'", frame, cell, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, frameStop));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data type for the selected cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR type/Returns the data type for the selected cell.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameCellType(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViInt32   cell,
                                                       ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CELL%ld:TYPE? '%s'", frame, cell, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, type));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data value for the selected cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR cell/This control selects the cell.
/// HIPAR data/Returns the data value for the selected cell.
ViStatus _VI_FUNC rsscope_QueryMPHYSearchFrameCellData(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViInt32   cell,
                                                       ViInt32*  data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MPHY:FRAM%ld:CELL%ld:DATA? '%s'", frame, cell, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, data));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the searching for an error condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/Selects the error name.
/// HIPAR enabled/Enables or disables the searching for an error condition.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:EREN '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the searching for an error condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorNumber/Selects the error name.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the searching for an error condition.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   errorNumber,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:ERR%ld:ENAB '%s',%s", errorNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame type to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame name.
/// HIPAR enabled/Defines the frame type to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FREN '%s','%s',%s", searchName, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame type to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:SSIC:FRAMe<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame name.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines the frame type to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:ENAB '%s',%s", frame, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FIEN '%s','%s','%s',%s", searchName, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:ENAB '%s',%s", frame, field, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR bitState/Sets the bit state of a field to be searched that only consists of one
/// HIPAR bitState/bit.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNameBitState(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:BIT '%s','%s','%s',%s", searchName, frame, field,
             indicatorBits_Arr[bitState]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the bit state of a field to be searched that only consists of one
/// HIFN  bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame name.
/// HIPAR field/Sets the field parameter.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bitState/Sets the bit state of a field to be searched that only consists of one
/// HIPAR bitState/bit.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNumericBitState(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   bitState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:BIT '%s',%s", frame, field, searchName,
             indicatorBits_Arr[bitState]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range if the corresponding
/// HIPAR max/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNameData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:DOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:DMIN '%s','%s','%s',%s", searchName, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:DMAX '%s','%s','%s',%s", searchName, frame, field,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range if the corresponding
/// HIPAR max/condition is set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:DOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:DMIN '%s',%s", frame, field, searchName,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:DMAX '%s',%s", frame, field,
                 searchName, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field name index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Specifies the index at which the data is to be searched, or sets the
/// HIPAR min/start value of an index range in which the data is to be searched.
/// HIPAR max/Sets the end value of an index range if the corresponding condition is
/// HIPAR max/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNameIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:IOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:IMIN '%s','%s','%s',%ld", searchName, frame, field,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:IMAX '%s','%s','%s',%ld", searchName, frame, field,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the frame field numeric index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:MPHY:SSIC:FRAMe<m>:FLD<n>:IOPerator
/// HIFN     SEARch:TRIGger:MPHY:SSIC:FRAMe<m>:FLD<n>:IMIN
/// HIFN     SEARch:TRIGger:MPHY:SSIC:FRAMe<m>:FLD<n>:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Specifies the index at which the data is to be searched, or sets the
/// HIPAR min/start value of an index range in which the data is to be searched.
/// HIPAR max/Sets the end value of an index range if the corresponding condition is
/// HIPAR max/set to In Range or Out Of Range.
ViStatus _VI_FUNC rsscope_ConfigureMPHYSSICSearchFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:IOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:IMIN '%s',%ld", frame, field, searchName,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:MPHY:SSIC:FRAM%ld:FLD%ld:IMAX '%s',%ld", frame, field,
                 searchName, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source of the space wire signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signal/This control selects DPHY lane.
/// HIPAR source/Defines the source of the signal.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSource(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   signal,
                                                   ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_STROBE_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (signal)
    {
    case RSSCOPE_VAL_SPACE_WIRE_SIGNAL_STROBE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_STROBE_SOURCE, source),
                4, "Source");
        break;
    case RSSCOPE_VAL_SPACE_WIRE_SIGNAL_DATA:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_DATA_SOURCE, source),
                4, "Source");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, signal), 3, "Signal");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function presets the threshold and hysteresis value of the strobe
/// HIFN  and data signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Prests the threshold and hysteresis value of the strobe and data
/// HIPAR preset/signal.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireThresholdPreset(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_THRESHOLD_PRESET, preset),
            3, "Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables coupling, i.e. the same threshold and hysteresis value is used
/// HIFN  for the strob and the data signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables coupling, i.e. the same threshold and hysteresis value is used
/// HIPAR state/for the strob and the data signal.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireThresholdCoupling(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_THRESHOLD_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_THRESHOLD_COUPLING, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the space wire high speed threshold and hysteresis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signal/This control selects DPHY lane.
/// HIPAR threshold/Sets the threshold value for the digitization of the data signal.
/// HIPAR hysteresis/Sets a value for the hysteresis of the data signal.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireThreshold(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   signal,
                                                      ViReal64  threshold,
                                                      ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_STROBE_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (signal)
    {
    case RSSCOPE_VAL_SPACE_WIRE_SIGNAL_STROBE:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_STROBE_THRESHOLD, threshold),
                4, "Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_STROBE_HYSTERESIS, hysteresis),
                5, "Hysteresis");
        break;
    case RSSCOPE_VAL_SPACE_WIRE_SIGNAL_DATA:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_DATA_THRESHOLD, threshold),
                4, "Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_DATA_HYSTERESIS, hysteresis),
                5, "Hysteresis");
        break;
    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, signal), 3, "Signal");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the mode for the synchronisation of the signal. In
/// HIFN  the auto mode the decoder will automatically do the packet align. In
/// HIFN  the manual mode you can set the align point man
/// HIFN     ually with the "Bit Position" setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the mode for the synchronisation settings, i.e i.e. packet
/// HIPAR mode/align.
/// HIPAR bitPosition/Sets the bit position, the align position for the manual
/// HIPAR bitPosition/synchronisation mode. This can be useful when parity errors exist in
/// HIPAR bitPosition/the signal, and parity check is the main indicator for the decoder to
/// HIPAR bitPosition/do packet alignment. Use this setting only when Mode is set to Manual.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSyncSettings(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   mode,
                                                         ViInt32   bitPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_SYNC_SETTINGS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_SYNC_SETTINGS, mode),
            3, "mode");

    if (mode == RSSCOPE_VAL_MANUAL)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_SYNC_BIT_POSITION, bitPosition),
                4, "bitPosition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the minimum duration of a gap. Any inactivity
/// HIFN  greater than this time will be interpreted as a gap and lead to a
/// HIFN  resynchronization to the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR minGap/Sets the minimum duration of a gap. Any inactivity greater than this
/// HIPAR minGap/time will be interpreted as a gap and lead to a resynchronization to
/// HIPAR minGap/the signal.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireMinGap(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViReal64  minGap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_MIN_GAP,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_SPACE_WIRE_MIN_GAP, minGap),
            3, "minGap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the trigger type for the Space Wire analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the trigger type for the Space Wire analysis.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireTriggerType(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Triggers on a specific control type character.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Triggers on a specific control type character.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireTriggerControlType(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_CONTROL_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_CONTROL_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the space wire time code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the data value for the time code. You can define
/// HIPAR condition/an exact value or a value range.
/// HIPAR minimum/Specifies a pattern for the data value for the time code, or sets the
/// HIPAR minimum/the start value of a pattern range.
/// HIPAR maximum/Sets the end value of a data value for the time code range for
/// HIPAR maximum/condition In Range or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireTriggerTimeCode(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   condition,
                                                            ViString  minimum,
                                                            ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TIME_CODE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TIME_CODE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TIME_CODE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_TIME_CODE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the space wire data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the data value. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a data for the data pattern, or sets the start value of
/// HIPAR minimum/a pattern range.
/// HIPAR maximum/Sets the end value of a data for the data pattern range for
/// HIPAR maximum/condition In Range or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireTriggerData(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  minimum,
                                                        ViString  maximum)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_DATA_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_DATA_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR parityError/Enables triggering on a parity error (even parity).
/// HIPAR escapeError/Enables trigger on an escape error.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireTriggerErrorConditions(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean parityError,
                            ViBoolean escapeError)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_PARITY_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_PARITY_ERROR, parityError),
            3, "Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPACE_WIRE_TRIGGER_ESCAPE_ERROR, escapeError),
            4, "Escape Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded frames for the selected serial bus.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameCount(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32*  count)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DPHY_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DPHY_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of space wire frame.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameStatus(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32*  frameStatus,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameDataBytes(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt32   arraySize,
                                                       ViInt32   data[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65|K0"));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the field with the selected frame number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:STATus?
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:STARt?
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:STOP?
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of space wire frame.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameFieldStatus(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   frame,
                                                         ViInt32   field,
                                                         ViInt32*  frameStatus,
                                                         ViReal64* frameStart,
                                                         ViReal64* frameStop,
                                                         ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:STAT?", serialBus, frame, field);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:STAR?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:STOP?", serialBus, frame, field);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:TYPE?", serialBus, frame, field);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the value of the field with the selected field number within
/// HIFN  the frame with the selected frame number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:VAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameFieldDataBytes(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt32   field,
                                                            ViInt32   arraySize,
                                                            ViInt32   data[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65|K0"));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:VAL?", serialBus, frame, field);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected frame number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:SWIRe:RESults:FRAMe<n>:FLD<o>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field.
/// HIPAR bufferSize/Pass the number of values allocated for Data result array.
/// HIPAR formatedValue/Returns the formatted value of the field with the selected field
/// HIPAR formatedValue/number within the frame with the selected frame number.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySpaceWireFrameFieldFormattedValue(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   field,
                            ViInt32   bufferSize,
                            ViChar    formatedValue[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:SWIR:RES:FRAM%ld:FLD%ld:FVAL?", serialBus, frame, field);
    checkErr(
        RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, cmd, 0, bufferSize, formatedValue,
            returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the search type for the Space Wire analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the search type for the Space Wire analysis.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSearchType(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Searches for a specific control type character.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Searches for a specific control type character.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSearchControlType(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_CONTROL_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the space wire time code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the data value for the time code. You can define
/// HIPAR condition/an exact value or a value range.
/// HIPAR minimum/Specifies a pattern for the data value for the time code, or sets the
/// HIPAR minimum/the start value of a pattern range.
/// HIPAR maximum/Sets the end value of a data value for the time code range for
/// HIPAR maximum/condition In Range or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSearchTimeCode(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   condition,
                                                           ViString  minimum,
                                                           ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_TIME_CODE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_TIME_CODE_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_TIME_CODE_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for the space wire data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the data value. You can define an exact value or
/// HIPAR condition/a value range.
/// HIPAR minimum/Specifies a data for the data pattern, or sets the start value of
/// HIPAR minimum/a pattern range.
/// HIPAR maximum/Sets the end value of a data for the data pattern range for
/// HIPAR maximum/condition In Range or Out of Range.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSearchData(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  minimum,
                                                       ViString  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_DATA_MIN, minimum),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_DATA_MAX, maximum),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR parityError/Enables triggering on a parity error (even parity).
/// HIPAR escapeError/Enables trigger on an escape error.
ViStatus _VI_FUNC rsscope_ConfigureSpaceWireSearchErrorConditions(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean parityError,
                            ViBoolean escapeError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_PARITY_ERROR, parityError),
            3, "Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SPACE_WIRE_SEARCH_PARITY_ERROR, escapeError),
            4, "Escape Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of frames within the search result for the selected
/// HIPAR count/serial bus.
ViStatus _VI_FUNC rsscope_QuerySpaceWireSearchFrameCount(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPACE_WIRE_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of space wire frame.
ViStatus _VI_FUNC rsscope_QuerySpaceWireSearchFrameStatus(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32*  frameStatus,
                                                          ViReal64* frameStart,
                                                          ViReal64* frameStop,
                                                          ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SWIR:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SWIR:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SWIR:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SWIR:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QuerySpaceWireSearchFrameDataBytes(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   frame,
                                                             ViInt32   arraySize,
                                                             ViInt32   data[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K65|K0"));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SWIR:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PCIe.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_GENERATION
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_LINK
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_DESCRAMBLING
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:GEN
/// HIFN     BUS<m>:PCIE:LNKW
/// HIFN     BUS<m>:PCIE:DSCRambling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR generation/Selects the generation of the PCIe technology.
/// HIPAR link/Selects the link width, the number of lanes that are used for the
/// HIPAR link/transmission of the data.
/// HIPAR descrambling/Enables descrambling of the data.
ViStatus _VI_FUNC rsscope_ConfigurePCIe(ViSession instrSession,
                                        ViInt32   serialBus,
                                        ViInt32   generation,
                                        ViInt32   link,
                                        ViBoolean descrambling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_GENERATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_GENERATION, generation),
            3, "Generation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_LINK, link),
            4, "Link");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_DESCRAMBLING, descrambling),
            5, "Descrambling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the filtering of idle packets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR filterIdlePackets/Enables the filtering of idle packets.
ViStatus _VI_FUNC rsscope_ConfigurePCIeFilterIdle(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViBoolean filterIdlePackets)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_FILTER_IDLE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_FILTER_IDLE, filterIdlePackets),
            3, "Filter Idle Packets");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the signal for selected Lane.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Sets the signal sources for selected Lane.
/// HIPAR thresholdLow/Sets the lower threshold value for the respective lane.
/// HIPAR thresholdHigh/Sets the higher threshold value for the respective lane.
ViStatus _VI_FUNC rsscope_ConfigurePCIeLane(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   lane,
                                            ViInt32   source,
                                            ViReal64  thresholdLow,
                                            ViReal64  thresholdHigh)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrPCIeSourceAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_PCIE_SOURCE_LANE_ZERO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_SOURCE_LANE_ONE,
             RSSCOPE_ATTR_PROTOCOL_PCIE_SOURCE_LANE_TWO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_SOURCE_LANE_THREE
             };
    ViAttr   arrPCIeThresholdLowAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_LOW_LANE_ZERO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_LOW_LANE_ONE,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_LOW_LANE_TWO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_LOW_LANE_THREE
             };
    ViAttr   arrPCIeThresholdHighAttr[] = {
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_HIGH_LANE_ZERO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_HIGH_LANE_ONE,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_HIGH_LANE_TWO,
             RSSCOPE_ATTR_PROTOCOL_PCIE_THRESHOLD_HIGH_LANE_THREE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lane, 0, 3),
            3, "Lane");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_PROTOCOL_WAV_NONE, RSSCOPE_VAL_PROTOCOL_WAV_M8),
            4, "Source");
    if ((lane == 0) && (source == RSSCOPE_VAL_PROTOCOL_WAV_NONE))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 4, "Waveform Source");
    }
    else if ((RSSCOPE_VAL_PROTOCOL_WAV_R4 < source) && (source < RSSCOPE_VAL_PROTOCOL_WAV_M5))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 4, "Source");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrPCIeSourceAttr[lane],
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, arrPCIeSourceAttr[lane], source),
            4, "Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, arrPCIeThresholdLowAttr[lane], thresholdLow),
            5, "Threshold Low");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, arrPCIeThresholdHighAttr[lane], thresholdHigh),
            6, "Threshold High");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CDR settings.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_NOMINAL_BITRATE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_INITIAL_PHASE_SYNC
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_SELECTED_RESULTS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CDR:BITRate
/// HIFN     BUS<m>:CDR:SYNC
/// HIFN     BUS<m>:CDR:RESults
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR nominalBitRate/Sets the quiescent frequency of the PLL. It corresponds to the data
/// HIPAR nominalBitRate/rate of the data stream from which the clock is to be recovered.
/// HIPAR initialPhaseSync/Defines the phase reference for the first clock edge.
/// HIPAR selectedResults/The PLL requires some time to synchronize to the phase of the data
/// HIPAR selectedResults/stream. You can select when the CDR algorithm returns clock edges.
ViStatus _VI_FUNC rsscope_ConfigurePCIeCDR(ViSession instrSession,
                                           ViInt32   serialBus,
                                           ViInt64   nominalBitRate,
                                           ViInt32   initialPhaseSync,
                                           ViInt32   selectedResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_NOMINAL_BITRATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_NOMINAL_BITRATE,
                    nominalBitRate), 3, "Nominal Bit Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_INITIAL_PHASE_SYNC, initialPhaseSync),
            4, "Initial Phase Sync");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_SELECTED_RESULTS, selectedResults),
            5, "Selected Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CDR PPL settings.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_ORDER
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_DAMPING
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_BANDWIDTH
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_RELATIVE_BANDWIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CDR:PLL:ORDer
/// HIFN     BUS<m>:CDR:PLL:DAMPing
/// HIFN     BUS<m>:CDR:PLL:BWIDth
/// HIFN     BUS<m>:CDR:PLL:RELBwidth
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR order/Sets the order of the PLL: first or second order. PLL of higher order
/// HIPAR order/can compensate for more complex jitter behavior.
/// HIPAR damping/Sets the damping factor, which is only relevant for second order PLL.
/// HIPAR bandwidth/Sets the PLL bandwidth. It defines the part of the spectrum that the
/// HIPAR bandwidth/PLL can follow during synchronization. The PLL bandwidth is usually
/// HIPAR bandwidth/defined by the transmission standard.
/// HIPAR relativeBandwidth/Sets the PLL bandwidth. It defines the part of the spectrum that the
/// HIPAR relativeBandwidth/PLL can follow during synchronization. The PLL bandwidth is usually
/// HIPAR relativeBandwidth/defined by the transmission standard.
ViStatus _VI_FUNC rsscope_ConfigurePCIeCDRPLL(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   order,
                                              ViReal64  damping,
                                              ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_ORDER,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_ORDER, order),
            3, "Order");
    if (order == RSSCOPE_VAL_PCIE_PLL_CDR_ORDER_SECOND)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_DAMPING, damping),
                4, "Damping");
    }
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_BANDWIDTH, bandwidth),
            5, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the CDR PPL relative bandwidth.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_RELATIVE_BANDWIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CDR:PLL:RELBwidth
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR relativeBandwidth/Sets the PLL bandwidth. It defines the part of the spectrum that the
/// HIPAR relativeBandwidth/PLL can follow during synchronization. The PLL bandwidth is usually
/// HIPAR relativeBandwidth/defined by the transmission standard.
ViStatus _VI_FUNC rsscope_ConfigurePCIeCDRPLLRelativeBandwidth(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViReal64  relativeBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_RELATIVE_BANDWIDTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_CDR_PLL_RELATIVE_BANDWIDTH, relativeBandwidth),
            3, "Relative Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the display of the result in multiple lanes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR showMultipleLanes/Enables the display of the result in multiple lanes.
ViStatus _VI_FUNC rsscope_ConfigurePCIeShowMultipleLanes(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViBoolean showMultipleLanes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_SHOW_MULTIPLE_LANES,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_SHOW_MULTIPLE_LANES, showMultipleLanes),
            2, "Show Multiple Lanes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger type for the PCI express analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Sets the trigger type for the PCI express analysis.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger level for PCIe Gen 3.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR level/Sets the trigger level for PCIe Gen 3.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerLevel(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrigger = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &maxTrigger);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, maxTrigger),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_LEVEL,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the ordered set to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR orderedsetcondition/Sets the ordered set to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerOSET(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   orderedsetcondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_OSET_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_OSET_TYPE, orderedsetcondition),
            3, "Ordered set condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PCIe trigger error.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_CRC16
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_LCRC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_ECRC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_INVALID_SYMBOL
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_DISP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:ERRC:CRC
/// HIFN     TRIGger<m>:PCIE:ERRC:LCRC
/// HIFN     TRIGger<m>:PCIE:ERRC:ECRC
/// HIFN     TRIGger<m>:PCIE:ERRC:INVP
/// HIFN     TRIGger<m>:PCIE:ERRC:DISP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR crc16/Enables triggering on 16-bit cyclic redundancy check (CRC) errors.
/// HIPAR lcrc/Enables triggering on link cyclic redundancy check (LCRC) errors.
/// HIPAR ecrc/Enables triggering on end-to-end cyclic redundancy checksum (ECRC)
/// HIPAR ecrc/errors.
/// HIPAR invalidSymbol/Enables triggering on invalid symbol errors.
/// HIPAR disparityError/Enables triggering on disparity errors.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerError(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViBoolean crc16,
                                                    ViBoolean lcrc,
                                                    ViBoolean ecrc,
                                                    ViBoolean invalidSymbol,
                                                    ViBoolean disparityError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_CRC16,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_CRC16, crc16),
            3, "CRC 16");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_LCRC, lcrc),
            4, "L CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_ECRC, ecrc),
            5, "E CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_INVALID_SYMBOL, invalidSymbol),
            6, "Invalid Symbol");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_DISP, disparityError),
            7, "Disparity Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables triggering for errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects repeated capability Trigger.
/// HIPAR length/Enables triggering on a length error, for frames that do not have the
/// HIPAR length/specified length or data.
/// HIPAR malformError/Enables triggering on a malformed TLP: a TLP that violates specific
/// HIPAR malformError/TLP formation rules that are defined in the specification.
/// HIPAR maxEndPrefix/Enables triggering on an end-end prefix error. The maximum number of
/// HIPAR maxEndPrefix/permitted end-end TLP prefixes is 4.
/// HIPAR prefixOrderError/Enables triggering on a wrong prefix order.
/// HIPAR syncBit/Enables triggering on a sync bit.
/// HIPAR valueOutOfRange/Ennables triggering, if a value of a field is out of the range defined
/// HIPAR valueOutOfRange/by the PCIe standard.
/// HIPAR unknownPacket/Enables triggering on an unknown packet.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerErrorCondition(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViBoolean length,
                                                             ViBoolean malformError,
                                                             ViBoolean maxEndPrefix,
                                                             ViBoolean prefixOrderError,
                                                             ViBoolean syncBit,
                                                             ViBoolean valueOutOfRange,
                                                             ViBoolean unknownPacket)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrigger = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &maxTrigger);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, maxTrigger),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_LENGTH,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_LENGTH, length),
            3, "Length Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_MALFORM_ERROR, malformError),
            4, "Malformed Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_MAX_END_PREFIX, maxEndPrefix),
            5, "Max End Prefix");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_PREFIX_ORDER_ERROR, prefixOrderError),
            6, "Prefix Order Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_SYNC_BIT, syncBit),
            7, "Sync Bit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_VALUE_OUT_OF_RANGE, valueOutOfRange),
            8, "Value Out Of Range");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_ERRC_UNKNOWN_PACKET, unknownPacket),
            9, "Unknown Packet");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the transaction type to be triggered on for the transaction layer
/// HIFN  type trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR tlp/Sets the transaction type to be triggered on for the transaction layer
/// HIPAR tlp/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPType(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   tlp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP, tlp),
            3, "TLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP any type trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPAnyType(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViString  sequenceNumber,
                                                         ViInt32   tc,
                                                         ViInt32   ordering,
                                                         ViInt32   snoop,
                                                         ViString  requesterID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP mem trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:MERW
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:ADRT
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR readWrite/Sets the read/ write type to be triggered on for the transaction layer
/// HIPAR readWrite/type trigger.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR addressType/Sets the address type to be triggered on for the transaction layer
/// HIPAR addressType/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPMem(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViInt32   readWrite,
                                                     ViString  requesterID,
                                                     ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE, readWrite),
            7, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            8, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE, addressType),
            9, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP IO trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:MERW
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR readWrite/Sets the read/ write type to be triggered on for the transaction layer
/// HIPAR readWrite/type trigger.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPIO(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViInt32   readWrite,
                                                    ViString  requesterID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE, readWrite),
            7, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            8, "Requester ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP Cfg trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_DEVICE_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:MERW
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:CFGT
/// HIFN     TRIGger<m>:PCIE:TLP:DEID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR readWrite/Sets the read/ write type to be triggered on for the transaction layer
/// HIPAR readWrite/type trigger.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR type/Triggers on a configuration type.
/// HIPAR deviceID/Sets the device ID to be triggered on for the transaction layer type
/// HIPAR deviceID/trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPCfg(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViInt32   readWrite,
                                                     ViString  requesterID,
                                                     ViInt32   type,
                                                     ViString  deviceID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_READ_WRITE, readWrite),
            7, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            8, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TYPE, type),
            9, "Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_DEVICE_ID, deviceID),
            10, "Device ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP msg trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ROUTING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_MESSAGE_CODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:MSGR
/// HIFN     TRIGger<m>:PCIE:TLP:MSGC
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR routing/Sets the message routing type to be triggered on for the transaction
/// HIPAR routing/layer type trigger.
/// HIPAR messageCode/Sets the message code to be triggered on for the transaction layer
/// HIPAR messageCode/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPMsg(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViString  requesterID,
                                                     ViInt32   routing,
                                                     ViString  messageCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ROUTING, routing),
            8, "Routing");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_MESSAGE_CODE, messageCode),
            9, "Message Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP Cpl trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_STATUS
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_COMPLETER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:CPLS
/// HIFN     TRIGger<m>:PCIE:TLP:CPID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR status/Sets the completion status to be triggered on for the transaction
/// HIPAR status/layer type trigger.
/// HIPAR completerID/Sets the completer ID to be triggered on for the transaction layer
/// HIPAR completerID/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPCpl(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViString  requesterID,
                                                     ViInt32   status,
                                                     ViString  completerID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_STATUS, status),
            8, "Status");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_COMPLETER_ID, completerID),
            9, "Completer ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP fetch add trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:ADRT
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR addressType/Sets the address type to be triggered on for the transaction layer
/// HIPAR addressType/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPFetchAdd(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViString  sequenceNumber,
                                                          ViInt32   tc,
                                                          ViInt32   ordering,
                                                          ViInt32   snoop,
                                                          ViString  requesterID,
                                                          ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP swap trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:ADRT
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR addressType/Sets the address type to be triggered on for the transaction layer
/// HIPAR addressType/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPSwap(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViString  sequenceNumber,
                                                      ViInt32   tc,
                                                      ViInt32   ordering,
                                                      ViInt32   snoop,
                                                      ViString  requesterID,
                                                      ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP CAS trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:TLP:SNUM
/// HIFN     TRIGger<m>:PCIE:TLP:TCHN
/// HIFN     TRIGger<m>:PCIE:TLP:ORDE
/// HIFN     TRIGger<m>:PCIE:TLP:SNOO
/// HIFN     TRIGger<m>:PCIE:TLP:REID
/// HIFN     TRIGger<m>:PCIE:TLP:ADRT
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sequenceNumber/Sets the sequence number to be triggered on for the transaction layer
/// HIPAR sequenceNumber/type trigger.
/// HIPAR tc/Sets the traffic class to be triggered on for the transaction layer
/// HIPAR tc/type trigger.
/// HIPAR ordering/Sets the ordering type to be triggered on for the transaction layer
/// HIPAR ordering/type trigger.
/// HIPAR snoop/Triggers on a snoop state.
/// HIPAR requesterID/Sets the requester ID to be triggered on for the transaction layer
/// HIPAR requesterID/type trigger.
/// HIPAR addressType/Sets the address type to be triggered on for the transaction layer
/// HIPAR addressType/type trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerTLPCAS(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViString  requesterID,
                                                     ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the DLLP t type to be triggered on for the data link layer
/// HIFN  trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR dllp/Sets the DLLP t type to be triggered on for the data link layer
/// HIPAR dllp/trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPType(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   dllp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP, dllp),
            3, "DLLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the multi-root type for the data link layer to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR mrDLLP/Sets the multi-root type for the data link layer to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPMRType(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   mrDLLP)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TYPE, mrDLLP),
            3, "Mr DLLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR init trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_PHASE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_VH_FC
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_MIXED_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_AUTHORIZED
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_DEVICE_PORT_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:PHAS
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:VHFC
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:MIXT
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:AUTH
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:DVPT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR phase/Sets the phase state to be triggered on for the data link layer
/// HIPAR phase/trigger.
/// HIPAR vhFC/Sets the virtual hierarchies flow control presence state to be
/// HIPAR vhFC/triggered on for the data link layer trigger.
/// HIPAR mixedType/Sets the mixed type presence state to be triggered on for the data
/// HIPAR mixedType/link layer trigger.
/// HIPAR authorized/Sets the authorized presence state to be triggered on for the data
/// HIPAR authorized/link layer trigger.
/// HIPAR devicePortType/Sets the device/port type to be triggered on for the data link layer
/// HIPAR devicePortType/trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPMRInit(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   phase,
                                                         ViInt32   vhFC,
                                                         ViInt32   mixedType,
                                                         ViInt32   authorized,
                                                         ViInt32   devicePortType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_PHASE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_PHASE, phase),
            3, "Phase");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_VH_FC, vhFC),
            4, "VH FC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_MIXED_TYPE, mixedType),
            5, "Mixed Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_AUTHORIZED, authorized),
            6, "Authorized");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_INIT_DEVICE_PORT_TYPE, devicePortType),
            7, "Device Port Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR reset trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_A
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VH_GROUP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:RESA
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:VHGR
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR a/Sets the on the value of the A bit to be triggered on for the data
/// HIPAR a/link layer trigger.
/// HIPAR vhGroup/Sets the virtual hierarchies group to be triggered on for the data
/// HIPAR vhGroup/link layer trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPMRReset(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   a,
                                                          ViInt32   vhGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_A,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_A, a),
            3, "A");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VH_GROUP, vhGroup),
            4, "VH Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR update/init FC trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VL_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VH_ABSENT
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TLP_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_CREDIT_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:TYPE
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:VLNR
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:HABS
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:TLPT
/// HIFN     TRIGger<m>:PCIE:DLLP:MULT:CRET
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR mrDLLP/Sets the multi-root type for the data link layer to be triggered on.
/// HIPAR vlNumber/Sets the virtual link (VL) number to be triggered on for the data link
/// HIPAR vlNumber/layer trigger.
/// HIPAR vhAbsent/Sets the state of the absent virtual hieararchies (VH) to be triggered
/// HIPAR vhAbsent/on for the data link layer trigger.
/// HIPAR tlpType/Sets the multi-root type to be triggered on for the data link layer
/// HIPAR tlpType/trigger.
/// HIPAR creditType/Sets the credit type to be triggered on for the multi-root data link
/// HIPAR creditType/layer trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPMRFC(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   mrDLLP,
                                                       ViInt32   vlNumber,
                                                       ViInt32   vhAbsent,
                                                       ViInt32   tlpType,
                                                       ViInt32   creditType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_A,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TYPE, mrDLLP),
            3, "Mr DLLP");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VL_NUMBER, vlNumber),
            4, "VL Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_VH_ABSENT, vhAbsent),
            5, "VH Absent");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_TLP_TYPE, tlpType),
            6, "TLP Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_MR_CREDIT_TYPE, creditType),
            7, "Credit Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the ACK/NAK trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_SEQUENCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:TYPE
/// HIFN     TRIGger<m>:PCIE:DLLP:SEQ
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR dllp/Sets the DLLP t type to be triggered on for the data link layer
/// HIPAR dllp/trigger.
/// HIPAR sequence/Sets the sequence field, indicating what TLPs are affected by the
/// HIPAR sequence/ACK/NAK, to be triggered on for the data link layer trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPAck(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   dllp,
                                                      ViString  sequence)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP, dllp),
            3, "DLLP");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_SEQUENCE, sequence),
            4, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP update/init FC trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_FC_CREDIT_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_FC_VC_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:TYPE
/// HIFN     TRIGger<m>:PCIE:DLLP:FCTL:CRET
/// HIFN     TRIGger<m>:PCIE:DLLP:FCTL:VCID
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR dllp/Sets the DLLP t type to be triggered on for the data link layer
/// HIPAR dllp/trigger.
/// HIPAR creditType/Sets credit type value to be triggered on for the data link layer
/// HIPAR creditType/trigger.
/// HIPAR vcID/Sets the virtual channel ID to be triggered on for the data link layer
/// HIPAR vcID/trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPFC(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   dllp,
                                                     ViInt32   creditType,
                                                     ViInt32   vcID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP, dllp),
            3, "DLLP");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_FC_CREDIT_TYPE, creditType),
            4, "Credit Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_FC_VC_ID, vcID),
            5, "VC ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP PM trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_PM_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:POWM
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR pmType/Sets the power management type to be triggered on for the data link
/// HIPAR pmType/layer trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPPM(ViSession instrSession,
                                                     ViInt32   trigger,
                                                     ViInt32   pmType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_PM_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_PM_TYPE, pmType),
            3, "PM Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the vendor pattern to be triggered on for the data link layer
/// HIFN  trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_VENDOR_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:PCIE:DLLP:VPAT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR vendorPattern/Sets the vendor pattern to be triggered on for the data link layer
/// HIPAR vendorPattern/trigger.
ViStatus _VI_FUNC rsscope_ConfigurePCIeTriggerDLLPVendor(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViString  vendorPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_VENDOR_PATTERN,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PCIE_TRIGGER_DLLP_VENDOR_PATTERN, vendorPattern),
            3, "Vendor Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the packet count for the selected serial bus, i.e. the number
/// HIFN  of packets in the present acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_PCIE_PACKET_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packetCount/Returns the packet count for the selected serial bus, i.e. the number
/// HIPAR packetCount/of packets in the present acquisition.
ViStatus _VI_FUNC rsscope_QueryPCIePacketCount(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32*  packetCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_PACKET_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_PACKET_COUNT, packetCount),
            3, "Packet Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:STATe?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:STARt?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:STOP?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR packetStatus/Returns the overall state of the selected packet.
/// HIPAR packetStart/Time of packet start.
/// HIPAR packetStop/Time of packet stop.
/// HIPAR packetType/Returns the type of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketStatus(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   packet,
                                                ViInt32*  packetStatus,
                                                ViReal64* packetStart,
                                                ViReal64* packetStop,
                                                ViInt32*  packetType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:STAT?", serialBus, packet);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketStatusArr, packetStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:STAR?", serialBus, packet);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, packetStart));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:STOP?", serialBus, packet);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, packetStop));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:TYPE?", serialBus, packet);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketTypeArr, packetType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified packet
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryPCIePacketDataBytes(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   packet,
                                                   ViInt32   arraySize,
                                                   ViInt32   data[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:DATA?", serialBus, packet);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the sequence number of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:SEQN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR sequenceNumber/Returns the sequence number of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketSequenceNumber(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   packet,
                                                        ViInt32*  sequenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:SEQN?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sequenceNumber));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the tag of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:TAG?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR tag/Returns the tag of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketTag(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   packet,
                                             ViInt32*  tag)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:TAG?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, tag));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the length of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:LEN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR length/Returns the length of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketLength(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   packet,
                                                ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:LEN?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, length));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the packet address for the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:ADDR?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR address/Returns the packet address for thespecified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketAddress(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   packet,
                                                 ViInt32*  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:ADDR?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the credit value of the header of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:HFC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR hfc/Returns the credit value of the header of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketHFC(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   packet,
                                             ViInt32*  hfc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:HFC?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, hfc));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the credit value of the payload data of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:DFC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR dfc/Returns the credit value of the payload data of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketDFC(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   packet,
                                             ViInt32*  dfc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:DFC?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, dfc));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the completion status field value of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:CPS?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR completionStatus/Returns the completion status field value of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketCompletionStatus(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   packet,
                                                          ViInt32*  completionStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:CPS?", serialBus, packet);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketCompletionStatusArr, completionStatus));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the requester ID of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:RQID?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR requesterID/Returns the requester ID of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketRequesterID(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   packet,
                                                     ViInt32*  requesterID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:RQID?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, requesterID));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the completer ID of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:CPID?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR completerID/Returns the completer ID of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketCompleterID(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   packet,
                                                     ViInt32*  completerID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:CPID?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, completerID));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of decoded fields in the selected packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR decodedFieldsCount/Returns the number of decoded fields in the selected packet.
ViStatus _VI_FUNC rsscope_QueryPCIePacketDecodedFieldsCount(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   packet,
                                                            ViInt32*  decodedFieldsCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:FCO?", serialBus, packet);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, decodedFieldsCount));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet with the selected field
/// HIFN  number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:FLD<o>:STATus?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:FLD<o>:TYPE?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:FLD<o>:VAL?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR field/This control selects the field.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Field Type'.
/// HIPAR fieldStatus/Returns the status of the frame with the selected field number within
/// HIPAR fieldStatus/the packet with the selected packet number (for the selected serial
/// HIPAR fieldStatus/bus).
/// HIPAR fieldType/Returns the value of the field with the selected field number within
/// HIPAR fieldType/the packet with the selected packet number (for the selected serial
/// HIPAR fieldType/bus).
/// HIPAR fieldValue/Returns the value of the field with the selected field number within
/// HIPAR fieldValue/the packet with the selected packet number (for the selected serial
/// HIPAR fieldValue/bus).
ViStatus _VI_FUNC rsscope_QueryPCIePacketField(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   packet,
                                               ViInt32   field,
                                               ViInt32   bufferSize,
                                               ViInt32*  fieldStatus,
                                               ViChar    fieldType[],
                                               ViInt32*  fieldValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:FLD%ld:STAT?", serialBus, packet, field);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketStatusArr, fieldStatus));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:FLD%ld:TYPE?", serialBus, packet, field);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(fieldType, bufferSize, extBuf);
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:FLD%ld:VAL?", serialBus, packet, field);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected packet number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:FLD<o>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR field/This control selects the field.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Formated Value'.
/// HIPAR formattedValue/Formated value.
ViStatus _VI_FUNC rsscope_QueryPCIePacketFormattedValue(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   packet,
                                                        ViInt32   field,
                                                        ViInt32   bufferSize,
                                                        ViChar    formattedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PCIE:RES:PACK%ld:FLD%ld:FVAL?", serialBus, packet, field);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(formattedValue, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the bit rate of the selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR packet/Selects the packet.
/// HIPAR bitRate/Returns the bit rate of the selected packet.
ViStatus _VI_FUNC rsscope_QueryProtocolPCIePacketBitRate(ViSession instrSession,
                                                         ViInt32   serialBus,
                                                         ViInt32   packet,
                                                         ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_PACKET_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_PCIE_SELECTED_PACKET, packet),
            3, "Packet");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_PACKET_BIT_RATE, bitRate),
            4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the use of the decision feedback equalization for PCIe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR dfe/Enables the use of the decision feedback equalization for PCIe.
ViStatus _VI_FUNC rsscope_ConfigurePCIeDSPDFEEnabled(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViBoolean dfe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_ENABLED, dfe),
            3, "DFE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the use of estimate coefficients
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR estimateCoefficients/Enables the use of estimate coefficients
ViStatus _VI_FUNC rsscope_ConfigurePCIeDSPDFEEstimateCoefficientsEnabled(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViBoolean estimateCoefficients)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_ESTIMATE_COEFFICIENTS_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_ESTIMATE_COEFFICIENTS_ENABLED, estimateCoefficients),
            3, "Estimate Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the value of the coefficient for the DFE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR coefficient/Sets the value of the coefficient for the DFE.
ViStatus _VI_FUNC rsscope_ConfigureCIeDSPDFECoefficient(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViReal64  coefficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_COEFFICIENT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_COEFFICIENT, coefficient),
            3, "Coefficient");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a number of taps for the decision feedback filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects repeated capability SerialBus.
/// HIPAR numberOfTaps/Sets a number of taps for the decision feedback filter.
ViStatus _VI_FUNC rsscope_ConfigurePCIeDSPDFENumberOfTaps(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViInt32   numberOfTaps)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  busCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SerialBus", &busCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, busCount),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_NUMBER_OF_TAPS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_PCIE_DSP_DFE_NUMBER_OF_TAPS, numberOfTaps),
            3, "Number Of Taps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger type for the PCI express analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchType/Sets the trigger type for the PCI express analysis.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   searchType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TYPE, searchType),
            3, "Search Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PCIe ordered sets search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR ts1/Enables searching for training sequence 1 ordered sets.
/// HIPAR ts2/Enables searching for training sequence 2 ordered sets.
/// HIPAR fastTrainingSeq/Enables searching for fast training sequence ordered sets.
/// HIPAR electricalIdle/Enables searching for electrical idle ordered sets.
/// HIPAR electricalIdleExit/Enables searching for electrical idle exit ordered sets.
/// HIPAR skp/Enables searching for SKP ordered sets.
/// HIPAR compliancePattern/Enables searching forcompliance pattern ordered sets.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchOrderedSets(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViBoolean ts1,
                                                         ViBoolean ts2,
                                                         ViBoolean fastTrainingSeq,
                                                         ViBoolean electricalIdle,
                                                         ViBoolean electricalIdleExit,
                                                         ViBoolean skp,
                                                         ViBoolean compliancePattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_TRAINING_SEQUENCE_1, ts1),
            3, "TS 1");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_TRAINING_SEQUENCE_2, ts2),
            4, "TS 2");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_FAST_TRAINING_SEQUENCE, fastTrainingSeq),
            5, "Fast Training Sequence");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_EIDL, electricalIdle),
            6, "Electrical Idle");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_EIDE, electricalIdleExit),
            7, "Electrical Idle Exit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_SKP, skp),
            8, "SKP");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_OSET_COMP, compliancePattern),
            9, "Compliance Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables searching for a start of data sequence order set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR startOfDataSequence/Enables searching for a start of data sequence order set.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchOrderedSetsStartOfDataSequence(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean startOfDataSequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_OSET_START_OF_DATA_SEQUENCE, startOfDataSequence),
            3, "Start Of Data Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PCIe search error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR crc16/Enables searching for 16-bit cyclic redundancy check (CRC) errors.
/// HIPAR lcrc/Enables searching for link cyclic redundancy check (LCRC) errors.
/// HIPAR ecrc/Enables searching for end-to-end cyclic redundancy checksum (ECRC)
/// HIPAR ecrc/errors.
/// HIPAR invalidSymbol/Enables searching for invalid symbol errors.
/// HIPAR disparityError/Enables searching for disparity errors.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchError(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViBoolean crc16,
                                                   ViBoolean lcrc,
                                                   ViBoolean ecrc,
                                                   ViBoolean invalidSymbol,
                                                   ViBoolean disparityError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_ERRC_CRC16, crc16),
            3, "CRC 16");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_ERRC_LCRC, lcrc),
            4, "LCRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_ERRC_ECRC, ecrc),
            5, "ECRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_ERRC_INVALID_SYMBOL, invalidSymbol),
            6, "Invalid Symbol");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_ERRC_DISP, disparityError),
            7, "Disparity Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures PCIe errors to search for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR lengthError/Enables searching for length error.
/// HIPAR malformedError/Searches for a malformed TLP: a TLP that violates specific TLP
/// HIPAR malformedError/formation rules that are defined in the specification.
/// HIPAR maxEndPrefix/Enables searching for an end-end prefix error. The maximum number of
/// HIPAR maxEndPrefix/permitted end-end TLP prefixes is 4.
/// HIPAR prefixOrderError/Enables searching for a wrong prefix order
/// HIPAR syncBit/Enables searching for a sync bit
/// HIPAR unknownPacket/Enables searching for an unknown packet.
/// HIPAR valueOutOfRange/Triggers, if a value is out of range.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchErrorCondition(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViBoolean lengthError,
                                                            ViBoolean malformedError,
                                                            ViBoolean maxEndPrefix,
                                                            ViBoolean prefixOrderError,
                                                            ViBoolean syncBit,
                                                            ViBoolean unknownPacket,
                                                            ViBoolean valueOutOfRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_LENGTH_ERROR, lengthError),
            3, "Length Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_MALFORMED_ERROR, malformedError),
            4, "Malformed Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_MAX_END_PREFIX, maxEndPrefix),
            5, "Max End Prefix");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_PREFIX_ORDER_ERROR, prefixOrderError),
            6, "Prefix Order Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_SYNC_BIT, syncBit),
            7, "Sync Bit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_UNKNOWN_PACKET, unknownPacket),
            8, "Unknown Packet");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PCIE_SEARCH_ERRC_VALUE_OUT_OF_RANGE, valueOutOfRange),
            9, "Value Out Of Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the transaction type to be searched for in the transaction layer
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR tlp/Sets the transaction type to be searched for in the transaction layer
/// HIPAR tlp/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPType(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   tlp)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP, tlp),
            3, "TLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP any type search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPAnyType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViString  sequenceNumber,
                                                        ViInt32   tc,
                                                        ViInt32   ordering,
                                                        ViInt32   snoop,
                                                        ViString  requesterID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP mem search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:MERW
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:ADRT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR readWrite/Sets the read/ write type to be searched for in the transaction layer
/// HIPAR readWrite/type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR addressType/Sets the address type to be searched for in the transaction layer
/// HIPAR addressType/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPMem(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViInt32   readWrite,
                                                    ViString  requesterID,
                                                    ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE, readWrite),
            7, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            8, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE, addressType),
            9, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP IO search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:MERW
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR readWrite/Sets the read/ write type to be searched for in the transaction layer
/// HIPAR readWrite/type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPIO(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViString  sequenceNumber,
                                                   ViInt32   tc,
                                                   ViInt32   ordering,
                                                   ViInt32   snoop,
                                                   ViInt32   readWrite,
                                                   ViString  requesterID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            3, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            4, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            5, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE, readWrite),
            6, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP Cfg search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_DEVICE_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:MERW
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:CFGT
/// HIFN     SEARch:TRIGger:PCIE:TLP:DEID
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR readWrite/Sets the read/ write type to be searched for in the transaction layer
/// HIPAR readWrite/type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR type/Sets the configuration type to be searched for in the transaction
/// HIPAR type/layer type.
/// HIPAR deviceID/Sets the device ID to be searched for in the transaction layer type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPCfg(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViInt32   readWrite,
                                                    ViString  requesterID,
                                                    ViInt32   type,
                                                    ViString  deviceID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_READ_WRITE, readWrite),
            7, "Read Write");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            8, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TYPE, type),
            9, "Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_DEVICE_ID, deviceID),
            10, "Device ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP IO search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ROUTING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_MESSAGE_CODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:MSGR
/// HIFN     SEARch:TRIGger:PCIE:TLP:MSGC
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR routing/Sets the message routing type to be searched for in the transaction
/// HIPAR routing/layer type.
/// HIPAR messageCode/Sets the message code to be searched for in the transaction layer
/// HIPAR messageCode/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPMsg(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViString  requesterID,
                                                    ViInt32   routing,
                                                    ViString  messageCode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ROUTING, routing),
            8, "Routing");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_MESSAGE_CODE, messageCode),
            9, "Message Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP Cpl search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_STATUS
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_COMPLETER_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:CPLS
/// HIFN     SEARch:TRIGger:PCIE:TLP:CPID
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR status/Sets the completion status to be searched for in the transaction layer
/// HIPAR status/type.
/// HIPAR completerID/Sets the completer ID to be searched for in the transaction layer
/// HIPAR completerID/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPCpl(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViString  requesterID,
                                                    ViInt32   status,
                                                    ViString  completerID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_STATUS, status),
            8, "Status");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_COMPLETER_ID, completerID),
            9, "Completer ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP Fetch Add search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:ADRT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR addressType/Sets the address type to be searched for in the transaction layer
/// HIPAR addressType/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPFetchAdd(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViString  sequenceNumber,
                                                         ViInt32   tc,
                                                         ViInt32   ordering,
                                                         ViInt32   snoop,
                                                         ViString  requesterID,
                                                         ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP swap search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ROUTING  *
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_MESSAGE_CODE  *
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_STATUS    *
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_COMPLETER_ID *
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:MSGR
/// HIFN     SEARch:TRIGger:PCIE:TLP:MSGC
/// HIFN     SEARch:TRIGger:PCIE:TLP:CPLS
/// HIFN     SEARch:TRIGger:PCIE:TLP:CPID
/// HIFN     SEARch:TRIGger:PCIE:TLP:ADRT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR addressType/Sets the address type to be searched for in the transaction layer
/// HIPAR addressType/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPSwap(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViString  sequenceNumber,
                                                     ViInt32   tc,
                                                     ViInt32   ordering,
                                                     ViInt32   snoop,
                                                     ViString  requesterID,
                                                     ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the TLP CAS search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNUM
/// HIFN     SEARch:TRIGger:PCIE:TLP:TCHN
/// HIFN     SEARch:TRIGger:PCIE:TLP:ORDE
/// HIFN     SEARch:TRIGger:PCIE:TLP:SNOO
/// HIFN     SEARch:TRIGger:PCIE:TLP:REID
/// HIFN     SEARch:TRIGger:PCIE:TLP:ADRT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR sequenceNumber/Sets the sequence numbe to be searched for in the transaction layer
/// HIPAR sequenceNumber/type.
/// HIPAR tc/Sets the traffic class to be searched for in the transaction layer
/// HIPAR tc/type.
/// HIPAR ordering/Sets the ordering type to be searched for in the transaction layer
/// HIPAR ordering/type.
/// HIPAR snoop/Sets the snoop state to be searched for in the transaction layer type.
/// HIPAR requesterID/Sets the requester ID to be searched for in the transaction layer
/// HIPAR requesterID/type.
/// HIPAR addressType/Sets the address type to be searched for in the transaction layer
/// HIPAR addressType/type.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchTLPCAS(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViString  sequenceNumber,
                                                    ViInt32   tc,
                                                    ViInt32   ordering,
                                                    ViInt32   snoop,
                                                    ViString  requesterID,
                                                    ViInt32   addressType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SEQUENCE_NUMBER, sequenceNumber),
            3, "Sequence Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_TC, tc),
            4, "TC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ORDERING, ordering),
            5, "Ordering");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_SNOOP, snoop),
            6, "Snoop");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_REQUESTER_ID, requesterID),
            7, "Requester ID");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_TLP_ADDRESS_TYPE, addressType),
            8, "Address Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the DLLP t type to be searched for, for the data link layer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dllp/Sets the DLLP t type to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPType(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   dllp)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP, dllp),
            3, "DLLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the multi-root type to be searched for, for the data link layer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR mrDLLP/Sets the multi-root type to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPMRType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   mrDLLP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TYPE, mrDLLP),
            3, "MR DLLP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR init search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_PHASE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_VH_FC
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_MIXED_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_AUTHORIZED
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_DEVICE_PORT_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:PHAS
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:VHFC
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:MIXT
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:AUTH
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:DVPT
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR phase/Sets the phase state to be searched for, for the data link layer.
/// HIPAR vhFC/Sets the virtual hierarchies flow control presence state to be
/// HIPAR vhFC/searched for, for the data link layer.
/// HIPAR mixedType/Sets the mixed type presence state to be searched for, for the data
/// HIPAR mixedType/link layer.
/// HIPAR authorized/Sets the authorized presence state to be searched for, for the data
/// HIPAR authorized/link layer.
/// HIPAR devicePortType/Sets the device/port type to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPMRInit(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   phase,
                                                        ViInt32   vhFC,
                                                        ViInt32   mixedType,
                                                        ViInt32   authorized,
                                                        ViInt32   devicePortType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_PHASE, phase),
            3, "Phase");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_VH_FC, vhFC),
            4, "VH FC");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_MIXED_TYPE, mixedType),
            5, "Mixed Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_AUTHORIZED, authorized),
            6, "Authorized");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_INIT_DEVICE_PORT_TYPE, devicePortType),
            7, "Device Port Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR reset search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_A
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VH_GROUP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:RESA
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:VHGR
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR a/Sets the on the value of the A bit to be searched for, for the data
/// HIPAR a/link layer.
/// HIPAR vhGroup/Sets the virtual hierarchies group to be searched for, for the data
/// HIPAR vhGroup/link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPMRReset(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   a,
                                                         ViInt32   vhGroup)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_A, a),
            3, "A");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VH_GROUP, vhGroup),
            4, "VH Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP MR update/init FC trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VL_NUMBER
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VH_ABSENT
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TLP_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_CREDIT_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:TYPE
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:VLNR
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:HABS
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:TLPT
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:CRET
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR mrDLLP/Sets the multi-root type to be searched for, for the data link layer.
/// HIPAR vlNumber/Sets the virtual link (VL) number to be searched for, for the data
/// HIPAR vlNumber/link layer.
/// HIPAR vhAbsent/Sets the state of the absent virtual hieararchies (VH) to be searched
/// HIPAR vhAbsent/for, for the data link layer.
/// HIPAR tlpType/Sets the multi-root type to be searched for, for the data link layer.
/// HIPAR creditType/Sets the credit type to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPMRFC(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   mrDLLP,
                                                      ViInt32   vlNumber,
                                                      ViInt32   vhAbsent,
                                                      ViInt32   tlpType,
                                                      ViInt32   creditType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TYPE, mrDLLP),
            3, "Mr DLLP");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VL_NUMBER, vlNumber),
            4, "VL Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_VH_ABSENT, vhAbsent),
            5, "VH Absent");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_TLP_TYPE, tlpType),
            6, "TLP Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_CREDIT_TYPE, creditType),
            7, "Credit Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the ACK/NAK trigger.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_SEQUENCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:TYPE
/// HIFN     SEARch:TRIGger:PCIE:DLLP:SEQ
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dllp/Sets the DLLP t type to be searched for, for the data link layer.
/// HIPAR sequence/Sets the sequence field, indicating what TLPs are affected by the
/// HIPAR sequence/ACK/NAK, to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPAck(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   dllp,
                                                     ViString  sequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP, dllp),
            3, "DLLP");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_SEQUENCE, sequence),
            4, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP update/init FC search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_CREDIT_TYPE
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_FC_VC_ID
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:TYPE
/// HIFN     SEARch:TRIGger:PCIE:DLLP:MULT:CRET
/// HIFN     SEARch:TRIGger:PCIE:DLLP:FCTL:VCID
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dllp/Sets the DLLP t type to be searched for, for the data link layer.
/// HIPAR creditType/Sets the credit type to be searched for, for the data link layer.
/// HIPAR vcID/Sets the virtual channel ID to be searched for, for the data link
/// HIPAR vcID/layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPFC(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   dllp,
                                                    ViInt32   creditType,
                                                    ViInt32   vcID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP, dllp),
            3, "DLLP");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_MR_CREDIT_TYPE, creditType),
            4, "Credit Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_FC_VC_ID, vcID),
            5, "VC ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MRDLLP PM search.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_PM_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:POWM
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR pmType/Sets the power management type to be searched for, for the data link
/// HIPAR pmType/layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPPM(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   pmType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_PM_TYPE, pmType),
            3, "PM Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the vendor pattern to be searched for, for the data link layer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PCIE_SEARCH_DLLP_VENDOR_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:PCIE:DLLP:VPAT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR vendorPatter/Sets the vendor pattern to be searched for, for the data link layer.
ViStatus _VI_FUNC rsscope_ConfigurePCIeSearchDLLPVendor(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViString  vendorPattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PCIE_SEARCH_DLLP_VENDOR_PATTERN, vendorPattern),
            3, "Vendor Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the packet count for the selected serial bus, i.e. the number
/// HIFN  of packets in the present acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_PCIE_PACKET_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR searchPCIePacketCount/Returns the packet count for the selected serial bus, i.e. the number
/// HIPAR searchPCIePacketCount/of packets in the present acquisition.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketCount(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32*  searchPCIePacketCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_PCIE_PACKET_COUNT, searchPCIePacketCount),
            3, "Search PCIe Packet Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:STATe?
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:STARt?
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:STOP?
/// HIFN     BUS<m>:PCIE:RESult:PACKet<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR packet/This control selects the packet.
/// HIPAR packetStatus/Returns the overall state of the selected packet.
/// HIPAR packetStart/Time of packet start.
/// HIPAR packetStop/Time of packet stop.
/// HIPAR packetType/Returns the type of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketStatus(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   packet,
                                                      ViInt32*  packetStatus,
                                                      ViReal64* packetStart,
                                                      ViReal64* packetStop,
                                                      ViInt32*  packetType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:STAT? '%s'", packet, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketStatusArr, packetStatus));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:STAR? '%s'", packet, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, packetStart));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:STOP? '%s'", packet, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, packetStop));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:TYPE? '%s'", packet, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketTypeArr, packetType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified packet
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketDataBytes(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   packet,
                                                         ViInt32   arraySize,
                                                         ViInt32   data[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:DATA? '%s'", packet, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the sequence number of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:SEQN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR sequenceNumber/Returns the sequence number of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketSequenceNumber(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   packet,
                            ViInt32*  sequenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:SEQN? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sequenceNumber));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the tag of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:TAG?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR tag/Returns the tag of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketTag(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   packet,
                                                   ViInt32*  tag)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:TAG? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, tag));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the length of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:LEN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR length/Returns the length of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketLength(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   packet,
                                                      ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:LEN? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, length));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the packet address for the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:ADDR?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR address/Returns the packet address for thespecified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketAddress(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   packet,
                                                       ViInt32*  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:ADDR? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, address));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the credit value of the header of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:HFC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR hfc/Returns the credit value of the header of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketHFC(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   packet,
                                                   ViInt32*  hfc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:HFC? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, hfc));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the credit value of the payload data of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:DFC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR dfc/Returns the credit value of the payload data of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketDFC(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   packet,
                                                   ViInt32*  dfc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:DFC? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, dfc));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the completion status field value of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:CPS?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR completionStatus/Returns the completion status field value of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketCompletionStatus(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   packet,
                            ViInt32*  completionStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:CPS? '%s'", packet, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketCompletionStatusArr, completionStatus));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the requester ID of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:RQID?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR requesterID/Returns the requester ID of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketRequesterID(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   packet,
                                                           ViInt32*  requesterID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:RQID? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, requesterID));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the completer ID of the specified packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:CPID?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR completerID/Returns the completer ID of the specified packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketCompleterID(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   packet,
                                                           ViInt32*  completerID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:CPID? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, completerID));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of decoded fields in the selected packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR decodedFieldsCount/Returns the number of decoded fields in the selected packet.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketDecodedFieldsCount(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   packet,
                            ViInt32*  decodedFieldsCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:FCO? '%s'", packet, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, decodedFieldsCount));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the packet with the selected field
/// HIFN  number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:FLD<n>:STATus?
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:FLD<n>:TYPE?
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:FLD<n>:VAL?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR field/This control selects the field.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Field Type'.
/// HIPAR fieldStatus/Returns the status of the frame with the selected field number within
/// HIPAR fieldStatus/the packet with the selected packet number (for the selected serial
/// HIPAR fieldStatus/bus).
/// HIPAR fieldType/Returns the value of the field with the selected field number within
/// HIPAR fieldType/the packet with the selected packet number (for the selected serial
/// HIPAR fieldType/bus).
/// HIPAR fieldValue/Returns the value of the field with the selected field number within
/// HIPAR fieldValue/the packet with the selected packet number (for the selected serial
/// HIPAR fieldValue/bus).
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketField(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   packet,
                                                     ViInt32   field,
                                                     ViInt32   bufferSize,
                                                     ViInt32*  fieldStatus,
                                                     ViChar    fieldType[],
                                                     ViInt32*  fieldValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:FLD%ld:STAT? '%s'", packet, field, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolPCIEPacketStatusArr, fieldStatus));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:FLD%ld:TYPE? '%s'", packet, field, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(fieldType, bufferSize, extBuf);
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:FLD%ld:VAL? '%s'", packet, field, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, fieldValue));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the formatted value of the field with the selected field
/// HIFN  number within the frame with the selected packet number (for the
/// HIFN  selected serial bus).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:PCIE:PACKet<m>:FLD<n>:FVAL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR packet/This control selects the packet.
/// HIPAR field/This control selects the field.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Formated Value'.
/// HIPAR formattedValue/Formated value.
ViStatus _VI_FUNC rsscope_QueryPCIeSearchPacketFormattedValue(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   packet,
                            ViInt32   field,
                            ViInt32   bufferSize,
                            ViChar    formattedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:PCIE:PACK%ld:FLD%ld:FVAL? '%s'", packet, field, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(formattedValue, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the CXPI Protocol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_POLARITY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:SDATa
/// HIFN     BUS<m>:CXPI:POLarity
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Selects the source of the strobe signal.
/// HIPAR polarity/Sets the polarity of the transmitted waveform to normal (high = 1) or
/// HIPAR polarity/inverted (low = 1).
ViStatus _VI_FUNC rsscope_ConfigureCXPI(ViSession instrSession,
                                        ViInt32   serialBus,
                                        ViInt32   source,
                                        ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, source, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_SOURCE, source),
            3, "Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_POLARITY, polarity),
            4, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the thresholds for the signal.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_THRESHOLD
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_HYSTERESIS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:THReshold
/// HIFN     BUS<m>:CXPI:HYSTeresis
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR threshold/Sets the threshold value for the signal.
/// HIPAR hysteresis/Sets a value for the hysteresis.
ViStatus _VI_FUNC rsscope_ConfigureCXPIThresholds(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViReal64  threshold,
                                                  ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_THRESHOLD,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_THRESHOLD, threshold),
            3, "Threshold");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_HYSTERESIS, hysteresis),
            4, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the order in which the signal is displayed in the honeycomb.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_DISPLAY_ORDER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:DORD
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR displayOrder/Selects the order in which the signal is displayed in the honeycomb.
ViStatus _VI_FUNC rsscope_ConfigureCXPIDisplayOrder(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   displayOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_DISPLAY_ORDER,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_DISPLAY_ORDER, displayOrder),
            3, "Display Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  COnfigures the CXPI Bit Rate.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_AVERAGE_BIT_RATE_ENABLED
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_BIT_RATE_VALUE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:BITRate:ENABle
/// HIFN     BUS<m>:CXPI:BITRate:VALue
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR enabled/Enables the manual setting of a bit rate.
/// HIPAR bitRateValue/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsscope_ConfigureCXPIBitRate(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViBoolean enabled,
                                               ViInt32   bitRateValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_AVERAGE_BIT_RATE_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_AVERAGE_BIT_RATE_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_BIT_RATE_VALUE, bitRateValue),
                4, "Bit Rate Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the measured average bit rate, which is calculated.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_MEASURED_BIT_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESult:BITRate?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR measuredBitRate/Queries the measured average bit rate, which is calculated.
ViStatus _VI_FUNC rsscope_QueryCXPIMeasuredBitRate(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32*  measuredBitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_MEASURED_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_MEASURED_BIT_RATE, measuredBitRate),
            3, "Measured Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the CXPI Expected IBS and IFS length.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IBS_MIN
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IBS_MAX
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IFS_MIN
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IFS_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:IFS:MIN
/// HIFN     BUS<m>:CXPI:IBS:MAX
/// HIFN     BUS<m>:CXPI:IFS:MIN
/// HIFN     BUS<m>:CXPI:IFS:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR ibsMin/Sets the minimum for the inter-byte-space range.
/// HIPAR ibsMax/Sets the maximum for the inter-byte-space range.
/// HIPAR ifsMin/Sets the minimum for the inter-frame-space range.
/// HIPAR ifsMax/Sets the maximum for the inter-frame-space range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISpacing(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   ibsMin,
                                               ViInt32   ibsMax,
                                               ViInt32   ifsMin,
                                               ViInt32   ifsMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IBS_MIN,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IBS_MIN, ibsMin),
            3, "IBS Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IBS_MAX, ibsMax),
            4, "IBS Max");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IFS_MIN, ifsMin),
            5, "IFS Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_EXPECTED_IFS_MAX, ifsMax),
            6, "IFS Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the trigger type for the CXPI analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Selects the trigger type for the CXPI analysis.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerType(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the value of the trigger settings field.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_NM
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_CT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:NM
/// HIFN     TRIGger<m>:CXPI:CT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR nm/Sets the value of the network management (NM) field.
/// HIPAR ct/Sets the value of the counter (CT), 2 bits indicating the continuity
/// HIPAR ct/of the frame.
ViStatus _VI_FUNC rsscope_ConfigureCXPITrigger(ViSession instrSession,
                                               ViInt32   trigger,
                                               ViString  nm,
                                               ViString  ct)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_NM,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_NM, nm),
            3, "NM");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_CT, ct),
            4, "CT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CXPI trigger frame ID.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:FID:CONDition
/// HIFN     TRIGger<m>:CXPI:FID:MIN
/// HIFN     TRIGger<m>:CXPI:FID:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the frame ID. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a frame ID pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a frame ID range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerFrameID(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   condition,
                                                      ViString  min,
                                                      ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_FRAME_ID_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger DLC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:DLC:CONDition
/// HIFN     TRIGger<m>:CXPI:DLC:MIN
/// HIFN     TRIGger<m>:CXPI:DLC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the DLC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a DLC pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerDLC(ViSession instrSession,
                                                  ViInt32   trigger,
                                                  ViInt32   condition,
                                                  ViString  min,
                                                  ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger DLC extend value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:DEXTension:CONDition
/// HIFN     TRIGger<m>:CXPI:DEXTension:MIN
/// HIFN     TRIGger<m>:CXPI:DEXTension:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the DLC extend value. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a data extension bit pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a DLC extend range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerDLCExtend(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  min,
                                                        ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DLC_EXTEND_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CXPI data pattern.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DATA_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:DATA:DCONdition
/// HIFN     TRIGger<m>:CXPI:DATA:DMIN
/// HIFN     TRIGger<m>:CXPI:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the data pattern. You can define no value, an
/// HIPAR condition/exact value or a value range.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerData(ViSession instrSession,
                                                   ViInt32   trigger,
                                                   ViInt32   condition,
                                                   ViString  min,
                                                   ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DATA_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_DATA_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CXPI trigger data index value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:DATA:ICONdition
/// HIFN     TRIGger<m>:CXPI:DATA:IMIN
/// HIFN     TRIGger<m>:CXPI:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the data index value. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a data index bit pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data index range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerIndex(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViInt32   condition,
                                                    ViInt32   min,
                                                    ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_INDEX_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the CXPI trigger error.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_CRC
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_DLC
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_IBS
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_IFS
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_LENGTH
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_PARITY
/// HIFN     RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_UART
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:CXPI:ERRor:CRC
/// HIFN     TRIGger<m>:CXPI:ERRor:DLC
/// HIFN     TRIGger<m>:CXPI:ERRor:IBS
/// HIFN     TRIGger<m>:CXPI:ERRor:IFS
/// HIFN     TRIGger<m>:CXPI:ERRor:LENGth
/// HIFN     TRIGger<m>:CXPI:ERRor:PARity
/// HIFN     TRIGger<m>:CXPI:ERRor:UART
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR crc/Triggers on a cyclic redundancy check error.
/// HIPAR dlc/Triggers on a data length code (DLC) error.
/// HIPAR ibs/Triggers on an inter-byte-space (IBS) error.
/// HIPAR ifs/Triggers on an inter-frame-space (IFS) error.
/// HIPAR length/Triggers on a length error.
/// HIPAR parity/Triggers on a parity error indicating a transmission error.
/// HIPAR uart/Triggers on an UART error.
ViStatus _VI_FUNC rsscope_ConfigureCXPITriggerError(ViSession instrSession,
                                                    ViInt32   trigger,
                                                    ViBoolean crc,
                                                    ViBoolean dlc,
                                                    ViBoolean ibs,
                                                    ViBoolean ifs,
                                                    ViBoolean length,
                                                    ViBoolean parity,
                                                    ViBoolean uart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_CRC,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_CRC, crc),
            3, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_DLC, dlc),
            4, "DLC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_IBS, ibs),
            5, "IBS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_IFS, ifs),
            6, "IFS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_LENGTH, length),
            7, "Length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_PARITY, parity),
            8, "Parity");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CXPI_TRIGGER_ERROR_UART, uart),
            9, "UART");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_CXPI_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESult:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryCXPIFrameCount(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_CXPI_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_CXPI_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESult:FRAMe<n>:STATe?
/// HIFN     BUS<m>:CXPI:RESults:FRAMe<n>:STARt?
/// HIFN     BUS<m>:CXPI:RESults:FRAMe<n>:STOP?
/// HIFN     BUS<m>:CXPI:RESults:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of space wire frame.
ViStatus _VI_FUNC rsscope_QueryCXPIFrameStatus(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   frame,
                                               ViInt32*  frameStatus,
                                               ViReal64* frameStart,
                                               ViReal64* frameStop,
                                               ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCXPITypeArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESults:FRAMe<n>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryCXPIFrameDataBytes(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32   arraySize,
                                                  ViInt32   data[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the data length code/ extension data length code field value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESult:FRAMe<n>:DLCV?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR dlcField/Returns the data length code/ extension data length code field value.
ViStatus _VI_FUNC rsscope_QueryCXPIFrameDLCField(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViReal64* dlcField)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:DLCV?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, dlcField));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:CXPI:RESult:FRAMe<n>:WORD<o>:STATus?
/// HIFN     BUS<m>:CXPI:RESult:FRAMe<n>:WORD<o>:TYPE?
/// HIFN     BUS<m>:CXPI:RESult:FRAMe<n>:WORD<o>:VALue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Selects the word number.
/// HIPAR status/Returns the status of the specified field.
/// HIPAR type/Returns the type (name) of the specified field.
/// HIPAR value/Returns the numeric value of the specified field.
ViStatus _VI_FUNC rsscope_QueryCXPIFrameWordStatus(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   word,
                                                   ViInt32*  status,
                                                   ViChar    type[],
                                                   ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:WORD%ld:STAT?", serialBus, frame, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCXPITypeArr, status));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:WORD%ld:TYPE?", serialBus, frame, word);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, type));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:CXPI:RES:FRAM%ld:WORD%ld:VAL?", serialBus, frame, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the trigger type for the CXPI analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR cxpiSearchType/Selects the trigger type for the CXPI analysis.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchType(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   searchType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_TYPE, searchType),
            3, "Search Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search fields NM and CT.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_NM
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_CT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:NM
/// HIFN     SEARch:TRIGger:CXPI:CT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR nm/Sets the value of the network management (NM) field.
/// HIPAR ct/Sets the value of the counter (CT), 2 bits indicating the continuity
/// HIPAR ct/of the frame.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearch(ViSession instrSession,
                                              ViString  searchName,
                                              ViString  nm,
                                              ViString  ct)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_NM, nm),
            3, "NM");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_CT, ct),
            4, "CT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for the frame ID.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:FID:CONDition
/// HIFN     SEARch:TRIGger:CXPI:FID:MIN
/// HIFN     SEARch:TRIGger:CXPI:FID:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the frame ID. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a frame ID pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a frame ID range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchFrameID(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   condition,
                                                     ViString  min,
                                                     ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_FRAME_ID_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for the DLC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:DLC:CONDition
/// HIFN     SEARch:TRIGger:CXPI:DLC:MIN
/// HIFN     SEARch:TRIGger:CXPI:DLC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the DLC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a DLC pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchDLC(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   condition,
                                                 ViString  min,
                                                 ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for the DLC Extend.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:DEXTension:CONDition
/// HIFN     SEARch:TRIGger:CXPI:DEXTension:MIN
/// HIFN     SEARch:TRIGger:CXPI:DEXTension:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the DLC extend value. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a data extension bit pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a DLC extend range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchDLCExtend(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  min,
                                                       ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DLC_EXTEND_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for the Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DATA_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:DATA:DCONdition
/// HIFN     SEARch:TRIGger:CXPI:DATA:DMIN
/// HIFN     SEARch:TRIGger:CXPI:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the data pattern. You can define no value, an
/// HIPAR condition/exact value or a value range.
/// HIPAR min/Specifies a data bit pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchData(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   condition,
                                                  ViString  min,
                                                  ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DATA_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_DATA_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for the Index.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_INDEX_CONDITION
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_INDEX_MIN
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_INDEX_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:DATA:ICONdition
/// HIFN     SEARch:TRIGger:CXPI:DATA:IMIN
/// HIFN     SEARch:TRIGger:CXPI:DATA:IMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the data index value. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a data index bit pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data index range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchIndex(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   condition,
                                                   ViInt32   min,
                                                   ViInt32   max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_INDEX_MIN, min),
            4, "Min");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_INDEX_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the searches for errors.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_CRC
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_DLC
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_IBS
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_IFS
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_LENGTH
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_PARITY
/// HIFN     RSSCOPE_ATTR_CXPI_SEARCH_ERROR_UART
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:CXPI:ERRor:CRC
/// HIFN     SEARch:TRIGger:CXPI:ERRor:DLC
/// HIFN     SEARch:TRIGger:CXPI:ERRor:IBS
/// HIFN     SEARch:TRIGger:CXPI:ERRor:IFS
/// HIFN     SEARch:TRIGger:CXPI:ERRor:LENGth
/// HIFN     SEARch:TRIGger:CXPI:ERRor:PARity
/// HIFN     SEARch:TRIGger:CXPI:ERRor:UART
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR crc/Searches for a cyclic redundancy check error.
/// HIPAR dlc/Searches for a data length code (DLC) error.
/// HIPAR ibs/Searches for an inter-byte-space (IBS) error.
/// HIPAR ifs/Searches for an inter-frame-space (IFS) error.
/// HIPAR length/Searches for a length error.
/// HIPAR parity/Searches for a parity error indicating a transmission error.
/// HIPAR uart/Searches for an UART error.
ViStatus _VI_FUNC rsscope_ConfigureCXPISearchError(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViBoolean crc,
                                                   ViBoolean dlc,
                                                   ViBoolean ibs,
                                                   ViBoolean ifs,
                                                   ViBoolean length,
                                                   ViBoolean parity,
                                                   ViBoolean uart)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_CRC, crc),
            3, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_DLC, dlc),
            4, "DLC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_IBS, ibs),
            5, "IBS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_IFS, ifs),
            6, "IFS");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_LENGTH, length),
            7, "Length");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_PARITY, parity),
            8, "Parity");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_CXPI_SEARCH_ERROR_UART, uart),
            9, "UART");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames in the acquisition.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_CXPI_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CXPI:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of frames in the acquisition.
ViStatus _VI_FUNC rsscope_QueryCXPISearchFrameCount(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_CXPI_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:STATe?
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:STARt?
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:STOP?
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the selected frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the type of space wire frame.
ViStatus _VI_FUNC rsscope_QueryCXPISearchFrameStatus(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   frame,
                                                     ViInt32*  frameStatus,
                                                     ViReal64* frameStart,
                                                     ViReal64* frameStop,
                                                     ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCXPITypeArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolSpaceWireType_Arr, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data length code/ extension data length code field value.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:DLCV?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR dlcField/Returns the data length code/ extension data length code field value.
ViStatus _VI_FUNC rsscope_QueryCXPISearchFrameDLCField(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViReal64* dlcField)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:DLCV? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, dlcField));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryCXPISearchFrameDataBytes(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32   arraySize,
                                                        ViInt32   data[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:WORD<n>:STATus?
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:WORD<n>:TYPE?
/// HIFN     SEARch:RESult:CXPI:FRAMe<m>:WORD<n>:VALue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Selects the word number.
/// HIPAR status/Returns the status of the specified field.
/// HIPAR type/Returns the type (name) of the specified field.
/// HIPAR value/Returns the numeric value of the specified field.
ViStatus _VI_FUNC rsscope_QueryCXPISearchFrameWordStatus(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   word,
                                                         ViInt32*  status,
                                                         ViChar    type[],
                                                         ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K76"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:WORD%ld:STAT? '%s'", frame, word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolCXPITypeArr, status));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:WORD%ld:TYPE? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, 4096, type));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:CXPI:FRAM%ld:WORD%ld:VAL? '%s'", frame, word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DDR protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR type/Selects the DDR type.
/// HIPAR source/Selects the source for the signal.
/// HIPAR threshold/Sets the threshold value for the digitization of the data line.
/// HIPAR hysteresis/Sets a value for the hysteresis of the data.
ViStatus _VI_FUNC rsscope_ConfigureDDR(ViSession instrSession,
                                       ViInt32   serialBus,
                                       ViInt32   type,
                                       ViInt32   source,
                                       ViReal64  threshold,
                                       ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DDR_TYPE_DATA, RSSCOPE_VAL_DDR_TYPE_STROBE),
            3, "Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DDR_DATA_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    if (type == RSSCOPE_VAL_DDR_TYPE_DATA)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_DATA_SOURCE, source),
                4, "Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_DATA_THRESHOLD, threshold),
                5, "Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_DATA_HYSTERESIS, hysteresis),
                6, "Hysteresis");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_STROBE_SOURCE, source),
                4, "Source");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_STROBE_THRESHOLD, threshold),
                5, "Threshold");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_STROBE_HYSTERESIS, hysteresis),
                6, "Hysteresis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded frames in the selected serial bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of decoded frames in the selected serial bus.
ViStatus _VI_FUNC rsscope_QueryDDRFrameCount(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the status of the frame with the selected frame number (for
/// HIPAR frameStatus/the selected serial bus).
/// HIPAR frameStart/Returns the start time of the frame with the selected frame number
/// HIPAR frameStart/(for the selected serial bus).
/// HIPAR frameStop/Returns the stop time of the frame with the selected frame number (for
/// HIPAR frameStop/the selected serial bus).
/// HIPAR frameType/Returns the frame type for the selected serial bus and frame number.
ViStatus _VI_FUNC rsscope_QueryDDRFrameStatus(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   frame,
                                              ViInt32*  frameStatus,
                                              ViReal64* frameStart,
                                              ViReal64* frameStop,
                                              ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_STATUS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_STATUS, frameStatus),
            4, "Frame Status");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_START_TIME, frameStart),
            5, "Frame Start");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_STOP_TIME, frameStop),
            6, "Frame Stop");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_TYPE, frameType),
            7, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of words for the selected serial bus and frame
/// HIFN  number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR words/Returns the number of words for the selected serial bus and frame
/// HIPAR words/number.
ViStatus _VI_FUNC rsscope_QueryDDRFrameNumberOfWords(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32*  words)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_NUMBER_OF_WORDS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_NUMBER_OF_WORDS, words),
            4, "Words");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame field.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field.
/// HIPAR frameStatus/Returns the status of the field with the selected field number within
/// HIPAR frameStatus/the frame with the selected frame number (for the selected serial bus).
/// HIPAR frameStart/Returns the start time of the field with the selected field number
/// HIPAR frameStart/within the frame with the selected frame number (for the selected
/// HIPAR frameStart/serial bus).
/// HIPAR frameStop/Returns the stop time of the field with the selected field number
/// HIPAR frameStop/within the frame with the selected frame number (for the selected
/// HIPAR frameStop/serial bus).
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number (for the selected serial bus).
/// HIPAR formattedValue/Returns the formatted value of the field with the selected field
/// HIPAR formattedValue/number within the frame with the selected frame number (for the
/// HIPAR formattedValue/selected serial bus).
ViStatus _VI_FUNC rsscope_QueryDDRFrameFieldStatus(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   field,
                                                   ViInt32*  frameStatus,
                                                   ViReal64* frameStart,
                                                   ViReal64* frameStop,
                                                   ViInt32*  value,
                                                   ViInt32   bufferSize,
                                                   ViChar    formattedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_NUMBER_OF_WORDS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_SELECTED_FIELD, field),
            4, "Field");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_FIELD_STATUS, frameStatus),
            5, "Frame Status");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_FIELD_START_TIME, frameStart),
            6, "Frame Start");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_FIELD_STOP_TIME, frameStop),
            7, "Frame Stop");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_FIELD_VALUE, value),
            8, "Value");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_DDR_FRAME_FIELD_FORMATTED_VALUE,
            bufferSize, formattedValue),
            10, "Formatted Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the eye diagram display.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EYE_DIAGRAM_MAX_SLICES_PER_ACQUISITION
/// HIFN     RSSCOPE_ATTR_EYE_DIAGRAM_EYE_STRIPE_DISPLAY_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EYE<m>:MSLices
/// HIFN     EYE<m>:DISPlay[:MODE]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR maxSlicesPerAcquisition/Sets the number of waveform slices for a single acquisition.
/// HIPAR stripeDisplayMode/Defines when the eye stripe is shown: always, never, or during mask
/// HIPAR stripeDisplayMode/tests.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramDisplay(ViSession instrSession,
                                                        ViInt32   eyeDiagram,
                                                        ViInt32   maxSlicesPerAcquisition,
                                                        ViInt32   stripeDisplayMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_MAX_SLICES_PER_ACQUISITION,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_MAX_SLICES_PER_ACQUISITION, maxSlicesPerAcquisition),
            3, "Max Slices Per Acquisition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_EYE_STRIPE_DISPLAY_MODE, stripeDisplayMode),
            4, "Stripe Display Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Each eye diagram has its specific persistence settings,
///HINF  independent of the general waveform persistence.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR persistenceState/If enabled, each new data point in the
///HIPAR persistenceState/diagram area remains on the screen for the duration
///HIPAR persistenceState/defined using Time parameter, or as long as
///HIPAR persistenceState/Infinite Persistence State is enabled.
///HIPAR infinitePersistenceState/If persistence is enabled, each new
///HIPAR infinitePersistenceState/data point in the diagram area remains
///HIPAR infinitePersistenceState/on the screen infinitely until this
///HIPAR infinitePersistenceState/command is set to OFF.
///HIPAR time/If persistence is enabled, each new data point in the
///HIPAR time/diagram area remains on the screen for the duration defined
///HIPAR time/here.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramPersistence(ViSession instrSession,
                                                            ViInt32   eyeDiagram,
                                                            ViBoolean persistenceState,
                                                            ViBoolean infinitePersistenceState,
                                                            ViReal64  time)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_INFINITE_PERSISTENCE_STATE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_DISPLAY_PERSISTENCE_STATE, persistenceState),
            3, "Persistence State");

    if(persistenceState == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_INFINITE_PERSISTENCE_STATE, infinitePersistenceState),
                4, "Infinite Persistence State");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_DISPLAY_PERSISTENCE_TIME, time),
                5, "Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Resets the display, removing persistent values.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
ViStatus _VI_FUNC rsscope_ResetDDREyeDiagramPersistence(ViSession instrSession,
                                                        ViInt32   eyeDiagram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_PERSISTENCE_RESET,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    checkErr(rsscope_SetAttributeViString(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_PERSISTENCE_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR enabled/Activates or deactivates the eye diagram.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramEnabled(ViSession instrSession,
                                                        ViInt32   eyeDiagram,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_STATE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the eye diagram sources.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR source/Selects the waveform from which the eye diagram is generated (data
/// HIPAR source/source). For analog channels, peak detect, envelope, and average
/// HIPAR source/waveforms are not supported.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramSource(ViSession instrSession,
                                                       ViInt32   eyeDiagram,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_SOURCE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Declares the nominal bit rate of the data signal for the DDR eye
/// HIFN  diagram analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR bitRate/Declares the nominal bit rate of the data signal for the DDR eye
/// HIPAR bitRate/diagram analysis.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramBitRate(ViSession instrSession,
                                                        ViInt32   eyeDiagram,
                                                        ViInt64   bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_BIT_RATE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_EYE_DIAGRAM_BIT_RATE, bitRate),
                3, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the eye diagram timing reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR timingReferenceSource/Selects the waveform that is used to obtain the timing information
/// HIPAR timingReferenceSource/required to slice the data source waveform.
/// HIPAR slope/Sets the edge on the timing reference waveform to define the
/// HIPAR slope/timestamps for slicing the data waveform. A timestamp is set each time,
/// HIPAR slope/when the selected slope of the timing waveform crosses the selected
/// HIPAR slope/reference level.
/// HIPAR level/Sets the reference level on the timing reference waveform to define
/// HIPAR level/the timestamps for slicing the data waveform. A timestamp is set each
/// HIPAR level/time, when the selected slope of the timing waveform crosses the
/// HIPAR level/selected reference level.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramTimingReference(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   timingReferenceSource,
                            ViInt32   slope,
                            ViInt32   level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SOURCE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SOURCE, timingReferenceSource),
            3, "Timing Reference Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SLOPE, slope),
            4, "Slope");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIME_REFERENCE_VERTICAL_REFERENCE_LEVEL, level),
            5, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the source of the reference signal used for the eye
///HINF  diagram.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR referenceSignalSource/Selects the source of the reference
///HIPAR referenceSignalSource/signal used for the eye diagram.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceSignalSource(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   referenceSignalSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SIGNAL_SOURCE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SIGNAL_SOURCE, referenceSignalSource),
            3, "Reference Signal Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects hardware or software clock data recovery.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR cdrType/Selects hardware or software clock data recovery.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceCDRType(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   cdrType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CDR_TYPE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CDR_TYPE, cdrType),
            3, "CDR Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the software CDR used for the reference signal of the
///HINF  eye diagram.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR softwareCDR/Selects the software CDR used for the reference
///HIPAR softwareCDR/signal of the eye diagram.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceSoftwareCDR(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   softwareCDR)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SOFTWARE_CDR,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_SOFTWARE_CDR, softwareCDR),
            3, "Software CDR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the settings of the reference waveform of the CDR
///HINF  reference waveform.
///HIRET Returns the status code of this operation.
///HIPAR referenceNumber/Selects the reference waveform.
///HIPAR algorithm/Queries the algorithm of the CDR reference waveform.
///HIPAR bandwidth/Queries the bandwidth of the CDR reference waveform.
///HIPAR relativeBandwidth/Queries the relative bandwidth of the CDR
///HIPAR relativeBandwidth/reference waveform.
///HIPAR bitRate/Queries the bit rate of the CDR reference waveform.
///HIPAR sampling/Queries the sampling time of the CDR reference
///HIPAR sampling/waveform.
///HIPAR analogSource/Queries the analog source of the CDR refernce
///HIPAR analogSource/waveform.
ViStatus _VI_FUNC rsscope_QueryEyeDiagramCDRReferenceFiles(ViSession instrSession,
                                                           ViInt32   referenceNumber,
                                                           ViInt32*  algorithm,
                                                           ViReal64* bandwidth,
                                                           ViInt32*  relativeBandwidth,
                                                           ViInt64*  bitRate,
                                                           ViReal64* sampling,
                                                           ViInt32*  analogSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   referenceNumberRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  referenceNumberCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "ReferenceWfm", &referenceNumberCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, referenceNumberCount),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_ALGORITHM,
        "ReferenceWfm", referenceNumber - 1, RS_REPCAP_BUF_SIZE, referenceNumberRepCap), 2 ,"Reference Number");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, referenceNumberRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_ALGORITHM, algorithm),
            3, "Algorithm");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, referenceNumberRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_BANDWIDTH, bandwidth),
            4, "Bandwidth");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, referenceNumberRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_RELATIVE_BANDWIDTH, relativeBandwidth),
            5, "Relative Bandwidth");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, referenceNumberRepCap,
        RSSCOPE_ATTR_EYE_DIAGRAM_CDR_BIT_RATE, bitRate), 6, "Bit Rate");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, referenceNumberRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_SAMPLING, sampling),
            7, "Sampling");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, referenceNumberRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_CDR_ANALOG_SOURCE, analogSource),
            8, "Analog Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Executes an estimation of the signal for the clock frequency.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
ViStatus _VI_FUNC rsscope_EstimateEyeDiagramTimingReferenceClockFrequency(
                            ViSession instrSession,
                            ViInt32   eyeDiagram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_FREQUENCY_ESTIMATION,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    checkErr(rsscope_SetAttributeViString(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_FREQUENCY_ESTIMATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the setting of the explicit clock, if
///HINF  rsscope_ConfigureEyeDiagramTimingReferenceSignalSource is set to CLOCk.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR clockSource/Selects the source of the clock channel, if
///HIPAR clockSource/rsscope_ConfigureEyeDiagramTimingReferenceSignalSource
///HIPAR clockSource/is set to CLOCk.
///HIPAR clockLevel/Sets the vertical reference level of the timing
///HIPAR clockLevel/reference source.
///HIPAR clockFrequency/Sets the clock frequency for the explicit
///HIPAR clockFrequency/clock signal.
///HIPAR clockFactor/Sets a clock multiplier factor.
///HIPAR edgeType/The clock edge sets the point in time when the state
///HIPAR edgeType/of the data signal is checked.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceExplicitClock(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   clockSource,
                            ViInt32   clockLevel,
                            ViReal64  clockFrequency,
                            ViInt32   clockFactor,
                            ViInt32   edgeType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_SOURCE_TYPE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_SOURCE_TYPE, clockSource),
            3, "Clock Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_LEVEL, clockLevel),
            4, "Clock Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_FREQUENCY, clockFrequency),
            5, "Clock Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_FACTOR, clockFactor),
            6, "Clock Factor");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_EDGE_TYPE, edgeType),
            7, "Edge Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Configures clock delay of the explicit clock, if
///HINF  rsscope_ConfigureEyeDiagramTimingReferenceSignalSource is set to CLOCk.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR clockOffset/Sets a clock offset for the explicit clock signal.
///HIPAR samplingTime/Sets a sampling time for the explicit clock
///HIPAR samplingTime/signal, an offset for the clock edge in relation to
///HIPAR samplingTime/the bit start. The clock edge sets the beginning of
///HIPAR samplingTime/the unit interval. The sampling time is a number
///HIPAR samplingTime/between 0 and 1. Value 0 sets the clock edge to the
///HIPAR samplingTime/beginning of the bit period; value 0.5 sets the
///HIPAR samplingTime/clock edge to the middle of the bit period.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceClockDelay(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViReal64  clockOffset,
                            ViReal64  samplingTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_OFFSET,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_OFFSET, clockOffset),
            3, "Clock Offset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CLOCK_SAMPLING_TIME, samplingTime),
            4, "Sampling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the active bus for the eye analysis, if
///HINF  rsscope_ConfigureEyeDiagramTimingReferenceSignalSource is set to PROTocol.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR source/Selects the active bus for the eye analysis, if
///HIPAR source/rsscope_ConfigureEyeDiagramTimingReferenceSignalSource is set
///HIPAR source/to PROTocol.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramTimingReferenceSerialBus(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_PROTOCOL_SERIAL_BUS_INDEX,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_PROTOCOL_SERIAL_BUS_INDEX, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the eye diagram horizontal settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR timeSettings/Sets the method to define the horizontal scale of the eye diagram.
/// HIPAR timeRange/Sets the time range that is covered by the eye diagram.
/// HIPAR timePosition/Sets the place of the zero point in the diagram, in seconds. The zero
/// HIPAR timePosition/point is the alignment point on which the slice timestamps are
/// HIPAR timePosition/superimposed.
/// HIPAR bitRate/Sets the nominal frequency or bit rate of the data signal.
/// HIPAR rangeUI/Sets the number of unit intervals that are visible in the eye diagram.
/// HIPAR positionUI/Sets the zero point in the diagram, in unit intervals. The zero point
/// HIPAR positionUI/is the alignment point on which the segment timestamps are
/// HIPAR positionUI/superimposed.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramHorizontalSettings(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   timeSettings,
                            ViReal64  timeRange,
                            ViReal64  timePosition,
                            ViReal64  bitRate,
                            ViReal64  rangeUI,
                            ViReal64  positionUI)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_TIME_UNITS,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_TIME_UNITS, timeSettings),
            3, "Time Settings");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_TIME_RANGE, timeRange),
            4, "Time Range");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_TIME_POSITION, timePosition),
            5, "Time Position");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_BIT_RATE, bitRate),
            6, "Bit Rate");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_RANGE_UI, rangeUI),
            7, "Range UI");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_HORIZONTAL_SETTINGS_POSITION_UI, positionUI),
            8, "Position UI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the vertical settings for the eye analysis.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR scale/Sets the vertical scale for the indicated eye.
///HIPAR offset/The offset voltage is subtracted to correct an
///HIPAR offset/offset-affected signal.
///HIPAR position/Sets the vertical position of the indicated eye as a
///HIPAR position/graphical value.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramVerticalSettings(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViReal64  scale,
                            ViReal64  offset,
                            ViReal64  position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_VERTICAL_SETTINGS_SCALE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_VERTICAL_SETTINGS_SCALE, scale),
            3, "Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_VERTICAL_SETTINGS_OFFSET, offset),
            4, "Offset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_VERTICAL_SETTINGS_POSITION, position),
            5, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Returns the results of the Eye Diagram.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR numberofFilteredBits/Returns the number of filtered bits.
///HIPAR filteredBitPosition/Returns the position of the filtered bit
///HIPAR filteredBitPosition/in the acquisition.
///HIPAR numberofSlices/Returns the total number of slices accumulated
///HIPAR numberofSlices/into the eye.
///HIPAR lastSlicePosition/Returns the position of the last slice in
///HIPAR lastSlicePosition/the acquistion.
///HIPAR numberofAcquisitions/Returns the total number of acquisitions
///HIPAR numberofAcquisitions/accumulated into the eye.
ViStatus _VI_FUNC rsscope_QueryEyeDiagramResultsofEye(ViSession instrSession,
                                                      ViInt32   eyeDiagram,
                                                      ViInt32*  numberofFilteredBits,
                                                      ViReal64* filteredBitPosition,
                                                      ViInt32*  numberofSlices,
                                                      ViReal64* lastSlicePosition,
                                                      ViInt32*  numberofAcquisitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "EyeDiagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_BITS_COUNT,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"EyeDiagram");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_BITS_COUNT, numberofFilteredBits),
            3, "Number of Filtered Bits");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_BIT_POSITION, filteredBitPosition),
            4, "Filtered Bit Position");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_TOTAL_NUMBER_OF_SLICES, numberofSlices),
            5, "Number of Slices");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_LAST_POSITION, lastSlicePosition),
            6, "Last Slice Position");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_RESULTS_TOTAL_NUMBER_OF_ACQUISITIONS, numberofAcquisitions),
            7, "Number of Acquisitions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  If this function is enabled, the rising and falling
///HINF  transitions for the eye measurmens Eye rise time, Eye fall time and Duty
///HINF  cycle distortion are separated.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR maxPrecision/If this function is enabled, the rising and
///HIPAR maxPrecision/falling transitions for the eye measurmens Eye rise
///HIPAR maxPrecision/time, Eye fall time and Duty cycle distortion are
///HIPAR maxPrecision/separated.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramMeasurementMaxPrecision(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViBoolean maxPrecision)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_MEASUREMENT_MAX_PRECISION,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_MEASUREMENT_MAX_PRECISION, maxPrecision),
            3, "Max Precision");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the eye diagram gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR enabled/Enables the gate.
/// HIPAR couplingMode/Sets the gate coupling. To use cursor lines or zoom areas as gate
/// HIPAR couplingMode/limits, the cursor or zoom must be defined before.
/// HIPAR mode/Defines whether the gate limit is defined in absolute or relative
/// HIPAR mode/values when no gate coupling is set.
/// HIPAR start/Set the time values for gate start.
/// HIPAR stop/Set the time values for gate stop.
/// HIPAR cursor/Selects the cursor set to be used for eye gating. The gate area is
/// HIPAR cursor/defined by the cursor lines.
/// HIPAR zoomDiagram/Selects the zoom diagram to be used for eye gating. The zoom area is
/// HIPAR zoomDiagram/is used as gate.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramGate(ViSession instrSession,
                                                     ViInt32   eyeDiagram,
                                                     ViBoolean enabled,
                                                     ViInt32   couplingMode,
                                                     ViInt32   mode,
                                                     ViReal64  start,
                                                     ViReal64  stop,
                                                     ViInt32   cursor,
                                                     ViString  diagram,
                                                     ViString  zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_STATE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_STATE, enabled),
            3, "Enabled");
    if (enabled == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_COUPLING_MODE, couplingMode),
                4, "Coupling Mode");
        switch (couplingMode)
        {
        case RSSCOPE_VAL_GATE_COUPLING_MODE_NONE:

            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_MODE, mode),
                    5, "Mode");

            switch (mode)
            {
            case RSSCOPE_VAL_ABS:
                viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_ABSOLUTE_START, start),
                        6, "Start");

                viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_ABSOLUTE_STOP, stop),
                        7, "Stop");
                break;

            case RSSCOPE_VAL_REL:
                viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_RELATIVE_START, start),
                        6, "Start");

                viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_RELATIVE_STOP, stop),
                        7, "Stop");
                break;

            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, couplingMode), 5, "Coupling Mode");
            }

            break;

        case RSSCOPE_VAL_GATE_COUPLING_MODE_CURSOR:
            viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_GATE_CURSOR, cursor),
                    8, "Cursor");
            break;

        case RSSCOPE_VAL_GATE_COUPLING_MODE_ZOOM:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "EYE%ld:QUALify:GATE:ZDI '%s','%s'", eyeDiagram, diagram, zoom);
            checkErr(RsCore_Write(instrSession, cmd));
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, couplingMode), 4, "Coupling Mode");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the eye diagram gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR enabled/Enables the waveform-based qualification for the eye generation.
/// HIPAR source/Selects the waveform to be compared.
/// HIPAR condition/Selects the comparison condition: the waveform level must be greater
/// HIPAR condition/or less than the threshold.
/// HIPAR verticalReferenceLevel/Selects the threshold. You can use the lower, middle, or upper
/// HIPAR verticalReferenceLevel/reference level. These levels are the reference levels that are defined
/// HIPAR verticalReferenceLevel/for measurements.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramSignal(ViSession instrSession,
                                                       ViInt32   eyeDiagram,
                                                       ViBoolean enabled,
                                                       ViInt32   source,
                                                       ViInt32   condition,
                                                       ViInt32   verticalReferenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_STATE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_STATE, enabled),
            3, "Enabled");
    if (enabled == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_SOURCE, source),
                4, "Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_CONDITION, condition),
                5, "Condition");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_VERTICAL_REFERENCE_LEVEL, verticalReferenceLevel),
                6, "Vertical Reference Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the stop threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR verticalReferenceLevelStop/Selects the stop threshold. You can use the lower,
/// HIPAR verticalReferenceLevelStop/middle, or upper reference level.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramSignalReferenceLevelStop(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   verticalReferenceLevelStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_TIMING_REFERENCE_CDR_TYPE,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_SIGNAL_VERTICAL_REFERENCE_LEVEL_STOP, verticalReferenceLevelStop),
            3, "Vertical Reference Level Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the filter settings to the data signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR enabled/Applies the filter settings to the data signal.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterEnabled(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_STATE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the bit sequence filter to the default (DC for all bits).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterReset(ViSession instrSession,
                                                            ViInt32   eyeDiagram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_RESET,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects which bits contribute to the eye generation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR mode/Selects which bits contribute to the eye generation.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterBitSequenceMode(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_BIT_SEQUENCE_MODE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_BIT_SEQUENCE_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the values of the indicated prefix bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR bit/Selects the bit.
/// HIPAR bitValue/Sets the bit value.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterPatternPrefixBit(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   bit,
                            ViInt32   bitValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bit, 1, 8),
            3, "Bit");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitValue, &p2string),
                4, "Bit Value");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "EYE%ld:FILT:BPAT:PREF%ld %s", eyeDiagram, bit, p2string);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the values of the indicated center bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR bitValue/Set the values of the indicated center bits.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterPatternCenterBit(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   bitValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_PATTERN_CENTER_BIT,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_PATTERN_CENTER_BIT, bitValue),
            3, "Bit Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the values of the indicated suffix bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR bit/Selects the bit.
/// HIPAR bitValue/Sets the bit value.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterPatternSuffixBit(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   bit,
                            ViInt32   bitValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bit, 1, 5),
            3, "Bit");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngBitState, bitValue, &p2string),
                4, "Bit Value");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "EYE%ld:FILT:BPAT:SUFF%ld %s", eyeDiagram, bit, p2string);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Shifts the pattern one bit to the left or right.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR shift/Sets the shift direction.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterPatternShiftBit(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   shift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_PATTERN_SHIFT_BIT_LEFT,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");
    switch (shift)
    {
    case RSSCOPE_VAL_SHIFT_BIT_LEFT:
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_PATTERN_SHIFT_BIT_LEFT, NULL));
        break;

    case RSSCOPE_VAL_SHIFT_BIT_RIGHT:
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_PATTERN_SHIFT_BIT_RIGHT, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, shift), 3, "Shift");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets which frame types contribute to the eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR frameType/Sets which frame types contribute to the eye diagram.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterFrameType(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_FRAME_TYPE,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_FRAME_TYPE, frameType),
            3, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the length errors contribute to the eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eyeDiagram/This control selects the Eye Diagram.
/// HIPAR lengthErrorsEnabled/If enabled, the length errors contribute to the eye diagram.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramFilterLengthErrors(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViBoolean lengthErrorsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, 4),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_LENGTH_ERRORS,
            "EyeDiagram", eyeDiagram- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_EYE_DIAGRAM_FILTER_LENGTH_ERRORS, lengthErrorsEnabled),
            3, "Length Errors Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the Eye Diagram Mask Test Slices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR completed/Returns the number of slices that contributed to the eye diagram.
/// HIPAR hits/Returns the number of slices that violated the mask.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramMaskTestsSlices(
                            ViSession instrSession,
                            ViInt32*  completed,
                            ViInt32*  hits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_SLICES_COMPLETED, completed),
            2, "Completed");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_SLICE_HITS, hits),
            3, "Hits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the Eye Diagram Mask Test Violation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR violation/Sets the violation parameter.
/// HIPAR count/Returns the count of violations of mask test result.
/// HIPAR start/Return the start time of the slice with the indicated violation.
/// HIPAR stop/Return the stop time of the slice with the indicated violation.
/// HIPAR slice/Returns the slice at the violation of mask test result.
ViStatus _VI_FUNC rsscope_ConfigureDDREyeDiagramMaskTestsViolation(
                            ViSession instrSession,
                            ViInt32   violation,
                            ViInt32*  count,
                            ViReal64* start,
                            ViReal64* stop,
                            ViInt32*  slice)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_INDEX, violation),
            2, "Violation");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_COUNT, count),
            3, "Count");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_START_TIME, start),
            4, "Start");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_STOP_TIME, stop),
            5, "Stop");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_VIOLATION_SLICE, slice),
            6, "Slice");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  If enabled, the mask eye area is set to the limits of a zoom
///HINF  area.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR zoomCouplingEnabled/If enabled, the mask eye area is set to
///HIPAR zoomCouplingEnabled/the limits of a zoom area.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramMaskTestsZoomCoupling(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViBoolean zoomCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_ZOOM_COUPLING,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2 ,"Eye Diagram");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_EYE_DIAGRAM_MASK_TESTS_ZOOM_COUPLING, zoomCouplingEnabled),
            3, "Zoom Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the zoom to which the eye mask area is set if
///HINF  rsscope_ConfigureEyeDiagramMaskTestsZoomCoupling is enabled.
///HIRET Returns the status code of this operation.
///HIPAR eyeDiagram/This control selects the Eye Diagram.
///HIPAR diagram/String with the name of the diagram on which the zoom
///HIPAR diagram/is based
///HIPAR zoom/String with the name of the zoom.
ViStatus _VI_FUNC rsscope_ConfigureEyeDiagramMaskTestsZoomDiagram(
                            ViSession instrSession,
                            ViInt32   eyeDiagram,
                            ViString  diagram,
                            ViString  zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   eyeDiagramRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  eyeDiagramCount = 0;
    ViChar   parameters[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "EyeDiagram", &eyeDiagramCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, eyeDiagram, 1, eyeDiagramCount),
            2, "Eye Diagram");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HIDDEN_EYE_DIAGRAM_MASK_TESTS_ZOOM_DIAGRAM,
        "EyeDiagram", eyeDiagram - 1, RS_REPCAP_BUF_SIZE, eyeDiagramRepCap), 2, "Eye Diagram");

    snprintf (parameters, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", diagram, zoom);

    viCheckParm(rsscope_SetAttributeViString(instrSession, eyeDiagramRepCap, RSSCOPE_ATTR_HIDDEN_EYE_DIAGRAM_MASK_TESTS_ZOOM_DIAGRAM, parameters),
            3, "Diagram");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the error type to be searched for. You can search for all
/// HIFN  error types in parallel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR errorName/Specifies search error.
/// HIPAR enabled/Defines the error type to be searched for. You can search for all
/// HIPAR enabled/error types in parallel.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchErrorTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  errorName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:EREN '%s','%s',%s", searchName, errorName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the error type to be searched for. You can search for all
/// HIFN  error types in parallel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorNumber/Sets the error number.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Defines the error type to be searched for. You can search for all
/// HIPAR enabled/error types in parallel.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchErrorTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   errorNumber,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:ERR%ld:ENAB '%s',%s", errorNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameTypeNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FREN '%s','%s',%s", searchName, frame,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific frame to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameNumber/Sets the frame number.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the specific frame to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameTypeNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frameNumber,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:ENAB '%s',%s", frameNumber, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the specific field within the defined frame to be
/// HIFN  searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR enabled/Enables or disables the specific field within the defined frame to be
/// HIPAR enabled/searched for.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNameEnabled(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FIEN '%s','%s','%s',%s", searchName, frame, field,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the checking condition for searching a specific
/// HIFN  data pattern in the selected field of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR enabled/Enables or disables the checking condition for searching a specific
/// HIPAR enabled/data pattern in the selected field of the selected frame.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNumericEnabled(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:ENAB '%s',%s", frame, field, searchName,
             enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DDR Frame Field Name Data Search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNameData(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:DOP '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:DMIN '%s','%s','%s','%s'", searchName, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:DMAX '%s','%s','%s','%s'", searchName, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DDR Frame Field Name Index Search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Sets the index at which the data is to be searched, or sets the start
/// HIPAR min/value of an index range in which the data is to be searched.
/// HIPAR max/Sets the end value of the bit index range.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNameIndex(
                            ViSession instrSession,
                            ViString  searchName,
                            ViString  frame,
                            ViString  field,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:IOPerator '%s','%s','%s',%s", searchName, frame, field,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:IMIN '%s','%s','%s',%ld", searchName, frame, field, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:IMAX '%s','%s','%s',%ld", searchName, frame, field, max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DDR Frame Field Numeric Data Search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern to be searched in the
/// HIPAR condition/selected field of the selected frame.
/// HIPAR min/Specifies the data pattern to be searched, or sets the start value of
/// HIPAR min/a data pattern range to be searched.
/// HIPAR max/Sets the end value of a data pattern range.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNumericData(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:DOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:DMIN '%s','%s'", frame, field, searchName,
             min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:DMAX '%s','%s'", frame, field, searchName,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DDR Frame Field Numeric Index Search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/Selects the frame.
/// HIPAR field/Selects the field.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator for the index for searching in the selected field of
/// HIPAR condition/the selected frame.
/// HIPAR min/Sets the index at which the data is to be searched, or sets the start
/// HIPAR min/value of an index range in which the data is to be searched.
/// HIPAR max/Sets the end value of the bit index range.
ViStatus _VI_FUNC rsscope_ConfigureDDRSearchFrameFieldNumericIndex(
                            ViSession instrSession,
                            ViInt32   frame,
                            ViInt32   field,
                            ViString  searchName,
                            ViInt32   condition,
                            ViInt32   min,
                            ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSCOPE_VAL_COND_NQ, RSSCOPE_VAL_COND_OUT),
            3, "Condition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:IOP '%s',%s", frame, field, searchName,
             condition_Arr[condition]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:IMIN '%s',%ld", frame, field, searchName, min);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:TRIG:DDR:FRAM%ld:FLD%ld:IMAX '%s',%ld", frame, field, searchName,
                 max);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded frames in the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of decoded frames in the search result.
ViStatus _VI_FUNC rsscope_QueryDDRSearchFrameCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_DDR_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the status of the frame with the selected frame number within
/// HIPAR frameStatus/the search result.
/// HIPAR frameStart/Returns the start time of the frame with the selected frame number
/// HIPAR frameStart/within the search result.
/// HIPAR frameStop/Returns the stop time of the frame with the selected frame number
/// HIPAR frameStop/within the search result.
/// HIPAR frameType/Returns the frame type for the selected frame number within the search
/// HIPAR frameType/result.
ViStatus _VI_FUNC rsscope_QueryDDRSearchFrameStatus(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   frame,
                                                    ViInt32*  frameStatus,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngDDRFrameStatus, response, frameStatus), 4, "Frame Status");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, response));
    checkErr(rsscope_CheckStatus(instrSession));
    viCheckParm(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsScope_rngProtocolDDRFrameType, response, frameType), 7, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of words of the selected frame in the search
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR words/Returns the number of words of the selected frame in the search
/// HIPAR words/result.
ViStatus _VI_FUNC rsscope_QueryDDRSearchFrameNumberOfWords(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32*  words)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:NWRD? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, words));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/Sets the field parameter.
/// HIPAR frameStatus/Returns the status of the frame with the selected frame number within
/// HIPAR frameStatus/the search result.
/// HIPAR frameStart/Returns the start time of the selected field number within the frame
/// HIPAR frameStart/with the selected frame number within the search result.
/// HIPAR frameStop/Returns the stop time of the field with the selected field number
/// HIPAR frameStop/within the frame with the selected frame number within the search
/// HIPAR frameStop/result.
/// HIPAR value/Returns the value of the field with the selected field number within
/// HIPAR value/the frame with the selected frame number within the search result.
/// HIPAR formattedValue/Returns the formatted value of the field with the selected field
/// HIPAR formattedValue/number within the frame with the selected frame number within the
/// HIPAR formattedValue/search result.
ViStatus _VI_FUNC rsscope_QueryDDRSearchFrameFieldStatus(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   field,
                                                         ViInt32*  frameStatus,
                                                         ViReal64* frameStart,
                                                         ViReal64* frameStop,
                                                         ViInt32*  value,
                                                         ViInt32   bufferSize,
                                                         ViChar    formattedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:FLD%ld:STAT? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolDDRFrameStatusArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:FLD%ld:STAR? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:FLD%ld:STOP? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDR:FRAM%ld:FLD%ld:VAL? '%s'", frame, field, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:DDRF:FRAM%ld:FLD%ld:FVAL? '%s'", frame, field, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, formattedValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the 100BASE-T1 protocol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_ATTENUATION
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_POLARITY
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_THRESHOLD
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_MODE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SYMBOL_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:ATTN
/// HIFN     BUS<m>:HBTO:POLarity
/// HIFN     BUS<m>:HBTO:THReshold
/// HIFN     BUS<m>:HBTO:MODE
/// HIFN     BUS<m>:HBTO:SYMRate
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR attenuation/Sets the attenuation factor. It is used to de-amplify the reverse
/// HIPAR attenuation/signal before subtracting it from the forward signal.
/// HIPAR polarity/Selects the polarity of the data signal.
/// HIPAR threshold/Sets an user-defined threshold value.
/// HIPAR mode/Selects the direction of the full-duplex signal you want to look at.
/// HIPAR symbolRate/Defines the rate of ternary symbols.
ViStatus _VI_FUNC rsscope_Configure100BASET1(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViReal64  attenuation,
                                             ViInt32   polarity,
                                             ViReal64  threshold,
                                             ViInt32   mode,
                                             ViReal64  symbolRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_ATTENUATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_ATTENUATION, attenuation),
            3, "Attenuation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_THRESHOLD, threshold),
            5, "Threshold");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_MODE, mode),
            6, "Mode");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SYMBOL_RATE, symbolRate),
            7, "SymbolRate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the 100BASE-T1 protocol source.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_POSITIVE_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_POSITIVE_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_NEGATIVE_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_NEGATIVE_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_DIFFERENTIAL_SOURCE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_DIFFERENTIAL_SOURCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:FDAP
/// HIFN     BUS<m>:HBTO:RDAP
/// HIFN     BUS<m>:HBTO:FDAM
/// HIFN     BUS<m>:HBTO:RDAM
/// HIFN     BUS<m>:HBTO:FDIF
/// HIFN     BUS<m>:HBTO:RDIF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR direction/Sets the direction.
/// HIPAR positiveSource/Sets the channel for the positive signal source.
/// HIPAR negativeSource/Sets the channel for the negative signal source.
/// HIPAR differentialSource/Sets the channel for the differential signal source.
ViStatus _VI_FUNC rsscope_Configure100BASET1Source(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   direction,
                                                   ViInt32   positiveSource,
                                                   ViInt32   negativeSource,
                                                   ViInt32   differentialSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSSCOPE_VAL_DIRECTION_FORWARD, RSSCOPE_VAL_DIRECTION_REVERSE),
            3, "Direction");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_POSITIVE_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    if (direction == RSSCOPE_VAL_DIRECTION_REVERSE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_POSITIVE_SOURCE, positiveSource),
                4, "Positive Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_NEGATIVE_SOURCE, negativeSource),
                5, "Negative Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_DIFFERENTIAL_SOURCE, differentialSource),
                6, "Differential Source");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_POSITIVE_SOURCE, positiveSource),
                4, "Positive Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_NEGATIVE_SOURCE, negativeSource),
                5, "Negative Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_DIFFERENTIAL_SOURCE, differentialSource),
                6, "Differential Source");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the type of signal measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_SIGNAL_TYPE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_SIGNAL_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:FTYP
/// HIFN     BUS<m>:HBTO:RTYP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR direction/Sets the direction.
/// HIPAR type/Sets the type of signal measurement.
ViStatus _VI_FUNC rsscope_Configure100BASET1SignalType(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   direction,
                                                       ViInt32   type)
{
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_SIGNAL_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    switch (direction)
    {
    case RSSCOPE_VAL_DIRECTION_FORWARD:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FORWARD_SIGNAL_TYPE, type),
                4, "Type");
        break;

    case RSSCOPE_VAL_DIRECTION_REVERSE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_REVERSE_SIGNAL_TYPE, type),
                4, "Type");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, direction), 3, "Direction");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of frame to be triggered on.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Selects the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerType(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Destination Address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_CONDITION
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_MIN
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:DADDress:CONDition
/// HIFN     TRIGger<m>:HBTO:DADDress:MIN
/// HIFN     TRIGger<m>:HBTO:DADDress:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the Destination Address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a Destination Address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerDestinationAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DESTINATION_ADDRESS_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Source Address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_CONDITION
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_MIN
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:SADDress:CONDition
/// HIFN     TRIGger<m>:HBTO:SADDress:MIN
/// HIFN     TRIGger<m>:HBTO:SADDress:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the Source Address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a Source Address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerSourceAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_SOURCE_ADDRESS_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger Source Address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_CONDITION
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_MIN
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:LENGth:CONDition
/// HIFN     TRIGger<m>:HBTO:LENGth:MIN
/// HIFN     TRIGger<m>:HBTO:LENGth:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the Type Length. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR min/Specifies a Type Length pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerTypeLength(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_TYPE_LENGTH_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_CONDITION
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_MIN
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:CRC:CONDition
/// HIFN     TRIGger<m>:HBTO:CRC:MIN
/// HIFN     TRIGger<m>:HBTO:CRC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the CRC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a CRC pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerCRC(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   condition,
                                                       ViString  min,
                                                       ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_CRC_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_MIN
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:DATA:DCONdition
/// HIFN     TRIGger<m>:HBTO:DATA:DMIN
/// HIFN     TRIGger<m>:HBTO:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the Data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a Data pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerData(ViSession instrSession,
                                                        ViInt32   trigger,
                                                        ViInt32   condition,
                                                        ViString  min,
                                                        ViString  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_DATA_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trigger CRC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Set the condition for the Index. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a Index pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerIndex(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   condition,
                                                         ViInt32   min,
                                                         ViInt32   max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_INDEX_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_100BASE_T1_TRIGGER_INDEX_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the 100BASE T1 trigger error.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_PREAMBLE
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_CRC
/// HIFN     RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_SFD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger<m>:HBTO:ERRor:PREamble
/// HIFN     TRIGger<m>:HBTO:ERRor:CRC
/// HIFN     TRIGger<m>:HBTO:ERRor:SFD
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR preamble/Triggers on an preamble error.
/// HIPAR crc/Triggers on a Cyclic Redundancy Check (CRC) error.
/// HIPAR sfd/Triggers on start frame delimiter (SFD) error.
ViStatus _VI_FUNC rsscope_Configure100BASET1TriggerError(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViBoolean preamble,
                                                         ViBoolean crc,
                                                         ViBoolean sfd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_PREAMBLE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "repCap", RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_PREAMBLE, preamble),
            3, "Preamble");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "repCap", RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_CRC, crc),
            4, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "repCap", RSSCOPE_ATTR_100BASE_T1_TRIGGER_ERROR_SFD, sfd),
            5, "SFD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded frames.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of decoded frames.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameCount(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_COUNT, frameCount),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:STATe?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:STARt?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:STOP?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the state of the frame within the search result.
/// HIPAR frameStart/Returns the start time of the selected frame.
/// HIPAR frameStop/Returns the end time of the selected frame.
/// HIPAR frameType/Returns the value of length / type field of the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameStatus(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   frame,
                                                    ViInt32*  frameStatus,
                                                    ViReal64* frameStart,
                                                    ViReal64* frameStop,
                                                    ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocol100BASE_T1FrameStatusArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:TYPE?", serialBus, frame);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:DESTaddress?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:SRCaddress?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR destination/Returns the destination address of the selected frame.
/// HIPAR source/Returns the source address of the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameAddress(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32   bufferSize,
                                                     ViChar    destination[],
                                                     ViChar    source[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:DEST?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, destination));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:SRC?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, source));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the CRC (frame check) checksum of the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_CRC
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:CRC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR crc/Returns the CRC (frame check) checksum of the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameCRC(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   frame,
                                                 ViInt64*  crc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_CRC,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_CRC, crc),
                4, "CRC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the type of frame for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:FTYPe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR type/Returns the type of frame for the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameType(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:FTYP?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocol100BASE_T1FrameTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the frame label.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_DESTINATION_LABEL
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_SOURCE_LABEL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:DSYMbol?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:SSYMbol?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR destination/Returns the destination symbols of the selected frame.
/// HIPAR source/Returns the source symbols of the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameLabel(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   bufferSize,
                                                   ViChar    destination[],
                                                   ViChar    source[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:DSYM?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, destination));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:SSYM?", serialBus, frame);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, source));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameBitRate(ViSession instrSession,
                                                     ViInt32   frame,
                                                     ViInt32   serialBus,
                                                     ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_100BASE_T1_SELECTED_FRAME, frame),
            3, "Frame");

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_BIT_RATE, bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data for the selected frame, corresponds to the Data
/// HIFN  column in the decode results table.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameDataBytes(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt32   arraySize,
                                                       ViInt32   data[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:DATA?", serialBus, frame);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words for the selected frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_FUNC_SEARCH_100BASE_T1_FRAME_NUMBER_OF_WORDS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:NUMWords?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR words/Returns the number of decoded words for the selected frame.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameNumberOfWords(ViSession instrSession,
                                                           ViInt32   serialBus,
                                                           ViInt32   frame,
                                                           ViInt64*  words)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:NUMW?", serialBus, frame);
    checkErr(RsCore_QueryViInt64(instrSession, cmd, words));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_WORD_TYPE
/// HIFN     RSSCOPE_ATTR_PROTOCOL_100BASE_T1_FRAME_WORD_VALUE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:WORD<o>:TYPE?
/// HIFN     BUS<m>:HBTO:RESult:FRAMe<n>:WORD<o>:VALue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Selects the word number.
/// HIPAR type/Returns the data type for the selected word.
/// HIPAR value/Returns the data value for the selected word.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameWordStatus(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt64   word,
                                                        ViInt32   bufferSize,
                                                        ViChar    type[],
                                                        ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:WORD%lld:TYPE?", serialBus, frame, word);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, type));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:WORD%lld:VAL?", serialBus, frame, word);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the string equivalent of data value for the selected cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Selects the word number.
/// HIPAR value/Returns the string equivalent of data value for the selected cell.
ViStatus _VI_FUNC rsscope_Query100BASET1FrameWordStringValue(ViSession instrSession,
                                                             ViInt32   serialBus,
                                                             ViInt32   frame,
                                                             ViInt64   word,
                                                             ViInt32   bufferSize,
                                                             ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:HBTO:RES:FRAM%ld:WORD%lld:VSTR?", serialBus, frame, word);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of frame to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR triggerType/Selects the type of frame to be searched for.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchType(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for Destination Address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:DADDress:CONDition
/// HIFN     SEARch:TRIGger:HBTO:DADDress:MIN
/// HIFN     SEARch:TRIGger:HBTO:DADDress:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the Destination Address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a Destination Address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchDestinationAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_MIN, min),
            4, "Minimum");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DESTINATION_ADDRESS_MAX, max),
                5, "Maximum");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for Source Address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:SADDress:CONDition
/// HIFN     SEARch:TRIGger:HBTO:SADDress:MIN
/// HIFN     SEARch:TRIGger:HBTO:SADDress:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the Source Address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR min/Specifies a Source Address pattern, or sets the start value of a
/// HIPAR min/pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchSourceAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  min,
                            ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_SOURCE_ADDRESS_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for Type Length.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:LENGth:CONDition
/// HIFN     SEARch:TRIGger:HBTO:LENGth:MIN
/// HIFN     SEARch:TRIGger:HBTO:LENGth:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the Type Length. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR min/Specifies a Type Length pattern, or sets the start value of a pattern
/// HIPAR min/range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchTypeLength(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   condition,
                                                             ViString  min,
                                                             ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_TYPE_LENGTH_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:CRC:CONDition
/// HIFN     SEARch:TRIGger:HBTO:CRC:MIN
/// HIFN     SEARch:TRIGger:HBTO:CRC:MAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the CRC. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a CRC pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchCRC(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32   condition,
                                                      ViString  min,
                                                      ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_CRC_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the search for Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_CONDITION
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_MIN
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_MAX
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:DATA:DCONdition
/// HIFN     SEARch:TRIGger:HBTO:DATA:DMIN
/// HIFN     SEARch:TRIGger:HBTO:DATA:DMAX
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the Data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a Data pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchData(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   condition,
                                                       ViString  min,
                                                       ViString  max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_DATA_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for Index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Set the condition for the Index. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR min/Specifies a Index pattern, or sets the start value of a pattern range.
/// HIPAR max/Sets the end value of a data type range for condition In Range or Out
/// HIPAR max/of Range.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchIndex(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   condition,
                                                        ViInt32   min,
                                                        ViInt32   max)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_INDEX_MIN, min),
            4, "Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_INDEX_MAX, max),
                5, "Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the 100BASE T1 search error.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_PREAMBLE
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_CRC
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_SFD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:HBTO:ERRor:PREamble
/// HIFN     SEARch:TRIGger:HBTO:ERRor:CRC
/// HIFN     SEARch:TRIGger:HBTO:ERRor:SFD
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR preamble/Search for preamble error.
/// HIPAR crc/Search for Cyclic Redundancy Check (CRC) error.
/// HIPAR sfd/Search for start frame delimiter (SFD) error.
ViStatus _VI_FUNC rsscope_Configure100BASET1SearchError(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViBoolean preamble,
                                                        ViBoolean crc,
                                                        ViBoolean sfd)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_PREAMBLE, preamble),
            3, "Preamble");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_CRC, crc),
            4, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_100BASE_T1_ERROR_SFD, sfd),
            5, "SFD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded frames within the search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_100BASE_T1_FRAME_COUNT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of decoded frames within the search result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameCount(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_100BASE_T1_FRAME_COUNT, frameCount),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:STATe?
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:STARt?
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:STOP?
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:TYPE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the state of the frame within the search result.
/// HIPAR frameStart/Returns the start time of the selected frame within the search result.
/// HIPAR frameStop/Returns the end time of the selected frame within the search result.
/// HIPAR frameType/Returns the value of length / type field of the selected frame within
/// HIPAR frameType/the search result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameStatus(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   frame,
                                                          ViInt32*  frameStatus,
                                                          ViReal64* frameStart,
                                                          ViReal64* frameStop,
                                                          ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocol100BASE_T1FrameStatusArr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:TYPE? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, frameType));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame address.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:DESTaddress?
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:SRCaddress?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR destination/Returns the destination address of the selected frame within the
/// HIPAR destination/search result.
/// HIPAR source/Returns the source address of the selected frame within the search
/// HIPAR source/result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameAddress(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32   bufferSize,
                                                           ViChar    destination[],
                                                           ViChar    source[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd1[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   cmd2[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd1, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:DEST? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd1, bufferSize, destination));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd2, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:SRC? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd2, bufferSize, source));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame CRC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:CRC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR crc/Returns the CRC checksum of the selected frame within the search
/// HIPAR crc/result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameCRC(ViSession instrSession,
                                                       ViString  searchName,
                                                       ViInt32   frame,
                                                       ViInt32*  crc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:CRC? '%s'", frame, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, crc));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame Type.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:FTYPe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR type/Returns the type of frame for the selected frame within the search
/// HIPAR type/result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:FTYP? '%s'", frame, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocol100BASE_T1FrameTypeArr, type));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame label.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:DSYMbol?
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:SSYMbol?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR destination/Returns the destination symbols of the selected frame within the
/// HIPAR destination/search result.
/// HIPAR source/Returns the source symbols of the selected frame within the search
/// HIPAR source/result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameLabel(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32   bufferSize,
                                                         ViChar    destination[],
                                                         ViChar    source[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd1[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   cmd2[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K57|K0"));

    snprintf(cmd1, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:DSYM? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd1, bufferSize, destination));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd2, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:SSYM? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd2, bufferSize, source));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data of the specified frame
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameDataBytes(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   frame,
                                                             ViInt32   arraySize,
                                                             ViInt32   data[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded words for the selected frame within the
/// HIFN  search result.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:HBTO:FRAMe<m>:NUMWords?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR words/Returns the number of decoded words for the selected frame within the
/// HIPAR words/search result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameNumberOfWords(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32*  words)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:NUMW? '%s'", frame, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, words));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the frame word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Selects the word number.
/// HIPAR type/Returns the data type of the selected word within the search result.
/// HIPAR value/Returns the data value of the selected word within the search result.
ViStatus _VI_FUNC rsscope_Query100BASET1SearchFrameWordStatus(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   word,
                            ViInt32   bufferSize,
                            ViChar    type[],
                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:WORD%ld:TYPE? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, type));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:HBTO:FRAM%ld:WORD%ld:VAL? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Count.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the SPI Frame Count.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameCount(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FCO? '%s'", searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the SPI Frame start and stop.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:STARt?
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR start/Returns the SPI Frame Start.
/// HIPAR stop/Returns the SPI Frame Stop.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrame(ViSession instrSession,
                                              ViInt32   frame,
                                              ViString  searchName,
                                              ViReal64* start,
                                              ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:STAR? '%s'", frame, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, start));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:STOP? '%s'", frame, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, stop));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Status.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR status/Returns the SPI Frame Status.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameStatus(ViSession instrSession,
                                                    ViInt32   frame,
                                                    ViString  searchName,
                                                    ViInt32   bufferSize,
                                                    ViChar    status[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:STAT? '%s'", frame, searchName);
    checkErr(rsscope_QueryViString(instrSession, cmd, bufferSize, status));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of frames that have matched the search criteria. In
/// HIFN  the search result table on the display, the number of rows is the
/// HIFN  number of frames that match the search criteria.
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:COUNt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR resultCount/Returns the number of frames that have matched the search criteria. In
/// HIPAR resultCount/the search result table on the display, the number of rows is the
/// HIPAR resultCount/number of frames that match the search criteria.
ViStatus _VI_FUNC rsscope_QuerySPIFrameResultCount(ViSession instrSession,
                                                   ViInt32   frame,
                                                   ViInt32*  resultCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:COUN?", frame);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, resultCount));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the SPI Frame Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR data/Returns the SPI Frame Data.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameData(ViSession instrSession,
                                                  ViInt32   frame,
                                                  ViString  searchName,
                                                  ViInt32   arraySize,
                                                  ViInt32   data[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K1|K0"));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:DATA? '%s'", frame, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Word Count.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:WCOunt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the SPI Frame Word Count.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameWordCount(ViSession instrSession,
                                                       ViInt32   frame,
                                                       ViString  searchName,
                                                       ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:WCO? '%s'", frame, searchName);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Word.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:WORD<n>:STARt?
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:WORD<n>:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Sets the word.
/// HIPAR searchName/Specifies search definition.
/// HIPAR start/Returns the SPI Frame Word Start.
/// HIPAR stop/Returns the SPI Frame Word Stop.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameWord(ViSession instrSession,
                                                  ViInt32   frame,
                                                  ViInt32   word,
                                                  ViString  searchName,
                                                  ViReal64* start,
                                                  ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:WORD%ld:STAR? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, start));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:WORD%ld:STOP? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, stop));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Word MISO.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:WORD<n>:MISO?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Sets the word.
/// HIPAR searchName/Specifies search definition.
/// HIPAR miso/Returns the SPI Frame Word MISO.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameWordMISO(ViSession instrSession,
                                                      ViInt32   frame,
                                                      ViInt32   word,
                                                      ViString  searchName,
                                                      ViReal64* miso)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:WORD%ld:MISO? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, miso));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the SPI Frame Word MOSI.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:RESult:SPI:FRAMe<m>:WORD<n>:MOSI?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/Sets the word.
/// HIPAR searchName/Specifies search definition.
/// HIPAR mosi/Returns the SPI Frame Word MOSI.
ViStatus _VI_FUNC rsscope_QuerySPISearchFrameWordMOSI(ViSession instrSession,
                                                      ViInt32   frame,
                                                      ViInt32   word,
                                                      ViString  searchName,
                                                      ViReal64* mosi)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:SPI:FRAM%ld:WORD%ld:MOSI? '%s'", frame, word, searchName);
    checkErr(rsscope_QueryViReal64(instrSession, cmd, mosi));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the event to be searched for.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR mode/Sets the event to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchMode(ViSession instrSession,
                                                 ViString  searchName,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the operator for the data pattern: equal or not equal.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_DATA_OPERATOR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:FCONdition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dataOperator/Selects the operator for the data pattern: equal or not equal.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchDataOperator(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   dataOperator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_DATA_OPERATOR, dataOperator),
            3, "Data Operator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Operator for the data position. You can define an exact position, a
/// HIFN  position range, or let the position undefined (ANY).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION_OPERATOR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:DPOPerator
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dataPositionOperator/Operator for the data position. You can define an exact position, a
/// HIPAR dataPositionOperator/position range, or let the position undefined (ANY).
ViStatus _VI_FUNC rsscope_ConfigureSPISearchDataPositionOperator(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   dataPositionOperator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION_OPERATOR, dataPositionOperator),
            3, "Data Position Operator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of bits or words before the first word of interest.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:DPOSition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR position/Sets the number of bits or words before the first word of interest.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchDataPosition(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION, position),
            3, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the specified data pattern is searched.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_DATA_ALIGNMENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:PALignmen
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR dataAlignment/Defines how the specified data pattern is searched.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchDataAlignment(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   dataAlignment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_DATA_ALIGNMENT, dataAlignment),
            3, "Data Alignment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the end value of a data postion range.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION_TO
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:DPTO
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR positionTo/Sets the end value of a data postion range.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchDataPositionTo(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   positionTo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_DATA_POSITION_TO, positionTo),
            3, "Position To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies a data pattern for the MISO or MOSI line, respectively.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_MISO_PATTERN
/// HIFN     RSSCOPE_ATTR_SEARCH_SPI_MOSI_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SEARch:TRIGger:SPI:MISopattern
/// HIFN     SEARch:TRIGger:SPI:MOSipattern
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR selectPattern/Selects the pattern to be set.
/// HIPAR pattern/Specifies a data pattern for the MISO or MOSI line, respectively.
ViStatus _VI_FUNC rsscope_ConfigureSPISearchPattern(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   selectPattern,
                                                    ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    switch (selectPattern)
    {
    case RSSCOPE_VAL_SPI_PATTERN_MISO:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_MISO_PATTERN, pattern),
                3, "Pattern");
        break;
    case RSSCOPE_VAL_SPI_PATTERN_MOSI:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_SPI_MOSI_PATTERN, pattern),
                3, "Pattern");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectPattern), 2, "Select Pattern");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the 1000BASE-T1 protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR attenuation/Sets the attenuation factor. It is used to de-amplify the reverse
/// HIPAR attenuation/signal before subtracting it from the forward signal.
/// HIPAR polarity/Selects the polarity of the data signal.
/// HIPAR mode/Selects the direction of the full-duplex signal you want to look at.
/// HIPAR symbolRate/Defines the rate of ternary symbols.
ViStatus _VI_FUNC rsscope_ConfigureProtocol1000BASET1(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViReal64  attenuation,
                                                      ViInt32   polarity,
                                                      ViInt32   mode,
                                                      ViReal64  symbolRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_ATTENUATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_ATTENUATION, attenuation),
            3, "Attenuation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_MODE, mode),
            5, "Mode");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SYMBOL_RATE, symbolRate),
            6, "Symbol Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the type and source of the 1000BASE-T1 forward signal
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signalType/Sets the type of forward signal measurement.
/// HIPAR differentialSource/Sets the source of the provided forward differential signal, if Signal
/// HIPAR differentialSource/Type is set to Differential.
/// HIPAR positiveSource/Selects the DA+ source of the provided forward single ended signal, if
/// HIPAR positiveSource/Signal Type is set to Single.
/// HIPAR negativeSource/Selects the DA- source of the provided forward single ended signal, if
/// HIPAR negativeSource/Signal Type is set to Single.
ViStatus _VI_FUNC rsscope_ConfigureProtocol1000BASET1ForwardSignal(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   signalType,
                            ViInt32   differentialSource,
                            ViInt32   positiveSource,
                            ViInt32   negativeSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FORWARD_SIGNAL_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FORWARD_SIGNAL_TYPE, signalType),
            3, "Signal Type");

    switch (signalType)
    {
    case RSSCOPE_VAL_SIGNAL_TYPE_DIFF:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FORWARD_DIFFERENTIAL_SOURCE, differentialSource),
                4, "Differential Source");
        break;
    case RSSCOPE_VAL_SIGNAL_TYPE_SING:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FORWARD_POSITIVE_SOURCE, positiveSource),
                5, "Positive Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FORWARD_NEGATIVE_SOURCE, negativeSource),
                6, "Negative Source");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the type and source of the 1000BASE-T1 reverse signal
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR signalType/Sets the type of reverse signal measurement.
/// HIPAR differentialSource/Sets the source of the provided reversed differential signal, if
/// HIPAR differentialSource/Signal Type is set to Differential.
/// HIPAR positiveSource/Selects the DA+ source of the provided reversed single ended signal,
/// HIPAR positiveSource/if Signal Type is set to Single.
/// HIPAR negativeSource/Selects the DA- source of the provided reversed single ended signal,
/// HIPAR negativeSource/if Signal Type is set to Single.
ViStatus _VI_FUNC rsscope_ConfigureProtocol1000BASET1ReverseSignal(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   signalType,
                            ViInt32   differentialSource,
                            ViInt32   positiveSource,
                            ViInt32   negativeSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_REVERSE_SIGNAL_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_REVERSE_SIGNAL_TYPE, signalType),
            3, "Signal Type");

    switch (signalType)
    {
    case RSSCOPE_VAL_SIGNAL_TYPE_DIFF:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_REVERSE_DIFFERENTIAL_SOURCE, differentialSource),
                4, "Differential Source");
        break;
    case RSSCOPE_VAL_SIGNAL_TYPE_SING:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_REVERSE_POSITIVE_SOURCE, positiveSource),
                5, "Positive Source");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_REVERSE_NEGATIVE_SOURCE, negativeSource),
                6, "Negative Source");
        break;
    case RSSCOPE_VAL_SIGNAL_TYPE_DIS:
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of frame to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Selects the type of frame to be triggered on.
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerType(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_TYPE, triggerType),
            3, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR level/Sets the trigger level.
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerLevel(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrigger = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Trigger", &maxTrigger);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, 1, maxTrigger),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LEVEL,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the destination address trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the destination address. You can define an
/// HIPAR condition/exact value or a value range.
/// HIPAR destinationAddressMin/Specifies the destination address bit pattern, or sets the start value
/// HIPAR destinationAddressMin/of a pattern range. The string parameter accepts the bit value X (do
/// HIPAR destinationAddressMin/not care).
/// HIPAR destinationAddressMax/Sets the end value of the destination address range if Condition is
/// HIPAR destinationAddressMax/set to In Range or Out Of Range. The string parameter accepts the bit
/// HIPAR destinationAddressMax/value X (do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerDestinationAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  destinationAddressMin,
                            ViString  destinationAddressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DESTINATION_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DESTINATION_ADDRESS_MIN, destinationAddressMin),
            4, "Destination Address Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DESTINATION_ADDRESS_MAX, destinationAddressMax),
                5, "Destination Address Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the source address trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the source address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR sourceAddressMin/Specifies the source address bit pattern, or sets the start value of a
/// HIPAR sourceAddressMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR sourceAddressMin/care).
/// HIPAR sourceAddressMax/Sets the end value of the source address range, if Condition is set to
/// HIPAR sourceAddressMax/In Range or Out Of Range. The string parameter accepts the bit value X
/// HIPAR sourceAddressMax/(do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerSourceAddress(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  sourceAddressMin,
                            ViString  sourceAddressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_SOURCE_ADDRESS_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_SOURCE_ADDRESS_MIN, sourceAddressMin),
            4, "Source Address Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_SOURCE_ADDRESS_MAX, sourceAddressMax),
                5, "Source Address Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the length/type field trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the length / type. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR length/typeMin/Specifies the length / type bit pattern, or sets the start value of a
/// HIPAR length/typeMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR length/typeMin/care).
/// HIPAR length/typeMax/Sets the end value of the length / type range if Condition is set to
/// HIPAR length/typeMax/In Range or Out Of Range. The string parameter accepts the bit value X
/// HIPAR length/typeMax/(do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerLengthType(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  lengthTypeMin,
                            ViString  lengthTypeMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LENGTH_TYPE_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LENGTH_TYPE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LENGTH_TYPE_MIN, lengthTypeMin),
            4, "Length Type Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_LENGTH_TYPE_MAX, lengthTypeMax),
                5, "Length Type Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frame check (CRC) trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the frame check. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR crcMin/Specifies the frame check bit pattern, or sets the start value of a
/// HIPAR crcMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR crcMin/care).
/// HIPAR crcMax/Sets the end value of the frame check range if Condition is set to In
/// HIPAR crcMax/Range or Out Of Range. The string parameter accepts the bit value X (do
/// HIPAR crcMax/not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerFrameCheckCRC(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  crcMin,
                            ViString  crcMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_FRAME_CHECK_CRC_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_FRAME_CHECK_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_FRAME_CHECK_CRC_MIN, crcMin),
            4, "CRC Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_FRAME_CHECK_CRC_MAX, crcMax),
                5, "CRC Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR dataMin/Specifies the data bit pattern, or sets the start value of a pattern
/// HIPAR dataMin/range. The string parameter accepts the bit value X (do not care).
/// HIPAR dataMax/Sets the end value of the data range if Condition is set to In Range
/// HIPAR dataMax/or Out Of Range. The string parameter accepts the bit value X (do not
/// HIPAR dataMax/care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerData(ViSession instrSession,
                                                         ViInt32   trigger,
                                                         ViInt32   condition,
                                                         ViString  dataMin,
                                                         ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DATA_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DATA_MIN, dataMin),
            4, "Data Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_DATA_MAX, dataMax),
                5, "Data Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the data index trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the index. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR indexMin/Specifies the index bit pattern, or sets the start value of a pattern
/// HIPAR indexMin/range. In binary format, use the following characters: 1; 0; or X (do
/// HIPAR indexMin/not care).
/// HIPAR indexMax/Sets the end value of the index range if Condition is set to In Range
/// HIPAR indexMax/or Out Of Range. In binary format, use the following characters: 1; 0;
/// HIPAR indexMax/or X (do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerIndex(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   condition,
                                                          ViInt32   indexMin,
                                                          ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_INDEX_MIN, indexMin),
            4, "Index Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_INDEX_MAX, indexMax),
                5, "Index Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the 1000BASE-T1 error triggers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR rsFEC/Enables triggering, if more than 22 RS-FEC errors are detected.
/// HIPAR crc/Enables triggering on cyclic redundancy check (CRC) error.
/// HIPAR outOfRange/Enables triggering if values are out of range (OOR). For example, a
/// HIPAR outOfRange/preamble error, or invalid symbols in control blocks.
/// HIPAR zeroTernarySymbol/Enables triggering, if a zero ternary symbol pair (0-0) is detected.
ViStatus _VI_FUNC rsscope_Configure1000BASET1TriggerErrors(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViBoolean rsFEC,
                                                           ViBoolean crc,
                                                           ViBoolean outOfRange,
                                                           ViBoolean zeroTernarySymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_ERROR_RS_FEC,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_ERROR_RS_FEC, rsFEC),
            3, "RS-FEC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_ERROR_CRC, crc),
            4, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_ERROR_OUT_OF_RANGE, outOfRange),
            5, "Out Of Range");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_TRIGGER_ERROR_ZERO_TERNARY_SYMBOL, zeroTernarySymbol),
            6, "Zero Ternary Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frameCount/Returns the number of decoded frames.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameCount(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_COUNT, frameCount),
            3, "Integer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR status/Returns the state of the frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameStatus(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   frame,
                                                     ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_STATUS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_STATUS, status),
            4, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start and end time of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR startTime/Returns the start time of the selected frame.
/// HIPAR stopTime/Returns the end time of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameTime(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViReal64* startTime,
                                                   ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_START_TIME,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_START_TIME, startTime),
            4, "Start Time");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_STOP_TIME, stopTime),
            5, "Stop Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value of length / type field of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR length/typeField/Returns the value of length / type field of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameLengthTypeField(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32*  lengthTypeField)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_LENGTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_LENGTH, lengthTypeField),
            4, "Length/Type Field");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the CRC (frame check) checksum of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR crc/Returns the CRC (frame check) checksum of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameCRC(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   frame,
                                                  ViInt64*  crc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_CRC,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_CRC,
                    crc),
                4, "CRC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data for the selected frame, corresponds to the Data
/// HIFN  column in the decode results table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameDataBytes(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32   arraySize,
                                                        ViInt32   data[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   aux[RS_REPCAP_BUF_SIZE];
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DATA_BYTES_RAWSTRING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    checkErr(RsCore_BuildAttrCommandString(instrSession,
        &g_RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DATA_BYTES_RAWSTRING,
        repCap,
        &cmd));
    snprintf(aux, RS_REPCAP_BUF_SIZE, "%ld", frame);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the destination and source addresses of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Size of output arrays.
/// HIPAR destination/Returns the destination address of the selected frame.
/// HIPAR source/Returns the source address of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameAddresses(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt32   bufferSize,
                                                        ViChar    destination[],
                                                        ViChar    source[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DESTINATION,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DESTINATION,
                    bufferSize, destination, NULL),
                4, "Destination");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_SOURCE,
                    bufferSize, source, NULL),
                5, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the destination symbols of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR symbols/Returns the destination symbols of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameDestinationSymbols(
                            ViSession instrSession,
                            ViInt32   serialBus,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    symbols[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DESTINATION_SYMBOLS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_DESTINATION_SYMBOLS,
                    bufferSize, symbols, NULL),
                4, "Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the source symbols of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR symbols/Returns the source symbols of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameSourceSymbols(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt32   bufferSize,
                                                            ViChar    symbols[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_SOURCE_SYMBOLS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_SOURCE_SYMBOLS,
                    bufferSize, symbols, NULL),
                4, "Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the primary bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR bitRate/Returns the primary bit rate.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameBitRate(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   frame,
                                                      ViInt64*  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_BIT_RATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_BIT_RATE,
                    bitRate),
                4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the type of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameType/Returns the type of the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameType(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_TYPE, frameType),
            4, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words for the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR numberOfWords/Returns the number of decoded words for the selected frame.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameNumberOfWords(ViSession instrSession,
                                                            ViInt32   serialBus,
                                                            ViInt32   frame,
                                                            ViInt64*  numberOfWords)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_NUMBER_OF_WORDS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_NUMBER_OF_WORDS,
                    numberOfWords),
                4, "Number Of Words");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data type for the selected word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Size of output array.
/// HIPAR type/Returns the data type for the selected word.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameWordType(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt64   word,
                                                       ViInt32   bufferSize,
                                                       ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_WORD_TYPE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")
    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD,
                    word),
                4, "Word")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_WORD_TYPE,
                    bufferSize, type, NULL),
                5, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data value for the selected word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/This control selects the word.
/// HIPAR value/Returns the data value for the selected word.
ViStatus _VI_FUNC rsscope_Query1000BASET1FrameWordValue(ViSession instrSession,
                                                        ViInt32   serialBus,
                                                        ViInt32   frame,
                                                        ViInt64   word,
                                                        ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_WORD_VALUE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")
    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_SELECTED_WORD,
                    word),
                4, "Word")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_1000BASE_T1_FRAME_WORD_VALUE, value),
            5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of frame to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameType/Selects the type of frame to be searched for.
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchType(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_TYPE, frameType),
            3, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for the destination address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the destination address. You can define an
/// HIPAR condition/exact value or a value range.
/// HIPAR destinationAddressMin/Specifies the destination address bit pattern, or sets the start value
/// HIPAR destinationAddressMin/of a pattern range. The string parameter accepts the bit value X (do
/// HIPAR destinationAddressMin/not care).
/// HIPAR destinationAddressMax/Sets the end value of the destination address range if Condition is
/// HIPAR destinationAddressMax/set to In Range or Out Of Range. The string parameter accepts the bit
/// HIPAR destinationAddressMax/value X (do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchDestinationAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  destinationAddressMin,
                            ViString  destinationAddressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DESTINATION_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DESTINATION_ADDRESS_MIN, destinationAddressMin),
            4, "Destination Address Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DESTINATION_ADDRESS_MAX, destinationAddressMax),
                5, "Destination Address Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for the source address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the source address. You can define an exact
/// HIPAR condition/value or a value range.
/// HIPAR sourceAddressMin/Specifies the source address bit pattern, or sets the start value of a
/// HIPAR sourceAddressMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR sourceAddressMin/care).
/// HIPAR sourceAddressMax/Sets the end value of the source address range if Condition is set to
/// HIPAR sourceAddressMax/In Range or Out Of Range. The string parameter accepts the bit value X
/// HIPAR sourceAddressMax/(do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchSourceAddress(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  sourceAddressMin,
                            ViString  sourceAddressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_SOURCE_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_SOURCE_ADDRESS_MIN, sourceAddressMin),
            4, "Source Address Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_SOURCE_ADDRESS_MAX, sourceAddressMax),
                5, "Source Address Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for the length/type field.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the length/type. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR length/typeMin/Specifies the length/ type bit pattern, or sets the start value of a
/// HIPAR length/typeMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR length/typeMin/care).
/// HIPAR length/typeMax/Sets the end value of the length/type range if Condition is set to In
/// HIPAR length/typeMax/Range or Out Of Range. The string parameter accepts the bit value X (do
/// HIPAR length/typeMax/not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchLengthType(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  lengthTypeMin,
                            ViString  lengthTypeMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_LENGTH_TYPE_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_LENGTH_TYPE_MIN, lengthTypeMin),
            4, "Length/Type Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_LENGTH_TYPE_MAX, lengthTypeMax),
                5, "Length/Type Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for the frame check (CRC).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the frame check. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR crcMin/Specifies the frame check bit pattern, or sets the start value of a
/// HIPAR crcMin/pattern range. The string parameter accepts the bit value X (do not
/// HIPAR crcMin/care).
/// HIPAR crcMax/Sets the end value of the frame check range if Condition is set to In
/// HIPAR crcMax/Range or Out Of Range. The string parameter accepts the bit value X (do
/// HIPAR crcMax/not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchFrameCheckCRC(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  crcMin,
                            ViString  crcMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_CHECK_CRC_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_CHECK_CRC_MIN, crcMin),
            4, "CRC Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_CHECK_CRC_MAX, crcMax),
                5, "CRC Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for frame data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the condition for the data. You can define an exact value or a
/// HIPAR condition/value range.
/// HIPAR dataMin/Specifies the data bit pattern, or sets the start value of a pattern
/// HIPAR dataMin/range. The string parameter accepts the bit value X (do not care).
/// HIPAR dataMax/Sets the end value of the data range if Condition is set to In Range
/// HIPAR dataMax/or Out Of Range. The string parameter accepts the bit value X (do not
/// HIPAR dataMax/care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchData(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   condition,
                                                        ViString  dataMin,
                                                        ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DATA_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DATA_MIN, dataMin),
            4, "Data Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_DATA_MAX, dataMax),
                5, "Data Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for the index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the search condition for the index. You can define an exact value
/// HIPAR condition/or a value range.
/// HIPAR indexMin/Specifies the index bit pattern, or sets the start value of a pattern
/// HIPAR indexMin/range. The string parameter accepts the bit value X (do not care).
/// HIPAR indexMax/Sets the end value of the index range to be searched for, if Condition
/// HIPAR indexMax/is set to In Range or Out Of Range. The string parameter accepts the
/// HIPAR indexMax/bit value X (do not care).
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchIndex(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   condition,
                                                         ViInt32   indexMin,
                                                         ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_INDEX_MIN, indexMin),
            4, "Index Min");
    switch (condition)
    {
    case RSSCOPE_VAL_COND_IN:
    case RSSCOPE_VAL_COND_OUT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_INDEX_MAX, indexMax),
                5, "Index Max");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the search for errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR rsFEC/Enables search for RS-FEC errors.
/// HIPAR crc/Enables search for cyclic redundancy check (CRC) error.
/// HIPAR outOfRange/Enables search for values that are out of range.
/// HIPAR zeroTernarySymbol/Enables search for zero ternary symbol pairs error.
ViStatus _VI_FUNC rsscope_Configure1000BASET1SearchErrors(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViBoolean rsFEC,
                                                          ViBoolean crc,
                                                          ViBoolean outOfRange,
                                                          ViBoolean zeroTernarySymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_ERROR_RS_FEC, rsFEC),
            3, "RS-FEC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_ERROR_CRC, crc),
            4, "CRC");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_ERROR_OUT_OF_RANGE, outOfRange),
            5, "Out Of Range");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_ERROR_ZERO_TERNARY_SYMBOL, zeroTernarySymbol),
            6, "Zero Ternary Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded frames within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frameCount/Returns the number of decoded frames within the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameCount(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_COUNT, frameCount),
            3, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the state of the frame within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR frameStatus/Returns the state of the frame within the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameStatus(ViSession instrSession,
                                                           ViString  searchName,
                                                           ViInt32   frame,
                                                           ViInt32*  frameStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_STATUS, frameStatus),
            4, "Frame Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the start time of the selected frame within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR startTime/Returns the start time of the selected frame within the search result.
/// HIPAR stopTime/Returns the end time of the selected frame within the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameTime(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViReal64* startTime,
                                                         ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_START_TIME, startTime),
            4, "Start Time");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_STOP_TIME, stopTime),
            5, "Stop Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the destination address of the selected frame within the
/// HIFN  search result.
// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR bufferSize/Size of output arrays.
/// HIPAR frame/Selects the frame.
/// HIPAR frameDestination/Returns the destination address of the selected frame within the
/// HIPAR frameDestination/search result.
/// HIPAR frameSource/Returns the source address of the selected frame within the search
/// HIPAR frameSource/result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameAddresses(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    frameDestination[],
                            ViChar    frameSource[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_DESTINATION,
                    bufferSize, frameDestination, NULL),
                4, "Frame Destination");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_SOURCE,
                    bufferSize, frameSource, NULL),
                5, "Frame Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the destination symbols of the selected frame within the
/// HIFN  search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR destinationSymbols/Returns the destination symbols of the selected frame within the
/// HIPAR destinationSymbols/search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameDestinationSymbols(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    destinationSymbols[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_DESTINATION_SYMBOLS,
                    bufferSize, destinationSymbols, NULL),
                4, "Destination Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the source symbols of the selected frame within the search
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR bufferSize/Size of output array.
/// HIPAR sourceSymbols/Returns the source symbols of the selected frame within the search
/// HIPAR sourceSymbols/result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameSourceSymbols(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   bufferSize,
                            ViChar    sourceSymbols[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_SOURCE_SYMBOLS,
                    bufferSize, sourceSymbols, NULL),
                4, "Source Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data for the selected frame, corresponding to the Data
/// HIFN  column in the decode results table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameDataBytes(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32   arraySize,
                            ViInt32   data[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   aux[RS_REPCAP_BUF_SIZE];
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    checkErr(RsCore_BuildAttrCommandString(instrSession,
        &g_RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_DATA_STRING,
        repCap,
        &cmd));
    snprintf(aux, RS_REPCAP_BUF_SIZE, "%ld", frame);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the CRC checksum of the selected frame within the search
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR frameCRC/Returns the CRC checksum of the selected frame within the search
/// HIPAR frameCRC/result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameCRC(ViSession instrSession,
                                                        ViString  searchName,
                                                        ViInt32   frame,
                                                        ViInt64*  frameCRC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_CRC,
                    frameCRC),
                4, "Frame CRC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the value of length / type field of the selected frame within
/// HIFN  the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR length/type/Returns the value of length / type field of the selected frame within
/// HIPAR length/type/the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameLengthType(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt32*  lengthType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_LENGTH_TYPE, lengthType),
            4, "Length/Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the type of frame for the selected frame within the search
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR frameType/Returns the type of frame for the selected frame within the search
/// HIPAR frameType/result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameType(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   frame,
                                                         ViInt32*  frameType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_TYPE, frameType),
            4, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words for the selected frame within the
/// HIFN  search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR numberOfWords/Returns the number of decoded words for the selected frame within the
/// HIPAR numberOfWords/search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameNumberOfWords(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt64*  numberOfWords)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")

    viCheckParm(rsscope_GetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_NUMBER_OF_WORDS,
                    numberOfWords),
                4, "Number Of Words");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data type of the selected word within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR word/Selects the word.
/// HIPAR bufferSize/Size of output array.
/// HIPAR wordType/Returns the data type of the selected word within the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameWordType(ViSession instrSession,
                                                             ViString  searchName,
                                                             ViInt32   frame,
                                                             ViInt64   word,
                                                             ViInt32   bufferSize,
                                                             ViChar    wordType[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")
    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD,
                    word),
                4, "Word")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_WORD_TYPE,
                    bufferSize, wordType, NULL),
                5, "Word Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data value of the selected word within the search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR word/Selects the word.
/// HIPAR value/Returns the data value of the selected word within the search result.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameWordValue(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt64   word,
                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")
    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD,
                    word),
                4, "Word")

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_WORD_VALUE, value),
            5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the string equivalent of data value for the selected cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR frame/Selects the frame.
/// HIPAR word/Selects the word.
/// HIPAR bufferSize/Size of output array.
/// HIPAR valueString/Returns the string equivalent of data value for the selected cell.
ViStatus _VI_FUNC rsscope_Query1000BASET1SearchFrameWordValueString(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   frame,
                            ViInt64   word,
                            ViInt32   bufferSize,
                            ViChar    valueString[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_FRAME,
                    frame),
                3, "Frame")
    viCheckParm(rsscope_SetAttributeViInt64(instrSession, NULL,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_SELECTED_WORD,
                    word),
                4, "Word")

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSSCOPE_ATTR_SEARCH_1000BASE_T1_FRAME_WORD_VALUE_STRING,
                    bufferSize, valueString, NULL),
                5, "Value String");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a destination file for the hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the file name when an image of the display is stored to a file
/// HIPAR name/rather than printed to a printer using the HCOPy:IMMediate command.
ViStatus _VI_FUNC rsscope_HardcopySetFileName(ViSession instrSession,
                                              ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_FILE_NAME, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a destination file for the hadcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR destination/Defines the output associated with device configuration.
/// HIPAR color/Defines the color mode for printing.
/// HIPAR pageOrientation/Defines the page orientation.
ViStatus _VI_FUNC rsscope_ConfigureHardcopyDevice(ViSession instrSession,
                                                  ViInt32   device,
                                                  ViInt32   destination,
                                                  ViBoolean color,
                                                  ViInt32   pageOrientation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, device, 1, 2),
            2, "Device");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, destination, RSSCOPE_VAL_HARDCOPY_DESTINATION_PNG, RSSCOPE_VAL_HARDCOPY_DESTINATION_PDF),
            3, "Destination");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION,
            "Destination", device- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Device");

    switch (destination)
    {
    case RSSCOPE_VAL_HARDCOPY_DESTINATION_PNG:
    case RSSCOPE_VAL_HARDCOPY_DESTINATION_JPG:
    case RSSCOPE_VAL_HARDCOPY_DESTINATION_BMP:
    case RSSCOPE_VAL_HARDCOPY_DESTINATION_TIFF:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION, RSSCOPE_VAL_HARDCOPY_DEVICE_MEM),
                3, "Destination");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_LANGUAGE_OUTPUT_FORMAT, destination),
                3, "Destination");
        break;

    case RSSCOPE_VAL_HARDCOPY_DESTINATION_PDF:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION, RSSCOPE_VAL_HARDCOPY_DEVICE_MEM),
                3, "Destination");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_LANGUAGE_OUTPUT_FORMAT, RSSCOPE_VAL_HARDCOPY_DEVICE_LANG_PDF),
                3, "Destination");
        break;

    case RSSCOPE_VAL_HARDCOPY_DESTINATION_CLP:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION, RSSCOPE_VAL_HARDCOPY_DEVICE_CLIPBOARD),
                3, "Destination");
        break;

    case RSSCOPE_VAL_HARDCOPY_DESTINATION_PRN:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION, RSSCOPE_VAL_HARDCOPY_DEVICE_PRINT),
                3, "Destination");

        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_COLOR, color),
                4, "Color");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_DEVICE_ORIENTATION, pageOrientation),
                5, "Page Orientation");
        break;

    default: break;
    }

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_COLOR, color),
            4, "Color");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inverts the background color. So you can print waveforms
/// HIFN  with normal waveform collors on white background.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Inverts the background color. So you can print waveforms with normal
/// HIPAR state/waveform collors on white background.
ViStatus _VI_FUNC rsscope_ConfigureHardcopyWhiteBackground(ViSession instrSession,
                                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_HARDCOPY_WHITE_BACKGROUND, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inverts the colors of the output, i.e. a dark waveform
/// HIFN  is printed on a white background.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR inverseColors/Inverts the colors of the output, i.e. a dark waveform is printed on a
/// HIPAR inverseColors/white background.
ViStatus _VI_FUNC rsscope_ConfigureHardcopyInverseColors(ViSession instrSession,
                                                         ViInt32   device,
                                                         ViBoolean inverseColors)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HARDCOPY_INVERSE_COLORS,
            "Destination", device- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Device");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_INVERSE_COLORS, inverseColors),
            3, "inverseColors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the default color set for printing of the
/// HIFN  display image.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR colorSet/Defines the default color set for printing of the display image.
ViStatus _VI_FUNC rsscope_ConfigureHardcopyPrintColorSet(ViSession instrSession,
                                                         ViInt32   device,
                                                         ViInt32   colorSet)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HARDCOPY_PRINT_COLOR_SET,
            "Destination", device- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Device");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_HARDCOPY_PRINT_COLOR_SET, colorSet),
            3, "Color Set");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function prints an image of the display to the printer or saves
/// HIFN  an image to a file or the clipboard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_HardcopyPrint(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_PRINT, NULL),
            1, "Session");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the output of the next display image to printer, file, or
/// HIFN  clipboard, depending on the RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION
/// HIFN  attribute.
/// HIFN     If the output is printed to a file, the file name used in the last
/// HIFN  saving process is automatically counted up to the next unused name.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_HARDCOPY_PRINT_NEXT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     HCOPy:IMMediate<m>:NEXT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_HardcopyPrintNext(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_PRINT_NEXT, NULL),
            1, "Session");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the list of printers under Windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control indicates the allocated memory in Printer List.
/// HIPAR printerList/This control returns list of printers available under  Windows NT
/// HIPAR printerList/system. Names are separated using commas (,).
ViStatus _VI_FUNC rsscope_HardcopyGetPrinterList(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    printerList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = printerList;
    ViInt32  count;

    checkErr(RsCore_LockSession(instrSession));

    count = 0;
    checkErr(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_GET_FIRST_PRINTER,
        RS_MAX_MESSAGE_BUF_SIZE, response));

    while (strlen(response) > 0)
    {
        count += (ViInt32)strlen(response);
        count++;
        if (count <= bufferSize)
            p2buf += sprintf(p2buf, "%s,", response);
        response[0] = '\0';
        checkErr(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_GET_NEXT_PRINTER,
            RS_MAX_MESSAGE_BUF_SIZE, response));
    }
    *--p2buf = '\0';

    if (count > bufferSize)
        error = count;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a configured printer. To determine which
/// HIFN  printers are installed, use the rsscope_HardcopyGetPrinterList
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR printerName/Selects a configured printer.
ViStatus _VI_FUNC rsscope_HardcopySetPrinter(ViSession instrSession,
                                             ViInt32   device,
                                             ViString  printerName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, device, 1, 2),
            2, "Device");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HARDCOPY_DEVICE_DESTINATION,
            "Destination", device- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Device");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HARDCOPY_SELECT_PRINTER, printerName),
            3, "Printer Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the audio signal type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR variant/Selects the audio signal type.
ViStatus _VI_FUNC rsscope_ConfigureAudioVariant(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   variant)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_VARIANT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_VARIANT, variant),
            3, "Variant");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the audio clock line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR waveformSource/Selects the source of the waveform.
/// HIPAR polarity/Selects line polarity.
/// HIPAR treshold/Sets the threshold value for the clock line SCLK.
ViStatus _VI_FUNC rsscope_ConfigureAudioClockLine(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   waveformSource,
                                                  ViInt32   polarity,
                                                  ViReal64  treshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_CLOCK_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CLOCK_SOURCE, waveformSource),
            3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CLOCK_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CLOCK_THRESHOLD, treshold),
            5, "Treshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the audio word select (WS) line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR waveformSource/Selects the source of the waveform.
/// HIPAR polarity/Selects line polarity.
/// HIPAR treshold/Sets the threshold value for the word select and FSYNC lines.
ViStatus _VI_FUNC rsscope_ConfigureAudioWSLine(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   waveformSource,
                                               ViInt32   polarity,
                                               ViReal64  treshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_WS_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_WS_SOURCE, waveformSource),
            3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_WS_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_WS_THRESHOLD, treshold),
            5, "Treshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the audio data line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR waveformSource/Selects the source of the waveform.
/// HIPAR polarity/Selects line polarity.
/// HIPAR treshold/Sets the threshold value for the data line.
ViStatus _VI_FUNC rsscope_ConfigureAudioDataLine(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   waveformSource,
                                                 ViInt32   polarity,
                                                 ViReal64  treshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_DATA_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, waveformSource, RSSCOPE_VAL_PROTOCOL_WAV_C1W1),
                3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_DATA_SOURCE, waveformSource),
            3, "Waveform Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_DATA_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_DATA_THRESHOLD, treshold),
            5, "Treshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures tresholds coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Enables or disables tresholds coupling.
/// HIPAR treshold/Sets the threshold value for all lines.
ViStatus _VI_FUNC rsscope_ConfigureAudioTresholdsCoupling(ViSession instrSession,
                                                          ViInt32   serialBus,
                                                          ViBoolean coupling,
                                                          ViReal64  treshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRESHOLDS_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRESHOLDS_COUPLING, coupling),
            3, "Coupling");

    if (coupling)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CLOCK_THRESHOLD, treshold),
                4, "Treshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures I2S settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR wordLength/Defines the number of bits in an audio data word.
/// HIPAR channelOrder/Defines if the left or the right channel is the first channel in the
/// HIPAR channelOrder/frame.
/// HIPAR bitOrder/Select if the data of the messages starts with msb or lsb.
/// HIPAR channelOffset/Sets the number of bits between the channel start and the start of the
/// HIPAR channelOffset/audio word. The setting is available for left-justified data format and
/// HIPAR channelOffset/TDM audio signals.
/// HIPAR channelLength/Sets the number of bits in a channel block for right-justified data
/// HIPAR channelLength/format and TDM audio signals.
ViStatus _VI_FUNC rsscope_ConfigureAudioI2SSettings(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   wordLength,
                                                    ViInt32   channelOrder,
                                                    ViInt32   bitOrder,
                                                    ViInt32   channelOffset,
                                                    ViInt32   channelLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_WORD_LENGTH,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_WORD_LENGTH, wordLength),
            3, "Word Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CHANNEL_ORDER, channelOrder),
            4, "Channel Order");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_BIT_ORDER, bitOrder),
            5, "Bit Order");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CHANNEL_OFFSET, channelOffset),
            6, "Channel Offset");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CHANNEL_LENGTH, channelLength),
            7, "Channel Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures I2S settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR TDMChannels/Sets the number of channels transmitted on the TDM audio line.
/// HIPAR frameOffset/Sets the number of bits between the frame start and the start of the
/// HIPAR frameOffset/first channel of a TDM audio line. Each FSYNC edge restarts the offset
/// HIPAR frameOffset/count.
/// HIPAR channelLength/Sets the number of bits in a channel block for right-justified data
/// HIPAR channelLength/format and TDM audio signals.
/// HIPAR channelOffset/Sets the number of bits between the channel start and the start of the
/// HIPAR channelOffset/audio word. The setting is available for left-justified data format and
/// HIPAR channelOffset/TDM audio signals.
/// HIPAR wordLength/Defines the number of bits in an audio data word.
/// HIPAR bitOrder/Select if the data of the messages starts with msb or lsb.
ViStatus _VI_FUNC rsscope_ConfigureAudioTDMSettings(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   TDMChannels,
                                                    ViInt32   frameOffset,
                                                    ViInt32   channelLength,
                                                    ViInt32   channelOffset,
                                                    ViInt32   wordLength,
                                                    ViInt32   bitOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TDM_CHANNELS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TDM_CHANNELS, TDMChannels),
            3, "TDM Channels");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_FRAME_OFFSET, frameOffset),
            4, "Frame Offset");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CHANNEL_LENGTH, channelLength),
            5, "Channel Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_CHANNEL_OFFSET, channelOffset),
            6, "Channel Offset");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_WORD_LENGTH, wordLength),
            7, "Word Length");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_BIT_ORDER, bitOrder),
            7, "Bit Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR conditionNumber/Specifies the condition number.
/// HIPAR channel/This control select the audio channel on which the instrument looks
/// HIPAR channel/for the specified data condition.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range..
/// HIPAR addressMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR addressMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR addressMax/Sets the end value of an data range if "Condition" "In range" and
/// HIPAR addressMax/"Out of range".
ViStatus _VI_FUNC rsscope_ConfigureAudioDataTrigger(ViSession instrSession,
                                                    ViInt32   conditionNumber,
                                                    ViInt32   channel,
                                                    ViInt32   condition,
                                                    ViString  addressMin,
                                                    ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, conditionNumber, 1, 4),
            2, "Condition Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL,
            "ConditionNumber", conditionNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Condition Number");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_DATA));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL, channel),
            3, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MIN, addressMin),
            5, "Address Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MAX, addressMax),
                6, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the window trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR conditionNumber/Specifies the condition number.
/// HIPAR channel/This control select the audio channel on which the instrument looks
/// HIPAR channel/for the specified data condition.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range..
/// HIPAR addressMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR addressMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR addressMax/Sets the end value of an data range if "Condition" "In range" and
/// HIPAR addressMax/"Out of range".
ViStatus _VI_FUNC rsscope_ConfigureAudioWindowTrigger(ViSession instrSession,
                                                      ViInt32   conditionNumber,
                                                      ViInt32   channel,
                                                      ViInt32   condition,
                                                      ViString  addressMin,
                                                      ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, conditionNumber, 1, 4),
            2, "Condition Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL,
            "ConditionNumber", conditionNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Condition Number");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_WINDOW));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL, channel),
            3, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MIN, addressMin),
            5, "Address Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MAX, addressMax),
                6, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the number of words that is used as time limit for
/// HIFN  the "Window" trigger type. The instrument triggers if the data
/// HIFN  condition is fulfilled on the same channel for the given number of
/// HIFN  subsequent frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowLength/Sets the number of words that is used as time limit for the "Window"
/// HIPAR windowLength/trigger type.
ViStatus _VI_FUNC rsscope_ConfigureAudioWindowTriggerLength(ViSession instrSession,
                                                            ViInt32   windowLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_WINDOW));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_WINDOW_LENGTH, windowLength),
            2, "Window Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR conditionNumber/Specifies the condition number.
/// HIPAR channel/This control select the audio channel on which the instrument looks
/// HIPAR channel/for the specified data condition.
/// HIPAR condition/Sets the operator to set a specific data pattern or a data pattern
/// HIPAR condition/range..
/// HIPAR addressMin/Sets a data pattern, or sets the start value of a data pattern
/// HIPAR addressMin/range. Enter the pattern MSB first and with Big endian byte order.
/// HIPAR addressMax/Sets the end value of an data range if "Condition" "In range" and
/// HIPAR addressMax/"Out of range".
ViStatus _VI_FUNC rsscope_ConfigureAudioConditionTrigger(ViSession instrSession,
                                                         ViInt32   conditionNumber,
                                                         ViInt32   channel,
                                                         ViInt32   condition,
                                                         ViString  addressMin,
                                                         ViString  addressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, conditionNumber, 1, 4),
            2, "Condition Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL,
            "ConditionNumber", conditionNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Condition Number");

    checkErr(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_COND));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL, channel),
            3, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_CONDITION, condition),
            4, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MIN, addressMin),
            5, "Address Min");

    if (condition == RSSCOPE_VAL_COND_IN || condition == RSSCOPE_VAL_COND_OUT)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_PATTERN_MAX, addressMax),
                6, "Address Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the edge of the WS or FSYNC signal as trigger
/// HIFN  condition. The instrument triggers on the first clock edge after the
/// HIFN  specified edge.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR slope/Sets the edge of the WS or FSYNC signal as trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureAudioWordSelectTrigger(ViSession instrSession,
                                                          ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_WSEL));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_WORD_SELECT_SLOPE, slope),
            2, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the error condition trigger,which triggers on
/// HIFN  irregularities between the WS or FSYNC edges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ConfigureAudioErrorConditionTrigger(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE, RSSCOPE_VAL_AUDIO_TRIGGER_ECON));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsscope_QueryAudioFrameCount(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_FRAME_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_FRAME_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/Returns the overall state of the frame.
/// HIPAR frameStart/Returns time of frame start.
/// HIPAR frameStop/Returns time of frame stop.
ViStatus _VI_FUNC rsscope_QueryAudioFrameStatus(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32   frame,
                                                ViInt32*  frameStatus,
                                                ViReal64* frameStart,
                                                ViReal64* frameStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K5|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:STAT?", serialBus, frame);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolAudioFrameStatus_Arr, frameStatus));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:STAR?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStart));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:STOP?", serialBus, frame);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, frameStop));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the overall state of the left or right channel
/// HIFN  of the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR channelStatus/Return the status of the left or right channel of the selected frame.
/// HIPAR channelValue/Return the data values of the left or right channel, respectively.
/// HIPAR channel/Selects the channel of the frame.
ViStatus _VI_FUNC rsscope_QueryAudioFrameChannelStatus(ViSession instrSession,
                                                       ViInt32   serialBus,
                                                       ViInt32   frame,
                                                       ViInt32   channel,
                                                       ViInt32*  channelStatus,
                                                       ViReal64* channelValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K5|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_LEFT, RSSCOPE_VAL_RIGHT),
            4, "channel");

    if (channel == RSSCOPE_VAL_LEFT)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:LEFT:STAT?", serialBus, frame);
        checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolAudioFrameStatus_Arr, channelStatus));
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:LEFT:VAL?", serialBus, frame);
        checkErr(RsCore_QueryViReal64(instrSession, cmd, channelValue));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:RIGH:STAT?", serialBus, frame);
        checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolAudioFrameStatus_Arr, channelStatus));
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:RIGH:VAL?", serialBus, frame);
        checkErr(RsCore_QueryViReal64(instrSession, cmd, channelValue));
    }

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the overall state of the indicated channel of
/// HIFN  the selected frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR frame/This control selects the frame.
/// HIPAR TDMStatus/Returns the state of the indicated channel of the selected frame.
/// HIPAR TDMValue/Return the data value of the indicated TDM channel.
/// HIPAR channelNumber/This control selects the TDM channel.
ViStatus _VI_FUNC rsscope_QueryAudioFrameTDMStatus(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   frame,
                                                   ViInt32   channelNumber,
                                                   ViInt32*  TDMStatus,
                                                   ViReal64* TDMValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K5|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 8),
            4, "Channel Number");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:TDM%ld:STAT?", serialBus, frame, channelNumber);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolAudioFrameStatus_Arr, TDMStatus));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:I2S:FRAM%ld:TDM%ld:VAL?", serialBus, frame, channelNumber);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, TDMValue));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the track of the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables or disables the track of the indicated channel.
/// HIPAR channel/This control select the channel.
ViStatus _VI_FUNC rsscope_ConfigureAudioTrackWaveform(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   channel,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_TDM_CHANNEL_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_AUDIO_CHANNEL_TDMC1, RSSCOPE_VAL_AUDIO_CHANNEL_RIGHT),
            3, "Channel");

    switch (channel)
    {
    case RSSCOPE_VAL_AUDIO_CHANNEL_LEFT:
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_LEFT_ENABLED, state),
                4, "State");
        break;
    case RSSCOPE_VAL_AUDIO_CHANNEL_RIGHT:
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_RIGHT_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_TDM_CHANNEL_ENABLED,
                "TDMChannel", channel- 1, RS_REPCAP_BUF_SIZE, repCap_channel),
                3, "Channel");
        strcat(repCap, ",");
        strcat(repCap, repCap_channel);
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_TDM_CHANNEL_ENABLED, state),
                4, "State");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the track of the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR state/Enables or disables the track of the indicated channel.
ViStatus _VI_FUNC rsscope_ConfigureAudioTrackEnabled(ViSession instrSession,
                                                     ViInt32   measurement,
                                                     ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_ENABLED,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_ENABLED, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of track points for transmission from
/// HIFN  the instrument to the controlling computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the data of track points for transmission from the instrument
/// HIPAR waveformArray/to the controlling computer.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchAudioTrackWaveform(ViSession instrSession,
                                                  ViInt32   measurement,
                                                  ViInt32   waveformSize,
                                                  ViReal64  waveformArray[],
                                                  ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:TRAC:DATA?", measurement);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, waveformSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of the track.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR referenceWaveformHeader/Returns the header of the track.
ViStatus _VI_FUNC rsscope_FetchAudioTrackWaveformHeader(ViSession instrSession,
                                                        ViInt32   measurement,
                                                        ViInt32   bufferSize,
                                                        ViChar    referenceWaveformHeader[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TRACK_WAVEFORM_HEADER,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_TRACK_WAVEFORM_HEADER,
            bufferSize, referenceWaveformHeader),
            4, "Reference Waveform Header");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR dataType/Returns the data type.
ViStatus _VI_FUNC rsscope_FetchAudioTrackWaveformDataType(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   bufferSize,
                                                          ViChar    dataType[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_HEADER,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_DATA_TYPE,
            bufferSize, dataType),
            4, "Data Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the channel for the signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Sets the channel for the signal source.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553Source(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the wire on which the bus signal is measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Selects the wire on which the bus signal is measured.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553Polarity(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_POLARITY, polarity),
            3, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold values used for digitization of the
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets the default threshold voltage.
/// HIPAR thresholdHigh/Sets the lower threshold level of the signal.
/// HIPAR thresholdLow/Sets the lower threshold level of the signal.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553Threshold(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   preset,
                                                    ViReal64  thresholdHigh,
                                                    ViReal64  thresholdLow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_THRESHOLD_PRESET, preset),
            3, "Preset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_THRESHOLD_HIGH, thresholdHigh),
            4, "Threshold High");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_THRESHOLD_LOW, thresholdLow),
            5, "Threshold Low");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ranges of the timeouts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR minGapState/Enables the selection of the min gap.
/// HIPAR minGap/Sets a value for the min timing gap between two words.
/// HIPAR maxResponseState/Enables the selection of the max gap.
/// HIPAR maxResponse/Sets the value for the maximum response time.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553Timing(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViBoolean minGapState,
                                                 ViReal64  minGap,
                                                 ViBoolean maxResponseState,
                                                 ViReal64  maxResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_MIN_GAP_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_MIN_GAP_ENABLED, minGapState),
            3, "Min Gap State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_MIN_GAP_TIME, minGap),
            4, "Min Gap");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_MAX_RESPONSE_ENABLED, maxResponseState),
            5, "Max Response State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_MAX_RESPONSE_TIME, maxResponse),
            6, "Max Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for MIL-1553 analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the trigger type for MIL-1553 analysis.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerType(ViSession instrSession,
                                                      ViInt32   trigger,
                                                      ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the sync impulse/ word type to be triggered
/// HIFN  on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the sync impulse/ word type to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerSyncType(ViSession instrSession,
                                                          ViInt32   trigger,
                                                          ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_SYNC_TYPE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_SYNC_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 data trigger remote terminal address
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific remote terminal address.
/// HIPAR rtaMin/Specifies a remote terminal address or sets the start value of a
/// HIPAR rtaMin/remote terminal address range.
/// HIPAR rtaMax/Sets the end value of a data range if "Condition" is set to InRange or
/// HIPAR rtaMax/OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerDataRTAPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_DATA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_DATA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_DATA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger data pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific data pattern.
/// HIPAR dataMin/Specifies a data pattern or sets the start value of a data pattern
/// HIPAR dataMin/range.
/// HIPAR dataMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR dataMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerDataPattern(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViInt32   condition,
                                                             ViString  dataMin,
                                                             ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_PATTERN_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_PATTERN_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger data index settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific range within this series of the
/// HIPAR condition/data words that is considered for the analysis.
/// HIPAR indexMin/Specifies an index or sets the start value of a data word series
/// HIPAR indexMin/index range.
/// HIPAR indexMax/Sets the end value of a data word series index range if "Data Index
/// HIPAR indexMax/Condition" is set to InRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerDataIndex(ViSession instrSession,
                                                           ViInt32   trigger,
                                                           ViInt32   condition,
                                                           ViInt32   indexMin,
                                                           ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_INDEX_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_INDEX_MIN, indexMin),
            4, "Index Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_DATA_INDEX_MAX, indexMax),
                5, "Index Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger remote terminal address
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific remote terminal address.
/// HIPAR rtaMin/Specifies a remote terminal address or sets the start value of a
/// HIPAR rtaMin/remote terminal address range.
/// HIPAR rtaMax/Sets the end value of a data range if "RTA Condition" is set to
/// HIPAR rtaMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandStatusRTAPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_RTA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger 11-Bit Information settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific info for the 9th to 19th bit of a
/// HIPAR condition/command or status word.
/// HIPAR infoMin/Specifies an info or sets the start value of an info range.
/// HIPAR infoMax/Sets the end value of the info range if "Info Condition" is set to
/// HIPAR infoMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandStatusInfoPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  infoMin,
                            ViString  infoMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_INFO_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_INFO_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_INFO_PATTERN_MIN, infoMin),
            4, "Info Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_INFO_PATTERN_MAX, infoMax),
                5, "Info Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a transmission mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR direction/Sets transmission direction.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandTransmissionDirection(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_TRANSMIT_RECEIVE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_TRANSMIT_RECEIVE, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger command RTA settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to define a remote terminal address.
/// HIPAR rtaMin/Specify a remote terminal address or set the start value of a
/// HIPAR rtaMin/remote terminal address range.
/// HIPAR rtaMax/Set the end value of a data range  if "Subaddress Condition" is set to
/// HIPAR rtaMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandRTAPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_RTA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_RTA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_RTA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_COMMAND_RTA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger subaddress settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific subaddress/mode pattern.
/// HIPAR subaddressMin/Specifies a subaddress or sets the start value of a subaddress
/// HIPAR subaddressMin/range.
/// HIPAR subaddressMax/Sets the end value of the info subaddress if "Subaddress Condition" is
/// HIPAR subaddressMax/set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandSubaddressPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  subaddressMin,
                            ViString  subaddressMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_SUBADDRESS_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_SUBADDRESS_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_SUBADDRESS_PATTERN_MIN, subaddressMin),
            4, "Subaddress Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_SUBADDRESS_PATTERN_MAX, subaddressMax),
                5, "Subaddress Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIL1553 trigger word count/ mode code
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific data word count or mode code
/// HIPAR condition/pattern.
/// HIPAR wordCountMin/Specifies a data word count/mode code pattern or sets the start
/// HIPAR wordCountMin/value of a pattern range.
/// HIPAR wordCountMax/Sets the end value of a data word count/mode code pattern if "Word
/// HIPAR wordCountMax/Count Condition" is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerCommandWordCountPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  wordCountMin,
                            ViString  wordCountMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_WORD_COUNT_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_WORD_COUNT_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_WORD_COUNT_PATTERN_MIN, wordCountMin),
            4, "Word Count Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_WORD_COUNT_PATTERN_MAX, wordCountMax),
                5, "Word Count Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the values of the status flags.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR messageError/Triggers on the state of the message error bit of the status word if
/// HIPAR messageError/"Trigger Mode" is set to Status Word.
/// HIPAR busy/Triggers on the state of the busy bit of the status word if "Trigger
/// HIPAR busy/Mode" is set to Status Word.
/// HIPAR instrumentation/Triggers on the state of the instrumentation bit of the status word if
/// HIPAR instrumentation/"Trigger Mode" is set to StatusWord.
/// HIPAR subsystem/Triggers on the state of the subsystem flag bit of the status word if
/// HIPAR subsystem/"Trigger Mode" is set to Status Word.
/// HIPAR serviceRequest/Triggers on the state of the service request bit of the status word if
/// HIPAR serviceRequest/"Trigger Mode" is set to Status Word.
/// HIPAR dynamicBusControl/Triggers on the state of the dynamic bus control accept bit of the
/// HIPAR dynamicBusControl/status word if "Trigger Mode" is set to Status Word.
/// HIPAR broadcastCommand/Triggers on the state of the broadcast command received bit of the
/// HIPAR broadcastCommand/status word if "Trigger Mode" is set to Status Word.
/// HIPAR terminal/Triggers on the state of the terminal flag bit of the status word if
/// HIPAR terminal/"Trigger Mode" is set to Status Word.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerStatusFlags(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViInt32   messageError,
                                                             ViInt32   busy,
                                                             ViInt32   instrumentation,
                                                             ViInt32   subsystem,
                                                             ViInt32   serviceRequest,
                                                             ViInt32   dynamicBusControl,
                                                             ViInt32   broadcastCommand,
                                                             ViInt32   terminal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_MESSAGE_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_MESSAGE_ERROR, messageError),
            3, "Message Error");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_BUSY, busy),
            4, "Busy");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_INSTRUMENTATION, instrumentation),
            5, "Instrumentation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_SUBSYSTEM, subsystem),
            6, "Subsystem");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_SERVICE_REQUEST, serviceRequest),
            7, "Service Request");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_DYNAMIC_BUS_CONTROL, dynamicBusControl),
            8, "Dynamic Bus Control");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_BROADCAST_COMMAND, broadcastCommand),
            9, "Broadcast Command");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_STATUS_FLAG_TERMINAL, terminal),
            10, "Tterminal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the error conditions to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR synchronizationError/Triggers on an error of the synchronization.
/// HIPAR manchesterCodingError/Triggers if there is an error in the Manchester coding of the signal.
/// HIPAR parityError/Checks the parity of every word and triggers if the parity is even.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerErrorCondition(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean synchronizationError,
                            ViBoolean manchesterCodingError,
                            ViBoolean parityError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_SYNCHRONIZATION_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_SYNCHRONIZATION_ERROR, synchronizationError),
            3, "Synchronization Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_MANCHESTER_CODING_ERROR, manchesterCodingError),
            4, "Manchester Coding Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_PARITY_ERROR, parityError),
            5, "Parity Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the timeouts to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR minGapState/Enables triggering when the minimum gap is out of range.
/// HIPAR minGap/Sets the value for the minimum gap to be triggered.
/// HIPAR maxResponseState/Enables triggering when the maximum gap is out of range.
/// HIPAR maxResponse/Sets the value for the maximum gap to be triggered.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553TriggerErrorTiming(ViSession instrSession,
                                                             ViInt32   trigger,
                                                             ViBoolean minGapState,
                                                             ViReal64  minGap,
                                                             ViBoolean maxResponseState,
                                                             ViReal64  maxResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_TIMING_MIN_GAP_ENABLED,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_TIMING_MIN_GAP_ENABLED, minGapState),
            3, "Min Gap State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_TIMING_MIN_GAP_TIME, minGap),
            4, "Min Gap");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_TIMING_MAX_RESPONSE_ENABLED, maxResponseState),
            5, "Max Response State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_TRIGGER_ERROR_TIMING_MAX_RESPONSE, maxResponse),
            6, "Max Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded words.
ViStatus _VI_FUNC rsscope_QueryMIL1553WordCount(ViSession instrSession,
                                                ViInt32   serialBus,
                                                ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_MIL1553_DECODING_RESULTS_WORDS_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_MIL1553_DECODING_RESULTS_WORDS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR status/Returns the overall state of the selected word.
/// HIPAR start/Return the start time of the selected word.
/// HIPAR stop/Return the stop time of the selected word.
/// HIPAR type/Returns the type of the specified word.
/// HIPAR info/Returns the info value for the specified word.
/// HIPAR rtAddress/Returns the RT address for the selected word.
ViStatus _VI_FUNC rsscope_QueryMIL1553Status(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   word,
                                             ViInt32*  status,
                                             ViReal64* start,
                                             ViReal64* stop,
                                             ViInt32*  type,
                                             ViInt32*  info,
                                             ViInt32*  rtAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:STAT?", serialBus, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMILStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:STAR?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:STOP?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:TYPE?", serialBus, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMILType_Arr, type));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:INFO?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, info));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:RTAD?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, rtAddress));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryMIL1553AllData(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   word,
                                              ViInt32   arraySize,
                                              ViInt32   data[],
                                              ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:DATA?", serialBus, word);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the label name of the word ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/Returns the label name of the word ID.
ViStatus _VI_FUNC rsscope_QueryMIL1553Label(ViSession instrSession,
                                            ViInt32   serialBus,
                                            ViInt32   word,
                                            ViInt32   bufferSize,
                                            ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:MILS:WORD%ld:SYMB?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the event to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the event to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchType(ViSession instrSession,
                                                     ViString  searchName,
                                                     ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the sync impulse/ word type to be triggered
/// HIFN  on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR type/Sets the sync impulse/ word type to be searched for.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchSyncType(ViSession instrSession,
                                                         ViString  searchName,
                                                         ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_SYNC_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures MIL1553 trigger search data pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data pattern.
/// HIPAR dataMin/Sets the pattern or the start value of a pattern range for the
/// HIPAR dataMin/corresponding data pattern in a data word search.
/// HIPAR dataMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR dataMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchDataPattern(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   condition,
                                                            ViString  dataMin,
                                                            ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_PATTERN_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_PATTERN_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures MIL1553 search data index settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a range within a series of the data words
/// HIPAR condition/that is considered for the search.
/// HIPAR indexMin/Specifies an index or sets the start value of a data word series
/// HIPAR indexMin/index range.
/// HIPAR indexMax/Sets the end value of a data word series index range if the operator
/// HIPAR indexMax/is set to InRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchDataIndex(ViSession instrSession,
                                                          ViString  searchName,
                                                          ViInt32   condition,
                                                          ViInt32   indexMin,
                                                          ViInt32   indexMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_INDEX_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_INDEX_MIN, indexMin),
            4, "Index Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_DATA_INDEX_MAX, indexMax),
                5, "Index Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures MIL1553 remote terminal address search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific remote terminal address.
/// HIPAR rtaMin/Specifies a remote terminal address or sets the start value of a
/// HIPAR rtaMin/remote terminal address range.
/// HIPAR rtaMax/Sets the end value of a data range if "RTA Condition" is set to
/// HIPAR rtaMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchDataRTAPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_RTA_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_RTA_DATA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_RTA_DATA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches MIL1553 remote terminal address settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific remote terminal address.
/// HIPAR rtaMin/Specifies a remote terminal address or sets the start value of a
/// HIPAR rtaMin/remote terminal address range.
/// HIPAR rtaMax/Sets the end value of a data range if "RTA Condition" is set to
/// HIPAR rtaMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandStatusRTAPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_STATUS_RTA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_STATUS_RTA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_STATUS_RTA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures MIL1553 trigger 11-Bit Information settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific info for the 9th to 19th bit of a
/// HIPAR condition/command or status word.
/// HIPAR infoMin/Sets the pattern or the start value of a pattern range for the
/// HIPAR infoMin/corresponding search.
/// HIPAR infoMax/Sets the end value of the info range if "Info Condition" is set to
/// HIPAR infoMax/InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandStatusInfoPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  infoMin,
                            ViString  infoMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_INFO_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_INFO_PATTERN_MIN, infoMin),
            4, "Info Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_INFO_PATTERN_MAX, infoMax),
                5, "Info Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches for a transmission mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR direction/Sets transmission direction.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandTransmissionDirection(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_TRANSMIT_RECEIVE, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches MIL1553 trigger subaddress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific subaddress/mode pattern.
/// HIPAR subaddressMin/Specifies a subaddress or sets the start value of a subaddress
/// HIPAR subaddressMin/range.
/// HIPAR subaddressMax/Sets the end value of the info subaddress if "Subaddress Condition" is
/// HIPAR subaddressMax/set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandRTAPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  rtaMin,
                            ViString  rtaMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_RTA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_RTA_PATTERN_MIN, rtaMin),
            4, "RTA Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_RTA_PATTERN_MAX, rtaMax),
                5, "RTA Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches MIL1553 trigger subaddress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific subaddress/mode pattern.
/// HIPAR subaddressMin/Specifies a subaddress or sets the start value of a subaddress
/// HIPAR subaddressMin/range.
/// HIPAR subaddressMax/Sets the end value of the info subaddress if "Subaddress Condition" is
/// HIPAR subaddressMax/set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandSubaddressPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  subaddressMin,
                            ViString  subaddressMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_SUBADDRESS_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_SUBADDRESS_PATTERN_MIN, subaddressMin),
            4, "Subaddress Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_SUBADDRESS_PATTERN_MAX, subaddressMax),
                5, "Subaddress Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches MIL1553 trigger word count/ mode code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR condition/Sets the operator to set a specific data word count or mode code
/// HIPAR condition/pattern.
/// HIPAR wordCountMin/Specifies a data word count/mode code pattern or sets the start
/// HIPAR wordCountMin/value of a pattern range.
/// HIPAR wordCountMax/Sets the end value of a data word count/mode code pattern if "Word
/// HIPAR wordCountMax/Count Condition" is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchCommandWordCountPattern(
                            ViSession instrSession,
                            ViString  searchName,
                            ViInt32   condition,
                            ViString  wordCountMin,
                            ViString  wordCountMax)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_WORD_COUNT_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_WORD_COUNT_PATTERN_MIN, wordCountMin),
            4, "Word Count Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_COMMAND_WORD_COUNT_PATTERN_MAX, wordCountMax),
                5, "Word Count Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches the values of the status flags.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR messageError/Searches for the state of the message error bit of the status word if
/// HIPAR messageError/"Trigger Mode" is set to Status Word.
/// HIPAR busy/Searches for the state of the busy bit of the status word if "Trigger
/// HIPAR busy/Mode" is set to Status Word.
/// HIPAR instrumentation/Searches for the state of the instrumentation bit of the status word
/// HIPAR instrumentation/if "Trigger Mode" is set to StatusWord.
/// HIPAR subsystem/Searches for the state of the subsystem flag bit of the status word if
/// HIPAR subsystem/"Trigger Mode" is set to Status Word.
/// HIPAR serviceRequest/Searches for the state of the service request bit of the status word
/// HIPAR serviceRequest/if "Trigger Mode" is set to Status Word.
/// HIPAR dynamicBusControl/Searches for the state of the dynamic bus control accept bit of the
/// HIPAR dynamicBusControl/status word if "Trigger Mode" is set to Status Word.
/// HIPAR broadcastCommand/Searches for the state of the broadcast command received bit of the
/// HIPAR broadcastCommand/status word if "Trigger Mode" is set to Status Word.
/// HIPAR terminal/Searches for the state of the terminal flag bit of the status word if
/// HIPAR terminal/"Trigger Mode" is set to Status Word.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchStatusFlags(ViSession instrSession,
                                                            ViString  searchName,
                                                            ViInt32   messageError,
                                                            ViInt32   busy,
                                                            ViInt32   instrumentation,
                                                            ViInt32   subsystem,
                                                            ViInt32   serviceRequest,
                                                            ViInt32   dynamicBusControl,
                                                            ViInt32   broadcastCommand,
                                                            ViInt32   terminal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_MESSAGE_ERROR, messageError),
            3, "Message Error");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_BUSY, busy),
            4, "Busy");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_INSTRUMENTATION, instrumentation),
            5, "Instrumentation");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_SUBSYSTEM, subsystem),
            6, "Subsystem");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_SERVICE_REQUEST, serviceRequest),
            7, "Service Request");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_DYNAMIC_BUS_CONTROL, dynamicBusControl),
            8, "Dynamic Bus Control");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_BROADCAST_COMMAND, broadcastCommand),
            9, "Broadcast Command");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_STATUS_FLAG_TERMINAL, terminal),
            10, "Tterminal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the error conditions to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR synchronizationError/Triggers on an error of the synchronization.
/// HIPAR manchesterCodingError/Triggers if there is an error in the Manchester coding of the signal.
/// HIPAR parityError/Checks the parity of every word and triggers if the parity is even.
/// HIPAR timing/Checks if the minimum gap is out of range.
ViStatus _VI_FUNC rsscope_ConfigureMIL1553SearchErrorCondition(
                            ViSession instrSession,
                            ViString  searchName,
                            ViBoolean synchronizationError,
                            ViBoolean manchesterCodingError,
                            ViBoolean parityError,
                            ViBoolean timing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_ERROR_SYNCHRONIZATION_ERROR, synchronizationError),
            3, "Synchronization Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_ERROR_MANCHESTER_CODING_ERROR, manchesterCodingError),
            4, "Manchester Coding Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_ERROR_PARITY_ERROR, parityError),
            5, "Parity Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_ERROR_TIMING_ERROR, timing),
            6, "Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of decoded words.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR count/Returns the number of decoded words.
ViStatus _VI_FUNC rsscope_QueryMIL1553SearchWordCount(ViSession instrSession,
                                                      ViString  searchName,
                                                      ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SELECTED_SEARCH_NAME, searchName),
            2, "Search Name");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SEARCH_MIL1553_WORDS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR status/Returns the overall state of the selected word.
/// HIPAR start/Return the start time of the selected word.
/// HIPAR stop/Return the stop time of the selected word.
/// HIPAR type/Returns the type of the specified word.
/// HIPAR info/Returns the info value for the specified word.
/// HIPAR rtAddress/Returns the RT address for the selected word.
ViStatus _VI_FUNC rsscope_QueryMIL1553SearchStatus(ViSession instrSession,
                                                   ViString  searchName,
                                                   ViInt32   word,
                                                   ViInt32*  status,
                                                   ViReal64* start,
                                                   ViReal64* stop,
                                                   ViInt32*  type,
                                                   ViInt32*  info,
                                                   ViInt32*  rtAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:STAT? '%s'", word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMILStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:STAR? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:STOP? '%s'", word, searchName);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:TYPE? '%s'", word, searchName);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolMILType_Arr, type));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:INFO? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, info));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:RTAD? '%s'", word, searchName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, rtAddress));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryMIL1553SearchAllData(ViSession instrSession,
                                                    ViString  searchName,
                                                    ViInt32   word,
                                                    ViInt32   arraySize,
                                                    ViInt32   data[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:DATA? '%s'", word, searchName);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the label name of the word ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchName/Specifies search definition.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR label/Returns the label name of the word ID.
ViStatus _VI_FUNC rsscope_QueryMIL1553SearchLabel(ViSession instrSession,
                                                  ViString  searchName,
                                                  ViInt32   word,
                                                  ViInt32   bufferSize,
                                                  ViChar    label[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SEAR:RES:MILS:WORD%ld:SYMB? '%s'", word, searchName);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(label, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the channel for the signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Sets the channel for the signal source.
ViStatus _VI_FUNC rsscope_ConfigureARINC429Source(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_SOURCE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the wire on which the bus signal is measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Sets the wire on which the bus signal is measured.
ViStatus _VI_FUNC rsscope_ConfigureARINC429Polarity(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_POLARITY,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_POLARITY, polarity),
            3, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRateMode/Sets the bit rate mode to high or low speed.
/// HIPAR bitRateValue/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsscope_ConfigureARINC429BitRate(ViSession instrSession,
                                                   ViInt32   serialBus,
                                                   ViInt32   bitRateMode,
                                                   ViInt32   bitRateValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_BIT_RATE_MODE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_BIT_RATE_MODE, bitRateMode),
            3, "Bit Rate Mode");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_BIT_RATE, bitRateValue),
            4, "Bit Rate Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures threshold values used for digitization of the
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR preset/Sets the default threshold voltage.
/// HIPAR thresholdHigh/Sets the high threshold level of the signal.
/// HIPAR thresholdLow/Sets the lower threshold level of the signal.
ViStatus _VI_FUNC rsscope_ConfigureARINC429Threshold(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   preset,
                                                     ViReal64  thresholdHigh,
                                                     ViReal64  thresholdLow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_THRESHOLD_PRESET,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_THRESHOLD_PRESET, preset),
            3, "Preset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_THRESHOLD_HIGH, thresholdHigh),
            4, "Threshold High");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_THRESHOLD_LOW, thresholdLow),
            5, "Threshold Low");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ranges of the timeouts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR minGapState/Enables the selection of the min gap.
/// HIPAR minGap/Sets a value for the min timing gap between two words.
/// HIPAR maxGapState/Enables the selection of the max gap.
/// HIPAR maxGap/Sets the value for the maximum response time.
ViStatus _VI_FUNC rsscope_ConfigureARINC429Timing(ViSession instrSession,
                                                  ViInt32   serialBus,
                                                  ViBoolean minGapState,
                                                  ViInt32   minGap,
                                                  ViBoolean maxGapState,
                                                  ViInt32   maxGap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_MIN_GAP_ENABLED,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_MIN_GAP_ENABLED, minGapState),
            3, "Min Gap State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_MIN_GAP_BITS, minGap),
            4, "Min Gap");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_MAX_GAP_ENABLED, maxGapState),
            5, "Max Gap State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_MAX_GAP_BITS, maxGap),
            6, "Max Gap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for ARINC 429 analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR type/Sets the trigger type for ARINC 429 analysis.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerType(ViSession instrSession,
                                                       ViInt32   trigger,
                                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_MODE,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_MODE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the values for the source/destination
/// HIFN  identifier (SDI) and the sign/status matrix (SSM) bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR sdi/Sets the source/destination identifier (SDI) bits.
/// HIPAR ssm/Sets the sign/status matrix (SSM) bits.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerLabelData(ViSession instrSession,
                                                            ViInt32   trigger,
                                                            ViInt32   sdi,
                                                            ViInt32   ssm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_SDI,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_SDI, sdi),
            3, "SDI");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_SSM, ssm),
            4, "SSM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures ARINC 429 trigger label pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the operator to set a specific data pattern.
/// HIPAR labelMin/Specifies a data pattern or sets the start value of a data pattern
/// HIPAR labelMin/range.
/// HIPAR labelMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR labelMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerLabelPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  labelMin,
                            ViString  labelMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_MIN, labelMin),
            4, "Label Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_MAX, labelMax),
                5, "Label Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures ARINC 429 trigger data pattern settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR condition/Sets the condition for the data. You can define an exact data patterm
/// HIPAR condition/or a data range.
/// HIPAR dataMin/Specifies a data pattern or sets the start value of a data pattern
/// HIPAR dataMin/range.
/// HIPAR dataMax/Sets the end value of a data pattern range if "Data Pattern Condition"
/// HIPAR dataMax/is set to InRange or OutOfRange.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerDataPattern(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViInt32   condition,
                            ViString  dataMin,
                            ViString  dataMax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_CONDITION,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_MIN, dataMin),
            4, "Data Min");

    if (condition >= RSSCOPE_VAL_COND_IN)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_MAX, dataMax),
                5, "Data Max");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the error conditions to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR codingError/Enables triggering when a coding error occurs.
/// HIPAR parityError/Enables triggering when a parity error occurs.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerErrorCondition(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean codingError,
                            ViBoolean parityError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_CODING_ERROR,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_CODING_ERROR, codingError),
            3, "Coding Error");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_PARITY_ERROR, parityError),
            4, "Parity Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the timeouts to be triggered on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR minGapState/Enables triggering when the minimum gap is out of range.
/// HIPAR minGap/Sets the value for the minimum gap to be triggered on.
/// HIPAR maxResponseState/Enables triggering when the maximum gap is out of range.
/// HIPAR maxResponse/Sets the value for the maximum gap to be triggered on.
ViStatus _VI_FUNC rsscope_ConfigureARINC429TriggerErrorTiming(
                            ViSession instrSession,
                            ViInt32   trigger,
                            ViBoolean minGapState,
                            ViInt32   minGap,
                            ViBoolean maxResponseState,
                            ViInt32   maxResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trigger, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_TRIGGER_R),
            2, "Trigger");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_TIMING_MIN_GAP_ENABLED,
            "Trigger", trigger- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_TIMING_MIN_GAP_ENABLED, minGapState),
            3, "Min Gap State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_TIMING_MIN_GAP_BITS, minGap),
            4, "Min Gap");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_TIMING_MAX_GAP_ENABLED, maxResponseState),
            5, "Max Response State");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_TIMING_MAX_GAP_BITS, maxResponse),
            6, "Max Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of decoded words.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR count/Returns the number of decoded words.
ViStatus _VI_FUNC rsscope_QueryARINC429WordCount(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_PROTOCOL_ARINC429_DECODING_RESULTS_WORDS_COUNT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_PROTOCOL_ARINC429_DECODING_RESULTS_WORDS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR status/Returns the overall state of the selected word.
/// HIPAR start/Return the start time of the selected word.
/// HIPAR stop/Return the stop time of the selected word.
/// HIPAR pattern/Returns all 32 bits of the specified word.
/// HIPAR sdi/Returns the source/destination identifier (SDI) bits of the specified
/// HIPAR sdi/word.
/// HIPAR ssm/Returns the sign/status matrix (SSM) bits of the specified word.
ViStatus _VI_FUNC rsscope_QueryARINC429Status(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   word,
                                              ViInt32*  status,
                                              ViReal64* start,
                                              ViReal64* stop,
                                              ViInt32*  pattern,
                                              ViInt32*  sdi,
                                              ViInt32*  ssm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:STAT?", serialBus, word);
    checkErr(RsCore_QueryStringIndex(instrSession, cmd, protocolARINCStatus_Arr, status));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:STAR?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, start));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:STOP?", serialBus, word);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stop));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:PATT?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, pattern));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:SDI?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, sdi));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:SSM?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, ssm));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsscope_QueryARINC429AllData(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViInt32   word,
                                               ViInt32   arraySize,
                                               ViInt32   data[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:DATA?", serialBus, word);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the label name of the word ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Translation'.
/// HIPAR labelName/Returns the label name of the word ID.
/// HIPAR label/Returns the label of the specified word.
ViStatus _VI_FUNC rsscope_QueryARINC429Label(ViSession instrSession,
                                             ViInt32   serialBus,
                                             ViInt32   word,
                                             ViInt32   bufferSize,
                                             ViChar    labelName[],
                                             ViInt32*  label)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SET_QUERY_PATTERN_FORMAT, RSSCOPE_VAL_PATTERN_DEC));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:SYMB?", serialBus, word);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(labelName, bufferSize, extBuf);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:ARIN:WORD%ld:LAB?", serialBus, word);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, label));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the shutdown of the firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_FirmwareShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_FIRMWARE_SHUTDOWN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the computer name that is currently defined.
/// HIFN  The computer name is required when configuring a network.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR computerName/Configures the computer name that is currently defined. The computer
/// HIPAR computerName/name is required when configuring a network.
ViStatus _VI_FUNC rsscope_ConfigureComputerName(ViSession instrSession,
                                                ViString  computerName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_COMPUTER_NAME, computerName),
            2, "Computer Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the firmware version, material number and serial
/// HIFN  number .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firmwaveVersion/Returns the firmware version that is currently installed on the
/// HIPAR firmwaveVersion/instrument.
/// HIPAR materialNumber/Returns the material number of your instrument. This number is
/// HIPAR materialNumber/required to order a new option, and in case of service.
/// HIPAR serialNumber/Returns the serial number of your instrument. This number is required
/// HIPAR serialNumber/to order a new option, and in case of service.
ViStatus _VI_FUNC rsscope_QueryInstrumentInformation(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    firmwaveVersion[],
                                                     ViChar    materialNumber[],
                                                     ViChar    serialNumber[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_FIRMWARE_VERSION,
            bufferSize, firmwaveVersion),
            3, "Firmwave Version");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_MATERIAL_NUMBER,
            bufferSize, materialNumber),
            3, "Material Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_SERIAL_NUMBER,
            bufferSize, serialNumber),
            3, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the selftest.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_StartSelftest(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_SELFTEST, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the summary result of the selftest.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selftestStatus/Returns the summary result of the selftest.
ViStatus _VI_FUNC rsscope_SelftestStatus(ViSession instrSession,
                                         ViInt32*  selftestStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SELFTEST_STATUS, selftestStatus),
            2, "Selftest Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures an external application that can be started
/// HIFN  at various events, for example, trigger event, or mask test violation.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_NAME
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_PARAMETER
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_WORKING_DIRECTORY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EXECutable:NAME
/// HIFN     EXECutable:PARameter
/// HIFN     EXECutable:WDIRectory
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Sets the path to the application executable.
/// HIPAR parameter/Sets optional parameters for the external executable.
/// HIPAR workingDirectory/Sets the working directory for the executable.
ViStatus _VI_FUNC rsscope_ConfigureExternalApplication(ViSession instrSession,
                                                       ViString  name,
                                                       ViString  parameter,
                                                       ViString  workingDirectory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_EXTERNAL_APPLICATION_NAME, name),
            2, "Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_EXTERNAL_APPLICATION_PARAMETER, parameter),
            3, "Parameter");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_EXTERNAL_APPLICATION_WORKING_DIRECTORY, workingDirectory),
            4, "Working Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures a quick action.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_QUICK_ACTION_MODE
/// HIFN     RSSCOPE_ATTR_QUICK_ACTION_APPLICATION_PATH
/// HIFN     RSSCOPE_ATTR_QUICK_ACTION_PARAMETER
/// HIFN     RSSCOPE_ATTR_EXTERNAL_APPLICATION_WORKING_DIRECTORY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     QACTion:MODE
/// HIFN     QACTion:PATH
/// HIFN     QACTion:PARameters
/// HIFN     EXECutable:WDIRectory
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the action that is initiated by the Quick Action key.
/// HIPAR name/Sets the path to the application executable.
/// HIPAR parameter/Sets optional parameters for the external executable.
/// HIPAR workingDirectory/Sets the working directory for the executable.
ViStatus _VI_FUNC rsscope_ConfigureQuickAction(ViSession instrSession,
                                               ViInt32   mode,
                                               ViString  name,
                                               ViString  parameter,
                                               ViString  workingDirectory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_QUICK_ACTION_MODE, mode),
            2, "Mode");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_QUICK_ACTION_APPLICATION_PATH, name),
            3, "Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_QUICK_ACTION_PARAMETER, parameter),
            4, "Parameter");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_EXTERNAL_APPLICATION_WORKING_DIRECTORY, workingDirectory),
            5, "Working Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Tests the Quick Action setup.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_QUICK_ACTION_EXECUTE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     QACTion:EXECute
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ExecuteQuickAction(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_QUICK_ACTION_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and disables the remote trace to file.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables and disables the remote trace to file.
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceState(ViSession instrSession,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the remote trace is started.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_START_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:STARtmode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startMode/Defines how the remote trace is started.
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceStartMode(ViSession instrSession,
                                                        ViInt32   startMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_START_MODE, startMode),
            2, "Start Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines when the remote trace is stopped.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_STOP_MODE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:STOPmode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stopMode/Defines when the remote trace is stopped.
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceStopMode(ViSession instrSession,
                                                       ViInt32   stopMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_STOP_MODE, stopMode),
            2, "Stop Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configure the remote trace file parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_FILE_NAME
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_FILE_FORMAT
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_FILE_SIZE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:NAME
/// HIFN     TRACe:REMote:MODE:FILE:FORMat
/// HIFN     TRACe:REMote:MODE:FILE:SIZE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Sets the directory and file name where the remote trace file is
/// HIPAR fileName/stored.
/// HIPAR format/Sets the file format of the remote trace file.
/// HIPAR maximumFileSize/Sets the maximum size of the remote trace file.
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceFile(ViSession instrSession,
                                                   ViString  fileName,
                                                   ViInt32   format,
                                                   ViInt32   maximumFileSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_FILE_NAME, fileName),
            2, "File Name");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_FILE_FORMAT, format),
            3, "Format");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_FILE_SIZE, maximumFileSize),
            4, "Maximum File Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the content of the remote trace file.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:FILTer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR input/Input data
/// HIPAR output/Output data.
/// HIPAR SCPIerror/New SCPI error queue entries
/// HIPAR trigger/Trigger events
/// HIPAR deviceClear/Device clear events
/// HIPAR statusRegister/Status register conditions
/// HIPAR connection/Open/close connection events
/// HIPAR remoteLocalEvents/Local/remote transition events.
/// HIPAR locking/Remote locking events
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceFileContent(ViSession instrSession,
                                                          ViBoolean input,
                                                          ViBoolean output,
                                                          ViBoolean SCPIerror,
                                                          ViBoolean trigger,
                                                          ViBoolean deviceClear,
                                                          ViBoolean statusRegister,
                                                          ViBoolean connection,
                                                          ViBoolean remoteLocalEvents,
                                                          ViBoolean locking)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:REM:MODE:FILE:FILT %s,%s,%s,%s,%s,%s,%s,%s,%s", input ? "ON" : "OFF",
             output ? "ON" : "OFF",
             SCPIerror ? "ON" : "OFF",
             trigger ? "ON" : "OFF",
             deviceClear ? "ON" : "OFF",
             statusRegister ? "ON" : "OFF",
             connection ? "ON" : "OFF",
             remoteLocalEvents ? "ON" : "OFF",
             locking ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures diagnostic remote trace parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_COMMAND_EXECUTION_DURATION_ENABLED
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_RPC_OUTPUT_ENABLED
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_PARSER_TRANSITIONS_OUTPUT_ENABLED
/// HIFN     RSSCOPE_ATTR_REMOTE_TRACE_FUNCTION_NAMES_OUTPUT_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe:REMote:MODE:FILE:DEXecution:DURation
/// HIFN     TRACe:REMote:MODE:FILE:RPC
/// HIFN     TRACe:REMote:MODE:FILE:PARSer
/// HIFN     TRACe:REMote:MODE:FILE:FUNCtions
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR commandExecutionDuration/Traces the device execution time of a command.
/// HIPAR rpcOutput/Enables and disables output of rpc calls to remote trace.
/// HIPAR parserTransitionsOutput/Enables and disables output of parser transitions to remote trace.
/// HIPAR functionNamesOutput/Enables and disables output of function names to remote trace.
ViStatus _VI_FUNC rsscope_ConfigureRemoteTraceDiagnostic(ViSession instrSession,
                                                         ViBoolean commandExecutionDuration,
                                                         ViBoolean rpcOutput,
                                                         ViBoolean parserTransitionsOutput,
                                                         ViBoolean functionNamesOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_COMMAND_EXECUTION_DURATION_ENABLED, commandExecutionDuration),
            2, "Command Execution Duration");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_RPC_OUTPUT_ENABLED, rpcOutput),
            3, "RPC Output");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_PARSER_TRANSITIONS_OUTPUT_ENABLED, parserTransitionsOutput),
            4, "P+arser Transitions Output");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_REMOTE_TRACE_FUNCTION_NAMES_OUTPUT_ENABLED, functionNamesOutput),
            5, "Function Names Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsscope_SetAttributeViInt32(ViSession instrSession,
                                              ViString  repeatedCapability,
                                              ViAttr    attributeId,
                                              ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsscope_SetAttributeViInt64(ViSession instrSession,
                                              ViString  repeatedCapability,
                                              ViAttr    attributeId,
                                              ViInt64   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsscope_SetAttributeViReal64(ViSession instrSession,
                                               ViString  repeatedCapability,
                                               ViAttr    attributeId,
                                               ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsscope_SetAttributeViString(ViSession instrSession,
                                               ViString  repeatedCapability,
                                               ViAttr    attributeId,
                                               ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsscope_SetAttributeViBoolean(ViSession instrSession,
                                                ViString  repeatedCapability,
                                                ViAttr    attributeId,
                                                ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsscope_GetAttributeViInt32(ViSession instrSession,
                                              ViString  repeatedCapability,
                                              ViAttr    attributeId,
                                              ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsscope_GetAttributeViInt64(ViSession instrSession,
                                              ViString  repeatedCapability,
                                              ViAttr    attributeId,
                                              ViInt64*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsscope_GetAttributeViReal64(ViSession instrSession,
                                               ViString  repeatedCapability,
                                               ViAttr    attributeId,
                                               ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufSize/Attribute Value parameter.
/// HIPAR value/The buffer in which the function returns the current value of the
/// HIPAR value/attribute. The buffer must be of type ViChar and have at least as many
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsscope_GetAttributeViString(ViSession instrSession,
                                               ViString  repeatedCapability,
                                               ViAttr    attributeId,
                                               ViInt32   bufferSize,
                                               ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
                                                   value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsscope_GetAttributeViBoolean(ViSession  instrSession,
                                                ViString   repeatedCapability,
                                                ViAttr     attributeId,
                                                ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapability, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
/// HIPAR responseLength/The length in bytes of the response read from the instrument. If
/// HIPAR responseLength/Response Length > allocated Buffer Size, some data from the instrument
/// HIPAR responseLength/was lost; resize your buffer to at least Response Length and call again
/// HIPAR responseLength/to capture the missing data from the instrument.
ViStatus _VI_FUNC rsscope_GetAttributeViStringWithRealLen(ViSession instrSession,
                                                          ViString  repeatedCapability,
                                                          ViAttr    attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    attributeValue[],
                                                          ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringWithRealLen(instrSession, repeatedCapability, attributeId,
                                                   RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue, responseLength);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a RawString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR responseLength/The length in bytes of the response read from the instrument. If
/// HIPAR responseLength/Response Length > allocated Buffer Size, some data from the instrument
/// HIPAR responseLength/was lost; resize your buffer to at least Response Length and call again
/// HIPAR responseLength/to capture the missing data from the instrument.
ViStatus _VI_FUNC rsscope_GetAttributeRawStringWithRealLen(ViSession instrSession,
                                                           ViString  repeatedCapability,
                                                           ViAttr    attributeId,
                                                           ViInt32   bufferSize,
                                                           ViChar    attributeValue[],
                                                           ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawStringWithRealLen(instrSession, repeatedCapability, attributeId,
                                                      RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue, responseLength);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsscope_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                            ViAttr    attributeId,
                                                            ViInt32   bufferSize,
                                                            ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeId, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsscope_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeId,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN  Enables the math waveform display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR mathEnabled/Enables the math waveform display.
ViStatus _VI_FUNC rsscope_ConfigureMathEnabled(ViSession instrSession,
                                               ViInt32   mathNumber,
                                               ViBoolean mathEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_WAVEFORM_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MATH_WAVEFORM_ENABLED, mathEnabled),
            3, "Math Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures general mathematics parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR calculationMethod/Selects the method to build the resulting math waveform from
/// HIPAR calculationMethod/consecutive acquisitions. The processing is similar to the waveform
/// HIPAR calculationMethod/arithmetics - instead of the acquired waveforms the math waveforms are
/// HIPAR calculationMethod/used to create envelope and average.
/// HIPAR expression/Defines the math expression to be calculated for the specified math
/// HIPAR expression/channel.
ViStatus _VI_FUNC rsscope_ConfigureMathSetup(ViSession instrSession,
                                             ViInt32   mathNumber,
                                             ViInt32   calculationMethod,
                                             ViString  expression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_ACQUISITION_TYPE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MATH_ACQUISITION_TYPE, calculationMethod),
            3, "Calculation Method");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MATH_EXPRESSION, expression),
            4, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the upper or lower part of the waveform for mathematic
/// HIFN  calculation, or a combination of both.
/// HIFN
/// HIFN     The setting is relevant for waveforms with waveform arithmetic mode
/// HIFN  "Envelope" or with "Peak detect" decimation. All mathematic operations
/// HIFN  - except for derivation - can be applied to envelope waveforms and
/// HIFN  waveforms with "Peak detect" decimation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR envelopeSelection/Selects the upper or lower part of the waveform for mathematic
/// HIPAR envelopeSelection/calculation, or a combination of both.
ViStatus _VI_FUNC rsscope_ConfigureMathEnvelopeWaveformSelection(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViInt32   envelopeSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_WAVEFORM_ENVELOPE_SELECTION,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MATH_WAVEFORM_ENVELOPE_SELECTION, envelopeSelection),
            3, "Envelope Selection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the range of FFT values to be displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR verticalRange/Defines the range of FFT values to be displayed.
ViStatus _VI_FUNC rsscope_ConfigureMathVerticalRange(ViSession instrSession,
                                                     ViInt32   mathNumber,
                                                     ViReal64  verticalRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_VERTICAL_RANGE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MATH_VERTICAL_RANGE, verticalRange),
            3, "Vertical Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the scale of the y-axis in the math function diagram. The
/// HIFN  value is defined as "V per division", e.g. 50V/div. In this case, the
/// HIFN  horizontal grid lines are displayed in intervals of 50 V.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR verticalScale/Defines the scale of the y-axis in the math function diagram. The
/// HIPAR verticalScale/value is defined as "V per division", e.g. 50V/div. In this case, the
/// HIPAR verticalScale/horizontal grid lines are displayed in intervals of 50 V.
ViStatus _VI_FUNC rsscope_ConfigureMathVerticalScale(ViSession instrSession,
                                                     ViInt32   mathNumber,
                                                     ViReal64  verticalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_VERTICAL_SCALE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MATH_VERTICAL_SCALE, verticalScale),
            3, "Vertical Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets a voltage offset to adjust the vertical position of the math
/// HIFN  function on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR verticalOffset/Sets a voltage offset to adjust the vertical position of the math
/// HIPAR verticalOffset/function on the screen.
ViStatus _VI_FUNC rsscope_ConfigureMathVerticalOffset(ViSession instrSession,
                                                      ViInt32   mathNumber,
                                                      ViReal64  verticalOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_VERTICAL_OFFSET,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MATH_VERTICAL_OFFSET, verticalOffset),
            3, "Vertical Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Forces the immediate restart of the envelope and average calculation
/// HIFN  for all waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_MathReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_MATH_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of acquired waveforms after which the envelope and
/// HIFN  average evaluation restarts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resetMode/Defines when the envelope and average evaluation restarts.
/// HIPAR envelopeTimeout/Defines the time after which the envelope and average evaluation
/// HIPAR envelopeTimeout/restarts.
/// HIPAR acquiredWaveforms/Defines the number of acquired waveforms after which the envelope and
/// HIPAR acquiredWaveforms/average evaluation restarts.
ViStatus _VI_FUNC rsscope_ConfigureMathResetMode(ViSession instrSession,
                                                 ViInt32   resetMode,
                                                 ViReal64  envelopeTimeout,
                                                 ViInt32   acquiredWaveforms)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MATH_RESET_MODE, resetMode),
            2, "Reset Mode");

    if (resetMode == RSSCOPE_VAL_MATH_RESET_TIME)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_MATH_ENVELOPE_TIMEOUT, envelopeTimeout),
                3, "Envelope Timeout");
    }
    if (resetMode == RSSCOPE_VAL_MATH_RESET_WFMS)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_MATH_ACQUIRED_WAVEFORMS, acquiredWaveforms),
                4, "Acquired Waveforms");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects between the magnitude and phase spectrum type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR math/Selects repeated capability Math.
/// HIPAR fftType/Selects between the magnitude and phase spectrum type.
ViStatus _VI_FUNC rsscope_ConfigureFFTType(ViSession instrSession,
                                           ViInt32   math,
                                           ViInt32   fftType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, math, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_TYPE,
            "Math", math- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_TYPE, fftType),
            3, "FFT Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the scaling method for the frequency (x-)axis of the
/// HIFN  spectrogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR frequencyAxis/Defines the scaling method for the frequency (x-)axis of the
/// HIPAR frequencyAxis/spectrogram.
ViStatus _VI_FUNC rsscope_ConfigureFFTFrequencyAxis(ViSession instrSession,
                                                    ViInt32   mathNumber,
                                                    ViInt32   frequencyAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_FREQUENCY_AXIS,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_FREQUENCY_AXIS, frequencyAxis),
            3, "Frequency Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency range by means of center frequency and frequency
/// HIFN  span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR centerFrequency/Defines the position of the displayed frequency range, which is
/// HIPAR centerFrequency/(Center - Span/2) to (Center + Span/2).
/// HIPAR frequencySpan/Defines the stop frequency of the displayed frequency span.
ViStatus _VI_FUNC rsscope_ConfigureFFTFrequencyCenterSpan(ViSession instrSession,
                                                          ViInt32   mathNumber,
                                                          ViReal64  centerFrequency,
                                                          ViReal64  frequencySpan)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_CENTER_FREQUENCY,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_CENTER_FREQUENCY, centerFrequency),
            3, "Center Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_FREQUENCY_SPAN, frequencySpan),
            4, "Frequency Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs FFT calculation for the full frequency span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
ViStatus _VI_FUNC rsscope_ConfigureFFTFrequencySpanFull(ViSession instrSession,
                                                        ViInt32   mathNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_FULL_SPAN,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_FFT_FULL_SPAN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency range by means of start and stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR startFrequency/Defines the start frequency of the displayed frequency span.
/// HIPAR stopFrequency/Defines the stop frequency of the displayed frequency span.
ViStatus _VI_FUNC rsscope_ConfigureFFTFrequencyStartStop(ViSession instrSession,
                                                         ViInt32   mathNumber,
                                                         ViReal64  startFrequency,
                                                         ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_START_FREQUENCY,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_START_FREQUENCY, startFrequency),
            3, "Start Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_STOP_FREQUENCY, stopFrequency),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the FFT frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR rbwandSpanCouplingEnabled/Couples the frequency span to the RBW.
/// HIPAR rbwRatio/Defines the ratio resolution bandwidth (Hz) / span (Hz).
/// HIPAR resolutionBandwidth/Defines the resolution bandwidth.
ViStatus _VI_FUNC rsscope_ConfigureFFTResolutionBandwidth(ViSession instrSession,
                                                          ViInt32   mathNumber,
                                                          ViBoolean rbwandSpanCouplingEnabled,
                                                          ViInt32   rbwRatio,
                                                          ViReal64  resolutionBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH_COUPLING_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH_COUPLING_ENABLED, rbwandSpanCouplingEnabled),
            3, "RBW and Span Coupling Enabled");

    if (rbwandSpanCouplingEnabled == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH_RATIO, rbwRatio),
                4, "RBW Ratio");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH, resolutionBandwidth),
                5, "Resolution Bandwidth");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the effective resolution bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR adjustedRBW/Queries the effective resolution bandwidth.
ViStatus _VI_FUNC rsscope_QueryFFTAdjustedResolutionBandwidth(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViReal64* adjustedRBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_EFFECTIVE_RESOLUTION_BANDWIDTH,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_EFFECTIVE_RESOLUTION_BANDWIDTH, adjustedRBW),
            3, "Adjusted RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the FFT window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR windowType/Windowing helps minimize the discontinuities at the end of the
/// HIPAR windowType/measured signal interval and thus reduces the effect of spectral
/// HIPAR windowType/leakage, increasing the frequency resolution.
ViStatus _VI_FUNC rsscope_ConfigureFFTWindowType(ViSession instrSession,
                                                 ViInt32   mathNumber,
                                                 ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_WINDOW_TYPE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_WINDOW_TYPE, windowType),
            3, "Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the FFT frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR arithmetic/The arithmetic mode defines how the final FFT result is calculated
/// HIPAR arithmetic/from the individual frame results.
/// HIPAR overlapFactor/Defines the minimum factor by which two neighboring frames overlap. If
/// HIPAR overlapFactor/the required number of frames to cover the input values allows for more
/// HIPAR overlapFactor/overlap, the factor is increased.
/// HIPAR maxFrameCount/Restricts the maximum number of frames to be calculated. Due to the
/// HIPAR maxFrameCount/other parameter settings, the required number of frames may become very
/// HIPAR maxFrameCount/high, thus slowing performance. By restricting the number of frames,
/// HIPAR maxFrameCount/you can avoid performance loss without changing the other parameters.
ViStatus _VI_FUNC rsscope_ConfigureFFTFrameSetup(ViSession instrSession,
                                                 ViInt32   mathNumber,
                                                 ViInt32   arithmetic,
                                                 ViInt32   overlapFactor,
                                                 ViInt32   maxFrameCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_FRAME_ARITHMETIC,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_FRAME_ARITHMETIC, arithmetic),
            3, "Arithmetic");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_FRAME_OVERLAP_FACTOR, overlapFactor),
            4, "Overlap Factor");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_FRAME_MAX_FRAME_COUNT, maxFrameCount),
            5, "Max Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Due to the restriction of the number of frames, the waveform may only
/// HIFN  be analyzed partially. This control queries the percentage of the trace
/// HIFN  that was analyzed, i.e. which part of the trace was included in the
/// HIFN  frame calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR frameCoverage/Due to the restriction of the number of frames, the waveform may only
/// HIPAR frameCoverage/be analyzed partially. This control queries the percentage of the trace
/// HIPAR frameCoverage/that was analyzed, i.e. which part of the trace was included in the
/// HIPAR frameCoverage/frame calculation.
ViStatus _VI_FUNC rsscope_QueryFFTFrameCoverage(ViSession instrSession,
                                                ViInt32   mathNumber,
                                                ViReal64* frameCoverage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_FRAME_COVERAGE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_FRAME_COVERAGE, frameCoverage),
            3, "Frame Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the selected waveform is displayed according to its
/// HIFN  assigned color table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR useColorTable/If enabled, the selected waveform is displayed according to its
/// HIPAR useColorTable/assigned color table.
ViStatus _VI_FUNC rsscope_ConfigureFFTUseColorTable(ViSession instrSession,
                                                    ViInt32   mathNumber,
                                                    ViBoolean useColorTable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_USE_COLOR_TABLE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_USE_COLOR_TABLE, useColorTable),
            3, "Use Color Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum value on y-axis for spectrum displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR math/Selects repeated capability Math.
/// HIPAR verticalMaximum/Defines the maximum value on y-axis for spectrum displays.
ViStatus _VI_FUNC rsscope_ConfigureFFTVerticalMaximum(ViSession instrSession,
                                                      ViInt32   math,
                                                      ViReal64  verticalMaximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, math, 1, maxMath),
            2, "Math");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_VERTICAL_MAXIMUM,
            "Math", math- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_VERTICAL_MAXIMUM, verticalMaximum),
            3, "Vertical Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the settings for the magnitude of the frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR magnitudeUnit/Defines the scaling of the y-axis. The display values are valid for
/// HIPAR magnitudeUnit/50Ohm termination impendance.
/// HIPAR referenceLevel/Defines the reference level for dB scaling.
/// HIPAR magnitudeRange/Defines the vertical value range in spectrum mode.
ViStatus _VI_FUNC rsscope_ConfigureFFTMagnitude(ViSession instrSession,
                                                ViInt32   mathNumber,
                                                ViInt32   magnitudeUnit,
                                                ViReal64  referenceLevel,
                                                ViInt32   magnitudeRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_MAGNITUDE_UNIT,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_MAGNITUDE_UNIT, magnitudeUnit),
            3, "Magnitude Unit");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_REFERENCE_LEVEL, referenceLevel),
            4, "Reference Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_MAGNITUDE_RANGE, magnitudeRange),
            5, "Magnitude Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define the settings for the phase of the frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR phaseUnit/Defines the scaling unit for phase display.
/// HIPAR phaseUnwrapEnabled/If enabled, phase shifts due to a limitation of the value range are
/// HIPAR phaseUnwrapEnabled/eliminated.
/// HIPAR suppressionEnabled/Enables noise suppression.
/// HIPAR suppressionThreshold/Defines the minimum frequency magnitude for which phases are
/// HIPAR suppressionThreshold/calculated.
ViStatus _VI_FUNC rsscope_ConfigureFFTPhase(ViSession instrSession,
                                            ViInt32   mathNumber,
                                            ViInt32   phaseUnit,
                                            ViBoolean phaseUnwrapEnabled,
                                            ViBoolean suppressionEnabled,
                                            ViReal64  suppressionThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_PHASE_UNIT,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_PHASE_UNIT, phaseUnit),
            3, "Phase Unit");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_PHASE_UNWRAP_ENABLED, phaseUnwrapEnabled),
            4, "Phase Unwrap Enabled");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_NOISE_SUPPRESSION_ENABLED, suppressionEnabled),
            5, "Suppression Enabled");

    if (suppressionEnabled == VI_TRUE)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_NOISE_SUPPRESSION_THRESHOLD, suppressionThreshold),
                6, "Suppression Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the FFT gating.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR useGate/Enables FFT gating.
ViStatus _VI_FUNC rsscope_ConfigureFFTGateState(ViSession instrSession,
                                                ViInt32   mathNumber,
                                                ViBoolean useGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_GATE_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_ENABLED, useGate),
            3, "Use Gate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is displayed in the
/// HIFN  source diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR showGate/Indicates the gate area in the source diagram.
ViStatus _VI_FUNC rsscope_ConfigureFFTGateShow(ViSession instrSession,
                                               ViInt32   mathNumber,
                                               ViBoolean showGate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_GATE_DISPLAY_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_DISPLAY_ENABLED, showGate),
            3, "Show Gate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the gate area is defined identically
/// HIFN  to the zoom area for the selected active zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR zoomCoupling/If enabled, the gate area is defined identically to the zoom area for
/// HIPAR zoomCoupling/the zoom diagram.
/// HIPAR reserved/This control is reserved for future use.
ViStatus _VI_FUNC rsscope_ConfigureFFTGateDefinitionZoom(ViSession instrSession,
                                                         ViInt32   mathNumber,
                                                         ViBoolean zoomCoupling,
                                                         ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_USE_COLOR_TABLE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_ZOOM_COUPLING_ENABLED, zoomCoupling),
            3, "Zoom Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and end values for the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR start/Defines the starting value for the gate.
/// HIPAR stop/Defines the ending value for the gate.
ViStatus _VI_FUNC rsscope_ConfigureFFTGateDefinitionStartStop(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViInt32   mode,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_GATE_MODE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_MODE, mode),
            3, "Mode");

    if (mode == RSSCOPE_VAL_ABS)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_ABSOLUTE_START, start),
                4, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_ABSOLUTE_STOP, stop),
                5, "Stop");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_RELATIVE_START, start),
                4, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_FFT_GATE_RELATIVE_STOP, stop),
                5, "Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the behaviour of the record length or RBW
/// HIFN  value in dependancy to the other FFT parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR coupling/Defines the behaviour of the record length or RBW value in dependancy
/// HIPAR coupling/to the other FFT parameters.
ViStatus _VI_FUNC rsscope_ConfigureFFTGateRecordLengthRBWCoupling(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH_GATE_COUPLING,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_RESOLUTION_BANDWIDTH_GATE_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the required acquisition time. If FFT gating is
/// HIFN  used and the resolution BW is set to constant, record length can be
/// HIFN  extended to acquire the required number of samples. In this case, the
/// HIFN  required acquisition time differs from the adjusted acquisition time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR requiredAcquisitionTime/Queries the required acquisition time. If FFT gating is used and the
/// HIPAR requiredAcquisitionTime/resolution BW is set to constant, record length can be extended to
/// HIPAR requiredAcquisitionTime/acquire the required number of samples. In this case, the required
/// HIPAR requiredAcquisitionTime/acquisition time differs from the adjusted acquisition time.
ViStatus _VI_FUNC rsscope_QueryFFTGateRequiredAcquisitionTime(
                            ViSession instrSession,
                            ViReal64* requiredAcquisitionTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_ACQUISITION_TIME, requiredAcquisitionTime),
            3, "Required Acquisition Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies the current FFT settings of the selected math waveform (m) to
/// HIFN  the other selected math waveform (m2), and couples the two waveforms.
/// HIFN  This can be repeated for all math waveforms.
/// HIFN     If any FFT setting for any of the coupled spectrums is changed, it is
/// HIFN  changed for all coupled spectrums.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR mathNumber2/Selects the math waveform.
/// HIPAR coupledSpectrum/Couples the math waveforms, selected by math number and math number 2.
ViStatus _VI_FUNC rsscope_ConfigureFFTCoupledSpectrum(ViSession instrSession,
                                                      ViInt32   mathNumber,
                                                      ViInt32   mathNumber2,
                                                      ViBoolean coupledSpectrum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_M2[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math2", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber2, RSSCOPE_REPCAP_MATH2_M21, maxMath),
            3, "Math Number 2");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_COUPLED_SPECTRUM,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_COUPLED_SPECTRUM,
            "Math2", mathNumber2- RSSCOPE_REPCAP_MATH2_M21, RS_REPCAP_BUF_SIZE, repCap_M2),
            3, "Math Number 2");
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap_M2, RS_REPCAP_BUF_SIZE - strlen(repCap) - strlen(repCap_M2) - 1);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_COUPLED_SPECTRUM, coupledSpectrum),
            4, "Coupled Spectrum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the spectrogram display for a math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR enabled/Enables the spectrogram display for a math waveform.
ViStatus _VI_FUNC rsscope_ConfigureFFTSpectrumEnabled(ViSession instrSession,
                                                      ViInt32   mathNumber,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_SPECTRUM_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_SPECTRUM_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the color table mode for the frequency analysis display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR colorTableMode/Selects the color table mode for the frequency analysis display.
ViStatus _VI_FUNC rsscope_ConfigureFFTSpectrumColorTableMode(ViSession instrSession,
                                                             ViInt32   mathNumber,
                                                             ViInt32   colorTableMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_SPECTRUM_COLOR_TABLE_MODE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_FFT_SPECTRUM_COLOR_TABLE_MODE, colorTableMode),
            3, "Color Table Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the spectrogram time line.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_ENABLED
/// HIFN     RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_POSITION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:MATH<m>:FFT:SPECtrogram:TIMeline<n>:STATe
/// HIFN     CALCulate:MATH<m>:FFT:SPECtrogram:TIMeline<n>:POSition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR timeLine/Selects the time line.
/// HIPAR enabled/Enables one of two possible time lines in a spectrogram diagram. A
/// HIPAR enabled/time line marks a single waveform in the spectrogram, that is: the
/// HIPAR enabled/power vs frequency results for the data acquired at a specific time.
/// HIPAR enabled/After enabling a time line, the results for that time are displayed in
/// HIPAR enabled/the spectrum diagram.
/// HIPAR position/Defines the position of one of the two possible time lines in a
/// HIPAR position/spectrogram. The time line must be enabled first.
ViStatus _VI_FUNC rsscope_ConfigureFFTSpectrumTimeLine(ViSession instrSession,
                                                       ViInt32   mathNumber,
                                                       ViInt32   timeline,
                                                       ViBoolean enabled,
                                                       ViInt64   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_TL[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeline, RSSCOPE_REPCAP_TIMELINE_TL1, RSSCOPE_REPCAP_TIMELINE_TL2),
            3, "Time Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_ENABLED,
            "Timeline", timeline- RSSCOPE_REPCAP_TIMELINE_TL1, RS_REPCAP_BUF_SIZE, repCap_TL),
            3, "Time Line");
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap_TL, RS_REPCAP_BUF_SIZE - strlen(repCap) - strlen(repCap_TL) - 1);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt64(instrSession, repCap,
                    RSSCOPE_ATTR_FFT_SPECTRUM_TIME_LINE_POSITION, position),
                5, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source waveform to be used as a reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR source/Selects the source waveform to be used as a reference.
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformSource(ViSession instrSession,
                                                           ViInt32   referenceNumber,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_SOURCE,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSSCOPE_VAL_WAV_NONE, RSSCOPE_VAL_WAV_M8),
            3, "Source");
    if ((RSSCOPE_VAL_WAV_R4 < source) && (source < RSSCOPE_VAL_WAV_TRK1))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 3, "Source");
    }

    if ((RSSCOPE_VAL_WAV_TRK8 < source) && (source < RSSCOPE_VAL_WAV_Z1V1))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 3, "Source");
    }

    if ((RSSCOPE_VAL_WAV_COMMON2 < source) && (source < RSSCOPE_VAL_WAV_M6))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 3, "Source");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies the selected source waveform with all its settings to the
/// HIFN  memory of the reference waveform
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsscope_UpdateReferenceWaveformWithSelectedSource(
                            ViSession instrSession,
                            ViInt32   referenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_UPDATE,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the reference waveform display in the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayEnabled/If enabled, the reference waveform is displayed in the diagram.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformDisplayState(
                            ViSession instrSession,
                            ViInt32   referenceNumber,
                            ViBoolean displayEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED, displayEnabled),
            3, "Display Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The selected reference waveform is no longer displayed, its memory is
/// HIFN  deleted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsscope_ClearReferenceWaveform(ViSession instrSession,
                                                 ViInt32   referenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_CLEAR,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the math waveform display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR waveformName/Defines the name of the reference waveform file to be loaded, saved or
/// HIPAR waveformName/deleted.
/// HIPAR operation/This control selects the type of waveform operation to be performed.
ViStatus _VI_FUNC rsscope_ReferenceWaveformFileOperations(ViSession instrSession,
                                                          ViInt32   referenceNumber,
                                                          ViString  waveformName,
                                                          ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_NAME,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    switch (operation)
    {
    case RSSCOPE_VAL_WAV_SELECT:
        viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_NAME, waveformName),
                3, "Waveform Name");
        break;

    case RSSCOPE_VAL_WAV_OPEN:
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_LOAD, NULL));
        break;

    case RSSCOPE_VAL_WAV_SAVE:
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_SAVE, NULL));
        break;

    case RSSCOPE_VAL_WAV_DELETE:
        checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_DELETE, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 4, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical scaling of the reference
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR verticalCoupling/Selects the coupling of vertical settings.
/// HIPAR verticalScale/Sets the scale factor for the reference waveform.
/// HIPAR verticalPosition/Moves the reference waveform and its horizontal axis up or down in the
/// HIPAR verticalPosition/diagram, if vertical scaling is set to "Independent".
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformVerticalScaling(
                            ViSession instrSession,
                            ViInt32   referenceNumber,
                            ViInt32   verticalCoupling,
                            ViReal64  verticalScale,
                            ViReal64  verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_COUPLING,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_COUPLING, verticalCoupling),
            3, "Vertical Coupling");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_SCALE, verticalScale),
            4, "Vertical Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_POSITION, verticalPosition),
            5, "Vertical Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the settings of the source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsscope_ReferenceWaveformVerticalScalingSetToOriginal(
                            ViSession instrSession,
                            ViInt32   referenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_RESTORE,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_RESTORE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the vertical stretching of the reference
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR verticalStretchingEnabled/Enables and disables the vertical stretching. Stretching changes the
/// HIPAR verticalStretchingEnabled/display of the waveform independent of the vertical scale and position.
/// HIPAR factor/Defines the vertical stretching factor. A factor greater than 1
/// HIPAR factor/stretches the waveform vertically, a factor lower than 1 compresses the
/// HIPAR factor/curve.
/// HIPAR offset/Moves the reference waveform vertically. Like vertical offset of
/// HIPAR offset/channel waveforms, the offset of a reference waveform is subtracted
/// HIPAR offset/from the measured value.
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformVerticalStretching(
                            ViSession instrSession,
                            ViInt32   referenceNumber,
                            ViBoolean verticalStretchingEnabled,
                            ViReal64  factor,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_STRETCHING_ENABLED,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_STRETCHING_ENABLED, verticalStretchingEnabled),
            3, "Vertical Stretching Enabled");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_FACTOR, factor),
            4, "Factor");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the original vertical settings of the reference waveform
/// HIFN  (vertical scale, position, and offset), if
/// HIFN  RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_COUPLING is set to independent
/// HIFN  (see rsscope_ConfigureReferenceWaveformVerticalScaling).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsscope_ResetReferenceWaveformVerticalSettings(
                            ViSession instrSession,
                            ViInt32   referenceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_RESET,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_VERTICAL_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the coupling of horizontal settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR horizontalCoupling/Selects the coupling of horizontal settings.
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformHorizontalScaling(
                            ViSession instrSession,
                            ViInt32   referenceNumber,
                            ViInt32   horizontalCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_COUPLING,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_COUPLING, horizontalCoupling),
            3, "Horizontal Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal stretching of the reference
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR horizontalStretchingEnabled/Enables and disables the horizontal stretching. Stretching changes the
/// HIPAR horizontalStretchingEnabled/display of the waveform independent of the horizontal settings of the
/// HIPAR horizontalStretchingEnabled/source waveform and of the horizontal diagram settings.
/// HIPAR factor/A factor greater than 1 stretches the waveform horizontally, a factor
/// HIPAR factor/lower than 1 compresses the curve.
/// HIPAR offset/Moves the waveform horizontally. Positive values shift the waveform to
/// HIPAR offset/the right, negative values shift it to the left.
ViStatus _VI_FUNC rsscope_ConfigureReferenceWaveformHorizontalStretching(
                            ViSession instrSession,
                            ViInt32   referenceNumber,
                            ViBoolean horizontalStretchingEnabled,
                            ViReal64  factor,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_STRETCHING_ENABLED,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_STRETCHING_ENABLED, horizontalStretchingEnabled),
            3, "Horizontal Stretching Enabled");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_FACTOR, factor),
            4, "Factor");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_HORZ_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the path and the file to be imported.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_NAME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     REFCurve<m>:MULTichannel:NAME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR name/Defines the path and the file to be imported.
ViStatus _VI_FUNC rsscope_ConfigureReferenceMultiwaveformName(
                            ViSession instrSession,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_FILE_NAME, name),
            3, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Assigns a waveform from the multichannel file to a reference waveform.
/// HIFN  To import all waveforms that are in the file, you must assign each
/// HIFN  waveform to another reference waveform.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_IMPORT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     REFCurve<m>:MULTichannel:IMPort
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR waveformImport/Assigns a waveform from the multichannel file to a reference waveform.
/// HIPAR waveformImport/To import all waveforms that are in the file, you must assign each
/// HIPAR waveformImport/waveform to another reference waveform.
ViStatus _VI_FUNC rsscope_ConfigureReferenceMultiwaveformImport(
                            ViSession instrSession,
                            ViInt32   waveformImport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_IMPORT, waveformImport),
            3, "Waveform Import");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the waveform data to the reference waveforms.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_OPEN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     REFCurve<m>:MULTichannel:OPEN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_OpenReferenceMultiwaveform(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REFERENCE_WAVEFORM_MULTICHANNEL_OPEN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, and returns the waveform for the channel you
/// HIFN  specify.  You call the rsscope_FetchWaveform function to obtain the
/// HIFN  waveforms for each of the remaining enabled channels without initiating
/// HIFN  another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time
/// HIPAR initialX/is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.
ViStatus _VI_FUNC rsscope_ReadWaveform(ViSession instrSession,
                                       ViInt32   channel,
                                       ViInt32   waveform,
                                       ViInt32   waveformSize,
                                       ViInt32   maximumTime,
                                       ViReal64  waveformArray[],
                                       ViInt32*  actualPoints,
                                       ViReal64* initialX,
                                       ViReal64* xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchWaveform(instrSession, channel, waveform, waveformSize,
        waveformArray, actualPoints, initialX,
        xIncrement));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, and returns the waveform for the channel you
/// HIFN  specify.  You call the rsscope_FetchWaveform function to obtain the
/// HIFN  waveforms for each of the remaining enabled channels without initiating
/// HIFN  another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time
/// HIPAR initialX/is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.
ViStatus _VI_FUNC rsscope_ReadWaveformFloat(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   waveform,
                                            ViInt32   waveformSize,
                                            ViInt32   maximumTime,
                                            ViReal32  waveformArray[],
                                            ViInt32*  actualPoints,
                                            ViReal64* initialX,
                                            ViReal64* xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchWaveformFloat(instrSession, channel, waveform, waveformSize,
        waveformArray, actualPoints, initialX,
        xIncrement));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates measurement and returns the data of the math waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_ReadMathWaveform(ViSession instrSession,
                                           ViInt32   mathNumber,
                                           ViInt32   waveformSize,
                                           ViInt32   maximumTime,
                                           ViReal64  waveformArray[],
                                           ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchMathWaveform(instrSession, mathNumber, waveformSize,
        waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates measurement and returns the data of the math waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_ReadMathWaveformFloat(ViSession instrSession,
                                                ViInt32   mathNumber,
                                                ViInt32   waveformSize,
                                                ViInt32   maximumTime,
                                                ViReal32  waveformArray[],
                                                ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchMathWaveformFloat(instrSession, mathNumber, waveformSize,
        waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a main waveform measurement on the
/// HIFN  channel you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the rsscope_FetchMainWaveformMeasurement
/// HIFN  function separately for any other waveform measurement that you want to
/// HIFN  obtain on a specific channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadMainWaveformMeasurement(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViInt32   statisticType,
                                                      ViInt32   maximumTime,
                                                      ViInt32   bufferSize,
                                                      ViReal64  measurementResult[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));

    checkErr(rsscope_FetchMainWaveformMeasurement(instrSession, measurement,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a waveform measurement on the channel
/// HIFN  you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the
/// HIFN  rsscope_FetchAmplitudeTimeWaveformMeasurement function separately for
/// HIFN  any other waveform measurement that you want to obtain on a specific
/// HIFN  channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadAmplitudeTimeWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   statisticType,
                            ViInt32   maximumTime,
                            ViInt32   bufferSize,
                            ViReal64  measurementResult[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchAmplitudeTimeWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new measurement and returns the start and stop
/// HIFN  times of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_ReadStartStopAmplitudeTimeWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   maximumTime,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchStartStopAmplitudeTimeWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        startTime,
        stopTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a waveform measurement on the channel
/// HIFN  you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the
/// HIFN  rsscope_FetchEyeJitterWaveformMeasurement function separately for any
/// HIFN  other waveform measurement that you want to obtain on a specific
/// HIFN  channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadEyeJitterWaveformMeasurement(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementFunction,
                                                           ViInt32   statisticType,
                                                           ViInt32   maximumTime,
                                                           ViInt32   bufferSize,
                                                           ViReal64  measurementResult[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchEyeJitterWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new measurement and returns the start and stop
/// HIFN  times of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_ReadStartStopEyeJitterWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   maximumTime,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchStartStopEyeJitterWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        startTime,
        stopTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a waveform measurement on the channel
/// HIFN  you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the
/// HIFN  rsscope_FetchHistogramWaveformMeasurement function separately for any
/// HIFN  other waveform measurement that you want to obtain on a specific
/// HIFN  channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadHistogramWaveformMeasurement(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementFunction,
                                                           ViInt32   statisticType,
                                                           ViInt32   maximumTime,
                                                           ViInt32   bufferSize,
                                                           ViReal64  measurementResult[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchHistogramWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new measurement and returns the start and stop
/// HIFN  times of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_ReadStartStopHistogramWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   maximumTime,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchStartStopHistogramWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        startTime,
        stopTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a waveform measurement on the channel
/// HIFN  you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the
/// HIFN  rsscope_FetchSpectrumWaveformMeasurement function separately for any
/// HIFN  other waveform measurement that you want to obtain on a specific
/// HIFN  channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadSpectrumWaveformMeasurement(ViSession instrSession,
                                                          ViInt32   measurement,
                                                          ViInt32   measurementFunction,
                                                          ViInt32   statisticType,
                                                          ViInt32   maximumTime,
                                                          ViInt32   bufferSize,
                                                          ViReal64  measurementResult[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchSpectrumWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new measurement and returns the start and stop
/// HIFN  times of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_ReadStartStopSpectrumWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   maximumTime,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchStartStopSpectrumWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        startTime,
        stopTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable
/// HIFN  with the rsscope_ConfigureChannel function.  It then waits for the
/// HIFN  acquisition to complete, obtains a waveform measurement on the channel
/// HIFN  you specify, and returns the measurement value.  You specify a
/// HIFN  particular measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.  You can call the
/// HIFN  rsscope_FetchSpectrumWaveformMeasurement function separately for any
/// HIFN  other waveform measurement that you want to obtain on a specific
/// HIFN  channel without initiating another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_ReadJitterWaveformMeasurement(ViSession instrSession,
                                                        ViInt32   measurement,
                                                        ViInt32   measurementFunction,
                                                        ViInt32   statisticType,
                                                        ViInt32   maximumTime,
                                                        ViInt32   bufferSize,
                                                        ViReal64  measurementResult[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchJitterWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        statisticType,
        bufferSize,
        measurementResult,
        returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new measurement and returns the start and stop
/// HIFN  times of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_ReadStartStopJitterWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   maximumTime,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchStartStopJitterWaveformMeasurement(instrSession, measurement,
        measurementFunction,
        startTime,
        stopTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns selected reference level measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR levelType/Selects level type to be read.
/// HIPAR referenceLevelResult/Returns the reference level value based on the selected Level Type
/// HIPAR referenceLevelResult/cotrol.
ViStatus _VI_FUNC rsscope_ReadReferenceLevelResult(ViSession instrSession,
                                                   ViInt32   measurement,
                                                   ViInt32   levelType,
                                                   ViReal64* referenceLevelResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   arrReferenceResultAttr[] = {
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_BOTTOM_INNER,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_BOTTOM_OUTER,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_LOWER,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_MIDDLE,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_UPPER,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_SIGNAL_HIGH,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_SIGNAL_LOW,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_TOP_INNER,
             RSSCOPE_ATTR_READ_REFERENCE_LEVEL_TOP_OUTER
             };
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, levelType, RSSCOPE_VAL_REF_LEVEL_MEAS_BINN, RSSCOPE_VAL_REF_LEVEL_MEAS_TOUT),
            3, "Level Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrReferenceResultAttr[levelType],
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, arrReferenceResultAttr[levelType], referenceLevelResult),
            4, "Reference Level Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the reference waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_ReadReferenceWaveform(ViSession instrSession,
                                                ViInt32   waveformSize,
                                                ViInt32   maximumTime,
                                                ViReal64  waveformArray[],
                                                ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchReferenceWaveform(instrSession, waveformSize,
        waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the reference waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_ReadReferenceWaveformFloat(ViSession instrSession,
                                                     ViInt32   waveformSize,
                                                     ViInt32   maximumTime,
                                                     ViReal32  waveformArray[],
                                                     ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    checkErr(rsscope_InitiateAcquisition(instrSession, RSSCOPE_VAL_INIT_SINGLE));
    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

    checkErr(rsscope_FetchReferenceWaveformFloat(instrSession, waveformSize,
        waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates a waveform acquisition.  After you call this
/// HIFN  function, the oscilloscope leaves the Idle state and waits for a
/// HIFN  trigger.  The oscilloscope acquires a waveform for each channel you
/// HIFN  have enabled with the  rsscope_ConfigureChannel function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR action/Select action to be performed.
ViStatus _VI_FUNC rsscope_InitiateAcquisition(ViSession instrSession,
                                              ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 attr = RSSCOPE_ATTR_RUN_SINGLE;

    checkErr(RsCore_LockSession(instrSession));

    switch (action)
    {
    case RSSCOPE_VAL_INIT_SINGLE:
        attr = RSSCOPE_ATTR_RUN_SINGLE;
        break;
    case RSSCOPE_VAL_INIT_CONTINUOUS:
        attr = RSSCOPE_ATTR_RUN_CONTINUOUS;
        break;
    case RSSCOPE_VAL_INIT_FORCE:
        attr = RSSCOPE_ATTR_FORCE;
        break;
    case RSSCOPE_VAL_INIT_SINGLE_WITHOUT_WAIT:
        attr = RSSCOPE_ATTR_RUN_SINGLE_WITHOUT_WAIT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 2, "Action");
    }

    checkErr(rsscope_SetAttributeViString(instrSession, "", attr, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns true if the measurement is ready to be fetched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR isMeasurementComplete/Defines timeout in milliseconds.
ViStatus _VI_FUNC rsscope_IsMeasurementComplete(ViSession  instrSession,
                                                ViBoolean* isMeasurementComplete)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  val = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_QueryViInt32(instrSession, "STAT:OPER:COND?", &val));
    *isMeasurementComplete = (val & RSSCOPE_VAL_MEASUREMENT_RUNNING_MASK) == 0;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Waits for measurement to complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR timeout/Defines timeout in milliseconds.
ViStatus _VI_FUNC rsscope_WaitForMeasurementComplete(ViSession instrSession,
                                                     ViInt32   timeout)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean measurement_complete = VI_FALSE;
    clock_t   start_time = clock();

    checkErr(RsCore_LockSession(instrSession));

    double elapsed_time_ms;
    double ms_divider = (double)CLOCKS_PER_SEC / 1000.0;

    do
    {
        checkErr(rsscope_IsMeasurementComplete(instrSession, &measurement_complete));

        elapsed_time_ms = ((double)clock() - (double)start_time) / ms_divider;
        if (elapsed_time_ms > timeout)
            return VI_ERROR_TMO;

        if (elapsed_time_ms < 10.0)
        {
            // Do it as fast as possible, therefore no Sleep
        }
        else if (elapsed_time_ms < 100.0)
            Sleep(1);
        else if (elapsed_time_ms < 1000.0)
            Sleep(5);
        else if (elapsed_time_ms < 5000.0)
            Sleep(10);
        else if (elapsed_time_ms < 10000.0)
            Sleep(50);
        else if (elapsed_time_ms < 20000.0)
            Sleep(100);
        else if (elapsed_time_ms < 30000.0)
            Sleep(500);
        else
            Sleep(1000);
    }
    while (!measurement_complete);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns true, when the instrument indicates waiting for trigger. This
/// HIFN happens if no valid trigger has been found for more than one second since
/// HIFN starting the acquisition. Returns false, if the trigger has been found or
/// HIFN the instrument is in free run.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR isWaitingForTrigger/Returns true, when the instrument indicates waiting for trigger.
/// HIPAR isWaitingForTrigger/This happens if no valid trigger has been found for more than
/// HIPAR isWaitingForTrigger/one second since starting the acquisition. Returns false, if the
/// HIPAR isWaitingForTrigger/trigger has been found or the instrument is in free run.
ViStatus _VI_FUNC rsscope_IsWaitingForTrigger(ViSession  instrSession,
                                              ViBoolean* isWaitingForTrigger)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  val = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_QueryViInt32(instrSession, "STAT:OPER:COND?", &val));

    *isWaitingForTrigger = (val & RSSCOPE_VAL_WAIT_FOR_TRIGGER_MASK) > 0;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sends software trigger and waits for operation complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR maximumTime/Pass the maximum length of time (in milliseconds) in which to allow
/// HIPAR maximumTime/the read waveform operation to complete.
ViStatus _VI_FUNC rsscope_SendSoftwareTrigger(ViSession instrSession,
                                              ViInt32   maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, &timeout));
    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, maximumTime));
    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SEND_SW_TRIGGER_AND_WAIT, NULL));

Error:
    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the
/// HIFN  channel you specify.  The waveform is from an acquisition that you
/// HIFN  previously initiated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/Selects the waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time
/// HIPAR initialX/is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.
ViStatus _VI_FUNC rsscope_FetchWaveform(ViSession instrSession,
                                        ViInt32   channel,
                                        ViInt32   waveform,
                                        ViInt32   waveformSize,
                                        ViReal64  waveformArray[],
                                        ViInt32*  actualPoints,
                                        ViReal64* initialX,
                                        ViReal64* xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 lastX = 0;

    checkErr(RsCore_LockSession(instrSession));

    sprintf(header, "CHAN%ld:WAV%ld:DATA?", channel, waveform);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, header, waveformSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_START, initialX),
            7, "initialX");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_STOP, &lastX),
            8, "lastX");

    if (*actualPoints != 0)
        *xIncrement = (lastX - *initialX) / *actualPoints;
    else
        *xIncrement = RS_VAL_NAN_VI_REAL64;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the
/// HIFN  channel you specify.  The waveform is from an acquisition that you
/// HIFN  previously initiated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/Selects the waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time
/// HIPAR initialX/is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.
ViStatus _VI_FUNC rsscope_FetchWaveformFloat(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   waveform,
                                             ViInt32   waveformSize,
                                             ViReal32  waveformArray[],
                                             ViInt32*  actualPoints,
                                             ViReal64* initialX,
                                             ViReal64* xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 lastX = 0;

    checkErr(RsCore_LockSession(instrSession));

    sprintf(header, "CHAN%ld:WAV%ld:DATA?", channel, waveform);
    checkErr(RsCore_QuerySingleFloatArrayToUserBuffer(instrSession, header, waveformSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_START, initialX),
            7, "initialX");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_STOP, &lastX),
            8, "lastX");

    if (*actualPoints != 0)
        *xIncrement = (lastX - *initialX) / *actualPoints;
    else
        *xIncrement = RS_VAL_NAN_VI_REAL64;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of channel waveform data. The header contains setup
/// HIFN  and acquisition settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveform/Selects the waveform.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR waveformHeader/Returns the header of channel waveform data. The header contains setup
/// HIPAR waveformHeader/and acquisition settings.
ViStatus _VI_FUNC rsscope_FetchWaveformHeader(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   waveform,
                                              ViInt32   bufferSize,
                                              ViChar    waveformHeader[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_waveform[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (channel > RSSCOPE_VAL_CHANNEL4)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL1, RSSCOPE_VAL_CHANNEL4),
                2, "Channel");
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_HEADER,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_HEADER,
            "Waveform", waveform, RS_REPCAP_BUF_SIZE, repCap_waveform),
            3, "Waveform");

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_waveform);

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_HEADER,
            bufferSize, waveformHeader),
            5, "Waveform Header");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the math waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchMathWaveform(ViSession instrSession,
                                            ViInt32   mathNumber,
                                            ViInt32   waveformSize,
                                            ViReal64  waveformArray[],
                                            ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");

    sprintf(header, "CALC:MATH%ld:DATA?", mathNumber);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, header, waveformSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the math waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchMathWaveformFloat(ViSession instrSession,
                                                 ViInt32   mathNumber,
                                                 ViInt32   waveformSize,
                                                 ViReal32  waveformArray[],
                                                 ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");

    sprintf(header, "CALC:MATH%ld:DATA?", mathNumber);
    checkErr(RsCore_QuerySingleFloatArrayToUserBuffer(instrSession, header, waveformSize, waveformArray, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of math waveform data. The header contains
/// HIFN  attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR mathWaveformHeader/Returns the header of math waveform data. The header contains
/// HIPAR mathWaveformHeader/attributes of the waveform.
ViStatus _VI_FUNC rsscope_FetchMathWaveformHeader(ViSession instrSession,
                                                  ViInt32   mathNumber,
                                                  ViInt32   bufferSize,
                                                  ViChar    mathWaveformHeader[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MATH_WAVEFORM_HEADER,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MATH_WAVEFORM_HEADER,
            bufferSize, mathWaveformHeader),
            3, "Math Waveform Header");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the reference waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchReferenceWaveform(ViSession instrSession,
                                                 ViInt32   waveformSize,
                                                 ViReal64  waveformArray[],
                                                 ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "REFC:DATA?");
    RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, waveformSize, waveformArray, actualPoints);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the data of the reference waveform points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples,
/// HIPAR waveformArray/or magnitudes of a spectrum.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchReferenceWaveformFloat(ViSession instrSession,
                                                      ViInt32   waveformSize,
                                                      ViReal32  waveformArray[],
                                                      ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "REFC:DATA?");
    RsCore_QuerySingleFloatArrayToUserBuffer(instrSession, cmd, waveformSize, waveformArray, actualPoints);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of reference waveform data. The header contains
/// HIFN  scaling and original attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR referenceWaveformHeader/Returns the header of reference waveform data. The header contains
/// HIPAR referenceWaveformHeader/scaling and original attributes of the waveform.
ViStatus _VI_FUNC rsscope_FetchReferenceWaveformHeader(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    referenceWaveformHeader[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_REFERENCE_WAVEFORM_HEADER,
            bufferSize, referenceWaveformHeader),
            3, "Reference Waveform Header");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the signal type of the source of the reference waveform.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_REFERENCE_WAVEFORM_SIGNAL_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     REFCurve<m>:DATA:STYPe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR signalType/Returns the signal type of the source of the reference waveform.
ViStatus _VI_FUNC rsscope_QueryReferenceWaveformSignalType(ViSession instrSession,
                                                           ViInt32   referenceNumber,
                                                           ViInt32*  signalType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceNumber, 1, 4),
            2, "Reference Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_REFERENCE_WAVEFORM_SIGNAL_TYPE,
            "ReferenceWfm", referenceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Reference Number");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_REFERENCE_WAVEFORM_SIGNAL_TYPE, signalType),
            3, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function obtains a main waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchMainWaveformMeasurement(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViInt32   statisticType,
                                                       ViInt32   bufferSize,
                                                       ViReal64  measurementResult[],
                                                       ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32   maxMeas = 0;
    ViChar*   response = NULL;
    ViReal64* results = NULL;
    ViAttr    arrMainMeasAttr[] = {
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_ACTUAL_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_AVERAGE_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_EVENT_COUNT_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_NEGATIVE_PEAK_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_POSITIVE_PEAK_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_RMS_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_STANDARD_DEVIATION_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_WAVEFORM_COUNT_STRING,
              RSSCOPE_ATTR_READ_MAIN_MEASUREMENT_RELIABILITY_STRING
              };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            3, "Statistic Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, arrMainMeasAttr[statisticType],
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                4, "Buffer Size")

    viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, repCap, arrMainMeasAttr[statisticType], RS_VAL_DIRECT_USER_CALL, &response),
                5, "Measurement Result");

    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, returnedValues));
    checkErr(RsCore_CopyToUserBufferViReal64Array(measurementResult, bufferSize, results, (ViInt64)(*returnedValues)));

Error:
    if (response) free(response);
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  his function obtains a waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.  You specify a particular
/// HIFN  measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchAmplitudeTimeWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViInt32   statisticType,
                            ViInt32   bufferSize,
                            ViReal64  measurementResult[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_HIGH, RSSCOPE_VAL_MEAS_ATAR),
            3, "Measurement Function");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngAmplitudeTimeMeasurementType, measurementFunction, &p2string),
                3, "Measurement Function");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            4, "Statistic Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                5, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MEAS%ld:RES:%s? %s", measurement, measurementStatisticsResultArr[statisticType],
             p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, measurementResult, returnedValues));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the start and stop times of the specified
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_FetchStartStopAmplitudeTimeWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2meas;
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                &rsscope_rngAmplitudeTimeMeasurementType, measurementFunction, &p2meas),
            3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STAR? %s", measurement, p2meas);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STOP? %s", measurement, p2meas);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function obtains a waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.  You specify a particular
/// HIFN  measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchEyeJitterWaveformMeasurement(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViInt32   measurementFunction,
                                                            ViInt32   statisticType,
                                                            ViInt32   bufferSize,
                                                            ViReal64  measurementResult[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas),
                2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_EXTINCTION_RATIO_PCT, RSSCOPE_VAL_MEAS_RMS_JITTER),
            3, "Measurement Function");
    switch (measurementFunction)
    {
    case 8:
    case 9:
    case 11:
    case 12:
    case 13:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementFunction),
                    3, "Measurement Function");
    }
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngEyeJitterMeasurementType, measurementFunction, &p2string),
                3, "Measurement Function");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            4, "Statistic Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                5, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MEAS%ld:RES:%s? %s", measurement,
             measurementStatisticsResultArr[statisticType], p2string);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, measurementResult, returnedValues));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the start and stop times of the specified
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_FetchStartStopEyeJitterWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");

    if ((measurementFunction < RSSCOPE_VAL_MEAS_EXTINCTION_RATIO_PCT) ||
        ((measurementFunction > RSSCOPE_VAL_MEAS_EYE_BASE) && (measurementFunction < RSSCOPE_VAL_MEAS_Q_FACTOR)) ||
        ((measurementFunction > RSSCOPE_VAL_MEAS_Q_FACTOR) && (measurementFunction < RSSCOPE_VAL_MEAS_RMS_NOISE)) ||
        ((measurementFunction > RSSCOPE_VAL_MEAS_EYE_AMPLITUDE) && (measurementFunction <
            RSSCOPE_VAL_MEAS_PEAK_TO_PEAK_JITTER)) ||
        (measurementFunction > RSSCOPE_VAL_MEAS_RMS_JITTER))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementFunction),
                    3, "Measurement Function");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STAR? %s", measurement, eyeJitterMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STOP? %s", measurement, eyeJitterMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function obtains a waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.  You specify a particular
/// HIFN  measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchHistogramWaveformMeasurement(ViSession instrSession,
                                                            ViInt32   measurement,
                                                            ViInt32   measurementFunction,
                                                            ViInt32   statisticType,
                                                            ViInt32   bufferSize,
                                                            ViReal64  measurementResult[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas),
                2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_WAVEFORM_COUNT, RSSCOPE_VAL_MEAS_MKN),
            3, "Measurement Function");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngHistogramMeasurementType, measurementFunction, &p2string),
                3, "Measurement Function");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            4, "Statistic Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                5, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MEAS%ld:RES:%s? %s", measurement,
             measurementStatisticsResultArr[statisticType], p2string);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, measurementResult, returnedValues));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the start and stop times of the specified
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_FetchStartStopHistogramWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_WAVEFORM_COUNT, RSSCOPE_VAL_MEAS_MKN),
            3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STAR? %s", measurement, histogramMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STOP? %s", measurement, histogramMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function obtains a waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.  You specify a particular
/// HIFN  measurement type, such as rise time, frequency, and voltage
/// HIFN  peak-to-peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchSpectrumWaveformMeasurement(ViSession instrSession,
                                                           ViInt32   measurement,
                                                           ViInt32   measurementFunction,
                                                           ViInt32   statisticType,
                                                           ViInt32   bufferSize,
                                                           ViReal64  measurementResult[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas),
                2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_CHANNEL_POWER, RSSCOPE_VAL_MEAS_SPAR),
            3, "Measurement Function");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngSpectrumMeasurementType, measurementFunction, &p2string),
                3, "Measurement Function");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            4, "Statistic Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                5, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MEAS%ld:RES:%s? %s", measurement,
             measurementStatisticsResultArr[statisticType], p2string);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, measurementResult, returnedValues));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the number of result groups that are returned by
/// HIFN  harmonics search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR numberOfResults/Returns the number of result groups that are returned by harmonics
/// HIPAR numberOfResults/search.
ViStatus _VI_FUNC rsscope_FetchSpectrumHarmonicsResultsNumber(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32*  numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_MEASUREMENT_HARMONICS_RESULTS_NUMBER,
            "Measurement", measurement- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_MEASUREMENT_HARMONICS_RESULTS_NUMBER, numberOfResults),
            2, "Number Of Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the start and stop times of the specified
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_FetchStartStopSpectrumWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  maxMeas = 0;
    ViChar*  p2meas;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsscope_rngSpectrumMeasurementType, measurementFunction, &p2meas),
            3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STAR? %s", measurement, p2meas);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STOP? %s", measurement, p2meas);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function obtains a waveform measurement on the channel you
/// HIFN  specify, and returns the measurement value.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) Only with RTO-K12 option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR statisticType/Select statistic result of measurement to be read.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Measurement Result'.
/// HIPAR measurementResult/Returns the measured value(s).
/// HIPAR returnedValues/Returns the count of values returned by the instrument. If this is
/// HIPAR returnedValues/larger than the allocated Buffer Size, not all values were returned in
/// HIPAR returnedValues/the buffer. Consider reallocating and fetching results again.
ViStatus _VI_FUNC rsscope_FetchJitterWaveformMeasurement(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   measurementFunction,
                                                         ViInt32   statisticType,
                                                         ViInt32   bufferSize,
                                                         ViReal64  measurementResult[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  maxMeas = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K12"));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas),
                2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_JITTER_CCJ, RSSCOPE_VAL_MEAS_JITTER_SKWP),
            3, "Measurement Function");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsscope_rngJitterMainMeasurement, measurementFunction, &p2string),
                3, "Measurement Function");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticType, RSSCOPE_VAL_STATISTIC_ACTUAL, RSSCOPE_VAL_STATISTIC_RELIABILITY),
            4, "Statistic Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
                5, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MEAS%ld:RES:%s? %s", measurement,
             measurementStatisticsResultArr[statisticType], p2string);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, measurementResult, returnedValues));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the start and stop times of the specified
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Selects measurement to be read.
/// HIPAR startTime/Return the start time of the specified measurement.
/// HIPAR stopTime/Return the stop time of the specified measurement.
ViStatus _VI_FUNC rsscope_FetchStartStopJitterWaveformMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   measurementFunction,
                            ViReal64* startTime,
                            ViReal64* stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  maxMeas = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Measurement", &maxMeas));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 1, maxMeas),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementFunction, RSSCOPE_VAL_MEAS_JITTER_CCJ, RSSCOPE_VAL_MEAS_JITTER_SKWP),
            3, "Measurement Function");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STAR? %s", measurement, jitterMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, startTime));

    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:RES:STOP? %s", measurement, jitterMeasFunctionArr[measurementFunction]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, stopTime));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function aborts an acquisition and returns the oscilloscope to
/// HIFN  the Idle state.  You initiate an acquisition with the
/// HIFN  rsscope_InitiateAcquisition function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_Abort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates or deactivates the power analysis
/// HIFN measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_ConfigurePowerAnalysisMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the channel for current source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR sourceNumber/Sets the number of configured current source.
/// HIPAR channel/Sets the chasnnel for selected current source.
ViStatus _VI_FUNC rsscope_ConfigurePowerAnalysisCurrentSource(
                            ViSession instrSession,
                            ViInt32   sourceNumber,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_ANALYSIS_CURRENT_SOURCE,
            "SourceNumber", sourceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Source Number");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_POWER_ANALYSIS_CURRENT_SOURCE, channel),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the channel for voltage source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR sourceNumber/Sets the number of configured current source.
/// HIPAR channel/Sets the chasnnel for selected voltage source.
ViStatus _VI_FUNC rsscope_ConfigurePowerAnalysisVoltageSource(
                            ViSession instrSession,
                            ViInt32   sourceNumber,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_ANALYSIS_VOLTAGE_SOURCE,
            "SourceNumber", sourceNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Source Number");
    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_POWER_ANALYSIS_VOLTAGE_SOURCE, channel),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the auto deskew.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read
/// HIPAR maximumTime/waveform operation to complete.
ViStatus _VI_FUNC rsscope_StartPowerAnalysisDeskew(ViSession instrSession,
                                                   ViInt32   maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &old_timeout));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));
    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_START_DESKEW, NULL),
            2, "Maximum Time");

Error:
    (void)rsscope_SetOPCTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function overwrites the present skew setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR state/This control enables the power analysis.
ViStatus _VI_FUNC rsscope_ResetPowerAnalysisSkew(ViSession instrSession,
                                                 ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DESKEW_RESET, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates or deactivates a user defined setup. If
/// HIFN ON, the instrument setup including probe setup and the deskew
/// HIFN values are written to a user defined preset file (saveset) that
/// HIFN can be loaded using MMEMory:RCL
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR state/Document parameter here.
ViStatus _VI_FUNC rsscope_UserDefinedPowerAnalysisSkew(ViSession instrSession,
                                                       ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DESKEW_USER_DEFINED_RESET, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function selects the input voltages to be measured by the
/// HIFN Probe Meter of an R&S differential active probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR channel/This control selects the channel.
/// HIPAR mode/Selects the voltage measurement mode.
ViStatus _VI_FUNC rsscope_PowerAnalysisProbeMode(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CHANNEL_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_POWER_ANALYSE_PROBE_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the result of the auto deskew.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available only if "Reset Power Analysis Skew" is disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deskewValue/Queries the result of the auto deskew.
ViStatus _VI_FUNC rsscope_QueryPowerAnalysisAutoDeskewValue(ViSession instrSession,
                                                            ViReal64* deskewValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DESKEW_TIME, deskewValue),
            2, "Deskew Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies the result of the auto deskew to the "Skew
/// HIFN  offset" value.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available only if "Reset Power Analysis Skew" is disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_PowerAnalysisAutoDeskewToOffset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DESKEW_CURRENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manages the selected measurement into the report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the report managenemt operation.
/// HIPAR measurement/This control selects the measurement type, should be added into
/// HIPAR measurement/report.
/// HIPAR index/This control sets the index in report, the measurement should be
/// HIPAR index/inserted/removed.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportManagement(ViSession instrSession,
                                                        ViInt32   operation,
                                                        ViInt32   measurement,
                                                        ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_ADD:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_ADD, measurement),
                3, "Measurement");

        break;
    case RSSCOPE_VAL_INSERT:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &
                        p2string),
                    3, "Measurement Function");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:INS %s,%ld", p2string, index);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSCOPE_VAL_REMOVE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &
                        p2string),
                    2, "Measurement");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:REM %s,%ld", p2string, index);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns LSend
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/This control selects the measurement type, should be added into
/// HIPAR measurement/report.
/// HIPAR returnedValue/Returns LSend value
ViStatus _VI_FUNC rsscope_PowerAnalysisReportLSend(ViSession instrSession,
                                                   ViInt32   measurement,
                                                   ViChar    returnedValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &p2parameter
                ),
                2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:LSEN? %s", p2parameter);
    checkErr(rsscope_QueryViString(instrSession, cmd, 512, returnedValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the comment into selected measurement..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects the measurement type, should be added into report.
/// HIPAR comment/Sets the comment to measurement.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportComment(ViSession instrSession,
                                                     ViInt32   measurement,
                                                     ViString  comment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &p2string),
                2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:COMM %s,'%s'", p2string, comment);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manages the selection of the result reports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the report management operation.
/// HIPAR measurement/Selects the measurement type, should be added into report.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportSelection(ViSession instrSession,
                                                       ViInt32   operation,
                                                       ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_POWER_ANALYSIS_SELECT_ALL:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_SEL_ALL, measurement),
                3, "Measurement");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_SELECT_NONE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_SEL_NONE, measurement),
                3, "Measurement");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_SELECT_REMOVE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_SEL_REMOVE, measurement),
                3, "Measurement");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_SELECT_INVERT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_SEL_INVERT, measurement),
                3, "Measurement");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the directory, in which the reports are saved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/This control selects the measurement type, should be added into report.
/// HIPAR directory/This control sets the comment to measurement.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportDirectory(ViSession instrSession,
                                                       ViInt32   measurement,
                                                       ViString  directory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &p2string),
                2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:DIR %s,'%s'", p2string, directory);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manages the report file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the file managenemt operation.
/// HIPAR fileName/This control selects the file name.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportFileManagement(ViSession instrSession,
                                                            ViInt32   operation,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_POWER_ANALYSIS_FILE_NEW:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FILE_NEW, fileName),
                3, "File Name");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_FILE_NAME:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FILE_NAME, fileName),
                3, "File Name");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_FILE_SAVE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FILE_SAVE, fileName),
                3, "File Name");
        break;
    case RSSCOPE_VAL_POWER_ANALYSIS_FILE_DELETE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FILE_DELETE, fileName),
                3, "File Name");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the informations about DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR DUTName/Sets the name for your DUT.
/// HIPAR user/Sets the user name.
/// HIPAR site/Sets the site name
/// HIPAR temperature/Sets the temperature
/// HIPAR description/Sets the description
ViStatus _VI_FUNC rsscope_PowerAnalysisReportDUT(ViSession instrSession,
                                                 ViString  DUTName,
                                                 ViString  user,
                                                 ViString  site,
                                                 ViInt32   temperature,
                                                 ViString  description)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DUT_NAME, DUTName),
            2, "DUT Name");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_USER, user),
            3, "User");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_SITE, site),
            4, "Site");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_TEMPERATURE, temperature),
            5, "Temperature");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_DESCRIPTION, description),
            6, "Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the page layout of report.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR paperSize/Sets the layout of the report
/// HIPAR font/Selects the font family.
/// HIPAR fontSize/Sets the font size
/// HIPAR colorFont/Sets the font color
/// HIPAR logo/Sets the path to logo picture file.
/// HIPAR invertScreenshotColors/Defines the invert screenshot color state.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportLayout(ViSession instrSession,
                                                    ViInt32   paperSize,
                                                    ViInt32   font,
                                                    ViInt32   fontSize,
                                                    ViInt32   colorFont,
                                                    ViString  logo,
                                                    ViBoolean invertScreenshotColors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_PAPER_SIZE, paperSize),
            2, "Paper Size");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FONT, font),
            3, "Font");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FONT_SIZE, fontSize),
            4, "Font Size");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_FONT_COLOR, colorFont),
            5, "Color Font");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_LOGO, logo),
            6, "Logo");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_INV_COLOR, invertScreenshotColors),
            7, "Invert Screenshot Colors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the contents of your report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR title/Adds a Title page to the report.
/// HIPAR measurementSetup/Adds a graphic of the measurement setup.
/// HIPAR settings/Adds the settings of the analysis.
/// HIPAR verticalSetup/Adds the vertical setup settings.
/// HIPAR triggerSetup/Adds the trigger setup settings.
/// HIPAR horizontalSetup/Adds the horizontal setup settings.
/// HIPAR measuredSignals/Adds a diagram of the measured signal.
/// HIPAR results/Adds the result box.
ViStatus _VI_FUNC rsscope_PowerAnalysisReportContent(ViSession instrSession,
                                                     ViBoolean title,
                                                     ViInt32   measurementSetup,
                                                     ViInt32   settings,
                                                     ViInt32   verticalSetup,
                                                     ViInt32   triggerSetup,
                                                     ViInt32   horizontalSetup,
                                                     ViInt32   measuredSignals,
                                                     ViInt32   results)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_TITLE, title),
            2, "Title");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_MSETUP, measurementSetup),
            3, "Measurememnt Setup");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_SETTINGS, settings),
            4, "Settings");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_VSETUP, verticalSetup),
            5, "Vertical Setup");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_TSETUP, triggerSetup),
            6, "Trigger Setup");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_HSETUP, horizontalSetup),
            7, "Horizontal Setup");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_MEAS, measuredSignals),
            8, "Measured Signal");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ANALYSIS_REPORT_CONTENT_RESULTS, results),
            9, "Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurement/Selects the measurement type.
/// HIPAR count/This control returns the count.
ViStatus _VI_FUNC rsscope_GetPowerAnalysisReportCount(ViSession instrSession,
                                                      ViInt32   measurement,
                                                      ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerAnalysisMeasurementType, measurement, &p2string),
                2, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:REP:TEST:COUN? %s", p2string);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, count));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power quality measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function..
ViStatus _VI_FUNC rsscope_StartPowerQualityMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_QUALITY_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the input frequency of the source signal in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function..
/// HIPAR frequency/Document parameter here.
ViStatus _VI_FUNC rsscope_PowerQualityInputFrequency(ViSession instrSession,
                                                     ViInt32   frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_QUALITY_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the user-defined frequency if
/// HIFN  rsscope_PowerQualityInputFrequency is set to Custom.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR customFrequency/Sets the user-defined frequency if rsscope_PowerQualityInputFrequency
/// HIPAR customFrequency/is set to Custom.
ViStatus _VI_FUNC rsscope_PowerQualityCustomFrequency(ViSession instrSession,
                                                      ViReal64  customFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_QUALITY_CUSTOM_FREQUENCY, customFrequency),
            2, "Custom Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the scaling for the display of the results
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function..
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerQualityScaling(ViSession instrSession,
                                              ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_QUALITY_SCALING, scaling),
            2, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerQualityAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_QUALITY_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the value of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Selects quality type.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsscope_QueryPowerQualityMeasurementVoltageResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, measurementResult), 3, "Measurement Result");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngQualityType, type, &p2string),
                2, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:QUAL:RES:VOLT:%s?", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the value of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Selects quality type.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsscope_QueryPowerQualityMeasurementCurrentResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, measurementResult), 3, "Measurement Result");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngQualityType, type, &p2string),
                2, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:QUAL:RES:CURR:%s?", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the value of the specified measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Selects quality type.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsscope_QueryPowerQualityMeasurementPowerResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, measurementResult), 3, "Measurement Result");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngQualityTypePower, type, &p2string),
                2, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:QUAL:RES:POW:%s?", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power current harmonics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerCurrentHarmonicsMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_CURRENT_HARMONICS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power current harmonics measurement
/// HIFN parameters
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR standard/Sets the standard in use.
/// HIPAR frequency/Sets the frequency of input signal.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerCurrentHarmonicsSettings(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   frequency,
                                                        ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_STANDARD, standard),
            2, "Standard");
    switch (standard)
    {
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_ENA:
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_ENB:
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_ENC:
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_END:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_EN_FREQUENCY, frequency),
                3, "Frequency");
        break;
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_MIL:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_MI_FREQUENCY, frequency),
                3, "Frequency");
        break;
    case RSSCOPE_VAL_CURRENT_HARMONICS_STANDARD_RTC:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_DO_FREQUENCY, frequency),
                3, "Frequency");
        break;
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_SCALING, scaling),
            4, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects how the results are evaluated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function..
/// HIPAR revisedCurrent/Sets how the results are evaluated
/// HIPAR voltageResultDisplay/Sets the voltage display
ViStatus _VI_FUNC rsscope_PowerCurrentHarmonicsEvaluation(ViSession instrSession,
                                                          ViInt32   revisedCurrent,
                                                          ViInt32   voltageResultDisplay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_EVALUATION, revisedCurrent),
            2, "Revised Current");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_EVALUATION_VOLT, voltageResultDisplay),
            3, "Voltage Result Display");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerCurrentHarmonicAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the used standard for selected result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR resultNumber/Result number.
/// HIPAR standard/Returns the used standard.
ViStatus _VI_FUNC rsscope_QueryPowerCurrentHarmonicsUsedStandard(
                            ViSession instrSession,
                            ViInt32   resultNumber,
                            ViInt32*  standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_USED_STANDARD,
            "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Result Number");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_USED_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR type/Selects harmonics type.
/// HIPAR resultNumber/Document parameter here.
/// HIPAR harmonic/Document parameter here.
/// HIPAR measurementResult/Document parameter here.
ViStatus _VI_FUNC rsscope_QueryPowerCurrentHarmonicsResult(ViSession instrSession,
                                                           ViInt32   type,
                                                           ViInt32   resultNumber,
                                                           ViInt32   harmonic,
                                                           ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_harm[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSCOPE_VAL_HARMONICS_TYPE_FREQUENCY:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_FREQUENCY,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Result Number");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_FREQUENCY,
                "ResultHarmonics", harmonic- 1, RS_REPCAP_BUF_SIZE, repCap_harm),
                4, "Harmonic");
        strcat(repCap, ",");
        strcat(repCap, repCap_harm);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_FREQUENCY, measurementResult),
                5, "Measurement Result");
        break;
    case RSSCOPE_VAL_HARMONICS_TYPE_VALUE:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_VALUE,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Result Number");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_VALUE,
                "ResultHarmonics", harmonic- 1, RS_REPCAP_BUF_SIZE, repCap_harm),
                4, "Harmonic");
        strcat(repCap, ",");
        strcat(repCap, repCap_harm);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_VALUE, measurementResult),
                5, "Measurement Result");
        break;
    case RSSCOPE_VAL_HARMONICS_TYPE_MAXIMUM:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_MAX,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Result Number");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_MAX,
                "ResultHarmonics", harmonic- 1, RS_REPCAP_BUF_SIZE, repCap_harm),
                4, "Harmonic");
        strcat(repCap, ",");
        strcat(repCap, repCap_harm);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_MAX, measurementResult),
                5, "Measurement Result");
        break;
    case RSSCOPE_VAL_HARMONICS_TYPE_LIMIT:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_STD,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Result Number");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_STD,
                "ResultHarmonics", harmonic- 1, RS_REPCAP_BUF_SIZE, repCap_harm),
                4, "Harmonic");
        strcat(repCap, ",");
        strcat(repCap, repCap_harm);
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_CURRENT_HARMONICS_RESULT_STD, measurementResult),
                5, "Measurement Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power inrush current measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerInrushCurrentMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_INRUSH_CURRENT_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power inrush current measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR maxCurrent/Sets the maximum expected current for the vertical scale.
/// HIPAR trigger/Sets the current value for the trigger.
ViStatus _VI_FUNC rsscope_PowerInrushCurrentSettings(ViSession instrSession,
                                                     ViReal64  maxCurrent,
                                                     ViReal64  trigger)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_MAX, maxCurrent),
            2, "Max Current");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_TRIGGER, trigger),
            3, "Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the selected gate (time period)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR gate/Sets the gate number
/// HIPAR start/Sets the start measuring time for the selected gate.
/// HIPAR stop/Sets the stop measuring time for the selected gate.
ViStatus _VI_FUNC rsscope_PowerInrushCurrentGateSettings(ViSession instrSession,
                                                         ViInt32   gate,
                                                         ViReal64  start,
                                                         ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_START,
            "Gate", gate- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_START, start),
            3, "Start");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_STOP,
            "Gate", gate- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_STOP, stop),
            3, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the value of the inrush current.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR gate/Sets the gate number
/// HIPAR value/Sets the value of the inrush current.
ViStatus _VI_FUNC rsscope_PowerInrushCurrentGateValue(ViSession instrSession,
                                                      ViInt32   gate,
                                                      ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_VALUE,
            "Gate", gate- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_VALUE, value),
            3, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manages the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the report managenemt operation.
/// HIPAR gateIndex/Sets the gate index.
ViStatus _VI_FUNC rsscope_PowerInrushCurrentGateManagement(ViSession instrSession,
                                                           ViInt32   operation,
                                                           ViInt32   gateIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_ADD:
        checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_ADD, NULL));
        break;
    case RSSCOPE_VAL_INSERT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_INS, gateIndex),
                3, "Gate Index");
        break;
    case RSSCOPE_VAL_REMOVE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_REMOVE, gateIndex),
                3, "Gate Index");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerInrushCurrentGateAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of inrush current gates.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR count/Returns the number of inrush gates
ViStatus _VI_FUNC rsscope_QueryInrushCurrentGateCount(ViSession instrSession,
                                                      ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_INRUSH_CURRENT_GATE_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the result of the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR gateNumber/Gate number.
/// HIPAR measurementResult/Returns the result of the selected measurement.

ViStatus _VI_FUNC rsscope_QueryPowerInrushCurrentMeasurementResult(
                            ViSession instrSession,
                            ViInt32   gateNumber,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_RESULT_VALUE,
            "Gate", gateNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_INRUSH_CURRENT_RESULT_VALUE, measurementResult),
            3, "Measurement Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power ripple measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerRippleMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_RIPPLE_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the input current.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates or deactivates the input current..
ViStatus _VI_FUNC rsscope_PowerRippleCurrent(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_RIPPLE_CURRENT_ENABLED, state),
            2, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power ripple measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR frequency/Sets the SMPS switching frequency.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerRippleSettings(ViSession instrSession,
                                              ViReal64  frequency,
                                              ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_RIPPLE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_RIPPLE_SCALING, scaling),
            3, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerRippleAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_RIPPLE_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the specified statistic result of the specified
/// HIFN measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measType/Sets the measurement type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsscope_QueryPowerRippleMeasurementResult(ViSession instrSession,
                                                            ViInt32   statistics,
                                                            ViInt32   measType,
                                                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2measType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerRippleType, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSource, measType, &p2measType),
                2, "Meas Type");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:RIPP:RES:%s? %s", p2string, p2measType);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the specified statistic result of the specified measurement
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR statistics/Selects statistic value.
/// HIPAR rippleType/Selects measure type.
/// HIPAR measType/Sets the measurement type.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsscope_QueryPowerRippleMeasureResult(ViSession instrSession,
                                                        ViInt32   statistics,
                                                        ViInt32   rippleType,
                                                        ViInt32   measType,
                                                        ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2rippleType;
    ViChar*  p2measType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerRippleType, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerRippleMeasure, rippleType, &p2rippleType),
                3, "Ripple Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSource, measType, &p2measType),
                4, "Meas Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:RIPP:RES:%s:%s? %s", p2string, p2rippleType, p2measType);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power ripple measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerSpectrumMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_SPECTRUM_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power output spectrum measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR frequency/Sets the SMPS switching frequency.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerSpectrumSettings(ViSession instrSession,
                                                ViReal64  frequency,
                                                ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SPECTRUM_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_SPECTRUM_SCALING, scaling),
            3, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerSpectrumAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_SPECTRUM_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the specified statistic result of the specified
/// HIFN measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR resultNumber/Result number.
/// HIPAR type/Selects spectrum measurement type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsscope_QueryPowerSpectrumMeasurementResult(
                            ViSession instrSession,
                            ViInt32   resultNumber,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSCOPE_VAL_SPECTRUM_TYPE_FREQUENCY:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_SPECTRUM_MEASUREMENT_FREQUENCY_RESULT,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Result Number");
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_SPECTRUM_MEASUREMENT_FREQUENCY_RESULT, measurementResult),
                3, "Measurement Result");
        break;
    case RSSCOPE_VAL_SPECTRUM_TYPE_LEVEL:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_SPECTRUM_MEASUREMENT_LEVEL_RESULT,
                "ResultNumber", resultNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Result Number");
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_SPECTRUM_MEASUREMENT_LEVEL_RESULT, measurementResult),
                3, "Measurement Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 3, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the total number of harmonics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR harmonicsCount/Returns the total number of harmonics.
ViStatus _VI_FUNC rsscope_QueryPowerSpectrumHarmonicsCount(ViSession instrSession,
                                                           ViInt32*  harmonicsCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_SPECTRUM_MEASUREMENT_RESULT_COUNT, harmonicsCount),
            2, "Harmonics Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power ripple measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerSlewRateMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_SLEW_RATE_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power slew rate measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR source/Selects the source for the measurement.
/// HIPAR average/Enables/disables averaging.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerSlewRateSettings(ViSession instrSession,
                                                ViInt32   source,
                                                ViBoolean average,
                                                ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_SOURCE, source),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_AVERAGE, average),
            3, "Average");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_SCALING, scaling),
            4, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerSlewRateAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the gate (time period)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR start/Sets the start measuring time for the gate.
/// HIPAR stop/Sets the stop measuring time for the gate.
ViStatus _VI_FUNC rsscope_PowerSlewRateGateSettings(ViSession instrSession,
                                                    ViReal64  start,
                                                    ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_GATE_START, start),
            2, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SLEW_RATE_GATE_STOP, stop),
            3, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR statistics/Selects statistic value.
/// HIPAR type/Selects modulation measurement type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsscope_QueryPowerSlewRateMeasurementResult(
                            ViSession instrSession,
                            ViInt32   statistics,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerStatistics, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSlewRateResultType, type, &p2type),
                2, "Meas Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:SLEW:RES:%s? %s", p2string, p2type);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the safe operating area measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerSafeOperatingAreaMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_SAFE_OPERATING_AREA_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the safe operation area parameters
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR maskTest/Enables a mask test.
/// HIPAR scale/Selects the scale for the result diagram.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaSettings(ViSession instrSession,
                                                         ViBoolean maskTest,
                                                         ViInt32   scale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_MASK_TEST, maskTest),
            2, "Mask Test");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_SCALE, scale),
            3, "Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches between linear and logarithmic scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scale/Switches between linear and logarithmic scale.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaSwitch(ViSession instrSession,
                                                       ViInt32   scale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_SWITCH, scale),
            3, "Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manages the safe operating area linear point
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the operation.
/// HIPAR pointNumber/Sets the point number.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaLinPointManagement(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViInt32   pointNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_ADD:
        checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LIN_ADD, ""));
        break;
    case RSSCOPE_VAL_REMOVE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LIN_REMOVE, pointNumber),
                3, "Point Number");
        break;
    case RSSCOPE_VAL_INSERT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LIN_INS, pointNumber),
                3, "Point Number");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the voltage/current value of the SOA point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the value type.
/// HIPAR value/Sets the point value.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaLinPointValue(
                            ViSession instrSession,
                            ViInt32   pointNumber,
                            ViInt32   valueType,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViReal64 val_min = 1.0e-3;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSource, valueType, &p2string),
                3, "Value Type");
    switch (valueType)
    {
    case RSSCOPE_VAL_POWER_SOURCE_VOLTAGE:
        val_min = 1.0e-3;
        break;
    case RSSCOPE_VAL_POWER_SOURCE_CURRENT:
        val_min = 0.01;
        break;
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, value, val_min, 1000.0), 4, "Value");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:SOA:LIN:POIN%ld:%s %lf", pointNumber, p2string, value);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function manages the safe operating area logarithm point
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR operation/Selects the operation.
/// HIPAR pointNumber/Sets the point number.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaLogPointManagement(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViInt32   pointNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_ADD:
        checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LOG_ADD, ""));
        break;
    case RSSCOPE_VAL_REMOVE:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LOG_REMOVE, pointNumber),
                3, "Point Number");
        break;
    case RSSCOPE_VAL_INSERT:
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LOG_INS, pointNumber),
                3, "Point Number");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the voltage/current value of the SOA point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the value type.
/// HIPAR value/Sets the point value.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaLogPointValue(
                            ViSession instrSession,
                            ViInt32   pointNumber,
                            ViInt32   valueType,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSource, valueType, &p2string),
                3, "Value Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:SOA:LOG:POIN%ld:%s %.12lG", pointNumber, p2string, value);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerSafeOperatingAreaAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_SAFE_OPERATING_AREA_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR count/Returns the number of points
ViStatus _VI_FUNC rsscope_QueryPowerSafeOperatingAreaLinCount(
                            ViSession instrSession,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LIN_COUNT, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR count/Returns the number of points.
ViStatus _VI_FUNC rsscope_QueryPowerSafeOperatingAreaLogCount(
                            ViSession instrSession,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SAFE_OPERATING_AREA_LOG_COUNT, count));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power modulation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerModulationMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_MODULATION_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power modulation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR source/Selects the source for the measurement.
/// HIPAR type/Selects the type of signal flow for the measurement.
/// HIPAR displayHistogram/Activates or deactivates the display of a histogram.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerModulationSettings(ViSession instrSession,
                                                  ViInt32   source,
                                                  ViInt32   type,
                                                  ViBoolean displayHistogram,
                                                  ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_MODULATION_SOURCE, source),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_MODULATION_TYPE, type),
            3, "Type");
    if (type == RSSCOPE_VAL_MODULATION_TYPE_CONT)
        viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_MODULATION_HISTOGRAM, displayHistogram),
                4, "Display Histogram");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_MODULATION_SCALING, scaling),
            5, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerModulationAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_MODULATION_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR statistics/Selects statistic value.
/// HIPAR type/Selects modulation measurement type.
/// HIPAR measurementResult/Returns the result of the selected me
ViStatus _VI_FUNC rsscope_QueryPowerModulationMeasurementResult(
                            ViSession instrSession,
                            ViInt32   statistics,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerStatistics, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerModulationResultType, type, &p2type),
                2, "Meas Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:MOD:RES:%s? %s", p2string, p2type);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power dynamic on resistance measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerDynamicOnResistanceMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power dynamic on resistance
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR average/Enables/disables averaging.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerDynamicOnResistanceSettings(ViSession instrSession,
                                                           ViBoolean average,
                                                           ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_AVERAGE, average),
            2, "Average");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_SCALING, scaling),
            3, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerDynamicOnResistanceAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the selected gate (time period)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR gate/Sets the gate number
/// HIPAR start/Sets the start measuring time for the selected gate.
/// HIPAR stop/Sets the stop measuring time for the selected gate.
ViStatus _VI_FUNC rsscope_PowerDynamicOnResistanceGateSettings(
                            ViSession instrSession,
                            ViInt32   gate,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_GATE_START,
            "Gate", gate- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_GATE_START, start),
            3, "Start");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_GATE_STOP,
            "Gate", gate- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Gate");
    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_GATE_STOP, stop),
            3, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the result of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsscope_QueryPowerDynamicOnResistanceMeasurementResult(
                            ViSession instrSession,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_DYNAMIC_ON_RESISTANCE_RESULT, measurementResult),
            3, "Measurement Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power efficiency measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerEfficiencyMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_EFFICIENCY_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power efficiency scaling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerEfficiencyScaling(ViSession instrSession,
                                                 ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_EFFICIENCY_SCALING, scaling),
            3, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerEfficiencyAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_EFFICIENCY_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the specified statistic result of the specified
/// HIFN measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR resultNumber/Result number.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsscope_QueryPowerEfficiencyMeasurementResult(
                            ViSession instrSession,
                            ViInt32   resultNumber,
                            ViInt32   statistics,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerStatistics, statistics, &p2string),
                2, "Statistics");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:EFF:RES%ld:%s?", resultNumber, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power switchingloss measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerSwitchinglossMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_SWITCHINGLOSS_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power switchingloss measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR switchingFrequency/Enables the measurements of the switching frequency.
/// HIPAR turnOn/Enables the measurements during the turn on period
/// HIPAR conduction/Enables the measurements during the conduction period
/// HIPAR turnOff/Enables the measurements during the turn off period
/// HIPAR nonConduction/Enables the measurements during the non conduction period
/// HIPAR total/Enables the measurement of the total period
/// HIPAR scaling/Sets the scaling for display results.
ViStatus _VI_FUNC rsscope_PowerSwitchinglossSettings(ViSession instrSession,
                                                     ViBoolean switchingFrequency,
                                                     ViBoolean turnOn,
                                                     ViBoolean conduction,
                                                     ViBoolean turnOff,
                                                     ViBoolean nonConduction,
                                                     ViBoolean total,
                                                     ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_SWITCHING_FREQUENCY_STATE, switchingFrequency),
            2, "Switching Frequency");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_TON_STATE, turnOn),
            3, "Turn On");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_CONDUCTION_STATE, conduction),
            4, "Conduction");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_TOF_STATE, turnOff),
            5, "Turn Off");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_NON_CONDUCTION_STATE, nonConduction),
            6, "Non Conduction");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_TOTAL_STATE, total),
            7, "Total");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_SCALING, scaling),
            8, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the switching loss frequency value. This
/// HIFN value can be set, if the switching frequency measurement is set
/// HIFN Off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR switchingFrequency/Sets the switching frequency value.
ViStatus _VI_FUNC rsscope_PowerSwitchinglossFrequency(ViSession instrSession,
                                                      ViReal64  switchingFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_SWITCHING_FREQUENCY_VALUE, switchingFrequency),
            2, "Switching Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the gate parameters for selected
/// HIFN measurement period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR period/Selects switchingloss measurement type.
/// HIPAR start/Sets the value for the respective cursor.
/// HIPAR stop/Sets the value for the respective cursor.
ViStatus _VI_FUNC rsscope_PowerSwitchinglossGateSettings(ViSession instrSession,
                                                         ViInt32   period,
                                                         ViReal64  start,
                                                         ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (period)
    {
    case RSSCOPE_VAL_POWER_SWITCHINGLOSS_TYPE_TOF:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_TURN_OFF_START, start),
                3, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_TURN_OFF_STOP, stop),
                4, "Stop");
        break;
    case RSSCOPE_VAL_POWER_SWITCHINGLOSS_TYPE_TON:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_TURN_ON_START, start),
                3, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_TURN_ON_STOP, stop),
                4, "Stop");
        break;
    case RSSCOPE_VAL_POWER_SWITCHINGLOSS_TYPE_CON:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_CONDUCTION_START, start),
                3, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_CONDUCTION_STOP, stop),
                4, "Stop");
        break;
    case RSSCOPE_VAL_POWER_SWITCHINGLOSS_TYPE_NCO:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_NO_CONDUCTION_START, start),
                3, "Start");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_GATE_NO_CONDUCTION_STOP, stop),
                4, "Stop");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, period), 2, "Period");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerSwitchinglossAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_SWITCHINGLOSS_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the specified statistic result of the specified
/// HIFN measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR statistics/Selects statistic value.
/// HIPAR type/Selects switchingloss measurement type.
/// HIPAR measurementResult/Returns the result of the measurement.
ViStatus _VI_FUNC rsscope_QueryPowerSwitchinglossEnergyResult(
                            ViSession instrSession,
                            ViInt32   statistics,
                            ViInt32   type,
                            ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerStatistics, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSwitchLossMeasTypes, type, &p2type),
                2, "Type");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:SWIT:RES:ENER:%s? %s", p2string, p2type);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the specified statistic result of the specified
/// HIFN measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR statistics/Selects statistic value.
/// HIPAR type/Selects switchingloss measurement type.
/// HIPAR measurementResult/Returns the result of the measurement.
ViStatus _VI_FUNC rsscope_QueryPowerSwitchinglossPowerResult(ViSession instrSession,
                                                             ViInt32   statistics,
                                                             ViInt32   type,
                                                             ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerStatistics, statistics, &p2string),
                2, "Statistics");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerSwitchLossMeasTypes, type, &p2type),
                2, "Type");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:SWIT:RES:POW:%s? %s", p2string, p2type);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, measurementResult));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power on off measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerOnOffMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_ON_OFF_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the input type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR input/Sets the input type.
ViStatus _VI_FUNC rsscope_PowerOnOffInputType(ViSession instrSession,
                                              ViInt32   input)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ON_OFF_INPUT_TYPE, input),
            2, "Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power on off measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Sets the On/Off measurement type.
/// HIPAR state/Reserved for future use
/// HIPAR triggerLevel/Sets the trigger level value.
/// HIPAR steadyStateLevel/Sets the percentage of the steady state level of the DC output
/// HIPAR steadyStateLevel/that has to be reached.
/// HIPAR time/Sets the time the start of the measurement of the turn off time
/// HIPAR time/is delay with, after the trigger point.
ViStatus _VI_FUNC rsscope_PowerOnOffSettings(ViSession instrSession,
                                             ViInt32   type,
                                             ViBoolean reserved,
                                             ViReal64  triggerLevel,
                                             ViReal64  steadyStateLevel,
                                             ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    reserved; //reserved for future use

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_ON_OFF_TYPE, type),
            2, "Type");
    switch (type)
    {
    case RSSCOPE_VAL_ON_OFF_TYPE_OFF:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_OFF_TRIGGER_LEVEL, triggerLevel),
                4, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_OFF_STEADY_STATE_LEVEL, steadyStateLevel),
                5, "Steady State Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_ON_OFF_TIME, time),
                6, "Time");
        break;
    case RSSCOPE_VAL_ON_OFF_TYPE_ON:
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_ON_TRIGGER_LEVEL, triggerLevel),
                4, "Trigger Level");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_ON_STEADY_STATE_LEVEL, steadyStateLevel),
                5, "Steady State Level");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger level for DC input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the On/Off measurement type.
/// HIPAR triggerLevel/Sets the trigger level value. Triggers the beginning of the
/// HIPAR triggerLevel/measurements at the moment the DC voltage reaches the set value.
ViStatus _VI_FUNC rsscope_PowerOnOffDCLevel(ViSession instrSession,
                                            ViInt32   type,
                                            ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSSCOPE_ATTR_POWER_OFF_TRIGGER_LEVEL_DC, RSSCOPE_ATTR_POWER_ON_TRIGGER_LEVEL_DC};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_ON_OFF_TYPE_OFF, RSSCOPE_VAL_ON_OFF_TYPE_ON),
            2, "Type");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", attr[type], triggerLevel),
            3, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_PowerOnOffAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_ON_OFF_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the result of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Selects the result type.
/// HIPAR result/Returns the turn On/Off time
ViStatus _VI_FUNC rsscope_QueryPowerOnOffMeasurementResult(ViSession instrSession,
                                                           ViInt32   type,
                                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSCOPE_VAL_ON_OFF_TYPE_OFF:
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_OFF_RESULT, result),
                3, "Result");
        break;
    case RSSCOPE_VAL_ON_OFF_TYPE_ON:
        viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_ON_RESULT, result),
                3, "Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts the power transient response measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_StartPowerTransientResponseMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_START_POWER_TRANSIENT_RESPONSE_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates/deactivates the input voltage
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR inputVoltage/Activates or deactivates the input voltage.
ViStatus _VI_FUNC rsscope_TransientResponseInputVoltage(ViSession instrSession,
                                                        ViBoolean inputVoltage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_INPUT_STATE, inputVoltage),
            2, "Input Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures power transient response measurement
/// HIFN Ttrigger
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR source/Selects the source channel of the trigger.
/// HIPAR level/Sets the trigger level.
/// HIPAR edgeTriggerSlope/Sets the slope for the edge trigger.
ViStatus _VI_FUNC rsscope_TransientResponseTrigger(ViSession instrSession,
                                                   ViInt32   source,
                                                   ViReal64  level,
                                                   ViInt32   edgeTriggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_TRIGGER_SOURCE, source),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_TRIGGER_LEVEL, level),
            3, "Level");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_TRIGGER_SLOPE, edgeTriggerSlope),
            4, "Edge Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power transient response
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR frequency/Sets the SMPS switching frequency.
/// HIPAR scaling/Sets the scaling for display results.
/// HIPAR tubeTolerance/Sets a tolerated error band for the signal level.
/// HIPAR signalHigh/Sets the expected signal high voltage value.
/// HIPAR signalLow/Sets the expected signal low voltage value.
ViStatus _VI_FUNC rsscope_TransientResponseSettings(ViSession instrSession,
                                                    ViReal64  frequency,
                                                    ViInt32   scaling,
                                                    ViInt32   tubeTolerance,
                                                    ViReal64  signalHigh,
                                                    ViReal64  signalLow)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_SCALING, scaling),
            3, "Scaling");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_TUBE_TOLERANCE, tubeTolerance),
            4, "Tube Tolerance");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_HIGH, signalHigh),
            5, "Signal High");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_LOW, signalLow),
            6, "Signal Low");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_TransientResponseAddToReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_POWER_TRANSIENT_RESPONSE_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the results of the transient response
/// HIFN measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR type/Sets the resul type.
/// HIPAR result/Returns the result.
ViStatus _VI_FUNC rsscope_QueryPowerTransientResponseResult(ViSession instrSession,
                                                            ViInt32   type,
                                                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngPowerTransientResponseResult, type, &p2string),
                2, "Statistics");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "POW:TRAN:RES? %s", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the deembedding - the correction of parasitic effects of the
/// HIFN  measurement setup on the measured signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR enabled/Activates the deembedding - the correction of parasitic effects of the
/// HIPAR enabled/measurement setup on the measured signal.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingEnabled(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_STATE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the maximum bandwidth until which the signal is corrected. This
/// HIFN  maximum value is the minimum bandwidth value of probe, probe tip and
/// HIFN  oscilloscope bandwidth, and can not be higher than the highest
/// HIFN  frequency in a used S-parameter file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR effectiveBandwidth/Sets the maximum bandwidth until which the signal is corrected. This
/// HIPAR effectiveBandwidth/maximum value is the minimum bandwidth value of probe, probe tip and
/// HIPAR effectiveBandwidth/oscilloscope bandwidth, and can not be higher than the highest
/// HIPAR effectiveBandwidth/frequency in a used S-parameter file.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingEffectiveBandwidth(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  effectiveBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_EFFECTIVE_BANDWIDTH,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_EFFECTIVE_BANDWIDTH, effectiveBandwidth),
            3, "Effective Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the compensation for the group delay of the complete
/// HIFN  measurement setup. If enabled, the calibration time reference point is
/// HIFN  the start point of the measurement setup. If disabled, the calibration
/// HIFN  time reference point is the oscilloscope's channel input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR compensation/Enables the compensation for the group delay of the complete
/// HIPAR compensation/measurement setup. If enabled, the calibration time reference point is
/// HIPAR compensation/the start point of the measurement setup. If disabled, the calibration
/// HIPAR compensation/time reference point is the oscilloscope's channel input.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingDelayCompensation(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean compensation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_DELAY_COMPENSATION,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_DELAY_COMPENSATION, compensation),
            3, "Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If ON, the deembedding calculation is done by the hardware in real
/// HIFN  time. You can trigger on the corrected signal, and the acquisition rate
/// HIFN  remains unchanged.
/// HIFN     If OFF, the deembedding calculation is done by software. This method
/// HIFN  is slower, therefore, triggering on the corrected signal is not
/// HIFN  possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects repcap Channel.
/// HIPAR enabled/If ON, the deembedding calculation is done by the hardware in real
/// HIPAR enabled/time. You can trigger on the corrected signal, and the acquisition rate
/// HIPAR enabled/remains unchanged.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingRealtimeEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_REALTIME_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_REALTIME_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a component to the measurement setup.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DEEMbedding<m>:ADD
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR index/Defines the place of the component in the measurement path. The DUT is
/// HIPAR index/always the first component (index = 1), the probe or input is the last
/// HIPAR index/component.
/// HIPAR type/Selects the type to be added.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingAddComponent(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   index,
                                                           ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K44"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_EXT),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_TYPE_CABLE, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_TYPE_EXTF),
            4, "Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsScope_rngDeembedingComponentType, type, &p2string), 4, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:ADD %ld,%s", channel, index, p2string);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the component with the indicated index from the measurement
/// HIFN  setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Deletes the component with the indicated index from the measurement
/// HIPAR component/setup.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingRemoveComponent(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_REMOVE_COMPONENT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_REMOVE_COMPONENT, component),
            3, "Component");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the total number of components in the measurement setup,
/// HIFN  including disabled components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR count/Returns the total number of components in the measurement setup,
/// HIPAR count/including disabled components.
ViStatus _VI_FUNC rsscope_QueryDeembeddingComponentCount(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_COUNT,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Deembeding file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR fileName/Sets the file name, file format and path of the deembedding setup
/// HIPAR fileName/file. The file format is always .xml.
/// HIPAR action/Do the action with the deembedding configuration file specified with
/// HIPAR action/RSSCOPE_ATTR_DEEMBEDDING_SETTINGS_FILE_NAME.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingFile(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViString  fileName,
                                                   ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSCOPE_ATTR_DEEMBEDDING_SAVE_SETTINGS,
             RSSCOPE_ATTR_DEEMBEDDING_LOAD_SETTINGS,
             RSSCOPE_ATTR_DEEMBEDDING_DELETE_SETTINGS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, RSSCOPE_VAL_DEEMBEDDING_FILE_ACTION_SAVE, RSSCOPE_VAL_DEEMBEDDING_FILE_ACTION_DELETE),
            4, "Action");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_SETTINGS_FILE_NAME,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_SETTINGS_FILE_NAME, fileName),
            3, "File Name");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, attr[action], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the complete deembedding configuration to the default values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingPreset(ViSession instrSession,
                                                     ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_PRESET,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the indicated component for deembedding. The first
/// HIFN  and the last component are always enabled and cannot be disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR enabled/Enables or disables the indicated component for deembedding. The first
/// HIPAR enabled/and the last component are always enabled and cannot be disabled.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld %s", channel, component, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the type of the indicated component. The first component is
/// HIFN  always DUT, the last component is PROBe or INPut.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR type/Sets the type of the indicated component. The first component is
/// HIPAR type/always DUT, the last component is PROBe or INPut.
/// HIPAR name/Sets a name for the indicated component that helps you identifying it.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentType(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   component,
                                                            ViInt32   type,
                                                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_EXT),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_TYPE_CABLE, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_TYPE_EXTF),
            4, "Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_SELECTED_COMPONENT, component),
            3, "Component");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_TYPE, type),
            4, "Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_NAME, name),
            5, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of all ports of the selected component (input ports
/// HIFN  and output ports). The number of ports depends on the attached probe.
/// HIFN  If a modular probe R&S RT-ZM or differential probe R&S& RT-ZD is used,
/// HIFN  the number of ports is 4. Otherwise, 2 ports are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR count/Returns the number of all ports of the selected component (input ports
/// HIPAR count/and output ports). The number of ports depends on the attached probe.
/// HIPAR count/If a modular probe R&S RT-ZM or differential probe R&S& RT-ZD is used,
/// HIPAR count/the number of ports is 4. Otherwise, 2 ports are available.
ViStatus _VI_FUNC rsscope_QueryDeembeddingComponentPortCount(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   component,
                                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:PCO?", channel, component);
    checkErr(rsscope_QueryViInt32(instrSession, cmd, count));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the probing mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR mode/Sets the probing mode.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentMode(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   component,
                                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_MODE_PROBING, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_MODE_TERMINATING),
            4, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:MODE %s", channel, component,
             deembeddingComponentModeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the loading of the probe if
/// HIFN  RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_MODE is set to Probing. If the probe
/// HIFN  loading is removed, you see the signal that would be at the measurement
/// HIFN  point if the probe's input impedance is ideal (infinite impedance).
/// HIFN  Without removing the loading, you see the real signal at the
/// HIFN  measurement point, including the probe loading.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR loading/Removes the loading of the probe if
/// HIPAR loading/RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_MODE is set to Probing. If the probe
/// HIPAR loading/loading is removed, you see the signal that would be at the measurement
/// HIPAR loading/point if the probe's input impedance is ideal (infinite impedance).
/// HIPAR loading/Without removing the loading, you see the real signal at the
/// HIPAR loading/measurement point, including the probe loading.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentRemoveProbeLoading(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViBoolean loading)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:LOAD:REMP %s", channel, component, loading ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the deembedding impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR impedanceType/Sets the impedance type of the DUT.
/// HIPAR impedance/Impedance of the DUT. Shows the default source impedance or sets the
/// HIPAR impedance/user-defined constant impedance value. If an R&S RT-ZM probe is
/// HIPAR impedance/connected, the value depends on the selected probe mode: common mode,
/// HIPAR impedance/differential, or single-ended measurement.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentImpedance(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViInt32   impedanceType,
                            ViReal64  impedance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, impedanceType, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_IMPEDANCE_TYPE_DEFAULT, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_IMPEDANCE_TYPE_S2P),
            3, "Impedance Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:LOAD %s", channel, component,
             deembeddingComponentImpedanceTypeArr[impedanceType]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:LOAD:IMP %.12lG", channel, component, impedance);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the file name, file format and path of the S-parameter file, and
/// HIFN  loads the S-parameters.
/// HIFN     The attribute is relevant if:
/// HIFN     RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_MODE is Probing
/// HIFN     RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_REMOVE_PROBE_LOADING is ON
/// HIFN     RSSCOPE_ATTR_DEEMBEDDING_COMPONENT_IMPEDANCE_TYPE is S1P or S2P
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentLoadSParameterFile(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:LOAD '%s'", channel, component, fileName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The setting is relevant if the measurement setup is a 4-port, and the
/// HIFN  component is a 2- port. Usually, these components are cables and
/// HIFN  adapters, which are described by *.s2p files. In case of a 4-port
/// HIFN  setup, 2 cables or 2 adapters are used. If they are identical, for
/// HIFN  example, when a matched-pair cable is used, they can be described by
/// HIFN  the same file. In this case, set the attribute to ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR identicalPaths/The setting is relevant if the measurement setup is a 4-port, and the
/// HIPAR identicalPaths/component is a 2- port. Usually, these components are cables and
/// HIPAR identicalPaths/adapters, which are described by *.s2p files. In case of a 4-port
/// HIPAR identicalPaths/setup, 2 cables or 2 adapters are used. If they are identical, for
/// HIPAR identicalPaths/example, when a matched-pair cable is used, they can be described by
/// HIPAR identicalPaths/the same file. In this case, set the attribute to ON.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentIdenticalPaths(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViBoolean identicalPaths)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:IDEN %s", channel, component,
             identicalPaths ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the deembedding component input ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR port1/Assign the ports to the input 1 of the component according to the data
/// HIPAR port1/in the Touchstone file. This information is not directly written in the
/// HIPAR port1/file. You need background information how the S-parameters were
/// HIPAR port1/determined, i.e. which port was input and which port was output.
/// HIPAR port2/Assign the ports to the input 2 of the component according to the data
/// HIPAR port2/in the Touchstone file. This information is not directly written in the
/// HIPAR port2/file. You need background information how the S-parameters were
/// HIPAR port2/determined, i.e. which port was input and which port was output.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentInputPorts(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViInt32   port1,
                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_4),
            4, "Port 1");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port2, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_4),
            5, "Port 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:IPON %s", channel, component, deembeddingPortArr[port1]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:IPTW %s", channel, component, deembeddingPortArr[port2]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the deembedding component output ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR port1/Assign the ports to the output 1 of the component according to the
/// HIPAR port1/data in the Touchstone file. This information is not directly written
/// HIPAR port1/in the file. You need background information how the S-parameters were
/// HIPAR port1/determined, i.e. which port was input and which port was output.
/// HIPAR port2/Assign the ports to the output 2 of the component according to the
/// HIPAR port2/data in the Touchstone file. This information is not directly written
/// HIPAR port2/in the file. You need background information how the S-parameters were
/// HIPAR port2/determined, i.e. which port was input and which port was output.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentOutputPorts(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViInt32   port1,
                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_4),
            4, "Port 1");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port2, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_1, RSSCOPE_VAL_DEEMBEDDING_COMPONENT_PORT_4),
            5, "Port 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:OPON %s", channel, component, deembeddingPortArr[port1]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:OPTW %s", channel, component, deembeddingPortArr[port2]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the file name, file format and path of the S-parameter file, and
/// HIFN  load the S-parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR component/Sets the component parameter.
/// HIPAR port/Selects the port to be loaded.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsscope_ConfigureDeembeddingComponentLoadSParameterFilePort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   component,
                            ViInt32   port,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSSCOPE_VAL_DEEMBEDDING_FILE_PORT_1, RSSCOPE_VAL_DEEMBEDDING_FILE_PORT_2),
            4, "Port");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEEM%ld:COMP%ld:%s '%s'", channel, component, deembeddingPortLoadArr[port],
             fileName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the deembedding attenuation parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR dc/Maximum attenuation or gain of the measurement setup before
/// HIPAR dc/deembedding at DC. This value is the expected basic attenuation or
/// HIPAR dc/gain, which is corrected by deembedding.
/// HIPAR max/Maximum attenuation of the measurement setup before deembedding, with
/// HIPAR max/reference to a frequency response normalized at DC to 0 dB. The
/// HIPAR max/deembedding filter must amplify the amplitude response by the 'Attmax'
/// HIPAR max/value, and thus also increases the noise by this value. The value also
/// HIPAR max/helps to decide whether deembedding is useful for the defined effective
/// HIPAR max/bandwidth.
ViStatus _VI_FUNC rsscope_QueryDeembeddingAttenuation(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64* dc,
                                                      ViReal64* max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_ATTENUATION_DC,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_ATTENUATION_DC, dc),
            3, "DC");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_ATTENUATION_MAX, max),
            4, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Maximum gain of the measurement setup before deembedding, with
/// HIFN  reference to a frequency response normalized at DC to 0dB. The
/// HIFN  deembedding filter must attenuate the amplitude response by this value.
/// HIFN  The value helps also to avoid an overload of the ADC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR max/Maximum gain of the measurement setup before deembedding, with
/// HIPAR max/reference to a frequency response normalized at DC to 0dB. The
/// HIPAR max/deembedding filter must attenuate the amplitude response by this value.
/// HIPAR max/The value helps also to avoid an overload of the ADC.
ViStatus _VI_FUNC rsscope_QueryDeembeddingMaxGain(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64* max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_DEEMBEDDING_GAIN_MAX,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_DEEMBEDDING_GAIN_MAX, max),
            3, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables the indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR state/Enables the indicated parallel bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOBusState(ViSession instrSession,
                                               ViInt32   serialBus,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_BUS_STATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_BUS_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures display settings of the indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR showDigitalSignals/If enabled, the selected digital signals are shown in the diagram.
/// HIPAR showBus/Shows or hides the indicated parallel bus.
/// HIPAR busRepresentation/Selects the display type of the indicated parallel bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOVisualization(ViSession instrSession,
                                                    ViInt32   serialBus,
                                                    ViBoolean showDigitalSignals,
                                                    ViBoolean showBus,
                                                    ViInt32   busRepresentation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DISPLAY_SHOW_DIGITAL_SIGNALS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_DISPLAY_SHOW_DIGITAL_SIGNALS, showDigitalSignals),
            3, "Show Digital Signals");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_DISPLAY_SHOW_BUS, showBus),
            4, "Show Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_DISPLAY_REPRESENTATION, busRepresentation),
            5, "Bus Representation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines if the bus is a clocked bus - one of the digital channels
/// HIFN  serves as clock of the bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR enableClockedBus/Defines if the bus is a clocked bus - one of the digital channels
/// HIPAR enableClockedBus/serves as clock of the bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOClockedBusState(ViSession instrSession,
                                                      ViInt32   bus,
                                                      ViBoolean enableClockedBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_2),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_CLOCKED_BUS_STATE,
            "SerialBus", bus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_CLOCKED_BUS_STATE, enableClockedBus),
            3, "Enable Clocked Bus");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the digital channel used as clock of the indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR clockSource/Selects the digital channel used as clock of the indicated parallel bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOClockedBus(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   clockSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_CLOCKED_BUS,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_CLOCKED_BUS, clockSource),
            3, "Clock Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the slope of the clock signal at which all digital channels of
/// HIFN  the bus are analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR clockSlope/Selects the slope of the clock signal at which all digital channels of
/// HIPAR clockSlope/the bus are analyzed.
ViStatus _VI_FUNC rsscope_ConfigureMSOClockedBusSlope(ViSession instrSession,
                                                      ViInt32   bus,
                                                      ViInt32   clockSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_2),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_CLOCKED_BUS_SLOPE,
            "SerialBus", bus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_CLOCKED_BUS_SLOPE, clockSlope),
            3, "Clock Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function assigns the selected digital channel to the indicated bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR assignState/Assigns the selected digital channel to the indicated bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOAssignChannels(ViSession instrSession,
                                                     ViInt32   serialBus,
                                                     ViInt32   digitalChannel,
                                                     ViBoolean assignState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_SIGNAL_ASSIGN_STATE,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_SIGNAL_ASSIGN_STATE,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap_channel),
            3, "Digital Channel");

    strcat(repCap, ",");
    strcat(repCap, repCap_channel);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_SIGNAL_ASSIGN_STATE, assignState),
            4, "Assign State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects and configure the digital channels that are used in the selected bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR label/Sets a name for the indicated digital channel.
/// HIPAR deskew/Sets an individual delay for each digital channel to time-align it with other digital channels.
ViStatus _VI_FUNC rsscope_ConfigureMSOSignalSelection(ViSession instrSession,
                                                      ViInt32   digitalChannel,
                                                      ViString  label,
                                                      ViReal64  deskew)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_SIGNAL_LABEL,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap),
            2, "Digital Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_SIGNAL_DESKEW, deskew),
            4, "Deskew");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MSO_SIGNAL_LABEL, label),
            3, "Label");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the indicated digital channel, displays it, and
/// HIFN  enables the parallel Bus1 if the bus was disabled. That is,
/// HIFN  RSSCOPE_ATTR_MSO_DISPLAY_SHOW_DIGITAL_SIGNALS
/// HIFN  and RSSCOPE_ATTR_MSO_BUS_STATE are set to ON automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR display/Enables or disables the indicated digital channel, displays it, and
/// HIPAR display/enables the parallel Bus1 if the bus was disabled.
ViStatus _VI_FUNC rsscope_ConfigureMSODigitalChannelDisplay(ViSession instrSession,
                                                            ViInt32   digitalChannel,
                                                            ViBoolean display)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DIGITAL_CHANNEL_DISPLAY,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap),
            2, "Digital Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_DIGITAL_CHANNEL_DISPLAY, display),
            3, "Display");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects and configure the digital channels that are used
/// HIFN  in the selected bus.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-B1 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_LABEL
/// HIFN     RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_DESKEW
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     BUS<m>:PARallel:BIT<n>:LABel
/// HIFN     BUS<m>:PARallel:BIT<n>:DESKew
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR label/Sets a name for the indicated digital channel. The name is displayed
/// HIPAR label/in the diagram.
/// HIPAR deskew/Sets an individual delay for each digital channel to time-align it
/// HIPAR deskew/with other digital channels. The deskew value compensates delays that
/// HIPAR deskew/are known from the circuit specifics or caused by the different length
/// HIPAR deskew/of cables.
ViStatus _VI_FUNC rsscope_ConfigureMSOBusSignalSelection(ViSession instrSession,
                                                         ViInt32   bus,
                                                         ViInt32   digitalChannel,
                                                         ViString  label,
                                                         ViReal64  deskew)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bus, RSSCOPE_VAL_BUS_1, RSSCOPE_VAL_BUS_4),
            2, "Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_LABEL,
            "SerialBus", bus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_LABEL,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap_channel),
            3, "Digital Channel");

    strcat(repCap, ",");
    strcat(repCap, repCap_channel);

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_LABEL, label),
            3, "Label");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_INDIVIDUAL_SIGNAL_DESKEW, deskew),
            4, "Deskew");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets a general delay for all digital channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR deskew/Sets a general delay for all digital channels.
ViStatus _VI_FUNC rsscope_ConfigureMSODeskew(ViSession instrSession,
                                             ViInt32   bus,
                                             ViReal64  deskew)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_SIGNAL_DESKEW_ALL_CHANNELS,
            "SerialBus", bus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Bus");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_SIGNAL_DESKEW_ALL_CHANNELS, deskew),
            3, "Deskew");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the threshold voltage for various types of
/// HIFN  integrated circuits and applies it to all digital channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR technology/Selects the threshold voltage for various types of integrated circuits
/// HIPAR technology/and applies it to all digital channels.
ViStatus _VI_FUNC rsscope_ConfigureMSOTechnology(ViSession instrSession,
                                                 ViInt32   digitalChannel,
                                                 ViInt32   technology)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TECHNOLOGY,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap),
            2, "Digital Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_TECHNOLOGY, technology),
            4, "Technology");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the logical threshold and hysteresis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR treshold/Sets the logical threshold for the indicated digital channel.
/// HIPAR hysteresis/This control defines the size of the hysteresis to avoid the
/// HIPAR hysteresis/change of signal states due to noise.
ViStatus _VI_FUNC rsscope_ConfigureMSOTresholdSetup(ViSession instrSession,
                                                    ViInt32   digitalChannel,
                                                    ViReal64  treshold,
                                                    ViInt32   hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TRESHOLD,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap),
            2, "Digital Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_TRESHOLD, treshold),
            3, "treshold");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_HYSTERESIS, hysteresis),
            4, "hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the logical threshold and hysteresis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR treshold/Sets the logical threshold for the indicated channel group.
/// HIPAR channelGroup/This control selects the channel group for treshold and hystersis.
/// HIPAR hysteresis/This control defines the size of the hysteresis for the channel group
/// HIPAR hysteresis/to avoid the change of signal states due to noise.
/// HIPAR technology/Selects the threshold voltage for various types of integrated circuits
/// HIPAR technology/and applies it to all digital channels.
ViStatus _VI_FUNC rsscope_ConfigureMSOTresholdAllChannels(ViSession instrSession,
                                                          ViInt32   bus,
                                                          ViInt32   technology,
                                                          ViInt32   channelGroup,
                                                          ViReal64  treshold,
                                                          ViInt32   hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_group[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TECHNOLOGY_ALL_CHANNELS,
            "SerialBus", bus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_TRESHOLD_ALL_CHANNELS,
            "ChannelGroup", channelGroup- 1, RS_REPCAP_BUF_SIZE, repCap_group),
            4, "Digital Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_TECHNOLOGY_ALL_CHANNELS, technology),
            3, "technology");

    strcat(repCap, ",");
    strcat(repCap, repCap_group);

    if (technology == RSSCOPE_VAL_MAN)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_MSO_TRESHOLD_ALL_CHANNELS, treshold),
                5, "treshold");
    }

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_HYSTERESIS_ALL_CHANNELS, hysteresis),
            6, "hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables the channel coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR coupling/Enables the channel coupling.
ViStatus _VI_FUNC rsscope_ConfigureMSOChannelCoupling(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_CHANNEL_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_CHANNEL_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the threshold and the hysteresis for all digital channels of
/// HIFN  parallel bus 1 to the same value. RSSCOPE_ATTR_MSO_CHANNEL_COUPLING is
/// HIFN  used to set all buses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Sets the threshold and the hysteresis for all digital channels of
/// HIPAR coupling/parallel bus 1 to the same value. RSSCOPE_ATTR_MSO_CHANNEL_COUPLING is
/// HIPAR coupling/used to set all buses.
ViStatus _VI_FUNC rsscope_ConfigureMSODigitalChannelCoupling(ViSession instrSession,
                                                             ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_MSO_DIGITAL_CHANNEL_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the current digital record length used by each
/// HIFN  digital channel, and the digital resolution of the digital channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reolution/Returns the current digital resolution of the digital channels.
/// HIPAR recordLength/Returns the current digital record length used by each digital
/// HIPAR recordLength/channel.
ViStatus _VI_FUNC rsscope_QueryMSODigitalResolution(ViSession instrSession,
                                                    ViReal64* reolution,
                                                    ViReal64* recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_DIGITAL_RESOLUTION, reolution),
            2, "Reolution");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_DIGITAL_RECORD_LENGTH, recordLength),
            3, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function removes all assigned digital channels from the bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsscope_MSORemoveAllAssignedChannels(ViSession instrSession,
                                                       ViInt32   serialBus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_CHANNEL_COUPLING,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_MSO_REMOVE_ALL_ASSIGNED_CHANNELS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads signals from defined digital channel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR arraySize/Pass the number of elements in the Signal Array parameter.
/// HIPAR maximumTime/Maximum length of time in which to allow the read waveformoperation to complete
/// HIPAR signalArray/Returns the signal data (zeroes and ones).
/// HIPAR actualPoints/Indicates the number of points the function places in the Signal Array parameter.
ViStatus _VI_FUNC rsscope_ReadSignals(ViSession instrSession,
                                      ViInt32   digitalChannel,
                                      ViUInt32  arraySize,
                                      ViInt32   maximumTime,
                                      ViBoolean signalArray[],
                                      ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, "FORM ASC"));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIG%ld:DATA?", digitalChannel);
    checkErr(
        RsCore_QueryAsciiViBooleanArrayToUserBufferWithOpc(instrSession, cmd, maximumTime, arraySize, signalArray,
            actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Available only if RSSCOPE_ATTR_MSO_CLOCKED_BUS_STATE is set to TRUE.
/// HIFN  If enabled, a results box opens with decoded values of the bus signal
/// HIFN  and its time. You can query the number of values
/// HIFN  (RSSCOPE_ATTR_MSO_DECODE_TABLE_EDGE_COUNT) and their data value
/// HIFN  (RSSCOPE_ATTR_MSO_DECODE_TABLE_DATA) using rsscope_QueryMSODecodeTable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR decodeTable/This control selects the decode table.
/// HIPAR tableEnabled/If enabled, a results box opens with decoded values of the bus signal
/// HIPAR tableEnabled/and its time.
ViStatus _VI_FUNC rsscope_ConfigureMSODecodeTableShow(ViSession instrSession,
                                                      ViInt32   serialBus,
                                                      ViInt32   decodeTable,
                                                      ViBoolean tableEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_table[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, 4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DECODE_TABLE_SHOW,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    strcat(repCap, ",");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, decodeTable, 1, 5),
            3, "Decode Table");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DECODE_TABLE_SHOW,
            "DecodeTable", decodeTable, RS_REPCAP_BUF_SIZE, repCap_table),
            3, "Decode Table");
    strcat(repCap, repCap_table);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_MSO_DECODE_TABLE_SHOW, tableEnabled),
            3, "Table Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number format for decoded data values on the display for the
/// HIFN  indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR format/Sets the number format for decoded data values on the display for the
/// HIPAR format/indicated parallel bus.
ViStatus _VI_FUNC rsscope_ConfigureMSODataFormat(ViSession instrSession,
                                                 ViInt32   serialBus,
                                                 ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DATA_FORMAT,
            "SerialBus", serialBus- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_DATA_FORMAT, format),
            3, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures MSO trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerType/Selects the MSO trigger type.
ViStatus _VI_FUNC rsscope_ConfigureMSOTrigger(ViSession instrSession,
                                              ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_TYPE, triggerType),
            2, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the edge - the state transition - of the signal to trigger on a single digital
/// HIFN channel (a logic bit), or a logical combination of digital channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerSlope/Sets the slope for the MSO edge trigger.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureMSOEdgeTriggerSource(ViSession instrSession,
                                                        ViInt32   triggerSource,
                                                        ViInt32   triggerSlope,
                                                        ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_EDGE_TRIGGER_SLOPE, triggerSlope),
            3, "Trigger Slope");

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_EDGE_TRIGGER_EXPRESSION, expression),
                4, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the MSO width trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR widthCondition/Selects how the range of a pulse width is defined.
/// HIPAR width/For the ranges "Within" and "Outside", the width defines the center of a range which
/// HIPAR width/is defined by the limits "+/-Delta". For the ranges "Shorter" and "Longer",
/// HIPAR width/the width defines the maximum and minimum pulse width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition.
ViStatus _VI_FUNC rsscope_ConfigureMSOWidthTriggerSource(ViSession instrSession,
                                                         ViInt32   triggerSource,
                                                         ViInt32   widthPolarity,
                                                         ViInt32   widthCondition,
                                                         ViReal64  width,
                                                         ViReal64  widthDelta,
                                                         ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_WIDTH_TRIGGER_POLARITY, widthPolarity),
            3, "Width Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_WIDTH_TRIGGER_CONDITION, widthCondition),
            4, "Width Condition");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_WIDTH_TRIGGER_WIDTH, width),
            5, "Width");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_WIDTH_TRIGGER_DELTA, widthDelta),
            6, "Width Delta");

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_WIDTH_TRIGGER_EXPRESSION, expression),
                7, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the MSO timeout trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition
/// HIPAR timeoutValue/Defines the time limit for the timeout at which the instrument triggers.
/// HIPAR timeoutCondition/Defines the relation of the signal level to the trigger level.
ViStatus _VI_FUNC rsscope_ConfigureMSOTimeoutTriggerSource(ViSession instrSession,
                                                           ViInt32   triggerSource,
                                                           ViReal64  timeoutValue,
                                                           ViInt32   timeoutCondition,
                                                           ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_TIMEOUT_TRIGGER_VALUE, timeoutValue),
            3, "Timeout Value");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TIMEOUT_TRIGGER_CONDITION, timeoutCondition),
            4, "Timeout Condition");

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_TIMEOUT_TRIGGER_EXPRESSION, expression),
                5, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Data2Clock trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR clockSource/Sets the clock signal source.
/// HIPAR data2ClockPolarity/Sets the edge of the clock signal to define the time reference
/// HIPAR data2ClockPolarity/point for the setup and hold time.
/// HIPAR holdTime/Sets the minimum time after the clock edge while the data signal must stay
/// HIPAR holdTime/steady above or below the data level.
/// HIPAR setupTime/Sets the minimum time before the clock edge while the data
/// HIPAR setupTime/signal must stay steady above or below the data level.
ViStatus _VI_FUNC rsscope_ConfigureMSOData2ClockTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource,
                            ViInt32   clockSource,
                            ViInt32   data2ClockPolarity,
                            ViReal64  holdTime,
                            ViReal64  setupTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_CLOCK_SOURCE, clockSource),
            3, "Clock Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_CLOCK_EDGE, data2ClockPolarity),
            4, "Data2Clock Polarity");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_HOLD_TIME, holdTime),
            5, "Hold Time");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_DATA2CLOCK_TRIGGER_SETUP_TIME, setupTime),
            6, "Setup Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures MSO state trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR clockSource/Sets the edge of the clock signal to define the time reference
/// HIPAR clockSource/point for the setup and hold time.
/// HIPAR clockPolarity/Document parameter here.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition
ViStatus _VI_FUNC rsscope_ConfigureMSOStateTriggerSource(ViSession instrSession,
                                                         ViInt32   triggerSource,
                                                         ViInt32   clockSource,
                                                         ViInt32   clockPolarity,
                                                         ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_STATE_TRIGGER_CLOCK_SOURCE, clockSource),
            3, "Clock Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_STATE_TRIGGER_CLOCK_EDGE, clockPolarity),
            4, "Clock Polarity");

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_STATE_TRIGGER_EXPRESSION, expression),
                5, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the required state for each digital channel that is used in the bus..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR triggerType/Sets the MSO trigger type.
/// HIPAR statusBit/Sets the required state for each digital channel that is used in the bus.
ViStatus _VI_FUNC rsscope_ConfigureMSOTriggerChannelState(ViSession instrSession,
                                                          ViInt32   digitalChannel,
                                                          ViInt32   triggerType,
                                                          ViInt32   statusBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerType, RSSCOPE_VAL_MSO_STATE_TRIGGER, RSSCOPE_VAL_MSO_PATTERN_TRIGGER),
            3, "Trigger Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_CHANNEL_STATUS_BIT,
            "DigitalChannel", digitalChannel, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Digital Channel");

    if (triggerType == RSSCOPE_VAL_MSO_STATE_TRIGGER)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_channel, RSSCOPE_ATTR_MSO_STATE_TRIGGER_CHANNEL_STATUS_BIT, statusBit),
                4, "Status Bit");
    }
    else
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap_channel, RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_CHANNEL_STATUS_BIT, statusBit),
                4, "Status Bit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the MSO pattern trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR patternMode/Sets the mode of the timing condition.
/// HIPAR timeoutCondition/Sets the state condition for the timeout qualification.
/// HIPAR timeout/Defines how long the result of the pattern condition must be true or false.
/// HIPAR patternWidth/For the ranges "Within" and "Outside", the width defines the center of a range
/// HIPAR patternWidth/which is defined by the limits "Delta". For the ranges "Shorter" and "Longer",
/// HIPAR patternWidth/the width defines the maximum and minimum pulse width, respectively.
/// HIPAR patternDelta/Defines a range around the width value.
/// HIPAR patternCondition/Defines how the range of a pulse width is defined for keeping up
/// HIPAR patternCondition/the true result of the pattern condition.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition
ViStatus _VI_FUNC rsscope_ConfigureMSOPatternTriggerSource(ViSession instrSession,
                                                           ViInt32   triggerSource,
                                                           ViInt32   patternMode,
                                                           ViInt32   timeoutCondition,
                                                           ViReal64  timeout,
                                                           ViReal64  patternWidth,
                                                           ViReal64  patternDelta,
                                                           ViInt32   patternCondition,
                                                           ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_MODE, patternMode),
            3, "Pattern Mode");

    if (patternMode == RSSCOPE_VAL_PATTERN_TIMEOUT)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_TIMEOUT_MODE, timeoutCondition),
                4, "Timeout Condition");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_TIMEOUT, timeout),
                5, "Timeout");
    }
    if (patternMode == RSSCOPE_VAL_PATTERN_WIDTH)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_WIDTH, patternWidth),
                6, "Pattern Width");

        viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_DELTA, patternDelta),
                7, "Pattern Delta");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_CONDITION, patternCondition),
                8, "Pattern Condition");
    }

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_PATTERN_TRIGGER_EXPRESSION, expression),
                9, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures MSO serial pattern trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR patternPolarity/Sets the edge of the clock signal. The bit value is determined
/// HIPAR patternPolarity/at the crossing of the clock edge and the logical threshold.
/// HIPAR clockSource/Sets the edge of the clock signal to define the time reference
/// HIPAR clockSource/point for the setup and hold time.
/// HIPAR pattern/The pattern contains the bits of the serial data to be found in the data stream.
/// HIPAR expression/Defines a logical combination of several digital channels as trigger condition
ViStatus _VI_FUNC rsscope_ConfigureMSOSerialPatternTriggerSource(
                            ViSession instrSession,
                            ViInt32   triggerSource,
                            ViInt32   patternPolarity,
                            ViInt32   clockSource,
                            ViString  pattern,
                            ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_CLOCK_EDGE, patternPolarity),
            3, "Pattern Polarity");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_CLOCK_SOURCE, clockSource),
            3, "Clock Source");

    if (triggerSource == RSSCOPE_VAL_MSO_DALL)
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_EXPRESSION, expression),
                6, "Expression");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_MSO_SERIAL_PATTERN_TRIGGER_PATTERN, pattern),
            5, "pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR arraySize/Pass the number of elements in the Results parameter.
/// HIPAR results/Array of the recorded parallel bus data.
/// HIPAR numberOfResults/Indicates the number of results the function places in the Results
/// HIPAR numberOfResults/parameter.
ViStatus _VI_FUNC rsscope_QueryMSOBusData(ViSession instrSession,
                                          ViInt32   bus,
                                          ViInt32   arraySize,
                                          ViReal64  results[],
                                          ViInt32*  numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bus, 1, 4),
            2, "Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PAR:DATA?", bus);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, numberOfResults));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the indicated parallel bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR arraySize/Pass the number of elements in the Results parameter.
/// HIPAR results/Array of the recorded digital channel data.
/// HIPAR numberOfResults/Indicates the number of results the function places in the Results
/// HIPAR numberOfResults/parameter.
ViStatus _VI_FUNC rsscope_QueryMSOChannelData(ViSession instrSession,
                                              ViInt32   digitalChannel,
                                              ViInt32   arraySize,
                                              ViReal64  results[],
                                              ViInt32*  numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, digitalChannel, 0, 15),
            2, "Digital Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIG%ld:DATA?", digitalChannel);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, numberOfResults));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of data of the indicated bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bus/This control selects the bus.
/// HIPAR xStart/Acquisition time before trigger, in s
/// HIPAR xStop/Acquisition time after trigger, in s
/// HIPAR recordLength/Record length of the waveform in Samples
/// HIPAR numberOfValues/Number of values per sample interval.
ViStatus _VI_FUNC rsscope_QueryMSOBusDataHeader(ViSession instrSession,
                                                ViInt32   bus,
                                                ViReal64* xStart,
                                                ViReal64* xStop,
                                                ViInt32*  recordLength,
                                                ViInt32*  numberOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf;
    ViChar*  data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bus, RSSCOPE_VAL_SERIAL_BUS_1, RSSCOPE_VAL_SERIAL_BUS_4),
            2, "Serial Bus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BUS%ld:PAR:DATA:HEAD?", bus);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &data));

    p2buf = strtok(data, ",");
    *xStart = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *xStop = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *recordLength = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ",");
    *numberOfValues = RsCore_Convert_String_To_ViInt32(p2buf);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (data)
        free(data);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of data of the indicated digital channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR xStart/Acquisition time before trigger, in s
/// HIPAR xStop/Acquisition time after trigger, in s
/// HIPAR recordLength/Record length of the waveform in Samples
/// HIPAR numberOfValues/Number of values per sample interval.
ViStatus _VI_FUNC rsscope_QueryMSOChannelDataHeader(ViSession instrSession,
                                                    ViInt32   digitalChannel,
                                                    ViReal64* xStart,
                                                    ViReal64* xStop,
                                                    ViInt32*  recordLength,
                                                    ViInt32*  numberOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64 response[4];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B1"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, digitalChannel, 0, 15),
            2, "Digital Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIG%ld:DATA:HEAD?", digitalChannel);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, 0, 4, response, NULL));

    *xStart = response[0];
    *xStop = response[1];
    *recordLength = (ViInt32)response[2];
    *numberOfValues = (ViInt32)response[3];

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads the number of clock edges and the data in the decode table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalChannel/This control selects the digital channel.
/// HIPAR decodeTable/This control selects the decode table.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Table
/// HIPAR bufferSize/Data parameter.
/// HIPAR edgeCount/Returns the number of clock edges in the decode table.
/// HIPAR tableData/Returns the data of the decoded values of the bus signal and its time.
/// HIPAR returnedCharacters/Returns number of characters returned from the instrument in the Table
/// HIPAR returnedCharacters/Data field.
ViStatus _VI_FUNC rsscope_QueryMSODecodeTable(ViSession instrSession,
                                              ViInt32   serialBus,
                                              ViInt32   decodeTable,
                                              ViInt32   bufferSize,
                                              ViInt32*  edgeCount,
                                              ViChar    tableData[],
                                              ViInt32*  returnedCharacters)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd = NULL;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_table[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, serialBus, 1, 4),
            2, "Serial Bus");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DECODE_TABLE_EDGE_COUNT,
            "SerialBus", serialBus, RS_REPCAP_BUF_SIZE, repCap),
            2, "Serial Bus");
    strcat(repCap, ",");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, decodeTable, 1, 5),
            3, "Decode Table");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_MSO_DECODE_TABLE_EDGE_COUNT,
            "DecodeTable", decodeTable, RS_REPCAP_BUF_SIZE, repCap_table),
            3, "Decode Table");
    strcat(repCap, repCap_table);

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_MSO_DECODE_TABLE_EDGE_COUNT, edgeCount),
            5, "Edge Count");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSSCOPE_ATTR_MSO_DECODE_TABLE_DATA, repCap, &cmd));
    checkErr(
        RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, tableData, returnedCharacters));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enable and configure the settings for the pulse source, R&S Option B7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Switches the pulse output on and off.
/// HIPAR outputLevel/Sets the low level of the output pulse.
/// HIPAR repetitionRate/Sets the pulse frequency, the repetition rate of the pulse.
/// HIPAR dutyCycle/Sets the duty cycle of the pulse, which is the ratio of the positive
/// HIPAR dutyCycle/pulse width to the period of the signal.
/// HIPAR clockMode/Sets the dependency of the pulse clock on the instrument's reference
/// HIPAR clockMode/clock.
ViStatus _VI_FUNC rsscope_ConfigurePulseSourceSetup(ViSession instrSession,
                                                    ViBoolean enabled,
                                                    ViReal64  outputLevel,
                                                    ViInt32   repetitionRate,
                                                    ViInt32   dutyCycle,
                                                    ViInt32   clockMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_STATE, enabled),
            2, "Enabled");

    if (enabled)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_LOW_OUTPUT_LEVEL, outputLevel),
                3, "Output Level");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_REPETITION_RATE, repetitionRate),
                4, "Repetition Rate");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_DUTY_CYCLE, dutyCycle),
                5, "Duty Cycle");

        viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_CLOCK_MODE, clockMode),
                6, "Clock Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the pulse source to the default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ResetPulseSource(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_PULSE_SOURCE_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures options for TDR/TDT measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects the analysis type: TDR (time domain reflectometry), TDT (time
/// HIPAR measurementType/domain transmission), or TDR/TDT (both TDR and TDT).
/// HIPAR configuration/Selects the connection: single-ended or differential.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTMeasurement(ViSession instrSession,
                                                     ViInt32   measurementType,
                                                     ViInt32   configuration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_ANALYSIS_TYPE, measurementType),
            2, "Measurement Type");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_CONNECTION_CONFIGURATION, configuration),
            3, "Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the input channels that are connected to the "Ref" connector
/// HIFN  and the "Out" connector of the pulse source for single-ended
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceChannel/Sets the channel to which the Ref connector of the pulse source is
/// HIPAR referenceChannel/connected.
/// HIPAR outChannel/Sets the channel to which the Out connector of the pulse source is
/// HIPAR outChannel/connected via calibration standard or DUT.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTSingleEndedChannels(ViSession instrSession,
                                                             ViInt32   referenceChannel,
                                                             ViInt32   outChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_REFERENCE_CHANNEL, referenceChannel),
            2, "Reference Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_OUT_CHANNEL, outChannel),
            3, "Out Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the input channels that are connected to the pulse sourcefor
/// HIFN  differential measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceChannel/Sets the channel to which the Ref connector of the pulse source is
/// HIPAR referenceChannel/connected.
/// HIPAR outChannel/Sets the channel to which the Out connector of the pulse source is
/// HIPAR outChannel/connected via calibration standard or DUT.
/// HIPAR diffOutChannel/Sets the channel to which the Diff Out connector of the pulse source
/// HIPAR diffOutChannel/is connected via calibration standard or DUT. Only required for
/// HIPAR diffOutChannel/differential measurements.
/// HIPAR diffReferenceChannel/Sets the channel to which the Diff Reference connector of the pulse
/// HIPAR diffReferenceChannel/source is connected. Only required for differential measurements.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTDifferentialChannels(
                            ViSession instrSession,
                            ViInt32   referenceChannel,
                            ViInt32   outChannel,
                            ViInt32   diffOutChannel,
                            ViInt32   diffReferenceChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_REFERENCE_CHANNEL, referenceChannel),
            2, "Reference Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_OUT_CHANNEL, outChannel),
            3, "Out Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_DIFF_OUT_CHANNEL, diffOutChannel),
            4, "Diff Out Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_DIFF_REFERENCE_CHANNEL, diffReferenceChannel),
            5, "Diff Reference Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the measurement with default correction values instead of
/// HIFN  calibrating the setup. Default values are rough values without time
/// HIFN  reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_MeasureTDRTDTWithDefaultValues(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_START_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Opens and loads the TDR/TDT calibration from the file specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Sets the file name, file format and path of the TDR/TDT calibration
/// HIPAR filename/file to be opened. The file format is always .xml.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTCalibrationLoad(ViSession instrSession,
                                                         ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_CALIBRATION_LOAD_FILENAME, filename),
            2, "Filename");
    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_CALIBRATION_LOAD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the TDR/TDT calibration results to the file specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Sets the file name, file format and path of the TDR/TDT calibration
/// HIPAR filename/file to be saved. The file format is always .xml.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTCalibrationSave(ViSession instrSession,
                                                         ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_CALIBRATION_SAVE_FILENAME, filename),
            2, "Filename");
    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_CALIBRATION_SAVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the channel to which the Ref connector of the pulse source is
/// HIFN  connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tdtPulseReferenceChannel*N/Sets the channel to which the Ref connector of the pulse source is
/// HIPAR tdtPulseReferenceChannel*N/connected.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTPulseReferenceChannel(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_PULSE_REFERENCE_CHANNEL, channel),
            2, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the electrical length of a user-defined short. The default value
/// HIFN  is the electrical length of the short hat is delivered with the pulse
/// HIFN  source R&S RTO-B7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR shortLength/Sets the electrical length of a user-defined short. The default value
/// HIPAR shortLength/is the electrical length of the short hat is delivered with the pulse
/// HIPAR shortLength/source R&S RTO-B7.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTShortLength(ViSession instrSession,
                                                     ViReal64  shortLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_SHORT_LENGTH, shortLength),
            2, "Short Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the frequency response calibration. Without
/// HIFN  calibration, the measured signals are only rescaled, and you can see
/// HIFN  the impact of the frequency response calibration on the time domain
/// HIFN  waveform. The command is not relevant when measuring with default
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Disables or enables the frequency response calibration. Without
/// HIPAR enabled/calibration, the measured signals are only rescaled, and you can see
/// HIPAR enabled/the impact of the frequency response calibration on the time domain
/// HIPAR enabled/waveform. The command is not relevant when measuring with default
/// HIPAR enabled/values.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTUseCalibration(ViSession instrSession,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_USE_CALIBRATION_DATA, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement parameters for TDR/TDT testing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averaging/The "Averaging" value is the "Average count (N-single count)", which
/// HIPAR averaging/you can also set in rsscope_ConfigureNumberOfAverages. Increase the
/// HIPAR averaging/value to reduce noise.
/// HIPAR bandwidth/Sets the bandwidth of the DUT.
/// HIPAR domain/Sets the domain of the x-axis: time or distance (in m or inch). If you
/// HIPAR domain/select distance, enter also the permittivity.
/// HIPAR permittivity/Sets the permittivity epsilon_r of the used DUT. This parameter is
/// HIPAR permittivity/only used if Domain is not Time.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTMeasurementParameters(
                            ViSession instrSession,
                            ViInt32   averaging,
                            ViReal64  bandwidth,
                            ViInt32   domain,
                            ViReal64  permittivity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_NUM_AVERAGES, averaging),
            2, "Averaging");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_DOMAIN, domain),
            4, "Domain");

    if (domain != RSSCOPE_VAL_TDR_TDT_DOMAIN_TIME)
    {
        viCheckParm(rsscope_SetAttributeViReal64(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_DUT_PERMITTIVITY, permittivity),
                5, "Permittivity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the display and analysis of TDR/TDT signals. TDR delivers
/// HIFN  the impedance Z and the reflection coefficient p. TDT delivers the
/// HIFN  pulse response, a normalized and corrected step response.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Selects the TDR/TDT signal to be displayed and analyzed. TDR delivers
/// HIPAR signal/the impedance Z and the reflection coefficient p. TDT delivers the
/// HIPAR signal/pulse response, a normalized and corrected step response.
/// HIPAR state/Enables the indicated TDR/TDT signal.
/// HIPAR verticalScale/Set the vertical scale of the indicated TDR/TDT signal.
/// HIPAR offset/Set the offset of the indicated TDR/TDT signal.
/// HIPAR position/Set the vertical position of the indicated TDR/TDT signal.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTSignal(ViSession instrSession,
                                                ViInt32   signal,
                                                ViBoolean state,
                                                ViReal64  verticalScale,
                                                ViReal64  offset,
                                                ViReal64  position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, RSSCOPE_VAL_TDR_TDT_SIGNAL_IMPEDANCE, RSSCOPE_VAL_TDR_TDT_SIGNAL_PULSE_RESPONSE),
            2, "Signal");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TDR_TDT_SIGNAL_STATE,
            "TDRTDTSignal", signal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Signal");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_TDR_TDT_SIGNAL_STATE, state),
            3, "State");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TDR_TDT_SIGNAL_VERTICAL_SCALE, verticalScale),
            4, "Vertical Scale");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TDR_TDT_SIGNAL_OFFSET, offset),
            5, "Offset");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_TDR_TDT_SIGNAL_VERTICAL_POSITION, position),
            6, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of channel waveform data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Selects the TDR/TDT signal. TDR delivers the impedance Z and the
/// HIPAR signal/reflection coefficient p. TDT delivers the pulse response, a normalized
/// HIPAR signal/and corrected step response.
/// HIPAR xStart/Acquisition time before trigger, in s
/// HIPAR xStop/Acquisition time after trigger, in s
/// HIPAR recordLength/Record length of the waveform in Samples
/// HIPAR valuesperSample/Number of values per sample interval.
ViStatus _VI_FUNC rsscope_FetchTDRTDTSignalWaveformHeader(ViSession instrSession,
                                                          ViInt32   signal,
                                                          ViReal64* xStart,
                                                          ViReal64* xStop,
                                                          ViInt32*  recordLength,
                                                          ViInt32*  valuesperSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   data[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, RSSCOPE_VAL_TDR_TDT_SIGNAL_IMPEDANCE, RSSCOPE_VAL_TDR_TDT_SIGNAL_PULSE_RESPONSE),
            2, "Signal");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_TDR_TDT_SIGNAL_WAVEFORM_HEADER,
            "TDRTDTSignal", signal- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Signal");

    checkErr(rsscope_GetAttributeViString(instrSession, repCap,
        RSSCOPE_ATTR_TDR_TDT_SIGNAL_WAVEFORM_HEADER, RS_MAX_MESSAGE_LEN, data));

    p2buf = strtok(data, ",");
    *xStart = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *xStop = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *recordLength = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ",");
    *valuesperSample = RsCore_Convert_String_To_ViInt32(p2buf);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the waveform points for transmission from the
/// HIFN  instrument to the controlling computer. The data can be used in MATLAB,
/// HIFN  for example. With this method, the complete waveform is retrieved.
/// HIFN  Using rsscope_FetchTDRTDTSignalPartialWaveform instead, data can be
/// HIFN  retrieved in smaller portions, which makes the command faster.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Selects the TDR/TDT signal. TDR delivers the impedance Z and the
/// HIPAR signal/reflection coefficient p. TDT delivers the pulse response, a normalized
/// HIPAR signal/and corrected step response.
/// HIPAR timeout/Defines timeout in milliseconds.
/// HIPAR bufferLength/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the data of track points for transmission from the instrument
/// HIPAR waveformArray/to the controlling computer.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchTDRTDTSignalCompleteWaveform(ViSession instrSession,
                                                            ViInt32   signal,
                                                            ViUInt32  timeout,
                                                            ViInt32   bufferLength,
                                                            ViReal64  waveformArray[],
                                                            ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, RSSCOPE_VAL_TDR_TDT_SIGNAL_IMPEDANCE, RSSCOPE_VAL_TDR_TDT_SIGNAL_PULSE_RESPONSE),
            2, "Signal");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "TDRT:SIGN%ld:DATA:VAL?", signal);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, timeout,
        bufferLength, waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the waveform points for transmission from the
/// HIFN  instrument to the controlling computer. The data can be used in MATLAB,
/// HIFN  for example. Using the offset and length parameters, data can be
/// HIFN  retrieved in smaller portions than using
/// HIFN  rsscope_FetchTDRTDTSignalPartialWaveform, which makes the
/// HIFN  command faster.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Selects the TDR/TDT signal. TDR delivers the impedance Z and the
/// HIPAR signal/reflection coefficient p. TDT delivers the pulse response, a normalized
/// HIPAR signal/and corrected step response.
/// HIPAR timeout/Defines timeout in milliseconds.
/// HIPAR offset/Number of offset waveform points.
/// HIPAR length/Number of waveform points to be retrieved.
/// HIPAR bufferLength/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the data of track points for transmission from the instrument
/// HIPAR waveformArray/to the controlling computer.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsscope_FetchTDRTDTSignalPartialWaveform(ViSession instrSession,
                                                           ViInt32   signal,
                                                           ViUInt32  timeout,
                                                           ViInt32   offset,
                                                           ViInt32   bufferLength,
                                                           ViReal64  waveformArray[],
                                                           ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, RSSCOPE_VAL_TDR_TDT_SIGNAL_IMPEDANCE, RSSCOPE_VAL_TDR_TDT_SIGNAL_PULSE_RESPONSE),
            2, "Signal");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "TDRT:SIGN%ld:DATA:VAL? %ld,%ld",
             signal, offset, bufferLength);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd,
        timeout, bufferLength, waveformArray, actualPoints));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Activates or deactivates the TDR/TDT measurements.
///HIRET Returns the status code of this operation.
///HIPAR enabled/Activates or deactivates the TDR/TDT measurements.
ViStatus _VI_FUNC rsscope_ConfigureTDRTDTEnabled(ViSession instrSession,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_TDR_TDT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the waveform generator and outputs the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR enable/Enables the waveform generator and outputs the waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorState(ViSession instrSession,
                                                          ViInt32   generator,
                                                          ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ENABLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ENABLE, enable),
            3, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the parameters of the waveform generator to their default values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
ViStatus _VI_FUNC rsscope_WaveformGeneratorPreset(ViSession instrSession,
                                                  ViInt32   generator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PRESET,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the operation mode for the waveform generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR operationMode/Sets the operation mode for the waveform generator.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOperationMode(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   operationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OPERATION_MODE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OPERATION_MODE, operationMode),
            3, "Operation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies all settings from Gen1/Gen2 and applies them to Gen2/Gen1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
ViStatus _VI_FUNC rsscope_WaveformGeneratorCopySettings(ViSession instrSession,
                                                        ViInt32   generator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COPY_SETTINGS,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COPY_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the type of waveform to be generated for the function generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR functionTyp/Sets the type of waveform to be generated for the function generator.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFunctionType(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   functionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FUNCTION_TYPE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FUNCTION_TYPE, functionType),
            3, "Function Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the frequency of the waveform. The available frequency range
/// HIFN  depends on the selected function type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR frequency/Sets the frequency of the waveform. The available frequency range
/// HIPAR frequency/depends on the selected function type.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FREQUENCY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the period of the waveform. The available period range depends on
/// HIFN  the selected function type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR period/Sets the period of the waveform. The available period range depends on
/// HIPAR period/the selected function type.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorPeriod(ViSession instrSession,
                                                           ViInt32   generator,
                                                           ViReal64  period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PERIOD,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PERIOD, period),
            3, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duty cycle for a square waveform. The duty cycle expresses
/// HIFN  for what percentage of the period, the signal state is high.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dutyCycle/Sets the duty cycle for a square waveform. The duty cycle expresses
/// HIPAR dutyCycle/for what percentage of the period, the signal state is high.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorDutyCycle(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dutyCycle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_DUTY_CYCLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_DUTY_CYCLE, dutyCycle),
            3, "Duty Cycle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the symmetry for a ramp waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR symmetry/Sets the symmetry for a ramp waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorSymmetry(ViSession instrSession,
                                                             ViInt32   generator,
                                                             ViReal64  symmetry)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SYMMETRY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SYMMETRY, symmetry),
            3, "Symmetry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the pulse duration for a pulse waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR pulseWidth/Sets the pulse duration for a pulse waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorPulseWidth(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  pulseWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PULSE_WIDTH,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PULSE_WIDTH, pulseWidth),
            3, "Pulse Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the modulation type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR modulationType/Sets the modulation type.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorModulationType(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   modulationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_MODULATION_TYPE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_MODULATION_TYPE, modulationType),
            3, "Modulation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the frequency of the carrier signal for a modulation waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR carrierFrequency/Sets the frequency of the carrier signal for a modulation waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCarrierFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_CARRIER_FREQUENCY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the period of the carrier signal for a modulation waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR carrierPeriod/Sets the period of the carrier signal for a modulation waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCarrierPeriod(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  carrierPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_CARRIER_PERIOD,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_CARRIER_PERIOD, carrierPeriod),
            3, "Carrier Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the AM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR signal/Selects the type of the modulation signal for the AM modulation types.
/// HIPAR frequency/Sets the frequency of the modulation waveform for amplitude
/// HIPAR frequency/modulation.
/// HIPAR depth/Sets the amplitude modulation depth.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorAMModulation(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   signal,
                            ViReal64  frequency,
                            ViReal64  depth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_SIGNAL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_SIGNAL, signal),
            3, "Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_FREQUENCY, frequency),
            4, "Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_DEPTH, depth),
            5, "Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duty cycle of a square waveform for amplitude modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dutyCycle/Sets the duty cycle of a square waveform for amplitude modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorAMModulationDutyCycle(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dutyCycle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_DUTY_CYCLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_DUTY_CYCLE, dutyCycle),
            3, "Duty Cycle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIFN  is rising, for amplitude modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR symmetry/Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIPAR symmetry/is rising, for amplitude modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorAMModulationSymmetry(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  symmetry)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_SYMMETRY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_AM_MODULATION_SYMMETRY, symmetry),
            3, "Symmetry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures FM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR signal/Selects the type of the modulation signal for the FM modulation types.
/// HIPAR frequency/Sets the frequency of the modulation waveform for frequency
/// HIPAR frequency/modulation.
/// HIPAR frequencyDeviation/Sets the frequency deviation, the maximum difference between and FM
/// HIPAR frequencyDeviation/modulated signal and the carrier signal.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFMModulation(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   signal,
                            ViReal64  frequency,
                            ViReal64  frequencyDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_SIGNAL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_SIGNAL, signal),
            3, "Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_FREQUENCY, frequency),
            4, "Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_FREQUENCY_DEVIATION, frequencyDeviation),
            5, "Frequency Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duty cycle of a square waveform for frequency modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dutyCycle/Sets the duty cycle of a square waveform for frequency modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFMModulationDutyCycle(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dutyCycle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_DUTY_CYCLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_DUTY_CYCLE, dutyCycle),
            3, "Duty Cycle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIFN  is rising, for frequency modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR symmetry/Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIPAR symmetry/is rising, for frequency modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFMModulationSymmetry(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  symmetry)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_SYMMETRY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FM_MODULATION_SYMMETRY, symmetry),
            3, "Symmetry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures PWM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR signal/Selects the type of the modulation signal for the PWM modulation
/// HIPAR signal/types.
/// HIPAR frequency/Sets the type of the modulation signal for the pulse width modulation.
/// HIPAR depth/Sets the modulation depth for pulse width modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorPWMModulation(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   signal,
                            ViReal64  frequency,
                            ViReal64  depth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_SIGNAL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_SIGNAL, signal),
            3, "Signal");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_FREQUENCY, frequency),
            4, "Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_DEPTH, depth),
            5, "Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duty cycle of a square waveform for pulse width modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dutyCycle/Sets the duty cycle of a square waveform for pulse width modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorPWMModulationDutyCycle(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dutyCycle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_DUTY_CYCLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_DUTY_CYCLE, dutyCycle),
            3, "Duty Cycle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIFN  is rising, for pulse width modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR symmetry/Sets the symmetry, the percentage of time the ramp modulation waveform
/// HIPAR symmetry/is rising, for pulse width modulation.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorPWMModulationSymmetry(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  symmetry)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_SYMMETRY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_PWM_MODULATION_SYMMETRY, symmetry),
            3, "Symmetry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures FSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR frequency1/Sets the frequency of the carrier waveform.
/// HIPAR frequency2/Sets the frequency of the modulated waveform.
/// HIPAR hopRate/Sets the hop rate, the time before a switch from the carrier frequency
/// HIPAR hopRate/and the modulation frequency.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorFSKModulation(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  frequency1,
                            ViReal64  frequency2,
                            ViReal64  hopRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FSK_MODULATION_FREQUENCY_1,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FSK_MODULATION_FREQUENCY_1, frequency1),
            3, "Frequency 1");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FSK_MODULATION_FREQUENCY_2, frequency2),
            4, "Frequency 2");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_FSK_MODULATION_HOP_RATE, hopRate),
            5, "Hop Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR startFrequency/Sets the start frequency of the sweep range.
/// HIPAR stopFrequency/Sets the stop frequency of the sweep range.
/// HIPAR sweepTime/Sets the duration of the sweep.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorSweep(ViSession instrSession,
                                                          ViInt32   generator,
                                                          ViReal64  startFrequency,
                                                          ViReal64  stopFrequency,
                                                          ViReal64  sweepTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SWEEP_START_FREQUENCY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SWEEP_START_FREQUENCY, startFrequency),
            3, "Start Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SWEEP_STOP_FREQUENCY, stopFrequency),
            4, "Stop Frequency");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_SWEEP_TIME, sweepTime),
            5, "Sweep Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the arbitrary waveform source. You can load an existing file
/// HIFN  or load the current oscilloscope waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR source/Selects the arbitrary waveform source. You can load an existing file
/// HIPAR source/or load the current oscilloscope waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorArbitrarySource(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SOURCE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duration for which the signal of the arbitrary generator will
/// HIFN  be output after the trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR runMode/Sets the duration for which the signal of the arbitrary generator will
/// HIPAR runMode/be output after the trigger event.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorArbitraryRunMode(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   runMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_RUN_MODE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_RUN_MODE, runMode),
            3, "Run Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the sample rate for the arbitrary waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR sampleRate/Sets the sample rate for the arbitrary waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorArbitrarySampleRate(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   sampleRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SAMPLE_RATE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SAMPLE_RATE, sampleRate),
            3, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays the number of samples in the loaded waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR samples/Displays the number of samples in the loaded waveform.
ViStatus _VI_FUNC rsscope_QueryWaveformGeneratorArbitrarySamples(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32*  samples)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SAMPLES,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SAMPLES, samples),
            3, "Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the arbitrary waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR file/Sets the file path and the file for an arbitrary waveform.
ViStatus _VI_FUNC rsscope_WaveformGeneratorArbitraryOpenFile(ViSession instrSession,
                                                             ViInt32   generator,
                                                             ViString  file)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_FILE_NAME,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_FILE_NAME, file),
            3, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_OPEN_FILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the arbitrary waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR file/Defines the path and the multichannel arbitrary waveform file to be
/// HIPAR file/imported.
ViStatus _VI_FUNC rsscope_WaveformGeneratorArbitraryMultichannelOpenFile(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViString  file)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_MULTICHANNEL_FILE_NAME,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_MULTICHANNEL_FILE_NAME, file),
            3, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_MULTICHANNEL_OPEN_FILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Assigns a waveform from the multichannel file to the arbitrary
/// HIFN  waveform generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR source/Assigns a waveform from the multichannel file to the arbitrary
/// HIPAR source/waveform generator.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorArbitraryMultichannelWaveform(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_MULTICHANNEL_WAVEFORM,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_MULTICHANNEL_WAVEFORM, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the oscilloscope source, from which the arbitrary signal is
/// HIFN  loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR source/Selects the oscilloscope source, from which the arbitrary signal is
/// HIPAR source/loaded.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorArbitraryScopeSource(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SCOPE_SOURCE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SCOPE_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the waveform from the selected signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
ViStatus _VI_FUNC rsscope_WaveformGeneratorArbitraryScopeOpen(
                            ViSession instrSession,
                            ViInt32   generator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SCOPE_OPEN,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SCOPE_OPEN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the amplitude, peak to peak voltage, of the output waveform. This
/// HIFN  is defined as the voltage difference between the maximum and the
/// HIFN  minimum voltage levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR amplitude/Sets the amplitude, peak to peak voltage, of the output waveform. This
/// HIPAR amplitude/is defined as the voltage difference between the maximum and the
/// HIPAR amplitude/minimum voltage levels.
/// HIPAR offset/Sets a voltage offset.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOutputAmplitude(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  amplitude,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_AMPLITUDE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_AMPLITUDE, amplitude),
            3, "Amplitude");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the signal level of the output waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR lowLevel/Sets the low signal level of the output waveform.
/// HIPAR highLevel/Sets the high signal level of the output waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOutputLevel(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  lowLevel,
                            ViReal64  highLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_LOW_LEVEL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_LOW_LEVEL, lowLevel),
            3, "Low Level");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_HIGH_LEVEL, highLevel),
            4, "High Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Inverts the waveform at the offset level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR inversion/Inverts the waveform at the offset level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOutputInversion(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean inversion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_INVERSION,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_INVERSION, inversion),
            3, "Inversion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the load of the DUT at its connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR userLoad/Sets the load of the DUT at its connection.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOutputUserLoad(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViInt32   userLoad)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_USER_LOAD,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_USER_LOAD, userLoad),
            3, "User Load");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the DC level of the generated DC signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dcLevel/Sets the DC level of the generated DC signal.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorOutputDCLevel(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dcLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_DC_LEVEL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_OUTPUT_DC_LEVEL, dcLevel),
            3, "DC Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the adding of noise to the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR addNoise/Enables the adding of noise to the waveform.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorAddNoise(ViSession instrSession,
                                                             ViInt32   generator,
                                                             ViBoolean addNoise)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ADD_NOISE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ADD_NOISE, addNoise),
            3, "Add Noise");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the noise level in percentage of the amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR level/Sets the noise level in percentage of the amplitude.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorNoiseLevelPercent(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_NOISE_LEVEL_PERCENT,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_NOISE_LEVEL_PERCENT, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the level of the noise in volts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR absoluteLevel/Queries the level of the noise in volts.
ViStatus _VI_FUNC rsscope_QueryWaveformGeneratorNoiseAbsoluteLevel(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64* absoluteLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_ARBITRARY_SAMPLES,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_NOISE_ABSOLUTE_LEVEL, absoluteLevel),
            3, "Absolute Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the DC noise level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR dcLevel/Sets the DC noise level.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorNoiseDCLevel(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  dcLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_NOISE_DC_LEVEL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_NOISE_DC_LEVEL, dcLevel),
            3, "DC Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the coupling of the generators, with the selected set of
/// HIFN  parameters: amplitude, frequency and noise.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects repeated capability WaveformGenerator.
/// HIPAR coupling/Enables the coupling of the generators, with the selected set of
/// HIPAR coupling/parameters: amplitude, frequency and noise.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingEnable(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  generatorCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "WaveformGenerator", &generatorCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, generator, 1, generatorCount),
            2, "Generator");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ENABLE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ENABLE, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the coupling of all parameters of the generators, except of
/// HIFN  load and inversion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR couplingAll/Enables the coupling of all parameters of the generators, except of
/// HIPAR couplingAll/load and inversion.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingAll(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean couplingAll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ALL,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ALL, couplingAll),
            3, "Coupling All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples all parameters including output enable, but excluding
/// HIFN  inversion and load.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR couplingOutput/Couples all parameters including output enable, but excluding
/// HIPAR couplingOutput/inversion and load.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingOutput(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean couplingOutput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_OUTPUT,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_OUTPUT, couplingOutput),
            3, "Coupling Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples all noise parameters of the generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects repeated capability WaveformGenerator.
/// HIPAR noiseCoupling/Couples all noise parameters of the generators.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingNoise(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean noiseCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  generatorCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "WaveformGenerator", &generatorCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, generator, 1, generatorCount),
            2, "Generator");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_NOISE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_NOISE, noiseCoupling),
            3, "Noise Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the coupling of the arbitrary waveform of the generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects repeated capability WaveformGenerator.
/// HIPAR arbitraryCoupling/Enables the coupling of the arbitrary waveform of the generators.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingArbitrary(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean arbitraryCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  generatorCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "WaveformGenerator", &generatorCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, generator, 1, generatorCount),
            2, "Generator");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ARBITRARY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_ARBITRARY, arbitraryCoupling),
            3, "Arbitrary Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the coupling of all amplitude parameters of the generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR couplingAmplitude/Enables the coupling of all amplitude parameters of the generators.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingAmplitude(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean couplingAmplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_AMPLITUDE,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_AMPLITUDE, couplingAmplitude),
            3, "Coupling Amplitude");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the phase shift between the waveform of Gen1 and Gen2 when the
/// HIFN  frequency parameters of the two waveforms are coupled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR phaseShift/Sets the phase shift between the waveform of Gen1 and Gen2 when the
/// HIPAR phaseShift/frequency parameters of the two waveforms are coupled.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingPhaseShift(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  phaseShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_PHASE_SHIFT,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_PHASE_SHIFT, phaseShift),
            3, "Phase Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the coupling of all frequency parameters of the generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generator/Selects the generator.
/// HIPAR couplingFrequency/Enables the coupling of all frequency parameters of the generators.
ViStatus _VI_FUNC rsscope_ConfigureWaveformGeneratorCouplingFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean couplingFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_FREQUENCY,
            "WaveformGenerator", generator- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_GENERATOR_COUPLING_FREQUENCY, couplingFrequency),
            3, "Coupling Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the pattern generator and outputs the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables the pattern generator and outputs the waveform.
ViStatus _VI_FUNC rsscope_ConfigurePatternGeneratorState(ViSession instrSession,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_ENABLE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the parameters of the waveform generator to their default values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_PatternGeneratorPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the duration for which the signal of the pattern generator will
/// HIFN  be output after the trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR runMode/Sets the duration for which the signal of the pattern generator will
/// HIPAR runMode/be output after the trigger event.
ViStatus _VI_FUNC rsscope_ConfigurePatternGeneratorRunMode(ViSession instrSession,
                                                           ViInt32   runMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_RUN_MODE, runMode),
            2, "Run Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a single period of the pattern generator, for
/// HIFN  PGENerator:RUNMode SINGle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ExecutePatternGeneratorSingleRun(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_PATTERN_GENERATOR_RUN_SINGLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of transmitted bits per second for the pattern
/// HIFN  generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitRate/Sets the number of transmitted bits per second for the pattern
/// HIPAR bitRate/generator.
ViStatus _VI_FUNC rsscope_ConfigurePatternGeneratorBitRate(ViSession instrSession,
                                                           ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_BIT_RATE, bitRate),
            2, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the high level of the pattern generator signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR highLevel/Sets the high level of the pattern generator signal.
ViStatus _VI_FUNC rsscope_ConfigurePatternGeneratorHighLevel(ViSession instrSession,
                                                             ViReal64  highLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_HIGH_LEVEL, highLevel),
            2, "High Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the specified pattern file to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Specifies a file path to open a pattern generator file.
ViStatus _VI_FUNC rsscope_PatternGeneratorLoadPattern(ViSession instrSession,
                                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_FILE_NAME, fileName),
            2, "File Name");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_PATTERN_GENERATOR_LOAD_PATTERN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets which signals generated from the waveform generator are
/// HIFN  synchronized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorSynchronization/Sets which signals generated from the waveform generator are
/// HIPAR generatorSynchronization/synchronized.
ViStatus _VI_FUNC rsscope_ConfigureGeneratorSynchronization(ViSession instrSession,
                                                            ViInt32   generatorSynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_GENERATOR_SYNCHRONIZATION, generatorSynchronization),
            2, "Generator Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the alignment of the DC offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_GeneratorStartAlignment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_GENERATOR_START_ALIGNMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the alignment of the DC offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_GeneratorStartAlignmentWithOPC(ViSession instrSession,
                                                         ViInt32   maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldOPC = 10000;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_GetOPCTimeout(instrSession, &oldOPC));
    checkErr(rsscope_SetOPCTimeout(instrSession, maximumTime));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_GENERATOR_START_ALIGNMENT_WITH_OPC, NULL));

    checkErr(rsscope_SetOPCTimeout(instrSession, oldOPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Aborts a DC offset alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_GeneratorAbortAlignment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_GENERATOR_ABORT_ALIGNMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays the date of the last performed DC offset alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Alignment Date.
/// HIPAR alignmentDate/Displays the date of the last performed DC offset alignment.
ViStatus _VI_FUNC rsscope_QueryGeneratorAlignmentDate(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    alignmentDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_GENERATOR_ALIGNMENT_DATE,
            bufferSize, alignmentDate),
            3, "Alignment Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the time of the last performed DC offset alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Alignment Time'.
/// HIPAR alignmentTime/Queries the time of the last performed DC offset alignment.
ViStatus _VI_FUNC rsscope_QueryGeneratorAlignmentTime(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    alignmentTime[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_GENERATOR_ALIGNMENT_TIME,
            bufferSize, alignmentTime),
            3, "Alignment Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the result of a DC offset alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignmentOverallState/Queries the result of a DC offset alignment.
ViStatus _VI_FUNC rsscope_QueryGeneratorAlignmentOverallState(
                            ViSession instrSession,
                            ViInt32*  alignmentOverallState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_GENERATOR_ALIGNMENT_OVERALL_STATE, alignmentOverallState),
            2, "Alignment Overall State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the number of connected frontends.
///HIRET Returns the status code of this operation.
///HIPAR numberofFrontends/Sets the number of connected frontends.
ViStatus _VI_FUNC rsscope_ConfigureNumberofFrontends(ViSession instrSession,
                                                     ViInt32   numberofFrontends)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_NUMBER_OF_FRONTENDS, numberofFrontends),
            2, "Number of Frontends");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Resets the complete configuration of the external frontends
///HINF  to the default values including the network connections.
///HINF  External frontends have their own preset. The general preset
///HINF  disconnect the frontends but does not affect the frontend settings.
///HIRET Returns the status code of this operation.
ViStatus _VI_FUNC rsscope_PresetExternalFrontend(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_FRONTEND_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the type of the test setup.
///HIRET Returns the status code of this operation.
///HIPAR measurementSetup/Selects the type of the test setup.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendMeasurementSetup(
                            ViSession instrSession,
                            ViInt32   measurementSetup)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MEASUREMENT_SETUP, measurementSetup),
            2, "Measurement Setup");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the input channel to which the indicated external
///HINF  frontend is connected.
///HIRET Returns the status code of this operation.
///HIPAR channel/Selects the input channel to which the indicated
///HIPAR channel/external frontend is connected.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendChannel(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CHANNEL,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CHANNEL, channel),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches the connection of the indicated frontend on or off.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR connectionEnabled/Switches the connection of the indicated
///HIPAR connectionEnabled/frontend on or off.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendConnectionState(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViBoolean connectionEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_STATE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_STATE, connectionEnabled),
            3, "Connection Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the current connection state.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR connectionState/Queries the current connection state.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendConnectionCurrentState(
                            ViSession  instrSession,
                            ViInt32    frontendPosition,
                            ViBoolean* connectionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_CURRENT_STATE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_CURRENT_STATE, connectionState),
            3, "Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the supported external frontend types.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR type/Queries the supported external frontend types.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendType(ViSession instrSession,
                                                    ViInt32   frontendPosition,
                                                    ViInt32   bufferSize,
                                                    ViChar    type[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_TYPE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
            RSSCOPE_ATTR_EXTERNAL_FRONTEND_TYPE, bufferSize, type, returnedValues), 4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the connection of the indicated frontend: type of the
///HINF  frontend and the IP address or host name. Optionally, the
///HINF  identifier string can be used.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR type/Type of the connected frontend. For a list of available
///HIPAR type/type strings, use RSSCOPE_ATTR_EXTERNAL_FRONTEND_TYPE.
///HIPAR ipaddress/IP address or host name of the connected frontend.
///HIPAR identifier/IDN string of the connected frontend. Pass empty
///HIPAR identifier/string to skip this parameter.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendConnectionType(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViString  type,
                            ViString  ipaddress,
                            ViString  identifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;
    ViChar   parameters[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HIDDEN_EXTERNAL_FRONTEND_CONNECTION_TYPE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    snprintf (parameters, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s','%s'", type, ipaddress, identifier);

    viCheckParm(rsscope_SetAttributeViString(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_HIDDEN_EXTERNAL_FRONTEND_CONNECTION_TYPE, parameters),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the identifier of the indicated frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR identity/Queries the identifier of the indicated frontend.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendIdentifier(ViSession instrSession,
                                                          ViInt32   frontendPosition,
                                                          ViInt32   bufferSize,
                                                          ViChar    identity[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IDENTIFIER,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_IDENTIFIER, bufferSize, identity, returnedValues), 4, "Identity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the RF frequency of the signal that the external
///HINF  frontend receives from the DUT. The value is the center frequency that
///HINF  determines the LO frequency generation.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR rfInput/Sets the RF frequency of the signal that the external
///HIPAR rfInput/frontend receives from the DUT. The value is the
///HIPAR rfInput/center frequency that determines the LO frequency generation.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendRFInput(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViReal64  rfInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_INPUT,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_INPUT, rfInput),
            3, "RF Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the reference level of the signal at the RF input of the
///HINF  external frontend. If this level is entered correctly, the
///HINF  external frontend is configured such that the level at the input
///HINF  of the R&SRTP is optimal.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR rfLevel/Sets the reference level of the signal at the RF
///HIPAR rfLevel/input of the external frontend. If this level is entered
///HIPAR rfLevel/correctly, the  external frontend is configured such that
///HIPAR rfLevel/the level at the input of the R&SRTP is optimal.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendRFLevel(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViInt32   rfLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_LEVEL,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_LEVEL, rfLevel),
            3, "RF Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the reference frequency that is used for frequency
///HINF  conversion on the external frontend. The frontend needs an external
///HINF  reference frequency to generate its own LO frequency if no
///HINF  external LO is available.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR referenceFrequency/Sets the reference frequency that is used
///HIPAR referenceFrequency/for frequency conversion on the external
///HIPAR referenceFrequency/frontend. The frontend needs an external
///HIPAR referenceFrequency/reference frequency to generate its own LO
///HIPAR referenceFrequency/frequency if no external LO is available.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendReferenceFrequency(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViReal64  referenceFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY, referenceFrequency),
            3, "Reference Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Shows information about the intermediate frequency, maximum
///HINF  analysis bandwidth, cable connected local oscillator frequency
///HINF  and gain of the external frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR ifOutput/Queries the intermediate frequency to which the
///HIPAR ifOutput/signal is downconverted by the frontend.
///HIPAR rfBandwidth/Queries the maximum analysis bandwidth of the
///HIPAR rfBandwidth/connected external frontend.
///HIPAR loInFrequencyCable/Queries the required frequency on LO input
///HIPAR loInFrequencyCable/if the LO input mode is external. See also:
///HIPAR loInFrequencyCable/RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_INPUT
///HIPAR externalFrontendGain/Returns the gain if the connected
///HIPAR externalFrontendGain/external frontend.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendSettings(ViSession instrSession,
                                                        ViInt32   frontendPosition,
                                                        ViReal64* ifOutput,
                                                        ViReal64* rfBandwidth,
                                                        ViReal64* loInFrequencyCable,
                                                        ViInt32*  externalFrontendGain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_OUTPUT,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_OUTPUT, ifOutput),
            3, "IF Output");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_BANDWIDTH, rfBandwidth),
            4, "RF Bandwidth");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_IN_FREQUENCY_CABLE, loInFrequencyCable),
            5, "LO In Frequency Cable");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_GAIN, externalFrontendGain),
            6, "External Frontend Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  For measurement setups Coupled LO and Coupled Ref, the
///HINF  command queries the input mode of the local oscillator. The mode is
///HINF  defined by the selected setup definition.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR loInput/For measurement setups Coupled LO and Coupled Ref,
///HIPAR loInput/the command queries the input mode of the local
///HIPAR loInput/oscillator. The mode is defined by the selected setup definition.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendLOInput(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViInt32   loInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_INPUT,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_INPUT, loInput),
            3, "LO Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  For measurement setups Coupled LO and Coupled Ref, it queries
///HINF  the connection state of the LO output. The state is defined by
///HINF  the selected setup definition.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR loOutputState/For measurement setups Coupled LO and Coupled
///HIPAR loOutputState/Ref, it queries the connection state of the LO
///HIPAR loOutputState/output. The state is defined by the selected setup
///HIPAR loOutputState/definition.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendLOOutputState(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViBoolean loOutputState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_OUTPUT_STATE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_OUTPUT_STATE, loOutputState),
            3, "LO Output State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Enables or disables cable deembedding and defines the
///HINF  S-parameter file.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR cableAlignmentEnable/Activates the correction of the IF
///HIPAR cableAlignmentEnable/signal due to IF cable loss from the frontend
///HIPAR cableAlignmentEnable/to the oscilloscope.
///HIPAR sParameters/Sets the path to the S-parameter file that
///HIPAR sParameters/contains the correction values for the cable that
///HIPAR sParameters/connects the frontend to the oscilloscope.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendAccessories(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViBoolean cableAlignmentEnable,
                            ViString  sParameters)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CABLE_ALIGNMENT,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CABLE_ALIGNMENT, cableAlignmentEnable),
            3, "Cable Alignment Enable");

    viCheckParm(rsscope_SetAttributeViString(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FILE_PATH, sParameters),
            4, "S-Parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Enables or disables the RF attenuation auto mode. If auto
///HINF  mode is OFF, then RF Attenuation can be used.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR rfAttenuationAutoMode/Defines the RF attenuation, which is
///HIPAR rfAttenuationAutoMode/meant to protect the mixer of the external
///HIPAR rfAttenuationAutoMode/frontend from high RF input levels.
///HIPAR rfAttenuation/Sets the RF attenuation if
///HIPAR rfAttenuation/RF Attenuation Auto Mode is OFF.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendRFAttenuation(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViBoolean rfAttenuationAutoMode,
                            ViInt32   rfAttenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_ATTENUATION_AUTO_MODE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_ATTENUATION_AUTO_MODE, rfAttenuationAutoMode),
            3, "RF Attenuation Auto Mode");

    if(rfAttenuationAutoMode == VI_FALSE)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_ATTENUATION, rfAttenuation),
                4, "RF Attenuation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the vertical scale of the signal.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR amplitudeMode/Defines the vertical scale of the signal.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendAmplitudeMode(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   amplitudeMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_AMPLITUDE_MODE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_AMPLITUDE_MODE, amplitudeMode),
            3, "Amplitude Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the possible attenuation step values.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR stepValues/Queries the possible attenuation step values.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendRFAttenuationStepValues(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   bufferSize,
                            ViChar    stepValues[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_ATTENUATION_STEP_VALUES,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_ATTENUATION_STEP_VALUES, bufferSize, stepValues, returnedValues), 4, "Step Values");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the errors that occur in the external frontend and in
///HINF  the communication between frontend and oscilloscope.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR connectionError/Queries the errors that occur in the external
///HIPAR connectionError/frontend and in the communication between
///HIPAR connectionError/frontend and oscilloscope.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendConnectionError(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   bufferSize,
                            ViChar    connectionError[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "FrontendPosition");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_ERROR,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"FrontendPosition");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_CONNECTION_ERROR, bufferSize, connectionError, returnedValues), 4, "Connection Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the RF band selection auto mode. Currently only auto
///HINF  mode and the lower IF band are supported.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR autoMode/Sets the RF band selection auto mode. Currently only
///HIPAR autoMode/auto mode and the lower IF band are supported.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendFrequencyBandConfigAutoMode(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViBoolean autoMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "FrontendPosition");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_AUTO_MODE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"FrontendPosition");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_AUTO_MODE, autoMode),
            3, "Auto Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the number of available band configurations.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR count/Queries the number of available band configurations.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendFrequencyBandConfigCount(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_COUNT,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Returns the available intermediate frequency (output) ranges
///HINF  of the external frontend, depending on the connected external
///HINF  frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR bandConfigList/Returns the available intermediate frequency
///HIPAR bandConfigList/(output) ranges of the external frontend,
///HIPAR bandConfigList/depending on the connected external frontend.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendFrequencyBandConfigList(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   bufferSize,
                            ViChar    bandConfigList[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_LIST,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_LIST, bufferSize, bandConfigList, returnedValues), 4, "Band Config List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines the intermediate frequency (output) range of the
///HINF  external frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR frequencyRange/Defines the intermediate frequency (output)
///HIPAR frequencyRange/range of the external frontend.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendFrequencyBandConfigSelection(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   frequencyRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_SELECTION,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_SELECTION, frequencyRange),
            3, "Frequency Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the IF or RF range of the indicated band.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR frequencyRange/Sets the frequency range.
///HIPAR arraySize/Size of output array.
///HIPAR bandConfigID/Frequency band configuration
///HIPAR parameter/String parameter
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendFrequencyBandConfigRange(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   frequencyRange,
                            ViInt32   bandConfigID,
                            ViString  parameter,
                            ViInt32   arraySize,
                            ViChar    result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;
    ViChar   commandParameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAttr   attr[] = {RSSCOPE_ATTR_HIDDEN_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_IF_RANGE,
             RSSCOPE_ATTR_HIDDEN_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_RF_RANGE};

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_HIDDEN_EXTERNAL_FRONTEND_FREQUENCY_BAND_CONFIG_IF_RANGE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyRange, RSSCOPE_VAL_EXTERNAL_FRONTEND_IF, RSSCOPE_VAL_EXTERNAL_FRONTEND_RF),
            3, "Frequency Range");

    snprintf (commandParameter, RS_MAX_MESSAGE_BUF_SIZE, "%d,'%s'", bandConfigID, parameter);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, frontendPositionRepCap, attr[frequencyRange], commandParameter, arraySize, result, returnedValues), 7, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the maximum or minimum IF frequency.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR ifFrequencyMax/Queries the maximum IF frequency.
///HIPAR ifFrequencyMin/Queries the minimum IF frequency.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendIFFrequency(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViReal64* ifFrequencyMax,
                                                           ViReal64* ifFrequencyMin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MAX_IF_FREQUENCY,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MAX_IF_FREQUENCY, ifFrequencyMax),
            3, "IF Frequency Max");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MIN_IF_FREQUENCY, ifFrequencyMin),
            4, "IF Frequency Min");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the frequency band configuration, which is always
///HINF  IFLow.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR sideband/Queries the frequency band configuration, which is
///HIPAR sideband/always IFLow.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendIFFrequencySideband(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   bufferSize,
                            ViChar    sideband[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_FREQUENCY_SIDEBAND,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_FREQUENCY_SIDEBAND, bufferSize, sideband, returnedValues), 4, "Sideband");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the frequency of the local oscillator of the
///HINF  connected frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR loFrequency/Queries the frequency of the local oscillator of
///HIPAR loFrequency/the connected frontend.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendLOFrequency(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViReal64* loFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_FREQUENCY,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_FREQUENCY, loFrequency),
            3, "LO Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the current LO output frequency.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR loOutputFrequency/Queries the current LO output frequency.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendLOOutputFrequency(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViReal64* loOutputFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_OUTPUT_FREQUENCY,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_LO_OUTPUT_FREQUENCY, loOutputFrequency),
            3, "LO Output Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the maximum or minimum RF frequency.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR rfFrequencyMax/Queries the maximum RF frequency.
///HIPAR rfFrequencyMin/Queries the minimum RF frequency.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendRFFrequency(ViSession instrSession,
                                                           ViInt32   frontendPosition,
                                                           ViReal64* rfFrequencyMax,
                                                           ViReal64* rfFrequencyMin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MAX_RF_FREQUENCY,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MAX_RF_FREQUENCY, rfFrequencyMax),
            3, "RF Frequency Max");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_MIN_RF_FREQUENCY, rfFrequencyMin),
            4, "RF Frequency Min");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the possible reference frequencies, which are
///HINF  supported by the connected external frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR bufferSize/Size of output array.
///HIPAR list/Queries the possible reference frequencies, which are
///HIPAR list/supported by the connected external frontend.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendReferenceFrequencyList(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   bufferSize,
                            ViChar    list[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST, bufferSize, list, returnedValues), 4, "List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the IF attenuation.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR ifAttenuation/Queries the IF attenuation.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendIFAttenuation(ViSession instrSession,
                                                             ViInt32   frontendPosition,
                                                             ViInt32*  ifAttenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_ATTENUATION,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_IF_ATTENUATION, ifAttenuation),
            3, "IF Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the RF connector.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR connector/Sets the RF connector.
ViStatus _VI_FUNC rsscope_ConfigureExternalFrontendRFConnector(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   connector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_CONNECTOR,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_RF_CONNECTOR, connector),
            3, "Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Updates the firmware on the connected external frontend. This
///HINF  process can take some time.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
ViStatus _VI_FUNC rsscope_UpdateExternalFrontend(ViSession instrSession,
                                                 ViInt32   frontendPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_UPDATE,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

     checkErr(rsscope_SetAttributeViString(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the number of frequency bands provided by the
///HINF  selected frontend.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR numberofFrequencyBands/Queries the number of frequency bands
///HIPAR numberofFrequencyBands/provided by the selected frontend.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendNumberofFrequencyBands(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32*  numberofFrequencyBands)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_NUMBER_OF_FREQUENCY_BANDS,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_NUMBER_OF_FREQUENCY_BANDS, numberofFrequencyBands),
            3, "Number of Frequency Bands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the frequency range that is supported by the selected
///HINF  frontend frequency band.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR multiBand/Selects the band for multi band frontends.
///HIPAR bufferSize/Size of output array.
///HIPAR frequencyBandList/Queries the frequency range that is
///HIPAR frequencyBandList/supported by the selected frontend frequency band.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendFrequencyBandList(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   multiBand,
                            ViInt32   bufferSize,
                            ViChar    frequencyBandList[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_LIST,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND, multiBand),
            3, "Multi Band");

    viCheckParm(rsscope_GetAttributeViStringWithRealLen(instrSession, frontendPositionRepCap,
        RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_LIST, bufferSize, frequencyBandList, returnedValues), 5, "Frequency Band List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the lower and the upper frequency limit of the band
///HINF  addressed by suffix <n>.
///HIRET Returns the status code of this operation.
///HIPAR frontendPosition/This control selects the position of the
///HIPAR frontendPosition/frontend, counted from left to right.
///HIPAR multiBand/Selects the band for multi band frontends.
///HIPAR start/Queries the lower frequency limit of the band addressed
///HIPAR start/by suffix <n>.
///HIPAR end/Queries the upper frequency limit of the band addressed
///HIPAR end/by suffix <n>.
ViStatus _VI_FUNC rsscope_QueryExternalFrontendFrequencyBandLimits(
                            ViSession instrSession,
                            ViInt32   frontendPosition,
                            ViInt32   multiBand,
                            ViReal64* start,
                            ViReal64* end)
{
    ViStatus error = VI_SUCCESS;
    ViChar   frontendPositionRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  frontendPositionCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "FrontendPosition", &frontendPositionCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frontendPosition, 1, frontendPositionCount),
            2, "Frontend Position");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_START,
        "FrontendPosition", frontendPosition - 1, RS_REPCAP_BUF_SIZE, frontendPositionRepCap), 2 ,"Frontend Position");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXTERNAL_FRONTEND_SELECTED_MULTI_BAND, multiBand),
            3, "Multi Band");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_START, start),
            4, "Start");

    viCheckParm(rsscope_GetAttributeViReal64(instrSession, frontendPositionRepCap, RSSCOPE_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_END, end),
            5, "End");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the I/Q mode of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates the I/Q mode of the instrument.
ViStatus _VI_FUNC rsscope_ConfigureIQModeState(ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_IQ_MODE_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the IQ Warning.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR enabled/Returns ON if the sampling setup results in an unfavorable
/// HIPAR enabled/combination.
/// HIPAR message/Returns the message string if RSSCOPE_ATTR_IQ_WARNING_ENABLED is true.
ViStatus _VI_FUNC rsscope_QueryIQWarning(ViSession  instrSession,
                                         ViInt32    channel,
                                         ViInt32    bufferSize,
                                         ViBoolean* enabled,
                                         ViChar     message[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_IQ_WARNING_ENABLED,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_GetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_IQ_WARNING_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsscope_GetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_IQ_WARNING_MESSAGE,
            bufferSize, message),
            5, "Message");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the relative bandwidth, sample rate and record length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR relativeBandwidth/Sets the bandwidth factor to define the filter bandwidth.
/// HIPAR sampleRate/Sets the required sample rate of the output I/Q data.
/// HIPAR recordLength/Sets the required record length of the output I/Q data.
ViStatus _VI_FUNC rsscope_ConfigureIQSamplingInputSettings(ViSession instrSession,
                                                           ViReal64  relativeBandwidth,
                                                           ViReal64  sampleRate,
                                                           ViInt32   recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_IQ_RELATIVE_BANDWIDTH, relativeBandwidth),
            2, "Relative Bandwidth");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_IQ_SAMPLE_RATE, sampleRate),
            3, "Sample Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_IQ_RECORD_LENGTH, recordLength),
            4, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the filter bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterBandwidth/Returns the filter bandwidth.
ViStatus _VI_FUNC rsscope_QueryIQSamplingFilterBandwidth(ViSession instrSession,
                                                         ViInt32*  filterBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_IQ_FILTER_BANDWIDTH, filterBandwidth),
            2, "filterBandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  "Input type" sets the format of the input signal. "Input mode" selects
/// HIFN  the frequency band of a complex input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR inputType/Sets the type of the input signal.
/// HIPAR inputMode/Selects the frequency band of a complex input signal.
ViStatus _VI_FUNC rsscope_ConfigureIQSamplingInputType(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   inputType,
                                                       ViInt32   inputMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_IQ_INPUT_TYPE,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_IQ_INPUT_TYPE, inputType),
            3, "Input Type");

    if (inputType == RSSCOPE_VAL_IQ_INPUT_TYPE_COMPLEX)
    {
        viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_IQ_INPUT_MODE, inputMode),
                3, "Input Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the real input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR carrierFrequency/Sets the carrier frequency of the modulated RF signal.
/// HIPAR sideband/Defines the frequency position of the RF spectrum in the input signal:
/// HIPAR sideband/normal or inverse. The position is important for correct
/// HIPAR sideband/down-conversion and filtering.
ViStatus _VI_FUNC rsscope_ConfigureIQSamplingRealInput(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64  carrierFrequency,
                                                       ViInt32   sideband)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSSCOPE_VAL_CHANNEL_1, RSSCOPE_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_IQ_CARRIER_FREQUENCY,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_IQ_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_IQ_REAL_SIDEBAND, sideband),
            4, "Sideband");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the complex intermediate frequency (RFIF) input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR carrierFrequency/Sets the carrier frequency of the modulated RF signal.
/// HIPAR sideband/Defines the sideband and the frequency position of complex modulated
/// HIPAR sideband/input signal in IF range.
ViStatus _VI_FUNC rsscope_ConfigureIQSamplingComplexInput(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  carrierFrequency,
                                                          ViInt32   sideband)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_IQ_CARRIER_FREQUENCY,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_IQ_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_IQ_COMPLEX_SIDEBAND, sideband),
            4, "Sideband");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows the sample rate of capturing the input data in I/Q mode: 20
/// HIFN  Gsample/s.
/// HIFN     If you use R&S RTP134 or R&S RTP164 instruments, you can change the
/// HIFN  sample rate to 40 Gsample/s.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Shows the sample rate of capturing the input data in I/Q mode: 20
/// HIPAR sampleRate/Gsample/s.
ViStatus _VI_FUNC rsscope_QueryIQSamplingInputRate(ViSession instrSession,
                                                   ViInt32*  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_IQ_INPUT_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the recorded I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the input channel.
/// HIPAR arraySize/Pass the number of elements in the Results parameter.
/// HIPAR i/Array of the recorded I data.
/// HIPAR i/Array of the recorded Q data.
/// HIPAR numberOfResults/Indicates the number of results the function places in the each of I and Q
/// HIPAR numberOfResults/parameters.
ViStatus _VI_FUNC rsscope_QueryIQData(ViSession instrSession,
                                      ViInt32   channel,
                                      ViInt32   arraySize,
                                      ViReal64  i[],
                                      ViReal64  q[],
                                      ViInt32*  numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K11"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:IQ:DATA?", channel);
    checkErr(RsCore_QueryFloatArraysInterleavedToUserBuffer(instrSession, cmd, arraySize, numberOfResults, i, q, NULL, NULL, NULL));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the header of I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the input channel.
/// HIPAR xStart/Acquisition time before trigger, in s
/// HIPAR xStop/Acquisition time after trigger, in s
/// HIPAR recordLength/Record length of the waveform in Samples
/// HIPAR numberOfValues/Number of values per sample interval.
ViStatus _VI_FUNC rsscope_QueryIQDataHeader(ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64* xStart,
                                            ViReal64* xStop,
                                            ViInt32*  recordLength,
                                            ViInt32*  valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf;
    ViChar*  data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K11|K0"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:IQ:DATA:HEAD?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, 0, &data));

    p2buf = strtok(data, ",");
    *xStart = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *xStop = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok(NULL, ",");
    *recordLength = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ",");
    *valuesPerSample = RsCore_Convert_String_To_ViInt32(p2buf);

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (data)
        free(data);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the spectrogram display for a math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR spectrogram/Enables the spectrogram display for a math waveform.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAnalysisSpectrogramEnabled(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViBoolean spectrogram)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_SPECTROGRAM_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_SPECTROGRAM_ENABLED, spectrogram),
            3, "Spectrogram");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the scaling method for the frequency (x-)axis of the
/// HIFN  spectrogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR xScaling/Defines the scaling method for the frequency (x-)axis of the
/// HIPAR xScaling/spectrogram.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAnalysisXScaling(ViSession instrSession,
                                                            ViInt32   mathNumber,
                                                            ViInt32   xScaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_X_SCALING,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_X_SCALING, xScaling),
            3, "X Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies the current FFT settings of the selected math waveform (m) to
/// HIFN  the other selected math waveform (m2), and couples the two waveforms.
/// HIFN  This can be repeated for all math waveforms.
/// HIFN     If any FFT setting for any of the coupled spectrums is changed, it is
/// HIFN  changed for all coupled spectrums.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR mathNumber2/Selects the math waveform.
/// HIPAR coupled/Copies the current FFT settings of the selected math waveform (m) to
/// HIPAR coupled/the other selected math waveform (m2), and couples the two waveforms.
/// HIPAR coupled/This can be repeated for all math waveforms.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAnalysisCoupledSpectrum(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViInt32   mathNumber2,
                            ViBoolean coupled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_M2[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math2", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber2, RSSCOPE_REPCAP_MATH2_M21, maxMath),
            3, "Math Number 2");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_COUPLED_SPECTRUM,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_COUPLED_SPECTRUM,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH2_M21, RS_REPCAP_BUF_SIZE, repCap_M2),
            3, "Math Number 2");
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap_M2, RS_REPCAP_BUF_SIZE - strlen(repCap) - strlen(repCap_M2) - 1);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_COUPLED_SPECTRUM, coupled),
            4, "Coupled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the signal type of the source of the math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR dataSignalType/Returns the signal type of the source of the math waveform.
ViStatus _VI_FUNC rsscope_QuerySpectrumAnalysisDataSignalType(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViInt32*  dataSignalType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_WAVEFORM_DATA_SIGNAL_TYPE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_WAVEFORM_DATA_SIGNAL_TYPE, dataSignalType),
            3, "Data Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the spectrum analysis Color.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR enabled/If enabled, the selected waveform is displayed according to its
/// HIPAR enabled/assigned color table.
/// HIPAR colorTableMode/Selects the color table mode for the frequency analysis display.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAnalysisColorTable(
                            ViSession instrSession,
                            ViInt32   mathNumber,
                            ViBoolean enabled,
                            ViInt32   colorTableMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_USE_COLOR_TABLE,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_USE_COLOR_TABLE, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_COLOR_TABLE_MODE, colorTableMode),
            4, "Color Table Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the spectrum analysis Timeline.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR timeline/Selects the timeline.
/// HIPAR enabled/Enables one of two possible time lines in a spectrogram diagram. A
/// HIPAR enabled/time line marks a single waveform in the spectrogram, that is: the
/// HIPAR enabled/power vs frequency results for the data acquired at a specific time.
/// HIPAR enabled/After enabling a time line, the results for that time are displayed in
/// HIPAR enabled/the spectrum diagram.
/// HIPAR position/Defines the position of one of the two possible time lines in a
/// HIPAR position/spectrogram.
ViStatus _VI_FUNC rsscope_ConfigureSpectrumAnalysisTimeline(ViSession instrSession,
                                                            ViInt32   mathNumber,
                                                            ViInt32   timeline,
                                                            ViBoolean enabled,
                                                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_TL[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMath = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Math", &maxMath));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mathNumber, RSSCOPE_REPCAP_MATH_M1, maxMath),
            2, "Math Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_TIMELINE_ENABLED,
            "Math", mathNumber- RSSCOPE_REPCAP_MATH_M1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Math Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeline, RSSCOPE_REPCAP_TIMELINE_TL1, RSSCOPE_REPCAP_TIMELINE_TL2),
            3, "Time Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_TIMELINE_ENABLED,
            "Timeline", timeline- RSSCOPE_REPCAP_TIMELINE_TL1, RS_REPCAP_BUF_SIZE, repCap_TL),
            3, "Time Line");
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap_TL, RS_REPCAP_BUF_SIZE - strlen(repCap) - strlen(repCap_TL) - 1);

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_TIMELINE_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_SPECTRUM_ANALYSIS_TIMELINE_POSITION, position),
            5, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  You can define two independent CDR instances to recover clock signals.
/// HIFN  These clock signals are used for data timing measurements: time
/// HIFN  interval error, unit interval, and datarate (frequency of the clock
/// HIFN  signal).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
/// HIPAR nominalBitRate/Sets the quiescent frequency of the PLL. It corresponds to the data
/// HIPAR nominalBitRate/rate of the data stream from which the clock is to be recovered.
/// HIPAR dataEdges/Selects the edges of the data stream that are used for the clock
/// HIPAR dataEdges/recovery.
/// HIPAR initialPhaseSync/Defines the phase reference for the first clock edge.
/// HIPAR selectedResults/Selects measurement to be read.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftware(ViSession instrSession,
                                               ViInt32   softwareNumber,
                                               ViReal64  nominalBitRate,
                                               ViInt32   dataEdges,
                                               ViInt32   initialPhaseSync,
                                               ViInt32   selectedResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_BIT_RATE,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_BIT_RATE, nominalBitRate),
            3, "Nominal Bit Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_DATA_EDGES, dataEdges),
            4, "Data Edges");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_INITIAL_PHASE_SYNC, initialPhaseSync),
            5, "Initial Phase Sync");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_SELECTED_RESULTS, selectedResults),
            6, "Selected Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures software Phase-locked loop parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CDR_SOFTWARE_ALGORITHM
/// HIFN     RSSCOPE_ATTR_CDR_SOFTWARE_PLL_ORDER
/// HIFN     RSSCOPE_ATTR_CDR_SOFTWARE_PLL_DAMPING
/// HIFN     RSSCOPE_ATTR_CDR_SOFTWARE_PLL_BANDWIDTH
/// HIFN     RSSCOPE_ATTR_CDR_SOFTWARE_PLL_REL_BANDWIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CDR:SOFTware<SW>:ALGorithm PLL
/// HIFN     CDR:SOFTware<SW>:PLL:ORDer
/// HIFN     CDR:SOFTware<SW>:PLL:DAMPing
/// HIFN     CDR:SOFTware<SW>:PLL:BWIDth
/// HIFN     CDR:SOFTware<SW>:PLL:RELBwidth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
/// HIPAR order/Sets the order of the PLL: first or second order. PLL of higher order
/// HIPAR order/can compensate for more complex jitter behavior.
/// HIPAR damping/Sets the damping factor, which is only relevant for second order PLL.
/// HIPAR bandwidth/Sets the PLL bandwidth. It defines the part of the spectrum that the
/// HIPAR bandwidth/PLL can follow during synchronization. The PLL bandwidth is usually
/// HIPAR bandwidth/defined by the transmission standard.
/// HIPAR relativeBandwidth/Sets the relative bandwidth, that is the ratio of the nominal bit rate
/// HIPAR relativeBandwidth/to the PLL bandwidth.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwarePLL(ViSession instrSession,
                                                  ViInt32   softwareNumber,
                                                  ViInt32   order,
                                                  ViReal64  damping,
                                                  ViReal64  bandwidth,
                                                  ViReal64  relativeBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_ALGORITHM,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    // checkErr(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_ALGORITHM, RSSCOPE_VAL_CDR_ALGORITHM_PLL));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_ORDER, order),
            3, "Order");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_DAMPING, damping),
            4, "Damping");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_BANDWIDTH, bandwidth),
            5, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_REL_BANDWIDTH, relativeBandwidth),
            6, "Relative Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the reverse lock PLL algorithm. The signal is locked on the
/// HIFN  first edge.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
/// HIPAR enabled/Enables the reverse lock PLL algorithm. The signal is locked on the
/// HIPAR enabled/first edge.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwarePLLReverseLockEnabled(
                            ViSession instrSession,
                            ViInt32   softwareNumber,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_REVERSE_LOCK_ENABLED,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");
    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_PLL_REVERSE_LOCK_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the serial standard. The nominal bit rate and bandwidth settings
/// HIFN  are set according to the selected standard, no estimation is required.
/// HIFN  Set CUSTOM to adjust the bit rate and the bandwidth manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
/// HIPAR serialStandard/Sets the serial standard. The nominal bit rate and bandwidth settings
/// HIPAR serialStandard/are set according to the selected standard, no estimation is required.
/// HIPAR serialStandard/Set CUSTOM to adjust the bit rate and the bandwidth manually.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwareSerialStandard(ViSession instrSession,
                                                             ViInt32   softwareNumber,
                                                             ViInt32   serialStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, softwareNumber, 1, 2),
            2, "Software Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_SERIAL_STANDARD,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_SERIAL_STANDARD, serialStandard),
            3, "Serial Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures ongoing correction of the bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
/// HIPAR enabled/Enables ongoing correction of the bit rate.
/// HIPAR source/Selects the source of the data stream, which is analized to recover
/// HIPAR source/the clock.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwareBitRateCorrection(
                            ViSession instrSession,
                            ViInt32   softwareNumber,
                            ViBoolean enabled,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, softwareNumber, 1, 2),
            2, "Software Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_BIT_RATE_CORRECTION_ENABLED,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_BIT_RATE_CORRECTION_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_BIT_RATE_SOURCE, source),
            4, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Estimates the bit rate once, for the current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects the software number.
ViStatus _VI_FUNC rsscope_CDRSoftwareEstimateBitRateOnce(ViSession instrSession,
                                                         ViInt32   softwareNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, softwareNumber, 1, 2),
            2, "Software Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_ESTIMATE_BIT_RATE_ONCE,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    checkErr(rsscope_SetAttributeViString(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_ESTIMATE_BIT_RATE_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the software algorithm that is used for clock data recovery.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects repeated capability SW.
/// HIPAR algorithm/Sets the software algorithm that is used for clock data recovery.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwareAlgorithm(ViSession instrSession,
                                                        ViInt32   softwareNumber,
                                                        ViInt32   algorithm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  softNumberCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SW", &softNumberCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, softwareNumber, 1, softNumberCount),
            2, "Software Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_ALGORITHM,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_ALGORITHM, algorithm),
            3, "Algorithm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets a sampling time for the clock signal, an offset for the clock
/// HIFN  edge in relation to the bit start. The clock edge sets the beginning of
/// HIFN  the unit interval. The sampling time is a number between 0 and 1. Value
/// HIFN  0 sets the clock edge to the beginning of the bit period; value 0.5
/// HIFN  sets the clock edge to the middle of the bit period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR softwareNumber/Selects repeated capability SW.
/// HIPAR samplingTime/Sets a sampling time for the clock signal, an offset for the clock
/// HIPAR samplingTime/edge in relation to the bit start. The clock edge sets the beginning of
/// HIPAR samplingTime/the unit interval. The sampling time is a number between 0 and 1. Value
/// HIPAR samplingTime/0 sets the clock edge to the beginning of the bit period; value 0.5
/// HIPAR samplingTime/sets the clock edge to the middle of the bit period.
ViStatus _VI_FUNC rsscope_ConfigureCDRSoftwareSamplingTime(ViSession instrSession,
                                                           ViInt32   softwareNumber,
                                                           ViReal64  samplingTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  softNumberCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "SW", &softNumberCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, softwareNumber, 1, softNumberCount),
            2, "Software Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_CDR_SOFTWARE_SAMPLING_TIME,
            "SW", softwareNumber- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Software Number");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, repCap, RSSCOPE_ATTR_CDR_SOFTWARE_SAMPLING_TIME, samplingTime),
            3, "Sampling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Hardware-based clock data recovery uses the digital CDR module to
/// HIFN  recover the clock edges. These clock edges can be used as trigger
/// HIFN  source for the CDR trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the channel signal that is used for clock recovery.
/// HIPAR nominalBitRate/Sets the quiescent frequency of the PLL. It corresponds to the data
/// HIPAR nominalBitRate/rate of the data stream from which the clock is to be recovered.
/// HIPAR dataEdges/Selects the edges of the data stream that are used for the clock
/// HIPAR dataEdges/recovery.
ViStatus _VI_FUNC rsscope_ConfigureCDRHardware(ViSession instrSession,
                                               ViInt32   source,
                                               ViReal64  nominalBitRate,
                                               ViInt32   dataEdges)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_SOURCE, source),
            2, "Source");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_BIT_RATE, nominalBitRate),
            3, "Nominal Bit Rate");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_DATA_EDGES, dataEdges),
            4, "Data Edges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures hardware Phase-locked loop parameters.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Only with RTO-K13 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CDR_HARDWARE_PLL_ORDER
/// HIFN     RSSCOPE_ATTR_CDR_HARDWARE_PLL_DAMPING
/// HIFN     RSSCOPE_ATTR_CDR_HARDWARE_PLL_BANDWIDTH
/// HIFN     RSSCOPE_ATTR_CDR_HARDWARE_PLL_REL_BANDWIDTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CDR:HARDware:PLL:ORDer
/// HIFN     CDR:HARDware:PLL:DAMPing
/// HIFN     CDR:HARDware:PLL:BWIDth
/// HIFN     CDR:HARDware:PLL:RELBwidth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR order/Sets the order of the PLL: first or second order. PLL of higher order
/// HIPAR order/can compensate for more complex jitter behavior.
/// HIPAR damping/Sets the damping factor, which is only relevant for second order PLL.
/// HIPAR bandwidth/Sets the PLL bandwidth. It defines the part of the spectrum that the
/// HIPAR bandwidth/PLL can follow during synchronization. The PLL bandwidth is usually
/// HIPAR bandwidth/defined by the transmission standard.
/// HIPAR relativeBandwidth/Sets the relative bandwidth, that is the ratio of the nominal bit rate
/// HIPAR relativeBandwidth/to the PLL bandwidth.
ViStatus _VI_FUNC rsscope_ConfigureCDRHardwarePLL(ViSession instrSession,
                                                  ViInt32   order,
                                                  ViReal64  damping,
                                                  ViReal64  bandwidth,
                                                  ViInt32   relativeBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_PLL_ORDER, order),
            2, "order");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_PLL_DAMPING, damping),
            3, "Damping");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_PLL_BANDWIDTH, bandwidth),
            4, "Bandwidth");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_PLL_REL_BANDWIDTH, relativeBandwidth),
            5, "Rrelative Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the serial standard. Set CUSTOM to adjust the bit rate and the
/// HIFN  bandwidth manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialStandard/Sets the serial standard. Set CUSTOM to adjust the bit rate and the
/// HIPAR serialStandard/bandwidth manually.
ViStatus _VI_FUNC rsscope_ConfigureCDRHardwareSerialStandard(ViSession instrSession,
                                                             ViInt32   serialStandard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_CDR_HARDWARE_SERIAL_STANDARD, serialStandard),
            2, "Serial Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Estimates the bite rate once, for the current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_CDRHardwareEstimateBitRateOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_CDR_HARDWARE_ESTIMATE_BIT_RATE_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an offset for the bit start, the beginning of the unit
/// HIFN  interval. The trigger point corresponds to the clock edge that
/// HIFN  indicates the bit start. The UI offset is a number between 0 and 1.
/// HIFN  Value 0 matches sets the clock edge to the beginning of the bit period;
/// HIFN  value 0.5 sets the clock edge to the middle of the bit period.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Only with RTO-K13 option.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_CDR_HARDWARE_TRIGGER_UI_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CDR:HARDware:UIOFfset
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uiOffset/Defines an offset for the bit start, the beginning of the unit
/// HIPAR uiOffset/interval.
ViStatus _VI_FUNC rsscope_ConfigureCDRHardwareTriggerUIOffset(
                            ViSession instrSession,
                            ViReal64  uiOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_CDR_HARDWARE_TRIGGER_UI_OFFSET, uiOffset),
            2, "UI Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  opc;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsscope_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(rsscope_QueryOPC(instrSession, &opc));
    }

    checkErr(rsscope_DefaultInstrSetup(instrSession));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsscope_self_test(ViSession instrSession,
                                    ViInt16*  testResult,
                                    ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;
    ViInt32  result = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy(testMessage, "Self-Test Passed.");
    else
        strcpy(testMessage, "Self-Test Failed.");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsscope_revision_query(ViSession instrSession,
                                         ViChar    instrumentDriverRevision[],
                                         ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION,
        0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION,
        0, 256, firmwareRevision));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the error/event queue for all unread items and removes them
/// HIFN  from the queue. The response is a comma-separated list of error number
/// HIFN  and a short description of the error in FIFO order.
/// HIFN     Positive error numbers are instrument-dependent. Negative error
/// HIFN  numbers are reserved by the SCPI standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the allocated buffer passed to the System Errors
/// HIPAR bufferSize/parameter.
/// HIPAR systemErrors/Queries the error/event queue for all unread items and removes them
/// HIPAR systemErrors/from the queue. The response is a comma-separated list of error number
/// HIPAR systemErrors/and a short description of the error in FIFO order.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument. If this
/// HIPAR bytesRead/number is larger than Buffer Size, some data was lost.
ViStatus _VI_FUNC rsscope_FetchAllSystemErrors(ViSession instrSession,
                                               ViUInt32  bufferSize,
                                               ViChar    systemErrors[],
                                               ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSSCOPE_ATTR_SYSTEM_ERROR_ALL, NULL, &cmd));
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, systemErrors, bytesRead));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsscope_error_query(ViSession instrSession,
                                      ViInt32*  errorCode,
                                      ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsscope_error_message(ViSession instrSession,
                                        ViStatus  errorCode,
                                        ViChar    errorMessage[])
{
    return rsscope_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  Sets the endianess.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR byteOrder/Sets the endianess.
ViStatus _VI_FUNC rsscope_ByteOrder(ViSession instrSession,
                                    ViInt32   byteOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_BYTE_ORDER, byteOrder),
            2, "Byte Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the Enable, NTransition, and PTransition bits
/// HIFN  of the questionable registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR ptransition/register.
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR ntransition/register.
ViStatus _VI_FUNC rsscope_SetStatusRegister(ViSession instrSession,
                                            ViInt32   questionableRegister,
                                            ViInt32   enable,
                                            ViInt32   ptransition,
                                            ViInt32   ntransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, questionableRegister, RSSCOPE_VAL_REG_COV, RSSCOPE_VAL_REG_DEEM),
            2, "Questionable Register");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, enable, 0, 65535),
            3, "Enable");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ptransition, 0, 65535),
            4, "PTransition");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ntransition, 0, 65535),
            5, "NTransition");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:QUES:%s:ENAB %ld;PTR %ld;NTR %ld",
             questionableRegisterArr[questionableRegister], enable, ptransition, ntransition);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/Selects register part to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsscope_GetStatusRegister(ViSession instrSession,
                                            ViInt32   questionableRegister,
                                            ViInt32   registerPart,
                                            ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, questionableRegister, RSSCOPE_VAL_REG_COV, RSSCOPE_VAL_REG_DEEM),
            2, "Questionable Register");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSSCOPE_VAL_REG_COND, RSSCOPE_VAL_REG_PTR),
            3, "Register Part");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:QUES:%s:%s?", questionableRegisterArr[questionableRegister], registerPartArr[registerPart]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsscope_ConfigureErrorChecking(ViSession instrSession,
                                                 ViBoolean optionChecking,
                                                 ViBoolean rangeChecking,
                                                 ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN This function sets OPC timeout value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Timeout value
ViStatus _VI_FUNC rsscope_SetOPCTimeout(ViSession instrSession,
                                        ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32)timeout));

Error:
    return error;
}

/// HIFN This function gets OPC timeout value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Timeout value
ViStatus _VI_FUNC rsscope_GetOPCTimeout(ViSession instrSession,
                                        ViUInt32* timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32*)timeout));

Error:
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsscope_SetVISATimeout(ViSession instrSession,
                                         ViUInt32  VISATimeout)
{
    return RsCore_SetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsscope_GetVISATimeout(ViSession instrSession,
                                         ViUInt32* VISATimeout)
{
    return RsCore_GetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rsscope_IDQueryResponse(ViSession instrSession,
                                          ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the instrument ID - that is the material number and the serial
/// HIFN  number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the allocated buffer passed to the Device ID parameter.
/// HIPAR deviceID/Returns the instrument ID - that is the material number and the serial
/// HIPAR deviceID/number. The array should consist of at least 256 elements.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument. If this
/// HIPAR bytesRead/number is larger than Buffer Size, some data was lost.
ViStatus _VI_FUNC rsscope_QueryDeviceSerialNumber(ViSession instrSession,
                                                  ViUInt32  bufferSize,
                                                  ViChar    deviceID[],
                                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSSCOPE_ATTR_SYSTEM_DEVICE_ID, NULL, &cmd));
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, deviceID, bytesRead));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rsscope_QueryOPC(ViSession instrSession,
                                   ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the model name of the oscilloscope.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SCOPE_MODEL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DIAGnostic:SERVice:WFAModel?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR model/Returns the model name of the oscilloscope.
ViStatus _VI_FUNC rsscope_ScopeModel(ViSession instrSession,
                                     ViInt32*  model)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SCOPE_MODEL, model),
            2, "Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the model series of the oscilloscope.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SCOPE_SERIES
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DIAGnostic:SERVice:WFASeries?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR series/Returns the model series of the oscilloscope.
ViStatus _VI_FUNC rsscope_ScopeSeries(ViSession instrSession,
                                      ViInt32*  series)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SCOPE_SERIES, series),
            2, "Series");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the instrument family of the oscilloscope.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SCOPE_FAMILY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DIAGnostic:SERVice:WFAType?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR family/Returns the instrument family of the oscilloscope.
ViStatus _VI_FUNC rsscope_ScopeFamily(ViSession instrSession,
                                      ViInt32*  family)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_SCOPE_FAMILY, family),
            2, "Family");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the date of the internal calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Year, to be entered as a four-digit number (including the century and
/// HIPAR year/millennium information)
/// HIPAR month/Month, 1 (January) to 12 (December)
/// HIPAR day/Day, 1 to the maximum number of days in the specified month
ViStatus _VI_FUNC rsscope_SetSystemDate(ViSession instrSession,
                                        ViInt32   year,
                                        ViInt32   month,
                                        ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "SYST:DATE '";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1000, 9999),
            2, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");

    snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "SYST:DATE %ld,%ld,%ld", year, month, day);
    checkErr(rsscope_WriteInstrData(instrSession, cmd));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the date of the internal calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Returns the year of the current instrument system date.
/// HIPAR month/Returns the month of the current instrument system date.
/// HIPAR day/Returns the day of the current instrument system date.
ViStatus _VI_FUNC rsscope_QuerySystemDate(ViSession instrSession,
                                          ViInt32*  year,
                                          ViInt32*  month,
                                          ViInt32*  day)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViInt32(instrSession, "SYST:DATE?", year, month, day, NULL));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the UTC (Universal Time Coordinated) of the internal clock. To
/// HIFN  define the current local time, use the time zone setting of the
/// HIFN  operating system (Setup > Time, date).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hour/Returns the hour of the current instrument system time.
/// HIPAR minutes/Returns the minutes of the current instrument system time.
/// HIPAR seconds/Returns the seconds of the current instrument system time.
ViStatus _VI_FUNC rsscope_QuerySystemTime(ViSession instrSession,
                                          ViInt32*  hour,
                                          ViInt32*  minutes,
                                          ViInt32*  seconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViInt32(instrSession, "SYST:TIME?", hour, minutes, seconds, NULL));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of the status bit that belongs to the indicated
/// HIFN  mask test in the STATus:QUEStionable:MASK register. The status bits are
/// HIFN  assigned in the order of mask creation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maskTestName/String with the name of the mask test.
/// HIPAR status/Returns the number of the status bit that belongs to the indicated
/// HIPAR status/mask test in the STATus:QUEStionable:MASK register. The status bits are
/// HIPAR status/assigned in the order of mask creation.
ViStatus _VI_FUNC rsscope_QueryStatusBitNumber(ViSession instrSession,
                                               ViString  maskTestName,
                                               ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MTES:SBIT? '%s'", maskTestName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, status));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches OPC-synchronization ON/OFF for writes.
///HINF
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for writes.
ViStatus _VI_FUNC rsscope_OPCSyncWriteEnable(ViSession instrSession,
                                             ViBoolean enable)
{  
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrWrites = enable;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches OPC-synchronization ON/OFF for queries.
///HINF
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for queries.
ViStatus _VI_FUNC rsscope_OPCSyncQueryEnable(ViSession instrSession,
                                             ViBoolean enable)
{  
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrQueries = enable;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the mechanism which is used for OPC-synchronised queries
///HINF  (valid for read attributes and all OPC-synchronised queries):
///HINF  Default value: 'useEsb' = true, 'clsBefore' = true, 'useMav' =
///HINF  false
///HINF
///HINF  Scenario A: useEsb = true:
///HINF  The query command is appended by the ';*OPC', and the wait
///HINF  loop ends if the ESB bit signals that the command was completed,
///HINF  or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore has no effect and is always set to true.
///HINF      - set the 'useMav' to true, to enable the opc-wait loop
///HINF  sucessfully finish if the MAV bit signals, that the response is
///HINF  available, even if the ESB bit is still 0.
///HINF
///HINF  Scenario B: useEsb = false:
///HINF  The query command is not appended by the ';*OPC', and the wait
///HINF  loop ends if the MAV bit signals that the message is
///HINF  available, or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore causes sending query *STB? before sending the
///HINF  command to clear the error queue and output buffers.
///HINF      - useMav has no effect, and is always set to true.
///HINF
///HIRET Returns the status code of this operation.
///HIPAR useEsb/Switches queries OPC-synchronization to use the Event
///HIPAR useEsb/Status Bit.
///HIPAR clsBefore/Switches queries OPC-synchronization to use call
///HIPAR clsBefore/the Clear Before Read procedure before sending the
///HIPAR clsBefore/query.
///HIPAR useMav/Switches queries OPC-synchronization to react on
///HIPAR useMav/Message Available bit (MAV)
ViStatus _VI_FUNC rsscope_SetOpcSyncQueryMechanism(ViSession instrSession,
                                                   ViBoolean useEsb,
                                                   ViBoolean clsBefore,
                                                   ViBoolean useMav)
{
    return RsCore_SetOpcSyncQueryMechanism(instrSession, useEsb, clsBefore, useMav);
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsscope_GetError(ViSession instrSession,
                                   ViStatus* errorCode,
                                   ViInt32   bufferSize,
                                   ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(
        RsCore_GetErrorCompleteDescription(instrSession, &rsscope_utility_error_message, errorCode, bufferSize, description));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsscope_WriteInstrData(ViSession instrSession,
                                         ViString  writeBuffer)
{
    return RsCore_Write(instrSession, writeBuffer);
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsscope_ReadInstrData(ViSession instrSession,
                                        ViInt32   numberOfBytesToRead,
                                        ViChar    readBuffer[],
                                        ViUInt32* numBytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(readBuffer, numberOfBytesToRead, responseString));
    *numBytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsscope_QueryViBoolean(ViSession  instrSession,
                                         ViString   command,
                                         ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViBoolean(instrSession, command, value));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsscope_QueryViInt32(ViSession instrSession,
                                       ViString  command,
                                       ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViInt32(instrSession, command, value));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsscope_QueryViReal64(ViSession instrSession,
                                        ViString  command,
                                        ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViReal64(instrSession, command, value));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsscope_QueryViString(ViSession instrSession,
                                        ViString  command,
                                        ViInt32   bufferSize,
                                        ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  extBuf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (null string length)");
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, command, 0, &extBuf));
    RsCore_CopyToUserBufferAsciiData(value, bufferSize, extBuf);

Error:
    if (extBuf) free(extBuf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current instrument settings under the specified number in
/// HIFN  an intermediate memory. The settings can be recalled using
/// HIFN  rsscope_RecallInstrumentSettings with the associated number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR number/Specifies a number under which to store the current instrument
/// HIPAR number/settings.
ViStatus _VI_FUNC rsscope_SaveInstrumentSettings(ViSession instrSession,
                                                 ViInt32   number)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_SAVE, number),
            2, "Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the instrument settings from an intermediate memory identified
/// HIFN  by the specified number. The instrument settings can be stored to this
/// HIFN  memory using rsscope_SaveInstrumentSettings with the associated number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR number/Specifies a number under which instrument settings have been stored.
ViStatus _VI_FUNC rsscope_RecallInstrumentSettings(ViSession instrSession,
                                                   ViInt32   number)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_RECALL, number),
            2, "Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function provides mass storage capabilities for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If
/// HIPAR source/the operation requires a single parameter, this control specifies that
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsscope_FileManagerOperations(ViSession instrSession,
                                                ViInt32   operation,
                                                ViString  source,
                                                ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_FILE_COPY:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:COPY '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
        break;
    case RSSCOPE_VAL_FILE_MOVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:MOVE '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
        break;
    case RSSCOPE_VAL_FILE_DELETE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_DELETE_FILE, source),
                3, "Source");
        break;
    case RSSCOPE_VAL_FILE_NEW_DIR:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CREATE_DIRECTORY, source),
                3, "Source");
        break;
    case RSSCOPE_VAL_FILE_CDIR:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CURRENT_DIRECTORY, source),
                3, "Source");
        break;
    case RSSCOPE_VAL_FILE_RDIR:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REMOVE_DIRECTORY, source),
                3, "Source");
        break;
    case RSSCOPE_VAL_FILE_CDRIVE:
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_CHANGE_DRIVE, source),
                3, "Source");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets file attributes for the specified file(s). The function can be
/// HIFN  used for files only.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:ATTRibute
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR file/Contains name and path of the file. Wildcards (* and ?) are allowed.
/// HIPAR attributes/String with attributes and setting information.
ViStatus _VI_FUNC rsscope_ChangeFileAttributes(ViSession instrSession,
                                               ViString  file,
                                               ViString  attributes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:ATTR '%s','%s'", file, attributes);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of files and subdirectories of the current or of a
/// HIFN  specified directory. The number includes the parent directory strings
/// HIFN  "." and ".." so that it corresponds to the number of strings returned
/// HIFN  by the rsscope_FileDirectoryContent function after the initial numeric
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberofFilesAndFolders/Returns the number of files and subdirectories of the current or of a
/// HIPAR numberofFilesAndFolders/specified directory.
ViStatus _VI_FUNC rsscope_FileCount(ViSession instrSession,
                                    ViString  directory,
                                    ViInt32*  numberofFilesAndFolders)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckAlloc(numberofFilesAndFolders);

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG?");

    if ((directory != NULL) && (strlen(directory) > 0))
        sprintf(p2buf, " '%s'", directory);

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberofFilesAndFolders));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the indicated directory. According to DOS
/// HIFN  convention, wild card characters can be entered in order to query eg. a
/// HIFN  list of all file of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as comma
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsscope_FileDirectoryContent(ViSession instrSession,
                                               ViString  directory,
                                               ViInt32   bufferSize,
                                               ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckAlloc(output);

    p2buf += sprintf(p2buf, "MMEM:CAT?");

    if ((directory != NULL) && (strlen(directory) > 0))
        p2buf += sprintf(p2buf, " '%s'", directory);

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, cmd, 0, bufferSize, output, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of subdirectories of the current or of a specified
/// HIFN  directory. The number includes the parent directory strings "." and
/// HIFN  ".." so that it corresponds to the number of strings returned by the
/// HIFN  rsscope_FileSubdirectories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberofSubdirectories/Returns the number of files and subdirectories of the current or of a
/// HIPAR numberofSubdirectories/specified directory.
ViStatus _VI_FUNC rsscope_FileSubdirectoriesCount(ViSession instrSession,
                                                  ViString  directory,
                                                  ViInt32*  numberofSubdirectories)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckAlloc(numberofSubdirectories);

    p2buf += sprintf(p2buf, "MMEM:DCAT:LENG?");
    if ((directory != NULL) && (strlen(directory) > 0))
        sprintf(p2buf, " '%s'", directory);

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberofSubdirectories));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the subdirectories of the current or of a specified directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as comma
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsscope_FileSubdirectories(ViSession instrSession,
                                             ViString  directory,
                                             ViInt32   bufferSize,
                                             ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckAlloc(output);

    p2buf += sprintf(p2buf, "MMEM:DCAT?");
    if ((directory != NULL) && (strlen(directory) > 0))
        p2buf += sprintf(p2buf, " '%s'", directory);

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, cmd, 0, bufferSize, output, NULL));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    if (p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of the drives of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/Returns a list of the drives of the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
ViStatus _VI_FUNC rsscope_FileDrives(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    output[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViString(instrSession, "", RSSCOPE_ATTR_DRIVES,
            bufferSize, output),
            2, "Drives");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rsscope_ReadToFileFromInstrument(ViSession instrSession,
                                                   ViString  source,
                                                   ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the host computer and write it
/// HIFN  to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/control computer to instrument applies.
ViStatus _VI_FUNC rsscope_WriteFromFileToInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows to provide data set operation for Save/Recall
/// HIFN  items of memory subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR stateNumber/Defines save and recall state position.
/// HIPAR path/This control specifies the source of the operation to be performed.
ViStatus _VI_FUNC rsscope_DataSetFileOperations(ViSession instrSession,
                                                ViInt32   operation,
                                                ViInt32   stateNumber,
                                                ViString  path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSCOPE_VAL_MEM_SAVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:STAT %ld,'%s'", stateNumber, path);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
        break;
    case RSSCOPE_VAL_MEM_RECALL:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:STAT %ld,'%s'", stateNumber, path);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsscope_CheckStatus(instrSession));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If set to Off, the saveset is stored without the preview image to
/// HIFN  reduce the file size. Use this function each time before you save a
/// HIFN  saveset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR includeImage/If set to Off, the saveset is stored without the preview image to
/// HIPAR includeImage/reduce the file size.
ViStatus _VI_FUNC rsscope_ConfigureSavesetPreviewImage(ViSession instrSession,
                                                       ViBoolean includeImage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_SAVESET_PREVIEW_IMAGE, includeImage),
            2, "Include Image");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  If ON, the user preferences settings are also restored
///HINF  together with the saveset and reference waveform. User preferences
///HINF  settings include the user-specific display settings like diagram
///HINF  layout, toolbar, intensity and transparency settings.
///HIRET Returns the status code of this operation.
///HIPAR status/If ON, the user preferences settings are also restored
///HIPAR status/together with the saveset and reference waveform.
ViStatus _VI_FUNC rsscope_ConfigureSavesetOneFileUserPreference(
                            ViSession instrSession,
                            ViBoolean status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_SAVESET_ONE_FILE_USER_PREFERENCE, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The One File function stores the savesets file and the data of active
/// HIFN  reference waveforms together in a ZIP file. If you want to keep channel
/// HIFN  waveform data together with the instrument settings, make sure to
/// HIFN  create a reference waveform from the channel waveform before you save
/// HIFN  the One File.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_SAVESET_ONE_FILE_NAME
/// HIFN     RSSCOPE_ATTR_SAVESET_ONE_FILE_SAVE
/// HIFN     RSSCOPE_ATTR_SAVESET_ONE_FILE_OPEN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SAVeset:ONEFile:NAME
/// HIFN     SAVeset:ONEFile:SAVE
/// HIFN     SAVeset:ONEFile:OPEN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of operation to be performed.
/// HIPAR name/Sets the path and the file name of the One File. The file format is
/// HIPAR name/ZIP.
ViStatus _VI_FUNC rsscope_OneFileOperations(ViSession instrSession,
                                            ViInt32   operation,
                                            ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAVESET_ONE_FILE_NAME, name),
            3, "Name");
    switch (operation)
    {
    case RSSCOPE_VAL_SAVE:
        checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAVESET_ONE_FILE_SAVE, NULL));
        break;
    case RSSCOPE_VAL_OPEN:
        checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAVESET_ONE_FILE_OPEN, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes the indicated channel in waveform export. The
/// HIFN  data of channel waveform 1 is exported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR exportChannel/Includes or excludes the indicated channel in waveform export. The
/// HIPAR exportChannel/data of channel waveform 1 is exported.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportChannel(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean exportChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSCOPE_ATTR_WAVEFORM_EXPORT_CHANNEL,
            "Channel", channel- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, repCap, RSSCOPE_ATTR_WAVEFORM_EXPORT_CHANNEL, exportChannel),
            3, "Export Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the multichannel export.
/// HIFN  If enabled, select the channels to be exported using the
/// HIFN  rsscope_ConfigureWaveformExportChannel function.
/// HIFN  If disabled, select the waveform to be exported using the
/// HIFN  rsscope_ConfigureWaveformExportSource function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multichannelExport/Enables or disables the multichannel export.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportMultichannel(
                            ViSession instrSession,
                            ViBoolean multichannelExport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_MULTICHANNEL, multichannelExport),
            3, "Multichannel Export");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source waveform for export from the
/// HIFN  active waveforms of input channels, math signals and reference
/// HIFN  waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Waveform to be assigned.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportSource(ViSession instrSession,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_WAVEFORM_EXPORT_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the zoom area to be used for limited data
/// HIFN  export. The data included in the zoom area are saved, if at least one
/// HIFN  zoom is defined for the source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramName/Name of the zoom diagram
/// HIPAR zoomName/Name of the diagram on which the zoom area is based.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeZoom(ViSession instrSession,
                                                           ViString  diagramName,
                                                           ViString  zoomName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE, RSSCOPE_VAL_WAV_EXPORT_ZOOM));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "EXP:WAV:ZOOM '%s','%s'", diagramName, zoomName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the cursor set to be used for limited data
/// HIFN  export. The data between the cursor lines are saved, if at least one
/// HIFN  cursor measurement is defined for the source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cursor/This control selects the cursor set to be used for limited data
/// HIPAR cursor/export.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeCursor(ViSession instrSession,
                                                             ViInt32   cursor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE, RSSCOPE_VAL_WAV_EXPORT_CURSOR));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_CURSOR, cursor),
            2, "Cursor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures gate to be used for limited data export. The
/// HIFN  data included in the measurement gate are saved, if a gated measurement
/// HIFN  is defined for the source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement for which the gate is defined.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeGate(ViSession instrSession,
                                                           ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE, RSSCOPE_VAL_WAV_EXPORT_GATE));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_GATE, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures manual interval used for limited data export.
/// HIFN  The data are saved between user-defined "Start" and "Stop" values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR start/Sets the start value of the waveform section for export.
/// HIPAR stop/Sets the end value of the waveform section for export.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeManual(ViSession instrSession,
                                                             ViReal64  start,
                                                             ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE, RSSCOPE_VAL_WAV_EXPORT_MANUAL));

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_START, start),
            2, "Start");

    viCheckParm(rsscope_SetAttributeViReal64(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE_STOP, stop),
            3, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures export to save the complete waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeComplete(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_SCOPE, RSSCOPE_VAL_WAV_EXPORT_COMPLETE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables "Data logging". This enables the
/// HIFN  export of all waveforms of a running acquisition into one file. The
/// HIFN  waveform records are written in historical order one after the other,
/// HIFN  either the complete records or the sections as defined in "Scope". Set
/// HIFN  the number of acquisitions to be acquired and stored with "Acq. count".
/// HIFN  The maximum amount of data that can be written is shown in "Max. file
/// HIFN  size".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataLogging/Enables the export of several subsequent waveforms into one file.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportScopeDataLogging(
                            ViSession instrSession,
                            ViBoolean dataLogging)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_DATA_LOGGING, dataLogging),
            2, "Data Logging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the relative timestamps of all history waveforms to the
/// HIFN  waveform data file. The time is written at the beginning of each
/// HIFN  waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timestamps/Exports the relative timestamps of all history waveforms to the
/// HIPAR timestamps/waveform data file. The time is written at the beginning of each
/// HIPAR timestamps/waveform record.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportTimestamps(ViSession instrSession,
                                                            ViBoolean timestamps)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_TIMESTAMPS, timestamps),
            2, "Timestamps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  To improve the performance of data export to file, the measurements
/// HIFN  are performed slower while the data export speeds up.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastExport/To improve the performance of data export to file, the measurements
/// HIPAR fastExport/are performed slower while the data export speeds up.
ViStatus _VI_FUNC rsscope_ConfigureWaveformExportFastExport(ViSession instrSession,
                                                            ViBoolean fastExport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_FAST_EXPORT, fastExport),
            3, "Fast Export");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes horizontal values in the export data (time or frequency
/// HIFN  values, depending on the waveform). X and Y-values are written
/// HIFN  alternately to the file. If disabled, only Y-values - mostly voltage
/// HIFN  values - are written.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interleavedXY/Includes horizontal values in the export data.
ViStatus _VI_FUNC rsscope_WaveformExportInterleavedXY(ViSession instrSession,
                                                      ViBoolean interleavedXY)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_INTERLEAVED_XY, interleavedXY),
            2, "Interleaved XY");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the export of raw sample data of the ADC, and sets the data
/// HIFN  format to integer 8 bit. This format reduces the file size (1
/// HIFN  Byte/sample instead of 4 Bytes/sample in binary files) but decreases
/// HIFN  the precision of the values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rawValues/Enables the export of raw sample data of the ADC, and sets the data
/// HIPAR rawValues/format to integer 8 bit.
ViStatus _VI_FUNC rsscope_WaveformExportRawValues(ViSession instrSession,
                                                  ViBoolean rawValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_RAW_VALUES, rawValues),
            2, "Raw Values");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the waveform(s) to the specified file name and
/// HIFN  path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/Stes the file name and path to save the waveform to.
ViStatus _VI_FUNC rsscope_WaveformExportFile(ViSession instrSession,
                                             ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_FILE, file),
            2, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the display update during an Nx SINGLE
/// HIFN  acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayUpdate/Enables or disables the display update during an Nx SINGLE
/// HIPAR displayUpdate/acquisition.
ViStatus _VI_FUNC rsscope_ConfigureWaveformNxSingleAcquisitionDisplayUpdate(
                            ViSession instrSession,
                            ViBoolean displayOff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_WAVEFORM_EXPORT_DISPLAY_OFF, displayOff),
            2, "Display Update");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the histogram to be exported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR histogramName/Selects the histogram to be exported.
ViStatus _VI_FUNC rsscope_ConfigureHistogramExportName(ViSession instrSession,
                                                       ViString  histogramName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HISTOGRAM_EXPORT_NAME, histogramName),
            2, "Histogram Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the mode of exported data: relative or absolute
/// HIFN  frequency of amplitude values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR incidence/Sets the mode of exported data: relative or absolute frequency of
/// HIPAR incidence/amplitude values.
ViStatus _VI_FUNC rsscope_ConfigureHistogramExportIncidence(ViSession instrSession,
                                                            ViInt32   incidence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_HISTOGRAM_EXPORT_INCIDENCE, incidence),
            2, "Incidence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the histogram waveform to the specified file name
/// HIFN  and path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/Sets the file name and path to save the histogram to. The file
/// HIPAR file/extension defines the file format: XML, CSV, or BIN.
ViStatus _VI_FUNC rsscope_HistogramExportFile(ViSession instrSession,
                                              ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HISTOGRAM_EXPORT_FILE, file),
            2, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_HISTOGRAM_EXPORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function transfers the histogram data to the controlling
/// HIFN  computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Pass the number of elements in the Data parameter.
/// HIPAR data/Returns the histogram waveform data.
/// HIPAR actualPoints/Indicates the number of points the function places in the Data
/// HIPAR actualPoints/parameter.
ViStatus _VI_FUNC rsscope_FetchHistogramExportData(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViReal64  data[],
                                                   ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "EXP:HIST:DATA?");
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the results that you want to save to file. All results are
/// HIFN  written into one file. To save several result boxes into one file, use
/// HIFN  the function several times, one command for each result box.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EXPort:RESult:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR item/String parameter, contains the name of the result box as written in
/// HIPAR item/the "Numeric Results" dialog box.
/// HIPAR select/Sets the state.
ViStatus _VI_FUNC rsscope_NumericResultsSelectItem(ViSession instrSession,
                                                   ViString  item,
                                                   ViBoolean select)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "EXPort:RESult:SELect '%s',%s", item, select ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures teh numeric results delimiter and sets the units usage.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_NUMERIC_RESULTS_DELIMITER
/// HIFN     RSSCOPE_ATTR_NUMERIC_RESULTS_OMIT_UNITS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EXPort:RESult:DELimiter
/// HIFN     EXPort:RESult:NUMeric
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR delimiter/Selects the value delimiter that is used to convert the values in
/// HIPAR delimiter/columns. For MS Excel, the semicolon is recommended to be used.
/// HIPAR omitUnits/If ON, the result values are saved without unit and with more decimal
/// HIPAR omitUnits/places.
ViStatus _VI_FUNC rsscope_ConfigureNumericResultsParameters(ViSession instrSession,
                                                            ViInt32   delimiter,
                                                            ViBoolean omitUnits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NUMERIC_RESULTS_DELIMITER, delimiter),
            2, "Delimiter");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_NUMERIC_RESULTS_OMIT_UNITS, omitUnits),
            3, "Omit Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets if point or comma is used as a decimal symbol.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_NUMERIC_RESULTS_DECIMAL_SYMBOL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EXPort:RESult:DECSymbol
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR decimal/Sets if point or comma is used as a decimal symbol.
ViStatus _VI_FUNC rsscope_ConfigureNumericResultsDecimalSymbol(
                            ViSession instrSession,
                            ViInt32   decimal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_NUMERIC_RESULTS_DECIMAL_SYMBOL, decimal),
            2, "Decimal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the path, the file name, and the file format for the numeric
/// HIFN  results file and saves the file.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSCOPE_ATTR_NUMERIC_RESULTS_EXPORT_FILE_NAME
/// HIFN     RSSCOPE_ATTR_NUMERIC_RESULTS_SAVE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     EXPort:RESult:NAME
/// HIFN     EXPort:RESult:SAVE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR file/Sets the path, the file name, and the file format for the numeric
/// HIPAR file/results file. Available file formats are CSV and HTML.
ViStatus _VI_FUNC rsscope_SaveNumericResultsBoxes(ViSession instrSession,
                                                  ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_NUMERIC_RESULTS_EXPORT_FILE_NAME, file),
            2, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_NUMERIC_RESULTS_SAVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the measurement for export of long term or
/// HIFN  measurement histogram data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement to be exported.
ViStatus _VI_FUNC rsscope_ConfigureLongTermExportResult(ViSession instrSession,
                                                        ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_LONG_TERM_EXPORT_RESULT, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects which result data you want to export, the long
/// HIFN  term measurement, or the measurement histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects which result data you want to export, the long term
/// HIPAR type/Seasurement, or the measurement histogram.
ViStatus _VI_FUNC rsscope_ConfigureLongTermExportType(ViSession instrSession,
                                                      ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_LONG_TERM_EXPORT_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the long term data to the specified file name and
/// HIFN  path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/Sets the file name and path to save the long term data to. The file
/// HIPAR file/extension defines the file format: XML, CSV, or BIN.
ViStatus _VI_FUNC rsscope_LongTermExportFile(ViSession instrSession,
                                             ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_LONG_TERM_EXPORT_FILE, file),
            2, "File");

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_LONG_TERM_EXPORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function transfers the long term measurement data to the
/// HIFN  controlling computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Pass the number of elements in the Data parameter.
/// HIPAR data/Returns the long term measurement data.
/// HIPAR actualPoints/Indicates the number of points the function places in the Data
/// HIPAR actualPoints/parameter.
ViStatus _VI_FUNC rsscope_FetchLongTermExportData(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViReal64  data[],
                                                  ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "EXP:MEAS:DATA?");
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, actualPoints));
    checkErr(rsscope_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the page layout of report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR language/Selects the language to be used in the report. Available languages are
/// HIPAR language/listed in the data sheet.
/// HIPAR paperSize/Selects the paper size: A4 or US Letter.
ViStatus _VI_FUNC rsscope_ReportControl(ViSession instrSession,
                                        ViString  language,
                                        ViInt32   paperSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REPORT_LANGUAGE, language),
            2, "Language");

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REPORT_PAPER_SIZE, paperSize),
            3, "Paper Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the report comment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR user/Enter the user name that appears in the general information section at
/// HIPAR user/the beginning of the report.
/// HIPAR comment/Enter a comment that appears in the general information section at the
/// HIPAR comment/beginning of the report.
ViStatus _VI_FUNC rsscope_ReportComment(ViSession instrSession,
                                        ViString  user,
                                        ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REPORT_USER, user),
            2, "User");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REPORT_COMMENT, comment),
            3, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the logo of the report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR logoType/By default, the Rohde & Schwarz logo is shown in the header of the
/// HIPAR logoType/report pages. You can switch the logo off, or select your logo to be
/// HIPAR logoType/shown.
/// HIPAR logoFile/Defines the logo file that is used on the report if Logo Type is set
/// HIPAR logoFile/to custom.
ViStatus _VI_FUNC rsscope_ReportLogo(ViSession instrSession,
                                     ViInt32   logoType,
                                     ViString  logoFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, "", RSSCOPE_ATTR_REPORT_LOGO_TYPE, logoType),
            2, "Logo Type");

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REPORT_LOGO_FILE, logoFile),
            3, "Logo File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the file name and path to save the report to. The
/// HIFN  file extension defines the file format: PDF, HTML, or DOC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Sets the file name and path to save the report to. The file extension
/// HIPAR fileName/defines the file format: PDF, HTML, or DOC.
ViStatus _VI_FUNC rsscope_ReportFileName(ViSession instrSession,
                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_REPORT_FILE_NAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function saves the report to the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_SaveReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_SAVE_REPORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Selects the eye that you want to save to a file.
///HIRET Returns the status code of this operation.
///HIPAR eyeSelect/Selects the eye that you want to save to a file.
ViStatus _VI_FUNC rsscope_ConfigureExportEyeSelect(ViSession instrSession,
                                                   ViInt32   eyeSelect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXPORT_EYE_SELECT, eyeSelect),
            2, "Eye Select");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Returns the source of the eye measurement, selected with
///HINF  RSSCOPE_ATTR_EXPORT_EYE_SELECT.
///HIRET Returns the status code of this operation.
///HIPAR exportEyeSource/Returns the source of the eye measurement,
///HIPAR exportEyeSource/selected with RSSCOPE_ATTR_EXPORT_EYE_SELECT.
ViStatus _VI_FUNC rsscope_QueryExportEyeSource(ViSession instrSession,
                                               ViInt32*  exportEyeSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_GetAttributeViInt32(instrSession, NULL, RSSCOPE_ATTR_EXPORT_EYE_SOURCE, exportEyeSource),
            2, "Export Eye Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the path, the file name, and the file format for the eye
///HINF  file. Available file formats are *csv and *bin.
///HIRET Returns the status code of this operation.
///HIPAR eyePath/Sets the path, the file name, and the file format for
///HIPAR eyePath/the eye file. Available file formats are *csv and *bin.
ViStatus _VI_FUNC rsscope_ConfigureExportEyePath(ViSession instrSession,
                                                 ViString  eyePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_EXPORT_EYE_PATH, eyePath),
            2, "Eye Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Saves the selected eye  to the file that is specified with
///HINF  RSSCOPE_ATTR_EXPORT_EYE_PATH.
///HIRET Returns the status code of this operation.
ViStatus _VI_FUNC rsscope_SaveExportEye(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_EXPORT_EYE_SAVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Transfers the eye data to the controlling computer. The data
///HINF  can be used in MATLAB, for example.
///HIRET Returns the status code of this operation.
///HIPAR bufferSize/Size of output array.
///HIPAR data/Transfers the eye data to the controlling computer. The
///HIPAR data/data can be used in MATLAB, for example.
///HIPAR returnedValues/Returns number of results returned from the
///HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsscope_ExportEyeData(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViReal64  data[],
                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
                 RSSCOPE_ATTR_HIDDEN_EXPORT_EYE_DATA, NULL, bufferSize, data, returnedValues), 3, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  If ON, the settings from a previously defined preset file are
///HINF  restored when the Preset key is pressed. The preset file is
///HINF  defined with RSSCOPE_ATTR_USER_DEFINED_PRESET_PATH_AND_FILE_NAME.
///HIRET Returns the status code of this operation.
///HIPAR enabled/If ON, the settings from a previously defined preset
///HIPAR enabled/file are restored when the Preset key is pressed.
ViStatus _VI_FUNC rsscope_ConfigureUserDefinedPresetEnabled(ViSession instrSession,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, NULL, RSSCOPE_ATTR_USER_DEFINED_PRESET_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the path and file name of a user-defined preset file.
///HIRET Returns the status code of this operation.
///HIPAR pathName/Sets the path and file name of a user-defined preset
///HIPAR pathName/file.
ViStatus _VI_FUNC rsscope_ConfigureUserDefinedPresetPathandFileName(
                            ViSession instrSession,
                            ViString  pathName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_USER_DEFINED_PRESET_PATH_AND_FILE_NAME, pathName),
            2, "Path Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Opens and loads a user-defined preset file. Name and path of
///HINF  the preset file are defined with
///HINF  RSSCOPE_ATTR_USER_DEFINED_PRESET_PATH_AND_FILE_NAME.
///HIRET Returns the status code of this operation.
ViStatus _VI_FUNC rsscope_OpenUserDefinedPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_USER_DEFINED_PRESET_OPEN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Saves the current instrument settings to a user-defined
///HINF  preset file. Name and path of the preset file are defined with
///HINF  RSSCOPE_ATTR_USER_DEFINED_PRESET_PATH_AND_FILE_NAME.
///HIRET Returns the status code of this operation.
ViStatus _VI_FUNC rsscope_SaveUserDefinedPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, NULL, RSSCOPE_ATTR_USER_DEFINED_PRESET_SAVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the autonaming file pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR textInput/User-defined text to be inserted after the prefix.
/// HIPAR prefix/If enabled, inserts the default prefix in the file name. The prefix
/// HIPAR prefix/indicates the type of data that is saved, for example, Histogram,
/// HIPAR prefix/RefCurve, Settings.
/// HIPAR userText/If enabled, inserts the specified user text after the prefix.
/// HIPAR date/If enabled, inserts the current date.
/// HIPAR index/If enabled, inserts an index.
/// HIPAR time/If enabled, inserts the current time.
ViStatus _VI_FUNC rsscope_AutonamingFilePattern(ViSession instrSession,
                                                ViString  textInput,
                                                ViBoolean prefix,
                                                ViBoolean userText,
                                                ViBoolean date,
                                                ViBoolean index,
                                                ViBoolean time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (userText)
    {
        viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_AUTONAMING_TEXT, textInput),
                2, "Text Input");
    }

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_AUTONAMING_PREFIX, prefix),
            3, "Prefix");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_AUTONAMING_USER_TEXT, userText),
            3, "user Text");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_AUTONAMING_DATE, date),
            3, "date");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_AUTONAMING_INDEX, index),
            3, "index");

    viCheckParm(rsscope_SetAttributeViBoolean(instrSession, "", RSSCOPE_ATTR_AUTONAMING_TIME, time),
            3, "time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the default path displayed in the file
/// HIFN  selection dialog box for loading and storing operations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR defaultPath/Defines the default path displayed in the file selection dialog box
/// HIPAR defaultPath/for loading and storing operations.
ViStatus _VI_FUNC rsscope_AutonamingDefaultPath(ViSession instrSession,
                                                ViString  defaultPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_AUTONAMING_DEFAULT_PATH, defaultPath),
            2, "Default Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function resets the path for file operations to the default path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_AutonamingResetPath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_AUTONAMING_RESET_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function resets all autonaming settings to the default value,
/// HIFN  including the path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsscope_AutonamingResetAllSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsscope_SetAttributeViString(instrSession, "", RSSCOPE_ATTR_AUTONAMING_RESET_ALL_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
ViStatus _VI_FUNC rsscope_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rsscope_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsscope_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                      ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsscope_ConfigureMultiThreadLocking(ViSession instrSession,
                                                      ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeId
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsscope_GetAttributeRepCapName(ViSession instrSession,
                                                 ViUInt32  attributeId,
                                                 ViString  repCapNameId,
                                                 ViInt32   index,
                                                 ViInt32   bufferSize,
                                                 ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
